@@ -489,6 +489,7 @@ namespace Magick
   using MagickLib::DestroyImageInfo;
   using MagickLib::DestroyImageList;
   using MagickLib::DestroyMagick;
+  using MagickLib::DestroyMagickWand;
   using MagickLib::DestroyPixelWand;
   using MagickLib::DestroyQuantizeInfo;
   using MagickLib::DestroyStringInfo;
@@ -551,8 +552,8 @@ namespace Magick
   using MagickLib::DrawSetClipPath;
   using MagickLib::DrawSetClipRule;
   using MagickLib::DrawSetClipUnits;
+  using MagickLib::DrawSetFillAlpha;
   using MagickLib::DrawSetFillColor;
-  using MagickLib::DrawSetFillOpacity;
   using MagickLib::DrawSetFillPatternURL;
   using MagickLib::DrawSetFillRule;
   using MagickLib::DrawSetFont;
@@ -562,6 +563,7 @@ namespace Magick
   using MagickLib::DrawSetFontStyle;
   using MagickLib::DrawSetFontWeight;
   using MagickLib::DrawSetGravity;
+  using MagickLib::DrawSetStrokeAlpha;
   using MagickLib::DrawSetStrokeAntialias;
   using MagickLib::DrawSetStrokeColor;
   using MagickLib::DrawSetStrokeDashArray;
@@ -569,7 +571,6 @@ namespace Magick
   using MagickLib::DrawSetStrokeLineCap;
   using MagickLib::DrawSetStrokeLineJoin;
   using MagickLib::DrawSetStrokeMiterLimit;
-  using MagickLib::DrawSetStrokeOpacity;
   using MagickLib::DrawSetStrokePatternURL;
   using MagickLib::DrawSetStrokeWidth;
   using MagickLib::DrawSetTextAntialias;
@@ -584,12 +585,13 @@ namespace Magick
   using MagickLib::EdgeImage;
   using MagickLib::EmbossImage;
   using MagickLib::EnhanceImage;
-  using MagickLib::ExecuteModuleProcess;
   using MagickLib::EqualizeImage;
   using MagickLib::EvaluateImage;
   using MagickLib::EvaluateImageChannel;
   using MagickLib::ExceptionInfo;
   using MagickLib::ExceptionType;
+  using MagickLib::ExecuteModuleProcess;
+  using MagickLib::ExportQuantumPixels;
   using MagickLib::FileOpenError;
   using MagickLib::FileOpenFatalError;
   using MagickLib::FileOpenWarning;
@@ -604,9 +606,9 @@ namespace Magick
   using MagickLib::GaussianBlurImage;
   using MagickLib::GetCacheView;
   using MagickLib::GetCacheViewIndexes;
-  using MagickLib::GetImageClipMask;
   using MagickLib::GetImageChannelExtrema;
   using MagickLib::GetImageChannelMean;
+  using MagickLib::GetImageClipMask;
   using MagickLib::GetColorTuple;
   using MagickLib::GetDrawInfo;
   using MagickLib::GetExceptionInfo;
@@ -644,6 +646,7 @@ namespace Magick
   using MagickLib::ImageToBlob;
   using MagickLib::ImageWarning;
   using MagickLib::ImplodeImage;
+  using MagickLib::ImportQuantumPixels;
   using MagickLib::IsEventLogging;
   using MagickLib::IsGeometry;
   using MagickLib::IsImagesEqual;
@@ -654,6 +657,7 @@ namespace Magick
   using MagickLib::MagickInfo;
   using MagickLib::MagickPixelPacket;
   using MagickLib::MagickToMime;
+  using MagickLib::MagickWand;
   using MagickLib::MagnifyImage;
   using MagickLib::MapImage;
   using MagickLib::MatteFloodfillImage;
@@ -672,6 +676,7 @@ namespace Magick
   using MagickLib::MontageInfo;
   using MagickLib::NegateImage;
   using MagickLib::NewPixelWand;
+  using MagickLib::NewMagickWandFromImage;
   using MagickLib::NoValue;
   using MagickLib::NoiseType;
   using MagickLib::NormalizeImage;
@@ -687,10 +692,8 @@ namespace Magick
   using MagickLib::PixelSetQuantumColor;
 	using MagickLib::PixelWand;
   using MagickLib::PointInfo;
-  using MagickLib::PopImagePixels;
   using MagickLib::ProfileImage;
   using MagickLib::ProfileInfo;
-  using MagickLib::PushImagePixels;
   using MagickLib::QuantizeImage;
   using MagickLib::QuantizeInfo;
   using MagickLib::QueryColorDatabase;
