@@ -1769,7 +1769,7 @@ namespace Magick
     MagickLib::GetExceptionInfo( &exceptionInfo );
     linkImages( first_, last_ );
     MagickLib::Image* image = MagickLib::AppendImages( first_->image(),
-						       stack_,
+						       (MagickBooleanType) stack_,
 						       &exceptionInfo ); 
     unlinkImages( first_, last_ );
     appendedImage_->replaceImage( image );
@@ -1841,7 +1841,7 @@ namespace Magick
     MagickLib::ExceptionInfo exceptionInfo;
     MagickLib::GetExceptionInfo( &exceptionInfo );
     char **coder_list =
-      MagickLib::GetMagickList( "*", &number_formats );
+      MagickLib::GetMagickList( "*", &number_formats, &exceptionInfo );
     if( !coder_list )
       {
         throwException( exceptionInfo );
@@ -2251,7 +2251,7 @@ namespace Magick
     MagickLib::GetExceptionInfo( &exceptionInfo );
 
     linkImages( first_, last_ );
-    int errorStat = MagickLib::WriteImages( first_->imageInfo(),
+    int errorStat = MagickLib::WriteImages( first_->constImageInfo(),
                                             first_->image(),
                                             imageSpec_.c_str(),
                                             &exceptionInfo );
