@@ -14,7 +14,7 @@
 
 namespace Magick
 {
-  class TypeMetric
+  class MagickDLLDecl TypeMetric
   {
     friend class Image;
   public:
