@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -41,9 +41,12 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
 #include "magick/colorspace.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
@@ -127,7 +130,7 @@ static Image *ReadAVSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
-      DestroyImageList(image);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
   /*
@@ -173,10 +176,12 @@ static Image *ReadAVSImage(const ImageInfo *image_info,ExceptionInfo *exception)
       if (SyncImagePixels(image) == MagickFalse)
         break;
       if (image->previous == (Image *) NULL)
-        if (QuantumTick(y,image->rows) != MagickFalse)
+        if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+            (QuantumTick(y,image->rows) != MagickFalse))
           {
-            status=MagickMonitor(LoadImageTag,y,image->rows,exception);
-            if (status== MagickFalse)
+            status=image->progress_monitor(LoadImageTag,y,image->rows,
+              image->client_data);
+            if (status == MagickFalse)
               break;
           }
     }
@@ -203,14 +208,17 @@ static Image *ReadAVSImage(const ImageInfo *image_info,ExceptionInfo *exception)
         AllocateNextImage(image_info,image);
         if (image->next == (Image *) NULL)
           {
-            DestroyImageList(image);
+            image=DestroyImageList(image);
             return((Image *) NULL);
           }
         image=SyncNextImageInList(image);
-        status=MagickMonitor(LoadImagesTag,TellBlob(image),GetBlobSize(image),
-          exception);
-        if (status == MagickFalse)
-          break;
+        if (image->progress_monitor != (MagickProgressMonitor) NULL)
+          {
+            status=image->progress_monitor(LoadImagesTag,TellBlob(image),
+              GetBlobSize(image),image->client_data);
+            if (status == MagickFalse)
+              break;
+          }
       }
   } while ((width != (unsigned long) ~0) && (height != (unsigned long) ~0));
   CloseBlob(image);
@@ -370,9 +378,11 @@ static MagickBooleanType WriteAVSImage(const ImageInfo *image_info,Image *image)
       }
       (void) WriteBlob(image,(size_t) (q-pixels),pixels);
       if (image->previous == (Image *) NULL)
-        if (QuantumTick(y,image->rows) != MagickFalse)
+        if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+            (QuantumTick(y,image->rows) != MagickFalse))
           {
-            status=MagickMonitor(SaveImageTag,y,image->rows,&image->exception);
+            status=image->progress_monitor(SaveImageTag,y,image->rows,
+              image->client_data);
             if (status == MagickFalse)
               break;
           }
@@ -381,10 +391,14 @@ static MagickBooleanType WriteAVSImage(const ImageInfo *image_info,Image *image)
     if (image->next == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
-      &image->exception);
-    if (status == MagickFalse)
-      break;
+    if (image->progress_monitor != (MagickProgressMonitor) NULL)
+      {
+        status=image->progress_monitor(SaveImagesTag,scene,
+          GetImageListLength(image),image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
+    scene++;
   } while (image_info->adjoin != MagickFalse);
   CloseBlob(image);
   return(MagickTrue);
