@@ -20,7 +20,7 @@
 %                               October 2003                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -48,9 +48,12 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
 #include "magick/colorspace.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
@@ -178,7 +181,7 @@ static Image *ReadCINImage(const ImageInfo *image_info,
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
-      DestroyImageList(image);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
   /*
@@ -248,9 +251,14 @@ static Image *ReadCINImage(const ImageInfo *image_info,
         if (!SyncImagePixels(image))
           break;
         if (image->previous == (Image *) NULL)
-          if (QuantumTick(y,image->rows) != MagickFalse)
-            if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
-              break;
+          if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+              (QuantumTick(y,image->rows) != MagickFalse))
+            {
+              status=image->progress_monitor(LoadImageTag,y,image->rows,
+                image->client_data);
+              if (status == MagickFalse)
+                break;
+            }
       }
       break;
     }
