@@ -5,39 +5,26 @@
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%                    Read/Write ImageMagick Image Format.                     %
-%                                                                             %
+%                        Read/Write Windows Clipboard.                        %
 %                                                                             %
 %                              Software Design                                %
 %                             Leonard Rosenthol                               %
 %                                 May 2002                                    %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -47,14 +34,7 @@
 /*
   Include declarations.
 */
-
 #include "magick/studio.h"
-#include "magick/blob.h"
-#include "magick/error.h"
-#include "magick/list.h"
-#include "magick/magick.h"
-#include "magick/static.h"
-#include "magick/utility.h"
 #if defined(HasWINGDI32)
 #  if defined(__CYGWIN__)
 #    include <windows.h>
@@ -63,7 +43,26 @@
 #    include <wingdi.h>
 #  endif
 #endif
-
+#include "magick/blob.h"
+#include "magick/blob_private.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
+#include "magick/image.h"
+#include "magick/image_private.h"
+#include "magick/list.h"
+#include "magick/magick.h"
+#include "magick/memory_.h"
+#include "magick/nt_feature.h"
+#include "magick/static.h"
+#include "magick/string_.h"
+
+/*
+  Forward declarations.
+*/
+#if defined(HasWINGDI32)
+static MagickBooleanType
+  WriteCLIPBOARDImage(const ImageInfo *,Image *);
+#endif
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -76,20 +75,17 @@
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ReadCLIPBOARDImage reads an image from the system clipboard and
-%  returns it.  It allocates the memory necessary for the new Image structure
-%  and returns a pointer to the new image.
+%  ReadCLIPBOARDImage() reads an image from the system clipboard and returns
+%  it.  It allocates the memory necessary for the new Image structure and
+%  returns a pointer to the new image.
 %
 %  The format of the ReadCLIPBOARDImage method is:
 %
-%      Image *ReadCLIPBOARDImage(const ImageInfo *image_info,ExceptionInfo *exception)
+%      Image *ReadCLIPBOARDImage(const ImageInfo *image_info,
+%        ExceptionInfo exception)
 %
 %  A description of each parameter follows:
 %
-%    o image:  Method ReadCLIPBOARDImage returns a pointer to the image after
-%      reading.  A null image is returned if there is a memory shortage or
-%      if the image cannot be read.
-%
 %    o image_info: The image info.
 %
 %    o exception: return any errors or warnings in this structure.
@@ -97,7 +93,8 @@
 %
 */
 #if defined(HasWINGDI32)
-static Image *ReadCLIPBOARDImage(const ImageInfo *image_info,ExceptionInfo *exception)
+static Image *ReadCLIPBOARDImage(const ImageInfo *image_info,
+  ExceptionInfo *exception)
 {
   Image
     *image;
@@ -113,110 +110,117 @@ static Image *ReadCLIPBOARDImage(const ImageInfo *image_info,ExceptionInfo *exce
 
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
+  if (image_info->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
-
   {
     HBITMAP
       bitmapH;
 
-    OpenClipboard( NULL );
-    bitmapH = GetClipboardData( CF_BITMAP );
+    HPALETTE
+      hPal;
+
+    OpenClipboard(NULL);
+    bitmapH=GetClipboardData(CF_BITMAP);
+    hPal = GetClipboardData(CF_PALETTE);
     CloseClipboard();
+    if ( bitmapH == NULL )
+      ThrowReaderException(CoderError,"NoBitmapOnClipboard");
+    {
+      BITMAPINFO
+        DIBinfo;
 
-    if ( bitmapH != NULL )
+      BITMAP
+        bitmap;
+
+      HBITMAP
+        hBitmap,
+        hOldBitmap;
+
+      HDC
+        hDC,
+        hMemDC;
+
+      RGBQUAD
+        *pBits,
+        *ppBits;
+
+      // create an offscreen DC for the source
+      hMemDC=CreateCompatibleDC(NULL);
+      hOldBitmap=SelectObject(hMemDC,bitmapH);
+      GetObject(bitmapH,sizeof(BITMAP),(LPSTR) &bitmap);
+      if ((image->columns == 0) || (image->rows == 0))
+        {
+          image->rows=bitmap.bmHeight;
+          image->columns=bitmap.bmWidth;
+        }
+      /*
+        Initialize the bitmap header info.
+      */
+      (void) ResetMagickMemory(&DIBinfo,0,sizeof(BITMAPINFO));
+      DIBinfo.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
+      DIBinfo.bmiHeader.biWidth=image->columns;
+      DIBinfo.bmiHeader.biHeight=(-1)*image->rows;
+      DIBinfo.bmiHeader.biPlanes=1;
+      DIBinfo.bmiHeader.biBitCount=32;
+      DIBinfo.bmiHeader.biCompression=BI_RGB;
+      hDC=GetDC(NULL);
+      if (hDC == 0)
+        ThrowReaderException(CoderError,"UnableToCreateADC");
+      hBitmap=CreateDIBSection(hDC,&DIBinfo,DIB_RGB_COLORS,(void **) &ppBits,
+        NULL,0);
+      ReleaseDC(NULL,hDC);
+      if (hBitmap == 0)
+        ThrowReaderException(CoderError,"UnableToCreateBitmap");
+      // create an offscreen DC
+      hDC=CreateCompatibleDC(NULL);
+      if (hDC == 0)
+        {
+          DeleteObject(hBitmap);
+          ThrowReaderException(CoderError,"UnableToCreateADC");
+        }
+      hOldBitmap=(HBITMAP) SelectObject(hDC,hBitmap);
+      if (hOldBitmap == 0)
+        {
+          DeleteDC(hDC);
+          DeleteObject(hBitmap);
+          ThrowReaderException(CoderError,"UnableToCreateBitmap");
+        }
+      if (hPal != NULL)
       {
-        BITMAPINFO
-          DIBinfo;
-
-        BITMAP
-          bitmap;
-
-        HBITMAP
-          hBitmap,
-          hOldBitmap;
-
-        HDC
-          hDC,
-          hMemDC;
-
-        RGBQUAD
-          *pBits,
-          *ppBits;
-
-        // create an offscreen DC for the source
-        hMemDC = CreateCompatibleDC( NULL );
-        hOldBitmap = SelectObject( hMemDC, bitmapH );
-        GetObject( bitmapH, sizeof( BITMAP ), (LPSTR) &bitmap );
-        if ((image->columns == 0) || (image->rows == 0))
-          {
-            image->rows=bitmap.bmHeight;
-            image->columns=bitmap.bmWidth;
-          }
-
-        /*
-          Initialize the bitmap header info.
-          Create a temp DC for the DIB to go into
-        */
-        (void) memset(&DIBinfo,0,sizeof(BITMAPINFO));
-        DIBinfo.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
-        DIBinfo.bmiHeader.biWidth=image->columns;
-        DIBinfo.bmiHeader.biHeight=(-1)*image->rows;
-        DIBinfo.bmiHeader.biPlanes=1;
-        DIBinfo.bmiHeader.biBitCount=32;
-        DIBinfo.bmiHeader.biCompression=BI_RGB;
-        hDC=GetDC(NULL);
-        if (!hDC)
-          ThrowReaderException(CoderError,"UnableToCreateADC",image);
-        hBitmap= CreateDIBSection(hDC,&DIBinfo,DIB_RGB_COLORS,(void **) &ppBits,NULL,0);
-        ReleaseDC(NULL,hDC);
-        if (!hBitmap)
-          ThrowReaderException(CoderError,"UnableToCreateBitmap",image);
-
-        // create an offscreen DC
-        hDC=CreateCompatibleDC(NULL);
-        if (!hDC)
-          {
-            DeleteObject(hBitmap);
-            ThrowReaderException(CoderError,"UnableToCreateADC",image);
-          }
-        hOldBitmap=(HBITMAP) SelectObject(hDC,hBitmap);
-        if (!hOldBitmap)
-          {
-            DeleteDC(hDC);
-            DeleteObject(hBitmap);
-            ThrowReaderException(CoderError,"UnableToCreateBitmap",image);
-          }
-
-        // bitblt from the memory to the DIB-based one
-        BitBlt( hDC, 0, 0, image->columns, image->rows, hMemDC, 0, 0, SRCCOPY );
-
-        // finally copy the pixels!
-        pBits=ppBits;
-        for (y=0; y < (long) image->rows; y++)
-          {
-            q=SetImagePixels(image,0,y,image->columns,1);
-            if (q == (PixelPacket *) NULL)
-              break;
-            for (x=0; x < (long) image->columns; x++)
-              {
-                q->red    = ScaleCharToQuantum(pBits->rgbRed);
-                q->green  = ScaleCharToQuantum(pBits->rgbGreen);
-                q->blue    = ScaleCharToQuantum(pBits->rgbBlue);
-                q->opacity  = OpaqueOpacity;
-                pBits++;
-                q++;
-              }
-            if (!SyncImagePixels(image))
-              break;
-          }
-      } else
-        ThrowReaderException(CoderError,"NoBitmapOnClipboard",image);
+        // Kenichi Masuko says this is needed
+        SelectPalette(hDC, hPal, FALSE);
+        RealizePalette(hDC);
+      }
+      // bitblt from the memory to the DIB-based one
+      BitBlt(hDC,0,0,image->columns,image->rows,hMemDC,0,0,SRCCOPY);
+      // finally copy the pixels!
+      pBits=ppBits;
+      for (y=0; y < (long) image->rows; y++)
+      {
+        q=SetImagePixels(image,0,y,image->columns,1);
+        if (q == (PixelPacket *) NULL)
+          break;
+        for (x=0; x < (long) image->columns; x++)
+        {
+          q->red=ScaleCharToQuantum(pBits->rgbRed);
+          q->green=ScaleCharToQuantum(pBits->rgbGreen);
+          q->blue=ScaleCharToQuantum(pBits->rgbBlue);
+          q->opacity=OpaqueOpacity;
+          pBits++;
+          q++;
+        }
+        if (SyncImagePixels(image) == MagickFalse)
+          break;
+      }
+      DeleteDC(hDC);
+      DeleteObject(hBitmap);
+    }
   }
-
   CloseBlob(image);
-  return(image);
+  return(GetFirstImageInList(image));
 }
 #endif /* HasWINGDI32 */
 
@@ -225,72 +229,13 @@ static Image *ReadCLIPBOARDImage(const ImageInfo *image_info,ExceptionInfo *exce
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%   W r i t e C L I P B O A R D I m a g e                                     %
+%   R e g i s t e r C L I P B O A R D I m a g e                               %
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method WriteCLIPBOARDImage writes an image to the system clipboard.
-%
-%  The format of the WriteCLIPBOARDImage method is:
-%
-%      unsigned int WriteCLIPBOARDImage(const ImageInfo *image_info,Image *image)
-%
-%  A description of each parameter follows.
-%
-%    o status: Method WriteCLIPBOARDImage return True if the image is written.
-%      False is returned is there is a memory shortage or if the image file
-%      fails to write.
-%
-%    o image_info: The image info.
-%
-%    o image:  The image.
-%
-%
-*/
-#if defined(HasWINGDI32)
-static unsigned int WriteCLIPBOARDImage(const ImageInfo *image_info,Image *image)
-{
-  /*
-    Allocate memory for pixels.
-  */
-  assert(image_info != (const ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
-  assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
-
-  {
-    HBITMAP
-      bitmapH;
-
-    OpenClipboard( NULL );
-    EmptyClipboard();
-
-  bitmapH = ImageToHBITMAP(image);
-  SetClipboardData(CF_BITMAP, bitmapH);
-
-  CloseClipboard();
-
-    return(True);
-  }
-
-  return(False);
-}
-#endif /* HasWINGDI32 */
-
-/*
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%   R e g i s t e r C L I P B O A R D I m a g e                                           %
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%
-%  Method RegisterCLIPBOARDImage adds attributes for the clipboard "image format" to
+%  RegisterCLIPBOARDImage() adds attributes for the clipboard "image format" to
 %  the list of supported formats.  The attributes include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
@@ -304,18 +249,18 @@ static unsigned int WriteCLIPBOARDImage(const ImageInfo *image_info,Image *image
 */
 ModuleExport void RegisterCLIPBOARDImage(void)
 {
-#if defined(HasWINGDI32)
   MagickInfo
     *entry;
 
   entry=SetMagickInfo("CLIPBOARD");
-  entry->decoder=(DecoderHandler) ReadCLIPBOARDImage;
-  entry->encoder=(EncoderHandler) WriteCLIPBOARDImage;
-  entry->adjoin = False;
-  entry->description=AcquireString("the system clipboard");
+#if defined(HasWINGDI32)
+  entry->decoder=(DecoderHandler *) ReadCLIPBOARDImage;
+  entry->encoder=(EncoderHandler *) WriteCLIPBOARDImage;
+#endif
+  entry->adjoin=MagickFalse;
+  entry->description=AcquireString("The system clipboard");
   entry->module=AcquireString("CLIPBOARD");
   (void) RegisterMagickInfo(entry);
-#endif
 }
 
 /*
@@ -323,13 +268,13 @@ ModuleExport void RegisterCLIPBOARDImage(void)
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%   U n r e g i s t e r C L I P B O A R D I m a g e                                       %
+%   U n r e g i s t e r C L I P B O A R D I m a g e                           %
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method UnregisterCLIPBOARDImage removes format registrations made by the
+%  UnregisterCLIPBOARDImage() removes format registrations made by the
 %  RGB module from the list of supported formats.
 %
 %  The format of the UnregisterCLIPBOARDImage method is:
@@ -339,8 +284,59 @@ ModuleExport void RegisterCLIPBOARDImage(void)
 */
 ModuleExport void UnregisterCLIPBOARDImage(void)
 {
-#if defined(HasWINGDI32)
   (void) UnregisterMagickInfo("CLIPBOARD");
-#endif
 }
 
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   W r i t e C L I P B O A R D I m a g e                                     %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  WriteCLIPBOARDImage() writes an image to the system clipboard.
+%
+%  The format of the WriteCLIPBOARDImage method is:
+%
+%      MagickBooleanType WriteCLIPBOARDImage(const ImageInfo *image_info,
+%        Image *image)
+%
+%  A description of each parameter follows.
+%
+%    o image_info: The image info.
+%
+%    o image:  The image.
+%
+%
+*/
+#if defined(HasWINGDI32)
+static MagickBooleanType WriteCLIPBOARDImage(const ImageInfo *image_info,
+  Image *image)
+{
+  /*
+    Allocate memory for pixels.
+  */
+  assert(image_info != (const ImageInfo *) NULL);
+  assert(image_info->signature == MagickSignature);
+  assert(image != (Image *) NULL);
+  assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+  {
+    HBITMAP
+      bitmapH;
+
+    OpenClipboard( NULL );
+    EmptyClipboard();
+    bitmapH=ImageToHBITMAP(image);
+    SetClipboardData(CF_BITMAP,bitmapH);
+    CloseClipboard();
+  }
+  return(MagickTrue);
+}
+#endif /* HasWINGDI32 */
+
