@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -41,14 +41,18 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
 #include "magick/colorspace.h"
 #include "magick/constitute.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/pixel.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 #include "magick/utility.h"
@@ -70,9 +74,9 @@ static MagickBooleanType
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  ReadCMYKImage() reads an image of raw red, green, and blue samples and
-%  returns it.  It allocates the memory necessary for the new Image structure
-%  and returns a pointer to the new image.
+%  ReadCMYKImage() reads an image of raw cyan, magenta, yellow, and black
+%  samples and returns it.  It allocates the memory necessary for the new Image
+%  structure and returns a pointer to the new image.
 %
 %  The format of the ReadCMYKImage method is:
 %
@@ -81,10 +85,6 @@ static MagickBooleanType
 %
 %  A description of each parameter follows:
 %
-%    o image:  Method ReadCMYKImage returns a pointer to the image after
-%      reading.  A null image is returned if there is a memory shortage or
-%      if the image cannot be read.
-%
 %    o image_info: The image info.
 %
 %    o exception: return any errors or warnings in this structure.
@@ -103,9 +103,11 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
   MagickBooleanType
     status;
 
+  MagickOffsetType
+    offset;
+
   register long
-    i,
-    x;
+    i;
 
   register PixelPacket
     *q;
@@ -120,6 +122,7 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
     *scanline;
 
   unsigned long
+    depth,
     width;
 
   assert(image_info != (const ImageInfo *) NULL);
@@ -139,7 +142,7 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
       status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
       if (status == MagickFalse)
         {
-          DestroyImageList(image);
+          image=DestroyImageList(image);
           return((Image *) NULL);
         }
       for (i=0; i < image->offset; i++)
@@ -148,11 +151,12 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
   /*
     Allocate memory for a scanline.
   */
-  packet_size=(size_t) (image->depth > 8 ? 8 : 4);
+  depth=GetImageQuantumDepth(image,MagickTrue);
+  packet_size=(size_t) (4*depth/8);
   if (LocaleCompare(image_info->magick,"CMYKA") == 0)
     {
+      packet_size+=depth/8;
       image->matte=MagickTrue;
-      packet_size=(size_t) (image->depth > 8 ? 10 : 8);
     }
   scanline=(unsigned char *)
     AcquireMagickMemory(packet_size*image->extract_info.width);
@@ -168,7 +172,7 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
       for (y=0; y < (long) image->rows; y++)
         (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
     }
-  x=(long) (packet_size*image->extract_info.x);
+  offset=(MagickOffsetType) (packet_size*image->extract_info.x);
   do
   {
     /*
@@ -197,15 +201,20 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
           if (q == (PixelPacket *) NULL)
             break;
           if (image->matte == MagickFalse)
-            (void) PushImagePixels(image,CMYKQuantum,scanline+x);
+            (void) ExportQuantumPixels(image,CMYKQuantum,0,scanline+offset);
           else
-            (void) PushImagePixels(image,CMYKAQuantum,scanline+x);
+            (void) ExportQuantumPixels(image,CMYKAQuantum,0,scanline+offset);
           if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows) != 0)
-              if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
-                break;
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(y,image->rows) != MagickFalse))
+              {
+                status=image->progress_monitor(LoadImageTag,y,image->rows,
+                  image->client_data);
+                if (status == MagickFalse)
+                  break;
+              }
         }
         width=image->extract_info.height-image->rows-image->extract_info.y;
         for (i=0; i < (long) width; i++)
@@ -228,25 +237,30 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
           q=SetImagePixels(image,0,y,image->columns,1);
           if (q == (PixelPacket *) NULL)
             break;
-          (void) PushImagePixels(image,CyanQuantum,scanline+x);
+          (void) ExportQuantumPixels(image,CyanQuantum,0,scanline+offset);
           (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
-          (void) PushImagePixels(image,MagentaQuantum,scanline+x);
+          (void) ExportQuantumPixels(image,MagentaQuantum,0,scanline+offset);
           (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
-          (void) PushImagePixels(image,YellowQuantum,scanline+x);
+          (void) ExportQuantumPixels(image,YellowQuantum,0,scanline+offset);
           (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
-          (void) PushImagePixels(image,BlackQuantum,scanline+x);
+          (void) ExportQuantumPixels(image,BlackQuantum,0,scanline+offset);
           if (image->matte != MagickFalse)
             {
               (void) ReadBlob(image,packet_size*image->extract_info.width,
                 scanline);
-              (void) PushImagePixels(image,AlphaQuantum,scanline+x);
+              (void) ExportQuantumPixels(image,AlphaQuantum,0,scanline+offset);
             }
           if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows) != 0)
-              if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
-                break;
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(y,image->rows) != MagickFalse))
+              {
+                status=image->progress_monitor(LoadImageTag,y,image->rows,
+                  image->client_data);
+                if (status == MagickFalse)
+                  break;
+              }
         }
         width=image->extract_info.height-image->rows-image->extract_info.y;
         for (i=0; i < (long) width; i++)
@@ -268,7 +282,7 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
             status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
             if (status == MagickFalse)
               {
-                DestroyImageList(image);
+                image=DestroyImageList(image);
                 return((Image *) NULL);
               }
           }
@@ -285,13 +299,18 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
           q=SetImagePixels(image,0,y,image->columns,1);
           if (q == (PixelPacket *) NULL)
             break;
-          (void) PushImagePixels(image,CyanQuantum,scanline+x);
+          (void) ExportQuantumPixels(image,CyanQuantum,0,scanline+offset);
           if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(i,span) != 0)
-              if (MagickMonitor(LoadImageTag,i,span,&image->exception) == MagickFalse)
-                break;
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(i,span) != MagickFalse))
+              {
+                status=image->progress_monitor(LoadImageTag,i,span,
+                  image->client_data);
+                if (status == MagickFalse)
+                  break;
+              }
           i++;
         }
         width=image->extract_info.height-image->rows-image->extract_info.y;
@@ -304,7 +323,7 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
             status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
             if (status == MagickFalse)
               {
-                DestroyImageList(image);
+                image=DestroyImageList(image);
                 return((Image *) NULL);
               }
           }
@@ -316,13 +335,18 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
           q=GetImagePixels(image,0,y,image->columns,1);
           if (q == (PixelPacket *) NULL)
             break;
-          (void) PushImagePixels(image,MagentaQuantum,scanline+x);
+          (void) ExportQuantumPixels(image,MagentaQuantum,0,scanline+offset);
           if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(i,span) != 0)
-              if (MagickMonitor(LoadImageTag,i,span,&image->exception) == MagickFalse)
-                break;
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(i,span) != MagickFalse))
+              {
+                status=image->progress_monitor(LoadImageTag,i,span,
+                  image->client_data);
+                if (status == MagickFalse)
+                  break;
+              }
           i++;
         }
         width=image->extract_info.height-image->rows-image->extract_info.y;
@@ -335,7 +359,7 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
             status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
             if (status == MagickFalse)
               {
-                DestroyImageList(image);
+                image=DestroyImageList(image);
                 return((Image *) NULL);
               }
           }
@@ -347,13 +371,18 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
           q=GetImagePixels(image,0,y,image->columns,1);
           if (q == (PixelPacket *) NULL)
             break;
-          (void) PushImagePixels(image,YellowQuantum,scanline+x);
+          (void) ExportQuantumPixels(image,YellowQuantum,0,scanline+offset);
           if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(i,span) != 0)
-              if (MagickMonitor(LoadImageTag,i,span,&image->exception) == MagickFalse)
-                break;
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(i,span) != MagickFalse))
+              {
+                status=image->progress_monitor(LoadImageTag,i,span,
+                  image->client_data);
+                if (status == MagickFalse)
+                  break;
+              }
           i++;
         }
         width=image->extract_info.height-image->rows-image->extract_info.y;
@@ -366,7 +395,7 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
             status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
             if (status == MagickFalse)
               {
-                DestroyImageList(image);
+                image=DestroyImageList(image);
                 return((Image *) NULL);
               }
           }
@@ -378,13 +407,18 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
           q=GetImagePixels(image,0,y,image->columns,1);
           if (q == (PixelPacket *) NULL)
             break;
-          (void) PushImagePixels(image,BlackQuantum,scanline+x);
+          (void) ExportQuantumPixels(image,BlackQuantum,0,scanline+offset);
           if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(i,span) != 0)
-              if (MagickMonitor(LoadImageTag,i,span,&image->exception) == MagickFalse)
-                break;
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(i,span) != MagickFalse))
+              {
+                status=image->progress_monitor(LoadImageTag,i,span,
+                  image->client_data);
+                if (status == MagickFalse)
+                  break;
+              }
           i++;
         }
         width=image->extract_info.height-image->rows-image->extract_info.y;
@@ -402,7 +436,7 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
                 status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
                 if (status == MagickFalse)
                   {
-                    DestroyImageList(image);
+                    image=DestroyImageList(image);
                     return((Image *) NULL);
                   }
               }
@@ -416,13 +450,15 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
               q=GetImagePixels(image,0,y,image->columns,1);
               if (q == (PixelPacket *) NULL)
                 break;
-              (void) PushImagePixels(image,AlphaQuantum,scanline+x);
+              (void) ExportQuantumPixels(image,AlphaQuantum,0,scanline+offset);
               if (SyncImagePixels(image) == MagickFalse)
                 break;
               if (image->previous == (Image *) NULL)
-                if (QuantumTick(i,span) != 0)
+                if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                    (QuantumTick(i,span) != MagickFalse))
                   {
-                    status=MagickMonitor(LoadImageTag,i,span,&image->exception);
+                    status=image->progress_monitor(LoadImageTag,i,span,
+                      image->client_data);
                     if (status == MagickFalse)
                       break;
                   }
@@ -462,14 +498,17 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
         AllocateNextImage(image_info,image);
         if (image->next == (Image *) NULL)
           {
-            DestroyImageList(image);
+            image=DestroyImageList(image);
             return((Image *) NULL);
           }
         image=SyncNextImageInList(image);
-        status=MagickMonitor(LoadImagesTag,TellBlob(image),GetBlobSize(image),
-          exception);
-        if (status == MagickFalse)
-          break;
+        if (image->progress_monitor != (MagickProgressMonitor) NULL)
+          {
+            status=image->progress_monitor(LoadImagesTag,TellBlob(image),
+              GetBlobSize(image),image->client_data);
+            if (status == MagickFalse)
+              break;
+          }
       }
   } while (count != 0);
   scanline=(unsigned char *) RelinquishMagickMemory(scanline);
@@ -509,6 +548,7 @@ ModuleExport void RegisterCMYKImage(void)
   entry->decoder=(DecoderHandler *) ReadCMYKImage;
   entry->encoder=(EncoderHandler *) WriteCMYKImage;
   entry->raw=MagickTrue;
+  entry->endian_support=MagickTrue;
   entry->description=
     AcquireString("Raw cyan, magenta, yellow, and black samples");
   entry->module=AcquireString("CMYK");
@@ -517,6 +557,7 @@ ModuleExport void RegisterCMYKImage(void)
   entry->decoder=(DecoderHandler *) ReadCMYKImage;
   entry->encoder=(EncoderHandler *) WriteCMYKImage;
   entry->raw=MagickTrue;
+  entry->endian_support=MagickTrue;
   entry->description=
     AcquireString("Raw cyan, magenta, yellow, black, and opacity samples");
   entry->module=AcquireString("CMYK");
@@ -559,30 +600,31 @@ ModuleExport void UnregisterCMYKImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  WriteCMYKImage() writes an image to a file in red, green, and blue
-%  rasterfile format.
+%  WriteCMYKImage() writes an image to a file in cyan, magenta, yellow, and
+%  black raw image format.
 %
 %  The format of the WriteCMYKImage method is:
 %
-%      MagickBooleanType WriteCMYKImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WriteCMYKImage(const ImageInfo *image_info,
+%        Image *image)
 %
 %  A description of each parameter follows.
 %
-%    o status: Method WriteCMYKImage returns True if the image is written.
-%      False is returned is there is a memory shortage or if the image file
-%      fails to write.
-%
 %    o image_info: The image info.
 %
 %    o image:  The image.
 %
 %
 */
-static MagickBooleanType WriteCMYKImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WriteCMYKImage(const ImageInfo *image_info,
+  Image *image)
 {
   long
     y;
 
+  MagickBooleanType
+    status;
+
   MagickOffsetType
     scene;
 
@@ -595,8 +637,8 @@ static MagickBooleanType WriteCMYKImage(const ImageInfo *image_info,Image *image
   unsigned char
     *pixels;
 
-  unsigned int
-    status;
+  unsigned long
+    depth;
 
   /*
     Allocate memory for pixels.
@@ -607,9 +649,10 @@ static MagickBooleanType WriteCMYKImage(const ImageInfo *image_info,Image *image
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
-  packet_size=(size_t) (image->depth > 8 ? 8 : 4);
+  depth=GetImageQuantumDepth(image,MagickTrue);
+  packet_size=(size_t) (4*depth/8);
   if (LocaleCompare(image_info->magick,"CMYKA") == 0)
-    packet_size=(size_t) (image->depth > 8 ? 10 : 8);
+    packet_size+=depth/8;
   pixels=(unsigned char *) AcquireMagickMemory(packet_size*image->columns);
   if (pixels == (unsigned char *) NULL)
     ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
@@ -647,19 +690,20 @@ static MagickBooleanType WriteCMYKImage(const ImageInfo *image_info,Image *image
             break;
           if (LocaleCompare(image_info->magick,"CMYKA") != 0)
             {
-              (void) PopImagePixels(image,CMYKQuantum,pixels);
+              (void) ImportQuantumPixels(image,CMYKQuantum,0,pixels);
               (void) WriteBlob(image,packet_size*image->columns,pixels);
             }
           else
             {
-              (void) PopImagePixels(image,CMYKAQuantum,pixels);
+              (void) ImportQuantumPixels(image,CMYKAQuantum,0,pixels);
               (void) WriteBlob(image,packet_size*image->columns,pixels);
             }
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows) != 0)
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(y,image->rows) != MagickFalse))
               {
-                status=MagickMonitor(SaveImageTag,y,image->rows,
-                  &image->exception);
+                status=image->progress_monitor(SaveImageTag,y,image->rows,
+                  image->client_data);
                 if (status == MagickFalse)
                   break;
               }
@@ -676,23 +720,24 @@ static MagickBooleanType WriteCMYKImage(const ImageInfo *image_info,Image *image
           p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
-          (void) PopImagePixels(image,CyanQuantum,pixels);
+          (void) ImportQuantumPixels(image,CyanQuantum,0,pixels);
           (void) WriteBlob(image,(size_t) image->columns,pixels);
-          (void) PopImagePixels(image,MagentaQuantum,pixels);
+          (void) ImportQuantumPixels(image,MagentaQuantum,0,pixels);
           (void) WriteBlob(image,(size_t) image->columns,pixels);
-          (void) PopImagePixels(image,YellowQuantum,pixels);
+          (void) ImportQuantumPixels(image,YellowQuantum,0,pixels);
           (void) WriteBlob(image,(size_t) image->columns,pixels);
-          (void) PopImagePixels(image,BlackQuantum,pixels);
+          (void) ImportQuantumPixels(image,BlackQuantum,0,pixels);
           (void) WriteBlob(image,(size_t) image->columns,pixels);
           if (LocaleCompare(image_info->magick,"CMYKA") == 0)
             {
-              (void) PopImagePixels(image,AlphaQuantum,pixels);
+              (void) ImportQuantumPixels(image,AlphaQuantum,0,pixels);
               (void) WriteBlob(image,(size_t) image->columns,pixels);
             }
-          if (QuantumTick(y,image->rows) != 0)
+          if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+              (QuantumTick(y,image->rows) != MagickFalse))
             {
-              status=MagickMonitor(SaveImageTag,y,image->rows,
-                &image->exception);
+              status=image->progress_monitor(SaveImageTag,y,image->rows,
+                image->client_data);
               if (status == MagickFalse)
                 break;
             }
@@ -718,9 +763,16 @@ static MagickBooleanType WriteCMYKImage(const ImageInfo *image_info,Image *image
           p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
-          (void) PopImagePixels(image,CyanQuantum,pixels);
+          (void) ImportQuantumPixels(image,CyanQuantum,0,pixels);
           (void) WriteBlob(image,(size_t) image->columns,pixels);
         }
+        if (image->progress_monitor != (MagickProgressMonitor) NULL)
+          {
+            status=image->progress_monitor(LoadImageTag,100,500,
+              image->client_data);
+            if (status == MagickFalse)
+              break;
+          }
         if (image_info->interlace == PartitionInterlace)
           {
             CloseBlob(image);
@@ -730,16 +782,21 @@ static MagickBooleanType WriteCMYKImage(const ImageInfo *image_info,Image *image
             if (status == MagickFalse)
               return(status);
           }
-        if (MagickMonitor(SaveImageTag,100,400,&image->exception) == MagickFalse)
-          break;
         for (y=0; y < (long) image->rows; y++)
         {
           p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
-          (void) PopImagePixels(image,MagentaQuantum,pixels);
+          (void) ImportQuantumPixels(image,MagentaQuantum,0,pixels);
           (void) WriteBlob(image,(size_t) image->columns,pixels);
         }
+        if (image->progress_monitor != (MagickProgressMonitor) NULL)
+          {
+            status=image->progress_monitor(LoadImageTag,200,500,
+              image->client_data);
+            if (status == MagickFalse)
+              break;
+          }
         if (image_info->interlace == PartitionInterlace)
           {
             CloseBlob(image);
@@ -749,16 +806,21 @@ static MagickBooleanType WriteCMYKImage(const ImageInfo *image_info,Image *image
             if (status == MagickFalse)
               return(status);
           }
-        if (MagickMonitor(SaveImageTag,200,400,&image->exception) == MagickFalse)
-          break;
         for (y=0; y < (long) image->rows; y++)
         {
           p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
-          (void) PopImagePixels(image,YellowQuantum,pixels);
+          (void) ImportQuantumPixels(image,YellowQuantum,0,pixels);
           (void) WriteBlob(image,(size_t) image->columns,pixels);
         }
+        if (image->progress_monitor != (MagickProgressMonitor) NULL)
+          {
+            status=image->progress_monitor(LoadImageTag,300,500,
+              image->client_data);
+            if (status == MagickFalse)
+              break;
+          }
         if (image_info->interlace == PartitionInterlace)
           {
             CloseBlob(image);
@@ -768,20 +830,23 @@ static MagickBooleanType WriteCMYKImage(const ImageInfo *image_info,Image *image
             if (status == MagickFalse)
               return(status);
           }
-        if (MagickMonitor(SaveImageTag,200,400,&image->exception) == MagickFalse)
-          break;
         for (y=0; y < (long) image->rows; y++)
         {
           p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
-          (void) PopImagePixels(image,BlackQuantum,pixels);
+          (void) ImportQuantumPixels(image,BlackQuantum,0,pixels);
           (void) WriteBlob(image,(size_t) image->columns,pixels);
         }
         if (LocaleCompare(image_info->magick,"CMYKA") == 0)
           {
-            if (MagickMonitor(SaveImageTag,300,400,&image->exception) == MagickFalse)
-              break;
+            if (image->progress_monitor != (MagickProgressMonitor) NULL)
+              {
+                status=image->progress_monitor(LoadImageTag,400,500,
+                  image->client_data);
+                if (status == MagickFalse)
+                  break;
+              }
             if (image_info->interlace == PartitionInterlace)
               {
                 CloseBlob(image);
@@ -797,25 +862,34 @@ static MagickBooleanType WriteCMYKImage(const ImageInfo *image_info,Image *image
                 &image->exception);
               if (p == (const PixelPacket *) NULL)
                 break;
-              (void) PopImagePixels(image,AlphaQuantum,pixels);
+              (void) ImportQuantumPixels(image,AlphaQuantum,0,pixels);
               (void) WriteBlob(image,(size_t) image->columns,pixels);
             }
           }
         if (image_info->interlace == PartitionInterlace)
           (void) CopyMagickString(image->filename,image_info->filename,
             MaxTextExtent);
-        if (MagickMonitor(SaveImageTag,400,400,&image->exception) == MagickFalse)
-          break;
+        if (image->progress_monitor != (MagickProgressMonitor) NULL)
+          {
+            status=image->progress_monitor(LoadImageTag,500,500,
+              image->client_data);
+            if (status == MagickFalse)
+              break;
+          }
         break;
       }
     }
     if (image->next == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
-      &image->exception);
-    if (status == MagickFalse)
-      break;
+    if (image->progress_monitor != (MagickProgressMonitor) NULL)
+      {
+        status=image->progress_monitor(SaveImagesTag,scene,
+          GetImageListLength(image),image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
+    scene++;
   } while (image_info->adjoin != MagickFalse);
   pixels=(unsigned char *) RelinquishMagickMemory(pixels);
   CloseBlob(image);
