@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -41,14 +41,18 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
 #include "magick/color.h"
 #include "magick/color_private.h"
 #include "magick/colorspace.h"
 #include "magick/draw.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/geometry.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
+#include "magick/log.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
@@ -136,17 +140,25 @@ static MagickBooleanType
 static MagickBooleanType DecodeImage(Image *image,
   const MagickBooleanType compression,unsigned char *pixels)
 {
+#if !defined(__WINDOWS__) || defined(__MINGW32__)
+#define BI_RLE8  1
+#endif
+
   int
     count;
 
   long
     y;
 
+  MagickBooleanType
+    status;
+
   register long
     i,
     x;
 
   register unsigned char
+    *p,
     *q;
 
   unsigned char
@@ -157,30 +169,38 @@ static MagickBooleanType DecodeImage(Image *image,
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   assert(pixels != (unsigned char *) NULL);
-  (void) ResetMagickMemory(pixels,0,image->columns*image->rows);
+  (void) ResetMagickMemory(pixels,0,(size_t) image->columns*image->rows*
+    sizeof(*pixels));
   byte=0;
   x=0;
-  q=pixels;
+  p=pixels;
+  q=pixels+(size_t) image->columns*image->rows;
   for (y=0; y < (long) image->rows; )
   {
+    if ((p < pixels) || (p >= q))
+      break;
     count=ReadBlobByte(image);
     if (count == EOF)
       break;
     if (count != 0)
       {
+        count=Min((unsigned long) count,(size_t) (q-p));
         /*
           Encoded mode.
         */
         byte=(unsigned char) ReadBlobByte(image);
-        for (i=0; i < (long) count; i++)
-        {
-          if (compression == 1)
-            *q++=byte;
-          else
-            *q++=(unsigned char)
-              ((i & 0x01) ? (byte & 0x0f) : ((byte >> 4) & 0x0f));
-          x++;
-        }
+        if (compression == BI_RLE8)
+          {
+            for (i=0; i < count; i++)
+              *p++=(unsigned char) byte;
+          }
+        else
+          {
+            for (i=0; i < count; i++)
+              *p++=(unsigned char)
+                ((i & 0x01) != 0 ? (byte & 0x0f) : ((byte >> 4) & 0x0f));
+          }
+        x+=count;
       }
     else
       {
@@ -199,7 +219,7 @@ static MagickBooleanType DecodeImage(Image *image,
             */
             x=0;
             y++;
-            q=pixels+y*image->columns;
+            p=pixels+y*image->columns;
             break;
           }
           case 0x02:
@@ -209,7 +229,7 @@ static MagickBooleanType DecodeImage(Image *image,
             */
             x+=ReadBlobByte(image);
             y+=ReadBlobByte(image);
-            q=pixels+y*image->columns+x;
+            p=pixels+y*image->columns+x;
             break;
           }
           default:
@@ -217,25 +237,25 @@ static MagickBooleanType DecodeImage(Image *image,
             /*
               Absolute mode.
             */
-            for (i=0; i < (long) count; i++)
-            {
-              if (compression == 1)
-                *q++=(unsigned char) ReadBlobByte(image);
-              else
-                {
-                  if ((i & 0x01) == 0)
-                    byte=(unsigned char) ReadBlobByte(image);
-                  *q++=(unsigned char)
-                    ((i & 0x01) ? (byte & 0x0f) : ((byte >> 4) & 0x0f));
-                }
-              x++;
-            }
+            count=Min((unsigned long) count,(size_t) (q-p));
+            if (compression == BI_RLE8)
+              for (i=0; i < count; i++)
+                *p++=(unsigned char) ReadBlobByte(image);
+            else
+              for (i=0; i < count; i++)
+              {
+                if ((i & 0x01) == 0)
+                  byte=(unsigned char) ReadBlobByte(image);
+                *p++=(unsigned char)
+                  ((i & 0x01) != 0 ? (byte & 0x0f) : ((byte >> 4) & 0x0f));
+              }
+            x+=count;
             /*
               Read pad byte.
             */
-            if (compression == 1)
+            if (compression == BI_RLE8)
               {
-                if (count & 0x01)
+                if ((count & 0x01) != 0)
                   (void) ReadBlobByte(image);
               }
             else
@@ -245,9 +265,14 @@ static MagickBooleanType DecodeImage(Image *image,
           }
         }
       }
-    if (QuantumTick(y,image->rows) != MagickFalse)
-      if (MagickMonitor(LoadImageTag,y,image->rows,&image->exception) == MagickFalse)
-        break;
+    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+        (QuantumTick(y,image->rows) != MagickFalse))
+      {
+        status=image->progress_monitor(LoadImageTag,y,image->rows,
+          image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
   }
   (void) ReadBlobByte(image);  /* end of line */
   (void) ReadBlobByte(image);
@@ -293,6 +318,9 @@ static size_t EncodeImage(Image *image,const unsigned long bytes_per_line,
   long
     y;
 
+  MagickBooleanType
+    status;
+
   register const unsigned char
     *p;
 
@@ -334,9 +362,14 @@ static size_t EncodeImage(Image *image,const unsigned long bytes_per_line,
     */
     *q++=0x00;
     *q++=0x00;
-    if (QuantumTick(y,image->rows) != MagickFalse)
-      if (MagickMonitor(SaveImageTag,y,image->rows,&image->exception) == MagickFalse)
-        break;
+    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+        (QuantumTick(y,image->rows) != MagickFalse))
+      {
+        status=image->progress_monitor(LoadImageTag,y,image->rows,
+          image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
   }
   /*
     End of bitmap.
@@ -464,7 +497,7 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
-      DestroyImageList(image);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
   /*
@@ -557,8 +590,8 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
     dib_info.bits_per_pixel<<=1;
   bytes_per_line=4*((image->columns*dib_info.bits_per_pixel+31)/32);
   length=bytes_per_line*image->rows;
-  pixels=(unsigned char *)
-    AcquireMagickMemory(Max(bytes_per_line,image->columns+1)*image->rows);
+  pixels=(unsigned char *) AcquireMagickMemory((size_t)
+    Max(bytes_per_line,image->columns+256)*image->rows*sizeof(*pixels));
   if (pixels == (unsigned char *) NULL)
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   if ((dib_info.compression == 0) || (dib_info.compression == 3))
@@ -619,10 +652,11 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
         if (SyncImagePixels(image) == MagickFalse)
           break;
         if (image->previous == (Image *) NULL)
-          if (QuantumTick(y,image->rows) != MagickFalse)
+          if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+              (QuantumTick(image->rows-y-1,image->rows) != MagickFalse))
             {
-              status=MagickMonitor(LoadImageTag,image->rows-y-1,image->rows,
-                exception);
+              status=image->progress_monitor(LoadImageTag,image->rows-y-1,
+                image->rows,image->client_data);
               if (status == MagickFalse)
                 break;
             }
@@ -661,10 +695,11 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
         if (SyncImagePixels(image) == MagickFalse)
           break;
         if (image->previous == (Image *) NULL)
-          if (QuantumTick(y,image->rows) != MagickFalse)
+          if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+              (QuantumTick(image->rows-y-1,image->rows) != MagickFalse))
             {
-              status=MagickMonitor(LoadImageTag,image->rows-y-1,image->rows,
-                exception);
+              status=image->progress_monitor(LoadImageTag,image->rows-y-1,
+                image->rows,image->client_data);
               if (status == MagickFalse)
                 break;
             }
@@ -696,10 +731,11 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
         if (SyncImagePixels(image) == MagickFalse)
           break;
         if (image->previous == (Image *) NULL)
-          if (QuantumTick(y,image->rows) != MagickFalse)
+          if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+              (QuantumTick(image->rows-y-1,image->rows) != MagickFalse))
             {
-              status=MagickMonitor(LoadImageTag,image->rows-y-1,image->rows,
-                exception);
+              status=image->progress_monitor(LoadImageTag,image->rows-y-1,
+                image->rows,image->client_data);
               if (status == MagickFalse)
                 break;
             }
@@ -744,10 +780,11 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
         if (SyncImagePixels(image) == MagickFalse)
           break;
         if (image->previous == (Image *) NULL)
-          if (QuantumTick(y,image->rows) != MagickFalse)
+          if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+              (QuantumTick(image->rows-y-1,image->rows) != MagickFalse))
             {
-              status=MagickMonitor(LoadImageTag,image->rows-y-1,image->rows,
-                exception);
+              status=image->progress_monitor(LoadImageTag,image->rows-y-1,
+                image->rows,image->client_data);
               if (status == MagickFalse)
                 break;
             }
@@ -778,10 +815,11 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
         if (SyncImagePixels(image) == MagickFalse)
           break;
         if (image->previous == (Image *) NULL)
-          if (QuantumTick(y,image->rows) != MagickFalse)
+          if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+              (QuantumTick(image->rows-y-1,image->rows) != MagickFalse))
             {
-              status=MagickMonitor(LoadImageTag,image->rows-y-1,image->rows,
-                exception);
+              status=image->progress_monitor(LoadImageTag,image->rows-y-1,
+                image->rows,image->client_data);
               if (status == MagickFalse)
                 break;
             }
@@ -806,10 +844,10 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
       flipped_image=FlipImage(image,exception);
       if (flipped_image == (Image *) NULL)
         {
-          DestroyImageList(image);
+          image=DestroyImageList(image);
           return((Image *) NULL);
         }
-      DestroyImage(image);
+      image=DestroyImage(image);
       image=flipped_image;
     }
   CloseBlob(image);
@@ -1045,12 +1083,14 @@ static MagickBooleanType WriteDIBImage(const ImageInfo *image_info,Image *image)
        for (x=(long) (image->columns+7)/8; x < (long) bytes_per_line; x++)
          *q++=0x00;
        if (image->previous == (Image *) NULL)
-         if (QuantumTick(y,image->rows) != MagickFalse)
-           {
-             status=MagickMonitor(SaveImageTag,y,image->rows,&image->exception);
-             if (status == MagickFalse)
-               break;
-           }
+          if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+              (QuantumTick(y,image->rows) != MagickFalse))
+            {
+              status=image->progress_monitor(SaveImageTag,y,image->rows,
+                image->client_data);
+              if (status == MagickFalse)
+                break;
+            }
       }
       break;
     }
@@ -1071,10 +1111,11 @@ static MagickBooleanType WriteDIBImage(const ImageInfo *image_info,Image *image)
         for ( ; x < (long) bytes_per_line; x++)
           *q++=0x00;
         if (image->previous == (Image *) NULL)
-          if (QuantumTick(y,image->rows) != MagickFalse)
+          if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+              (QuantumTick(y,image->rows) != MagickFalse))
             {
-              status=MagickMonitor(SaveImageTag,y,image->rows,
-                &image->exception);
+              status=image->progress_monitor(SaveImageTag,y,image->rows,
+                image->client_data);
               if (status == MagickFalse)
                 break;
             }
@@ -1106,10 +1147,11 @@ static MagickBooleanType WriteDIBImage(const ImageInfo *image_info,Image *image)
           for (x=3*image->columns; x < (long) bytes_per_line; x++)
             *q++=0x00;
         if (image->previous == (Image *) NULL)
-          if (QuantumTick(y,image->rows) != MagickFalse)
+          if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+              (QuantumTick(y,image->rows) != MagickFalse))
             {
-              status=MagickMonitor(SaveImageTag,y,image->rows,
-                &image->exception);
+              status=image->progress_monitor(SaveImageTag,y,image->rows,
+                image->client_data);
               if (status == MagickFalse)
                 break;
             }
@@ -1165,7 +1207,7 @@ static MagickBooleanType WriteDIBImage(const ImageInfo *image_info,Image *image)
       if (dib_colormap == (unsigned char *) NULL)
         ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
       q=dib_colormap;
-      for (i=0; i < (long) image->colors; i++)
+      for (i=0; i < (long) Min(image->colors,dib_info.number_colors); i++)
       {
         *q++=ScaleQuantumToChar(image->colormap[i].blue);
         *q++=ScaleQuantumToChar(image->colormap[i].green);
