@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -41,9 +41,12 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
 #include "magick/client.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
@@ -157,7 +160,7 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   if (display == (Display *) NULL)
     return((Image *) NULL);
   /*
-    Set our forgiving error handler.
+    Set our forgiving exception handler.
   */
   (void) XSetErrorHandler(XError);
   /*
@@ -191,7 +194,7 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   if ((map_info == (XStandardColormap *) NULL) ||
       (visual_info == (XVisualInfo *) NULL))
     {
-      DestroyImage(image);
+      image=DestroyImage(image);
       XFreeResources(display,visual_info,map_info,(XPixelInfo *) NULL,
         (XFontStruct *) NULL,&resource_info,(XWindowInfo *) NULL);
       return((Image *) NULL);
@@ -202,13 +205,14 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   screen=ScreenOfDisplay(display,visual_info->screen);
   pixels_per_point=XDPSPixelsPerPoint(screen);
   if ((image->x_resolution != 0.0) && (image->y_resolution != 0.0))
-    pixels_per_point=Min(image->x_resolution,image->y_resolution)/72.0;
+    pixels_per_point=Min(image->x_resolution,image->y_resolution)/
+      DefaultResolution;
   status=XDPSCreatePixmapForEPSF((DPSContext) NULL,screen,
     GetBlobFileHandle(image),visual_info->depth,pixels_per_point,&pixmap,
     &bits_per_pixel,&page);
   if ((status == dps_status_failure) || (status == dps_status_no_extension))
     {
-      DestroyImage(image);
+      image=DestroyImage(image);
       XFreeResources(display,visual_info,map_info,(XPixelInfo *) NULL,
         (XFontStruct *) NULL,&resource_info,(XWindowInfo *) NULL);
       return((Image *) NULL);
@@ -221,7 +225,7 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
     -page.x,-page.y,pixels_per_point,MagickTrue,MagickFalse,MagickTrue,&sans);
   if (status != dps_status_success)
     {
-      DestroyImage(image);
+      image=DestroyImage(image);
       XFreeResources(display,visual_info,map_info,(XPixelInfo *) NULL,
         (XFontStruct *) NULL,&resource_info,(XWindowInfo *) NULL);
       return((Image *) NULL);
@@ -234,7 +238,7 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   (void) XFreePixmap(display,pixmap);
   if (dps_image == (XImage *) NULL)
     {
-      DestroyImage(image);
+      image=DestroyImage(image);
       XFreeResources(display,visual_info,map_info,(XPixelInfo *) NULL,
         (XFontStruct *) NULL,&resource_info,(XWindowInfo *) NULL);
       return((Image *) NULL);
@@ -246,7 +250,7 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
     AcquireMagickMemory(visual_info->colormap_size*sizeof(XColor));
   if (colors == (XColor *) NULL)
     {
-      DestroyImage(image);
+      image=DestroyImage(image);
       XDestroyImage(dps_image);
       XFreeResources(display,visual_info,map_info,(XPixelInfo *) NULL,
         (XFontStruct *) NULL,&resource_info,(XWindowInfo *) NULL);
@@ -372,9 +376,14 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
           }
           if (SyncImagePixels(image) == MagickFalse)
             break;
-          if (QuantumTick(y,image->rows) != MagickFalse)
-            if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
-              break;
+          if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+              (QuantumTick(y,image->rows) != MagickFalse))
+            {
+              status=image->progress_monitor(LoadImageTag,y,image->rows,
+                image->client_data);
+              if (status == MagickFalse)
+                break;
+            }
         }
       else
         for (y=0; y < (long) image->rows; y++)
@@ -395,9 +404,14 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
           }
           if (SyncImagePixels(image) == MagickFalse)
             break;
-          if (QuantumTick(y,image->rows) != MagickFalse)
-            if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
-              break;
+          if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+              (QuantumTick(y,image->rows) != MagickFalse))
+            {
+              status=image->progress_monitor(LoadImageTag,y,image->rows,
+                image->client_data);
+              if (status == MagickFalse)
+                break;
+            }
         }
       break;
     }
@@ -408,7 +422,7 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
       */
       if (AllocateImageColormap(image,visual_info->colormap_size) == MagickFalse)
         {
-          DestroyImage(image);
+          image=DestroyImage(image);
           colors=(XColor *) RelinquishMagickMemory(colors);
           XDestroyImage(dps_image);
           XFreeResources(display,visual_info,map_info,(XPixelInfo *) NULL,
@@ -436,9 +450,14 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
           indexes[x]=(unsigned short) XGetPixel(dps_image,x,y);
         if (SyncImagePixels(image) == MagickFalse)
           break;
-        if (QuantumTick(y,image->rows) != MagickFalse)
-          if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
-            break;
+        if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+            (QuantumTick(y,image->rows) != MagickFalse))
+          {
+            status=image->progress_monitor(LoadImageTag,y,image->rows,
+              image->client_data);
+            if (status == MagickFalse)
+              break;
+          }
       }
       break;
     }
