@@ -16,7 +16,7 @@
 %                              Bill Radcliffe                                 %
 %                                   2001                                      %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -48,9 +48,12 @@
 #endif
 
 #include "magick/blob.h"
-#include "magick/error.h"
+#include "magick/blob_private.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/geometry.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
@@ -76,8 +79,6 @@
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method IsEMF() returns MagickTrue if the image format type is EMF.
-%
 %    o magick: This string is generally the first few bytes of an image file
 %      or blob.
 %
@@ -114,8 +115,6 @@ static MagickBooleanType IsEMF(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method IsEMF() returns MagickTrue if the image format type is WMF.
-%
 %    o magick: This string is generally the first few bytes of an image file
 %      or blob.
 %
@@ -157,10 +156,6 @@ static MagickBooleanType IsWMF(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o image:  Method ReadEMFImage returns a pointer to the image after
-%      reading.  A null image is returned if there is a a memory shortage or
-%      if the image cannot be read.
-%
 %    o image_info: The image info..
 %
 %    o exception: Return any errors or warnings in this structure.
@@ -348,9 +343,11 @@ static Image *ReadEMFImage(const ImageInfo *image_info,
   if ((image->columns == 0) || (image->rows == 0))
     {
       double
-        y_resolution = 72.0,
-        x_resolution = 72.0;
+        y_resolution,
+        x_resolution;
 
+       y_resolution=DefaultResolution;
+       x_resolution=DefaultResolution;
       if (image->y_resolution > 0)
         {
           y_resolution=image->y_resolution;
@@ -413,10 +410,10 @@ static Image *ReadEMFImage(const ImageInfo *image_info,
             &image->rows);
           if (image->x_resolution != 0.0)
             image->columns=(unsigned long)
-              (((image->columns*image->x_resolution)/72.0)+0.5);
+              (((image->columns*image->x_resolution)/DefaultResolution)+0.5);
           if (image->y_resolution != 0.0)
             image->rows=(unsigned long)
-              (((image->rows*image->y_resolution)/72.0)+0.5);
+              (((image->rows*image->y_resolution)/DefaultResolution)+0.5);
         }
       geometry=(char *) RelinquishMagickMemory(geometry);
     }
