@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -41,10 +41,13 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
 #include "magick/colorspace.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/compress.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
@@ -142,7 +145,7 @@ static Image *ReadFAXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
-      DestroyImageList(image);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
   /*
@@ -211,11 +214,10 @@ ModuleExport void RegisterFAXImage(void)
   static const char
     *Note=
     {
-      "See TIFF format.  Note that FAX machines use non-square pixels\n"
-      "which are 1.5 times wider than they are tall but computer displays\n"
-      "use square pixels, so FAX images may appear to be narrow unless\n"
-      "they are explicitly resized using a resize specification of\n"
-      "\"150x100%\"."
+      "See TIFF format.  Note that FAX machines use non-square pixels which\n"
+      "are 1.5 times wider than they are tall but computer displays use\n"
+      "square pixels. FAX images may appear to be narrow unless they are\n"
+      "explicitly resized using a resize specification of \"150x100%\"."
     };
 
   entry=SetMagickInfo("FAX");
@@ -323,12 +325,16 @@ static MagickBooleanType WriteFAXImage(const ImageInfo *image_info,Image *image)
     if (image->next == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
-      &image->exception);
-    if (status == MagickFalse)
-      break;
+    if (image->progress_monitor != (MagickProgressMonitor) NULL)
+      {
+        status=image->progress_monitor(SaveImagesTag,scene,
+          GetImageListLength(image),image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
+    scene++;
   } while (write_info->adjoin != MagickFalse);
-  DestroyImageInfo(write_info);
+  write_info=DestroyImageInfo(write_info);
   CloseBlob(image);
   return(status);
 }
