@@ -10,39 +10,26 @@
 %                        F      IIIII    T    SSSSS                           %
 %                                                                             %
 %                                                                             %
-%                    Read/Write ImageMagick Image Format.                     %
-%                                                                             %
+%            Read/Write Flexible Image Transport System Images.               %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -53,18 +40,28 @@
   Include declarations.
 */
 #include "magick/studio.h"
+#include "magick/attribute.h"
 #include "magick/blob.h"
-#include "magick/error.h"
+#include "magick/blob_private.h"
+#include "magick/color_private.h"
+#include "magick/colorspace.h"
+#include "magick/constitute.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
+#include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
+#include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/pixel.h"
 #include "magick/static.h"
-#include "magick/utility.h"
+#include "magick/string_.h"
 
 /*
   Forward declarations.
 */
-static unsigned int
+static MagickBooleanType
   WriteFITSImage(const ImageInfo *,Image *);
 
 /*
@@ -78,17 +75,15 @@ static unsigned int
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method IsFITS returns True if the image format type, identified by the
+%  IsFITS() returns MagickTrue if the image format type, identified by the
 %  magick string, is FITS.
 %
 %  The format of the IsFITS method is:
 %
-%      unsigned int IsFITS(const unsigned char *magick,const size_t length)
+%      MagickBooleanType IsFITS(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method IsFITS returns True if the image format type is FITS.
-%
 %    o magick: This string is generally the first few bytes of an image file
 %      or blob.
 %
@@ -96,15 +91,15 @@ static unsigned int
 %
 %
 */
-static unsigned int IsFITS(const unsigned char *magick,const size_t length)
+static MagickBooleanType IsFITS(const unsigned char *magick,const size_t length)
 {
   if (length < 6)
-    return(False);
+    return(MagickFalse);
   if (LocaleNCompare((char *) magick,"IT0",3) == 0)
-    return(True);
+    return(MagickTrue);
   if (LocaleNCompare((char *) magick,"SIMPLE",6) == 0)
-    return(True);
-  return(False);
+    return(MagickTrue);
+  return(MagickFalse);
 }
 
 /*
@@ -118,7 +113,7 @@ static unsigned int IsFITS(const unsigned char *magick,const size_t length)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ReadFITSImage reads a FITS image file and returns it.  It
+%  ReadFITSImage() reads a FITS image file and returns it.  It
 %  allocates the memory necessary for the new Image structure and returns a
 %  pointer to the new image.
 %
@@ -169,9 +164,6 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
     scale,
     scaled_pixel;
 
-  ExtendedSignedIntegralType
-    number_pixels;
-
   FITSInfo
     fits_info;
 
@@ -190,10 +182,16 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
     j,
     k,
     l,
-    packet_size,
     scene,
     y;
 
+  MagickBooleanType
+    status,
+    value_expected;
+
+  MagickSizeType
+    number_pixels;
+
   register IndexPacket
     *indexes;
 
@@ -209,36 +207,41 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
   register unsigned char
     *p;
 
-  size_t
+  ssize_t
     count;
 
+  size_t
+    packet_size;
+
   unsigned char
     *fits_pixels,
     long_quantum[8];
 
-  unsigned int
-    status,
-    value_expected;
-
   /*
     Open image file.
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
+  if (image_info->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-  if (status == False)
-    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
+  if (status == MagickFalse)
+    {
+      image=DestroyImageList(image);
+      return((Image *) NULL);
+    }
   /*
     Initialize image header.
   */
-  fits_info.simple=False;
+  fits_info.simple=MagickFalse;
   fits_info.bits_per_pixel=8;
   fits_info.columns=1;
   fits_info.rows=1;
   fits_info.number_scenes=1;
+  fits_info.number_axes=1;
   fits_info.min_data=0.0;
   fits_info.max_data=0.0;
   fits_info.zero=0.0;
@@ -249,12 +252,12 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
   c=ReadBlobByte(image);
   if (c == EOF)
     {
-      DestroyImage(image);
+      image=DestroyImage(image);
       return((Image *) NULL);
     }
   for ( ; ; )
   {
-    if (!isalnum((int) c))
+    if (isalnum((int) ((unsigned char) c)) == 0)
       c=ReadBlobByte(image);
     else
       {
@@ -267,26 +270,27 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
         p=keyword;
         do
         {
-          if ((p-keyword) < (MaxTextExtent-1))
-            *p++=c;
+          if ((size_t) (p-keyword) < (MaxTextExtent-1))
+            *p++=(char) c;
           c=ReadBlobByte(image);
-        } while (isalnum(c) || (c == '_'));
+        } while ((isalnum((int) ((unsigned char) c)) != MagickFalse) ||
+                 ((int) c == '_'));
         *p='\0';
         if (LocaleCompare(keyword,"END") == 0)
           break;
-        value_expected=False;
-        while (isspace(c) || (c == '='))
+        value_expected=MagickFalse;
+        while ((isspace((int) ((unsigned char) c)) != 0) || ((int) c == '='))
         {
-          if (c == '=')
-            value_expected=True;
+          if ((int) c == '=')
+            value_expected=MagickTrue;
           c=ReadBlobByte(image);
         }
-        if (value_expected == False)
+        if (value_expected == MagickFalse)
           continue;
         p=value;
         while (isalnum(c) || (c == '-') || (c == '+') || (c == '.'))
         {
-          if ((p-value) < (MaxTextExtent-1))
+          if ((size_t) (p-value) < (MaxTextExtent-1))
             *p++=c;
           c=ReadBlobByte(image);
         }
@@ -295,15 +299,15 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
           Assign a value to the specified keyword.
         */
         if (LocaleCompare(keyword,"SIMPLE") == 0)
-          fits_info.simple=(*value == 'T') || (*value == 't');
+          fits_info.simple=(int) ((*value == 'T') || (*value == 't'));
         if (LocaleCompare(keyword,"BITPIX") == 0)
           fits_info.bits_per_pixel=atoi(value);
         if (LocaleCompare(keyword,"NAXIS") == 0)
           fits_info.number_axes=atoi(value);
         if (LocaleCompare(keyword,"NAXIS1") == 0)
-          fits_info.columns= atoi(value);
+          fits_info.columns=atoi(value);
         if (LocaleCompare(keyword,"NAXIS2") == 0)
-          fits_info.rows= atoi(value);
+          fits_info.rows=atoi(value);
         if (LocaleCompare(keyword,"NAXIS3") == 0)
           fits_info.number_scenes=atoi(value);
         if (LocaleCompare(keyword,"DATAMAX") == 0)
@@ -314,6 +318,7 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
           fits_info.zero=atof(value);
         if (LocaleCompare(keyword,"BSCALE") == 0)
           fits_info.scale=atof(value);
+        (void) SetImageAttribute(image,keyword,value);
       }
     while ((TellBlob(image) % 80) != 0)
       c=ReadBlobByte(image);
@@ -324,10 +329,10 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
   /*
     Verify that required image information is defined.
   */
-  number_pixels=(ExtendedSignedIntegralType) fits_info.columns*fits_info.rows;
-  if ((!fits_info.simple) || (fits_info.number_axes < 1) ||
+  number_pixels=(MagickSizeType) fits_info.columns*fits_info.rows;
+  if ((fits_info.simple == 0) || (fits_info.number_axes < 1) ||
       (fits_info.number_axes > 4) || (number_pixels == 0))
-    ThrowReaderException(CorruptImageError,"ImageTypeNotSupported",image);
+    ThrowReaderException(CorruptImageError,"ImageTypeNotSupported");
   if (fits_info.bits_per_pixel == -32)
     {
       exponential[150]=1.0;
@@ -349,37 +354,40 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
     /*
       Create linear colormap.
     */
-    image->columns=fits_info.columns;
-    image->rows=fits_info.rows;
-    image->depth=fits_info.bits_per_pixel <= 8 ? 8 : QuantumDepth;
+    image->columns=(unsigned long) fits_info.columns;
+    image->rows=(unsigned long) fits_info.rows;
+    image->depth=fits_info.bits_per_pixel <= 8 ? 8UL : Min(QuantumDepth,16);
     image->storage_class=PseudoClass;
-    image->scene=scene;
-    if (!AllocateImageColormap(image,1 << image->depth))
-      ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
-    if (image_info->ping && (image_info->number_scenes != 0))
+    image->scene=(unsigned long) scene;
+    if (AllocateImageColormap(image,1UL << image->depth) == MagickFalse)
+      ThrowReaderException(ResourceLimitError,"UnableToAllocateColormap");
+    if ((image_info->ping != MagickFalse) && (image_info->number_scenes != 0))
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
     /*
       Initialize image structure.
     */
-    packet_size=fits_info.bits_per_pixel/8;
-    if (packet_size < 0)
-      packet_size=(-packet_size);
-    number_pixels=(ExtendedSignedIntegralType) image->columns*image->rows;
-    if ((packet_size*number_pixels) != (size_t) (packet_size*number_pixels))
-      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
+    packet_size=(size_t) fits_info.bits_per_pixel/8;
+    if (fits_info.bits_per_pixel < 0)
+      packet_size=(size_t) (-fits_info.bits_per_pixel/8);
+    number_pixels=(MagickSizeType) image->columns*image->rows;
+    if ((packet_size*number_pixels) !=
+        (MagickSizeType) ((size_t) (packet_size*number_pixels)))
+      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
     fits_pixels=(unsigned char *)
-      AcquireMemory(packet_size*image->columns*image->rows);
+      AcquireMagickMemory(packet_size*image->columns*image->rows);
     if (fits_pixels == (unsigned char *) NULL)
-      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
+      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
     /*
       Convert FITS pixels to pixel packets.
     */
     count=ReadBlob(image,packet_size*image->columns*image->rows,fits_pixels);
     if (count == 0)
-      ThrowReaderException(CorruptImageError,"InsufficientImageDataInFile",
-        image);
-    if ((fits_info.min_data == 0.0) && (fits_info.max_data == 0.0))
+      ThrowReaderException(CorruptImageError,"InsufficientImageDataInFile");
+    if ((fits_info.bits_per_pixel > 0) &&
+        (fits_info.min_data == 0.0) && (fits_info.max_data == 0.0))
+      fits_info.max_data=(1UL << fits_info.bits_per_pixel)-1;
+    else
       {
         /*
           Determine minimum and maximum intensity.
@@ -387,15 +395,12 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
         p=fits_pixels;
         long_quantum[0]=(*p);
         quantum=(*p++);
-        for (j=1; j <= (packet_size-1); j++)
+        for (j=1; j <= (long) (packet_size-1); j++)
         {
           long_quantum[j]=(*p);
           quantum=(quantum << 8) | (*p++);
         }
         pixel=(double) quantum;
-        if (fits_info.bits_per_pixel == 16)
-          if (pixel > 32767)
-            pixel-=65536L;
         if (fits_info.bits_per_pixel == -32)
           {
             j=((long) long_quantum[1] << 16) | ((long) long_quantum[2] << 8) |
@@ -434,15 +439,12 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
         {
           long_quantum[0]=(*p);
           quantum=(*p++);
-          for (j=1; j <= (packet_size-1); j++)
+          for (j=1; j <= (long) (packet_size-1); j++)
           {
             long_quantum[j]=(*p);
             quantum=(quantum << 8) | (*p++);
           }
           pixel=(double) quantum;
-          if (fits_info.bits_per_pixel == 16)
-            if (pixel > 32767)
-              pixel-=65536L;
           if (fits_info.bits_per_pixel == -32)
             {
               j=((long) long_quantum[1] << 16) | ((long) long_quantum[2] << 8) |
@@ -486,9 +488,10 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
       Convert FITS pixels to pixel packets.
     */
     scale=1.0;
-    if ((fits_info.bits_per_pixel < 0) ||
-        ((fits_info.max_data-fits_info.min_data) > ((1 << image->depth)-1)))
-      scale=((1 << image->depth)-1)/(fits_info.max_data-fits_info.min_data);
+    if ((fits_info.bits_per_pixel < 0) || ((fits_info.max_data-
+        fits_info.min_data) > (double) ((1UL << image->depth)-1)))
+      scale=(double) ((1UL << image->depth)-1)/
+        (fits_info.max_data-fits_info.min_data);
     p=fits_pixels;
     for (y=(long) image->rows-1; y >= 0; y--)
     {
@@ -500,15 +503,12 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
       {
         long_quantum[0]=(*p);
         quantum=(*p++);
-        for (j=1; j <= (packet_size-1); j++)
+        for (j=1; j <= (long) (packet_size-1); j++)
         {
           long_quantum[j]=(*p);
           quantum=(quantum << 8) | (*p++);
         }
         pixel=(double) quantum;
-        if (fits_info.bits_per_pixel == 16)
-          if (pixel > 32767)
-            pixel-=65536L;
         if (fits_info.bits_per_pixel == -32)
           {
             j=((long) long_quantum[1] << 16) | ((long) long_quantum[2] << 8) |
@@ -529,37 +529,43 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
               ((long) long_quantum[2] << 16) |
               ((long) long_quantum[3] << 8) |
                (long) long_quantum[4];
-            k=(int) *long_quantum;
+            k=(long) *long_quantum;
             l=((long) long_quantum[5] << 16) | ((long) long_quantum[6] << 8) |
                (long) long_quantum[7];
-            exponent=((k & 127) << 4) | (j >> 28);
+            exponent=(long) ((((unsigned long) k & 127) << 4) |
+              ((unsigned long) j >> 28));
             *(double *) long_quantum=exponential[exponent]*(16777216.0*
               (double) ((j & 0x0FFFFFFF) | 0x10000000)+(double) l);
             if ((exponent | j | l) == 0)
               *(double *) long_quantum=0.0;
-            if (k & 128)
+            if ((k & 128) != 0)
               *(double *)long_quantum=(-(*(double *) long_quantum));
             pixel=(double) (*((double *) long_quantum));
           }
         scaled_pixel=scale*
           (pixel*fits_info.scale-fits_info.min_data-fits_info.zero);
-        index=(IndexPacket) ((scaled_pixel < 0) ? 0 :
-          (scaled_pixel > ((1 << image->depth)-1)) ? ((1 << image->depth)-1) :
-          scaled_pixel+0.5);
+        index=(IndexPacket) ((scaled_pixel < 0.0) ? 0 :
+          ((unsigned long) scaled_pixel > ((1UL << image->depth)-1)) ?
+          ((1UL << image->depth)-1) : (unsigned long) (scaled_pixel+0.5));
         index=ConstrainColormapIndex(image,index);
         indexes[x]=index;
         *q++=image->colormap[index];
       }
-      if (!SyncImagePixels(image))
+      if (SyncImagePixels(image) == MagickFalse)
         break;
-      if (QuantumTick(y,image->rows))
-        if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
-          break;
+      if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+          (QuantumTick(y,image->rows) != MagickFalse))
+        {
+          status=image->progress_monitor(LoadImageTag,y,image->rows,
+            image->client_data);
+          if (status == MagickFalse)
+            break;
+        }
     }
-    LiberateMemory((void **) &fits_pixels);
-    if (EOFBlob(image))
+    fits_pixels=(unsigned char *) RelinquishMagickMemory(fits_pixels);
+    if (EOFBlob(image) != MagickFalse)
       {
-        (void) ThrowException(exception,CorruptImageError,"UnexpectedEndOfFile",
+        ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
           image->filename);
         break;
       }
@@ -577,20 +583,21 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
         AllocateNextImage(image_info,image);
         if (image->next == (Image *) NULL)
           {
-            DestroyImageList(image);
+            image=DestroyImageList(image);
             return((Image *) NULL);
           }
         image=SyncNextImageInList(image);
-        status=MagickMonitor(LoadImagesTag,TellBlob(image),GetBlobSize(image),
-          exception);
-        if (status == False)
-          break;
+        if (image->progress_monitor != (MagickProgressMonitor) NULL)
+          {
+            status=image->progress_monitor(LoadImagesTag,TellBlob(image),
+              GetBlobSize(image),image->client_data);
+            if (status == MagickFalse)
+              break;
+          }
       }
   }
-  while (image->previous != (Image *) NULL)
-    image=image->previous;
   CloseBlob(image);
-  return(image);
+  return(GetFirstImageInList(image));
 }
 
 /*
@@ -604,7 +611,7 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method RegisterFITSImage adds attributes for the FITS image format to
+%  RegisterFITSImage() adds attributes for the FITS image format to
 %  the list of supported formats.  The attributes include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
@@ -622,10 +629,10 @@ ModuleExport void RegisterFITSImage(void)
     *entry;
 
   entry=SetMagickInfo("FITS");
-  entry->decoder=(DecoderHandler) ReadFITSImage;
-  entry->encoder=(EncoderHandler) WriteFITSImage;
-  entry->magick=(MagickHandler) IsFITS;
-  entry->adjoin=False;
+  entry->decoder=(DecoderHandler *) ReadFITSImage;
+  entry->encoder=(EncoderHandler *) WriteFITSImage;
+  entry->magick=(MagickHandler *) IsFITS;
+  entry->adjoin=MagickFalse;
   entry->description=AcquireString("Flexible Image Transport System");
   entry->module=AcquireString("FITS");
   (void) RegisterMagickInfo(entry);
@@ -642,7 +649,7 @@ ModuleExport void RegisterFITSImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method UnregisterFITSImage removes format registrations made by the
+%  UnregisterFITSImage() removes format registrations made by the
 %  FITS module from the list of supported formats.
 %
 %  The format of the UnregisterFITSImage method is:
@@ -666,26 +673,22 @@ ModuleExport void UnregisterFITSImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method WriteFITSImage writes a Flexible Image Transport System image to a
+%  WriteFITSImage() writes a Flexible Image Transport System image to a
 %  file as gray scale intensities [0..255].
 %
 %  The format of the WriteFITSImage method is:
 %
-%      unsigned int WriteFITSImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WriteFITSImage(const ImageInfo *image_info,Image *image)
 %
 %  A description of each parameter follows.
 %
-%    o status: Method WriteFITSImage return True if the image is written.
-%      False is returned is there is a memory shortage or if the image file
-%      fails to write.
-%
 %    o image_info: The image info.
 %
 %    o image:  The image.
 %
 %
 */
-static unsigned int WriteFITSImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WriteFITSImage(const ImageInfo *image_info,Image *image)
 {
   char
     buffer[81],
@@ -694,21 +697,24 @@ static unsigned int WriteFITSImage(const ImageInfo *image_info,Image *image)
   long
     y;
 
+  MagickBooleanType
+    status;
+
+  MagickSizeType
+    length;
+
   register const PixelPacket
     *p;
 
   register long
     i;
 
+  size_t
+    packet_size;
+
   unsigned char
     *pixels;
 
-  unsigned int
-    status;
-
-  unsigned long
-    packet_size;
-
   /*
     Open output image file.
   */
@@ -716,18 +722,20 @@ static unsigned int WriteFITSImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
-  if (status == False)
-    ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
+  if (status == MagickFalse)
+    return(status);
   (void) SetImageColorspace(image,RGBColorspace);
   /*
     Allocate image memory.
   */
-  packet_size=image->depth > 8 ? 2 : 1;
-  fits_info=(char *) AcquireMemory(2880);
-  pixels=(unsigned char *) AcquireMemory(packet_size*image->columns);
+  packet_size=(size_t) (image->depth > 8 ? 2 : 1);
+  fits_info=(char *) AcquireMagickMemory(2880*sizeof(*fits_info));
+  pixels=(unsigned char *) AcquireMagickMemory(packet_size*image->columns);
   if ((fits_info == (char *) NULL) || (pixels == (unsigned char *) NULL))
-    ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",image);
+    ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
   /*
     Initialize image header.
   */
@@ -735,24 +743,27 @@ static unsigned int WriteFITSImage(const ImageInfo *image_info,Image *image)
     fits_info[i]=' ';
   (void) strcpy(buffer,"SIMPLE  =                    T");
   (void) strncpy(fits_info+0,buffer,strlen(buffer));
-  FormatString(buffer,"BITPIX  =                    %ld",image->depth);
+  (void) FormatMagickString(buffer,MaxTextExtent,
+    "BITPIX  =                    %ld",image->depth);
   (void) strncpy(fits_info+80,buffer,strlen(buffer));
   (void) strcpy(buffer,"NAXIS   =                    2");
   (void) strncpy(fits_info+160,buffer,strlen(buffer));
-  FormatString(buffer,"NAXIS1  =           %10lu",image->columns);
+  (void) FormatMagickString(buffer,MaxTextExtent,"NAXIS1  =           %10lu",
+    image->columns);
   (void) strncpy(fits_info+240,buffer,strlen(buffer));
-  FormatString(buffer,"NAXIS2  =           %10lu",image->rows);
+  (void) FormatMagickString(buffer,MaxTextExtent,"NAXIS2  =           %10lu",
+    image->rows);
   (void) strncpy(fits_info+320,buffer,strlen(buffer));
-  FormatString(buffer,"DATAMIN =           %10u",0);
+  (void) FormatMagickString(buffer,MaxTextExtent,"DATAMIN =           %10u",0);
   (void) strncpy(fits_info+400,buffer,strlen(buffer));
-  FormatString(buffer,"DATAMAX =           %10u",(1 << image->depth)-1);
+  (void) FormatMagickString(buffer,MaxTextExtent,"DATAMAX =           %10u",
+    (1 << image->depth)-1);
   (void) strncpy(fits_info+480,buffer,strlen(buffer));
   (void) strcpy(buffer,"HISTORY Created by ImageMagick.");
   (void) strncpy(fits_info+560,buffer,strlen(buffer));
   (void) strcpy(buffer,"END");
   (void) strncpy(fits_info+640,buffer,strlen(buffer));
-  (void) WriteBlob(image,2880,(char *) fits_info);
-  LiberateMemory((void **) &fits_info);
+  (void) WriteBlob(image,2880,(unsigned char *) fits_info);
   /*
     Convert image to fits scale PseudoColor class.
   */
@@ -761,17 +772,22 @@ static unsigned int WriteFITSImage(const ImageInfo *image_info,Image *image)
     p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
     if (p == (const PixelPacket *) NULL)
       break;
-    (void) PopImagePixels(image,GrayQuantum,pixels);
+    (void) ImportQuantumPixels(image,GrayQuantum,0,pixels);
     (void) WriteBlob(image,packet_size*image->columns,pixels);
-    if (QuantumTick(image->rows-y-1,image->rows))
+    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+        (QuantumTick(image->rows-y-1,image->rows) != MagickFalse))
       {
-        status=MagickMonitor(SaveImageTag,image->rows-y-1,image->rows,
-          &image->exception);
-        if (status == False)
+        status=image->progress_monitor(SaveImageTag,image->rows-y-1,image->rows,
+          image->client_data);
+        if (status == MagickFalse)
           break;
       }
   }
-  LiberateMemory((void **) &pixels);
+  (void) ResetMagickMemory(fits_info,0,2880*sizeof(*fits_info));
+  length=packet_size*image->columns*image->rows;
+  (void) WriteBlob(image,2880-(length % 2880),(unsigned char *) fits_info);
+  fits_info=(char *) RelinquishMagickMemory(fits_info);
+  pixels=(unsigned char *) RelinquishMagickMemory(pixels);
   CloseBlob(image);
-  return(True);
+  return(MagickTrue);
 }
