@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -42,17 +42,21 @@
 #include "magick/studio.h"
 #include "magick/attribute.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
 #include "magick/color.h"
 #include "magick/color_private.h"
 #include "magick/colorspace.h"
 #include "magick/constitute.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/geometry.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/pixel.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 #if defined(HasFPX)
@@ -210,7 +214,7 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
-      DestroyImageList(image);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
   CloseBlob(image);
@@ -252,7 +256,7 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
       FPX_ClearSystem();
       ThrowFileException(exception,FileOpenError,"UnableToOpenFile",
         image->filename);
-      DestroyImageList(image);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
   if (colorspace.numberOfComponents == 0)
@@ -322,7 +326,7 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
           }
         (void) CopyMagickString(comments,(char *) summary_info.comments.ptr,
           summary_info.comments.length+1);
-        (void) SetImageAttribute(image,"comment",comments);
+        (void) SetImageAttribute(image,"Comment",comments);
         comments=(char *) RelinquishMagickMemory(comments);
       }
   /*
@@ -467,9 +471,14 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
     }
     if (SyncImagePixels(image) == MagickFalse)
       break;
-    if (QuantumTick(y,image->rows) != MagickFalse)
-      if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
-        break;
+    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+        (QuantumTick(y,image->rows) != MagickFalse))
+      {
+        status=image->progress_monitor(LoadImageTag,y,image->rows,
+          image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
   }
   scanline=(char *) RelinquishMagickMemory(scanline);
   (void) FPX_CloseImage(flashpix);
@@ -513,6 +522,7 @@ ModuleExport void RegisterFPXImage(void)
 #endif
   entry->adjoin=MagickFalse;
   entry->seekable_stream=MagickTrue;
+  entry->blob_support=MagickFalse;
   entry->magick=(MagickHandler *) IsFPX;
   entry->description=AcquireString("FlashPix Format");
   entry->module=AcquireString("FPX");
@@ -869,7 +879,7 @@ static MagickBooleanType WriteFPXImage(const ImageInfo *image_info,Image *image)
         Initialize the compression by quality for the entire image.
       */
       fpx_status=FPX_SetJPEGCompression(flashpix,(unsigned short)
-        image_info->quality == 0 ? 75 : image_info->quality);
+        image->quality == UndefinedCompressionQuality ? 75 : image->quality);
       if (fpx_status != FPX_OK)
         ThrowWriterException(DelegateError,"UnableToSetJPEGLevel");
     }
@@ -909,7 +919,7 @@ static MagickBooleanType WriteFPXImage(const ImageInfo *image_info,Image *image)
       (void) CopyMagickString((char *) summary_info.title.ptr,label->value,
         MaxTextExtent);
     }
-  comment=GetImageAttribute(image,"comment");
+  comment=GetImageAttribute(image,"Comment");
   if (comment != (ImageAttribute *) NULL)
     {
       /*
@@ -965,18 +975,23 @@ static MagickBooleanType WriteFPXImage(const ImageInfo *image_info,Image *image)
     if (p == (const PixelPacket *) NULL)
       break;
     if (fpx_info.numberOfComponents == 1)
-      (void) PopImagePixels(image,GrayQuantum,pixels);
+      (void) ImportQuantumPixels(image,GrayQuantum,pixels);
     else
       if (image->matte == MagickFalse)
-        (void) PopImagePixels(image,RGBQuantum,pixels);
+        (void) ImportQuantumPixels(image,RGBQuantum,0,pixels);
       else
-        (void) PopImagePixels(image,RGBAQuantum,pixels);
+        (void) ImportQuantumPixels(image,RGBAQuantum,0,pixels);
     fpx_status=FPX_WriteImageLine(flashpix,&fpx_info);
     if (fpx_status != FPX_OK)
       break;
-    if (QuantumTick(y,image->rows) != MagickFalse)
-      if (MagickMonitor(SaveImageTag,y,image->rows,&image->exception) == MagickFalse)
-        break;
+    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+        (QuantumTick(y,image->rows) != MagickFalse))
+      {
+        status=image->progress_monitor(SaveImageTag,y,image->rows,
+          image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
   }
   if (image_info->view != (char *) NULL)
     {
