@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -42,10 +42,14 @@
 #include "magick/studio.h"
 #include "magick/attribute.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
+#include "magick/color.h"
 #include "magick/color_private.h"
 #include "magick/colorspace.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
@@ -160,8 +164,18 @@ static MagickBooleanType DecodeImage(Image *image,const Quantum opacity)
       (prefix == (short *) NULL) ||
       (suffix == (unsigned char *) NULL) ||
       (pixel_stack == (unsigned char *) NULL))
-    ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
-      image->filename);
+    {
+      if (packet != (unsigned char *) NULL)
+        packet=(unsigned char *) RelinquishMagickMemory(packet);
+      if (prefix != (short *) NULL)
+        prefix=(short *) RelinquishMagickMemory(prefix);
+      if (suffix != (unsigned char *) NULL)
+        suffix=(unsigned char *) RelinquishMagickMemory(suffix);
+      if (pixel_stack != (unsigned char *) NULL)
+        pixel_stack=(unsigned char *) RelinquishMagickMemory(pixel_stack);
+      ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
+        image->filename);
+    }
   /*
     Initialize GIF data stream decoder.
   */
@@ -258,6 +272,8 @@ static MagickBooleanType DecodeImage(Image *image,const Quantum opacity)
             }
           while (code >= clear)
           {
+            if ((top_stack-pixel_stack) >= MaxStackSize)
+              break;
             *top_stack++=suffix[code];
             code=(unsigned long) prefix[code];
           }
@@ -265,6 +281,8 @@ static MagickBooleanType DecodeImage(Image *image,const Quantum opacity)
           /*
             Add a new string to the string table,
           */
+          if ((top_stack-pixel_stack) >= MaxStackSize)
+            break;
           if (available >= MaxStackSize)
             break;
           *top_stack++=(unsigned char) first;
@@ -337,9 +355,11 @@ static MagickBooleanType DecodeImage(Image *image,const Quantum opacity)
     if (x < (long) image->columns)
       break;
     if (image->previous == (Image *) NULL)
-      if (QuantumTick(y,image->rows) != MagickFalse)
+      if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+          (QuantumTick(y,image->rows) != MagickFalse))
         {
-          status=MagickMonitor(LoadImageTag,y,image->rows,&image->exception);
+          status=image->progress_monitor(LoadImageTag,y,image->rows,
+            image->client_data);
           if (status == MagickFalse)
             break;
         }
@@ -479,7 +499,17 @@ static MagickBooleanType EncodeImage(const ImageInfo *image_info,Image *image,
   if ((packet == (unsigned char *) NULL) || (hash_code == (short *) NULL) ||
       (hash_prefix == (short *) NULL) ||
       (hash_suffix == (unsigned char *) NULL))
-    return(MagickFalse);
+    {
+      if (packet != (unsigned char *) NULL)
+        packet=(unsigned char *) RelinquishMagickMemory(packet);
+      if (hash_code != (short *) NULL)
+        hash_code=(short *) RelinquishMagickMemory(hash_code);
+      if (hash_prefix != (short *) NULL)
+        hash_prefix=(short *) RelinquishMagickMemory(hash_prefix);
+      if (hash_suffix != (unsigned char *) NULL)
+        hash_suffix=(unsigned char *) RelinquishMagickMemory(hash_suffix);
+      return(MagickFalse);
+    }
   /*
     Initialize GIF encoder.
   */
@@ -615,9 +645,11 @@ static MagickBooleanType EncodeImage(const ImageInfo *image_info,Image *image,
         }
       }
     if (image->previous == (Image *) NULL)
-      if (QuantumTick(y,image->rows) != MagickFalse)
+      if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+          (QuantumTick(y,image->rows) != MagickFalse))
         {
-          status=MagickMonitor(SaveImageTag,y,image->rows,&image->exception);
+          status=image->progress_monitor(SaveImageTag,y,image->rows,
+            image->client_data);
           if (status == MagickFalse)
             break;
         }
@@ -820,7 +852,7 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
-      DestroyImageList(image);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
   /*
@@ -830,8 +862,6 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   if ((count == 0) || ((LocaleNCompare((char *) magick,"GIF87",5) != 0) &&
       (LocaleNCompare((char *) magick,"GIF89",5) != 0)))
     ThrowReaderException(CorruptImageError,"ImproperImageHeader");
-  global_colors=0;
-  global_colormap=(unsigned char *) NULL;
   page.width=ReadBlobLSBShort(image);
   page.height=ReadBlobLSBShort(image);
   flag=(unsigned char) ReadBlobByte(image);
@@ -895,7 +925,7 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
               header[count]='\0';
               (void) ConcatenateString(&comments,(const char *) header);
             }
-            (void) SetImageAttribute(image,"comment",comments);
+            (void) SetImageAttribute(image,"Comment",comments);
             comments=(char *) RelinquishMagickMemory(comments);
             break;
           }
@@ -933,16 +963,19 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
         AllocateNextImage(image_info,image);
         if (image->next == (Image *) NULL)
           {
-            DestroyImageList(image);
+            image=DestroyImageList(image);
             global_colormap=(unsigned char *)
               RelinquishMagickMemory(global_colormap);
             return((Image *) NULL);
           }
         image=SyncNextImageInList(image);
-        status=MagickMonitor(LoadImagesTag,TellBlob(image),GetBlobSize(image),
-          exception);
-        if (status == MagickFalse)
-          break;
+        if (image->progress_monitor != (MagickProgressMonitor) NULL)
+          {
+            status=image->progress_monitor(LoadImagesTag,TellBlob(image),
+              GetBlobSize(image),image->client_data);
+            if (status == MagickFalse)
+              break;
+          }
       }
     image_count++;
     /*
@@ -1152,6 +1185,9 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
     opacity,
     y;
 
+  ImageInfo
+    *write_info;
+
   InterlaceType
     interlace;
 
@@ -1234,14 +1270,14 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
   /*
     Write GIF header.
   */
-  if ((GetImageAttribute(image,"comment") == (ImageAttribute *) NULL) &&
-      (image_info->adjoin == MagickFalse) && (image->matte == MagickFalse))
-    (void) WriteBlob(image,6,(unsigned char *) "GIF87a");
+  write_info=CloneImageInfo(image_info);
+  if (LocaleCompare(write_info->magick,"GIF87") != 0)
+    (void) WriteBlob(image,6,(unsigned char *) "GIF89a");
   else
-    if (LocaleCompare(image_info->magick,"GIF87") == 0)
+    {
       (void) WriteBlob(image,6,(unsigned char *) "GIF87a");
-    else
-      (void) WriteBlob(image,6,(unsigned char *) "GIF89a");
+      write_info->adjoin=MagickFalse;
+    }
   page.x=image->page.x;
   page.y=image->page.y;
   if ((image->page.width != 0) && (image->page.height != 0))
@@ -1251,8 +1287,8 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
   /*
     Write images to file.
   */
-  interlace=image_info->interlace;
-  if ((image_info->adjoin != MagickFalse) && (image->next != (Image *) NULL))
+  interlace=write_info->interlace;
+  if ((write_info->adjoin != MagickFalse) && (image->next != (Image *) NULL))
     interlace=NoInterlace;
   scene=0;
   do
@@ -1265,28 +1301,36 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
           GIF requires that the image is colormapped.
         */
         GetQuantizeInfo(&quantize_info);
-        quantize_info.dither=image_info->dither;
-        if (image->matte == MagickFalse)
-          (void) QuantizeImage(&quantize_info,image);
-        else
+        quantize_info.number_colors=256;
+        quantize_info.dither=IsPaletteImage(image,&image->exception) ==
+          MagickFalse ? MagickTrue : MagickFalse;
+        (void) QuantizeImage(&quantize_info,image);
+        if ((LocaleCompare(write_info->magick,"GIF87") != 0) &&
+            (image->matte != MagickFalse))
           {
+            MagickSizeType
+              length;
+
             /*
               Set transparent pixel.
             */
             quantize_info.number_colors=255;
             (void) QuantizeImage(&quantize_info,image);
             opacity=(long) image->colors++;
-            image->colormap=(PixelPacket *) ResizeMagickMemory(image->colormap,
-              (size_t) image->colors*sizeof(*image->colormap));
+            length=(size_t) image->colors*sizeof(*image->colormap);
+            image->colormap=(PixelPacket *)
+              ResizeMagickMemory(image->colormap,length);
             if (image->colormap == (PixelPacket *) NULL)
               {
                 global_colormap=(unsigned char *)
                   RelinquishMagickMemory(global_colormap);
                 colormap=(unsigned char *) RelinquishMagickMemory(colormap);
+                write_info=DestroyImageInfo(write_info);
                 ThrowWriterException(ResourceLimitError,
                   "MemoryAllocationFailed");
               }
-            image->colormap[opacity]=image->background_color;
+            (void) QueryColorDatabase("#cccccc",image->colormap+opacity,
+              &image->exception);
             for (y=0; y < (long) image->rows; y++)
             {
               p=AcquireImagePixels(image,0,y,image->columns,1,
@@ -1306,7 +1350,8 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
           }
       }
     else
-      if (image->matte != MagickFalse)
+      if ((LocaleCompare(write_info->magick,"GIF87") != 0) &&
+          (image->matte != MagickFalse))
         {
           /*
             Identify transparent pixel index.
@@ -1349,7 +1394,7 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
       *q++=(unsigned char) 0x0;
     }
     if ((image->previous == (Image *) NULL) ||
-        (image_info->adjoin == MagickFalse))
+        (write_info->adjoin == MagickFalse))
       {
         /*
           Write global colormap.
@@ -1358,17 +1403,18 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
         c|=(8-1) << 4;  /* color resolution */
         c|=(bits_per_pixel-1);   /* size of global colormap */
         (void) WriteBlobByte(image,(unsigned char) c);
-        for (j=0; j < (long) Max(image->colors-1,1); j++)
+        for (j=0; j < (long) image->colors; j++)
           if (ColorMatch(&image->background_color,image->colormap+j))
             break;
-        (void) WriteBlobByte(image,(unsigned char) j);  /* background color */
+        (void) WriteBlobByte(image,(unsigned char)
+          j == (long) image->colors ? 0 : j);  /* background color */
         (void) WriteBlobByte(image,(unsigned char) 0x00);  /* reserved */
         length=(size_t) (3*(1 << bits_per_pixel));
         (void) WriteBlob(image,length,colormap);
         for (j=0; j < 768; j++)
           global_colormap[j]=colormap[j];
       }
-    if (LocaleCompare(image_info->magick,"GIF87") != 0)
+    if (LocaleCompare(write_info->magick,"GIF87") != 0)
       {
         /*
           Write graphics control extension.
@@ -1384,7 +1430,8 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
         (void) WriteBlobByte(image,(unsigned char)
           (opacity >= 0 ? opacity : 0));
         (void) WriteBlobByte(image,(unsigned char) 0x00);
-        if (GetImageAttribute(image,"comment") != (ImageAttribute *) NULL)
+        if ((LocaleCompare(write_info->magick,"GIF87") != 0) &&
+            (GetImageAttribute(image,"Comment") != (ImageAttribute *) NULL))
           {
             const ImageAttribute
               *attribute;
@@ -1400,7 +1447,7 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
             */
             (void) WriteBlobByte(image,(unsigned char) 0x21);
             (void) WriteBlobByte(image,(unsigned char) 0xfe);
-            attribute=GetImageAttribute(image,"comment");
+            attribute=GetImageAttribute(image,"Comment");
             p=attribute->value;
             while (strlen(p) != 0)
             {
@@ -1460,26 +1507,32 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
     */
     c=(int) Max(bits_per_pixel,2);
     (void) WriteBlobByte(image,(unsigned char) c);
-    status=EncodeImage(image_info,image,Max(bits_per_pixel,2)+1);
+    status=EncodeImage(write_info,image,Max(bits_per_pixel,2)+1);
     if (status == MagickFalse)
       {
         global_colormap=(unsigned char *)
           RelinquishMagickMemory(global_colormap);
         colormap=(unsigned char *) RelinquishMagickMemory(colormap);
+        write_info=DestroyImageInfo(write_info);
         ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
       }
     (void) WriteBlobByte(image,(unsigned char) 0x00);
     if (image->next == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
-      &image->exception);
-    if (status == MagickFalse)
-      break;
-  } while (image_info->adjoin != MagickFalse);
+    if (image->progress_monitor != (MagickProgressMonitor) NULL)
+      {
+        status=image->progress_monitor(SaveImagesTag,scene,
+          GetImageListLength(image),image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
+    scene++;
+  } while (write_info->adjoin != MagickFalse);
   (void) WriteBlobByte(image,';'); /* terminator */
   global_colormap=(unsigned char *) RelinquishMagickMemory(global_colormap);
   colormap=(unsigned char *) RelinquishMagickMemory(colormap);
+  write_info=DestroyImageInfo(write_info);
   CloseBlob(image);
   return(MagickTrue);
 }
