@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -41,10 +41,13 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
 #include "magick/color.h"
 #include "magick/color_private.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
@@ -111,7 +114,7 @@ static Image *ReadGRADIENTImage(const ImageInfo *image_info,
   (void) sscanf(image_info->filename,"%[^-]",colorname);
   if (QueryColorDatabase(colorname,&start_color,exception) == MagickFalse)
     {
-      DestroyImage(image);
+      image=DestroyImage(image);
       return((Image *) NULL);
     }
   (void) strcpy(colorname,"white");
@@ -120,7 +123,7 @@ static Image *ReadGRADIENTImage(const ImageInfo *image_info,
   (void) sscanf(image_info->filename,"%*[^-]-%s",colorname);
   if (QueryColorDatabase(colorname,&stop_color,exception) == MagickFalse)
     {
-      DestroyImage(image);
+      image=DestroyImage(image);
       return((Image *) NULL);
     }
   (void) GradientImage(image,&start_color,&stop_color);
@@ -159,6 +162,7 @@ ModuleExport void RegisterGRADIENTImage(void)
   entry->decoder=(DecoderHandler *) ReadGRADIENTImage;
   entry->adjoin=MagickFalse;
   entry->raw=MagickTrue;
+  entry->endian_support=MagickTrue;
   entry->description=AcquireString("Gradual passing from one shade to another");
   entry->module=AcquireString("GRADIENT");
   (void) RegisterMagickInfo(entry);
