@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -41,14 +41,18 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
 #include "magick/colorspace.h"
 #include "magick/constitute.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/pixel.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 
@@ -99,6 +103,9 @@ static Image *ReadGRAYImage(const ImageInfo *image_info,
   MagickBooleanType
     status;
 
+  MagickOffsetType
+    offset;
+
   register long
     i,
     x;
@@ -116,6 +123,7 @@ static Image *ReadGRAYImage(const ImageInfo *image_info,
     *scanline;
 
   unsigned long
+    depth,
     width;
 
   /*
@@ -133,7 +141,7 @@ static Image *ReadGRAYImage(const ImageInfo *image_info,
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
-      DestroyImageList(image);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
   for (i=0; i < image->offset; i++)
@@ -141,7 +149,8 @@ static Image *ReadGRAYImage(const ImageInfo *image_info,
   /*
     Allocate memory for a scanline.
   */
-  packet_size=(size_t) (image->depth > 8 ? 2 : 1);
+  depth=GetImageQuantumDepth(image,MagickTrue);
+  packet_size=(size_t) (depth/8);
   scanline=(unsigned char *)
     AcquireMagickMemory(packet_size*image->extract_info.width);
   if (scanline == (unsigned char *) NULL)
@@ -156,14 +165,15 @@ static Image *ReadGRAYImage(const ImageInfo *image_info,
       for (y=0; y < (long) image->rows; y++)
         (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
     }
-  x=(long) (packet_size*image->extract_info.x);
+  offset=(MagickOffsetType) (packet_size*image->extract_info.x);
   do
   {
     /*
       Convert raster image to pixel packets.
     */
-    if (AllocateImageColormap(image,1 << image->depth) == MagickFalse)
-      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+    if ((image->depth <= QuantumDepth) && (image->depth <= 16))
+      if (AllocateImageColormap(image,1UL << image->depth) == MagickFalse)
+        ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
     if ((image_info->ping != MagickFalse) && (image_info->number_scenes != 0))
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
@@ -176,13 +186,29 @@ static Image *ReadGRAYImage(const ImageInfo *image_info,
       q=SetImagePixels(image,0,y,image->columns,1);
       if (q == (PixelPacket *) NULL)
         break;
-      (void) PushImagePixels(image,GrayQuantum,scanline+x);
+      if ((image->depth <= QuantumDepth) && (image->depth <= 16))
+        (void) ExportQuantumPixels(image,GrayQuantum,0,scanline+offset);
+      else
+        {
+          (void) ExportQuantumPixels(image,RedQuantum,0,scanline+offset);
+          for (x=0; x < (long) image->columns; x++)
+          {
+            q->green=q->red;
+            q->blue=q->red;
+            q++;
+          }
+        }
       if (SyncImagePixels(image) == MagickFalse)
         break;
       if (image->previous == (Image *) NULL)
-        if (QuantumTick(y,image->rows) != MagickFalse)
-          if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
-            break;
+        if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+            (QuantumTick(y,image->rows) != MagickFalse))
+          {
+            status=image->progress_monitor(LoadImageTag,y,image->rows,
+              image->client_data);
+            if (status == MagickFalse)
+              break;
+          }
     }
     width=image->extract_info.height-image->rows-image->extract_info.y;
     for (j=0; j < (long) width; j++)
@@ -208,14 +234,17 @@ static Image *ReadGRAYImage(const ImageInfo *image_info,
         AllocateNextImage(image_info,image);
         if (image->next == (Image *) NULL)
           {
-            DestroyImageList(image);
+            image=DestroyImageList(image);
             return((Image *) NULL);
           }
         image=SyncNextImageInList(image);
-        status=MagickMonitor(LoadImagesTag,TellBlob(image),GetBlobSize(image),
-          exception);
-        if (status == MagickFalse)
-          break;
+        if (image->progress_monitor != (MagickProgressMonitor) NULL)
+          {
+            status=image->progress_monitor(LoadImagesTag,TellBlob(image),
+              GetBlobSize(image),image->client_data);
+            if (status == MagickFalse)
+              break;
+          }
       }
   } while (count != 0);
   scanline=(unsigned char *) RelinquishMagickMemory(scanline);
@@ -255,6 +284,7 @@ ModuleExport void RegisterGRAYImage(void)
   entry->decoder=(DecoderHandler *) ReadGRAYImage;
   entry->encoder=(EncoderHandler *) WriteGRAYImage;
   entry->raw=MagickTrue;
+  entry->endian_support=MagickTrue;
   entry->description=AcquireString("Raw gray samples");
   entry->module=AcquireString("GRAY");
   (void) RegisterMagickInfo(entry);
@@ -310,7 +340,8 @@ ModuleExport void UnregisterGRAYImage(void)
 %
 %
 */
-static MagickBooleanType WriteGRAYImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WriteGRAYImage(const ImageInfo *image_info,
+  Image *image)
 {
   long
     y;
@@ -330,6 +361,9 @@ static MagickBooleanType WriteGRAYImage(const ImageInfo *image_info,Image *image
   unsigned char
     *scanline;
 
+  unsigned long
+    depth;
+
   /*
     Open output image file.
   */
@@ -352,7 +386,8 @@ static MagickBooleanType WriteGRAYImage(const ImageInfo *image_info,Image *image
       Allocate memory for scanline.
     */
     (void) SetImageColorspace(image,RGBColorspace);
-    packet_size=(size_t) (image->depth > 8 ? 2 : 1);
+    depth=GetImageQuantumDepth(image,MagickTrue);
+    packet_size=(size_t) (depth/8);
     scanline=(unsigned char *)
       AcquireMagickMemory(packet_size*image->columns);
     if (scanline == (unsigned char *) NULL)
@@ -365,12 +400,14 @@ static MagickBooleanType WriteGRAYImage(const ImageInfo *image_info,Image *image
       p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
       if (p == (const PixelPacket *) NULL)
         break;
-      (void) PopImagePixels(image,GrayQuantum,scanline);
+      (void) ImportQuantumPixels(image,GrayQuantum,0,scanline);
       (void) WriteBlob(image,packet_size*image->columns,scanline);
       if (image->previous == (Image *) NULL)
-        if (QuantumTick(y,image->rows) != MagickFalse)
+        if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+            (QuantumTick(y,image->rows) != MagickFalse))
           {
-            status=MagickMonitor(SaveImageTag,y,image->rows,&image->exception);
+            status=image->progress_monitor(SaveImageTag,y,image->rows,
+              image->client_data);
             if (status == MagickFalse)
               break;
           }
@@ -379,10 +416,14 @@ static MagickBooleanType WriteGRAYImage(const ImageInfo *image_info,Image *image
     if (image->next == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
-      &image->exception);
-    if (status == MagickFalse)
-      break;
+    if (image->progress_monitor != (MagickProgressMonitor) NULL)
+      {
+        status=image->progress_monitor(SaveImagesTag,scene,
+          GetImageListLength(image),image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
+    scene++;
   } while (image_info->adjoin != MagickFalse);
   CloseBlob(image);
   return(MagickTrue);
