@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -42,11 +42,14 @@
 #include "magick/studio.h"
 #include "magick/attribute.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
 #include "magick/color.h"
 #include "magick/color_private.h"
 #include "magick/constitute.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/geometry.h"
+#include "magick/image_private.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
@@ -229,7 +232,7 @@ static MagickBooleanType WriteHISTOGRAMImage(const ImageInfo *image_info,
     AcquireMagickMemory(length*sizeof(*histogram));
   if (histogram == (MagickPixelPacket *) NULL)
     {
-      DestroyImage(histogram_image);
+      histogram_image=DestroyImage(histogram_image);
       ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
     }
   /*
@@ -294,10 +297,11 @@ static MagickBooleanType WriteHISTOGRAMImage(const ImageInfo *image_info,
     }
     if (SyncImagePixels(histogram_image) == MagickFalse)
       break;
-    if (QuantumTick(x,(long) histogram_image->columns))
+    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+        (QuantumTick(y,histogram_image->rows) != MagickFalse))
       {
-        status=MagickMonitor(SaveImageTag,x,histogram_image->columns,
-          &image->exception);
+        status=image->progress_monitor(SaveImageTag,y,histogram_image->rows,
+          image->client_data);
         if (status == MagickFalse)
           break;
       }
@@ -321,7 +325,7 @@ static MagickBooleanType WriteHISTOGRAMImage(const ImageInfo *image_info,
       (void) GetNumberColors(image,file,&image->exception);
       (void) fclose(file);
       (void) FormatMagickString(command,MaxTextExtent,"@%s",filename);
-      (void) SetImageAttribute(histogram_image,"comment",command);
+      (void) SetImageAttribute(histogram_image,"Comment",command);
     }
   (void) RelinquishUniqueFileResource(filename);
   /*
@@ -331,6 +335,6 @@ static MagickBooleanType WriteHISTOGRAMImage(const ImageInfo *image_info,
   (void) FormatMagickString(histogram_image->filename,MaxTextExtent,
     "miff:%s",filename);
   status=WriteImage(image_info,histogram_image);
-  DestroyImage(histogram_image);
+  histogram_image=DestroyImage(histogram_image);
   return(status);
 }
