@@ -18,7 +18,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -43,9 +43,12 @@
 #include "magick/studio.h"
 #include "magick/attribute.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
+#include "magick/color_private.h"
 #include "magick/colorspace.h"
 #include "magick/constitute.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/geometry.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
@@ -80,8 +83,6 @@ static MagickBooleanType
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method IsHTML() returns MagickTrue if the image format type is HTML.
-%
 %    o magick: This string is generally the first few bytes of an image file
 %      or blob.
 %
@@ -197,17 +198,14 @@ ModuleExport void UnregisterHTMLImage(void)
 %
 %  A description of each parameter follows.
 %
-%    o status: Method WriteHTMLImage returns True if the image is written.
-%      False is returned is there is a memory shortage or if the image file
-%      fails to write.
-%
 %    o image_info: The image info.
 %
 %    o image:  The image.
 %
 %
 */
-static MagickBooleanType WriteHTMLImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WriteHTMLImage(const ImageInfo *image_info,
+  Image *image)
 {
   char
     basename[MaxTextExtent],
@@ -259,7 +257,8 @@ static MagickBooleanType WriteHTMLImage(const ImageInfo *image_info,Image *image
           (void) CopyMagickString(url,image_info->magick,MaxTextExtent);
           (void) ConcatenateMagickString(url,":",MaxTextExtent);
           url[strlen(url)+p-image->filename]='\0';
-          (void) strncat(url,image->filename,p-image->filename+1);
+          (void) ConcatenateMagickString(url,image->filename,
+            p-image->filename+2);
           (void) CopyMagickString(image->filename,p,MaxTextExtent);
         }
     }
@@ -355,23 +354,20 @@ static MagickBooleanType WriteHTMLImage(const ImageInfo *image_info,Image *image
                   (void) WriteBlobString(image,buffer);
                 }
               geometry.x+=geometry.width;
-              if (geometry.x >= (long) image->columns)
+              if ((geometry.x+geometry.width) > image->columns)
                 {
                   geometry.x=0;
                   geometry.y+=geometry.height;
                 }
             }
       (void) WriteBlobString(image,"</map>\n");
-      if (image->montage != (char *) NULL)
-        (void) TransparentImage(image,GetOnePixel(image,0,0),
-          TransparentOpacity);
       (void) CopyMagickString(filename,image->filename,MaxTextExtent);
       (void) WriteBlobString(image,"</center>\n");
       (void) WriteBlobString(image,"</body>\n");
       (void) WriteBlobString(image,"</html>\n");
       CloseBlob(image);
       /*
-        Write the image as transparent GIF.
+        Write the image as GIF.
       */
       (void) CopyMagickString(image->filename,filename,MaxTextExtent);
       AppendImageFormat("gif",image->filename);
@@ -383,15 +379,9 @@ static MagickBooleanType WriteHTMLImage(const ImageInfo *image_info,Image *image
       /*
         Determine image map filename.
       */
+      GetPathComponent(image->filename,BasePath,filename);
+      (void) ConcatenateMagickString(filename,"_map.shtml",MaxTextExtent);
       (void) CopyMagickString(image->filename,filename,MaxTextExtent);
-      for (p=filename+strlen(filename)-1; p > (filename+1); p--)
-        if (*p == '.')
-          {
-            (void) CopyMagickString(image->filename,filename,p-filename);
-            image->filename[p-filename]='\0';
-            break;
-          }
-      (void) ConcatenateMagickString(image->filename,"_map.shtml",MaxTextExtent);
     }
   /*
     Open image map.
@@ -399,7 +389,7 @@ static MagickBooleanType WriteHTMLImage(const ImageInfo *image_info,Image *image
   status=OpenBlob(write_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
-  DestroyImageInfo(write_info);
+  write_info=DestroyImageInfo(write_info);
   /*
     Determine the size and location of each image tile.
   */
@@ -439,7 +429,7 @@ static MagickBooleanType WriteHTMLImage(const ImageInfo *image_info,Image *image
               (void) WriteBlobString(image,buffer);
             }
           geometry.x+=geometry.width;
-          if (geometry.x >= (long) image->columns)
+          if (geometry.x > (long) image->columns)
             {
               geometry.x=0;
               geometry.y+=geometry.height;
