@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -41,11 +41,14 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
 #include "magick/colorspace.h"
 #include "magick/constitute.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/geometry.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
@@ -88,10 +91,6 @@ static MagickBooleanType
 %
 %  A description of each parameter follows:
 %
-%    o image:  Method ReadJBIGImage returns a pointer to the image after
-%      reading.  A null image is returned if there is a memory shortage or
-%      if the image cannot be read.
-%
 %    o image_info: The image info.
 %
 %    o exception: return any errors or warnings in this structure.
@@ -109,13 +108,13 @@ static Image *ReadJBIGImage(const ImageInfo *image_info,
   IndexPacket
     index;
 
-  int
-    status;
-
   long
     length,
     y;
 
+  MagickBooleanType
+    status;
+
   register IndexPacket
     *indexes;
 
@@ -152,7 +151,7 @@ static Image *ReadJBIGImage(const ImageInfo *image_info,
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
-      DestroyImageList(image);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
   /*
@@ -239,9 +238,14 @@ static Image *ReadJBIGImage(const ImageInfo *image_info,
     }
     if (SyncImagePixels(image) == MagickFalse)
       break;
-    if (QuantumTick(y,image->rows) != 0)
-      if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
-        break;
+    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+        (QuantumTick(y,image->rows) != MagickFalse))
+      {
+        status=image->progress_monitor(LoadImageTag,y,image->rows,
+          image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
   }
   /*
     Free scale resource.
@@ -369,10 +373,6 @@ ModuleExport void UnregisterJBIGImage(void)
 %
 %  A description of each parameter follows.
 %
-%    o status: Method WriteJBIGImage returns True if the image is written.
-%      False is returned is there is a memory shortage or if the image file
-%      fails to write.
-%
 %    o image_info: The image info.
 %
 %    o image:  The image.
@@ -479,9 +479,11 @@ static MagickBooleanType WriteJBIGImage(const ImageInfo *image_info,Image *image
        }
       if (bit != 0)
         *q++=byte << (8-bit);
-      if (QuantumTick(y,image->rows) != 0)
+      if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+          (QuantumTick(y,image->rows) != MagickFalse))
         {
-          status=MagickMonitor(SaveImageTag,y,image->rows,&image->exception);
+          status=image->progress_monitor(SaveImageTag,y,image->rows,
+            image->client_data);
           if (status == MagickFalse)
             break;
         }
@@ -538,10 +540,14 @@ static MagickBooleanType WriteJBIGImage(const ImageInfo *image_info,Image *image
     if (image->next == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
-      &image->exception);
-    if (status == MagickFalse)
-      break;
+    if (image->progress_monitor != (MagickProgressMonitor) NULL)
+      {
+        status=image->progress_monitor(SaveImagesTag,scene,
+          GetImageListLength(image),image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
+    scene++;
   } while (image_info->adjoin != MagickFalse);
   CloseBlob(image);
   return(MagickTrue);
