@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -46,13 +46,16 @@
 #include "magick/studio.h"
 #include "magick/attribute.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
 #include "magick/color.h"
 #include "magick/color_private.h"
 #include "magick/colorspace.h"
 #include "magick/constitute.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/geometry.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/log.h"
 #include "magick/magick.h"
@@ -217,7 +220,7 @@ static MagickBooleanType EmitMessage(j_common_ptr jpeg_info,int level)
     {
       if ((jpeg_info->err->num_warnings == 0) ||
           (jpeg_info->err->trace_level >= 3))
-        ThrowBinaryException(CorruptImageError,(char *) message,
+        ThrowBinaryException(CorruptImageWarning,(char *) message,
           image->filename);
       jpeg_info->err->num_warnings++;
     }
@@ -317,7 +320,7 @@ static boolean ReadComment(j_decompress_ptr jpeg_info)
   for (p=comment; i-- >= 0; p++)
     *p=(char) GetCharacter(jpeg_info);
   *p='\0';
-  (void) SetImageAttribute(image,"comment",comment);
+  (void) SetImageAttribute(image,"Comment",comment);
   comment=(char *) RelinquishMagickMemory(comment);
   return(MagickTrue);
 }
@@ -388,12 +391,12 @@ static boolean ReadICCProfile(j_decompress_ptr jpeg_info)
   if (icc_profile != (StringInfo *) NULL)
     {
       ConcatenateStringInfo(icc_profile,profile);
-      DestroyStringInfo(profile);
+      profile=DestroyStringInfo(profile);
     }
   else
     {
       status=SetImageProfile(image,"icc",profile);
-      DestroyStringInfo(profile);
+      profile=DestroyStringInfo(profile);
       if (status == MagickFalse)
         ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
           image->filename);
@@ -510,12 +513,12 @@ static boolean ReadIPTCProfile(j_decompress_ptr jpeg_info)
   if (iptc_profile != (StringInfo *) NULL)
     {
       ConcatenateStringInfo(iptc_profile,profile);
-      DestroyStringInfo(profile);
+      profile=DestroyStringInfo(profile);
     }
   else
     {
       status=SetImageProfile(image,"iptc",profile);
-      DestroyStringInfo(profile);
+      profile=DestroyStringInfo(profile);
       if (status == MagickFalse)
         ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
           image->filename);
@@ -583,7 +586,7 @@ static boolean ReadProfile(j_decompress_ptr jpeg_info)
         (void) CopyMagickString(name,"xmp",MaxTextExtent);
     }
   status=SetImageProfile(image,name,profile);
-  DestroyStringInfo(profile);
+  profile=DestroyStringInfo(profile);
   if (status == MagickFalse)
     ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
       image->filename);
@@ -663,6 +666,9 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
     debug,
     status;
 
+  MagickSizeType
+    number_pixels;
+
   register IndexPacket
     *indexes;
 
@@ -683,8 +689,6 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
 
   unsigned long
     index,
-    quality,
-    number_pixels,
     units;
 
   /*
@@ -701,7 +705,7 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
-      DestroyImageList(image);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
   /*
@@ -714,15 +718,14 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
   error_manager.image=image;
   if (setjmp(error_manager.error_recovery) != 0)
     {
-      if (jpeg_pixels != (JSAMPLE *) NULL)
-        jpeg_pixels=(unsigned char *) RelinquishMagickMemory(jpeg_pixels);
       jpeg_destroy_decompress(&jpeg_info);
       InheritException(exception,&image->exception);
-      number_pixels=image->columns*image->rows;
+      CloseBlob(image);
+      number_pixels=(MagickSizeType) image->columns*image->rows;
       if (number_pixels != 0)
         return(GetFirstImageInList(image));
-      DestroyImage(image);
-      return((Image *) NULL);
+      image=DestroyImage(image);
+      return(image);
     }
   jpeg_info.client_data=(void *) &error_manager;
   jpeg_create_decompress(&jpeg_info);
@@ -740,8 +743,8 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
     Set image resolution.
   */
   units=0;
-  if ((jpeg_info.saw_JFIF_marker != 0) && (jpeg_info.X_density != 1U) &&
-      (jpeg_info.Y_density != 1U))
+  if ((jpeg_info.saw_JFIF_marker != 0) && (jpeg_info.X_density != 1) &&
+      (jpeg_info.Y_density != 1))
     {
       image->x_resolution=(double) jpeg_info.X_density;
       image->y_resolution=(double) jpeg_info.Y_density;
@@ -751,7 +754,7 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
     image->units=PixelsPerInchResolution;
   if (units == 2)
     image->units=PixelsPerCentimeterResolution;
-  number_pixels=image->columns*image->rows;
+  number_pixels=(MagickSizeType) image->columns*image->rows;
   if (number_pixels != 0)
     {
       double
@@ -829,11 +832,11 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
       (void) LogMagickEvent(CoderEvent,GetMagickModule(),"Geometry: %dx%d",
         (int) jpeg_info.output_width,(int) jpeg_info.output_height);
     }
-  quality=0;
+  image->quality=0;
 #ifdef D_LOSSLESS_SUPPORTED
   if (image->compression == LosslessJPEGCompression)
     {
-      quality=100;
+      image->quality=100;
       (void) LogMagickEvent(CoderEvent,GetMagickModule(),
         "Quality: 100 (lossless)");
     }
@@ -902,15 +905,15 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
          {
            if ((hashval >= hash[i]) || (sum >= sums[i]))
              {
-               quality=(unsigned long) i+1;
+               image->quality=(unsigned long) i+1;
                if (image->debug != MagickFalse)
                  {
                    if ((hashval > hash[i]) || (sum > sums[i]))
                      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                       "Quality: %ld (approximate)",quality);
+                       "Quality: %ld (approximate)",image->quality);
                    else
                      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                       "Quality: %ld",quality);
+                       "Quality: %ld",image->quality);
                  }
                break;
              }
@@ -953,7 +956,7 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
            {
              if ((hashval >= bwhash[i]) || (sum >= bwsum[i]))
                {
-                 quality=(unsigned long) i+1;
+                 image->quality=(unsigned long) i+1;
                  if (image->debug != MagickFalse)
                    {
                      if ((hashval > bwhash[i]) || (sum > bwsum[i]))
@@ -968,8 +971,6 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
            }
          }
   }
-  (void) FormatMagickString(value,MaxTextExtent,"%lu",quality);
-  (void) SetImageAttribute(image,"JPEG-Quality",value);
   (void) FormatMagickString(value,MaxTextExtent,"%ld",(long)
     jpeg_info.out_color_space);
   (void) SetImageAttribute(image,"JPEG-Colorspace",value);
@@ -1045,6 +1046,36 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
   /*
     Convert JPEG pixels to pixel packets.
   */
+  if (setjmp(error_manager.error_recovery) != 0)
+    {
+      jpeg_pixels=(unsigned char *) RelinquishMagickMemory(jpeg_pixels);
+      jpeg_destroy_decompress(&jpeg_info);
+      InheritException(exception,&image->exception);
+      CloseBlob(image);
+      number_pixels=(MagickSizeType) image->columns*image->rows;
+      if (number_pixels != 0)
+        return(GetFirstImageInList(image));
+      image=DestroyImage(image);
+      return(image);
+    }
+  if (jpeg_info.quantize_colors != MagickFalse)
+    {
+      image->colors=jpeg_info.actual_number_of_colors;
+      if (jpeg_info.out_color_space == JCS_GRAYSCALE)
+        for (i=0; i < (long) image->colors; i++)
+        {
+          image->colormap[i].red=ScaleCharToQuantum(jpeg_info.colormap[0][i]);
+          image->colormap[i].green=image->colormap[i].red;
+          image->colormap[i].blue=image->colormap[i].red;
+        } 
+      else
+        for (i=0; i < (long) image->colors; i++)
+        {
+          image->colormap[i].red=ScaleCharToQuantum(jpeg_info.colormap[0][i]);
+          image->colormap[i].green=ScaleCharToQuantum(jpeg_info.colormap[1][i]);
+          image->colormap[i].blue=ScaleCharToQuantum(jpeg_info.colormap[2][i]);
+        }
+    }
   scanline[0]=(JSAMPROW) jpeg_pixels;
   for (y=0; y < (long) image->rows; y++)
   {
@@ -1097,17 +1128,15 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
         }
     if (SyncImagePixels(image) == MagickFalse)
       break;
-    if (QuantumTick(y,image->rows) != MagickFalse)
-      if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
-        break;
+    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+        (QuantumTick(y,image->rows) != MagickFalse))
+      {
+        status=image->progress_monitor(LoadImageTag,y,image->rows,
+          image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
   }
-  if (jpeg_info.quantize_colors != MagickFalse)
-    for (i=0; i < jpeg_info.actual_number_of_colors; i++)
-    {
-      image->colormap[i].red=ScaleCharToQuantum(jpeg_info.colormap[0][i]);
-      image->colormap[i].green=ScaleCharToQuantum(jpeg_info.colormap[1][i]);
-      image->colormap[i].blue=ScaleCharToQuantum(jpeg_info.colormap[2][i]);
-    }
   if (image->colorspace == CMYKColorspace)
     {
       /*
@@ -1211,7 +1240,7 @@ ModuleExport void RegisterJPEGImage(void)
 #endif
   entry->adjoin=MagickFalse;
   entry->description=
-    AcquireString("Progessive Joint Photographic Experts Group JFIF format");
+    AcquireString("Progessive Joint Photographic Experts Group JFIF");
   entry->module=AcquireString("JPEG");
   (void) RegisterMagickInfo(entry);
 }
@@ -1391,7 +1420,7 @@ static void WriteProfile(j_compress_ptr jpeg_info,Image *image)
             (unsigned int) (length+tag_length));
         }
       }
-    if (LocaleCompare(name,"8IPTC") == 0)
+    if (LocaleCompare(name,"IPTC") == 0)
       {
         unsigned long
           roundup;
@@ -1416,7 +1445,7 @@ static void WriteProfile(j_compress_ptr jpeg_info,Image *image)
       "%s profile: %lu bytes",name,(unsigned long) profile->length);
     name=GetNextImageProfile(image);
   }
-  DestroyStringInfo(custom_profile);
+  custom_profile=DestroyStringInfo(custom_profile);
 }
 
 static void JPEGDestinationManager(j_compress_ptr cinfo,Image * image)
@@ -1526,9 +1555,6 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
   struct jpeg_error_mgr
     jpeg_error;
 
-  unsigned long
-    quality;
-
   /*
     Open image file.
   */
@@ -1572,7 +1598,7 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
     case GRAYColorspace:
     {
       jpeg_info.in_color_space=JCS_GRAYSCALE;
-      SetImageColorspace(image,GRAYColorspace);
+      (void) SetImageColorspace(image,GRAYColorspace);
       break;
     }
     default:
@@ -1617,6 +1643,7 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
       if (image->units == PixelsPerCentimeterResolution)
         jpeg_info.density_unit=(UINT8) 2;
     }
+  jpeg_info.dct_method=JDCT_FLOAT;
   number_pixels=(MagickSizeType) image->columns*image->rows;
   if (number_pixels == (MagickSizeType) ((size_t) number_pixels))
     {
@@ -1644,28 +1671,16 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
 #else
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-      "Interlace:  nonprogressive");
+      "Interlace: nonprogressive");
 #endif
-  quality=image_info->quality;
-  if (quality == UndefinedCompressionQuality)
+  if ((image->compression != LosslessJPEGCompression) &&
+      (image->quality != UndefinedCompressionQuality) &&
+      (image->quality <= 100))
     {
-      quality=75;
-      attribute=GetImageAttribute(image,"JPEG-Quality");
-      if ((attribute != (const ImageAttribute *) NULL) &&
-          (attribute->value != (char *) NULL))
-        {
-          quality=(unsigned long) atol(attribute->value);
-          if (image->debug != MagickFalse)
-            (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  quality=%lu",
-              quality);
-        }
-    }
-  if ((image->compression != LosslessJPEGCompression) && (quality <= 100))
-    {
-      jpeg_set_quality(&jpeg_info,(int) quality,MagickTrue);
+      jpeg_set_quality(&jpeg_info,(int) image->quality,MagickTrue);
       if (image->debug != MagickFalse)
         (void) LogMagickEvent(CoderEvent,GetMagickModule(),"Quality: %lu",
-          quality);
+          image->quality);
     }
   else
     {
@@ -1674,7 +1689,7 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
       if (image->debug != MagickFalse)
         (void) LogMagickEvent(CoderEvent,GetMagickModule(),"Quality: 100");
 #else
-      if (quality < 100)
+      if (image->quality < 100)
         (void) ThrowMagickException(&image->exception,GetMagickModule(),
           CoderWarning,"LosslessToLossyJPEGConversion",image->filename);
       else
@@ -1683,8 +1698,8 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
             point_transform,
             predictor;
 
-          predictor=quality/100;  /* range 1-7 */
-          point_transform=quality % 20;  /* range 0-15 */
+          predictor=image->quality/100;  /* range 1-7 */
+          point_transform=image->quality % 20;  /* range 0-15 */
           jpeg_simple_lossless(&jpeg_info,predictor,point_transform);
           if (image->debug != MagickFalse)
             {
@@ -1713,7 +1728,7 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
     }
   if (sampling_factors == (char *) NULL)
     {
-      if (quality >= 90)
+      if (image->quality >= 90)
         for (i=0; i < MAX_COMPONENTS; i++)
         {
           jpeg_info.comp_info[i].h_samp_factor=1;
@@ -1872,7 +1887,7 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
   /*
     Write JPEG profiles.
   */
-  attribute=GetImageAttribute(image,"comment");
+  attribute=GetImageAttribute(image,"Comment");
   if ((attribute != (const ImageAttribute *) NULL) &&
       (attribute->value != (char *) NULL))
     for (i=0; i < (long) strlen(attribute->value); i+=65533L)
@@ -1903,10 +1918,11 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
             p++;
           }
           (void) jpeg_write_scanlines(&jpeg_info,scanline,1);
-          if (QuantumTick(y,image->rows) != MagickFalse)
+          if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+              (QuantumTick(y,image->rows) != MagickFalse))
             {
-              status=MagickMonitor(SaveImageTag,y,image->rows,
-                &image->exception);
+              status=image->progress_monitor(SaveImageTag,y,image->rows,
+                image->client_data);
               if (status == MagickFalse)
                 break;
             }
@@ -1928,10 +1944,11 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
               p++;
             }
             (void) jpeg_write_scanlines(&jpeg_info,scanline,1);
-            if (QuantumTick(y,image->rows) != MagickFalse)
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(y,image->rows) != MagickFalse))
               {
-                status=MagickMonitor(SaveImageTag,y,image->rows,
-                  &image->exception);
+                status=image->progress_monitor(SaveImageTag,y,image->rows,
+                  image->client_data);
                 if (status == MagickFalse)
                   break;
               }
@@ -1956,10 +1973,11 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
               p++;
             }
             (void) jpeg_write_scanlines(&jpeg_info,scanline,1);
-            if (QuantumTick(y,image->rows) != MagickFalse)
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(y,image->rows) != MagickFalse))
               {
-                status=MagickMonitor(SaveImageTag,y,image->rows,
-                  &image->exception);
+                status=image->progress_monitor(SaveImageTag,y,image->rows,
+                  image->client_data);
                 if (status == MagickFalse)
                   break;
               }
@@ -1979,9 +1997,11 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
           p++;
         }
         (void) jpeg_write_scanlines(&jpeg_info,scanline,1);
-        if (QuantumTick(y,image->rows) != MagickFalse)
+        if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+            (QuantumTick(y,image->rows) != MagickFalse))
           {
-            status=MagickMonitor(SaveImageTag,y,image->rows,&image->exception);
+            status=image->progress_monitor(SaveImageTag,y,image->rows,
+              image->client_data);
             if (status == MagickFalse)
               break;
           }
@@ -2003,10 +2023,11 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
             p++;
           }
           (void) jpeg_write_scanlines(&jpeg_info,scanline,1);
-          if (QuantumTick(y,image->rows) != MagickFalse)
+          if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+              (QuantumTick(y,image->rows) != MagickFalse))
             {
-              status=MagickMonitor(SaveImageTag,y,image->rows,
-                &image->exception);
+              status=image->progress_monitor(SaveImageTag,y,image->rows,
+                image->client_data);
               if (status == MagickFalse)
                 break;
             }
@@ -2031,10 +2052,11 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
             p++;
           }
           (void) jpeg_write_scanlines(&jpeg_info,scanline,1);
-          if (QuantumTick(y,image->rows) != MagickFalse)
+          if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+              (QuantumTick(y,image->rows) != MagickFalse))
             {
-              status=MagickMonitor(SaveImageTag,y,image->rows,
-                &image->exception);
+              status=image->progress_monitor(SaveImageTag,y,image->rows,
+                image->client_data);
               if (status == MagickFalse)
                 break;
             }
