@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -42,9 +42,12 @@
 #include "magick/studio.h"
 #include "magick/annotate.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
 #include "magick/draw.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
@@ -99,7 +102,6 @@ static Image *ReadLABELImage(const ImageInfo *image_info,
     metrics;
 
   unsigned long
-    height,
     width;
 
   /*
@@ -114,40 +116,36 @@ static Image *ReadLABELImage(const ImageInfo *image_info,
   image=AllocateImage(image_info);
   draw_info=CloneDrawInfo(image_info,(DrawInfo *) NULL);
   draw_info->fill=image_info->pen;
-  draw_info->text=TranslateText(image_info,image,image_info->filename);
-  if (draw_info->text == (char *) NULL)
-    ThrowReaderException(CoderError,"UnableToTranslateText");
+  draw_info->text=AcquireString(image_info->filename);
   if ((image->columns != 0) || (image->rows != 0))
     {
       /*
         Fit label to canvas size.
       */
-      status=GetTypeMetrics(image,draw_info,&metrics);
+      status=GetMultilineTypeMetrics(image,draw_info,&metrics);
       for ( ; status != MagickFalse; draw_info->pointsize*=2)
       {
         width=(unsigned long) (metrics.width+metrics.max_advance/2+0.5);
-        height=(unsigned long) (metrics.ascent-metrics.descent+0.5);
         if (((image->columns != 0) && (width >= image->columns)) ||
-            ((image->rows != 0) && (height >= image->rows)))
+            ((image->rows != 0) && (metrics.height >= image->rows)))
           break;
-        status=GetTypeMetrics(image,draw_info,&metrics);
+        status=GetMultilineTypeMetrics(image,draw_info,&metrics);
       }
       for ( ; status != MagickFalse; draw_info->pointsize--)
       {
         width=(unsigned long) (metrics.width+metrics.max_advance/2+0.5);
-        height=(unsigned long) (metrics.ascent-metrics.descent+0.5);
         if ((image->columns != 0) && (width <= image->columns) &&
-           ((image->rows == 0) || (height <= image->rows)))
+           ((image->rows == 0) || (metrics.height <= image->rows)))
           break;
-        if ((image->rows != 0) && (height <= image->rows) &&
+        if ((image->rows != 0) && (metrics.height <= image->rows) &&
            ((image->columns == 0) || (width <= image->columns)))
           break;
         if (draw_info->pointsize < 2.0)
           break;
-        status=GetTypeMetrics(image,draw_info,&metrics);
+        status=GetMultilineTypeMetrics(image,draw_info,&metrics);
       }
     }
-  status=GetTypeMetrics(image,draw_info,&metrics);
+  status=GetMultilineTypeMetrics(image,draw_info,&metrics);
   if (status == MagickFalse)
     ThrowReaderException(TypeError,"UnableToGetTypeMetrics");
   (void) FormatMagickString(geometry,MaxTextExtent,"+%g+%g",
@@ -156,14 +154,14 @@ static Image *ReadLABELImage(const ImageInfo *image_info,
     image->columns=(unsigned long) (metrics.width+metrics.max_advance/2+0.5);
   if (image->rows == 0)
     {
-      image->rows=(unsigned long) (metrics.ascent-metrics.descent+0.5);
+      image->rows=(unsigned long) metrics.height;
       (void) FormatMagickString(geometry,MaxTextExtent,"+%g+%g",
         image->columns/2.0-metrics.width/2.0,metrics.ascent);
     }
   draw_info->geometry=AcquireString(geometry);
   SetImage(image,OpaqueOpacity);
   (void) AnnotateImage(image,draw_info);
-  DestroyDrawInfo(draw_info);
+  draw_info=DestroyDrawInfo(draw_info);
   return(GetFirstImageInList(image));
 }
 
