@@ -49,9 +49,12 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
 #include "magick/color_private.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
@@ -84,7 +87,7 @@ typedef struct
 }
 MATHeader;
 
-static void InsertRow(unsigned char *p, int y, Image * image)
+static void InsertRow(unsigned char *p, int y, Image *image)
 {
   int x;
   register PixelPacket *q;
@@ -133,7 +136,8 @@ static void InsertRow(unsigned char *p, int y, Image * image)
   }
 }
 
-static void InsertFloatRow(double *p, int y, Image * image, double Min, double Max)
+static void InsertFloatRow(double *p, int y, Image * image, double Min,
+  double Max)
 {
   double f;
   int x;
@@ -159,8 +163,8 @@ static void InsertFloatRow(double *p, int y, Image * image, double Min, double M
   return;
 }
 
-static void InsertComplexFloatRow(double *p, int y, Image * image, double Min,
-                                  double Max)
+static void InsertComplexFloatRow(double *p,int y,Image *image,double Min,
+  double Max)
 {
   double f;
   int x;
@@ -209,30 +213,30 @@ static void InsertComplexFloatRow(double *p, int y, Image * image, double Min,
 }
 
 /*This function reads one block of unsigned shortS*/
-static void ReadBlobWordLSB(Image * I, size_t len, unsigned short *data)
+static void ReadBlobWordLSB(Image *image,size_t len,unsigned short *data)
 {
   while (len >= 2)
   {
-    *data++ = ReadBlobLSBShort(I);
+    *data++ = ReadBlobLSBShort(image);
     len -= 2;
   }
   if (len > 0)
-    (void) SeekBlob(I, len, SEEK_CUR);
+    (void) SeekBlob(image,len,SEEK_CUR);
 }
 
 /*This function reads one block of unsigned shortS*/
-static void ReadBlobWordMSB(Image * I, size_t len, unsigned short *data)
+static void ReadBlobWordMSB(Image *image, size_t len, unsigned short *data)
 {
   while (len >= 2)
   {
-    *data++ = ReadBlobMSBShort(I);
+    *data++ = ReadBlobMSBShort(image);
     len -= 2;
   }
   if (len > 0)
-    (void) SeekBlob(I, len, SEEK_CUR);
+    (void) SeekBlob(image,len,SEEK_CUR);
 }
 
-static double ReadBlobLSBdouble(Image * image)
+static double ReadBlobLSBdouble(Image *image)
 {
   typedef union
   {
@@ -268,18 +272,18 @@ static double ReadBlobLSBdouble(Image * image)
   return (buffer.d);
 }
 
-static void ReadBlobDoublesLSB(Image * I, size_t len, double *data)
+static void ReadBlobDoublesLSB(Image *image, size_t len, double *data)
 {
   while (len >= 8)
   {
-    *data++ = ReadBlobLSBdouble(I);
+    *data++ = ReadBlobLSBdouble(image);
     len -= sizeof(double);
   }
   if (len > 0)
-    (void) SeekBlob(I, len, SEEK_CUR);
+    (void) SeekBlob(image,len,SEEK_CUR);
 }
 
-static double ReadBlobMSBdouble(Image * image)
+static double ReadBlobMSBdouble(Image *image)
 {
   typedef union
   {
@@ -314,15 +318,15 @@ static double ReadBlobMSBdouble(Image * image)
   return (buffer.d);
 }
 
-static void ReadBlobDoublesMSB(Image * I, size_t len, double *data)
+static void ReadBlobDoublesMSB(Image *image,size_t len,double *data)
 {
   while (len >= 8)
   {
-    *data++ = ReadBlobMSBdouble(I);
+    *data++ = ReadBlobMSBdouble(image);
     len -= sizeof(double);
   }
   if (len > 0)
-    (void) SeekBlob(I, len, SEEK_CUR);
+    (void) SeekBlob(image,len,SEEK_CUR);
 }
 
 /*
@@ -346,17 +350,13 @@ static void ReadBlobDoublesMSB(Image * I, size_t len, double *data)
 %
 %  A description of each parameter follows:
 %
-%    o image:  Method ReadMATImage returns a pointer to the image after
-%      reading. A null image is returned if there is a memory shortage or if
-%      the image cannot be read.
-%
 %    o image_info: The image info.
 %
 %    o exception: return any errors or warnings in this structure.
 %
 %
 */
-static Image *ReadMATImage(const ImageInfo * image_info, ExceptionInfo * exception)
+static Image *ReadMATImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
   Image *image,
    *rotated_image;
@@ -377,13 +377,12 @@ static Image *ReadMATImage(const ImageInfo * image_info, ExceptionInfo * excepti
   /*
      Open image file.
    */
-  image = AllocateImage(image_info);
-
-  status = OpenBlob(image_info, image, ReadBinaryBlobMode, exception);
+  image=AllocateImage(image_info);
+  status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
-      DestroyImageList(image);
-      return(MagickFalse);
+      image=DestroyImageList(image);
+      return((Image *) NULL);
     }
   /*
      Read MATLAB image.
@@ -601,12 +600,12 @@ static Image *ReadMATImage(const ImageInfo * image_info, ExceptionInfo * excepti
   rotated_image = RotateImage(image, 90.0, exception);
   if (rotated_image != (Image *) NULL)
   {
-    DestroyImage(image);
+    image=DestroyImage(image);
     image = FlopImage(rotated_image, exception);
     if (image == NULL)
       image = rotated_image;    /*Obtain something if flop operation fails */
     else
-      DestroyImage(rotated_image);
+      rotated_image=DestroyImage(rotated_image);
   }
 
   return (image);
