@@ -17,7 +17,7 @@
 %                                 July 2001                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -41,8 +41,11 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
-#include "magick/error.h"
+#include "magick/blob_private.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
@@ -1057,7 +1060,7 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
-      DestroyImageList(image);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
   image->columns=1;
@@ -1075,7 +1078,7 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
       blob=(unsigned char *) AcquireMagickMemory(length);
       if (blob == (unsigned char *) NULL)
         {
-          DestroyImage(buff);
+          buff=DestroyImage(buff);
           ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
         }
       AttachBlob(buff->blob,blob,length);
@@ -1104,11 +1107,12 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
       profile=AcquireStringInfo(GetBlobSize(buff));
       SetStringInfoDatum(profile,GetBlobStreamData(buff));
       status=SetImageProfile(image,"iptc",profile);
-      DestroyStringInfo(profile);
+      profile=DestroyStringInfo(profile);
       if (status == MagickFalse)
         ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-      DetachBlob(buff->blob);
-      DestroyImage(buff);
+      blob=DetachBlob(buff->blob);
+      blob=(unsigned char *) RelinquishMagickMemory(blob);
+      buff=DestroyImage(buff);
     }
   if (LocaleNCompare(image_info->magick,"APP1",4) == 0)
     {
@@ -1122,7 +1126,7 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
       blob=(unsigned char *) AcquireMagickMemory(length);
       if (blob == (unsigned char *) NULL)
         {
-          DestroyImage(buff);
+          buff=DestroyImage(buff);
           ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
         }
       AttachBlob(buff->blob,blob,length);
@@ -1139,28 +1143,29 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
               (profile->datum == (unsigned char *) NULL) ||
               (profile->length == 0))
             {
-              DetachBlob(buff->blob);
+              blob=DetachBlob(buff->blob);
               blob=(char *) RelinquishMagickMemory(blob);
-              DestroyImage(buff);
-              ThrowReaderException(CoderError,"NoIPTCProfileAvailable");
+              buff=DestroyImage(buff);
+              ThrowReaderException(CoderError,"NoIPTCProfileIsAvailable");
             }
           iptc=AllocateImage((ImageInfo *) NULL);
           if (iptc == (Image *) NULL)
             {
-              DetachBlob(buff->blob);
+              blob=DetachBlob(buff->blob);
               blob=(char *) RelinquishMagickMemory(blob);
-              DestroyImage(buff);
+              buff=DestroyImage(buff);
               ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
             }
           AttachBlob(iptc->blob,profile->datum,profile->length);
           result=jpeg_embed(image,buff,iptc);
-          DetachBlob(iptc->blob);
-          DestroyImage(iptc);
+          blob=DetachBlob(iptc->blob);
+          blob=(char *) RelinquishMagickMemory(blob);
+          iptc=DestroyImage(iptc);
           if (result == 0)
             {
-              DetachBlob(buff->blob);
+              blob=DetachBlob(buff->blob);
               blob=(char *) RelinquishMagickMemory(blob);
-              DestroyImage(buff);
+              buff=DestroyImage(buff);
               ThrowReaderException(CoderError,"JPEGEmbeddingFailed");
             }
         }
@@ -1212,11 +1217,12 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
       profile=AcquireStringInfo(GetBlobSize(buff));
       SetStringInfoDatum(profile,GetBlobStreamData(buff));
       status=SetImageProfile(image,name,profile);
-      DestroyStringInfo(profile);
+      profile=DestroyStringInfo(profile);
       if (status == MagickFalse)
         ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-      DetachBlob(buff->blob);
-      DestroyImage(buff);
+      blob=DetachBlob(buff->blob);
+      blob=(char *) RelinquishMagickMemory(blob);
+      buff=DestroyImage(buff);
     }
   if ((LocaleCompare(image_info->magick,"ICC") == 0) ||
       (LocaleCompare(image_info->magick,"ICM") == 0))
@@ -1227,7 +1233,7 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
       blob=(unsigned char *) AcquireMagickMemory(length);
       if (blob == (unsigned char *) NULL)
         {
-          DestroyImage(buff);
+          buff=DestroyImage(buff);
           ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
         }
       AttachBlob(buff->blob,blob,length);
@@ -1241,9 +1247,10 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
       profile=AcquireStringInfo(GetBlobSize(buff));
       SetStringInfoDatum(profile,GetBlobStreamData(buff));
       (void) SetImageProfile(image,"icc",profile);
-      DestroyStringInfo(profile);
-      DetachBlob(buff->blob);
-      DestroyImage(buff);
+      profile=DestroyStringInfo(profile);
+      blob=DetachBlob(buff->blob);
+      blob=(unsigned char *) RelinquishMagickMemory(blob);
+      buff=DestroyImage(buff);
     }
   if (LocaleCompare(image_info->magick,"IPTC") == 0)
     {
@@ -1253,7 +1260,7 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
       blob=(unsigned char *) AcquireMagickMemory(length);
       if (blob == (unsigned char *) NULL)
         {
-          DestroyImage(buff);
+          buff=DestroyImage(buff);
           ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
         }
       AttachBlob(buff->blob,blob,length);
@@ -1287,9 +1294,10 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
       profile->datum[11]=length & 0xff;
       SetStringInfoDatum(profile,GetBlobStreamData(buff));
       (void) SetImageProfile(image,"iptc",profile);
-      DestroyStringInfo(profile);
-      DetachBlob(buff->blob);
-      DestroyImage(buff);
+      profile=DestroyStringInfo(profile);
+      blob=DetachBlob(buff->blob);
+      blob=(unsigned char *) RelinquishMagickMemory(blob);
+      buff=DestroyImage(buff);
     }
   CloseBlob(image);
   return(GetFirstImageInList(image));
@@ -2180,8 +2188,8 @@ static MagickBooleanType WriteMETAImage(const ImageInfo *image_info,
         ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
       AttachBlob(buff->blob,profile->datum,profile->length);
       format8BIM(buff,image);
-      DetachBlob(buff->blob);
-      DestroyImage(buff);
+      (void) DetachBlob(buff->blob);
+      buff=DestroyImage(buff);
       CloseBlob(image);
       return(MagickTrue);
     }
@@ -2213,8 +2221,8 @@ static MagickBooleanType WriteMETAImage(const ImageInfo *image_info,
         ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
       AttachBlob(buff->blob,info,length);
       formatIPTC(buff,image);
-      DetachBlob(buff->blob);
-      DestroyImage(buff);
+      (void) DetachBlob(buff->blob);
+      buff=DestroyImage(buff);
       CloseBlob(image);
       return(MagickTrue);
     }
@@ -2245,7 +2253,7 @@ static MagickBooleanType WriteMETAImage(const ImageInfo *image_info,
       */
       profile=GetImageProfile(image,"icc");
       if (profile == (StringInfo *) NULL)
-        ThrowWriterException(CoderError,"NoColorProfileAvailable");
+        ThrowWriterException(CoderError,"NoColorProfileIsAvailable");
       status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
       if (status == MagickFalse)
         return(status);
