@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -42,19 +42,23 @@
 #include "magick/studio.h"
 #include "magick/attribute.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
 #include "magick/color.h"
 #include "magick/color_private.h"
 #include "magick/colorspace.h"
 #include "magick/constitute.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/hashmap.h"
 #include "magick/geometry.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
 #include "magick/option.h"
+#include "magick/pixel.h"
 #include "magick/profile.h"
 #include "magick/static.h"
 #include "magick/string_.h"
@@ -354,8 +358,8 @@ static void RelinquishBZIPMemory(void *context,void *memory)
 static Image *ReadMIFFImage(const ImageInfo *image_info,
   ExceptionInfo *exception)
 {
-#define CompressMaxExtent(x) \
-  Max((x)+((x)/100)+600,(x)+(((x)+7) >> 3)+(((x)+63) >> 6)+11)
+#define BZipMaxExtent(x)  ((x)+((x)/100)+600)
+#define ZipMaxExtent(x)  ((x)+(((x)+7) >> 3)+(((x)+63) >> 6)+11)
 
 #if defined(HasBZLIB)
   bz_stream
@@ -447,7 +451,7 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
-      DestroyImageList(image);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
   /*
@@ -505,7 +509,7 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
           if (comment == (char *) NULL)
             ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
           *p='\0';
-          (void) SetImageAttribute(image,"comment",comment);
+          (void) SetImageAttribute(image,"Comment",comment);
           comment=(char *) RelinquishMagickMemory(comment);
           c=ReadBlobByte(image);
         }
@@ -513,41 +517,51 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
         if (isalnum(c) != MagickFalse)
           {
             /*
-              Determine a keyword and its value.
+              Get the keyword.
             */
             p=keyword;
             do
             {
+              if (isspace((int) ((unsigned char) c)) != 0)
+                break;
+              if (c == (int) '=')
+                break;
               if ((size_t) (p-keyword) < (MaxTextExtent-1))
                 *p++=(char) c;
               c=ReadBlobByte(image);
-            } while ((isalnum(c) != MagickFalse) || (c == (int) '-'));
+            } while (c != EOF);
             *p='\0';
-            while ((isspace((int) ((unsigned char) c)) != 0) ||
-                   (c == (int) '='))
-              c=ReadBlobByte(image);
             p=options;
-            while ((c != (int) '}') && (c != EOF))
-            {
-              if ((size_t) (p-options+1) >= length)
-                {
-                  *p='\0';
-                  length<<=1;
-                  options=(char *) ResizeMagickMemory(options,
-                    (length+MaxTextExtent)*sizeof(*options));
-                  if (options == (char *) NULL)
-                    break;
-                  p=options+strlen(options);
-                }
-              if (options == (char *) NULL)
-                ThrowReaderException(ResourceLimitError,
-                  "MemoryAllocationFailed");
-              *p++=(char) c;
+            while (isspace((int) ((unsigned char) c)) != 0)
               c=ReadBlobByte(image);
-              if (*options != '{')
-                if (isspace((int) ((unsigned char) c)) != 0)
-                  break;
-            }
+            if (c == (int) '=')
+              {
+                /*
+                  Get the keyword value.
+                */
+                c=ReadBlobByte(image);
+                while ((c != (int) '}') && (c != EOF))
+                {
+                  if ((size_t) (p-options+1) >= length)
+                    {
+                      *p='\0';
+                      length<<=1;
+                      options=(char *) ResizeMagickMemory(options,
+                        (length+MaxTextExtent)*sizeof(*options));
+                      if (options == (char *) NULL)
+                        break;
+                      p=options+strlen(options);
+                    }
+                  if (options == (char *) NULL)
+                    ThrowReaderException(ResourceLimitError,
+                      "MemoryAllocationFailed");
+                  *p++=(char) c;
+                  c=ReadBlobByte(image);
+                  if (*options != '{')
+                    if (isspace((int) ((unsigned char) c)) != 0)
+                      break;
+                }
+              }
             *p='\0';
             if (*options == '{')
               (void) CopyMagickString(options,options+1,MaxTextExtent);
@@ -751,7 +765,7 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
                       AcquireString(keyword+8));
                     profile=AcquireStringInfo((size_t) atol(options));
                     (void) SetImageProfile(image,keyword+8,profile);
-                    DestroyStringInfo(profile);
+                    profile=DestroyStringInfo(profile);
                     break;
                   }
                 (void) SetImageAttribute(image,keyword,options);
@@ -809,8 +823,8 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
               {
                 if (LocaleCompare(keyword,"units") == 0)
                   {
-                    image->units=(ResolutionType)
-                      ParseMagickOption(MagickResolutionOptions,MagickFalse,options);
+                    image->units=(ResolutionType) ParseMagickOption(
+                      MagickResolutionOptions,MagickFalse,options);
                     break;
                   }
                 (void) SetImageAttribute(image,keyword,options);
@@ -910,15 +924,16 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
           (void) ReadBlob(image,profile->length,profile->datum);
           name=(const char *) GetNextElementInLinkedList(profiles);
         }
-        DestroyLinkedList(profiles,RelinquishMagickMemory);
+        profiles=DestroyLinkedList(profiles,RelinquishMagickMemory);
       }
-    depth=GetImageQuantumDepth(image);
+    depth=GetImageQuantumDepth(image,MagickFalse);
     if (image->storage_class == PseudoClass)
       {
         /*
           Create image colormap.
         */
-        if (AllocateImageColormap(image,colors != 0 ? colors : 256) == MagickFalse)
+        status=AllocateImageColormap(image,colors != 0 ? colors : 256);
+        if (status == MagickFalse)
           ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
         if (colors != 0)
           {
@@ -1006,7 +1021,8 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
       packet_size++;
     length=packet_size*image->columns;
     pixels=(unsigned char *) AcquireMagickMemory(packet_size*image->columns);
-    length=(size_t) CompressMaxExtent(packet_size*image->columns);
+    length=(size_t) Max(BZipMaxExtent(packet_size*image->columns),
+      ZipMaxExtent(packet_size*image->columns));
     compress_pixels=(unsigned char *) AcquireMagickMemory(length);
     if ((pixels == (unsigned char *) NULL) ||
         (compress_pixels == (unsigned char *) NULL))
@@ -1053,13 +1069,13 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
             if (zip_info.avail_in == 0)
               {
                 zip_info.next_in=compress_pixels;
-                length=(size_t) CompressMaxExtent(packet_size*image->columns);
+                length=(size_t) ZipMaxExtent(packet_size*image->columns);
                 if (version != 0)
                   length=(size_t) ReadBlobMSBLong(image);
                 zip_info.avail_in=(unsigned int)
                   ReadBlob(image,length,zip_info.next_in);
               }
-            if (inflate(&zip_info,Z_NO_FLUSH) == Z_STREAM_END)
+            if (inflate(&zip_info,Z_SYNC_FLUSH) == Z_STREAM_END)
               break;
           } while (zip_info.avail_out != 0);
           if (y == (long) (image->rows-1))
@@ -1071,7 +1087,7 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
               if (code >= 0)
                 status=MagickTrue;
             }
-          (void) PushImagePixels(image,quantum_type,pixels);
+          (void) ExportQuantumPixels(image,quantum_type,0,pixels);
           break;
         }
 #endif
@@ -1096,7 +1112,7 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
             if (bzip_info.avail_in == 0)
               {
                 bzip_info.next_in=(char *) compress_pixels;
-                length=(size_t) CompressMaxExtent(packet_size*image->columns);
+                length=(size_t) BZipMaxExtent(packet_size*image->columns);
                 if (version != 0)
                   length=(size_t) ReadBlobMSBLong(image);
                 bzip_info.avail_in=(unsigned int)
@@ -1114,7 +1130,7 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
               if (code >= 0)
                 status=MagickTrue;
             }
-          (void) PushImagePixels(image,quantum_type,pixels);
+          (void) ExportQuantumPixels(image,quantum_type,0,pixels);
           break;
         }
 #endif
@@ -1144,7 +1160,7 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
         default:
         {
           (void) ReadBlob(image,packet_size*image->columns,pixels);
-          (void) PushImagePixels(image,quantum_type,pixels);
+          (void) ExportQuantumPixels(image,quantum_type,0,pixels);
           break;
         }
       }
@@ -1155,7 +1171,7 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
     compress_pixels=(unsigned char *) RelinquishMagickMemory(compress_pixels);
     if (((y != (long) image->rows)) || (status == MagickFalse))
       {
-        DestroyImageList(image);
+        image=DestroyImageList(image);
         return((Image *) NULL);
       }
     if (EOFBlob(image) != MagickFalse)
@@ -1182,14 +1198,17 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
         AllocateNextImage(image_info,image);
         if (image->next == (Image *) NULL)
           {
-            DestroyImageList(image);
+            image=DestroyImageList(image);
             return((Image *) NULL);
           }
         image=SyncNextImageInList(image);
-        status=MagickMonitor(LoadImagesTag,TellBlob(image),GetBlobSize(image),
-          exception);
-        if (status == MagickFalse)
-          break;
+        if (image->progress_monitor != (MagickProgressMonitor) NULL)
+          {
+            status=image->progress_monitor(LoadImagesTag,TellBlob(image),
+              GetBlobSize(image),image->client_data);
+            if (status == MagickFalse)
+              break;
+          }
       }
   } while (c != EOF);
   CloseBlob(image);
@@ -1436,7 +1455,8 @@ static inline size_t PopRunlengthPacket(Image *image,const unsigned long depth,
   return((size_t) (q-pixels));
 }
 
-static MagickBooleanType WriteMIFFImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WriteMIFFImage(const ImageInfo *image_info,
+  Image *image)
 {
 #if defined(HasBZLIB)
   bz_stream
@@ -1522,12 +1542,13 @@ static MagickBooleanType WriteMIFFImage(const ImageInfo *image_info,Image *image
     /*
       Allocate image pixels.
     */
-    depth=GetImageQuantumDepth(image);
-    quality=image_info->quality == 0UL ? 75UL : image_info->quality;
+    depth=GetImageQuantumDepth(image,MagickTrue);
     if ((image->storage_class == PseudoClass) &&
         (image->colors > (1UL << depth)))
       image->storage_class=DirectClass;
     compression=NoCompression;
+    quality=(unsigned long)
+      (image->quality == UndefinedCompressionQuality ? 75 : image->quality);
     switch (image->compression)
     {
 #if defined(HasZLIB)
@@ -1552,7 +1573,8 @@ static MagickBooleanType WriteMIFFImage(const ImageInfo *image_info,Image *image
       packet_size+=depth/8;
     length=packet_size*image->columns;
     pixels=(unsigned char *) AcquireMagickMemory(length);
-    length=(size_t) CompressMaxExtent(packet_size*image->columns);
+    length=(size_t) Max(BZipMaxExtent(packet_size*image->columns),
+      ZipMaxExtent(packet_size*image->columns));
     if ((compression == BZipCompression) || (compression == ZipCompression))
       if (length != (size_t) ((unsigned int) length))
         compression=NoCompression;
@@ -1578,7 +1600,7 @@ static MagickBooleanType WriteMIFFImage(const ImageInfo *image_info,Image *image
           MagickOptionToMnemonic(MagickColorspaceOptions,image->colorspace));
         (void) WriteBlobString(image,buffer);
       }
-    if (image->compression != UndefinedCompression)
+    if (compression != UndefinedCompression)
       {
         (void) FormatMagickString(buffer,MaxTextExtent,"compression=%s\n",
           MagickOptionToMnemonic(MagickCompressionOptions,compression));
@@ -1702,7 +1724,7 @@ static MagickBooleanType WriteMIFFImage(const ImageInfo *image_info,Image *image
           if (profile != (StringInfo *) NULL)
             {
               (void) FormatMagickString(buffer,MaxTextExtent,
-                "profile-%s=%lu\n",name,profile->length);
+                "profile-%s=%lu\n",name,(unsigned long) profile->length);
               (void) WriteBlobString(image,buffer);
             }
           name=GetNextImageProfile(image);
@@ -1782,7 +1804,7 @@ static MagickBooleanType WriteMIFFImage(const ImageInfo *image_info,Image *image
         */
         packet_size=(size_t) (3*depth/8);
         colormap=(unsigned char *)
-          AcquireMagickMemory(packet_size*image->colors);
+          AcquireMagickMemory(packet_size*image->colors*sizeof(*colormap));
         if (colormap == (unsigned char *) NULL)
           ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
         /*
@@ -1866,13 +1888,12 @@ static MagickBooleanType WriteMIFFImage(const ImageInfo *image_info,Image *image
             }
           zip_info.next_in=pixels;
           zip_info.avail_in=(uInt) (packet_size*image->columns);
-          (void) PopImagePixels(image,quantum_type,pixels);
+          (void) ImportQuantumPixels(image,quantum_type,0,pixels);
           do
           {
             zip_info.next_out=compress_pixels;
-            zip_info.avail_out=(uInt)
-              CompressMaxExtent(packet_size*image->columns);
-            code=deflate(&zip_info,Z_NO_FLUSH);
+            zip_info.avail_out=(uInt) ZipMaxExtent(packet_size*image->columns);
+            code=deflate(&zip_info,Z_SYNC_FLUSH);
             if (code >= 0)
               status=MagickTrue;
             length=(size_t) (zip_info.next_out-compress_pixels);
@@ -1888,10 +1909,10 @@ static MagickBooleanType WriteMIFFImage(const ImageInfo *image_info,Image *image
               {
                 zip_info.next_out=compress_pixels;
                 zip_info.avail_out=(uInt)
-                  CompressMaxExtent(packet_size*image->columns);
+                  ZipMaxExtent(packet_size*image->columns);
                 code=deflate(&zip_info,Z_FINISH);
                 length=(size_t) (zip_info.next_out-compress_pixels);
-                if (length != 0)
+                if (length > 6)
                   {
                     (void) WriteBlobMSBLong(image,(unsigned long) length);
                     (void) WriteBlob(image,length,compress_pixels);
@@ -1919,12 +1940,12 @@ static MagickBooleanType WriteMIFFImage(const ImageInfo *image_info,Image *image
             }
           bzip_info.next_in=(char *) pixels;
           bzip_info.avail_in=(unsigned int) (packet_size*image->columns);
-          (void) PopImagePixels(image,quantum_type,pixels);
+          (void) ImportQuantumPixels(image,quantum_type,0,pixels);
           do
           {
             bzip_info.next_out=(char *) compress_pixels;
             bzip_info.avail_out=(unsigned int)
-              CompressMaxExtent(packet_size*image->columns);
+              BZipMaxExtent(packet_size*image->columns);
             code=BZ2_bzCompress(&bzip_info,BZ_FLUSH);
             if (code >= 0)
               status=MagickTrue;
@@ -1941,7 +1962,7 @@ static MagickBooleanType WriteMIFFImage(const ImageInfo *image_info,Image *image
               {
                 bzip_info.next_out=(char *) compress_pixels;
                 bzip_info.avail_out=(unsigned int)
-                  CompressMaxExtent(packet_size*image->columns);
+                  BZipMaxExtent(packet_size*image->columns);
                 code=BZ2_bzCompress(&bzip_info,BZ_FINISH);
                 length=(size_t) (bzip_info.next_out-(char *) compress_pixels);
                 if (length != 0)
@@ -1989,15 +2010,17 @@ static MagickBooleanType WriteMIFFImage(const ImageInfo *image_info,Image *image
         }
         default:
         {
-          (void) PopImagePixels(image,quantum_type,pixels);
+          (void) ImportQuantumPixels(image,quantum_type,0,pixels);
           (void) WriteBlob(image,packet_size*image->columns,pixels);
           break;
         }
       }
       if (image->previous == (Image *) NULL)
-        if (QuantumTick(y,image->rows) != MagickFalse)
+        if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+            (QuantumTick(y,image->rows) != MagickFalse))
           {
-            status=MagickMonitor(SaveImageTag,y,image->rows,&image->exception);
+            status=image->progress_monitor(SaveImageTag,y,image->rows,
+              image->client_data);
             if (status == MagickFalse)
               break;
           }
@@ -2007,10 +2030,14 @@ static MagickBooleanType WriteMIFFImage(const ImageInfo *image_info,Image *image
     if (image->next == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
-      &image->exception);
-    if (status == MagickFalse)
-      break;
+    if (image->progress_monitor != (MagickProgressMonitor) NULL)
+      {
+        status=image->progress_monitor(SaveImagesTag,scene,
+          GetImageListLength(image),image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
+    scene++;
   } while (image_info->adjoin != MagickFalse);
   CloseBlob(image);
   return(status);
