@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -41,9 +41,12 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
 #include "magick/colorspace.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
@@ -128,7 +131,7 @@ static Image *ReadMONOImage(const ImageInfo *image_info,
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
-      DestroyImageList(image);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
   for (i=0; i < image->offset; i++)
@@ -166,9 +169,14 @@ static Image *ReadMONOImage(const ImageInfo *image_info,
     }
     if (SyncImagePixels(image) == MagickFalse)
       break;
-    if (QuantumTick(y,image->rows) != MagickFalse)
-      if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
-        break;
+    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+        (QuantumTick(y,image->rows) != MagickFalse))
+      {
+        status=image->progress_monitor(LoadImageTag,y,image->rows,
+          image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
   }
   SyncImage(image);
   if (EOFBlob(image) != MagickFalse)
@@ -210,6 +218,7 @@ ModuleExport void RegisterMONOImage(void)
   entry->decoder=(DecoderHandler *) ReadMONOImage;
   entry->encoder=(EncoderHandler *) WriteMONOImage;
   entry->raw=MagickTrue;
+  entry->endian_support=MagickTrue;
   entry->adjoin=MagickFalse;
   entry->description=
     AcquireString("Bi-level bitmap in least-significant-byte first order");
@@ -336,9 +345,14 @@ static MagickBooleanType WriteMONOImage(const ImageInfo *image_info,
     }
     if (bit != 0)
       (void) WriteBlobByte(image,(unsigned char) (byte >> (8-bit)));
-    if (QuantumTick(y,image->rows) != MagickFalse)
-      if (MagickMonitor(SaveImageTag,y,image->rows,&image->exception) == MagickFalse)
-        break;
+    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+        (QuantumTick(y,image->rows) != MagickFalse))
+      {
+        status=image->progress_monitor(SaveImageTag,y,image->rows,
+          image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
   }
   CloseBlob(image);
   return(MagickTrue);
