@@ -17,7 +17,7 @@
 %                                 March 2000                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -43,14 +43,17 @@
 #include "magick/studio.h"
 #include "magick/attribute.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
 #include "magick/cache.h"
 #include "magick/color.h"
 #include "magick/color_private.h"
 #include "magick/constitute.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/geometry.h"
 #include "magick/hashmap.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
@@ -226,7 +229,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
-      DestroyImageList(image);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
   (void) CopyMagickString(cache_filename,image->filename,MaxTextExtent);
@@ -234,7 +237,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
   c=ReadBlobByte(image);
   if (c == EOF)
     {
-      DestroyImage(image);
+      image=DestroyImage(image);
       return((Image *) NULL);
     }
   *id='\0';
@@ -285,7 +288,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
           if (comment == (char *) NULL)
             ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
           *p='\0';
-          (void) SetImageAttribute(image,"comment",comment);
+          (void) SetImageAttribute(image,"Comment",comment);
           comment=(char *) RelinquishMagickMemory(comment);
           c=ReadBlobByte(image);
         }
@@ -293,41 +296,51 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
         if (isalnum(c) != MagickFalse)
           {
             /*
-              Determine a keyword and its value.
+              Get the keyword.
             */
             p=keyword;
             do
             {
+              if (isspace((int) ((unsigned char) c)) != 0)
+                break;
+              if (c == (int) '=')
+                break;
               if ((size_t) (p-keyword) < (MaxTextExtent-1))
                 *p++=(char) c;
               c=ReadBlobByte(image);
-            } while ((isalnum(c) != MagickFalse) || (c == (int) '-'));
+            } while (c != EOF);
             *p='\0';
-            while ((isspace((int) ((unsigned char) c)) != 0) ||
-                   (c == (int) '='))
-              c=ReadBlobByte(image);
             p=options;
-            while ((c != (int) '}') && (c != EOF))
-            {
-              if ((p-options+1) >= (int) length)
-                {
-                  *p='\0';
-                  length<<=1;
-                  options=(char *) ResizeMagickMemory(options,
-                    (length+MaxTextExtent)*sizeof(*options));
-                  if (options == (char *) NULL)
-                    break;
-                  p=options+strlen(options);
-                }
-              if (options == (char *) NULL)
-                ThrowReaderException(ResourceLimitError,
-                  "MemoryAllocationFailed");
-              *p++=(char) c;
+            while (isspace((int) ((unsigned char) c)) != 0)
               c=ReadBlobByte(image);
-              if (*options != '{')
-                if (isspace((int) ((unsigned char) c)) != 0)
-                  break;
-            }
+            if (c == (int) '=')
+              {
+                /*
+                  Get the keyword value.
+                */
+                c=ReadBlobByte(image);
+                while ((c != (int) '}') && (c != EOF))
+                {
+                  if ((size_t) (p-options+1) >= length)
+                    {
+                      *p='\0';
+                      length<<=1;
+                      options=(char *) ResizeMagickMemory(options,
+                        (length+MaxTextExtent)*sizeof(*options));
+                      if (options == (char *) NULL)
+                        break;
+                      p=options+strlen(options);
+                    }
+                  if (options == (char *) NULL)
+                    ThrowReaderException(ResourceLimitError,
+                      "MemoryAllocationFailed");
+                  *p++=(char) c;
+                  c=ReadBlobByte(image);
+                  if (*options != '{')
+                    if (isspace((int) ((unsigned char) c)) != 0)
+                      break;
+                }
+              }
             *p='\0';
             if (*options == '{')
               (void) CopyMagickString(options,options+1,MaxTextExtent);
@@ -380,14 +393,14 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                   }
                 if (LocaleCompare(keyword,"colorspace") == 0)
                   {
-                    image->colorspace=(ColorspaceType)
-                      ParseMagickOption(MagickColorspaceOptions,MagickFalse,options);
+                    image->colorspace=(ColorspaceType) ParseMagickOption(
+                      MagickColorspaceOptions,MagickFalse,options);
                     break;
                   }
                 if (LocaleCompare(keyword,"compression") == 0)
                   {
-                    image->compression=(CompressionType)
-                      ParseMagickOption(MagickCompressionOptions,MagickFalse,options);
+                    image->compression=(CompressionType) ParseMagickOption(
+                      MagickCompressionOptions,MagickFalse,options);
                     break;
                   }
                 if (LocaleCompare(keyword,"columns") == 0)
@@ -413,8 +426,8 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                   }
                 if (LocaleCompare(keyword,"dispose") == 0)
                   {
-                    image->dispose=(DisposeType)
-                      ParseMagickOption(MagickDisposeOptions,MagickFalse,options);
+                    image->dispose=(DisposeType) ParseMagickOption(
+                      MagickDisposeOptions,MagickFalse,options);
                     break;
                   }
                 (void) SetImageAttribute(image,keyword,options);
@@ -522,7 +535,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                       AcquireString(keyword+8));
                     profile=AcquireStringInfo((size_t) atol(options));
                     (void) SetImageProfile(image,keyword+8,profile);
-                    DestroyStringInfo(profile);
+                    profile=DestroyStringInfo(profile);
                     break;
                   }
                 (void) SetImageAttribute(image,keyword,options);
@@ -564,8 +577,8 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                   }
                 if (LocaleCompare(keyword,"rendering-intent") == 0)
                   {
-                    image->rendering_intent=(RenderingIntent)
-                      ParseMagickOption(MagickIntentOptions,MagickFalse,options);
+                    image->rendering_intent=(RenderingIntent) ParseMagickOption(
+                      MagickIntentOptions,MagickFalse,options);
                     break;
                   }
                 if (LocaleCompare(keyword,"resolution") == 0)
@@ -601,8 +614,8 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
               {
                 if (LocaleCompare(keyword,"units") == 0)
                   {
-                    image->units=(ResolutionType)
-                      ParseMagickOption(MagickResolutionOptions,MagickFalse,options);
+                    image->units=(ResolutionType) ParseMagickOption(
+                      MagickResolutionOptions,MagickFalse,options);
                     break;
                   }
                 (void) SetImageAttribute(image,keyword,options);
@@ -695,9 +708,9 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
             (void) ReadBlob(image,profile->length,profile->datum);
           name=(const char *) GetNextElementInLinkedList(profiles);
         }
-        DestroyLinkedList(profiles,RelinquishMagickMemory);
+        profiles=DestroyLinkedList(profiles,RelinquishMagickMemory);
       }
-    depth=GetImageQuantumDepth(image);
+    depth=GetImageQuantumDepth(image,MagickFalse);
     if (image->storage_class == PseudoClass)
       {
         /*
@@ -804,14 +817,17 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
         AllocateNextImage(image_info,image);
         if (image->next == (Image *) NULL)
           {
-            DestroyImageList(image);
+            image=DestroyImageList(image);
             return((Image *) NULL);
           }
         image=SyncNextImageInList(image);
-        status=MagickMonitor(LoadImagesTag,TellBlob(image),GetBlobSize(image),
-          exception);
-        if (status == MagickFalse)
-          break;
+        if (image->progress_monitor != (MagickProgressMonitor) NULL)
+          {
+            status=image->progress_monitor(LoadImagesTag,TellBlob(image),
+              GetBlobSize(image),image->client_data);
+            if (status == MagickFalse)
+              break;
+          }
       }
   } while (c != EOF);
   CloseBlob(image);
@@ -928,6 +944,9 @@ static MagickBooleanType WriteMPCImage(const ImageInfo *image_info,Image *image)
   register long
     i;
 
+  unsigned long
+    depth;
+
   /*
     Open persistent cache.
   */
@@ -949,6 +968,10 @@ static MagickBooleanType WriteMPCImage(const ImageInfo *image_info,Image *image)
     /*
       Write persistent cache meta-information.
     */
+    depth=GetImageQuantumDepth(image,MagickTrue);
+    if ((image->storage_class == PseudoClass) &&
+        (image->colors > (1UL << depth)))
+      image->storage_class=DirectClass;
     (void) WriteBlobString(image,"id=MagickCache\n");
     (void) FormatMagickString(buffer,MaxTextExtent,"quantum-depth=%d\n",
       QuantumDepth);
@@ -1043,14 +1066,6 @@ static MagickBooleanType WriteMPCImage(const ImageInfo *image_info,Image *image)
           MagickOptionToMnemonic(MagickCompositeOptions,image->compose));
         (void) WriteBlobString(image,buffer);
       }
-    if (image->error.mean_error_per_pixel != 0.0)
-      {
-        (void) FormatMagickString(buffer,MaxTextExtent,
-          "error=%g  mean-error=%g  maximum-error=%g\n",
-          image->error.mean_error_per_pixel,image->error.normalized_mean_error,
-          image->error.normalized_maximum_error);
-        (void) WriteBlobString(image,buffer);
-      }
     if (image->rendering_intent != UndefinedIntent)
       {
         (void) FormatMagickString(buffer,MaxTextExtent,
@@ -1097,9 +1112,12 @@ static MagickBooleanType WriteMPCImage(const ImageInfo *image_info,Image *image)
         while (name != (const char *) NULL)
         {
           profile=GetImageProfile(image,name);
-          (void) FormatMagickString(buffer,MaxTextExtent,
-            "profile-%s=%lu\n",name,profile->length);
-          (void) WriteBlobString(image,buffer);
+          if (profile != (StringInfo *) NULL)
+            {
+              (void) FormatMagickString(buffer,MaxTextExtent,
+                "profile-%s=%lu\n",name,(unsigned long) profile->length);
+              (void) WriteBlobString(image,buffer);
+            }
           name=GetNextImageProfile(image);
         }
       }
@@ -1112,24 +1130,20 @@ static MagickBooleanType WriteMPCImage(const ImageInfo *image_info,Image *image)
     attribute=GetImageAttribute(image,(char *) NULL);
     for ( ; attribute != (const ImageAttribute *) NULL; attribute=attribute->next)
     {
-      if (attribute->value != NULL)
-        {
-          long
-            j;
-
-          (void) FormatMagickString(buffer,MaxTextExtent,"%s=",
-            attribute->key);
-          (void) WriteBlobString(image,buffer);
-          for (j=0; j < (long) strlen(attribute->value); j++)
-            if (isspace((int) ((unsigned char) attribute->value[j])) != 0)
-              break;
-          if (j < (long) strlen(attribute->value))
-            (void) WriteBlobByte(image,'{');
-          (void) WriteBlobString(image,attribute->value);
-          if (j < (long) strlen(attribute->value))
-            (void) WriteBlobByte(image,'}');
-          (void) WriteBlobByte(image,'\n');
-        }
+      if (*attribute->key == '[')
+        continue;
+      (void) FormatMagickString(buffer,MaxTextExtent,"%s=",attribute->key);
+      (void) WriteBlobString(image,buffer);
+      for (i=0; i < (long) strlen(attribute->value); i++)
+        if (isspace((int) ((unsigned char) attribute->value[i])) != 0)
+          break;
+      if (i <= (long) strlen(attribute->value))
+        (void) WriteBlobByte(image,'{');
+      (void) WriteBlob(image,strlen(attribute->value),(unsigned char *)
+        attribute->value);
+      if (i <= (long) strlen(attribute->value))
+        (void) WriteBlobByte(image,'}');
+      (void) WriteBlobByte(image,'\n');
     }
     (void) WriteBlobString(image,"\f\n:\032");
     if (image->montage != (char *) NULL)
@@ -1173,16 +1187,14 @@ static MagickBooleanType WriteMPCImage(const ImageInfo *image_info,Image *image)
           *colormap;
 
         unsigned long
-          depth,
           pixel;
 
         /*
           Allocate colormap.
         */
-        depth=GetImageQuantumDepth(image);
-        packet_size=(size_t) (depth > 8 ? 6 : 3);
+        packet_size=(size_t) (3*depth/8);
         colormap=(unsigned char *)
-          AcquireMagickMemory(packet_size*image->colors);
+          AcquireMagickMemory(packet_size*image->colors*sizeof(*colormap));
         if (colormap == (unsigned char *) NULL)
           return(MagickFalse);
         /*
@@ -1233,16 +1245,21 @@ static MagickBooleanType WriteMPCImage(const ImageInfo *image_info,Image *image)
     /*
       Initialize persistent pixel cache.
     */
-    status=PersistCache(image,cache_filename,MagickFalse,&offset,&image->exception);
+    status=PersistCache(image,cache_filename,MagickFalse,&offset,
+      &image->exception);
     if (status == MagickFalse)
       ThrowWriterException(CacheError,"UnableToPersistPixelCache");
     if (image->next == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
-      &image->exception);
-    if (status == MagickFalse)
-      break;
+    if (image->progress_monitor != (MagickProgressMonitor) NULL)
+      {
+        status=image->progress_monitor(SaveImagesTag,scene,
+          GetImageListLength(image),image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
+    scene++;
   } while (image_info->adjoin != MagickFalse);
   CloseBlob(image);
   return(status);
