@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -41,10 +41,13 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
 #include "magick/color.h"
 #include "magick/color_private.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
@@ -80,10 +83,6 @@ static MagickBooleanType
 %
 %  A description of each parameter follows:
 %
-%    o image:  Method ReadNULLImage returns a pointer to the image after
-%      creating it. A null image is returned if there is a memory shortage
-%      or if the image cannot be read.
-%
 %    o image_info: The image info.
 %
 %    o exception: return any errors or warnings in this structure.
@@ -118,7 +117,7 @@ static Image *ReadNULLImage(const ImageInfo *image_info,
     &image->background_color,exception);
   if (status == MagickFalse)
     {
-      DestroyImage(image);
+      image=DestroyImage(image);
       return((Image *) NULL);
     }
   if (AllocateImageColormap(image,1) == MagickFalse)
@@ -205,19 +204,19 @@ ModuleExport void UnregisterNULLImage(void)
 %
 %  The format of the WriteNULLImage method is:
 %
-%      MagickBooleanType WriteNULLImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WriteNULLImage(const ImageInfo *image_info,
+%        Image *image)
 %
 %  A description of each parameter follows.
 %
-%    o status: Always returns MagickTrue.
-%
 %    o image_info: The image info.
 %
 %    o image:  The image.
 %
 %
 */
-static MagickBooleanType WriteNULLImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WriteNULLImage(const ImageInfo *image_info,
+  Image *image)
 {
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
