@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -41,14 +41,17 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
 #include "magick/client.h"
 #include "magick/colorspace.h"
 #include "magick/constitute.h"
 #include "magick/decorate.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/gem.h"
 #include "magick/geometry.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
@@ -84,14 +87,11 @@ static MagickBooleanType
 %
 %  The format of the DecodeImage method is:
 %
-%      static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
+%      MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
 %        unsigned char *chroma1,unsigned char *chroma2)
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method DecodeImage returns MagickTrue if all the deltas are
-%      recovered without error, otherwise MagickFalse.
-%
 %    o image: The address of a structure of type Image.
 %
 %    o luma: The address of a character buffer that contains the
@@ -198,7 +198,7 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
   {
     PCDGetBits(8);
     length=(sum & 0xff)+1;
-    pcd_table[i]=(PCDTable *) AcquireMagickMemory(length*sizeof(PCDTable));
+    pcd_table[i]=(PCDTable *) AcquireMagickMemory(length*sizeof(**pcd_table));
     if (pcd_table[i] == (PCDTable *) NULL)
       {
         buffer=(unsigned char *) RelinquishMagickMemory(buffer);
@@ -219,7 +219,7 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
       r->sequence=(sum & 0xffff) << 16;
       PCDGetBits(8);
       r->key=(unsigned char) (sum & 0xff);
-      r->mask=(~((((MagickBooleanType) 1) << (32-r->length))-1));
+      r->mask=(~((1U << (32-r->length))-1));
       r++;
     }
     pcd_length[i]=length;
@@ -286,10 +286,11 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
           }
         }
         length=pcd_length[plane];
-        if (QuantumTick(row,image->rows))
+        if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+            (QuantumTick(row,image->rows) != MagickFalse))
           {
-            status=MagickMonitor(DecodeImageTag,row,image->rows,
-              &image->exception);
+            status=image->progress_monitor(DecodeImageTag,row,image->rows,
+              image->client_data);
             if (status == MagickFalse)
               break;
           }
@@ -350,8 +351,6 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method IsPCD() returns MagickTrue if the image format type is PCD.
-%
 %    o magick: This string is generally the first few bytes of an image file
 %      or blob.
 %
@@ -390,10 +389,6 @@ static MagickBooleanType IsPCD(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o image:  Method ReadPCDImage returns a pointer to the image after
-%      reading.  A null image is returned if there is a memory shortage or
-%      if the image cannot be read.
-%
 %    o image_info: The image info.
 %
 %    o exception: return any errors or warnings in this structure.
@@ -424,7 +419,7 @@ static Image *OverviewImage(const ImageInfo *image_info,Image *image,
   commands[1]=(char *) "-label";
   commands[2]=(char *) DefaultTileLabel;
   (void) MogrifyImages(mogrify_info,3,commands,&image);
-  DestroyImageInfo(mogrify_info);
+  mogrify_info=DestroyImageInfo(mogrify_info);
   /*
     Create the PCD Overview image.
   */
@@ -432,10 +427,10 @@ static Image *OverviewImage(const ImageInfo *image_info,Image *image,
   (void) CopyMagickString(montage_info->filename,image_info->filename,
     MaxTextExtent);
   montage_image=MontageImages(image,montage_info,exception);
-  DestroyMontageInfo(montage_info);
+  montage_info=DestroyMontageInfo(montage_info);
   if (montage_image == (Image *) NULL)
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-  DestroyImage(image);
+  image=DestroyImage(image);
   return(montage_image);
 }
 
@@ -448,7 +443,6 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
     x;
 
   MagickBooleanType
-    overview,
     status;
 
   MagickOffsetType
@@ -478,6 +472,9 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
     *header,
     *luma;
 
+  unsigned int
+    overview;
+
   unsigned long
     height,
     number_images,
@@ -498,7 +495,7 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
-      DestroyImageList(image);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
   /*
@@ -585,8 +582,8 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
       Image
         *overview_image;
 
-      MonitorHandler
-        handler;
+      MagickProgressMonitor
+        progress_monitor;
 
       register long
         j;
@@ -596,7 +593,8 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
       */
       for (j=1; j <= (long) number_images; j++)
       {
-        handler=SetMonitorHandler((MonitorHandler) NULL);
+        progress_monitor=SetImageProgressMonitor(image,
+          (MagickProgressMonitor) NULL,image->client_data);
         (void) FormatMagickString(image->filename,MaxTextExtent,
           "images/img%04ld.pcd",j);
         (void) FormatMagickString(image->magick_filename,MaxTextExtent,
@@ -654,14 +652,21 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
             AllocateNextImage(image_info,image);
             if (image->next == (Image *) NULL)
               {
-                DestroyImageList(image);
+                image=DestroyImageList(image);
                 return((Image *) NULL);
               }
             image=SyncNextImageInList(image);
           }
-        (void) SetMonitorHandler(handler);
-        if (MagickMonitor(LoadImageTag,j-1,number_images,exception) == MagickFalse)
-          break;
+        (void) SetImageProgressMonitor(image,progress_monitor,
+          image->client_data);
+        if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+            (QuantumTick(j-1,number_images) != MagickFalse))
+          {
+            status=image->progress_monitor(LoadImageTag,j-1,number_images,
+              image->client_data);
+            if (status == MagickFalse)
+              break;
+          }
       }
       chroma2=(unsigned char *) RelinquishMagickMemory(chroma2);
       chroma1=(unsigned char *) RelinquishMagickMemory(chroma1);
@@ -746,9 +751,14 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
     }
     if (SyncImagePixels(image) == MagickFalse)
       break;
-    if (QuantumTick(y,image->rows) != 0)
-      if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
-        break;
+    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+        (QuantumTick(y,image->rows) != MagickFalse))
+      {
+        status=image->progress_monitor(LoadImageTag,y,image->rows,
+          image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
   }
   chroma2=(unsigned char *) RelinquishMagickMemory(chroma2);
   chroma1=(unsigned char *) RelinquishMagickMemory(chroma1);
@@ -773,7 +783,7 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
         rotate_image=RotateImage(image,degrees,exception);
         if (rotate_image != (Image *) NULL)
           {
-            DestroyImage(image);
+            image=DestroyImage(image);
             image=rotate_image;
           }
       }
@@ -886,10 +896,6 @@ ModuleExport void UnregisterPCDImage(void)
 %
 %  A description of each parameter follows.
 %
-%    o status: Method WritePCDImage returns True if the image is written.
-%      False is returned is there is a memory shortage or if the image file
-%      fails to write.
-%
 %    o image_info: The image info.
 %
 %    o image:  The image.
@@ -963,7 +969,7 @@ static MagickBooleanType WritePCDTile(const ImageInfo *image_info,Image *image,
       bordered_image=BorderImage(tile_image,&border_info,&image->exception);
       if (bordered_image == (Image *) NULL)
         return(MagickFalse);
-      DestroyImage(tile_image);
+      tile_image=DestroyImage(tile_image);
       tile_image=bordered_image;
     }
   (void) TransformImage(&tile_image,(char *) NULL,tile_geometry);
@@ -1004,17 +1010,19 @@ static MagickBooleanType WritePCDTile(const ImageInfo *image_info,Image *image,
       (void) WriteBlobByte(image,ScaleQuantumToChar(q->blue));
       q++;
     }
-    if (QuantumTick(y,tile_image->rows))
+    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+        (QuantumTick(y,tile_image->rows) != MagickFalse))
       {
-        status=MagickMonitor(SaveImageTag,y,tile_image->rows,&image->exception);
+        status=image->progress_monitor(LoadImageTag,y,tile_image->rows,
+          image->client_data);
         if (status == MagickFalse)
           break;
       }
   }
   for (i=0; i < 0x800; i++)
     (void) WriteBlobByte(image,'\0');
-  DestroyImage(downsample_image);
-  DestroyImage(tile_image);
+  downsample_image=DestroyImage(downsample_image);
+  tile_image=DestroyImage(tile_image);
   return(MagickTrue);
 }
 
@@ -1094,12 +1102,12 @@ static MagickBooleanType WritePCDImage(const ImageInfo *image_info,Image *image)
   */
   status=WritePCDTile(image_info,pcd_image,(char *) "768x512>",
     (char *) "192x128");
-  status|=WritePCDTile(image_info,pcd_image,(char *) "768x512>",
+  status=WritePCDTile(image_info,pcd_image,(char *) "768x512>",
     (char *) "384x256");
-  status|=WritePCDTile(image_info,pcd_image,(char *) "768x512>",
+  status=WritePCDTile(image_info,pcd_image,(char *) "768x512>",
     (char *) "768x512");
   CloseBlob(pcd_image);
   if (pcd_image != image)
-    DestroyImage(pcd_image);
+    pcd_image=DestroyImage(pcd_image);
   return(status);
 }
