@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -42,12 +42,15 @@
 #include "magick/studio.h"
 #include "magick/attribute.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
 #include "magick/color.h"
 #include "magick/color_private.h"
 #include "magick/colorspace.h"
 #include "magick/constitute.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/geometry.h"
+#include "magick/image_private.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
@@ -284,9 +287,11 @@ static MagickBooleanType WritePCLImage(const ImageInfo *image_info,Image *image)
         }
         if (bit != 0)
           (void) WriteBlobByte(image,byte << (8-bit));
-        if (QuantumTick(y,image->rows) != MagickFalse)
+        if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+            (QuantumTick(y,image->rows) != MagickFalse))
           {
-            status=MagickMonitor(SaveImageTag,y,image->rows,&image->exception);
+            status=image->progress_monitor(SaveImageTag,y,image->rows,
+              image->client_data);
             if (status == MagickFalse)
               break;
           }
@@ -341,9 +346,11 @@ static MagickBooleanType WritePCLImage(const ImageInfo *image_info,Image *image)
           3*image->columns);  /* send row */
         (void) WriteBlobString(image,buffer);
         (void) WriteBlob(image,3*image->columns,pixels);
-        if (QuantumTick(y,image->rows) != MagickFalse)
+        if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+            (QuantumTick(y,image->rows) != MagickFalse))
           {
-            status=MagickMonitor(SaveImageTag,y,image->rows,&image->exception);
+            status=image->progress_monitor(SaveImageTag,y,image->rows,
+              image->client_data);
             if (status == MagickFalse)
               break;
           }
