@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -42,15 +42,19 @@
 #include "magick/studio.h"
 #include "magick/attribute.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
 #include "magick/color_private.h"
 #include "magick/colorspace.h"
 #include "magick/constitute.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/pixel.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 
@@ -137,9 +141,6 @@ static MagickBooleanType
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method DecodeImage returns MagickTrue if all the pixels are
-%      uncompressed without error, otherwise MagickFalse.
-%
 %    o image: The address of a structure of type Image.
 %
 %    o pixels:  The address of a byte (8 bits) array of pixel data created by
@@ -201,8 +202,6 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *pixels,
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method IsPDB() returns MagickTrue if the image format type is PDB.
-%
 %    o magick: This string is generally the first few bytes of an image file
 %      or blob.
 %
@@ -240,10 +239,6 @@ static MagickBooleanType IsPDB(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o image:  Method ReadPDBImage returns a pointer to the image after
-%      reading. A null image is returned if there is a memory shortage or if
-%      the image cannot be read.
-%
 %    o image_info: The image info.
 %
 %    o exception: return any errors or warnings in this structure.
@@ -310,7 +305,7 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
-      DestroyImageList(image);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
   /*
@@ -329,7 +324,7 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
   (void) ReadBlob(image,4,(unsigned char *) pdb_info.id);
   if ((count == 0) || (memcmp(pdb_info.type,"vIMG",4) != 0) ||
       (memcmp(pdb_info.id,"View",4) != 0))
-    ThrowReaderException(CoderError,"ImproperImageHeader");
+    ThrowReaderException(CorruptImageError,"ImproperImageHeader");
   pdb_info.seed=ReadBlobMSBLong(image);
   pdb_info.next_record=ReadBlobMSBLong(image);
   pdb_info.number_records=ReadBlobMSBShort(image);
@@ -412,7 +407,8 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
       break;
     }
     default:
-      ThrowReaderException(CorruptImageError,"UnrecognizedCompressionType");
+      ThrowReaderException(CorruptImageError,
+        "UnrecognizedImageCompressionType");
   }
   p=pixels;
   switch (bits_per_pixel)
@@ -443,9 +439,14 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
         }
         if (SyncImagePixels(image) == MagickFalse)
           break;
-        if (QuantumTick(y,image->rows) != 0)
-          if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
-            break;
+        if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+            (QuantumTick(y,image->rows) != MagickFalse))
+          {
+            status=image->progress_monitor(LoadImageTag,y,image->rows,
+              image->client_data);
+            if (status == MagickFalse)
+              break;
+          }
       }
       break;
     }
@@ -478,9 +479,14 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
         }
         if (SyncImagePixels(image) == MagickFalse)
           break;
-        if (QuantumTick(y,image->rows) != 0)
-          if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
-            break;
+        if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+            (QuantumTick(y,image->rows) != MagickFalse))
+          {
+            status=image->progress_monitor(LoadImageTag,y,image->rows,
+              image->client_data);
+            if (status == MagickFalse)
+              break;
+          }
       }
       break;
     }
@@ -507,9 +513,14 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
         }
         if (SyncImagePixels(image) == MagickFalse)
           break;
-        if (QuantumTick(y,image->rows) != 0)
-          if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
-            break;
+        if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+            (QuantumTick(y,image->rows) != MagickFalse))
+          {
+            status=image->progress_monitor(LoadImageTag,y,image->rows,
+              image->client_data);
+            if (status == MagickFalse)
+              break;
+          }
       }
       break;
     }
@@ -539,27 +550,27 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
       */
       c=ReadBlobByte(image);
       length=MaxTextExtent;
-      comment=(char *) AcquireMagickMemory(length+1);
-      p=comment;
-      if (comment != (char *) NULL)
-        for ( ; c != EOF; p++)
-        {
-          if ((p-comment) >= (int) length)
-            {
-              length<<=1;
-              length+=MaxTextExtent;
-              comment=(char *) ResizeMagickMemory(comment,length+1);
-              if (comment == (char *) NULL)
-                break;
-              p=comment+strlen(comment);
-            }
-          *p=c;
-          *(p+1)='\0';
-          c=ReadBlobByte(image);
-        }
+      comment=AcquireString((char *) NULL);
+      for (p=comment; c != EOF; p++)
+      {
+        if ((size_t) (p-comment+MaxTextExtent) >= length)
+          {
+            *p='\0';
+            length<<=1;
+            length+=MaxTextExtent;
+            comment=(char *) ResizeMagickMemory(comment,
+              (length+MaxTextExtent)*sizeof(*comment));
+            if (comment == (char *) NULL)
+              break;
+            p=comment+strlen(comment);
+          }
+        *p=c;
+        c=ReadBlobByte(image);
+      }
+      *p='\0';
       if (comment == (char *) NULL)
         ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-      (void) SetImageAttribute(image,"comment",comment);
+      (void) SetImageAttribute(image,"Comment",comment);
       comment=(char *) RelinquishMagickMemory(comment);
     }
   CloseBlob(image);
@@ -646,10 +657,6 @@ ModuleExport void UnregisterPDBImage(void)
 %
 %  A description of each parameter follows.
 %
-%    o status: Method WritePDBImage returns True if the image is written.
-%      False is returned is there is a memory shortage or if the image file
-%      fails to write.
-%
 %    o image_info: The image info.
 %
 %    o image:  The image.
@@ -658,7 +665,7 @@ ModuleExport void UnregisterPDBImage(void)
 */
 
 static unsigned char *EncodeRLE(unsigned char *destination,
-  unsigned char *source,MagickBooleanType literal,MagickBooleanType repeat)
+  unsigned char *source,unsigned long literal,unsigned long repeat)
 {
   if (literal > 0)
     *destination++=literal-1;
@@ -689,16 +696,21 @@ static MagickBooleanType WritePDBImage(const ImageInfo *image_info,Image *image)
   PDBInfo
     pdb_info;
 
+  register const PixelPacket
+    *p;
+
   register long
     x;
 
+  register unsigned char
+    *q;
+
   size_t
     packet_size;
 
   unsigned char
     *buffer,
-    *p,
-    *q,
+    *runlength,
     *scanline;
 
   unsigned long
@@ -742,7 +754,7 @@ static MagickBooleanType WritePDBImage(const ImageInfo *image_info,Image *image)
   CopyMagickMemory(pdb_info.id,"View",4);
   pdb_info.seed=0;
   pdb_info.next_record=0;
-  comment=GetImageAttribute(image,"comment");
+  comment=GetImageAttribute(image,"Comment");
   pdb_info.number_records=(comment == (ImageAttribute *) NULL ? 1 : 2);
   (void) WriteBlob(image,32,(unsigned char *) pdb_info.name);
   (void) WriteBlobMSBShort(image,pdb_info.attributes);
@@ -778,8 +790,8 @@ static MagickBooleanType WritePDBImage(const ImageInfo *image_info,Image *image)
     pdb_image.width=(short) (16*(image->columns/16+1));
   pdb_image.height=(short) image->rows;
   packets=(bits_per_pixel*image->columns/8)*image->rows;
-  p=(unsigned char *) AcquireMagickMemory(2*packets);
-  if (p == (unsigned char *) NULL)
+  runlength=(unsigned char *) AcquireMagickMemory(2*packets);
+  if (runlength == (unsigned char *) NULL)
     ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
   buffer=(unsigned char *) AcquireMagickMemory(256);
   if (buffer == (unsigned char *) NULL)
@@ -795,13 +807,14 @@ static MagickBooleanType WritePDBImage(const ImageInfo *image_info,Image *image)
   bits=8/(long) bits_per_pixel-1;  /* start at most significant bits */
   literal=0;
   repeat=0;
-  q=p;
+  q=runlength;
   buffer[0]=0x00;
   for (y=0; y < (long) image->rows; y++)
   {
-    if (AcquireImagePixels(image,0,y,image->columns,1,&image->exception) == MagickFalse)
+    p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+    if (p == (const PixelPacket *) NULL)
       break;
-    (void) PopImagePixels(image,GrayQuantum,scanline);
+    (void) ImportQuantumPixels(image,GrayQuantum,0,scanline);
     for (x=0; x < pdb_image.width; x++)
     {
       if (x < (long) image->columns)
@@ -849,9 +862,14 @@ static MagickBooleanType WritePDBImage(const ImageInfo *image_info,Image *image)
         buffer[literal+repeat]=0x00;
       }
     }
-    if (QuantumTick(y,image->rows) != 0)
-      if (MagickMonitor(SaveImageTag,y,image->rows,&image->exception) == MagickFalse)
-        break;
+    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+        (QuantumTick(y,image->rows) != MagickFalse))
+      {
+        status=image->progress_monitor(SaveImageTag,y,image->rows,
+          image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
   }
   q=EncodeRLE(q,buffer,literal,repeat);
   scanline=(unsigned char *) RelinquishMagickMemory(scanline);
@@ -870,7 +888,7 @@ static MagickBooleanType WritePDBImage(const ImageInfo *image_info,Image *image)
       /*
         Write the comment record header.
       */
-      (void) WriteBlobMSBLong(image,TellBlob(image)+8+58+q-p);
+      (void) WriteBlobMSBLong(image,TellBlob(image)+8+58+q-runlength);
       (void) WriteBlobByte(image,0x40);
       (void) WriteBlobByte(image,0x6f);
       (void) WriteBlobByte(image,0x80);
@@ -891,8 +909,8 @@ static MagickBooleanType WritePDBImage(const ImageInfo *image_info,Image *image)
   (void) WriteBlobMSBShort(image,pdb_image.y_anchor);
   (void) WriteBlobMSBShort(image,pdb_image.width);
   (void) WriteBlobMSBShort(image,pdb_image.height);
-  (void) WriteBlob(image,q-p,p);
-  p=(unsigned char *) RelinquishMagickMemory(p);
+  (void) WriteBlob(image,q-runlength,runlength);
+  runlength=(unsigned char *) RelinquishMagickMemory(runlength);
   if (pdb_info.number_records > 1)
     WriteBlobString(image,comment->value);
   CloseBlob(image);
