@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -42,14 +42,17 @@
 #include "magick/studio.h"
 #include "magick/attribute.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
 #include "magick/color.h"
 #include "magick/color_private.h"
 #include "magick/compress.h"
 #include "magick/constitute.h"
 #include "magick/delegate.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/geometry.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
@@ -173,7 +176,7 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
     status;
 
   RectangleInfo
-    box,
+    bounding_box,
     page;
 
   register char
@@ -205,14 +208,14 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
-      DestroyImageList(image);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
   /*
     Set the page density.
   */
-  dx_resolution=72.0;
-  dy_resolution=72.0;
+  dx_resolution=DefaultResolution;
+  dy_resolution=DefaultResolution;
   if ((image->x_resolution == 0.0) || (image->y_resolution == 0.0))
     {
       GeometryInfo
@@ -234,7 +237,7 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   */
   cmyk=MagickFalse;
   count=0;
-  (void) ResetMagickMemory(&box,0,sizeof(box));
+  (void) ResetMagickMemory(&bounding_box,0,sizeof(bounding_box));
   (void) ResetMagickMemory(&bounds,0,sizeof(bounds));
   (void) ResetMagickMemory(&page,0,sizeof(page));
   for (p=command; (c=ReadBlobByte(image)) != EOF; )
@@ -249,30 +252,21 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
       continue;
     *p='\0';
     p=command;
+    /*
+      Is this a CMYK document?
+    */
     if (LocaleNCompare(DeviceCMYK,command,strlen(DeviceCMYK)) == 0)
       cmyk=MagickTrue;
     if (LocaleNCompare(CropBox,command,strlen(CropBox)) == 0)
       {
         /*
-          Note region defined by media box.
+          Note region defined by crop box.
         */
         count=(ssize_t) sscanf(command,"CropBox [%lf %lf %lf %lf",
           &bounds.x1,&bounds.y1,&bounds.x2,&bounds.y2);
         if (count != 4)
           count=(ssize_t) sscanf(command,"CropBox[%lf %lf %lf %lf",
             &bounds.x1,&bounds.y1,&bounds.x2,&bounds.y2);
-        if (count != 4)
-          continue;
-        if ((bounds.x1 > bounds.x2) || (bounds.y1 > bounds.y2))
-          continue;
-        /*
-          Set PDF render geometry.
-        */
-        width=(unsigned long) (bounds.x2-bounds.x1+0.5);
-        height=(unsigned long) (bounds.y2-bounds.y1+0.5);
-        page.width=width;
-        page.height=height;
-        box=page;
       }
     if (LocaleNCompare(MediaBox,command,strlen(MediaBox)) == 0)
       {
@@ -284,24 +278,20 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
         if (count != 4)
           count=(ssize_t) sscanf(command,"MediaBox[%lf %lf %lf %lf",
             &bounds.x1,&bounds.y1,&bounds.x2,&bounds.y2);
-        if (count != 4)
-          continue;
-        if ((bounds.x1 > bounds.x2) || (bounds.y1 > bounds.y2))
-          continue;
-        /*
-          Set PDF render geometry.
-        */
-        width=(unsigned long) (bounds.x2-bounds.x1+0.5);
-        height=(unsigned long) (bounds.y2-bounds.y1+0.5);
-        if ((width <= box.width) && (height <= box.height))
-          continue;
-        page.width=width;
-        page.height=height;
-        box=page;
       }
+    if (count != 4)
+      continue;
+    /*
+      Set PDF render geometry.
+    */
+    width=(unsigned long) (bounds.x2-bounds.x1+0.5);
+    height=(unsigned long) (bounds.y2-bounds.y1+0.5);
+    if (width > page.width)
+      page.width=width;
+    if (height > page.height)
+      page.height=height;
   }
   CloseBlob(image);
-  DestroyImage(image);
   /*
     Create Ghostscript control file.
   */
@@ -310,10 +300,11 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
     {
       ThrowFileException(exception,FileOpenError,"UnableToCreateTemporaryFile",
         image_info->filename);
+      image=DestroyImage(image);
       return((Image *) NULL);
     }
   (void) write(file," ",1);
-  (void) close(file);
+  file=close(file)-1;
   /*
     Render postscript with the Ghostscript delegate.
   */
@@ -325,6 +316,7 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
     (page.width*image->x_resolution/dx_resolution+0.5);
   page.height=(unsigned long)
     (page.height*image->y_resolution/dy_resolution+0.5);
+  image=DestroyImage(image);
   (void) FormatMagickString(geometry,MaxTextExtent,"%lux%lu",
     page.width,page.height);
   if (image_info->monochrome != MagickFalse)
@@ -341,6 +333,7 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
     }
   *options='\0';
   read_info=CloneImageInfo(image_info);
+  *read_info->magick='\0';
   if (read_info->number_scenes != 0)
     {
       if (read_info->number_scenes != 1)
@@ -348,8 +341,11 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
           read_info->scene+read_info->number_scenes);
       else
         (void) FormatMagickString(options,MaxTextExtent,
-          "-dFirstPage=%lu -dLastPage=%lu",read_info->scene+1,
-          read_info->scene+read_info->number_scenes);
+          "-dFirstPage=%lu -dLastPage=%lu",read_info->scene+1,read_info->scene+
+          read_info->number_scenes);
+      read_info->number_scenes=0;
+      if (read_info->scenes != (char *) NULL)
+        *read_info->scenes='\0';
     }
   if (read_info->authenticate != (char *) NULL)
     (void) FormatMagickString(options+strlen(options),MaxTextExtent,
@@ -361,14 +357,11 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
     read_info->antialias != MagickFalse ? 4 : 1,
     read_info->antialias != MagickFalse ? 4 : 1,geometry,density,options,
     read_info->filename,postscript_filename,image_info->filename);
-  (void) MagickMonitor(RenderPostscriptText,0,8,&image->exception);
   status=InvokePostscriptDelegate(read_info->verbose,command);
-  *read_info->magick='\0';
   image=ReadImage(read_info,exception);
-  (void) MagickMonitor(RenderPostscriptText,7,8,&image->exception);
   (void) RelinquishUniqueFileResource(postscript_filename);
   (void) RelinquishUniqueFileResource(read_info->filename);
-  DestroyImageInfo(read_info);
+  read_info=DestroyImageInfo(read_info);
   if (image == (Image *) NULL)
     ThrowReaderException(DelegateError,"PostscriptDelegateFailed");
   if (LocaleCompare(image->magick,"BMP") == 0)
@@ -379,7 +372,7 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
       cmyk_image=ConsolidateCMYKImages(image,&image->exception);
       if (cmyk_image != (Image *) NULL)
         {
-          DestroyImageList(image);
+          image=DestroyImageList(image);
           image=cmyk_image;
         }
     }
@@ -563,7 +556,8 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
 
   MagickOffsetType
     offset,
-    scene;
+    scene,
+    *xref;
 
   MagickSizeType
     number_pixels;
@@ -605,12 +599,10 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
     info_id,
     object,
     pages_id,
+    quality,
     root_id,
     text_size;
 
-  MagickOffsetType
-    *xref;
-
   /*
     Open output image file.
   */
@@ -624,6 +616,8 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
   if (status == MagickFalse)
     return(status);
   compression=image->compression;
+  quality=(unsigned long)
+    (image->quality == UndefinedCompressionQuality ? 75 : image->quality);
   switch (compression)
   {
 #if !defined(HasJPEG)
@@ -675,8 +669,7 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
   (void) FormatMagickString(date,MaxTextExtent,"D:%04d%02d%02d%02d%02d%02d",
     time_meridian->tm_year+1900,time_meridian->tm_mon+1,time_meridian->tm_mday,
     time_meridian->tm_hour,time_meridian->tm_min,time_meridian->tm_sec);
-  (void) FormatMagickString(buffer,MaxTextExtent,"/CreationDate (%s)\n",
-    date);
+  (void) FormatMagickString(buffer,MaxTextExtent,"/CreationDate (%s)\n",date);
   (void) WriteBlobString(image,buffer);
   (void) FormatMagickString(buffer,MaxTextExtent,"/ModDate (%s)\n",date);
   (void) WriteBlobString(image,buffer);
@@ -740,15 +733,33 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
   do
   {
     /*
-      Scale image to size of Portable Document page.
+      Scale relative to dots-per-inch.
     */
-    text_size=0;
-    attribute=GetImageAttribute(image,"label");
-    if (attribute != (const ImageAttribute *) NULL)
-      text_size=(unsigned long)
-        (MultilineCensus(attribute->value)*image_info->pointsize+12);
+    dx_resolution=DefaultResolution;
+    dy_resolution=DefaultResolution;
+    flags=ParseGeometry(PSDensityGeometry,&geometry_info);
+    x_resolution=geometry_info.rho;
+    y_resolution=geometry_info.sigma;
+    if ((flags & SigmaValue) == 0)
+      y_resolution=x_resolution;
+    if (image->x_resolution != 0.0)
+      x_resolution=image->x_resolution;
+    if (image->y_resolution != 0.0)
+      y_resolution=image->y_resolution;
+    if (image_info->density != (char *) NULL)
+      {
+        flags=ParseGeometry(image_info->density,&geometry_info);
+        x_resolution=geometry_info.rho;
+        y_resolution=geometry_info.sigma;
+        if ((flags & SigmaValue) == 0)
+          y_resolution=x_resolution;
+      }
+    if (image->units == PixelsPerCentimeterResolution)
+      {
+        x_resolution*=2.54;
+        y_resolution*=2.54;
+      }
     SetGeometry(image,&geometry);
-    geometry.y=(long) text_size;
     (void) FormatMagickString(page_geometry,MaxTextExtent,"%lux%lu",
       image->columns,image->rows);
     if (image_info->page != (char *) NULL)
@@ -763,36 +774,22 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
           (void) strcpy(page_geometry,PSPageGeometry);
     (void) ParseMetaGeometry(page_geometry,&geometry.x,&geometry.y,
       &geometry.width,&geometry.height);
-    (void) ParseAbsoluteGeometry(page_geometry,&media_info);
-    (void) ParseGravityGeometry(image,page_geometry,&page_info);
-    /*
-      Scale relative to dots-per-inch.
-    */
-    dx_resolution=72.0;
-    dy_resolution=72.0;
-    x_resolution=72.0;
-    flags=ParseGeometry(PSDensityGeometry,&geometry_info);
-    x_resolution=geometry_info.rho;
-    y_resolution=geometry_info.sigma;
-    if ((flags & SigmaValue) == 0)
-      y_resolution=x_resolution;
-    if (image_info->density != (char *) NULL)
-      {
-        flags=ParseGeometry(image_info->density,&geometry_info);
-        x_resolution=geometry_info.rho;
-        y_resolution=geometry_info.sigma;
-        if ((flags & SigmaValue) == 0)
-          y_resolution=x_resolution;
-      }
-    if (image->units == PixelsPerCentimeterResolution)
-      {
-        x_resolution*=2.54;
-        y_resolution*=2.54;
-      }
     x_scale=(double) (geometry.width*dx_resolution)/x_resolution;
     geometry.width=(unsigned long) (x_scale+0.5);
     y_scale=(double) (geometry.height*dy_resolution)/y_resolution;
     geometry.height=(unsigned long) (y_scale+0.5);
+    (void) ParseAbsoluteGeometry(page_geometry,&media_info);
+    (void) ParseGravityGeometry(image,page_geometry,&page_info);
+    if (image->gravity != UndefinedGravity)
+      {
+        geometry.x=(-page_info.x);
+        geometry.y=(long) (media_info.height+page_info.y-image->rows);
+      }
+    text_size=0;
+    attribute=GetImageAttribute(image,"label");
+    if (attribute != (const ImageAttribute *) NULL)
+      text_size=(unsigned long)
+        (MultilineCensus(attribute->value)*image_info->pointsize+12);
     /*
       Write Page object.
     */
@@ -857,7 +854,8 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
             image->scene,image_info->pointsize);
           (void) WriteBlobString(image,buffer);
           (void) FormatMagickString(buffer,MaxTextExtent,"%ld %ld Td\n",
-            geometry.x,geometry.y+geometry.height+i*image_info->pointsize+12);
+            geometry.x,(long) (geometry.y+geometry.height+i*
+            image_info->pointsize+12));
           (void) WriteBlobString(image,buffer);
           (void) FormatMagickString(buffer,MaxTextExtent,"(%s) Tj\n",
             labels[i]);
@@ -988,7 +986,7 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
             pixels=(unsigned char *) AcquireMagickMemory(length);
             if (pixels == (unsigned char *) NULL)
               {
-                DestroyImage(image);
+                image=DestroyImage(image);
                 ThrowWriterException(ResourceLimitError,
                   "MemoryAllocationFailed");
               }
@@ -1010,8 +1008,7 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
             }
 #if defined(HasZLIB)
             if (compression == ZipCompression)
-              status=ZLIBEncodeImage(image,length,image_info->quality == 0UL ?
-                75UL : image_info->quality,pixels);
+              status=ZLIBEncodeImage(image,length,quality,pixels);
             else
 #endif
               if (compression == LZWCompression)
@@ -1192,18 +1189,18 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
                 p++;
               }
               if (image->previous == (Image *) NULL)
-                if (QuantumTick(y,image->rows) != MagickFalse)
+                if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                    (QuantumTick(y,image->rows) != MagickFalse))
                   {
-                    status=MagickMonitor(SaveImageTag,y,image->rows,
-                      &image->exception);
+                    status=image->progress_monitor(SaveImageTag,y,image->rows,
+                      image->client_data);
                     if (status == MagickFalse)
                       break;
                   }
             }
 #if defined(HasZLIB)
             if (compression == ZipCompression)
-              status=ZLIBEncodeImage(image,length,image_info->quality == 0UL ?
-                75UL : image_info->quality,pixels);
+              status=ZLIBEncodeImage(image,length,quality,pixels);
             else
 #endif
               if (compression == LZWCompression)
@@ -1237,10 +1234,11 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
                 p++;
               }
               if (image->previous == (Image *) NULL)
-                if (QuantumTick(y,image->rows) != MagickFalse)
+                if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                    (QuantumTick(y,image->rows) != MagickFalse))
                   {
-                    status=MagickMonitor(SaveImageTag,y,image->rows,
-                      &image->exception);
+                    status=image->progress_monitor(SaveImageTag,y,image->rows,
+                      image->client_data);
                     if (status == MagickFalse)
                       break;
                   }
@@ -1294,18 +1292,18 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
                 p++;
               }
               if (image->previous == (Image *) NULL)
-                if (QuantumTick(y,image->rows) != MagickFalse)
+                if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                    (QuantumTick(y,image->rows) != MagickFalse))
                   {
-                    status=MagickMonitor(SaveImageTag,y,image->rows,
-                      &image->exception);
+                    status=image->progress_monitor(SaveImageTag,y,image->rows,
+                      image->client_data);
                     if (status == MagickFalse)
                       break;
                   }
             }
 #if defined(HasZLIB)
             if (compression == ZipCompression)
-              status=ZLIBEncodeImage(image,length,image_info->quality == 0UL ?
-                75UL : image_info->quality,pixels);
+              status=ZLIBEncodeImage(image,length,quality,pixels);
             else
 #endif
               if (compression == LZWCompression)
@@ -1343,10 +1341,11 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
                 p++;
               }
               if (image->previous == (Image *) NULL)
-                if (QuantumTick(y,image->rows) != MagickFalse)
+                if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                    (QuantumTick(y,image->rows) != MagickFalse))
                   {
-                    status=MagickMonitor(SaveImageTag,y,image->rows,
-                      &image->exception);
+                    status=image->progress_monitor(SaveImageTag,y,image->rows,
+                      image->client_data);
                     if (status == MagickFalse)
                       break;
                   }
@@ -1387,18 +1386,18 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
                 for (x=0; x < (long) image->columns; x++)
                   *q++=(unsigned char) indexes[x];
                 if (image->previous == (Image *) NULL)
-                  if (QuantumTick(y,image->rows) != MagickFalse)
-                    {
-                      status=MagickMonitor(SaveImageTag,y,image->rows,
-                        &image->exception);
-                      if (status == MagickFalse)
-                        break;
-                    }
+                if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                    (QuantumTick(y,image->rows) != MagickFalse))
+                  {
+                    status=image->progress_monitor(SaveImageTag,y,image->rows,
+                      image->client_data);
+                    if (status == MagickFalse)
+                      break;
+                  }
               }
 #if defined(HasZLIB)
               if (compression == ZipCompression)
-                status=ZLIBEncodeImage(image,length,image_info->quality == 0UL ?
-                  75UL : image_info->quality,pixels);
+                status=ZLIBEncodeImage(image,length,quality,pixels);
               else
 #endif
                 if (compression == LZWCompression)
@@ -1429,13 +1428,14 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
                 for (x=0; x < (long) image->columns; x++)
                   Ascii85Encode(image,(unsigned char) indexes[x]);
                 if (image->previous == (Image *) NULL)
-                  if (QuantumTick(y,image->rows) != MagickFalse)
-                    {
-                      status=MagickMonitor(SaveImageTag,y,image->rows,
-                        &image->exception);
-                      if (status == MagickFalse)
-                        break;
-                    }
+                if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                    (QuantumTick(y,image->rows) != MagickFalse))
+                  {
+                    status=image->progress_monitor(SaveImageTag,y,image->rows,
+                      image->client_data);
+                    if (status == MagickFalse)
+                      break;
+                  }
               }
               Ascii85Flush(image);
               break;
@@ -1585,7 +1585,7 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
             pixels=(unsigned char *) AcquireMagickMemory(length);
             if (pixels == (unsigned char *) NULL)
               {
-                DestroyImage(tile_image);
+                tile_image=DestroyImage(tile_image);
                 ThrowWriterException(ResourceLimitError,
                   "MemoryAllocationFailed");
               }
@@ -1607,8 +1607,7 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
             }
 #if defined(HasZLIB)
             if (compression == ZipCompression)
-              status=ZLIBEncodeImage(image,length,image_info->quality == 0UL ?
-                75UL : image_info->quality,pixels);
+              status=ZLIBEncodeImage(image,length,quality,pixels);
             else
 #endif
               if (compression == LZWCompression)
@@ -1670,7 +1669,7 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
             pixels=(unsigned char *) AcquireMagickMemory(length);
             if (pixels == (unsigned char *) NULL)
               {
-                DestroyImage(tile_image);
+                tile_image=DestroyImage(tile_image);
                 ThrowWriterException(ResourceLimitError,
                   "MemoryAllocationFailed");
               }
@@ -1697,8 +1696,7 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
             }
 #if defined(HasZLIB)
             if (compression == ZipCompression)
-              status=ZLIBEncodeImage(image,length,image_info->quality == 0UL ?
-                75UL : image_info->quality,pixels);
+              status=ZLIBEncodeImage(image,length,quality,pixels);
             else
 #endif
               if (compression == LZWCompression)
@@ -1757,7 +1755,7 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
               pixels=(unsigned char *) AcquireMagickMemory(length);
               if (pixels == (unsigned char *) NULL)
                 {
-                  DestroyImage(tile_image);
+                  tile_image=DestroyImage(tile_image);
                   ThrowWriterException(ResourceLimitError,
                     "MemoryAllocationFailed");
                 }
@@ -1777,8 +1775,7 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
               }
 #if defined(HasZLIB)
               if (compression == ZipCompression)
-                status=ZLIBEncodeImage(image,length,image_info->quality == 0UL ?
-                  75UL : image_info->quality,pixels);
+                status=ZLIBEncodeImage(image,length,quality,pixels);
               else
 #endif
                 if (compression == LZWCompression)
@@ -1814,7 +1811,7 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
             }
           }
         }
-    DestroyImage(tile_image);
+    tile_image=DestroyImage(tile_image);
     offset=TellBlob(image)-offset;
     (void) WriteBlobString(image,"\nendstream\n");
     (void) WriteBlobString(image,"endobj\n");
@@ -1888,10 +1885,14 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
     if (image->next == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
-      &image->exception);
-    if (status == MagickFalse)
-      break;
+    if (image->progress_monitor != (MagickProgressMonitor) NULL)
+      {
+        status=image->progress_monitor(SaveImagesTag,scene,
+          GetImageListLength(image),image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
+    scene++;
   } while (image_info->adjoin != MagickFalse);
   /*
     Write Xref object.
