@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -41,12 +41,15 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
 #include "magick/color_private.h"
 #include "magick/colorspace.h"
 #include "magick/composite.h"
 #include "magick/constitute.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/log.h"
 #include "magick/magick.h"
@@ -323,9 +326,6 @@ static MagickBooleanType
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method DecodeImage returns MagickTrue if all the pixels are
-%      uncompressed without error, otherwise MagickFalse.
-%
 %    o image_info: The image info.
 %
 %    o blob,image: The address of a structure of type Image.
@@ -547,8 +547,6 @@ static unsigned char *DecodeImage(const ImageInfo *image_info,Image *blob,
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method EncodeImage returns the number of encoded pixels.
-%
 %    o image: The address of a structure of type Image.
 %
 %    o scanline: A pointer to an array of characters to pack.
@@ -700,8 +698,8 @@ static size_t EncodeImage(Image *image,const unsigned char *scanline,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  IsPICT()() returns MagickTrue if the image format type, identified by the magick
-%  string, is PICT.
+%  IsPICT()() returns MagickTrue if the image format type, identified by the
+%  magick string, is PICT.
 %
 %  The format of the ReadPICTImage method is:
 %
@@ -709,8 +707,6 @@ static size_t EncodeImage(Image *image,const unsigned char *scanline,
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method IsPICT() returns MagickTrue if the image format type is PICT.
-%
 %    o magick: This string is generally the first few bytes of an image file
 %      or blob.
 %
@@ -750,10 +746,6 @@ static MagickBooleanType IsPICT(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o image:  Method ReadPICTImage returns a pointer to the image after
-%      reading.  A null image is returned if there is a memory shortage or
-%      if the image cannot be read.
-%
 %    o image_info: The image info.
 %
 %    o exception: return any errors or warnings in this structure.
@@ -820,15 +812,18 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
+  image->depth=8;
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
-      DestroyImageList(image);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
   /*
     Read PICT header.
   */
+  pixmap.bits_per_pixel=0;
+  pixmap.component_count=0;
   for (i=0; i < 512; i++)
     (void) ReadBlobByte(image);  /* skip header */
   (void) ReadBlobMSBShort(image);  /* skip picture size */
@@ -925,6 +920,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
             length=ReadBlobMSBShort(image);
             ReadRectangle(frame);
             ReadPixmap(pixmap);
+            image->depth=pixmap.component_size;
             (void) ReadBlobMSBLong(image);
             flags=ReadBlobMSBShort(image);
             length=ReadBlobMSBShort(image);
@@ -932,9 +928,6 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
               (void) ReadBlobMSBLong(image);
             width=frame.bottom-frame.top;
             height=frame.right-frame.left;
-            image->depth=pixmap.bits_per_pixel <= 8 ? 8 : QuantumDepth;
-            if (pixmap.bits_per_pixel < 8)
-              image->depth=8;
             if (pixmap.bits_per_pixel <= 8)
               length&=0x7fff;
             if (pixmap.bits_per_pixel == 16)
@@ -1037,7 +1030,9 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
             if ((code == 0x9a) || (code == 0x9b) || (bytes_per_line & 0x8000))
               {
                 ReadPixmap(pixmap);
-                tile_image->matte=pixmap.component_count == 4;
+                tile_image->depth=pixmap.component_size;
+                tile_image->matte=
+                  pixmap.component_count == 4 ? MagickTrue : MagickFalse;
                 if (tile_image->matte != MagickFalse)
                   image->matte=tile_image->matte;
               }
@@ -1056,7 +1051,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                 status=AllocateImageColormap(tile_image,tile_image->colors);
                 if (status == MagickFalse)
                   {
-                    DestroyImage(tile_image);
+                    tile_image=DestroyImage(tile_image);
                     ThrowReaderException(ResourceLimitError,
                       "MemoryAllocationFailed");
                   }
@@ -1108,7 +1103,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                 pixmap.bits_per_pixel);
             if (pixels == (unsigned char *) NULL)
               {
-                DestroyImage(tile_image);
+                tile_image=DestroyImage(tile_image);
                 ThrowReaderException(ResourceLimitError,
                   "MemoryAllocationFailed");
               }
@@ -1171,10 +1166,11 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                   (pixmap.bits_per_pixel != 16))
                 p+=(pixmap.component_count-1)*tile_image->columns;
               if (destination.bottom == (long) image->rows)
-                if (QuantumTick(y,tile_image->rows))
+                if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                    (QuantumTick(y,tile_image->rows) != MagickFalse))
                   {
-                    status=MagickMonitor(LoadImageTag,y,tile_image->rows,
-                      &image->exception);
+                    status=image->progress_monitor(LoadImageTag,y,
+                      tile_image->rows,image->client_data);
                     if (status == MagickFalse)
                       break;
                   }
@@ -1185,11 +1181,11 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                   ((bytes_per_line & 0x8000) != 0))
                 (void) CompositeImage(image,CopyCompositeOp,tile_image,
                   destination.left,destination.top);
-            DestroyImage(tile_image);
-            if (destination.bottom != (long) image->rows)
+            tile_image=DestroyImage(tile_image);
+            if (image->progress_monitor != (MagickProgressMonitor) NULL)
               {
-                status=MagickMonitor(LoadImageTag,destination.bottom,
-                  image->rows,&image->exception);
+                status=image->progress_monitor(LoadImageTag,destination.bottom,
+                  image->rows,image->client_data);
                 if (status == MagickFalse)
                   break;
               }
@@ -1227,7 +1223,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                 profile=AcquireStringInfo(length);
                 SetStringInfoDatum(profile,info);
                 status=SetImageProfile(image,"icc",profile);
-                DestroyStringInfo(profile);
+                profile=DestroyStringInfo(profile);
                 if (status == MagickFalse)
                   ThrowReaderException(ResourceLimitError,
                     "MemoryAllocationFailed");
@@ -1239,11 +1235,11 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                   break;
                 profile=AcquireStringInfo(length);
                 SetStringInfoDatum(profile,info);
-                status=SetImageProfile(image,"8bim",profile);
+                status=SetImageProfile(image,"iptc",profile);
                 if (status == MagickFalse)
                   ThrowReaderException(ResourceLimitError,
                     "MemoryAllocationFailed");
-                DestroyStringInfo(profile);
+                profile=DestroyStringInfo(profile);
                 break;
               }
               default:
@@ -1308,8 +1304,8 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
               MaxTextExtent);
             ThrowFileException(exception,FileOpenError,
               "UnableToCreateTemporaryFile",image->filename);
-            DestroyImageList(image);
-            return(MagickFalse);
+            image=DestroyImageList(image);
+            return((Image *) NULL);
           }
         length=ReadBlobMSBLong(image);
         for (i=0; i < 6; i++)
@@ -1324,8 +1320,8 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
         }
         (void) fclose(file);
         tile_image=ReadImage(read_info,exception);
-        DestroyImageInfo(read_info);
         (void) RelinquishUniqueFileResource(read_info->filename);
+        read_info=DestroyImageInfo(read_info);
         if (tile_image == (Image *) NULL)
           continue;
         (void) FormatMagickString(geometry,MaxTextExtent,"%lux%lu",
@@ -1335,7 +1331,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
         (void) CompositeImage(image,CopyCompositeOp,tile_image,frame.left,
           frame.right);
         image->compression=tile_image->compression;
-        DestroyImage(tile_image);
+        tile_image=DestroyImage(tile_image);
         continue;
       }
     if ((code == 0xff) || (code == 0xffff))
@@ -1458,10 +1454,6 @@ ModuleExport void UnregisterPICTImage(void)
 %
 %  A description of each parameter follows.
 %
-%    o status: Method WritePICTImage returns True if the image is written.
-%      False is returned is there is a memory shortage or if the image file
-%      fails to write.
-%
 %    o image_info: The image info.
 %
 %    o image:  The image.
@@ -1578,8 +1570,8 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
   pixmap.table=0;
   pixmap.reserved=0;
   transfer_mode=0;
-  x_resolution=image->x_resolution ? image->x_resolution : 72.0;
-  y_resolution=image->y_resolution ? image->y_resolution : 72.0;
+  x_resolution=image->x_resolution ? image->x_resolution : DefaultResolution;
+  y_resolution=image->y_resolution ? image->y_resolution : DefaultResolution;
   storage_class=image->storage_class;
   if (image->compression == JPEGCompression)
     storage_class=DirectClass;
@@ -1633,7 +1625,7 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
   (void) WriteBlobMSBShort(image,frame_rectangle.bottom);
   (void) WriteBlobMSBShort(image,frame_rectangle.right);
   (void) WriteBlobMSBLong(image,0x00000000L);
-  profile=GetImageProfile(image,"8bim");
+  profile=GetImageProfile(image,"iptc");
   if (profile != (StringInfo *) NULL)
     {
       (void) WriteBlobMSBShort(image,0xa1);
@@ -1688,7 +1680,7 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
         &image->exception);
       if (blob == (unsigned char *) NULL)
         return(MagickFalse);
-      DestroyImage(jpeg_image);
+      jpeg_image=DestroyImage(jpeg_image);
       (void) WriteBlobMSBShort(image,PictJPEGOp);
       (void) WriteBlobMSBLong(image,length+154);
       (void) WriteBlobMSBShort(image,0x0000);
@@ -1821,9 +1813,11 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
       for (x=0; x < (long) image->columns; x++)
         scanline[x]=indexes[x];
       count+=EncodeImage(image,scanline,row_bytes & 0x7FFF,packed_scanline);
-      if (QuantumTick(y,image->rows) != 0)
+      if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+          (QuantumTick(y,image->rows) != MagickFalse))
         {
-          status=MagickMonitor(SaveImageTag,y,image->rows,&image->exception);
+          status=image->progress_monitor(SaveImageTag,y,image->rows,
+            image->client_data);
           if (status == MagickFalse)
             break;
         }
@@ -1873,10 +1867,11 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
           }
           count+=EncodeImage(image,scanline,bytes_per_line & 0x7FFF,
             packed_scanline);
-          if (QuantumTick(y,image->rows) != 0)
+          if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+              (QuantumTick(y,image->rows) != MagickFalse))
             {
-              status=MagickMonitor(SaveImageTag,y,image->rows,
-                &image->exception);
+              status=image->progress_monitor(SaveImageTag,y,image->rows,
+                image->client_data);
               if (status == MagickFalse)
                 break;
             }
