@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -42,11 +42,14 @@
 #include "magick/studio.h"
 #include "magick/attribute.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
 #include "magick/color.h"
 #include "magick/color_private.h"
 #include "magick/colorspace.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
@@ -80,8 +83,6 @@ static MagickBooleanType
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method IsPNM() returns MagickTrue if the image format type is PNM.
-%
 %    o magick: This string is generally the first few bytes of an image file
 %      or blob.
 %
@@ -120,10 +121,6 @@ static MagickBooleanType IsPNM(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o image:  Method ReadPNMImage returns a pointer to the image after
-%      reading.  A null image is returned if there is a memory shortage or
-%      if the image cannot be read.
-%
 %    o image_info: The image info.
 %
 %    o exception: return any errors or warnings in this structure.
@@ -168,32 +165,30 @@ static unsigned long PNMInteger(Image *image,const unsigned int base)
           Read comment.
         */
         length=MaxTextExtent;
-        comment=(char *) AcquireMagickMemory(length+strlen(P7Comment)+1);
-        p=comment;
-        offset=p-comment;
-        if (comment != (char *) NULL)
-          for ( ; (c != EOF) && (c != (int) '\n'); p++)
-          {
-            if ((size_t) (p-comment) >= length)
-              {
-                length<<=1;
-                length+=MaxTextExtent;
-                comment=(char *)
-                  ResizeMagickMemory(comment,length+strlen(P7Comment)+1);
-                if (comment == (char *) NULL)
-                  break;
-                p=comment+strlen(comment);
-              }
-            c=ReadBlobByte(image);
-            *p=(char) c;
-            *(p+1)='\0';
-          }
+        offset=0;
+        comment=AcquireString((char *) NULL);
+        for (p=comment; (c != EOF) && (c != (int) '\n'); p++)
+        {
+          if ((size_t) (p-comment+1) >= length)
+            {
+              *p='\0';
+              length<<=1;
+              comment=(char *) ResizeMagickMemory(comment,
+                (length+MaxTextExtent)*sizeof(*comment));
+              if (comment == (char *) NULL)
+                break;
+              p=comment+strlen(comment);
+            }
+          c=ReadBlobByte(image);
+          *p=(char) c;
+        }
+        *p='\0';
         if (comment == (char *) NULL)
           return(0);
         q=comment+offset;
         if (LocaleCompare(q,P7Comment) == 0)
           *q='\0';
-        (void) SetImageAttribute(image,"comment",comment);
+        (void) SetImageAttribute(image,"Comment",comment);
         comment=(char *) RelinquishMagickMemory(comment);
         continue;
       }
@@ -244,9 +239,6 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
   MagickBooleanType
     status;
 
-  MonitorHandler
-    handler;
-
   Quantum
     *scale;
 
@@ -291,7 +283,7 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
-      DestroyImageList(image);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
   /*
@@ -354,7 +346,7 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
           Compute pixel scaling table.
         */
         scale=(Quantum *)
-          AcquireMagickMemory((size_t) (max_value+1)*sizeof(Quantum));
+          AcquireMagickMemory((size_t) (max_value+1)*sizeof(*scale));
         if (scale == (Quantum *) NULL)
           ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
         for (i=0; i <= (long) max_value; i++)
@@ -394,9 +386,14 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
           if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows) != 0)
-              if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
-                break;
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(y,image->rows) != MagickFalse))
+              {
+                status=image->progress_monitor(LoadImageTag,y,image->rows,
+                  image->client_data);
+                if (status == MagickFalse)
+                  break;
+              }
         }
         break;
       }
@@ -432,9 +429,14 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
           if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows) != 0)
-              if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
-                break;
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(y,image->rows) != MagickFalse))
+              {
+                status=image->progress_monitor(LoadImageTag,y,image->rows,
+                  image->client_data);
+                if (status == MagickFalse)
+                  break;
+              }
         }
         break;
       }
@@ -467,9 +469,14 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
           if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows) != 0)
-              if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
-                break;
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(y,image->rows) != MagickFalse))
+              {
+                status=image->progress_monitor(LoadImageTag,y,image->rows,
+                  image->client_data);
+                if (status == MagickFalse)
+                  break;
+              }
         }
         break;
       }
@@ -505,9 +512,14 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
           if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows) != 0)
-              if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
-                break;
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(y,image->rows) != MagickFalse))
+              {
+                status=image->progress_monitor(LoadImageTag,y,image->rows,
+                  image->client_data);
+                if (status == MagickFalse)
+                  break;
+              }
         }
         if (EOFBlob(image) != MagickFalse)
           ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
@@ -566,9 +578,14 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
           if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows) != 0)
-              if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
-                break;
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(y,image->rows) != MagickFalse))
+              {
+                status=image->progress_monitor(LoadImageTag,y,image->rows,
+                  image->client_data);
+                if (status == MagickFalse)
+                  break;
+              }
         }
         pixels=(unsigned char *) RelinquishMagickMemory(pixels);
         if (EOFBlob(image) != MagickFalse)
@@ -631,13 +648,16 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
           if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows) != 0)
-              if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
-                break;
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(y,image->rows) != MagickFalse))
+              {
+                status=image->progress_monitor(LoadImageTag,y,image->rows,
+                  image->client_data);
+                if (status == MagickFalse)
+                  break;
+              }
         }
         pixels=(unsigned char *) RelinquishMagickMemory(pixels);
-        handler=SetMonitorHandler((MonitorHandler) NULL);
-        (void) SetMonitorHandler(handler);
         if (EOFBlob(image) != MagickFalse)
           (void) ThrowMagickException(exception,GetMagickModule(),
             CorruptImageError,"UnexpectedEndOfFile",image->filename);
@@ -654,9 +674,7 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
     if (image_info->number_scenes != 0)
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
-    if ((format != '1') && (format != '2') && (format != '3'))
-      count=ReadBlob(image,1,(unsigned char *) &format);
-    else
+    if ((format == '1') || (format == '2') || (format == '3'))
       do
       {
         /*
@@ -668,6 +686,7 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
         if ((count != 0) && (format == 'P'))
           break;
       } while (format != '\n');
+    count=ReadBlob(image,1,(unsigned char *) &format);
     if ((count != 0) && (format == 'P'))
       {
         /*
@@ -676,14 +695,17 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
         AllocateNextImage(image_info,image);
         if (image->next == (Image *) NULL)
           {
-            DestroyImageList(image);
+            image=DestroyImageList(image);
             return((Image *) NULL);
           }
         image=SyncNextImageInList(image);
-        status=MagickMonitor(LoadImagesTag,TellBlob(image),GetBlobSize(image),
-          exception);
-        if (status == MagickFalse)
-          break;
+        if (image->progress_monitor != (MagickProgressMonitor) NULL)
+          {
+            status=image->progress_monitor(LoadImagesTag,TellBlob(image),
+              GetBlobSize(image),image->client_data);
+            if (status == MagickFalse)
+              break;
+          }
       }
   } while ((count != 0) && (format == 'P'));
   CloseBlob(image);
@@ -799,10 +821,6 @@ ModuleExport void UnregisterPNMImage(void)
 %
 %  A description of each parameter follows.
 %
-%    o status: Method WritePNMImage returns True if the image is written.
-%      False is returned is there is a memory shortage or if the image file
-%      fails to write.
-%
 %    o image_info: The image info.
 %
 %    o image:  The image.
@@ -887,7 +905,7 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image)
         (void) strcpy(buffer,"P7 332\n");
       }
     (void) WriteBlobString(image,buffer);
-    attribute=GetImageAttribute(image,"comment");
+    attribute=GetImageAttribute(image,"Comment");
     if (attribute != (const ImageAttribute *) NULL)
       {
         register char
@@ -953,10 +971,11 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image)
               }
           }
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows) != 0)
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(y,image->rows) != MagickFalse))
               {
-                status=MagickMonitor(SaveImageTag,y,image->rows,
-                  &image->exception);
+                status=image->progress_monitor(SaveImageTag,y,image->rows,
+                  image->client_data);
                 if (status == MagickFalse)
                   break;
               }
@@ -999,10 +1018,11 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image)
             p++;
           }
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows) != 0)
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(y,image->rows) != MagickFalse))
               {
-                status=MagickMonitor(SaveImageTag,y,image->rows,
-                  &image->exception);
+                status=image->progress_monitor(SaveImageTag,y,image->rows,
+                  image->client_data);
                 if (status == MagickFalse)
                   break;
               }
@@ -1046,10 +1066,11 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image)
             p++;
           }
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows) != 0)
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(y,image->rows) != MagickFalse))
               {
-                status=MagickMonitor(SaveImageTag,y,image->rows,
-                  &image->exception);
+                status=image->progress_monitor(SaveImageTag,y,image->rows,
+                  image->client_data);
                 if (status == MagickFalse)
                   break;
               }
@@ -1102,10 +1123,11 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image)
           if (bit != 0)
             (void) WriteBlobByte(image,(unsigned char) (byte << (8-bit)));
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows) != 0)
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(y,image->rows) != MagickFalse))
               {
-                status=MagickMonitor(SaveImageTag,y,image->rows,
-                  &image->exception);
+                status=image->progress_monitor(SaveImageTag,y,image->rows,
+                  image->client_data);
                 if (status == MagickFalse)
                   break;
               }
@@ -1141,10 +1163,11 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image)
               p++;
             }
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows) != 0)
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(y,image->rows) != MagickFalse))
               {
-                status=MagickMonitor(SaveImageTag,y,image->rows,
-                  &image->exception);
+                status=image->progress_monitor(SaveImageTag,y,image->rows,
+                  image->client_data);
                 if (status == MagickFalse)
                   break;
               }
@@ -1203,10 +1226,11 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image)
             }
           (void) WriteBlob(image,(size_t) (q-pixels),pixels);
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows) != 0)
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(y,image->rows) != MagickFalse))
               {
-                status=MagickMonitor(SaveImageTag,y,image->rows,
-                  &image->exception);
+                status=image->progress_monitor(SaveImageTag,y,image->rows,
+                  image->client_data);
                 if (status == MagickFalse)
                   break;
               }
@@ -1254,11 +1278,11 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image)
           for (j=0; j < 16; j++)
           {
             red_map[i][j]=(unsigned long *)
-              AcquireMagickMemory(256*sizeof(unsigned long));
+              AcquireMagickMemory(256*sizeof(*red_map));
             green_map[i][j]=(unsigned long *)
-              AcquireMagickMemory(256*sizeof(unsigned long));
+              AcquireMagickMemory(256*sizeof(*green_map));
             blue_map[i][j]=(unsigned long *)
-              AcquireMagickMemory(256*sizeof(unsigned long));
+              AcquireMagickMemory(256*sizeof(*blue_map));
             if ((red_map[i][j] == (unsigned long *) NULL) ||
                 (green_map[i][j] == (unsigned long *) NULL) ||
                 (blue_map[i][j] == (unsigned long *) NULL))
@@ -1325,10 +1349,11 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image)
           i++;
           if (i == 2)
             i=0;
-          if (QuantumTick(y,image->rows) != 0)
+          if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+              (QuantumTick(y,image->rows) != MagickFalse))
             {
-              status=MagickMonitor(SaveImageTag,y,image->rows,
-                &image->exception);
+              status=image->progress_monitor(SaveImageTag,y,image->rows,
+                image->client_data);
               if (status == MagickFalse)
                 break;
             }
@@ -1352,10 +1377,14 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image)
     if (image->next == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
-      &image->exception);
-    if (status == MagickFalse)
-      break;
+    if (image->progress_monitor != (MagickProgressMonitor) NULL)
+      {
+        status=image->progress_monitor(SaveImagesTag,scene,
+          GetImageListLength(image),image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
+    scene++;
   } while (image_info->adjoin != MagickFalse);
   CloseBlob(image);
   return(MagickTrue);
