@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -42,11 +42,14 @@
 #include "magick/studio.h"
 #include "magick/attribute.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
 #include "magick/client.h"
 #include "magick/constitute.h"
 #include "magick/effect.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
@@ -142,21 +145,19 @@ ModuleExport void UnregisterPREVIEWImage(void)
 %
 %  The format of the WritePreviewImage method is:
 %
-%      MagickBooleanType WritePreviewImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WritePreviewImage(const ImageInfo *image_info,
+%        Image *image)
 %
 %  A description of each parameter follows.
 %
-%    o status: Method WritePreviewImage returns True if the image is written.
-%      False is returned is there is a memory shortage or if the image file
-%      fails to write.
-%
 %    o image_info: The image info.
 %
 %    o image:  The image.
 %
 %
 */
-static MagickBooleanType WritePreviewImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WritePreviewImage(const ImageInfo *image_info,
+  Image *image)
 {
   Image
     *preview_image;
@@ -179,6 +180,6 @@ static MagickBooleanType WritePreviewImage(const ImageInfo *image_info,Image *im
   (void) FormatMagickString(preview_image->filename,MaxTextExtent,
     "miff:%s",image_info->filename);
   status=WriteImage(image_info,preview_image);
-  DestroyImage(preview_image);
+  preview_image=DestroyImage(preview_image);
   return(status);
 }
