@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -42,14 +42,17 @@
 #include "magick/studio.h"
 #include "magick/attribute.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
 #include "magick/color.h"
 #include "magick/color_private.h"
 #include "magick/colorspace.h"
 #include "magick/constitute.h"
 #include "magick/delegate.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/geometry.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
@@ -58,6 +61,7 @@
 #include "magick/resource_.h"
 #include "magick/static.h"
 #include "magick/string_.h"
+#include "magick/token.h"
 #include "magick/transform.h"
 #include "magick/utility.h"
 
@@ -165,12 +169,10 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
 #define BoundingBox  "BoundingBox:"
 #define BeginDocument  "BeginDocument:"
 #define DocumentMedia  "DocumentMedia:"
-#define DocumentProcessColors  "DocumentProcessColors:"
 #define EndDocument  "EndDocument:"
 #define HiResBoundingBox  "HiResBoundingBox:"
 #define PageBoundingBox  "PageBoundingBox:"
 #define PageMedia  "PageMedia:"
-#define PageProcessColors  "PageProcessColors:"
 #define PhotoshopProfile  "BeginPhotoshop:"
 #define PostscriptLevel  "%!PS-"
 #define RenderPostscriptText  "  Rendering postscript...  "
@@ -182,6 +184,8 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
     geometry[MaxTextExtent],
     options[MaxTextExtent],
     postscript_filename[MaxTextExtent],
+    *q,
+    token[MaxTextExtent],
     translate_geometry[MaxTextExtent];
 
   const DelegateInfo
@@ -252,7 +256,7 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
-      DestroyImageList(image);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
   /*
@@ -283,8 +287,8 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   /*
     Set the page density.
   */
-  dx_resolution=72.0;
-  dy_resolution=72.0;
+  dx_resolution=DefaultResolution;
+  dy_resolution=DefaultResolution;
   if ((image->x_resolution == 0.0) || (image->y_resolution == 0.0))
     {
       flags=ParseGeometry(PSDensityGeometry,&geometry_info);
@@ -337,20 +341,28 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
           iptc_profile->datum[i]=(unsigned char)
             ProfileInteger(image,hex_digits);
       }
-    if (LocaleNCompare(DocumentProcessColors,command,strlen(DocumentProcessColors)) == 0)
-      cmyk=MagickTrue;
-    if (LocaleNCompare(PageProcessColors,command,strlen(PageProcessColors)) == 0)
-      cmyk=MagickTrue;
+    /*
+      Is this a CMYK document?
+    */
+    for (q=p; *q != '\0'; )
+    {
+      GetToken(q,&q,token);
+      if (LocaleCompare(token,"cmyk") == 0)
+        cmyk=MagickTrue;
+    }
     /*
       Note region defined by bounding box.
     */
+    if ((bounds.x1 != 0.0) || (bounds.y1 != 0.0) || (bounds.x2 != 0.0) ||
+        (bounds.y2 != 0.0))
+      continue;
     count=0;
     if (LocaleNCompare(BoundingBox,command,strlen(BoundingBox)) == 0)
-      count=(ssize_t) sscanf(command,"BoundingBox: %lf %lf %lf %lf",
-        &bounds.x1,&bounds.y1,&bounds.x2,&bounds.y2);
+      count=(ssize_t) sscanf(command,"BoundingBox: %lf %lf %lf %lf",&bounds.x1,
+        &bounds.y1,&bounds.x2,&bounds.y2);
     if (LocaleNCompare(DocumentMedia,command,strlen(DocumentMedia)) == 0)
-      count=(ssize_t) sscanf(command,"DocumentMedia: %*s %lf %lf",
-        &bounds.x2,&bounds.y2)+2;
+      count=(ssize_t) sscanf(command,"DocumentMedia: %*s %lf %lf",&bounds.x2,
+        &bounds.y2)+2;
     if (LocaleNCompare(HiResBoundingBox,command,strlen(HiResBoundingBox)) == 0)
       count=(ssize_t) sscanf(command,"HiResBoundingBox: %lf %lf %lf %lf",
         &bounds.x1,&bounds.y1,&bounds.x2,&bounds.y2);
@@ -358,12 +370,10 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
       count=(ssize_t) sscanf(command,"PageBoundingBox: %lf %lf %lf %lf",
         &bounds.x1,&bounds.y1,&bounds.x2,&bounds.y2);
     if (LocaleNCompare(PageMedia,command,strlen(PageMedia)) == 0)
-      count=(ssize_t) sscanf(command,"PageMedia: %*s %lf %lf",
-        &bounds.x2,&bounds.y2)+2;
+      count=(ssize_t) sscanf(command,"PageMedia: %*s %lf %lf",&bounds.x2,
+        &bounds.y2)+2;
     if (count != 4)
       continue;
-    if ((bounds.x1 > bounds.x2) || (bounds.y1 > bounds.y2))
-      continue;
     /*
       Set Postscript render geometry.
     */
@@ -375,7 +385,6 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
       page.height=height;
   }
   CloseBlob(image);
-  DestroyImageList(image);
   /*
     Create Ghostscript control file.
   */
@@ -384,12 +393,16 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
     {
       ThrowFileException(&image->exception,FileOpenError,"UnableToOpenFile",
         image_info->filename);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
+  (void) CopyMagickString(command,"/setpagedevice {pop} bind 1 index where {"
+    "dup wcheck {3 1 roll put} {pop def} ifelse} {def} ifelse\n",MaxTextExtent);
+  (void) write(file,command,strlen(command));
   (void) FormatMagickString(translate_geometry,MaxTextExtent,
     "%g %g translate\n",-bounds.x1,-bounds.y1);
   (void) write(file,translate_geometry,strlen(translate_geometry));
-  (void) close(file);
+  file=close(file)-1;
   /*
     Render postscript with the Ghostscript delegate.
   */
@@ -401,6 +414,7 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
     (page.width*image->x_resolution/dx_resolution+0.5);
   page.height=(unsigned long)
     (page.height*image->y_resolution/dy_resolution+0.5);
+  image=DestroyImageList(image);
   (void) FormatMagickString(geometry,MaxTextExtent,"%lux%lu",
     page.width,page.height);
   if (image_info->monochrome != MagickFalse)
@@ -417,6 +431,7 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
     }
   *options='\0';
   read_info=CloneImageInfo(image_info);
+  *read_info->magick='\0';
   if (read_info->number_scenes != 0)
     {
       if (read_info->number_scenes != 1)
@@ -426,6 +441,9 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
         (void) FormatMagickString(options,MaxTextExtent,
           "-dFirstPage=%lu -dLastPage=%lu",read_info->scene+1,
           read_info->scene+read_info->number_scenes);
+      read_info->number_scenes=0;
+      if (read_info->scenes != (char *) NULL)
+        *read_info->scenes='\0';
     }
   (void) CopyMagickString(filename,read_info->filename,MaxTextExtent);
   (void) AcquireUniqueFilename(read_info->filename);
@@ -434,9 +452,7 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
     read_info->antialias != MagickFalse ? 4 : 1,
     read_info->antialias != MagickFalse ? 4 : 1,geometry,density,options,
     read_info->filename,postscript_filename,image_info->filename);
-  (void) MagickMonitor(RenderPostscriptText,0,8,&image->exception);
   status=InvokePostscriptDelegate(read_info->verbose,command);
-  *read_info->magick='\0';
   image=ReadImage(read_info,exception);
   if (image == (Image *) NULL)
     {
@@ -446,7 +462,6 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
       image=ReadImage(read_info,exception);
     }
   (void) RelinquishUniqueFileResource(postscript_filename);
-  (void) MagickMonitor(RenderPostscriptText,7,8,&image->exception);
   (void) RelinquishUniqueFileResource(read_info->filename);
   if (image == (Image *) NULL)
     {
@@ -460,7 +475,7 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
         return(GetFirstImageInList(image));
       ThrowReaderException(DelegateError,"PostscriptDelegateFailed");
     }
-  DestroyImageInfo(read_info);
+  read_info=DestroyImageInfo(read_info);
   if (image == (Image *) NULL)
     ThrowReaderException(DelegateError,"PostscriptDelegateFailed");
   if (LocaleCompare(image->magick,"BMP") == 0)
@@ -471,7 +486,7 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
       cmyk_image=ConsolidateCMYKImages(image,&image->exception);
       if (cmyk_image != (Image *) NULL)
         {
-          DestroyImageList(image);
+          image=DestroyImageList(image);
           image=cmyk_image;
         }
     }
@@ -486,7 +501,7 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
       image=next_image;
   } while (next_image != (Image *) NULL);
   if (iptc_profile != (StringInfo *) NULL)
-    DestroyStringInfo(iptc_profile);
+    iptc_profile=DestroyStringInfo(iptc_profile);
   return(GetFirstImageInList(image));
 }
 
@@ -524,6 +539,7 @@ ModuleExport void RegisterPSImage(void)
   entry->magick=(MagickHandler *) IsPS;
   entry->adjoin=MagickFalse;
   entry->blob_support=MagickFalse;
+  entry->seekable_stream=MagickTrue;
   entry->description=
     AcquireString("Encapsulated PostScript Interchange format");
   entry->module=AcquireString("PS");
@@ -534,6 +550,7 @@ ModuleExport void RegisterPSImage(void)
   entry->magick=(MagickHandler *) IsPS;
   entry->adjoin=MagickFalse;
   entry->blob_support=MagickFalse;
+  entry->seekable_stream=MagickTrue;
   entry->description=AcquireString("Encapsulated PostScript");
   entry->module=AcquireString("PS");
   (void) RegisterMagickInfo(entry);
@@ -543,6 +560,7 @@ ModuleExport void RegisterPSImage(void)
   entry->magick=(MagickHandler *) IsPS;
   entry->adjoin=MagickFalse;
   entry->blob_support=MagickFalse;
+  entry->seekable_stream=MagickTrue;
   entry->description=AcquireString("Encapsulated PostScript");
   entry->module=AcquireString("PS");
   (void) RegisterMagickInfo(entry);
@@ -552,6 +570,7 @@ ModuleExport void RegisterPSImage(void)
   entry->magick=(MagickHandler *) IsPS;
   entry->adjoin=MagickFalse;
   entry->blob_support=MagickFalse;
+  entry->seekable_stream=MagickTrue;
   entry->description=
     AcquireString("Encapsulated PostScript Interchange format");
   entry->module=AcquireString("PS");
@@ -562,6 +581,7 @@ ModuleExport void RegisterPSImage(void)
   entry->magick=(MagickHandler *) IsPS;
   entry->module=AcquireString("PS");
   entry->blob_support=MagickFalse;
+  entry->seekable_stream=MagickTrue;
   entry->description=AcquireString("PostScript");
   (void) RegisterMagickInfo(entry);
 }
@@ -628,11 +648,11 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
 { \
   if ((image->matte != MagickFalse) && (p->opacity == TransparentOpacity)) \
     (void) FormatMagickString(buffer,MaxTextExtent,"ffffff%02lX", \
-      Min(length,0xff)); \
+      (unsigned long) Min(length,0xff)); \
   else \
     (void) FormatMagickString(buffer,MaxTextExtent,"%02X%02X%02X%02lX", \
       ScaleQuantumToChar(pixel.red),ScaleQuantumToChar(pixel.green), \
-      ScaleQuantumToChar(pixel.blue),Min(length,0xff)); \
+      ScaleQuantumToChar(pixel.blue),(unsigned long) Min(length,0xff)); \
   (void) WriteBlobString(image,buffer); \
 }
 
@@ -982,21 +1002,40 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
+  (void) ResetMagickMemory(&bounds,0,sizeof(bounds));
   page=1;
   scene=0;
   do
   {
     /*
-      Scale image to size of Postscript page.
+      Scale relative to dots-per-inch.
     */
     (void) SetImageColorspace(image,RGBColorspace);
-    text_size=0;
-    attribute=GetImageAttribute(image,"label");
-    if (attribute != (const ImageAttribute *) NULL)
-      text_size=(unsigned long)
-        (MultilineCensus(attribute->value)*image_info->pointsize+12);
+    dx_resolution=DefaultResolution;
+    dy_resolution=DefaultResolution;
+    flags=ParseGeometry(PSDensityGeometry,&geometry_info);
+    x_resolution=geometry_info.rho;
+    y_resolution=geometry_info.sigma;
+    if ((flags & SigmaValue) == 0)
+      y_resolution=x_resolution;
+    if (image->x_resolution != 0.0)
+      x_resolution=image->x_resolution;
+    if (image->y_resolution != 0.0)
+      y_resolution=image->y_resolution;
+    if (image_info->density != (char *) NULL)
+      {
+        flags=ParseGeometry(image_info->density,&geometry_info);
+        x_resolution=geometry_info.rho;
+        y_resolution=geometry_info.sigma;
+        if ((flags & SigmaValue) == 0)
+          y_resolution=x_resolution;
+      }
+    if (image->units == PixelsPerCentimeterResolution)
+      {
+        x_resolution*=2.54;
+        y_resolution*=2.54;
+      }
     SetGeometry(image,&geometry);
-    geometry.y=(long) text_size;
     (void) FormatMagickString(page_geometry,MaxTextExtent,"%lux%lu",
       image->columns,image->rows);
     if (image_info->page != (char *) NULL)
@@ -1011,38 +1050,22 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
           (void) strcpy(page_geometry,PSPageGeometry);
     (void) ParseMetaGeometry(page_geometry,&geometry.x,&geometry.y,
       &geometry.width,&geometry.height);
-    (void) ParseAbsoluteGeometry(page_geometry,&media_info);
-    (void) ParseGravityGeometry(image,page_geometry,&page_info);
-    geometry.x=(-page_info.x);
-    geometry.y=(long) (media_info.height+page_info.y-image->rows);
-    /*
-      Scale relative to dots-per-inch.
-    */
-    dx_resolution=72.0;
-    dy_resolution=72.0;
-    x_resolution=72.0;
-    flags=ParseGeometry(PSDensityGeometry,&geometry_info);
-    x_resolution=geometry_info.rho;
-    y_resolution=geometry_info.sigma;
-    if ((flags & SigmaValue) == 0)
-      y_resolution=x_resolution;
-    if (image_info->density != (char *) NULL)
-      {
-        flags=ParseGeometry(image_info->density,&geometry_info);
-        x_resolution=geometry_info.rho;
-        y_resolution=geometry_info.sigma;
-        if ((flags & SigmaValue) == 0)
-          y_resolution=x_resolution;
-      }
-    if (image->units == PixelsPerCentimeterResolution)
-      {
-        x_resolution*=2.54;
-        y_resolution*=2.54;
-      }
     x_scale=(double) (geometry.width*dx_resolution)/x_resolution;
     geometry.width=(unsigned long) (x_scale+0.5);
     y_scale=(double) (geometry.height*dy_resolution)/y_resolution;
     geometry.height=(unsigned long) (y_scale+0.5);
+    (void) ParseAbsoluteGeometry(page_geometry,&media_info);
+    (void) ParseGravityGeometry(image,page_geometry,&page_info);
+    if (image->gravity != UndefinedGravity)
+      {
+        geometry.x=(-page_info.x);
+        geometry.y=(long) (media_info.height+page_info.y-image->rows);
+      }
+    text_size=0;
+    attribute=GetImageAttribute(image,"label");
+    if (attribute != (const ImageAttribute *) NULL)
+      text_size=(unsigned long)
+        (MultilineCensus(attribute->value)*image_info->pointsize+12);
     count=0;
     if (page == 1)
       {
@@ -1069,7 +1092,8 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
         bounds.y1=(double) geometry.y;
         bounds.x2=(double) geometry.x+x_scale;
         bounds.y2=(double) geometry.y+(geometry.height+text_size);
-        if ((image_info->adjoin != MagickFalse) && (image->next != (Image *) NULL))
+        if ((image_info->adjoin != MagickFalse) &&
+            (image->next != (Image *) NULL))
           (void) strcpy(buffer,"%%%%BoundingBox: (atend)\n");
         else
           {
@@ -1158,7 +1182,7 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
                 if (bit == 8)
                   {
                     (void) FormatMagickString(buffer,MaxTextExtent,"%02X",
-                      byte & 0xff);
+                      (unsigned int) (byte & 0xff));
                     (void) WriteBlobString(image,buffer);
                     count++;
                     if (count == 36)
@@ -1174,7 +1198,7 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
                 {
                   byte<<=(8-bit);
                   (void) FormatMagickString(buffer,MaxTextExtent,"%02X",
-                    byte & 0xff);
+                    (unsigned int) (byte & 0xff));
                   (void) WriteBlobString(image,buffer);
                   count++;
                   if (count == 36)
@@ -1185,7 +1209,7 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
                 };
             }
             (void) WriteBlobString(image,"\n%%EndPreview\n");
-            DestroyImage(preview_image);
+            preview_image=DestroyImage(preview_image);
           }
         /*
           Output Postscript commands.
@@ -1296,10 +1320,11 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
                 p++;
               }
               if (image->previous == (Image *) NULL)
-                if (QuantumTick(y,image->rows) != MagickFalse)
+                if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                    (QuantumTick(y,image->rows) != MagickFalse))
                   {
-                    status=MagickMonitor(SaveImageTag,y,image->rows,
-                      &image->exception);
+                    status=image->progress_monitor(SaveImageTag,y,image->rows,
+                      image->client_data);
                     if (status == MagickFalse)
                       break;
                   }
@@ -1339,7 +1364,7 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
                 if (bit == 8)
                   {
                     (void) FormatMagickString(buffer,MaxTextExtent,"%02X",
-                      byte & 0xff);
+                      (unsigned int) (byte & 0xff));
                     (void) WriteBlobString(image,buffer);
                     count++;
                     if (count == 36)
@@ -1356,7 +1381,7 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
                 {
                   byte<<=(8-bit);
                   (void) FormatMagickString(buffer,MaxTextExtent,"%02X",
-                    byte & 0xff);
+                    (unsigned int) (byte & 0xff));
                   (void) WriteBlobString(image,buffer);
                   count++;
                   if (count == 36)
@@ -1366,10 +1391,11 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
                     };
                 };
               if (image->previous == (Image *) NULL)
-                if (QuantumTick(y,image->rows) != MagickFalse)
+                if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                    (QuantumTick(y,image->rows) != MagickFalse))
                   {
-                    status=MagickMonitor(SaveImageTag,y,image->rows,
-                      &image->exception);
+                    status=image->progress_monitor(SaveImageTag,y,image->rows,
+                      image->client_data);
                     if (status == MagickFalse)
                       break;
                   }
@@ -1428,13 +1454,14 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
                 }
                 WriteRunlengthPacket(image,pixel,length,p);
                 if (image->previous == (Image *) NULL)
-                  if (QuantumTick(y,image->rows) != MagickFalse)
-                    {
-                      status=MagickMonitor(SaveImageTag,y,image->rows,
-                        &image->exception);
-                      if (status == MagickFalse)
-                        break;
-                    }
+                if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                    (QuantumTick(y,image->rows) != MagickFalse))
+                  {
+                    status=image->progress_monitor(SaveImageTag,y,image->rows,
+                      image->client_data);
+                    if (status == MagickFalse)
+                      break;
+                  }
               }
               break;
             }
@@ -1470,13 +1497,14 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
                   p++;
                 }
                 if (image->previous == (Image *) NULL)
-                  if (QuantumTick(y,image->rows) != MagickFalse)
-                    {
-                      status=MagickMonitor(SaveImageTag,y,image->rows,
-                        &image->exception);
-                      if (status == MagickFalse)
-                        break;
-                    }
+                if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                    (QuantumTick(y,image->rows) != MagickFalse))
+                  {
+                    status=image->progress_monitor(SaveImageTag,y,image->rows,
+                      image->client_data);
+                    if (status == MagickFalse)
+                      break;
+                  }
               }
               break;
             }
@@ -1533,7 +1561,7 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
                       if (x > 0)
                         {
                           (void) FormatMagickString(buffer,MaxTextExtent,
-                            "%02X%02lX",index,Min(length,0xff));
+                            "%02X%02lX",index,(unsigned long) Min(length,0xff));
                           (void) WriteBlobString(image,buffer);
                           i++;
                           if (i == 18)
@@ -1549,16 +1577,17 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
                   p++;
                 }
                 (void) FormatMagickString(buffer,MaxTextExtent,"%02X%02lX",
-                  index,Min(length,0xff));
+                  index,(unsigned long) Min(length,0xff));
                 (void) WriteBlobString(image,buffer);
                 if (image->previous == (Image *) NULL)
-                  if (QuantumTick(y,image->rows) != MagickFalse)
-                    {
-                      status=MagickMonitor(SaveImageTag,y,image->rows,
-                        &image->exception);
-                      if (status == MagickFalse)
-                        break;
-                    }
+                if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                    (QuantumTick(y,image->rows) != MagickFalse))
+                  {
+                    status=image->progress_monitor(SaveImageTag,y,image->rows,
+                      image->client_data);
+                    if (status == MagickFalse)
+                      break;
+                  }
               }
               break;
             }
@@ -1590,13 +1619,14 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
                   p++;
                 }
                 if (image->previous == (Image *) NULL)
-                  if (QuantumTick(y,image->rows) != MagickFalse)
-                    {
-                      status=MagickMonitor(SaveImageTag,y,image->rows,
-                        &image->exception);
-                      if (status == MagickFalse)
-                        break;
-                    }
+                if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                    (QuantumTick(y,image->rows) != MagickFalse))
+                  {
+                    status=image->progress_monitor(SaveImageTag,y,image->rows,
+                      image->client_data);
+                    if (status == MagickFalse)
+                      break;
+                  }
               }
               break;
             }
@@ -1609,10 +1639,14 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
     if (image->next == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
-      &image->exception);
-    if (status == MagickFalse)
-      break;
+    if (image->progress_monitor != (MagickProgressMonitor) NULL)
+      {
+        status=image->progress_monitor(SaveImagesTag,scene,
+          GetImageListLength(image),image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
+    scene++;
   } while (image_info->adjoin != MagickFalse);
   (void) WriteBlobString(image,"%%Trailer\n");
   if (page > 2)
