@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -42,13 +42,16 @@
 #include "magick/studio.h"
 #include "magick/attribute.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
 #include "magick/color.h"
 #include "magick/color_private.h"
 #include "magick/compress.h"
 #include "magick/constitute.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/geometry.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
@@ -425,49 +428,25 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
     default:
       break;
   }
+  (void) ResetMagickMemory(&bounds,0,sizeof(bounds));
   page=1;
   scene=0;
   do
   {
-    /*
-      Scale image to size of Postscript page.
-    */
-    text_size=0;
-    attribute=GetImageAttribute(image,"label");
-    if (attribute != (const ImageAttribute *) NULL)
-      text_size=(unsigned long)
-        (MultilineCensus(attribute->value)*image_info->pointsize+12);
-    SetGeometry(image,&geometry);
-    geometry.y=(long) text_size;
-    (void) FormatMagickString(page_geometry,MaxTextExtent,"%lux%lu",
-      image->columns,image->rows);
-    if (image_info->page != (char *) NULL)
-      (void) CopyMagickString(page_geometry,image_info->page,MaxTextExtent);
-    else
-      if ((image->page.width != 0) && (image->page.height != 0))
-          (void) FormatMagickString(page_geometry,MaxTextExtent,
-          "%lux%lu%+ld%+ld",image->page.width,image->page.height,
-           image->page.x,image->page.y);
-      else
-        if (LocaleCompare(image_info->magick,"PS2") == 0)
-          (void) strcpy(page_geometry,PSPageGeometry);
-    (void) ParseMetaGeometry(page_geometry,&geometry.x,&geometry.y,
-       &geometry.width,&geometry.height);
-    (void) ParseAbsoluteGeometry(page_geometry,&media_info);
-    (void) ParseGravityGeometry(image,page_geometry,&page_info);
-    geometry.x=(-page_info.x);
-    geometry.y=(long) (media_info.height+page_info.y-image->rows);
     /*
       Scale relative to dots-per-inch.
     */
-    dx_resolution=72.0;
-    dy_resolution=72.0;
-    x_resolution=72.0;
+    dx_resolution=DefaultResolution;
+    dy_resolution=DefaultResolution;
     flags=ParseGeometry(PSDensityGeometry,&geometry_info);
     x_resolution=geometry_info.rho;
     y_resolution=geometry_info.sigma;
     if ((flags & SigmaValue) == 0)
       y_resolution=x_resolution;
+    if (image->x_resolution != 0.0)
+      x_resolution=image->x_resolution;
+    if (image->y_resolution != 0.0)
+      y_resolution=image->y_resolution;
     if (image_info->density != (char *) NULL)
       {
         flags=ParseGeometry(image_info->density,&geometry_info);
@@ -481,10 +460,37 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
         x_resolution*=2.54;
         y_resolution*=2.54;
       }
-    x_scale=(double) ((geometry.width*dx_resolution)/x_resolution);
+    SetGeometry(image,&geometry);
+    (void) FormatMagickString(page_geometry,MaxTextExtent,"%lux%lu",
+      image->columns,image->rows);
+    if (image_info->page != (char *) NULL)
+      (void) CopyMagickString(page_geometry,image_info->page,MaxTextExtent);
+    else
+      if ((image->page.width != 0) && (image->page.height != 0))
+        (void) FormatMagickString(page_geometry,MaxTextExtent,"%lux%lu%+ld%+ld",
+          image->page.width,image->page.height,image->page.x,image->page.y);
+      else
+        if ((image->gravity != UndefinedGravity) &&
+            (LocaleCompare(image_info->magick,"PS") == 0))
+          (void) strcpy(page_geometry,PSPageGeometry);
+    (void) ParseMetaGeometry(page_geometry,&geometry.x,&geometry.y,
+      &geometry.width,&geometry.height);
+    x_scale=(double) (geometry.width*dx_resolution)/x_resolution;
     geometry.width=(unsigned long) (x_scale+0.5);
-    y_scale=(double) ((geometry.height*dy_resolution)/y_resolution);
+    y_scale=(double) (geometry.height*dy_resolution)/y_resolution;
     geometry.height=(unsigned long) (y_scale+0.5);
+    (void) ParseAbsoluteGeometry(page_geometry,&media_info);
+    (void) ParseGravityGeometry(image,page_geometry,&page_info);
+    if (image->gravity != UndefinedGravity)
+      {
+        geometry.x=(-page_info.x);
+        geometry.y=(long) (media_info.height+page_info.y-image->rows);
+      }
+    text_size=0;
+    attribute=GetImageAttribute(image,"label");
+    if (attribute != (const ImageAttribute *) NULL)
+      text_size=(unsigned long)
+        (MultilineCensus(attribute->value)*image_info->pointsize+12);
     if (page == 1)
       {
         /*
@@ -510,7 +516,8 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
         bounds.y1=(double) geometry.y;
         bounds.x2=(double) geometry.x+geometry.width;
         bounds.y2=(double) geometry.y+geometry.height+text_size;
-        if ((image_info->adjoin != MagickFalse) && (image->next != (Image *) NULL))
+        if ((image_info->adjoin != MagickFalse) &&
+            (image->next != (Image *) NULL))
           (void) strcpy(buffer,"%%BoundingBox: (atend)\n");
         else
           {
@@ -569,6 +576,7 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
               break;
             }
             case FaxCompression:
+            case Group4Compression:
             {
               (void) FormatMagickString(buffer,MaxTextExtent,*q,
                 "ASCII85Decode");
@@ -656,7 +664,7 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
     number_pixels=(MagickSizeType) image->columns*image->rows;
     if (number_pixels != (MagickSizeType) ((size_t) number_pixels))
       ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
-    if ((compression == FaxCompression) ||
+    if ((compression == FaxCompression) || (compression == Group4Compression) ||
         ((image_info->type != TrueColorType) &&
          (IsGrayImage(image,&image->exception) != MagickFalse)))
       {
@@ -665,15 +673,18 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
           (image->colorspace == CMYKColorspace));
         (void) WriteBlobString(image,buffer);
         (void) FormatMagickString(buffer,MaxTextExtent,"%d\n",
-          (int) (compression != FaxCompression));
+          (int) ((compression != FaxCompression) &&
+           (compression != Group4Compression)));
         (void) WriteBlobString(image,buffer);
         (void) WriteBlobString(image,"0\n");
         (void) FormatMagickString(buffer,MaxTextExtent,"%d\n",
-           compression == FaxCompression ? 1 : 8);
+           (compression == FaxCompression) ||
+           (compression == Group4Compression) ? 1 : 8);
         (void) WriteBlobString(image,buffer);
         switch (compression)
         {
           case FaxCompression:
+          case Group4Compression:
           {
             if (LocaleCompare(CCITTParam,"0") == 0)
               {
@@ -721,10 +732,11 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
                 p++;
               }
               if (image->previous == (Image *) NULL)
-                if (QuantumTick(y,image->rows) != MagickFalse)
+                if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                    (QuantumTick(y,image->rows) != MagickFalse))
                   {
-                    status=MagickMonitor(SaveImageTag,y,image->rows,
-                      &image->exception);
+                    status=image->progress_monitor(SaveImageTag,y,image->rows,
+                      image->client_data);
                     if (status == MagickFalse)
                       break;
                   }
@@ -760,10 +772,11 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
                 p++;
               }
               if (image->previous == (Image *) NULL)
-                if (QuantumTick(y,image->rows) != MagickFalse)
+                if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                    (QuantumTick(y,image->rows) != MagickFalse))
                   {
-                    status=MagickMonitor(SaveImageTag,y,image->rows,
-                      &image->exception);
+                    status=image->progress_monitor(SaveImageTag,y,image->rows,
+                      image->client_data);
                     if (status == MagickFalse)
                       break;
                   }
@@ -845,13 +858,14 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
                   p++;
                 }
                 if (image->previous == (Image *) NULL)
-                  if (QuantumTick(y,image->rows) != MagickFalse)
-                    {
-                      status=MagickMonitor(SaveImageTag,y,image->rows,
-                        &image->exception);
-                      if (status == MagickFalse)
-                        break;
-                    }
+                if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                    (QuantumTick(y,image->rows) != MagickFalse))
+                  {
+                    status=image->progress_monitor(SaveImageTag,y,image->rows,
+                      image->client_data);
+                    if (status == MagickFalse)
+                      break;
+                  }
               }
               if (compression == LZWCompression)
                 status=LZWEncodeImage(image,length,pixels);
@@ -904,13 +918,14 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
                   p++;
                 }
                 if (image->previous == (Image *) NULL)
-                  if (QuantumTick(y,image->rows) != MagickFalse)
-                    {
-                      status=MagickMonitor(SaveImageTag,y,image->rows,
-                        &image->exception);
-                      if (status == MagickFalse)
-                        break;
-                    }
+                if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                    (QuantumTick(y,image->rows) != MagickFalse))
+                  {
+                    status=image->progress_monitor(SaveImageTag,y,image->rows,
+                      image->client_data);
+                    if (status == MagickFalse)
+                      break;
+                  }
               }
               Ascii85Flush(image);
               break;
@@ -969,13 +984,14 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
                 for (x=0; x < (long) image->columns; x++)
                   *q++=(unsigned char) indexes[x];
                 if (image->previous == (Image *) NULL)
-                  if (QuantumTick(y,image->rows) != MagickFalse)
-                    {
-                      status=MagickMonitor(SaveImageTag,y,image->rows,
-                        &image->exception);
-                      if (status == MagickFalse)
-                        break;
-                    }
+                if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                    (QuantumTick(y,image->rows) != MagickFalse))
+                  {
+                    status=image->progress_monitor(SaveImageTag,y,image->rows,
+                      image->client_data);
+                    if (status == MagickFalse)
+                      break;
+                  }
               }
               if (compression == LZWCompression)
                 status=LZWEncodeImage(image,length,pixels);
@@ -1005,13 +1021,14 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
                 for (x=0; x < (long) image->columns; x++)
                   Ascii85Encode(image,(unsigned char) indexes[x]);
                 if (image->previous == (Image *) NULL)
-                  if (QuantumTick(y,image->rows) != MagickFalse)
-                    {
-                      status=MagickMonitor(SaveImageTag,y,image->rows,
-                        &image->exception);
-                      if (status == MagickFalse)
-                        break;
-                    }
+                if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                    (QuantumTick(y,image->rows) != MagickFalse))
+                  {
+                    status=image->progress_monitor(SaveImageTag,y,image->rows,
+                      image->client_data);
+                    if (status == MagickFalse)
+                      break;
+                  }
               }
               Ascii85Flush(image);
               break;
@@ -1034,10 +1051,14 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
     if (image->next == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
-      &image->exception);
-    if (status == MagickFalse)
-      break;
+    if (image->progress_monitor != (MagickProgressMonitor) NULL)
+      {
+        status=image->progress_monitor(SaveImagesTag,scene,
+          GetImageListLength(image),image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
+    scene++;
   } while (image_info->adjoin != MagickFalse);
   (void) WriteBlobString(image,"%%Trailer\n");
   if (page > 1)
