@@ -18,7 +18,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -43,13 +43,16 @@
 #include "magick/studio.h"
 #include "magick/attribute.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
 #include "magick/color.h"
 #include "magick/color_private.h"
 #include "magick/compress.h"
 #include "magick/constitute.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/geometry.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
@@ -238,9 +241,11 @@ static MagickBooleanType SerializeImage(const ImageInfo *image_info,
         p++;
       }
     if (image->previous == (Image *) NULL)
-      if (QuantumTick(y,image->rows) != MagickFalse)
+      if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+          (QuantumTick(y,image->rows) != MagickFalse))
         {
-          status=MagickMonitor(SaveImageTag,y,image->rows,&image->exception);
+          status=image->progress_monitor(SaveImageTag,y,image->rows,
+            image->client_data);
           if (status == MagickFalse)
             break;
         }
@@ -319,10 +324,11 @@ static MagickBooleanType SerializeImageChannel(const ImageInfo *image_info,
         }
       }
     if (image->previous == (Image *) NULL)
-      if (QuantumTick(y,image->rows) != MagickFalse)
+      if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+          (QuantumTick(y,image->rows) != MagickFalse))
         {
-          status=MagickMonitor(SaveImageTag,y,image->rows,
-            &image->exception);
+          status=image->progress_monitor(SaveImageTag,y,image->rows,
+            image->client_data);
           if (status == MagickFalse)
             break;
         }
@@ -372,19 +378,22 @@ static MagickBooleanType SerializeImageIndexes(const ImageInfo *image_info,
     for (x=0; x < (long) image->columns; x++)
       *q++=(unsigned char) indexes[x];
     if (image->previous == (Image *) NULL)
-      if (QuantumTick(y,image->rows) != MagickFalse)
-        {
-          status=MagickMonitor(SaveImageTag,y,image->rows,&image->exception);
-          if (status == MagickFalse)
-            break;
-        }
+     if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+         (QuantumTick(y,image->rows) != MagickFalse))
+       {
+         status=image->progress_monitor(SaveImageTag,y,image->rows,
+           image->client_data);
+         if (status == MagickFalse)
+           break;
+       }
   }
   if (status == MagickFalse)
     *pixels=(unsigned char *) RelinquishMagickMemory(*pixels);
   return(status);
 }
 
-static MagickBooleanType WritePS3MaskImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WritePS3MaskImage(const ImageInfo *image_info,
+  Image *image)
 {
   char
     buffer[MaxTextExtent];
@@ -411,6 +420,9 @@ static MagickBooleanType WritePS3MaskImage(const ImageInfo *image_info,Image *im
   unsigned char
     *pixels;
 
+  unsigned long
+    quality;
+
   assert(image_info != (ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
@@ -420,6 +432,8 @@ static MagickBooleanType WritePS3MaskImage(const ImageInfo *image_info,Image *im
   assert(image->matte != MagickFalse);
   status=MagickTrue;
   compression=image->compression;
+  quality=(unsigned long)
+    (image->quality == UndefinedCompressionQuality ? 75 : image->quality);
   /*
     Note BeginData DSC comment for update later.
   */
@@ -441,6 +455,7 @@ static MagickBooleanType WritePS3MaskImage(const ImageInfo *image_info,Image *im
         image->columns,image->rows);
     }
     case FaxCompression:
+    case Group4Compression:
     default:
     {
       (void) FormatMagickString(buffer,MaxTextExtent,
@@ -485,7 +500,7 @@ static MagickBooleanType WritePS3MaskImage(const ImageInfo *image_info,Image *im
   status=SeparateImageChannel(mask_image,OpacityChannel);
   if (status == MagickFalse)
     {
-      DestroyImage(mask_image);
+      mask_image=DestroyImage(mask_image);
       return(MagickFalse);
     }
   (void) SetImageType(mask_image,BilevelType);
@@ -507,6 +522,7 @@ static MagickBooleanType WritePS3MaskImage(const ImageInfo *image_info,Image *im
       break;
     }
     case FaxCompression:
+    case Group4Compression:
     default:
     {
       if (LocaleCompare(CCITTParam,"0") == 0)
@@ -543,13 +559,12 @@ static MagickBooleanType WritePS3MaskImage(const ImageInfo *image_info,Image *im
       status=SerializeImageChannel(image_info,mask_image,&pixels,&length);
       if (status == MagickFalse)
         break;
-      status=ZLIBEncodeImage(image,length,image_info->quality == 0UL ?
-        75UL : image_info->quality,pixels);
+      status=ZLIBEncodeImage(image,length,quality,pixels);
       pixels=(unsigned char *) RelinquishMagickMemory(pixels);
       break;
     }
   }
-  DestroyImage(mask_image);
+  mask_image=DestroyImage(mask_image);
   (void) WriteBlobByte(image,'\n');
   length=(size_t) (TellBlob(image)-stop);
   stop=TellBlob(image);
@@ -862,48 +877,25 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
     default:
       break;
   }
+  (void) ResetMagickMemory(&bounds,0,sizeof(bounds));
   page=0;
   scene=0;
   do
   {
-    /*
-      Scale image to size of Postscript page.
-    */
-    page++;
-    text_size=0;
-    attribute=GetImageAttribute(image,"label");
-    if (attribute != (const ImageAttribute *) NULL)
-      text_size=(unsigned long) (MultilineCensus(attribute->value)*
-        image_info->pointsize+12);
-    SetGeometry(image,&geometry);
-    geometry.y=(long) text_size;
-    (void) FormatMagickString(page_geometry,MaxTextExtent,"%lux%lu",
-      image->columns,image->rows);
-    if (image_info->page != (char *) NULL)
-      (void) CopyMagickString(page_geometry,image_info->page,MaxTextExtent);
-    else
-      if ((image->page.width != 0) && (image->page.height != 0))
-        (void) FormatMagickString(page_geometry,MaxTextExtent,"%lux%lu%+ld%+ld",          image->page.width,image->page.height,image->page.x,image->page.y);
-      else
-        if (LocaleCompare(image_info->magick,"PS3") == 0)
-          (void) strcpy(page_geometry,PSPageGeometry);
-    (void) ParseMetaGeometry(page_geometry,&geometry.x,&geometry.y,
-      &geometry.width,&geometry.height);
-    (void) ParseAbsoluteGeometry(page_geometry,&media_info);
-    (void) ParseGravityGeometry(image,page_geometry,&page_info);
-    geometry.x=(-page_info.x);
-    geometry.y=(long) (media_info.height+page_info.y-image->rows);
     /*
       Scale relative to dots-per-inch.
     */
-    dx_resolution=72.0;
-    dy_resolution=72.0;
-    x_resolution=72.0;
+    dx_resolution=DefaultResolution;
+    dy_resolution=DefaultResolution;
     flags=ParseGeometry(PSDensityGeometry,&geometry_info);
     x_resolution=geometry_info.rho;
     y_resolution=geometry_info.sigma;
     if ((flags & SigmaValue) == 0)
       y_resolution=x_resolution;
+    if (image->x_resolution != 0.0)
+      x_resolution=image->x_resolution;
+    if (image->y_resolution != 0.0)
+      y_resolution=image->y_resolution;
     if (image_info->density != (char *) NULL)
       {
         flags=ParseGeometry(image_info->density,&geometry_info);
@@ -917,10 +909,38 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
         x_resolution*=2.54;
         y_resolution*=2.54;
       }
-    x_scale=(double) ((geometry.width*dx_resolution)/x_resolution);
+    SetGeometry(image,&geometry);
+    (void) FormatMagickString(page_geometry,MaxTextExtent,"%lux%lu",
+      image->columns,image->rows);
+    if (image_info->page != (char *) NULL)
+      (void) CopyMagickString(page_geometry,image_info->page,MaxTextExtent);
+    else
+      if ((image->page.width != 0) && (image->page.height != 0))
+        (void) FormatMagickString(page_geometry,MaxTextExtent,"%lux%lu%+ld%+ld",
+          image->page.width,image->page.height,image->page.x,image->page.y);
+      else
+        if ((image->gravity != UndefinedGravity) &&
+            (LocaleCompare(image_info->magick,"PS") == 0))
+          (void) strcpy(page_geometry,PSPageGeometry);
+    (void) ParseMetaGeometry(page_geometry,&geometry.x,&geometry.y,
+      &geometry.width,&geometry.height);
+    x_scale=(double) (geometry.width*dx_resolution)/x_resolution;
     geometry.width=(unsigned long) (x_scale+0.5);
-    y_scale=(double) ((geometry.height*dy_resolution)/y_resolution);
+    y_scale=(double) (geometry.height*dy_resolution)/y_resolution;
     geometry.height=(unsigned long) (y_scale+0.5);
+    (void) ParseAbsoluteGeometry(page_geometry,&media_info);
+    (void) ParseGravityGeometry(image,page_geometry,&page_info);
+    if (image->gravity != UndefinedGravity)
+      {
+        geometry.x=(-page_info.x);
+        geometry.y=(long) (media_info.height+page_info.y-image->rows);
+      }
+    text_size=0;
+    attribute=GetImageAttribute(image,"label");
+    if (attribute != (const ImageAttribute *) NULL)
+      text_size=(unsigned long)
+        (MultilineCensus(attribute->value)*image_info->pointsize+12);
+    page++;
     if (page == 1)
       {
         /*
@@ -1152,7 +1172,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
         (void) WriteBlobString(image,"true\n");
       else
         (void) WriteBlobString(image,"false\n");
-    if (compression == FaxCompression)
+    if ((compression == FaxCompression) || (compression == Group4Compression))
       (void) SetImageType(image,BilevelType);
     /*
       Showpage for non-EPS.
@@ -1204,6 +1224,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
             break;
           }
           case FaxCompression:
+          case Group4Compression:
           {
             (void) WriteBlobString(image,PS3_FaxCompression"\n");
             break;
@@ -1245,7 +1266,8 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
         if (compression == JPEGCompression)
           status=JPEGEncodeImage(image_info,image);
         else
-          if (compression == FaxCompression)
+          if ((compression == FaxCompression) ||
+              (compression == Group4Compression))
            {
              if (LocaleCompare(CCITTParam,"0") == 0)
                status=HuffmanEncodeImage(image_info,image);
@@ -1281,8 +1303,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
               }
               case ZipCompression:
               {
-                status=ZLIBEncodeImage(image,length,image_info->quality == 0UL ?
-                  75UL : image_info->quality,pixels);
+                status=ZLIBEncodeImage(image,length,image->quality,pixels);
                 break;
               }
             }
@@ -1366,8 +1387,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
                 }
                 case ZipCompression:
                 {
-                  status=ZLIBEncodeImage(image,length,image_info->quality ==
-                    0UL ? 75UL : image_info->quality,pixels);
+                  status=ZLIBEncodeImage(image,length,image->quality,pixels);
                   break;
                 }
               }
@@ -1479,8 +1499,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
               }
               case ZipCompression:
               {
-                status=ZLIBEncodeImage(image,length,image_info->quality == 0UL ?
-                  75UL : image_info->quality,pixels);
+                status=ZLIBEncodeImage(image,length,image->quality,pixels);
                 break;
               }
             }
@@ -1513,10 +1532,13 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
     if (image->next == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    status=MagickMonitor(SaveImagesTag,scene,GetImageListLength(image),
-      &image->exception);
-    if (status == MagickFalse)
-      break;
+    if (image->progress_monitor != (MagickProgressMonitor) NULL)
+      {
+        status=image->progress_monitor(SaveImagesTag,scene,
+          GetImageListLength(image),image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
     scene++;
   } while (image_info->adjoin != MagickFalse);
   (void) WriteBlobString(image,"%%Trailer\n");
