@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -41,14 +41,18 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
 #include "magick/colorspace.h"
 #include "magick/constitute.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/pixel.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 
@@ -97,9 +101,6 @@ static Image *ReadRAWImage(const ImageInfo *image_info,ExceptionInfo *exception)
   MagickBooleanType
     status;
 
-  QuantumType
-    quantum_type;
-
   register long
     i,
     x;
@@ -117,6 +118,7 @@ static Image *ReadRAWImage(const ImageInfo *image_info,ExceptionInfo *exception)
     *scanline;
 
   unsigned long
+    depth,
     width;
 
   /*
@@ -134,84 +136,33 @@ static Image *ReadRAWImage(const ImageInfo *image_info,ExceptionInfo *exception)
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
-      DestroyImageList(image);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
   switch (*image->magick)
   {
-    case 'A':
-    case 'a':
-    {
-      quantum_type=AlphaQuantum;
-      break;
-    }
-    case 'B':
-    case 'b':
-    {
-      quantum_type=BlueQuantum;
-      break;
-    }
     case 'C':
     case 'c':
-    {
-      quantum_type=CyanQuantum;
-      image->colorspace=CMYKColorspace;
-      break;
-    }
-    case 'g':
-    case 'G':
-    {
-      quantum_type=GreenQuantum;
-      break;
-    }
-    case 'I':
-    case 'i':
-    {
-      quantum_type=IndexQuantum;
-      break;
-    }
+    case 'M':
+    case 'm':
+    case 'Y':
+    case 'y':
     case 'K':
     case 'k':
     {
-      quantum_type=BlackQuantum;
       image->colorspace=CMYKColorspace;
-    }
-    case 'M':
-    case 'm':
-    {
-      quantum_type=MagentaQuantum;
-      image->colorspace=CMYKColorspace;
-    }
-    case 'o':
-    case 'O':
-    {
-      quantum_type=OpacityQuantum;
       break;
     }
-    case 'R':
-    case 'r':
-    {
-      quantum_type=RedQuantum;
-      break;
-    }
-    case 'Y':
-    case 'y':
-    {
-      quantum_type=YellowQuantum;
-      image->colorspace=CMYKColorspace;
-    }
     default:
-    {
-      quantum_type=GrayQuantum;
       break;
-    }
   }
   for (i=0; i < image->offset; i++)
     (void) ReadBlobByte(image);
   /*
     Allocate memory for a scanline.
   */
-  packet_size=(size_t) (image->depth > 8 ? 2 : 1);
+  depth=GetImageQuantumDepth(image,MagickTrue);
+  packet_size=(size_t) (depth/8);
   scanline=(unsigned char *)
     AcquireMagickMemory(packet_size*image->extract_info.width);
   if (scanline == (unsigned char *) NULL)
@@ -246,13 +197,18 @@ static Image *ReadRAWImage(const ImageInfo *image_info,ExceptionInfo *exception)
       q=SetImagePixels(image,0,y,image->columns,1);
       if (q == (PixelPacket *) NULL)
         break;
-      (void) PushImagePixels(image,GrayQuantum,scanline+x);
+      (void) ExportQuantumPixels(image,GrayQuantum,0,scanline+x);
       if (SyncImagePixels(image) == MagickFalse)
         break;
       if (image->previous == (Image *) NULL)
-        if (QuantumTick(y,image->rows) != 0)
-          if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
-            break;
+        if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+            (QuantumTick(y,image->rows) != MagickFalse))
+          {
+            status=image->progress_monitor(LoadImageTag,y,image->rows,
+              image->client_data);
+            if (status == MagickFalse)
+              break;
+          }
     }
     width=image->extract_info.height-image->rows-image->extract_info.y;
     for (j=0; j < (long) width; j++)
@@ -263,6 +219,8 @@ static Image *ReadRAWImage(const ImageInfo *image_info,ExceptionInfo *exception)
           image->filename);
         break;
       }
+    if (image->colorspace == CMYKColorspace)
+      image->storage_class=DirectClass;
     /*
       Proceed to next image.
     */
@@ -278,14 +236,17 @@ static Image *ReadRAWImage(const ImageInfo *image_info,ExceptionInfo *exception)
         AllocateNextImage(image_info,image);
         if (image->next == (Image *) NULL)
           {
-            DestroyImageList(image);
+            image=DestroyImageList(image);
             return((Image *) NULL);
           }
         image=SyncNextImageInList(image);
-        status=MagickMonitor(LoadImagesTag,TellBlob(image),GetBlobSize(image),
-          exception);
-        if (status == MagickFalse)
-          break;
+        if (image->progress_monitor != (MagickProgressMonitor) NULL)
+          {
+            status=image->progress_monitor(LoadImagesTag,TellBlob(image),
+              GetBlobSize(image),image->client_data);
+            if (status == MagickFalse)
+              break;
+          }
       }
   } while (count != 0);
   scanline=(unsigned char *) RelinquishMagickMemory(scanline);
@@ -324,6 +285,7 @@ ModuleExport void RegisterRAWImage(void)
   entry->decoder=(DecoderHandler *) ReadRAWImage;
   entry->encoder=(EncoderHandler *) WriteRAWImage;
   entry->raw=MagickTrue;
+  entry->endian_support=MagickTrue;
   entry->description=AcquireString("Raw red samples");
   entry->module=AcquireString("RAW");
   (void) RegisterMagickInfo(entry);
@@ -331,6 +293,7 @@ ModuleExport void RegisterRAWImage(void)
   entry->decoder=(DecoderHandler *) ReadRAWImage;
   entry->encoder=(EncoderHandler *) WriteRAWImage;
   entry->raw=MagickTrue;
+  entry->endian_support=MagickTrue;
   entry->description=AcquireString("Raw cyan samples");
   entry->module=AcquireString("RAW");
   (void) RegisterMagickInfo(entry);
@@ -338,6 +301,7 @@ ModuleExport void RegisterRAWImage(void)
   entry->decoder=(DecoderHandler *) ReadRAWImage;
   entry->encoder=(EncoderHandler *) WriteRAWImage;
   entry->raw=MagickTrue;
+  entry->endian_support=MagickTrue;
   entry->description=AcquireString("Raw green samples");
   entry->module=AcquireString("RAW");
   (void) RegisterMagickInfo(entry);
@@ -345,6 +309,7 @@ ModuleExport void RegisterRAWImage(void)
   entry->decoder=(DecoderHandler *) ReadRAWImage;
   entry->encoder=(EncoderHandler *) WriteRAWImage;
   entry->raw=MagickTrue;
+  entry->endian_support=MagickTrue;
   entry->description=AcquireString("Raw magenta samples");
   entry->module=AcquireString("RAW");
   (void) RegisterMagickInfo(entry);
@@ -352,6 +317,7 @@ ModuleExport void RegisterRAWImage(void)
   entry->decoder=(DecoderHandler *) ReadRAWImage;
   entry->encoder=(EncoderHandler *) WriteRAWImage;
   entry->raw=MagickTrue;
+  entry->endian_support=MagickTrue;
   entry->description=AcquireString("Raw blue samples");
   entry->module=AcquireString("RAW");
   (void) RegisterMagickInfo(entry);
@@ -359,6 +325,7 @@ ModuleExport void RegisterRAWImage(void)
   entry->decoder=(DecoderHandler *) ReadRAWImage;
   entry->encoder=(EncoderHandler *) WriteRAWImage;
   entry->raw=MagickTrue;
+  entry->endian_support=MagickTrue;
   entry->description=AcquireString("Raw yellow samples");
   entry->module=AcquireString("RAW");
   (void) RegisterMagickInfo(entry);
@@ -366,6 +333,7 @@ ModuleExport void RegisterRAWImage(void)
   entry->decoder=(DecoderHandler *) ReadRAWImage;
   entry->encoder=(EncoderHandler *) WriteRAWImage;
   entry->raw=MagickTrue;
+  entry->endian_support=MagickTrue;
   entry->description=AcquireString("Raw alpha samples");
   entry->module=AcquireString("RAW");
   (void) RegisterMagickInfo(entry);
@@ -373,6 +341,7 @@ ModuleExport void RegisterRAWImage(void)
   entry->decoder=(DecoderHandler *) ReadRAWImage;
   entry->encoder=(EncoderHandler *) WriteRAWImage;
   entry->raw=MagickTrue;
+  entry->endian_support=MagickTrue;
   entry->description=AcquireString("Raw opacity samples");
   entry->module=AcquireString("RAW");
   (void) RegisterMagickInfo(entry);
@@ -380,6 +349,7 @@ ModuleExport void RegisterRAWImage(void)
   entry->decoder=(DecoderHandler *) ReadRAWImage;
   entry->encoder=(EncoderHandler *) WriteRAWImage;
   entry->raw=MagickTrue;
+  entry->endian_support=MagickTrue;
   entry->description=AcquireString("Raw black samples");
   entry->module=AcquireString("RAW");
   (void) RegisterMagickInfo(entry);
@@ -465,6 +435,9 @@ static MagickBooleanType WriteRAWImage(const ImageInfo *image_info,Image *image)
   unsigned char
     *scanline;
 
+  unsigned long
+    depth;
+
   /*
     Open output image file.
   */
@@ -559,7 +532,8 @@ static MagickBooleanType WriteRAWImage(const ImageInfo *image_info,Image *image)
     /*
       Allocate memory for scanline.
     */
-    packet_size=(size_t) (image->depth > 8 ? 2 : 1);
+    depth=GetImageQuantumDepth(image,MagickTrue);
+    packet_size=(size_t) (depth/8);
     scanline=(unsigned char *)
       AcquireMagickMemory(packet_size*image->columns);
     if (scanline == (unsigned char *) NULL)
@@ -572,12 +546,14 @@ static MagickBooleanType WriteRAWImage(const ImageInfo *image_info,Image *image)
       p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
       if (p == (const PixelPacket *) NULL)
         break;
-      (void) PopImagePixels(image,quantum_type,scanline);
+      (void) ImportQuantumPixels(image,quantum_type,0,scanline);
       (void) WriteBlob(image,packet_size*image->columns,scanline);
       if (image->previous == (Image *) NULL)
-        if (QuantumTick(y,image->rows) != 0)
+        if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+            (QuantumTick(y,image->rows) != MagickFalse))
           {
-            status=MagickMonitor(SaveImageTag,y,image->rows,&image->exception);
+            status=image->progress_monitor(SaveImageTag,y,image->rows,
+              image->client_data);
             if (status == MagickFalse)
               break;
           }
@@ -586,10 +562,14 @@ static MagickBooleanType WriteRAWImage(const ImageInfo *image_info,Image *image)
     if (image->next == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
-      &image->exception);
-    if (status == MagickFalse)
-      break;
+    if (image->progress_monitor != (MagickProgressMonitor) NULL)
+      {
+        status=image->progress_monitor(SaveImagesTag,scene,
+          GetImageListLength(image),image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
+    scene++;
   } while (image_info->adjoin != MagickFalse);
   CloseBlob(image);
   return(MagickTrue);
