@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -41,14 +41,18 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
 #include "magick/colorspace.h"
 #include "magick/constitute.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/pixel.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 #include "magick/utility.h"
@@ -97,9 +101,11 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
   MagickBooleanType
     status;
 
+  MagickOffsetType
+    offset;
+
   register long
-    i,
-    x;
+    i;
 
   register PixelPacket
     *q;
@@ -114,6 +120,7 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
     *scanline;
 
   unsigned long
+    depth,
     width;
 
   assert(image_info != (const ImageInfo *) NULL);
@@ -133,7 +140,7 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
       status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
       if (status == MagickFalse)
         {
-          DestroyImageList(image);
+          image=DestroyImageList(image);
           return((Image *) NULL);
         }
       for (i=0; i < image->offset; i++)
@@ -142,11 +149,12 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
   /*
     Allocate memory for a scanline.
   */
-  packet_size=(size_t) (image->depth > 8 ? 6 : 3);
+  depth=GetImageQuantumDepth(image,MagickTrue);
+  packet_size=(size_t) (3*depth/8);
   if (LocaleCompare(image_info->magick,"RGBA") == 0)
     {
+      packet_size+=depth/8;
       image->matte=MagickTrue;
-      packet_size=(size_t) (image->depth > 8 ? 8 : 4);
     }
   scanline=(unsigned char *)
     AcquireMagickMemory(packet_size*image->extract_info.width);
@@ -162,7 +170,7 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
       for (y=0; y < (long) image->rows; y++)
         (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
     }
-  x=(long) (packet_size*image->extract_info.x);
+  offset=(MagickOffsetType) (packet_size*image->extract_info.x);
   do
   {
     /*
@@ -190,15 +198,20 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
           if (q == (PixelPacket *) NULL)
             break;
           if (image->matte == MagickFalse)
-            (void) PushImagePixels(image,RGBQuantum,scanline+x);
+            (void) ExportQuantumPixels(image,RGBQuantum,0,scanline+offset);
           else
-            (void) PushImagePixels(image,RGBAQuantum,scanline+x);
+            (void) ExportQuantumPixels(image,RGBAQuantum,0,scanline+offset);
           if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows) != MagickFalse)
-              if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
-                break;
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(y,image->rows) != MagickFalse))
+              {
+                status=image->progress_monitor(LoadImageTag,y,image->rows,
+                  image->client_data);
+                if (status == MagickFalse)
+                  break;
+              }
         }
         width=image->extract_info.height-image->rows-image->extract_info.y;
         for (i=0; i < (long) width; i++)
@@ -222,23 +235,28 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
           q=SetImagePixels(image,0,y,image->columns,1);
           if (q == (PixelPacket *) NULL)
             break;
-          (void) PushImagePixels(image,RedQuantum,scanline+x);
+          (void) ExportQuantumPixels(image,RedQuantum,0,scanline+offset);
           (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
-          (void) PushImagePixels(image,GreenQuantum,scanline+x);
+          (void) ExportQuantumPixels(image,GreenQuantum,0,scanline+offset);
           (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
-          (void) PushImagePixels(image,BlueQuantum,scanline+x);
+          (void) ExportQuantumPixels(image,BlueQuantum,0,scanline+offset);
           if (image->matte != MagickFalse)
             {
               (void) ReadBlob(image,packet_size*image->extract_info.width,
                 scanline);
-              (void) PushImagePixels(image,AlphaQuantum,scanline+x);
+              (void) ExportQuantumPixels(image,AlphaQuantum,0,scanline+offset);
             }
           if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows) != MagickFalse)
-              if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
-                break;
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(y,image->rows) != MagickFalse))
+              {
+                status=image->progress_monitor(LoadImageTag,y,image->rows,
+                  image->client_data);
+                if (status == MagickFalse)
+                  break;
+              }
         }
         width=image->extract_info.height-image->rows-image->extract_info.y;
         for (i=0; i < (long) width; i++)
@@ -260,7 +278,7 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
             status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
             if (status == MagickFalse)
               {
-                DestroyImageList(image);
+                image=DestroyImageList(image);
                 return((Image *) NULL);
               }
           }
@@ -277,13 +295,18 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
           q=SetImagePixels(image,0,y,image->columns,1);
           if (q == (PixelPacket *) NULL)
             break;
-          (void) PushImagePixels(image,RedQuantum,scanline+x);
+          (void) ExportQuantumPixels(image,RedQuantum,0,scanline+offset);
           if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(i,span) != MagickFalse)
-              if (MagickMonitor(LoadImageTag,i,span,&image->exception) == MagickFalse)
-                break;
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(i,span) != MagickFalse))
+              {
+                status=image->progress_monitor(LoadImageTag,i,span,
+                  image->client_data);
+                if (status == MagickFalse)
+                  break;
+              }
           i++;
         }
         width=image->extract_info.height-image->rows-image->extract_info.y;
@@ -296,7 +319,7 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
             status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
             if (status == MagickFalse)
               {
-                DestroyImageList(image);
+                image=DestroyImageList(image);
                 return((Image *) NULL);
               }
           }
@@ -308,13 +331,18 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
           q=GetImagePixels(image,0,y,image->columns,1);
           if (q == (PixelPacket *) NULL)
             break;
-          (void) PushImagePixels(image,GreenQuantum,scanline+x);
+          (void) ExportQuantumPixels(image,GreenQuantum,0,scanline+offset);
           if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(i,span) != MagickFalse)
-              if (MagickMonitor(LoadImageTag,i,span,&image->exception) == MagickFalse)
-                break;
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(i,span) != MagickFalse))
+              {
+                status=image->progress_monitor(LoadImageTag,i,span,
+                  image->client_data);
+                if (status == MagickFalse)
+                  break;
+              }
           i++;
         }
         width=image->extract_info.height-image->rows-image->extract_info.y;
@@ -327,7 +355,7 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
             status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
             if (status == MagickFalse)
               {
-                DestroyImageList(image);
+                image=DestroyImageList(image);
                 return((Image *) NULL);
               }
           }
@@ -339,13 +367,18 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
           q=GetImagePixels(image,0,y,image->columns,1);
           if (q == (PixelPacket *) NULL)
             break;
-          (void) PushImagePixels(image,BlueQuantum,scanline+x);
+          (void) ExportQuantumPixels(image,BlueQuantum,0,scanline+offset);
           if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(i,span) != MagickFalse)
-              if (MagickMonitor(LoadImageTag,i,span,&image->exception) == MagickFalse)
-                break;
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(i,span) != MagickFalse))
+              {
+                status=image->progress_monitor(LoadImageTag,i,span,
+                  image->client_data);
+                if (status == MagickFalse)
+                  break;
+              }
           i++;
         }
         width=image->extract_info.height-image->rows-image->extract_info.y;
@@ -363,7 +396,7 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
                 if (status == MagickFalse)
                   {
-                    DestroyImageList(image);
+                    image=DestroyImageList(image);
                     return((Image *) NULL);
                   }
               }
@@ -377,13 +410,15 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
               q=GetImagePixels(image,0,y,image->columns,1);
               if (q == (PixelPacket *) NULL)
                 break;
-              (void) PushImagePixels(image,AlphaQuantum,scanline+x);
+              (void) ExportQuantumPixels(image,AlphaQuantum,0,scanline+offset);
               if (SyncImagePixels(image) == MagickFalse)
                 break;
               if (image->previous == (Image *) NULL)
-                if (QuantumTick(i,span) != MagickFalse)
+                if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                    (QuantumTick(i,span) != MagickFalse))
                   {
-                    status=MagickMonitor(LoadImageTag,i,span,&image->exception);
+                    status=image->progress_monitor(LoadImageTag,i,span,
+                      image->client_data);
                     if (status == MagickFalse)
                       break;
                   }
@@ -423,14 +458,17 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
         AllocateNextImage(image_info,image);
         if (image->next == (Image *) NULL)
           {
-            DestroyImageList(image);
+            image=DestroyImageList(image);
             return((Image *) NULL);
           }
         image=SyncNextImageInList(image);
-        status=MagickMonitor(LoadImagesTag,TellBlob(image),GetBlobSize(image),
-          exception);
-        if (status == MagickFalse)
-          break;
+        if (image->progress_monitor != (MagickProgressMonitor) NULL)
+          {
+            status=image->progress_monitor(LoadImagesTag,TellBlob(image),
+              GetBlobSize(image),image->client_data);
+            if (status == MagickFalse)
+              break;
+          }
       }
   } while (count != 0);
   scanline=(unsigned char *) RelinquishMagickMemory(scanline);
@@ -470,6 +508,7 @@ ModuleExport void RegisterRGBImage(void)
   entry->decoder=(DecoderHandler *) ReadRGBImage;
   entry->encoder=(EncoderHandler *) WriteRGBImage;
   entry->raw=MagickTrue;
+  entry->endian_support=MagickTrue;
   entry->description=AcquireString("Raw red, green, and blue samples");
   entry->module=AcquireString("RGB");
   (void) RegisterMagickInfo(entry);
@@ -477,6 +516,7 @@ ModuleExport void RegisterRGBImage(void)
   entry->decoder=(DecoderHandler *) ReadRGBImage;
   entry->encoder=(EncoderHandler *) WriteRGBImage;
   entry->raw=MagickTrue;
+  entry->endian_support=MagickTrue;
   entry->description=AcquireString("Raw red, green, blue, and matte samples");
   entry->module=AcquireString("RGB");
   (void) RegisterMagickInfo(entry);
@@ -553,6 +593,9 @@ static MagickBooleanType WriteRGBImage(const ImageInfo *image_info,Image *image)
   unsigned char
     *pixels;
 
+  unsigned long
+    depth;
+
   /*
     Allocate memory for pixels.
   */
@@ -562,10 +605,12 @@ static MagickBooleanType WriteRGBImage(const ImageInfo *image_info,Image *image)
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
-  packet_size=(size_t) (image->depth > 8 ? 6 : 3);
+  depth=GetImageQuantumDepth(image,MagickTrue);
+  packet_size=(size_t) (3*depth/8);
   if (LocaleCompare(image_info->magick,"RGBA") == 0)
-    packet_size=(size_t) (image->depth > 8 ? 8 : 4);
-  pixels=(unsigned char *) AcquireMagickMemory(packet_size*image->columns);
+    packet_size+=depth/8;
+  pixels=(unsigned char *)
+    AcquireMagickMemory(packet_size*image->columns*sizeof(*pixels));
   if (pixels == (unsigned char *) NULL)
     ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
   if (image_info->interlace != PartitionInterlace)
@@ -602,19 +647,20 @@ static MagickBooleanType WriteRGBImage(const ImageInfo *image_info,Image *image)
             break;
           if (LocaleCompare(image_info->magick,"RGBA") != 0)
             {
-              (void) PopImagePixels(image,RGBQuantum,pixels);
+              (void) ImportQuantumPixels(image,RGBQuantum,0,pixels);
               (void) WriteBlob(image,packet_size*image->columns,pixels);
             }
           else
             {
-              (void) PopImagePixels(image,RGBAQuantum,pixels);
+              (void) ImportQuantumPixels(image,RGBAQuantum,0,pixels);
               (void) WriteBlob(image,packet_size*image->columns,pixels);
             }
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows) != MagickFalse)
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(y,image->rows) != MagickFalse))
               {
-                status=MagickMonitor(SaveImageTag,y,image->rows,
-                  &image->exception);
+                status=image->progress_monitor(SaveImageTag,y,image->rows,
+                  image->client_data);
                 if (status == MagickFalse)
                   break;
               }
@@ -631,21 +677,22 @@ static MagickBooleanType WriteRGBImage(const ImageInfo *image_info,Image *image)
           p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
-          (void) PopImagePixels(image,RedQuantum,pixels);
+          (void) ImportQuantumPixels(image,RedQuantum,0,pixels);
           (void) WriteBlob(image,(size_t) image->columns,pixels);
-          (void) PopImagePixels(image,GreenQuantum,pixels);
+          (void) ImportQuantumPixels(image,GreenQuantum,0,pixels);
           (void) WriteBlob(image,(size_t) image->columns,pixels);
-          (void) PopImagePixels(image,BlueQuantum,pixels);
+          (void) ImportQuantumPixels(image,BlueQuantum,0,pixels);
           (void) WriteBlob(image,(size_t) image->columns,pixels);
           if (LocaleCompare(image_info->magick,"RGBA") == 0)
             {
-              (void) PopImagePixels(image,AlphaQuantum,pixels);
+              (void) ImportQuantumPixels(image,AlphaQuantum,0,pixels);
               (void) WriteBlob(image,(size_t) image->columns,pixels);
             }
-          if (QuantumTick(y,image->rows) != MagickFalse)
+          if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+              (QuantumTick(y,image->rows) != MagickFalse))
             {
-              status=MagickMonitor(SaveImageTag,y,image->rows,
-                &image->exception);
+              status=image->progress_monitor(SaveImageTag,y,image->rows,
+                image->client_data);
               if (status == MagickFalse)
                 break;
             }
@@ -671,7 +718,7 @@ static MagickBooleanType WriteRGBImage(const ImageInfo *image_info,Image *image)
           p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
-          (void) PopImagePixels(image,RedQuantum,pixels);
+          (void) ImportQuantumPixels(image,RedQuantum,0,pixels);
           (void) WriteBlob(image,(size_t) image->columns,pixels);
         }
         if (image_info->interlace == PartitionInterlace)
@@ -683,14 +730,19 @@ static MagickBooleanType WriteRGBImage(const ImageInfo *image_info,Image *image)
             if (status == MagickFalse)
               return(status);
           }
-        if (MagickMonitor(SaveImageTag,100,400,&image->exception) == MagickFalse)
-          break;
+        if (image->progress_monitor != (MagickProgressMonitor) NULL)
+          {
+            status=image->progress_monitor(LoadImageTag,100,400,
+              image->client_data);
+            if (status == MagickFalse)
+              break;
+          }
         for (y=0; y < (long) image->rows; y++)
         {
           p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
-          (void) PopImagePixels(image,GreenQuantum,pixels);
+          (void) ImportQuantumPixels(image,GreenQuantum,0,pixels);
           (void) WriteBlob(image,(size_t) image->columns,pixels);
         }
         if (image_info->interlace == PartitionInterlace)
@@ -702,20 +754,30 @@ static MagickBooleanType WriteRGBImage(const ImageInfo *image_info,Image *image)
             if (status == MagickFalse)
               return(status);
           }
-        if (MagickMonitor(SaveImageTag,200,400,&image->exception) == MagickFalse)
-          break;
+        if (image->progress_monitor != (MagickProgressMonitor) NULL)
+          {
+            status=image->progress_monitor(LoadImageTag,200,400,
+              image->client_data);
+            if (status == MagickFalse)
+              break;
+          }
         for (y=0; y < (long) image->rows; y++)
         {
           p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
-          (void) PopImagePixels(image,BlueQuantum,pixels);
+          (void) ImportQuantumPixels(image,BlueQuantum,0,pixels);
           (void) WriteBlob(image,(size_t) image->columns,pixels);
         }
         if (LocaleCompare(image_info->magick,"RGBA") == 0)
           {
-            if (MagickMonitor(SaveImageTag,300,400,&image->exception) == MagickFalse)
-              break;
+            if (image->progress_monitor != (MagickProgressMonitor) NULL)
+              {
+                status=image->progress_monitor(LoadImageTag,300,400,
+                  image->client_data);
+                if (status == MagickFalse)
+                  break;
+              }
             if (image_info->interlace == PartitionInterlace)
               {
                 CloseBlob(image);
@@ -731,25 +793,34 @@ static MagickBooleanType WriteRGBImage(const ImageInfo *image_info,Image *image)
                 &image->exception);
               if (p == (const PixelPacket *) NULL)
                 break;
-              (void) PopImagePixels(image,AlphaQuantum,pixels);
+              (void) ImportQuantumPixels(image,AlphaQuantum,0,pixels);
               (void) WriteBlob(image,(size_t) image->columns,pixels);
             }
           }
         if (image_info->interlace == PartitionInterlace)
           (void) CopyMagickString(image->filename,image_info->filename,
             MaxTextExtent);
-        if (MagickMonitor(SaveImageTag,400,400,&image->exception) == MagickFalse)
-          break;
+        if (image->progress_monitor != (MagickProgressMonitor) NULL)
+          {
+            status=image->progress_monitor(LoadImageTag,400,400,
+              image->client_data);
+            if (status == MagickFalse)
+              break;
+          }
         break;
       }
     }
     if (image->next == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
-      &image->exception);
-    if (status == MagickFalse)
-      break;
+    if (image->progress_monitor != (MagickProgressMonitor) NULL)
+      {
+        status=image->progress_monitor(SaveImagesTag,scene,
+          GetImageListLength(image),image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
+    scene++;
   } while (image_info->adjoin != MagickFalse);
   pixels=(unsigned char *) RelinquishMagickMemory(pixels);
   CloseBlob(image);
