@@ -10,39 +10,26 @@
 %                            SSSSS   CCCC    T                                %
 %                                                                             %
 %                                                                             %
-%                    Read/Write ImageMagick Image Format.                     %
-%                                                                             %
+%                    Read Scitex HandShake Image Format.                      %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -54,12 +41,17 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
-#include "magick/error.h"
+#include "magick/blob_private.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
+#include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
+#include "magick/memory_.h"
 #include "magick/monitor.h"
 #include "magick/static.h"
-#include "magick/utility.h"
+#include "magick/string_.h"
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -72,17 +64,15 @@
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method IsSCT returns True if the image format type, identified by the
+%  IsSCT() returns MagickTrue if the image format type, identified by the
 %  magick string, is SCT.
 %
 %  The format of the IsSCT method is:
 %
-%      unsigned int IsSCT(const unsigned char *magick,const size_t length)
+%      MagickBooleanType IsSCT(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method IsSCT returns True if the image format type is SCT.
-%
 %    o magick: This string is generally the first few bytes of an image file
 %      or blob.
 %
@@ -90,13 +80,13 @@
 %
 %
 */
-static unsigned int IsSCT(const unsigned char *magick,const size_t length)
+static MagickBooleanType IsSCT(const unsigned char *magick,const size_t length)
 {
   if (length < 2)
-    return(False);
+    return(MagickFalse);
   if (LocaleNCompare((char *) magick,"CT",2) == 0)
-    return(True);
-  return(False);
+    return(MagickTrue);
+  return(MagickFalse);
 }
 
 /*
@@ -110,7 +100,7 @@ static unsigned int IsSCT(const unsigned char *magick,const size_t length)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ReadSCTImage reads a Scitex image file and returns it.  It allocates
+%  ReadSCTImage() reads a Scitex image file and returns it.  It allocates
 %  the memory necessary for the new Image structure and returns a pointer to
 %  the new image.
 %
@@ -120,10 +110,6 @@ static unsigned int IsSCT(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o image:  Method ReadSCTImage returns a pointer to the image after
-%      reading.  A null image is returned if there is a memory shortage or
-%      if the image cannot be read.
-%
 %    o image_info: The image info.
 %
 %    o exception: return any errors or warnings in this structure.
@@ -133,7 +119,6 @@ static unsigned int IsSCT(const unsigned char *magick,const size_t length)
 static Image *ReadSCTImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
   char
-    buffer[768],
     magick[2];
 
   Image
@@ -142,59 +127,92 @@ static Image *ReadSCTImage(const ImageInfo *image_info,ExceptionInfo *exception)
   long
     y;
 
+  MagickBooleanType
+    status;
+
+  MagickRealType
+    height,
+    width;
+
+  register IndexPacket
+    *indexes;
+
   register long
     x;
 
   register PixelPacket
     *q;
 
-  unsigned int
-    status;
+  unsigned char
+    buffer[768];
+
+  unsigned long
+    separations,
+    separations_mask,
+    units;
 
   /*
     Open image file.
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
+  if (image_info->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-  if (status == False)
-    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
+  if (status == MagickFalse)
+    {
+      image=DestroyImageList(image);
+      return((Image *) NULL);
+    }
   /*
     Read control block.
   */
-  (void) ReadBlob(image,80,(char *) buffer);
-  (void) ReadBlob(image,2,(char *) magick);
+  (void) ReadBlob(image,80,buffer);
+  (void) ReadBlob(image,2,(unsigned char *) magick);
   if ((LocaleNCompare((char *) magick,"CT",2) != 0) &&
       (LocaleNCompare((char *) magick,"LW",2) != 0) &&
       (LocaleNCompare((char *) magick,"BM",2) != 0) &&
       (LocaleNCompare((char *) magick,"PG",2) != 0) &&
       (LocaleNCompare((char *) magick,"TX",2) != 0))
-    ThrowReaderException(CorruptImageError,"NotASCTImageFile",image);
+    ThrowReaderException(CorruptImageError,"ImproperImageHeader");
   if ((LocaleNCompare((char *) magick,"LW",2) == 0) ||
       (LocaleNCompare((char *) magick,"BM",2) == 0) ||
       (LocaleNCompare((char *) magick,"PG",2) == 0) ||
       (LocaleNCompare((char *) magick,"TX",2) == 0))
-    ThrowReaderException(CoderError,"OnlyContinuousTonePictureSupported",image);
-  (void) ReadBlob(image,174,(char *) buffer);
-  (void) ReadBlob(image,768,(char *) buffer);
+    ThrowReaderException(CoderError,"OnlyContinuousTonePictureSupported");
+  (void) ReadBlob(image,174,buffer);
+  (void) ReadBlob(image,768,buffer);
   /*
     Read paramter block.
   */
-  (void) ReadBlob(image,32,(char *) buffer);
-  (void) ReadBlob(image,14,(char *) buffer);
-  image->rows=atol(buffer);
-  (void) ReadBlob(image,14,(char *) buffer);
-  image->columns=atol(buffer);
-  (void) ReadBlob(image,196,(char *) buffer);
-  (void) ReadBlob(image,768,(char *) buffer);
-  image->colorspace=CMYKColorspace;
-  if (image_info->ping)
+  units=ReadBlobByte(image);
+  if (units == 0)
+    image->units=PixelsPerCentimeterResolution;
+  separations=ReadBlobByte(image);
+  separations_mask=ReadBlobMSBShort(image);
+  (void) ReadBlob(image,14,buffer);
+  buffer[14]='\0';
+  height=atof((char *) buffer);
+  (void) ReadBlob(image,14,buffer);
+  width=atof((char *) buffer);
+  (void) ReadBlob(image,12,buffer);
+  buffer[12]='\0';
+  image->rows=atol((char *) buffer);
+  (void) ReadBlob(image,12,buffer);
+  image->columns=atol((char *) buffer);
+  (void) ReadBlob(image,200,buffer);
+  (void) ReadBlob(image,768,buffer);
+  if (separations_mask == 0x0f)
+    image->colorspace=CMYKColorspace;
+  image->x_resolution=image->columns/width;
+  image->y_resolution=image->rows/height;
+  if (image_info->ping != MagickFalse)
     {
       CloseBlob(image);
-      return(image);
+      return(GetFirstImageInList(image));
     }
   /*
     Convert SCT raster image to pixel packets.
@@ -224,6 +242,7 @@ static Image *ReadSCTImage(const ImageInfo *image_info,ExceptionInfo *exception)
     q=GetImagePixels(image,0,y,image->columns,1);
     if (q == (PixelPacket *) NULL)
       break;
+    indexes=GetIndexes(image);
     for (x=0; x < (long) image->columns; x++)
     {
       q->blue=(Quantum) (MaxRGB-ScaleCharToQuantum(ReadBlobByte(image)));
@@ -231,27 +250,36 @@ static Image *ReadSCTImage(const ImageInfo *image_info,ExceptionInfo *exception)
     }
     if ((image->columns % 2) != 0)
       (void) ReadBlobByte(image);  /* pad */
-    q=GetImagePixels(image,0,y,image->columns,1);
-    if (q == (PixelPacket *) NULL)
-      break;
-    for (x=0; x < (long) image->columns; x++)
-    {
-      q->opacity=(Quantum) (MaxRGB-ScaleCharToQuantum(ReadBlobByte(image)));
-      q++;
-    }
-    if (!SyncImagePixels(image))
-      break;
-    if ((image->columns % 2) != 0)
-      (void) ReadBlobByte(image);  /* pad */
-    if (QuantumTick(y,image->rows))
-      if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
-        break;
+    if (image->colorspace == CMYKColorspace)
+      {
+        q=GetImagePixels(image,0,y,image->columns,1);
+        if (q == (PixelPacket *) NULL)
+          break;
+        for (x=0; x < (long) image->columns; x++)
+        {
+          indexes[x]=(IndexPacket) (MaxRGB-
+            ScaleCharToQuantum(ReadBlobByte(image)));
+          q++;
+        }
+        if (SyncImagePixels(image) == MagickFalse)
+          break;
+        if ((image->columns % 2) != 0)
+          (void) ReadBlobByte(image);  /* pad */
+      }
+    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+        (QuantumTick(y,image->rows) != MagickFalse))
+      {
+        status=image->progress_monitor(LoadImageTag,y,image->rows,
+          image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
   }
-  if (EOFBlob(image))
-    (void) ThrowException(exception,CorruptImageError,"UnexpectedEndOfFile",
+  if (EOFBlob(image) != MagickFalse)
+    ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
       image->filename);
   CloseBlob(image);
-  return(image);
+  return(GetFirstImageInList(image));
 }
 
 /*
@@ -265,7 +293,7 @@ static Image *ReadSCTImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method RegisterSCTImage adds attributes for the SCT image format to
+%  RegisterSCTImage() adds attributes for the SCT image format to
 %  the list of supported formats.  The attributes include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
@@ -283,9 +311,9 @@ ModuleExport void RegisterSCTImage(void)
     *entry;
 
   entry=SetMagickInfo("SCT");
-  entry->decoder=(DecoderHandler) ReadSCTImage;
-  entry->magick=(MagickHandler) IsSCT;
-  entry->adjoin=False;
+  entry->decoder=(DecoderHandler *) ReadSCTImage;
+  entry->magick=(MagickHandler *) IsSCT;
+  entry->adjoin=MagickFalse;
   entry->description=AcquireString("Scitex HandShake");
   entry->module=AcquireString("SCT");
   (void) RegisterMagickInfo(entry);
@@ -302,7 +330,7 @@ ModuleExport void RegisterSCTImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method UnregisterSCTImage removes format registrations made by the
+%  UnregisterSCTImage() removes format registrations made by the
 %  SCT module from the list of supported formats.
 %
 %  The format of the UnregisterSCTImage method is:
