@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -41,9 +41,12 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
 #include "magick/constitute.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
@@ -238,7 +241,7 @@ static Image *ReadSFWImage(const ImageInfo *image_info,ExceptionInfo *exception)
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
-      DestroyImageList(image);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
   /*
@@ -251,7 +254,7 @@ static Image *ReadSFWImage(const ImageInfo *image_info,ExceptionInfo *exception)
   if ((count == 0) || (LocaleNCompare((char *) buffer,"SFW",3) != 0))
     ThrowReaderException(CorruptImageError,"ImproperImageHeader");
   CloseBlob(image);
-  DestroyImage(image);
+  image=DestroyImage(image);
   /*
     Find the start of the JFIF data
   */
@@ -299,12 +302,12 @@ static Image *ReadSFWImage(const ImageInfo *image_info,ExceptionInfo *exception)
   if ((unique_file == -1) || (file == (FILE *) NULL))
     {
       buffer=(unsigned char *) RelinquishMagickMemory(buffer);
-      DestroyImageInfo(read_info);
+      read_info=DestroyImageInfo(read_info);
       (void) CopyMagickString(image->filename,read_info->filename,
         MaxTextExtent);
       ThrowFileException(exception,FileOpenError,"UnableToCreateTemporaryFile",
         image->filename);
-      DestroyImageList(image);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
   (void) fwrite(header,offset-header+1,1,file);
@@ -316,10 +319,10 @@ static Image *ReadSFWImage(const ImageInfo *image_info,ExceptionInfo *exception)
   if (status == MagickFalse)
     {
       (void) remove(read_info->filename);
-      DestroyImageInfo(read_info);
+      read_info=DestroyImageInfo(read_info);
       (void) ThrowMagickException(&image->exception,GetMagickModule(),
         FileOpenError,"UnableToWriteFile",image->filename,strerror(errno));
-      DestroyImageList(image);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
   /*
@@ -327,7 +330,7 @@ static Image *ReadSFWImage(const ImageInfo *image_info,ExceptionInfo *exception)
   */
   image=ReadImage(read_info,exception);
   (void) RelinquishUniqueFileResource(read_info->filename);
-  DestroyImageInfo(read_info);
+  read_info=DestroyImageInfo(read_info);
   if (image == (Image *) NULL)
     return(GetFirstImageInList(image));
   /*
@@ -336,7 +339,7 @@ static Image *ReadSFWImage(const ImageInfo *image_info,ExceptionInfo *exception)
   flipped_image=FlipImage(image,exception);
   if (flipped_image == (Image *) NULL)
     return(GetFirstImageInList(image));
-  DestroyImage(image);
+  image=DestroyImage(image);
   return(flipped_image);
 }
 
