@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -41,11 +41,14 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
 #include "magick/color.h"
 #include "magick/color_private.h"
 #include "magick/colorspace.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
@@ -104,8 +107,6 @@ static MagickBooleanType
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method IsSGI() returns MagickTrue if the image format type is SGI.
-%
 %    o magick: This string is generally the first few bytes of an image file
 %      or blob.
 %
@@ -143,10 +144,6 @@ static MagickBooleanType IsSGI(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o image:  Method ReadSGIImage returns a pointer to the image after
-%      reading.  A null image is returned if there is a memory shortage or
-%      if the image cannot be read.
-%
 %    o image_info: The image info.
 %
 %    o exception: return any errors or warnings in this structure.
@@ -278,7 +275,7 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
-      DestroyImageList(image);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
   /*
@@ -381,11 +378,11 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
           Read runlength-encoded image format.
         */
         offsets=(ssize_t *) AcquireMagickMemory((size_t) iris_info.rows*
-          iris_info.depth*sizeof(unsigned long));
+          iris_info.depth*sizeof(*offsets));
         max_packets=(unsigned char *)
           AcquireMagickMemory(4*(size_t) iris_info.columns+10);
         runlength=(unsigned long *) AcquireMagickMemory(iris_info.rows*
-          iris_info.depth*sizeof(unsigned long));
+          iris_info.depth*sizeof(*runlength));
         if ((offsets == (ssize_t *) NULL) ||
             (max_packets == (unsigned char *) NULL) ||
             (runlength == (unsigned long *) NULL))
@@ -494,9 +491,11 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
               if (SyncImagePixels(image) == MagickFalse)
                 break;
               if (image->previous == (Image *) NULL)
-                if (QuantumTick(y,image->rows) != 0)
+                if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                    (QuantumTick(y,image->rows) != MagickFalse))
                   {
-                    status=MagickMonitor(LoadImageTag,y,image->rows,exception);
+                    status=image->progress_monitor(LoadImageTag,y,image->rows,
+                      image->client_data);
                     if (status == MagickFalse)
                       break;
                   }
@@ -521,9 +520,11 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
             if (SyncImagePixels(image) == MagickFalse)
               break;
             if (image->previous == (Image *) NULL)
-              if (QuantumTick(y,image->rows) != 0)
+              if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                  (QuantumTick(y,image->rows) != MagickFalse))
                 {
-                  status=MagickMonitor(LoadImageTag,y,image->rows,exception);
+                  status=image->progress_monitor(LoadImageTag,y,image->rows,
+                    image->client_data);
                   if (status == MagickFalse)
                     break;
                 }
@@ -558,9 +559,11 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
               if (SyncImagePixels(image) == MagickFalse)
                 break;
               if (image->previous == (Image *) NULL)
-                if (QuantumTick(y,image->rows) != 0)
+                if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                    (QuantumTick(y,image->rows) != MagickFalse))
                   {
-                    status=MagickMonitor(LoadImageTag,y,image->rows,exception);
+                    status=image->progress_monitor(LoadImageTag,y,image->rows,
+                      image->client_data);
                     if (status == MagickFalse)
                       break;
                   }
@@ -583,9 +586,11 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
             if (SyncImagePixels(image) == MagickFalse)
               break;
             if (image->previous == (Image *) NULL)
-              if (QuantumTick(y,image->rows) != 0)
+              if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                  (QuantumTick(y,image->rows) != MagickFalse))
                 {
-                  status=MagickMonitor(LoadImageTag,y,image->rows,exception);
+                  status=image->progress_monitor(LoadImageTag,y,image->rows,
+                    image->client_data);
                   if (status == MagickFalse)
                     break;
                 }
@@ -606,7 +611,6 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
     iris_info.magic=ReadBlobMSBShort(image);
-printf("%d %x\n",(int) TellBlob(image),iris_info.magic);
     if (iris_info.magic == 0x01DA)
       {
         /*
@@ -615,14 +619,17 @@ printf("%d %x\n",(int) TellBlob(image),iris_info.magic);
         AllocateNextImage(image_info,image);
         if (image->next == (Image *) NULL)
           {
-            DestroyImageList(image);
+            image=DestroyImageList(image);
             return((Image *) NULL);
           }
         image=SyncNextImageInList(image);
-        status=MagickMonitor(LoadImagesTag,TellBlob(image),GetBlobSize(image),
-          exception);
-        if (status == MagickFalse)
-          break;
+        if (image->progress_monitor != (MagickProgressMonitor) NULL)
+          {
+            status=image->progress_monitor(LoadImagesTag,TellBlob(image),
+              GetBlobSize(image),image->client_data);
+            if (status == MagickFalse)
+              break;
+          }
       }
   } while (iris_info.magic == 0x01DA);
   CloseBlob(image);
@@ -710,10 +717,6 @@ ModuleExport void UnregisterSGIImage(void)
 %
 %  A description of each parameter follows.
 %
-%    o status: Method WriteSGIImage returns True if the image is written.
-%      False is returned is there is a memory shortage or if the image file
-%      fails to write.
-%
 %    o image_info: The image info.
 %
 %    o image:  The image.
@@ -894,9 +897,11 @@ static MagickBooleanType WriteSGIImage(const ImageInfo *image_info,Image *image)
         p++;
       }
       if (image->previous == (Image *) NULL)
-        if (QuantumTick(y,image->rows) != 0)
+        if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+            (QuantumTick(y,image->rows) != MagickFalse))
           {
-            status=MagickMonitor(SaveImageTag,y,image->rows,&image->exception);
+            status=image->progress_monitor(SaveImageTag,y,image->rows,
+              image->client_data);
             if (status == MagickFalse)
               break;
           }
@@ -945,11 +950,11 @@ static MagickBooleanType WriteSGIImage(const ImageInfo *image_info,Image *image)
           Convert SGI uncompressed pixels.
         */
         offsets=(ssize_t *) AcquireMagickMemory(iris_info.rows*
-          iris_info.depth*sizeof(unsigned long));
+          iris_info.depth*sizeof(*offsets));
         packets=(unsigned char *)
           AcquireMagickMemory(4*(2*(size_t) iris_info.columns+10)*image->rows);
         runlength=(unsigned long *) AcquireMagickMemory(iris_info.rows*
-          iris_info.depth*sizeof(unsigned long));
+          iris_info.depth*sizeof(*runlength));
         if ((offsets == (ssize_t *) NULL) ||
             (packets == (unsigned char *) NULL) ||
             (runlength == (unsigned long *) NULL))
@@ -989,10 +994,14 @@ static MagickBooleanType WriteSGIImage(const ImageInfo *image_info,Image *image)
     if (image->next == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
-      &image->exception);
-    if (status == MagickFalse)
-      break;
+    if (image->progress_monitor != (MagickProgressMonitor) NULL)
+      {
+        status=image->progress_monitor(SaveImagesTag,scene,
+          GetImageListLength(image),image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
+    scene++;
   } while (image_info->adjoin != MagickFalse);
   if (image_info->adjoin != MagickFalse)
     while (image->previous != (Image *) NULL)
