@@ -10,39 +10,26 @@
 %                            SSSSS   UUU   N   N                              %
 %                                                                             %
 %                                                                             %
-%                    Read/Write ImageMagick Image Format.                     %
-%                                                                             %
+%                    Read/Write Sun Rasterfile Image Format.                  %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -54,17 +41,25 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
-#include "magick/error.h"
+#include "magick/blob_private.h"
+#include "magick/color.h"
+#include "magick/color_private.h"
+#include "magick/colorspace.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
+#include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
+#include "magick/memory_.h"
 #include "magick/monitor.h"
 #include "magick/static.h"
-#include "magick/utility.h"
+#include "magick/string_.h"
 
 /*
   Forward declarations.
 */
-static unsigned int
+static MagickBooleanType
   WriteSUNImage(const ImageInfo *,Image *);
 
 /*
@@ -78,17 +73,15 @@ static unsigned int
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method IsSUN returns True if the image format type, identified by the
+%  IsSUN() returns MagickTrue if the image format type, identified by the
 %  magick string, is SUN.
 %
 %  The format of the IsSUN method is:
 %
-%      unsigned int IsSUN(const unsigned char *magick,const size_t length)
+%      MagickBooleanType IsSUN(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method IsSUN returns True if the image format type is SUN.
-%
 %    o magick: This string is generally the first few bytes of an image file
 %      or blob.
 %
@@ -96,13 +89,13 @@ static unsigned int
 %
 %
 */
-static unsigned int IsSUN(const unsigned char *magick,const size_t length)
+static MagickBooleanType IsSUN(const unsigned char *magick,const size_t length)
 {
   if (length < 4)
-    return(False);
+    return(MagickFalse);
   if (memcmp(magick,"\131\246\152\225",4) == 0)
-    return(True);
-  return(False);
+    return(MagickTrue);
+  return(MagickFalse);
 }
 
 /*
@@ -116,19 +109,16 @@ static unsigned int IsSUN(const unsigned char *magick,const size_t length)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method DecodeImage unpacks the packed image pixels into
-%  runlength-encoded pixel packets.
+%  DecodeImage unpacks the packed image pixels into  runlength-encoded pixel
+%  packets.
 %
 %  The format of the DecodeImage method is:
 %
-%      unsigned int DecodeImage(const unsigned char *compressed_pixels,
+%      MagickBooleanType DecodeImage(const unsigned char *compressed_pixels,
 %        const size_t length,unsigned char *pixels)
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method DecodeImage returns True if all the pixels are
-%      uncompressed without error, otherwise False.
-%
 %    o compressed_pixels:  The address of a byte (8 bits) array of compressed
 %      pixel data.
 %
@@ -142,21 +132,22 @@ static unsigned int IsSUN(const unsigned char *magick,const size_t length)
 %
 %
 */
-static unsigned int DecodeImage(const unsigned char *compressed_pixels,
+static MagickBooleanType DecodeImage(const unsigned char *compressed_pixels,
   const size_t length,unsigned char *pixels)
 {
   register const unsigned char
     *p;
 
-  register long
-    count;
-
   register unsigned char
     *q;
 
+  ssize_t
+    count;
+
   unsigned char
     byte;
 
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(compressed_pixels != (unsigned char *) NULL);
   assert(pixels != (unsigned char *) NULL);
   p=compressed_pixels;
@@ -164,14 +155,14 @@ static unsigned int DecodeImage(const unsigned char *compressed_pixels,
   while ((size_t) (p-compressed_pixels) < length)
   {
     byte=(*p++);
-    if (byte != 128)
+    if (byte != 128U)
       *q++=byte;
     else
       {
         /*
           Runlength-encoded packet: <count><byte>
         */
-        count=(*p++);
+        count=(ssize_t) (*p++);
         if (count > 0)
           byte=(*p++);
         while (count >= 0)
@@ -181,7 +172,7 @@ static unsigned int DecodeImage(const unsigned char *compressed_pixels,
         }
      }
   }
-  return(True);
+  return(MagickTrue);
 }
 
 /*
@@ -195,7 +186,7 @@ static unsigned int DecodeImage(const unsigned char *compressed_pixels,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ReadSUNImage reads a SUN image file and returns it.  It allocates
+%  ReadSUNImage() reads a SUN image file and returns it.  It allocates
 %  the memory necessary for the new Image structure and returns a pointer to
 %  the new image.
 %
@@ -205,10 +196,6 @@ static unsigned int DecodeImage(const unsigned char *compressed_pixels,
 %
 %  A description of each parameter follows:
 %
-%    o image:  Method ReadSUNImage returns a pointer to the image after
-%      reading.  A null image is returned if there is a memory shortage or
-%      if the image cannot be read.
-%
 %    o image_info: The image info.
 %
 %    o exception: return any errors or warnings in this structure.
@@ -261,7 +248,7 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
   register unsigned char
     *p;
 
-  size_t
+  ssize_t
     count;
 
   SUNInfo
@@ -282,12 +269,17 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
+  if (image_info->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-  if (status == False)
-    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
+  if (status == MagickFalse)
+    {
+      image=DestroyImageList(image);
+      return((Image *) NULL);
+    }
   /*
     Read SUN raster header.
   */
@@ -298,7 +290,7 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
       Verify SUN identifier.
     */
     if (sun_info.magic != 0x59a66a95)
-      ThrowReaderException(CorruptImageError,"NotASUNRasterImageFile",image);
+      ThrowReaderException(CorruptImageError,"ImproperImageHeader");
     sun_info.width=ReadBlobMSBLong(image);
     sun_info.height=ReadBlobMSBLong(image);
     sun_info.depth=ReadBlobMSBLong(image);
@@ -308,7 +300,7 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
     sun_info.maplength=ReadBlobMSBLong(image);
     image->columns= sun_info.width;
     image->rows= sun_info.height;
-    image->depth=sun_info.depth <= 8 ? 8 : QuantumDepth;
+    image->depth=sun_info.depth <= 8 ? 8UL : QuantumDepth;
     if (sun_info.depth < 24)
       {
         image->storage_class=PseudoClass;
@@ -327,9 +319,8 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
             /*
               Create linear color ramp.
             */
-            if (!AllocateImageColormap(image,image->colors))
-              ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",
-                image);
+            if (AllocateImageColormap(image,image->colors) == MagickFalse)
+              ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
           }
         break;
       }
@@ -341,23 +332,21 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
         /*
           Read SUN raster colormap.
         */
-        if (!AllocateImageColormap(image,image->colors))
-          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",
-            image);
-        sun_colormap=(unsigned char *) AcquireMemory(image->colors);
+        if (AllocateImageColormap(image,image->colors) == MagickFalse)
+          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+        sun_colormap=(unsigned char *) AcquireMagickMemory(image->colors);
         if (sun_colormap == (unsigned char *) NULL)
-          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",
-            image);
-        (void) ReadBlob(image,image->colors,(char *) sun_colormap);
+          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+        (void) ReadBlob(image,image->colors,sun_colormap);
         for (i=0; i < (long) image->colors; i++)
           image->colormap[i].red=ScaleCharToQuantum(sun_colormap[i]);
-        (void) ReadBlob(image,image->colors,(char *) sun_colormap);
+        (void) ReadBlob(image,image->colors,sun_colormap);
         for (i=0; i < (long) image->colors; i++)
           image->colormap[i].green=ScaleCharToQuantum(sun_colormap[i]);
-        (void) ReadBlob(image,image->colors,(char *) sun_colormap);
+        (void) ReadBlob(image,image->colors,sun_colormap);
         for (i=0; i < (long) image->colors; i++)
           image->colormap[i].blue=ScaleCharToQuantum(sun_colormap[i]);
-        LiberateMemory((void **) &sun_colormap);
+        sun_colormap=(unsigned char *) RelinquishMagickMemory(sun_colormap);
         break;
       }
       case RMT_RAW:
@@ -368,31 +357,30 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
         /*
           Read SUN raster colormap.
         */
-        sun_colormap=(unsigned char *) AcquireMemory(sun_info.maplength);
+        sun_colormap=(unsigned char *) AcquireMagickMemory(sun_info.maplength);
         if (sun_colormap == (unsigned char *) NULL)
-          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",
-            image);
-        (void) ReadBlob(image,sun_info.maplength,(char *) sun_colormap);
-        LiberateMemory((void **) &sun_colormap);
+          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+        (void) ReadBlob(image,sun_info.maplength,sun_colormap);
+        sun_colormap=(unsigned char *) RelinquishMagickMemory(sun_colormap);
         break;
       }
       default:
-        ThrowReaderException(CoderError,"ColormapTypeNotSupported",image)
-    } 
-    image->matte=(sun_info.depth == 32);
+        ThrowReaderException(CoderError,"ColormapTypeNotSupported");
+    }
+    image->matte=sun_info.depth == 32 ? MagickTrue : MagickFalse;
     image->columns=sun_info.width;
     image->rows=sun_info.height;
-    if (image_info->ping)
+    if (image_info->ping != MagickFalse)
       {
         CloseBlob(image);
-        return(image);
+        return(GetFirstImageInList(image));
       }
-    sun_data=(unsigned char *) AcquireMemory(sun_info.length);
+    sun_data=(unsigned char *) AcquireMagickMemory(sun_info.length);
     if (sun_data == (unsigned char *) NULL)
-      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
-    count=ReadBlob(image,sun_info.length,(char *) sun_data);
+      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+    count=(ssize_t) ReadBlob(image,sun_info.length,sun_data);
     if ((count == 0) && (sun_info.type != RT_ENCODED))
-      ThrowReaderException(CorruptImageError,"UnableToReadImageData",image);
+      ThrowReaderException(CorruptImageError,"UnableToReadImageData");
     sun_pixels=sun_data;
     if (sun_info.type == RT_ENCODED)
       {
@@ -404,12 +392,11 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
         */
         height=sun_info.height;
         bytes_per_line=2*(sun_info.width*sun_info.depth+15)/16;
-        sun_pixels=(unsigned char *) AcquireMemory(bytes_per_line*height);
+        sun_pixels=(unsigned char *) AcquireMagickMemory(bytes_per_line*height);
         if (sun_pixels == (unsigned char *) NULL)
-          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",
-            image);
+          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
         (void) DecodeImage(sun_data,sun_info.length,sun_pixels);
-        LiberateMemory((void **) &sun_data);
+        sun_data=(unsigned char *) RelinquishMagickMemory(sun_data);
       }
     /*
       Convert SUN raster image to pixel packets.
@@ -425,23 +412,29 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
         for (x=0; x < ((long) image->columns-7); x+=8)
         {
           for (bit=7; bit >= 0; bit--)
-            indexes[x+7-bit]=((*p) & (0x01 << bit) ? 0x01 : 0x00);
+            indexes[x+7-bit]=(IndexPacket) ((*p) & (0x01 << bit) ? 0x00 : 0x01);
           p++;
         }
         if ((image->columns % 8) != 0)
           {
             for (bit=7; bit >= (long) (8-(image->columns % 8)); bit--)
-              indexes[x+7-bit]=((*p) & (0x01 << bit) ? 0x01 : 0x00);
+              indexes[x+7-bit]=(IndexPacket)
+                ((*p) & (0x01 << bit) ? 0x00 : 0x01);
             p++;
           }
         if ((((image->columns/8)+(image->columns % 8 ? 1 : 0)) % 2) != 0)
           p++;
-        if (!SyncImagePixels(image))
+        if (SyncImagePixels(image) == MagickFalse)
           break;
         if (image->previous == (Image *) NULL)
-          if (QuantumTick(y,image->rows))
-            if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
-              break;
+          if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+              (QuantumTick(y,image->rows) != MagickFalse))
+            {
+              status=image->progress_monitor(LoadImageTag,y,image->rows,
+                image->client_data);
+              if (status == MagickFalse)
+                break;
+            }
       }
     else
       if (image->storage_class == PseudoClass)
@@ -455,12 +448,17 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
             indexes[x]=(*p++);
           if ((image->columns % 2) != 0)
             p++;
-          if (!SyncImagePixels(image))
+          if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows))
-              if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
-                break;
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(y,image->rows) != MagickFalse))
+              {
+                status=image->progress_monitor(LoadImageTag,y,image->rows,
+                  image->client_data);
+                if (status == MagickFalse)
+                  break;
+              }
         }
       else
         for (y=0; y < (long) image->rows; y++)
@@ -470,7 +468,7 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
             break;
           for (x=0; x < (long) image->columns; x++)
           {
-            if (image->matte)
+            if (image->matte != MagickFalse)
               q->opacity=(Quantum) (MaxRGB-ScaleCharToQuantum(*p++));
             if (sun_info.type == RT_STANDARD)
               {
@@ -492,21 +490,26 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
               }
             q++;
           }
-          if (((image->columns % 2) != 0) && (image->matte == False))
+          if (((image->columns % 2) != 0) && (image->matte == MagickFalse))
             p++;
-          if (!SyncImagePixels(image))
+          if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows))
-              if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
-                break;
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(y,image->rows) != MagickFalse))
+              {
+                status=image->progress_monitor(LoadImageTag,y,image->rows,
+                  image->client_data);
+                if (status == MagickFalse)
+                  break;
+              }
         }
     if (image->storage_class == PseudoClass)
-      SyncImage(image);
-    LiberateMemory((void **) &sun_pixels);
-    if (EOFBlob(image))
+      (void) SyncImage(image);
+    sun_pixels=(unsigned char *) RelinquishMagickMemory(sun_pixels);
+    if (EOFBlob(image) != MagickFalse)
       {
-        (void) ThrowException(exception,CorruptImageError,"UnexpectedEndOfFile",
+        ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
           image->filename);
         break;
       }
@@ -525,20 +528,21 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
         AllocateNextImage(image_info,image);
         if (image->next == (Image *) NULL)
           {
-            DestroyImageList(image);
+            image=DestroyImageList(image);
             return((Image *) NULL);
           }
         image=SyncNextImageInList(image);
-        status=MagickMonitor(LoadImagesTag,TellBlob(image),GetBlobSize(image),
-          exception);
-        if (status == False)
-          break;
+        if (image->progress_monitor != (MagickProgressMonitor) NULL)
+          {
+            status=image->progress_monitor(LoadImageTag,TellBlob(image),
+              GetBlobSize(image),image->client_data);
+            if (status == MagickFalse)
+              break;
+          }
       }
   } while (sun_info.magic == 0x59a66a95);
-  while (image->previous != (Image *) NULL)
-    image=image->previous;
   CloseBlob(image);
-  return(image);
+  return(GetFirstImageInList(image));
 }
 
 /*
@@ -552,7 +556,7 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method RegisterSUNImage adds attributes for the SUN image format to
+%  RegisterSUNImage() adds attributes for the SUN image format to
 %  the list of supported formats.  The attributes include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
@@ -570,15 +574,15 @@ ModuleExport void RegisterSUNImage(void)
     *entry;
 
   entry=SetMagickInfo("RAS");
-  entry->decoder=(DecoderHandler) ReadSUNImage;
-  entry->encoder=(EncoderHandler) WriteSUNImage;
-  entry->magick=(MagickHandler) IsSUN;
+  entry->decoder=(DecoderHandler *) ReadSUNImage;
+  entry->encoder=(EncoderHandler *) WriteSUNImage;
+  entry->magick=(MagickHandler *) IsSUN;
   entry->description=AcquireString("SUN Rasterfile");
   entry->module=AcquireString("SUN");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("SUN");
-  entry->decoder=(DecoderHandler) ReadSUNImage;
-  entry->encoder=(EncoderHandler) WriteSUNImage;
+  entry->decoder=(DecoderHandler *) ReadSUNImage;
+  entry->encoder=(EncoderHandler *) WriteSUNImage;
   entry->description=AcquireString("SUN Rasterfile");
   entry->module=AcquireString("SUN");
   (void) RegisterMagickInfo(entry);
@@ -595,7 +599,7 @@ ModuleExport void RegisterSUNImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method UnregisterSUNImage removes format registrations made by the
+%  UnregisterSUNImage() removes format registrations made by the
 %  SUN module from the list of supported formats.
 %
 %  The format of the UnregisterSUNImage method is:
@@ -620,25 +624,21 @@ ModuleExport void UnregisterSUNImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method WriteSUNImage writes an image in the SUN rasterfile format.
+%  WriteSUNImage() writes an image in the SUN rasterfile format.
 %
 %  The format of the WriteSUNImage method is:
 %
-%      unsigned int WriteSUNImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WriteSUNImage(const ImageInfo *image_info,Image *image)
 %
 %  A description of each parameter follows.
 %
-%    o status: Method WriteSUNImage return True if the image is written.
-%      False is returned is there is a memory shortage or if the image file
-%      fails to write.
-%
 %    o image_info: The image info.
 %
 %    o image:  The image.
 %
 %
 */
-static unsigned int WriteSUNImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WriteSUNImage(const ImageInfo *image_info,Image *image)
 {
 #define RMT_EQUAL_RGB  1
 #define RMT_NONE  0
@@ -659,12 +659,18 @@ static unsigned int WriteSUNImage(const ImageInfo *image_info,Image *image)
       maplength;
   } SUNInfo;
 
-  ExtendedSignedIntegralType
-    number_pixels;
-
   long
     y;
 
+  MagickBooleanType
+    status;
+
+  MagickOffsetType
+    scene;
+
+  MagickSizeType
+    number_pixels;
+
   register const PixelPacket
     *p;
 
@@ -680,12 +686,6 @@ static unsigned int WriteSUNImage(const ImageInfo *image_info,Image *image)
   SUNInfo
     sun_info;
 
-  unsigned int
-    status;
-
-  unsigned long
-    scene;
-
   /*
     Open output image file.
   */
@@ -693,9 +693,11 @@ static unsigned int WriteSUNImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
-  if (status == False)
-    ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
+  if (status == MagickFalse)
+    return(status);
   scene=0;
   do
   {
@@ -706,20 +708,20 @@ static unsigned int WriteSUNImage(const ImageInfo *image_info,Image *image)
     sun_info.magic=0x59a66a95;
     sun_info.width=image->columns;
     sun_info.height=image->rows;
-    sun_info.type=
+    sun_info.type=(unsigned long) 
       (image->storage_class == DirectClass ? RT_FORMAT_RGB : RT_STANDARD);
     sun_info.maptype=RMT_NONE;
     sun_info.maplength=0;
-    number_pixels=(ExtendedSignedIntegralType) image->columns*image->rows;
+    number_pixels=(MagickSizeType) image->columns*image->rows;
     if ((4*number_pixels) != (size_t) (4*number_pixels))
-      ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",image);
+      ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
     if (image->storage_class == DirectClass)
       {
         /*
           Full color SUN raster.
         */
-        sun_info.depth=(image->matte ? 32 : 24);
-        sun_info.length=(image->matte ? 4 : 3)*number_pixels;
+        sun_info.depth=(image->matte ? 32UL : 24UL);
+        sun_info.length=(unsigned long) ((image->matte ? 4 : 3)*number_pixels);
         sun_info.length+=image->columns & 0x01 ? image->rows : 0;
       }
     else
@@ -739,7 +741,7 @@ static unsigned int WriteSUNImage(const ImageInfo *image_info,Image *image)
             Colormapped SUN raster.
           */
           sun_info.depth=8;
-          sun_info.length=number_pixels;
+          sun_info.length=(unsigned long) number_pixels;
           sun_info.length+=image->columns & 0x01 ? image->rows : 0;
           sun_info.maptype=RMT_EQUAL_RGB;
           sun_info.maplength=image->colors*3;
@@ -775,10 +777,9 @@ static unsigned int WriteSUNImage(const ImageInfo *image_info,Image *image)
           Allocate memory for pixels.
         */
         length=image->columns*sizeof(PixelPacket);
-        pixels=(unsigned char *) AcquireMemory(length);
+        pixels=(unsigned char *) AcquireMagickMemory(length);
         if (pixels == (unsigned char *) NULL)
-          ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",
-            image);
+          ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
         /*
           Convert DirectClass packet to SUN RGB pixel.
         */
@@ -790,7 +791,7 @@ static unsigned int WriteSUNImage(const ImageInfo *image_info,Image *image)
           q=pixels;
           for (x=0; x < (long) image->columns; x++)
           {
-            if (image->matte)
+            if (image->matte != MagickFalse)
               *q++=ScaleQuantumToChar(MaxRGB-p->opacity);
             *q++=ScaleQuantumToChar(p->red);
             *q++=ScaleQuantumToChar(p->green);
@@ -799,13 +800,18 @@ static unsigned int WriteSUNImage(const ImageInfo *image_info,Image *image)
           }
           if (image->columns & 0x01)
             *q++=0;  /* pad scanline */
-          (void) WriteBlob(image,q-pixels,(char *) pixels);
+          (void) WriteBlob(image,(size_t) (q-pixels),pixels);
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows))
-              if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
-                break;
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(y,image->rows) != MagickFalse))
+              {
+                status=image->progress_monitor(SaveImageTag,y,image->rows,
+                  image->client_data);
+                if (status == MagickFalse)
+                  break;
+              }
         }
-        LiberateMemory((void **) &pixels);
+        pixels=(unsigned char *) RelinquishMagickMemory(pixels);
       }
     else
       if (IsMonochromeImage(image,&image->exception))
@@ -818,7 +824,7 @@ static unsigned int WriteSUNImage(const ImageInfo *image_info,Image *image)
           /*
             Convert PseudoClass image to a SUN monochrome image.
           */
-          SetImageType(image,BilevelType);
+          (void) SetImageType(image,BilevelType);
           polarity=PixelIntensityToQuantum(&image->colormap[0]) < (MaxRGB/2);
           if (image->colors == 2)
             polarity=PixelIntensityToQuantum(&image->colormap[0]) <
@@ -834,7 +840,7 @@ static unsigned int WriteSUNImage(const ImageInfo *image_info,Image *image)
             for (x=0; x < (long) image->columns; x++)
             {
               byte<<=1;
-              if (indexes[x] == polarity)
+              if (indexes[x] != polarity)
                 byte|=0x01;
               bit++;
               if (bit == 8)
@@ -851,9 +857,14 @@ static unsigned int WriteSUNImage(const ImageInfo *image_info,Image *image)
                 (image->columns % 8 ? 1 : 0)) % 2) != 0)
               (void) WriteBlobByte(image,0);  /* pad scanline */
             if (image->previous == (Image *) NULL)
-              if (QuantumTick(y,image->rows))
-                if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
-                  break;
+              if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                  (QuantumTick(y,image->rows) != MagickFalse))
+                {
+                  status=image->progress_monitor(SaveImageTag,y,image->rows,
+                    image->client_data);
+                  if (status == MagickFalse)
+                    break;
+                }
           }
         }
       else
@@ -865,9 +876,11 @@ static unsigned int WriteSUNImage(const ImageInfo *image_info,Image *image)
             (void) WriteBlobByte(image,
               ScaleQuantumToChar(image->colormap[i].red));
           for (i=0; i < (long) image->colors; i++)
-            (void) WriteBlobByte(image,ScaleQuantumToChar(image->colormap[i].green));
+            (void) WriteBlobByte(image,
+              ScaleQuantumToChar(image->colormap[i].green));
           for (i=0; i < (long) image->colors; i++)
-            (void) WriteBlobByte(image,ScaleQuantumToChar(image->colormap[i].blue));
+            (void) WriteBlobByte(image,
+              ScaleQuantumToChar(image->colormap[i].blue));
           /*
             Convert PseudoClass packet to SUN colormapped pixel.
           */
@@ -879,28 +892,34 @@ static unsigned int WriteSUNImage(const ImageInfo *image_info,Image *image)
             indexes=GetIndexes(image);
             for (x=0; x < (long) image->columns; x++)
             {
-              (void) WriteBlobByte(image,indexes[x]);
+              (void) WriteBlobByte(image,(unsigned char) indexes[x]);
               p++;
             }
             if (image->columns & 0x01)
               (void) WriteBlobByte(image,0);  /* pad scanline */
             if (image->previous == (Image *) NULL)
-              if (QuantumTick(y,image->rows))
-                if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
-                  break;
+              if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                  (QuantumTick(y,image->rows) != MagickFalse))
+                {
+                  status=image->progress_monitor(SaveImageTag,y,image->rows,
+                    image->client_data);
+                  if (status == MagickFalse)
+                    break;
+                }
           }
         }
     if (image->next == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
-      &image->exception);
-    if (status == False)
-      break;
-  } while (image_info->adjoin);
-  if (image_info->adjoin)
-    while (image->previous != (Image *) NULL)
-      image=image->previous;
+    if (image->progress_monitor != (MagickProgressMonitor) NULL)
+      {
+        status=image->progress_monitor(SaveImagesTag,scene,
+          GetImageListLength(image),image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
+    scene++;
+  } while (image_info->adjoin != MagickFalse);
   CloseBlob(image);
-  return(True);
+  return(MagickTrue);
 }
