@@ -10,39 +10,26 @@
 %                              T     GGG   A   A                              %
 %                                                                             %
 %                                                                             %
-%                    Read/Write ImageMagick Image Format.                     %
-%                                                                             %
+%                    Read/Write Truevision Targa Image Format.                %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -55,17 +42,24 @@
 #include "magick/studio.h"
 #include "magick/attribute.h"
 #include "magick/blob.h"
-#include "magick/error.h"
+#include "magick/blob_private.h"
+#include "magick/color_private.h"
+#include "magick/colorspace.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
+#include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
+#include "magick/memory_.h"
 #include "magick/monitor.h"
 #include "magick/static.h"
-#include "magick/utility.h"
+#include "magick/string_.h"
 
 /*
   Forward declarations.
 */
-static unsigned int
+static MagickBooleanType
   WriteTGAImage(const ImageInfo *,Image *);
 
 /*
@@ -79,7 +73,7 @@ static unsigned int
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ReadTGAImage reads a Truevision TGA image file and returns it.
+%  ReadTGAImage() reads a Truevision TGA image file and returns it.
 %  It allocates the memory necessary for the new Image structure and returns
 %  a pointer to the new image.
 %
@@ -89,10 +83,6 @@ static unsigned int
 %
 %  A description of each parameter follows:
 %
-%    o image:  Method ReadTGAImage returns a pointer to the image after
-%      reading.  A null image is returned if there is a memory shortage or
-%      if the image cannot be read.
-%
 %    o image_info: The image info.
 %
 %    o exception: return any errors or warnings in this structure.
@@ -142,6 +132,9 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
   long
     y;
 
+  MagickBooleanType
+    status;
+
   PixelPacket
     pixel;
 
@@ -155,7 +148,7 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
   register PixelPacket
     *q;
 
-  size_t
+  ssize_t
     count;
 
   TGAInfo
@@ -166,9 +159,6 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
     k,
     runlength;
 
-  unsigned int
-    status;
-
   unsigned long
     base,
     flag,
@@ -181,45 +171,81 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
+  if (image_info->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-  if (status == False)
-    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
+  if (status == MagickFalse)
+    {
+      image=DestroyImageList(image);
+      return((Image *) NULL);
+    }
   /*
     Read TGA header information.
   */
-  count=ReadBlob(image,1,(char *) &tga_info.id_length);
-  tga_info.colormap_type=ReadBlobByte(image);
-  tga_info.image_type=ReadBlobByte(image);
+  count=ReadBlob(image,1,&tga_info.id_length);
+  tga_info.colormap_type=(unsigned char) ReadBlobByte(image);
+  tga_info.image_type=(unsigned char) ReadBlobByte(image);
   do
   {
-    if ((count == 0) || (tga_info.image_type == 0) ||
-        (tga_info.image_type > 11))
-      ThrowReaderException(CorruptImageError,"NotATGAImageFile",image);
+    if ((count == 0) ||
+        ((tga_info.image_type != TGAColormap) &&
+         (tga_info.image_type != TGARGB) &&
+         (tga_info.image_type != TGAMonochrome) &&
+         (tga_info.image_type != TGARLEColormap) &&
+         (tga_info.image_type != TGARLERGB) &&
+         (tga_info.image_type != TGARLEMonochrome)) ||
+        (((tga_info.image_type == TGAColormap) ||
+         (tga_info.image_type == TGARLEColormap)) &&
+         (tga_info.colormap_type == 0)))
+      ThrowReaderException(CorruptImageError,"ImproperImageHeader");
     tga_info.colormap_index=ReadBlobLSBShort(image);
     tga_info.colormap_length=ReadBlobLSBShort(image);
-    tga_info.colormap_size=ReadBlobByte(image);
+    tga_info.colormap_size=(unsigned char) ReadBlobByte(image);
     tga_info.x_origin=ReadBlobLSBShort(image);
     tga_info.y_origin=ReadBlobLSBShort(image);
     tga_info.width=ReadBlobLSBShort(image);
     tga_info.height=ReadBlobLSBShort(image);
-    tga_info.bits_per_pixel=ReadBlobByte(image);
-    tga_info.attributes=ReadBlobByte(image);
+    tga_info.bits_per_pixel=(unsigned char) ReadBlobByte(image);
+    tga_info.attributes=(unsigned char) ReadBlobByte(image);
     /*
       Initialize image structure.
     */
-    image->matte=tga_info.bits_per_pixel == 32;
+    image->matte=tga_info.attributes & 0x0FU ? MagickTrue : MagickFalse;
     image->columns=tga_info.width;
     image->rows=tga_info.height;
-    image->depth=tga_info.bits_per_pixel <= 8 ? 8 : QuantumDepth;
-    if (tga_info.colormap_type != 0)
+    if ((tga_info.image_type != TGAColormap) &&
+        (tga_info.image_type != TGARLEColormap))
+      image->depth=(unsigned long) ((tga_info.bits_per_pixel <= 8) ? 8 :
+        (tga_info.bits_per_pixel <= 16) ? 5 :
+        (tga_info.bits_per_pixel == 24) ? 8 :
+        (tga_info.bits_per_pixel == 32) ? 8 : 8);
+    else
+      image->depth=(unsigned long) ((tga_info.colormap_size <= 8) ? 8 :
+        (tga_info.colormap_size <= 16) ? 5 :
+        (tga_info.colormap_size == 24) ? 8 :
+        (tga_info.colormap_size == 32) ? 8 : 8);
+    if ((tga_info.image_type == TGAColormap) ||
+        (tga_info.image_type == TGAMonochrome) ||
+        (tga_info.image_type == TGARLEColormap) ||
+        (tga_info.image_type == TGARLEMonochrome))
+      image->storage_class=PseudoClass;
+    image->compression=NoCompression;
+    if ((tga_info.image_type == TGARLEColormap) ||
+        (tga_info.image_type == TGARLEMonochrome))
+      image->compression=RLECompression;
+    if (image->storage_class == PseudoClass)
       {
-        if ((tga_info.image_type == TGARLEColormap) ||
-            (tga_info.image_type == TGARLERGB))
-          image->storage_class=PseudoClass;
-        image->colors=tga_info.colormap_length;
+        if (tga_info.colormap_type != 0)
+          image->colors=tga_info.colormap_length;
+        else
+          {
+            image->colors=0x01U << tga_info.bits_per_pixel;
+            if (AllocateImageColormap(image,image->colors) == MagickFalse)
+              ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+          }
       }
     if (tga_info.id_length != 0)
       {
@@ -229,25 +255,23 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
         /*
           TGA image comment.
         */
-        comment=(char *) AcquireMemory(tga_info.id_length+1);
+        comment=(char *) AcquireMagickMemory(tga_info.id_length+MaxTextExtent);
         if (comment == (char *) NULL)
-          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",
-            image);
-        (void) ReadBlob(image,tga_info.id_length,comment);
+          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+        (void) ReadBlob(image,tga_info.id_length,(unsigned char *) comment);
         comment[tga_info.id_length]='\0';
-        (void) SetImageAttribute(image,"comment",comment);
-        LiberateMemory((void **) &comment);
+        (void) SetImageAttribute(image,"Comment",comment);
+        comment=(char *) RelinquishMagickMemory(comment);
       }
-    (void) memset(&pixel,0,sizeof(PixelPacket));
+    (void) ResetMagickMemory(&pixel,0,sizeof(pixel));
     pixel.opacity=TransparentOpacity;
     if (tga_info.colormap_type != 0)
       {
         /*
           Read TGA raster colormap.
         */
-        if (!AllocateImageColormap(image,image->colors))
-          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",
-            image);
+        if (AllocateImageColormap(image,image->colors) == MagickFalse)
+          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
         for (i=0; i < (long) image->colors; i++)
         {
           switch (tga_info.colormap_size)
@@ -269,14 +293,12 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
               /*
                 5 bits each of red green and blue.
               */
-              j=ReadBlobByte(image);
-              k=ReadBlobByte(image);
-              pixel.red=(Quantum)
-                (((double) MaxRGB*((int) (k & 0x7c) >> 2))/31+0.5);
-              pixel.green=(Quantum) ((MaxRGB*
-                (((int) (k & 0x03) << 3)+((int) (j & 0xe0) >> 5)))/31+0.5);
-              pixel.blue=(Quantum)
-                (((double) MaxRGB*((int) (j & 0x1f)))/31+0.5);
+              j=(unsigned char) ReadBlobByte(image);
+              k=(unsigned char) ReadBlobByte(image);
+              pixel.red=ScaleXToQuantum((int) (k & 0x7c) >> 2,31);
+              pixel.green=ScaleXToQuantum(((int) (k & 0x03) << 3)+
+                ((int) (j & 0xe0) >> 5),31);
+              pixel.blue=ScaleXToQuantum((int) (j & 0x1f),31);
               break;
             }
             case 24:
@@ -294,7 +316,7 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
           image->colormap[i]=pixel;
         }
       }
-    if (image_info->ping && (image_info->number_scenes != 0))
+    if ((image_info->ping != MagickFalse) && (image_info->number_scenes != 0))
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
     /*
@@ -302,7 +324,7 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
     */
     base=0;
     flag=0;
-    skip=False;
+    skip=MagickFalse;
     real=0;
     index=0;
     runlength=0;
@@ -329,17 +351,17 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
               }
             else
               {
-                count=ReadBlob(image,1,(char *) &runlength);
+                count=ReadBlob(image,1,&runlength);
                 if (count == 0)
                   ThrowReaderException(CorruptImageError,
-                    "UnableToReadImageData",image);
+                    "UnableToReadImageData");
                 flag=runlength & 0x80;
                 if (flag != 0)
                   runlength-=128;
-                skip=False;
+                skip=MagickFalse;
               }
           }
-        if (!skip)
+        if (skip == MagickFalse)
           switch (tga_info.bits_per_pixel)
           {
             case 8:
@@ -348,7 +370,7 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
               /*
                 Gray scale.
               */
-              index=ReadBlobByte(image);
+              index=(IndexPacket) ReadBlobByte(image);
               if (tga_info.colormap_type != 0)
                 pixel=image->colormap[ConstrainColormapIndex(image,index)];
               else
@@ -365,16 +387,14 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
               /*
                 5 bits each of red green and blue.
               */
-              j=ReadBlobByte(image);
-              k=ReadBlobByte(image);
-              pixel.red=(Quantum)
-                ((double) (MaxRGB*((int) (k & 0x7c) >> 2))/31+0.5);
-              pixel.green=(Quantum) (((double) MaxRGB*
-                (((int) (k & 0x03) << 3)+((int) (j & 0xe0) >> 5)))/31+0.5);
-              pixel.blue=(Quantum)
-                (((double) MaxRGB*((int) (j & 0x1f)))/31+0.5);
+              j=(unsigned char) ReadBlobByte(image);
+              k=(unsigned char) ReadBlobByte(image);
+              pixel.red=ScaleXToQuantum((int) (k & 0x7c) >> 2,31);
+              pixel.red=ScaleXToQuantum(((int) (k & 0x03) << 3)+
+                ((int) (j & 0xe0) >> 5),31);
+              pixel.blue=ScaleXToQuantum((int) (j & 0x1f),31);
               if (image->storage_class == PseudoClass)
-                index=ConstrainColormapIndex(image,((unsigned short) k << 8)+j);
+                index=ConstrainColormapIndex(image,((unsigned long) k << 8)+j);
               break;
             }
             case 24:
@@ -391,8 +411,8 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
               break;
             }
           }
-        if (status == False)
-          ThrowReaderException(CorruptImageError,"UnableToReadImageData",image);
+        if (status == MagickFalse)
+          ThrowReaderException(CorruptImageError,"UnableToReadImageData");
         if (image->storage_class == PseudoClass)
           indexes[x]=index;
         *q++=pixel;
@@ -409,16 +429,21 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
           base++;
           offset=base;
         }
-      if (!SyncImagePixels(image))
+      if (SyncImagePixels(image) == MagickFalse)
         break;
       if (image->previous == (Image *) NULL)
-        if (QuantumTick(y,image->rows))
-          if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
-            break;
+        if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+            (QuantumTick(y,image->rows) != MagickFalse))
+          {
+            status=image->progress_monitor(LoadImageTag,y,image->rows,
+              image->client_data);
+            if (status == MagickFalse)
+              break;
+          }
     }
-    if (EOFBlob(image))
+    if (EOFBlob(image) != MagickFalse)
       {
-        (void) ThrowException(exception,CorruptImageError,"UnexpectedEndOfFile",
+        ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
           image->filename);
         break;
       }
@@ -428,11 +453,16 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
     if (image_info->number_scenes != 0)
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
-    count=ReadBlob(image,1,(char *) &tga_info.id_length);
-    tga_info.colormap_type=ReadBlobByte(image);
-    tga_info.image_type=ReadBlobByte(image);
-    status=((tga_info.image_type != 0) && (tga_info.image_type <= 11));
-    if (status == True)
+    count=ReadBlob(image,1,&tga_info.id_length);
+    tga_info.colormap_type=(unsigned char) ReadBlobByte(image);
+    tga_info.image_type=(unsigned char) ReadBlobByte(image);
+    status=((tga_info.image_type == TGAColormap) ||
+      (tga_info.image_type == TGARGB) ||
+      (tga_info.image_type == TGAMonochrome) ||
+      (tga_info.image_type == TGARLEColormap) ||
+      (tga_info.image_type == TGARLERGB) ||
+      (tga_info.image_type == TGARLEMonochrome)) ? MagickTrue : MagickFalse;
+    if (status == MagickTrue)
       {
         /*
           Allocate next image structure.
@@ -440,20 +470,21 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
         AllocateNextImage(image_info,image);
         if (image->next == (Image *) NULL)
           {
-            DestroyImageList(image);
+            image=DestroyImageList(image);
             return((Image *) NULL);
           }
         image=SyncNextImageInList(image);
-        status=MagickMonitor(LoadImagesTag,TellBlob(image),GetBlobSize(image),
-          exception);
-        if (status == False)
-          break;
+        if (image->progress_monitor != (MagickProgressMonitor) NULL)
+          {
+            status=image->progress_monitor(LoadImagesTag,TellBlob(image),
+              GetBlobSize(image),image->client_data);
+            if (status == MagickFalse)
+              break;
+          }
       }
-  } while (status == True);
-  while (image->previous != (Image *) NULL)
-    image=image->previous;
+  } while (status == MagickTrue);
   CloseBlob(image);
-  return(image);
+  return(GetFirstImageInList(image));
 }
 
 /*
@@ -467,7 +498,7 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method RegisterTGAImage adds attributes for the TGA image format to
+%  RegisterTGAImage() adds attributes for the TGA image format to
 %  the list of supported formats.  The attributes include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
@@ -485,26 +516,26 @@ ModuleExport void RegisterTGAImage(void)
     *entry;
 
   entry=SetMagickInfo("ICB");
-  entry->decoder=(DecoderHandler) ReadTGAImage;
-  entry->encoder=(EncoderHandler) WriteTGAImage;
+  entry->decoder=(DecoderHandler *) ReadTGAImage;
+  entry->encoder=(EncoderHandler *) WriteTGAImage;
   entry->description=AcquireString("Truevision Targa image");
   entry->module=AcquireString("TGA");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("TGA");
-  entry->decoder=(DecoderHandler) ReadTGAImage;
-  entry->encoder=(EncoderHandler) WriteTGAImage;
+  entry->decoder=(DecoderHandler *) ReadTGAImage;
+  entry->encoder=(EncoderHandler *) WriteTGAImage;
   entry->description=AcquireString("Truevision Targa image");
   entry->module=AcquireString("TGA");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("VDA");
-  entry->decoder=(DecoderHandler) ReadTGAImage;
-  entry->encoder=(EncoderHandler) WriteTGAImage;
+  entry->decoder=(DecoderHandler *) ReadTGAImage;
+  entry->encoder=(EncoderHandler *) WriteTGAImage;
   entry->description=AcquireString("Truevision Targa image");
   entry->module=AcquireString("TGA");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("VST");
-  entry->decoder=(DecoderHandler) ReadTGAImage;
-  entry->encoder=(EncoderHandler) WriteTGAImage;
+  entry->decoder=(DecoderHandler *) ReadTGAImage;
+  entry->encoder=(EncoderHandler *) WriteTGAImage;
   entry->description=AcquireString("Truevision Targa image");
   entry->module=AcquireString("TGA");
   (void) RegisterMagickInfo(entry);
@@ -521,7 +552,7 @@ ModuleExport void RegisterTGAImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method UnregisterTGAImage removes format registrations made by the
+%  UnregisterTGAImage() removes format registrations made by the
 %  TGA module from the list of supported formats.
 %
 %  The format of the UnregisterTGAImage method is:
@@ -548,26 +579,22 @@ ModuleExport void UnregisterTGAImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method WriteTGAImage writes a image in the Truevision Targa rasterfile
+%  WriteTGAImage() writes a image in the Truevision Targa rasterfile
 %  format.
 %
 %  The format of the WriteTGAImage method is:
 %
-%      unsigned int WriteTGAImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WriteTGAImage(const ImageInfo *image_info,Image *image)
 %
 %  A description of each parameter follows.
 %
-%    o status: Method WriteTGAImage return True if the image is written.
-%      False is returned is there is a memory shortage or if the image file
-%      fails to write.
-%
 %    o image_info: The image info.
 %
 %    o image:  The image.
 %
 %
 */
-static unsigned int WriteTGAImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WriteTGAImage(const ImageInfo *image_info,Image *image)
 {
 #define TargaColormap 1
 #define TargaRGB 2
@@ -605,9 +632,14 @@ static unsigned int WriteTGAImage(const ImageInfo *image_info,Image *image)
     *attribute;
 
   long
-    count,
     y;
 
+  MagickBooleanType
+    status;
+
+  MagickOffsetType
+    scene;
+
   register const PixelPacket
     *p;
 
@@ -623,18 +655,15 @@ static unsigned int WriteTGAImage(const ImageInfo *image_info,Image *image)
   register unsigned char
     *q;
 
+  ssize_t
+    count;
+
   TargaInfo
     targa_info;
 
   unsigned char
     *targa_pixels;
 
-  unsigned int
-    status;
-
-  unsigned long
-    scene;
-
   /*
     Open output image file.
   */
@@ -642,9 +671,11 @@ static unsigned int WriteTGAImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
-  if (status == False)
-    ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
+  if (status == MagickFalse)
+    return(status);
   scene=0;
   do
   {
@@ -653,9 +684,9 @@ static unsigned int WriteTGAImage(const ImageInfo *image_info,Image *image)
     */
     (void) SetImageColorspace(image,RGBColorspace);
     targa_info.id_length=0;
-    attribute=GetImageAttribute(image,"comment");
+    attribute=GetImageAttribute(image,"Comment");
     if (attribute != (const ImageAttribute *) NULL)
-      targa_info.id_length=Min(strlen(attribute->value),255);
+      targa_info.id_length=(unsigned char) Min(strlen(attribute->value),255);
     targa_info.colormap_type=0;
     targa_info.colormap_index=0;
     targa_info.colormap_length=0;
@@ -666,30 +697,33 @@ static unsigned int WriteTGAImage(const ImageInfo *image_info,Image *image)
     targa_info.height=(unsigned short) image->rows;
     targa_info.bits_per_pixel=8;
     targa_info.attributes=0;
-    if ((image->storage_class == DirectClass) || (image->colors > 256))
-      {
-        /*
-          Full color TGA raster.
-        */
-        targa_info.image_type=TargaRGB;
-        targa_info.bits_per_pixel=24;
-        if (image->matte)
-          {
-            targa_info.bits_per_pixel=32;
-            targa_info.attributes=8;  /* # of alpha bits */
-          }
-      }
+    if ((image_info->type != TrueColorType) &&
+		    (IsGrayImage(image,&image->exception) != MagickFalse))
+      targa_info.image_type=TargaMonochrome;
     else
-      {
-        /*
-          Colormapped TGA raster.
-        */
-        targa_info.image_type=TargaColormap;
-        targa_info.colormap_type=1;
-        targa_info.colormap_index=0;
-        targa_info.colormap_length=(unsigned short) image->colors;
-        targa_info.colormap_size=24;
-      }
+      if ((image->storage_class == DirectClass) || (image->colors > 256))
+        {
+          /*
+            Full color TGA raster.
+          */
+          targa_info.image_type=TargaRGB;
+          targa_info.bits_per_pixel=24;
+          if (image->matte != MagickFalse)
+            {
+              targa_info.bits_per_pixel=32;
+              targa_info.attributes=8;  /* # of alpha bits */
+            }
+        }
+      else
+        {
+          /*
+            Colormapped TGA raster.
+          */
+          targa_info.image_type=TargaColormap;
+          targa_info.colormap_type=1;
+          targa_info.colormap_length=(unsigned short) image->colors;
+          targa_info.colormap_size=24;
+        }
     /*
       Write TGA header.
     */
@@ -706,7 +740,8 @@ static unsigned int WriteTGAImage(const ImageInfo *image_info,Image *image)
     (void) WriteBlobByte(image,targa_info.bits_per_pixel);
     (void) WriteBlobByte(image,targa_info.attributes);
     if (targa_info.id_length != 0)
-      (void) WriteBlob(image,targa_info.id_length,attribute->value);
+      (void) WriteBlob(image,targa_info.id_length,(unsigned char *)
+        attribute->value);
     if (targa_info.image_type == TargaColormap)
       {
         unsigned char
@@ -716,10 +751,9 @@ static unsigned int WriteTGAImage(const ImageInfo *image_info,Image *image)
           Dump colormap to file (blue, green, red byte order).
         */
         targa_colormap=(unsigned char *)
-          AcquireMemory(3*targa_info.colormap_length);
+          AcquireMagickMemory(3*targa_info.colormap_length);
         if (targa_colormap == (unsigned char *) NULL)
-          ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",
-            image);
+          ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
         q=targa_colormap;
         for (i=0; i < (long) image->colors; i++)
         {
@@ -727,17 +761,16 @@ static unsigned int WriteTGAImage(const ImageInfo *image_info,Image *image)
           *q++=ScaleQuantumToChar(image->colormap[i].green);
           *q++=ScaleQuantumToChar(image->colormap[i].red);
         }
-        (void) WriteBlob(image,3*targa_info.colormap_length,
-          (char *) targa_colormap);
-        LiberateMemory((void **) &targa_colormap);
+        (void) WriteBlob(image,3*targa_info.colormap_length,targa_colormap);
+        targa_colormap=(unsigned char *) RelinquishMagickMemory(targa_colormap);
       }
     /*
       Convert MIFF to TGA raster pixels.
     */
-    count=(long) ((targa_info.bits_per_pixel*targa_info.width) >> 3);
-    targa_pixels=(unsigned char *) AcquireMemory(count);
+    count=(ssize_t) ((targa_info.bits_per_pixel*targa_info.width) >> 3);
+    targa_pixels=(unsigned char *) AcquireMagickMemory((size_t) count);
     if (targa_pixels == (unsigned char *) NULL)
-      ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",image);
+      ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
     for (y=(long) (image->rows-1); y >= 0; y--)
     {
       p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
@@ -748,38 +781,46 @@ static unsigned int WriteTGAImage(const ImageInfo *image_info,Image *image)
       for (x=0; x < (long) image->columns; x++)
       {
         if (targa_info.image_type == TargaColormap)
-          *q++=indexes[x];
+          *q++=(unsigned char) indexes[x];
         else
-          {
-            *q++=ScaleQuantumToChar(p->blue);
-            *q++=ScaleQuantumToChar(p->green);
-            *q++=ScaleQuantumToChar(p->red);
-            if (image->colorspace == CMYKColorspace)
-              *q++=ScaleQuantumToChar(p->opacity);
-            else
-              if (image->matte)
-                *q++=ScaleQuantumToChar(MaxRGB-p->opacity);
-          }
+          if (targa_info.image_type == TargaMonochrome)
+            *q++=(unsigned char) ScaleQuantumToChar(PixelIntensityToQuantum(p));
+          else
+            {
+              *q++=ScaleQuantumToChar(p->blue);
+              *q++=ScaleQuantumToChar(p->green);
+              *q++=ScaleQuantumToChar(p->red);
+              if (image->matte != MagickFalse)
+                *q++=MaxRGB-ScaleQuantumToChar(p->opacity);
+              if (image->colorspace == CMYKColorspace)
+                *q++=ScaleQuantumToChar(indexes[x]);
+            }
         p++;
       }
-      (void) WriteBlob(image,q-targa_pixels,(char *) targa_pixels);
+      (void) WriteBlob(image,(size_t) (q-targa_pixels),targa_pixels);
       if (image->previous == (Image *) NULL)
-        if (QuantumTick(y,image->rows))
-          if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
-            break;
+        if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+            (QuantumTick(y,image->rows) != MagickFalse))
+          {
+            status=image->progress_monitor(SaveImageTag,y,image->rows,
+              image->client_data);
+            if (status == MagickFalse)
+              break;
+          }
     }
-    LiberateMemory((void **) &targa_pixels);
+    targa_pixels=(unsigned char *) RelinquishMagickMemory(targa_pixels);
     if (image->next == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
-      &image->exception);
-    if (status == False)
-      break;
-  } while (image_info->adjoin);
-  if (image_info->adjoin)
-    while (image->previous != (Image *) NULL)
-      image=image->previous;
+    if (image->progress_monitor != (MagickProgressMonitor) NULL)
+      {
+        status=image->progress_monitor(SaveImagesTag,scene,
+          GetImageListLength(image),image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
+    scene++;
+  } while (image_info->adjoin != MagickFalse);
   CloseBlob(image);
-  return(True);
+  return(MagickTrue);
 }
