@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -41,9 +41,12 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
 #include "magick/constitute.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
@@ -102,7 +105,7 @@ static Image *ReadTILEImage(const ImageInfo *image_info,
   read_info->length=0;
   *read_info->magick='\0';
   tile_image=ReadImage(read_info,exception);
-  DestroyImageInfo(read_info);
+  read_info=DestroyImageInfo(read_info);
   if (tile_image == (Image *) NULL)
     return((Image *) NULL);
   image=AllocateImage(image_info);
@@ -110,9 +113,11 @@ static Image *ReadTILEImage(const ImageInfo *image_info,
     ThrowReaderException(OptionError,"MustSpecifyImageSize");
   if (*image_info->filename == '\0')
     ThrowReaderException(OptionError,"MustSpecifyAnImageName");
+  SetImage(image,OpaqueOpacity);
+  image->matte=tile_image->matte;
   (void) CopyMagickString(image->filename,image_info->filename,MaxTextExtent);
   (void) TextureImage(image,tile_image);
-  DestroyImage(tile_image);
+  tile_image=DestroyImage(tile_image);
   return(GetFirstImageInList(image));
 }
 
@@ -147,6 +152,7 @@ ModuleExport void RegisterTILEImage(void)
   entry=SetMagickInfo("TILE");
   entry->decoder=(DecoderHandler *) ReadTILEImage;
   entry->raw=MagickTrue;
+  entry->endian_support=MagickTrue;
   entry->description=AcquireString("Tile image with a texture");
   entry->module=AcquireString("TILE");
   (void) RegisterMagickInfo(entry);
