@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -41,9 +41,12 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
 #include "magick/draw.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
@@ -51,6 +54,16 @@
 #include "magick/string_.h"
 #include "magick/type.h"
 #include "wand/magick_wand.h"
+#if defined(HasTTF)
+#if defined(HAVE_FT2BUILD_H)
+#  include <ft2build.h>
+#endif
+#if defined(FT_FREETYPE_H)
+#  include FT_FREETYPE_H
+#else
+#  include <freetype/freetype.h>
+#endif
+#endif
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -63,8 +76,8 @@
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  IsPFA()() returns MagickTrue if the image format type, identified by the magick
-%  string, is PFA.
+%  IsPFA()() returns MagickTrue if the image format type, identified by the
+%  magick string, is PFA.
 %
 %  The format of the IsPFA method is:
 %
@@ -72,9 +85,6 @@
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method IsPFA() returns MagickTrue if the image format type can be
-%               handled by this module.
-%
 %    o magick: This string is generally the first few bytes of an image file
 %      or blob.
 %
@@ -102,8 +112,8 @@ static MagickBooleanType IsPFA(const unsigned char *magick,const size_t length)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  IsTTF()() returns MagickTrue if the image format type, identified by the magick
-%  string, is TTF.
+%  IsTTF()() returns MagickTrue if the image format type, identified by the
+%  magick string, is TTF.
 %
 %  The format of the IsTTF method is:
 %
@@ -111,9 +121,6 @@ static MagickBooleanType IsPFA(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method IsTTF() returns MagickTrue if the image format type can be
-%               handled by this module.
-%
 %    o magick: This string is generally the first few bytes of an image file
 %      or blob.
 %
@@ -154,10 +161,6 @@ static MagickBooleanType IsTTF(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o image:  Method ReadTTFImage returns a pointer to the image after
-%      reading.  A null image is returned if there is a memory shortage or
-%      if the image cannot be read.
-%
 %    o image_info: The image info.
 %
 %    o exception: return any errors or warnings in this structure.
@@ -220,7 +223,7 @@ static Image *ReadTTFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
-      DestroyImageList(image);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
   /*
@@ -276,7 +279,7 @@ static Image *ReadTTFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   /*
     Free resources.
   */
-  DestroyDrawInfo(draw_info);
+  draw_info=DestroyDrawInfo(draw_info);
   CloseBlob(image);
   return(GetFirstImageInList(image));
 }
@@ -314,9 +317,9 @@ ModuleExport void RegisterTTFImage(void)
     *entry;
 
   *version='\0';
-#if defined(FREETYPE_MAJOR) && defined(FREETYPE_MINOR)
-  (void) FormatMagickString(version,MaxTextExtent,"%d.%d",
-    FREETYPE_MAJOR,FREETYPE_MINOR);
+#if defined(FREETYPE_MAJOR) && defined(FREETYPE_MINOR) && defined(FREETYPE_PATCH)
+  (void) FormatMagickString(version,MaxTextExtent,"Freetype %d.%d.%d",
+    FREETYPE_MAJOR,FREETYPE_MINOR,FREETYPE_PATCH);
 #endif
   entry=SetMagickInfo("PFA");
 #if defined(HasTTF)
