@@ -18,7 +18,7 @@
 %                                March 2000                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -42,9 +42,12 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
 #include "magick/constitute.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
@@ -147,12 +150,12 @@ static Image *ReadURLImage(const ImageInfo *image_info,ExceptionInfo *exception)
     file=fdopen(unique_file,"wb");
   if ((unique_file == -1) || (file == (FILE *) NULL))
     {
-      DestroyImageInfo(read_info);
+      read_info=DestroyImageInfo(read_info);
       (void) CopyMagickString(image->filename,read_info->filename,
         MaxTextExtent);
       ThrowFileException(exception,FileOpenError,"UnableToCreateTemporaryFile",
         image->filename);
-      DestroyImageList(image);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
   (void) CopyMagickString(filename,image_info->magick,MaxTextExtent);
@@ -168,7 +171,8 @@ static Image *ReadURLImage(const ImageInfo *image_info,ExceptionInfo *exception)
         bytes;
 
       type=(char *) NULL;
-      context=xmlNanoHTTPOpen(filename,&type);
+      context=xmlNanoHTTPMethod(filename,(const char *) NULL,
+        (const char *) NULL,&type,(const char *) NULL,0);
       if (context != (void *) NULL)
         {
           while ((bytes=xmlNanoHTTPRead(context,buffer,MaxBufferExtent)) > 0)
@@ -194,7 +198,7 @@ static Image *ReadURLImage(const ImageInfo *image_info,ExceptionInfo *exception)
   *read_info->magick='\0';
   image=ReadImage(read_info,exception);
   (void) RelinquishUniqueFileResource(read_info->filename);
-  DestroyImageInfo(read_info);
+  read_info=DestroyImageInfo(read_info);
   if (image == (Image *) NULL)
     (void) ThrowMagickException(exception,GetMagickModule(),CoderError,
       "NoDataReturned",filename);
