@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -41,10 +41,13 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
 #include "magick/color.h"
 #include "magick/colorspace.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
@@ -162,9 +165,14 @@ static Image *ReadUYVYImage(const ImageInfo *image_info,
     }
     if (SyncImagePixels(image) == MagickFalse)
       break;
-    if (QuantumTick(y,image->rows) != MagickFalse)
-      if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
-        break;
+    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+        (QuantumTick(y,image->rows) != MagickFalse))
+      {
+        status=image->progress_monitor(LoadImageTag,y,image->rows,
+          image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
   }
   image->colorspace=YCbCrColorspace;
   (void) SetImageColorspace(image,RGBColorspace);
@@ -208,6 +216,7 @@ ModuleExport void RegisterUYVYImage(void)
   entry->encoder=(EncoderHandler *) WriteUYVYImage;
   entry->adjoin=MagickFalse;
   entry->raw=MagickTrue;
+  entry->endian_support=MagickTrue;
   entry->description=AcquireString("16bit/pixel interleaved YUV");
   entry->module=AcquireString("UYVY");
   (void) RegisterMagickInfo(entry);
@@ -216,6 +225,7 @@ ModuleExport void RegisterUYVYImage(void)
   entry->encoder=(EncoderHandler *) WriteUYVYImage;
   entry->adjoin=MagickFalse;
   entry->raw=MagickTrue;
+  entry->endian_support=MagickTrue;
   entry->description=AcquireString("16bit/pixel interleaved YUV");
   entry->module=AcquireString("UYVY");
   (void) RegisterMagickInfo(entry);
@@ -339,11 +349,16 @@ static MagickBooleanType WriteUYVYImage(const ImageInfo *image_info,
       full=(MagickBooleanType) (full == MagickFalse);
       p++;
     }
-    if (QuantumTick(y,image->rows) != MagickFalse)
-      if (MagickMonitor(SaveImageTag,y,image->rows,&image->exception) == MagickFalse)
-        break;
+    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+        (QuantumTick(y,image->rows) != MagickFalse))
+      {
+        status=image->progress_monitor(SaveImageTag,y,image->rows,
+          image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
   }
-  DestroyImage(uyvy_image);
+  uyvy_image=DestroyImage(uyvy_image);
   CloseBlob(image);
   return(MagickTrue);
 }
