@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -41,14 +41,18 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
 #include "magick/colorspace.h"
 #include "magick/constitute.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/pixel.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 
@@ -173,7 +177,7 @@ static Image *ReadVICARImage(const ImageInfo *image_info,
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
-      DestroyImageList(image);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
   /*
@@ -183,7 +187,7 @@ static Image *ReadVICARImage(const ImageInfo *image_info,
   count=1;
   if (c == EOF)
     {
-      DestroyImage(image);
+      image=DestroyImage(image);
       return((Image *) NULL);
     }
   length=0;
@@ -281,12 +285,17 @@ static Image *ReadVICARImage(const ImageInfo *image_info,
     if (q == (PixelPacket *) NULL)
       break;
     (void) ReadBlob(image,image->columns,scanline);
-    (void) PushImagePixels(image,GrayQuantum,scanline);
+    (void) ExportQuantumPixels(image,GrayQuantum,0,scanline);
     if (SyncImagePixels(image) == MagickFalse)
       break;
-    if (QuantumTick(y,image->rows) != MagickFalse)
-      if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
-        break;
+    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+        (QuantumTick(y,image->rows) != MagickFalse))
+      {
+        status=image->progress_monitor(LoadImageTag,y,image->rows,
+          image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
   }
   scanline=(unsigned char *) RelinquishMagickMemory(scanline);
   if (EOFBlob(image) != MagickFalse)
@@ -445,14 +454,16 @@ static MagickBooleanType WriteVICARImage(const ImageInfo *image_info,
   for (y=0; y < (long) image->rows; y++)
   {
     p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
-		if (p == (const PixelPacket *) NULL)
+    if (p == (const PixelPacket *) NULL)
       break;
-    (void) PopImagePixels(image,GrayQuantum,scanline);
+    (void) ImportQuantumPixels(image,GrayQuantum,0,scanline);
     (void) WriteBlob(image,image->columns,scanline);
     if (image->previous == (Image *) NULL)
-      if (QuantumTick(y,image->rows) != MagickFalse)
+      if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+          (QuantumTick(y,image->rows) != MagickFalse))
         {
-          status=MagickMonitor(SaveImageTag,y,image->rows,&image->exception);
+          status=image->progress_monitor(SaveImageTag,y,image->rows,
+            image->client_data);
           if (status == MagickFalse)
             break;
         }
