@@ -10,39 +10,26 @@
 %                         W   W  BBBB   M   M  P                              %
 %                                                                             %
 %                                                                             %
-%                    Read/Write ImageMagick Image Format.                     %
-%                                                                             %
+%               Read/Write Wireless Bitmap (level 0) Image Format.            %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                               January 2000                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -53,17 +40,23 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
-#include "magick/error.h"
+#include "magick/blob_private.h"
+#include "magick/colorspace.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
+#include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
+#include "magick/memory_.h"
 #include "magick/monitor.h"
 #include "magick/static.h"
-#include "magick/utility.h"
+#include "magick/string_.h"
 
 /*
   Forward declarations.
 */
-static unsigned int
+static MagickBooleanType
   WriteWBMPImage(const ImageInfo *,Image *);
 
 /*
@@ -77,7 +70,7 @@ static unsigned int
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ReadWBMPImage reads a WBMP (level 0) image file and returns it.  It
+%  ReadWBMPImage() reads a WBMP (level 0) image file and returns it.  It
 %  allocates the memory necessary for the new Image structure and returns a
 %  pointer to the new image.
 %
@@ -90,10 +83,6 @@ static unsigned int
 %
 %  A description of each parameter follows:
 %
-%    o image:  Method ReadWBMPImage returns a pointer to the image after
-%      reading.  A null image is returned if there is a memory shortage or
-%      if the image cannot be read.
-%
 %    o image_info: The image info.
 %
 %    o exception: return any errors or warnings in this structure.
@@ -101,7 +90,7 @@ static unsigned int
 %
 */
 
-static unsigned int WBMPReadInteger(Image *image,unsigned long *value)
+static MagickBooleanType WBMPReadInteger(Image *image,unsigned long *value)
 {
   int
     byte;
@@ -111,11 +100,11 @@ static unsigned int WBMPReadInteger(Image *image,unsigned long *value)
   {
     byte=ReadBlobByte(image);
     if (byte == EOF)
-      return(False);
+      return(MagickFalse);
     *value<<=7;
     *value|=(unsigned int) (byte & 0x7f);
   } while (byte & 0x80);
-  return(True);
+  return(MagickTrue);
 }
 
 static Image *ReadWBMPImage(const ImageInfo *image_info,
@@ -130,6 +119,9 @@ static Image *ReadWBMPImage(const ImageInfo *image_info,
   long
     y;
 
+  MagickBooleanType
+    status;
+
   register IndexPacket
     *indexes;
 
@@ -145,9 +137,6 @@ static Image *ReadWBMPImage(const ImageInfo *image_info,
   unsigned char
     bit;
 
-  unsigned int
-    status;
-
   unsigned short
     header;
 
@@ -156,33 +145,38 @@ static Image *ReadWBMPImage(const ImageInfo *image_info,
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
+  if (image_info->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-  if (status == False)
-    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
-  if (!ReadBlob(image,2,(char *) &header)) 
-    ThrowReaderException(CorruptImageError,"NotAWBMPImageFile",image);
+  if (status == MagickFalse)
+    {
+      image=DestroyImageList(image);
+      return((Image *) NULL);
+    }
+  if (ReadBlob(image,2,(unsigned char *) &header) == 0)
+    ThrowReaderException(CorruptImageError,"ImproperImageHeader");
   if (header != 0)
-    ThrowReaderException(CoderError,"OnlyLevelZerofilesSupported",image);
+    ThrowReaderException(CoderError,"OnlyLevelZerofilesSupported");
   /*
     Initialize image structure.
   */
-  if (WBMPReadInteger(image,&image->columns) == False) 
-    ThrowReaderException(CorruptImageError,"CorruptWBMPimage",image);
-  if (WBMPReadInteger(image,&image->rows) == False) 
-    ThrowReaderException(CorruptImageError,"CorruptWBMPimage",image);
+  if (WBMPReadInteger(image,&image->columns) == MagickFalse)
+    ThrowReaderException(CorruptImageError,"CorruptWBMPimage");
+  if (WBMPReadInteger(image,&image->rows) == MagickFalse)
+    ThrowReaderException(CorruptImageError,"CorruptWBMPimage");
   if ((image->columns == 0) || (image->rows == 0))
-    ThrowReaderException(CorruptImageError,"NotAWBMPImageFile",image);
+    ThrowReaderException(CorruptImageError,"ImproperImageHeader");
   for (i=0; i < image->offset; i++)
     (void) ReadBlobByte(image);
-  if (!AllocateImageColormap(image,2))
-    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
-  if (image_info->ping)
+  if (AllocateImageColormap(image,2) == MagickFalse)
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+  if (image_info->ping != MagickFalse)
     {
       CloseBlob(image);
-      return(image);
+      return(GetFirstImageInList(image));
     }
   /*
     Convert bi-level image to pixel packets.
@@ -201,25 +195,30 @@ static Image *ReadWBMPImage(const ImageInfo *image_info,
         {
           byte=ReadBlobByte(image);
           if (byte == EOF)
-            ThrowReaderException(CorruptImageError,"CorruptWBMPImage",image);
+            ThrowReaderException(CorruptImageError,"CorruptImage");
         }
       indexes[x]=(byte & (0x01 << (7-bit))) ? 1 : 0;
       bit++;
       if (bit == 8)
         bit=0;
     }
-    if (!SyncImagePixels(image))
+    if (SyncImagePixels(image) == MagickFalse)
       break;
-    if (QuantumTick(y,image->rows))
-      if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
-        break;
+    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+        (QuantumTick(y,image->rows) != MagickFalse))
+      {
+        status=image->progress_monitor(LoadImageTag,y,image->rows,
+          image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
   }
   SyncImage(image);
-  if (EOFBlob(image))
-    (void) ThrowException(exception,CorruptImageError,"UnexpectedEndOfFile",
+  if (EOFBlob(image) != MagickFalse)
+    ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
       image->filename);
   CloseBlob(image);
-  return(image);
+  return(GetFirstImageInList(image));
 }
 
 /*
@@ -233,7 +232,7 @@ static Image *ReadWBMPImage(const ImageInfo *image_info,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method RegisterWBMPImage adds attributes for the WBMP image format to
+%  RegisterWBMPImage() adds attributes for the WBMP image format to
 %  the list of supported formats.  The attributes include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
@@ -251,9 +250,9 @@ ModuleExport void RegisterWBMPImage(void)
     *entry;
 
   entry=SetMagickInfo("WBMP");
-  entry->decoder=(DecoderHandler) ReadWBMPImage;
-  entry->encoder=(EncoderHandler) WriteWBMPImage;
-  entry->adjoin=False;
+  entry->decoder=(DecoderHandler *) ReadWBMPImage;
+  entry->encoder=(EncoderHandler *) WriteWBMPImage;
+  entry->adjoin=MagickFalse;
   entry->description=AcquireString("Wireless Bitmap (level 0) image");
   entry->module=AcquireString("WBMP");
   (void) RegisterMagickInfo(entry);
@@ -270,7 +269,7 @@ ModuleExport void RegisterWBMPImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method UnregisterWBMPImage removes format registrations made by the
+%  UnregisterWBMPImage() removes format registrations made by the
 %  WBMP module from the list of supported formats.
 %
 %  The format of the UnregisterWBMPImage method is:
@@ -294,21 +293,18 @@ ModuleExport void UnregisterWBMPImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method WriteWBMPImage writes an image to a file in the Wireless Bitmap
+%  WriteWBMPImage() writes an image to a file in the Wireless Bitmap
 %  (level 0) image format.
 %
 %  WriteWBMPImage was contributed by Milan Votava <votava@mageo.cz>.
 %
 %  The format of the WriteWBMPImage method is:
 %
-%      unsigned int WriteWBMPImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WriteWBMPImage(const ImageInfo *image_info,
+%        Image *image)
 %
 %  A description of each parameter follows.
 %
-%    o status: Method WriteWBMPImage return True if the image is written.
-%      False is returned is there is a memory shortage or if the image file
-%      fails to write.
-%
 %    o image_info: The image info.
 %
 %    o image:  The image.
@@ -332,26 +328,30 @@ static void WBMPWriteInteger(Image *image,const unsigned long value)
 
   n=1;
   bits=28;
-  flag=False;
+  flag=MagickFalse;
   for (i=4; i >= 0; i--)
   {
     octet=(unsigned char) ((value >> bits) & 0x7f);
-    if (!flag && octet)
+    if ((flag == 0) && (octet != 0))
       {
-        flag=True;
+        flag=MagickTrue;
         n=i+1;
       }
     buffer[4-i]=octet | (i && (flag || octet))*(0x01 << 7);
     bits-=7;
   }
-  (void) WriteBlob(image,n,(char *) buffer+5-n);
+  (void) WriteBlob(image,n,buffer+5-n);
 }
 
-static unsigned int WriteWBMPImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WriteWBMPImage(const ImageInfo *image_info,
+  Image *image)
 {
   long
     y;
 
+  MagickBooleanType
+    status;
+
   register const PixelPacket
     *p;
 
@@ -366,9 +366,6 @@ static unsigned int WriteWBMPImage(const ImageInfo *image_info,Image *image)
     byte,
     polarity;
 
-  unsigned int
-    status;
-
   /*
     Open output image file.
   */
@@ -376,9 +373,11 @@ static unsigned int WriteWBMPImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
-  if (status == False)
-    ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
+  if (status == MagickFalse)
+    return(status);
   (void) SetImageColorspace(image,RGBColorspace);
   /*
     Convert image to a bi-level image.
@@ -413,10 +412,15 @@ static unsigned int WriteWBMPImage(const ImageInfo *image_info,Image *image)
     }
     if (bit != 0)
       (void) WriteBlobByte(image,byte);
-    if (QuantumTick(y,image->rows))
-      if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
-        break;
+    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+        (QuantumTick(y,image->rows) != MagickFalse))
+      {
+        status=image->progress_monitor(SaveImageTag,y,image->rows,
+          image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
   }
   CloseBlob(image);
-  return(True);
+  return(MagickTrue);
 }
