@@ -16,7 +16,7 @@
 %                                 June 2000                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -40,11 +40,14 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
 #include "magick/color_private.h"
 #include "magick/constitute.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/cache.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magic.h"
 #include "magick/magick.h"
@@ -285,19 +288,19 @@ static void InsertRow(unsigned char *p,long y,Image *image, int bpp)
 }
 static int UnpackWPGRaster(Image *image,int bpp)
 {
-  int
-    x,
-    y,
-    i;
+  long
+    ldblk,
+    y;
+
+  register long
+    i,
+    x;
 
   unsigned char
     bbuf,
     *BImgBuff,
     RunCount;
 
-  long
-    ldblk;
-
   x=0;
   y=0;
 
@@ -305,8 +308,8 @@ static int UnpackWPGRaster(Image *image,int bpp)
   BImgBuff=(unsigned char *) AcquireMagickMemory(ldblk);
   if(BImgBuff==NULL) return(-2);
 
-  while(y<(long) image->rows)
-    {
+  while (y < (long) image->rows)
+  {
       bbuf=ReadBlobByte(image);
 
       /*
@@ -371,21 +374,20 @@ static int UnpackWPG2Raster(Image *image,int bpp)
   char
     SampleSize=1;
 
+  register long
+    i,
+    x;
+
+  long
+    ldblk,
+    y;
+
   unsigned char
     bbuf,
     *BImgBuff,
     RunCount,
     SampleBuffer[8];
 
-  int
-    x,
-    y,
-    i;
-
-  long
-    ldblk;
-
-
   x=0;
   y=0;
   ldblk=(long) ((bpp*image->columns+7)/8);
@@ -401,18 +403,20 @@ static int UnpackWPG2Raster(Image *image,int bpp)
         {
         case 0x7D:
           SampleSize=ReadBlobByte(image);  /* DSZ */
-          if(SampleSize>8) return(-2);
-          if(SampleSize<1) return(-2);
+          if (SampleSize > 8)
+            return(-2);
+          if (SampleSize < 1)
+            return(-2);
           break;
         case 0x7E:
           fprintf(stderr,"\nUnsupported WPG token XOR, please report!");
           break;
         case 0x7F:
           RunCount=ReadBlobByte(image);   /* BLK */
-          for(i=0;i<(int) SampleSize*((int)RunCount+1);i++)
-            {
-              InsertByte(0);
-            }
+          for(i=0; i < (long) SampleSize*((long)RunCount+1); i++)
+          {
+            InsertByte(0);
+          }
           break;
         case 0xFD:
           RunCount=ReadBlobByte(image);   /* EXT */
@@ -425,7 +429,7 @@ static int UnpackWPG2Raster(Image *image,int bpp)
           if(x!=0)
             {
               fprintf(stderr,
-                "\nUnsupported WPG2 unaligned token RST x=%d, please report!\n"
+                "\nUnsupported WPG2 unaligned token RST x=%ld, please report!\n"
                   ,x);
               return(-3);
             }
@@ -434,8 +438,8 @@ static int UnpackWPG2Raster(Image *image,int bpp)
            for(i=0;i<=RunCount;i++)
               {
               x=0;
-              InsertRow(BImgBuff,(long) (y< (long) image->rows?y:image->rows-1),
-                  image,bpp);
+              InsertRow(BImgBuff,(long) ( y< (long) image->rows ? y :
+                image->rows-1),image,bpp);
               y++;
               if(y<2) continue;
               if(y> (long) image->rows) return(-4);
@@ -476,7 +480,7 @@ static int UnpackWPG2Raster(Image *image,int bpp)
 unsigned LoadWPG2Flags(Image *image,char Precision,float *Angle)
 {
 const unsigned char TPR=1,TRN=2,SKW=4,SCL=8,ROT=0x10,OID=0x20,LCK=0x80;
-unsigned long  x;
+long  x;
 unsigned DenX;
 unsigned Flags;
 float CTM[3][3];        /* current transform matrix (currently ignored) */
@@ -517,9 +521,9 @@ float CTM[3][3];        /* current transform matrix (currently ignored) */
  if(Flags & TRN)
   {
   x=ReadBlobLSBLong(image); DenX=ReadBlobLSBLong(image);  /* Tx */
-  CTM[0][2]=(float)x + ((x>=0)?1:-1)*(float)DenX/0x10000;
+  CTM[0][2]=(float)x + ((x >= 0)?1:-1)*(float)DenX/0x10000;
   x=ReadBlobLSBLong(image); DenX=ReadBlobLSBLong(image);  /* Ty */
-  CTM[1][2]=(float)x + ((x>=0)?1:-1)*(float)DenX/0x10000;
+  CTM[1][2]=(float)x + ((x >= 0)?1:-1)*(float)DenX/0x10000;
   }
  if(Flags & TPR)
   {
@@ -599,7 +603,7 @@ static Image *ExtractPostscript(Image *image,const ImageInfo *image_info,
  FINISH_UNL:
   (void) RelinquishUniqueFileResource(clone_info->filename);/* */
  FINISH:
-  DestroyImageInfo(clone_info);
+  clone_info=DestroyImageInfo(clone_info);
   return(GetFirstImageInList(image));
 }
 
@@ -846,7 +850,7 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
-      DestroyImageList(image);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
   /*
@@ -982,13 +986,13 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
                   }
                 }
               else {
-                if(bpp<24)
+                if (bpp < 24)
                   if( image->colors<(1UL<<bpp) && bpp!=24 )
                     image->colormap=(PixelPacket *) ResizeMagickMemory(
                       image->colormap,(1<<bpp)*sizeof(PixelPacket));
               }
 
-              if(bpp==1)
+              if (bpp == 1)
                 {
                   if(image->colormap[0].red==0 &&
                      image->colormap[0].green==0 &&
@@ -1116,7 +1120,7 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
                     goto NoMemory;
                 }
               else {
-                if(bpp<24)
+                if (bpp < 24)
                   if( image->colors<(1UL<<bpp) && bpp!=24 )
                     image->colormap=(PixelPacket *) ResizeMagickMemory(
                       image->colormap,(1<<bpp)*sizeof(PixelPacket));
