@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -41,10 +41,13 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
 #include "magick/color.h"
 #include "magick/color_private.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
@@ -124,7 +127,7 @@ static Image *ReadXCImage(const ImageInfo *image_info,ExceptionInfo *exception)
   status=QueryMagickColor((char *) image_info->filename,&color,exception);
   if (status == MagickFalse)
     {
-      DestroyImage(image);
+      image=DestroyImage(image);
       return((Image *) NULL);
     }
   image->colorspace=color.colorspace;
@@ -194,6 +197,7 @@ ModuleExport void RegisterXCImage(void)
   entry->decoder=(DecoderHandler *) ReadXCImage;
   entry->adjoin=MagickFalse;
   entry->raw=MagickTrue;
+  entry->endian_support=MagickTrue;
   entry->description=AcquireString("Constant image uniform color");
   entry->module=AcquireString("XC");
   (void) RegisterMagickInfo(entry);
