@@ -17,7 +17,7 @@
 %                               November 2001                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -41,9 +41,13 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
+#include "magick/color.h"
 #include "magick/composite.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
@@ -406,8 +410,7 @@ static int load_tile_rle (Image* image, Image* tile_image, XCFDocInfo* inDocInfo
                 case 1: q->green= ScaleCharToQuantum(data);  break;
                 case 2: q->blue = ScaleCharToQuantum(data);  break;
                 case 3:
-                  q->opacity = (Quantum) ((int) data == 0 ? TransparentOpacity :
-                    ScaleCharToQuantum(255-inLayerInfo->opacity));
+                  q->opacity = (Quantum) (ScaleCharToQuantum(255-data));
                 break;
               }
               q++;
@@ -455,8 +458,7 @@ static int load_tile_rle (Image* image, Image* tile_image, XCFDocInfo* inDocInfo
                 case 1: q->green=ScaleCharToQuantum(data);  break;
                 case 2: q->blue=ScaleCharToQuantum(data);  break;
                 case 3:
-                  q->opacity=(Quantum) ((int) data == 0 ? TransparentOpacity :
-                    ScaleCharToQuantum(255-inLayerInfo->opacity));
+                  q->opacity = (Quantum) (ScaleCharToQuantum(255-data));
                   break;
               }
               q++;
@@ -591,7 +593,7 @@ static int load_level (Image* image, XCFDocInfo* inDocInfo, XCFLayerInfo*
       /* composite the tile onto the layer's image, and then destroy it */
         (void) CompositeImage(inLayerInfo->image,CopyCompositeOp,tile_image,
                   destLeft * TILE_WIDTH,destTop*TILE_HEIGHT);
-        DestroyImage(tile_image);
+        tile_image=DestroyImage(tile_image);
 
       /* adjust tile position */
       destLeft++;
@@ -780,9 +782,9 @@ static MagickBooleanType ReadOneLayer(Image* image,XCFDocInfo* inDocInfo,
         {
         amount = (ssize_t) Min (16, prop_size);
         amount = ReadBlob(image, (size_t) amount, (unsigned char *) &buf);
-	if (!amount)
-	  ThrowBinaryException(CorruptImageError,"CorruptImage",
-	                       image->filename);
+        if (!amount)
+          ThrowBinaryException(CorruptImageError,"CorruptImage",
+            image->filename);
         prop_size -= Min (16, amount);
         }
       }
@@ -912,7 +914,7 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
-      DestroyImageList(image);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
   count=ReadBlob(image,14,(unsigned char *) magick);
@@ -925,7 +927,7 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   doc_info.height=ReadBlobMSBLong(image);
   doc_info.image_type=ReadBlobMSBLong(image);
   /*
-	  Initialize image attributes.
+    Initialize image attributes.
   */
   image->columns=doc_info.width;
   image->rows=doc_info.height;
@@ -940,29 +942,28 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
     else
       if ( image_type == GIMP_INDEXED )
         ThrowReaderException(CoderError,"ColormapTypeNotSupported");
-  SetImage(image,OpaqueOpacity);
-  image->matte=MagickTrue;
+  SetImage(image,TransparentOpacity);
   /*
-	  Read properties.
+    Read properties.
   */
   while (foundPropEnd == MagickFalse)
-	{
+  {
     PropType prop_type = (PropType) ReadBlobMSBLong(image);
     unsigned long  prop_size = ReadBlobMSBLong(image);
 
     switch (prop_type)
-		{
+    {
       case PROP_END:
         foundPropEnd=1;
-	break;
+        break;
 
       case PROP_COLORMAP:
       {
-	/* Cannot rely on prop_size here--the value is set incorrectly
-	 * by some Gimp versions.
-	 */
-	unsigned long num_colours = ReadBlobMSBLong(image);
-	
+        /* Cannot rely on prop_size here--the value is set incorrectly
+           by some Gimp versions.
+        */
+        unsigned long num_colours = ReadBlobMSBLong(image);
+
         for (i=0; i < (long) (3L*num_colours); i++ )
           (void) ReadBlobByte(image);
     /*
@@ -1182,7 +1183,7 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
       if (layer_ok == MagickFalse) {
         int j;
         for (j=0; j < current_layer; j++)
-          DestroyImage(layer_info[j].image);
+          layer_info[j].image=DestroyImage(layer_info[j].image);
         ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
       }
 
@@ -1198,12 +1199,14 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
       /* composite the layer data onto the main image & then dispose the layer */
       (void) CompositeImage(image, OverCompositeOp, layer_info[0].image,
                layer_info[0].offset_x, layer_info[0].offset_y );
-      DestroyImage( layer_info[0].image );
+       layer_info[0].image =DestroyImage( layer_info[0].image );
 
       /* Bob says that if we do this, we'll get REAL gray images! */
       if ( image_type == GIMP_GRAY ) {
         QuantizeInfo  qi;
         GetQuantizeInfo(&qi);
+        qi.dither=IsPaletteImage(image,&image->exception) == MagickFalse ?
+          MagickTrue : MagickFalse;
         qi.colorspace = GRAYColorspace;
         (void) QuantizeImage( &qi, image );
       }
@@ -1218,7 +1221,7 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
           if ( layer_info[j].visible ) { /* only visible ones, please! */
             CompositeImage(image, OverCompositeOp, layer_info[j].image,
                      layer_info[j].offset_x, layer_info[j].offset_y );
-            DestroyImage( layer_info[j].image );
+             layer_info[j].image =DestroyImage( layer_info[j].image );
 
             /* Bob says that if we do this, we'll get REAL gray images! */
             if ( image_type == GIMP_GRAY ) {
@@ -1239,7 +1242,7 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
         (void) CompositeImage(image, CopyCompositeOp, layer_info[number_layers-1].image,
                  layer_info[number_layers-1].offset_x,
                  layer_info[number_layers-1].offset_y );
-        DestroyImage( layer_info[number_layers-1].image );
+         layer_info[number_layers-1].image =DestroyImage( layer_info[number_layers-1].image );
 
         /* Bob says that if we do this, we'll get REAL gray images! */
         if ( image_type == GIMP_GRAY ) {
