@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -42,10 +42,13 @@
 #include "magick/studio.h"
 #include "magick/attribute.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
 #include "magick/color_private.h"
 #include "magick/colorspace.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
@@ -201,7 +204,7 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
-      DestroyImageList(image);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
   /*
@@ -232,7 +235,7 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   count=ReadBlob(image,length,(unsigned char *) comment);
   comment[length]='\0';
-  (void) SetImageAttribute(image,"comment",comment);
+  (void) SetImageAttribute(image,"Comment",comment);
   comment=(char *) RelinquishMagickMemory(comment);
   if (count == 0)
     ThrowReaderException(CorruptImageError,
@@ -260,7 +263,7 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   ximage->blue_mask=header.blue_mask;
   status=(MagickBooleanType) XInitImage(ximage);
   if (status == MagickFalse)
-    ThrowReaderException(CorruptImageError,"UnrecognizedXWDHeader");
+    ThrowReaderException(CorruptImageError,"UnableToReadImageHeader");
   /*
     Read colormap.
   */
@@ -387,8 +390,14 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
             if (SyncImagePixels(image) == MagickFalse)
               break;
             if (QuantumTick(y,image->rows) != MagickFalse)
-              if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
-                break;
+              if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                  (QuantumTick(y,image->rows) != MagickFalse))
+                {
+                  status=image->progress_monitor(LoadImageTag,y,image->rows,
+                    image->client_data);
+                  if (status == MagickFalse)
+                    break;
+                }
           }
         else
           for (y=0; y < (long) image->rows; y++)
@@ -410,8 +419,14 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
             if (SyncImagePixels(image) == MagickFalse)
               break;
             if (QuantumTick(y,image->rows) != MagickFalse)
-              if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
-                break;
+              if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                  (QuantumTick(y,image->rows) != MagickFalse))
+                {
+                  status=image->progress_monitor(LoadImageTag,y,image->rows,
+                    image->client_data);
+                  if (status == MagickFalse)
+                    break;
+                }
           }
         break;
       }
@@ -444,8 +459,14 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
           if (SyncImagePixels(image) == MagickFalse)
             break;
           if (QuantumTick(y,image->rows) != MagickFalse)
-            if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
-              break;
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(y,image->rows) != MagickFalse))
+              {
+                status=image->progress_monitor(LoadImageTag,y,image->rows,
+                  image->client_data);
+                if (status == MagickFalse)
+                  break;
+              }
         }
         break;
       }
@@ -559,7 +580,7 @@ ModuleExport void UnregisterXWDImage(void)
 static MagickBooleanType WriteXWDImage(const ImageInfo *image_info,Image *image)
 {
   const ImageAttribute
-	  *attribute;
+    *attribute;
 
   long
     y;
@@ -614,7 +635,7 @@ static MagickBooleanType WriteXWDImage(const ImageInfo *image_info,Image *image)
     Initialize XWD file header.
   */
   xwd_info.header_size=(CARD32) sz_XWDheader;
-  attribute=GetImageAttribute(image,"comment");
+  attribute=GetImageAttribute(image,"Comment");
   if (attribute != (const ImageAttribute *) NULL)
     xwd_info.header_size+=strlen(attribute->value);
   xwd_info.header_size++;
@@ -741,9 +762,11 @@ static MagickBooleanType WriteXWDImage(const ImageInfo *image_info,Image *image)
       *q++='\0';
     (void) WriteBlob(image,(size_t) (q-pixels),pixels);
     if (image->previous == (Image *) NULL)
-      if (QuantumTick(y,image->rows) != MagickFalse)
+      if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+          (QuantumTick(y,image->rows) != MagickFalse))
         {
-          status=MagickMonitor(SaveImageTag,y,image->rows,&image->exception);
+          status=image->progress_monitor(SaveImageTag,y,image->rows,
+            image->client_data);
           if (status == MagickFalse)
             break;
         }
