@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -41,14 +41,18 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
 #include "magick/colorspace.h"
 #include "magick/constitute.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/pixel.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 #include "magick/utility.h"
@@ -99,9 +103,11 @@ static Image *ReadYCBCRImage(const ImageInfo *image_info,
   MagickBooleanType
     status;
 
+  MagickOffsetType
+    offset;
+
   register long
-    i,
-    x;
+    i;
 
   register PixelPacket
     *q;
@@ -116,6 +122,7 @@ static Image *ReadYCBCRImage(const ImageInfo *image_info,
     *scanline;
 
   unsigned long
+    depth,
     width;
 
   assert(image_info != (const ImageInfo *) NULL);
@@ -135,7 +142,7 @@ static Image *ReadYCBCRImage(const ImageInfo *image_info,
       status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
       if (status == MagickFalse)
         {
-          DestroyImageList(image);
+          image=DestroyImageList(image);
           return((Image *) NULL);
         }
       for (i=0; i < image->offset; i++)
@@ -144,11 +151,12 @@ static Image *ReadYCBCRImage(const ImageInfo *image_info,
   /*
     Allocate memory for a scanline.
   */
-  packet_size=(size_t) (image->depth > 8 ? 6 : 3);
+  depth=GetImageQuantumDepth(image,MagickTrue);
+  packet_size=(size_t) (3*depth/8);
   if (LocaleCompare(image_info->magick,"YCbCrA") == 0)
     {
+      packet_size+=depth/8;
       image->matte=MagickTrue;
-      packet_size=(size_t) (image->depth > 8 ? 8 : 6);
     }
   scanline=(unsigned char *)
     AcquireMagickMemory(packet_size*image->extract_info.width);
@@ -164,7 +172,7 @@ static Image *ReadYCBCRImage(const ImageInfo *image_info,
       for (y=0; y < (long) image->rows; y++)
         (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
     }
-  x=(long) (packet_size*image->extract_info.x);
+  offset=(MagickOffsetType) (packet_size*image->extract_info.x);
   do
   {
     /*
@@ -193,15 +201,17 @@ static Image *ReadYCBCRImage(const ImageInfo *image_info,
           if (q == (PixelPacket *) NULL)
             break;
           if (image->matte == MagickFalse)
-            (void) PushImagePixels(image,RGBQuantum,scanline+x);
+            (void) ExportQuantumPixels(image,RGBQuantum,0,scanline+offset);
           else
-            (void) PushImagePixels(image,RGBAQuantum,scanline+x);
+            (void) ExportQuantumPixels(image,RGBAQuantum,0,scanline+offset);
           if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows) != MagickFalse)
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(y,image->rows) != MagickFalse))
               {
-                status=MagickMonitor(LoadImageTag,y,image->rows,exception);
+                status=image->progress_monitor(LoadImageTag,y,image->rows,
+                  image->client_data);
                 if (status == MagickFalse)
                   break;
               }
@@ -227,23 +237,25 @@ static Image *ReadYCBCRImage(const ImageInfo *image_info,
           q=SetImagePixels(image,0,y,image->columns,1);
           if (q == (PixelPacket *) NULL)
             break;
-          (void) PushImagePixels(image,RedQuantum,scanline+x);
+          (void) ExportQuantumPixels(image,RedQuantum,0,scanline+offset);
           (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
-          (void) PushImagePixels(image,GreenQuantum,scanline+x);
+          (void) ExportQuantumPixels(image,GreenQuantum,0,scanline+offset);
           (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
-          (void) PushImagePixels(image,BlueQuantum,scanline+x);
+          (void) ExportQuantumPixels(image,BlueQuantum,0,scanline+offset);
           if (image->matte != MagickFalse)
             {
               (void) ReadBlob(image,packet_size*image->extract_info.width,
                 scanline);
-              (void) PushImagePixels(image,AlphaQuantum,scanline+x);
+              (void) ExportQuantumPixels(image,AlphaQuantum,0,scanline+offset);
             }
           if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows) != MagickFalse)
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(y,image->rows) != MagickFalse))
               {
-                status=MagickMonitor(LoadImageTag,y,image->rows,exception);
+                status=image->progress_monitor(LoadImageTag,y,image->rows,
+                  image->client_data);
                 if (status == MagickFalse)
                   break;
               }
@@ -268,7 +280,7 @@ static Image *ReadYCBCRImage(const ImageInfo *image_info,
             status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
             if (status == MagickFalse)
               {
-                DestroyImageList(image);
+                image=DestroyImageList(image);
                 return((Image *) NULL);
               }
           }
@@ -285,13 +297,15 @@ static Image *ReadYCBCRImage(const ImageInfo *image_info,
           q=SetImagePixels(image,0,y,image->columns,1);
           if (q == (PixelPacket *) NULL)
             break;
-          (void) PushImagePixels(image,RedQuantum,scanline+x);
+          (void) ExportQuantumPixels(image,RedQuantum,0,scanline+offset);
           if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(i,span) != MagickFalse)
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(i,span) != MagickFalse))
               {
-                status=MagickMonitor(LoadImageTag,i,span,&image->exception);
+                status=image->progress_monitor(LoadImageTag,i,span,
+                  image->client_data);
                 if (status == MagickFalse)
                   break;
               }
@@ -307,7 +321,7 @@ static Image *ReadYCBCRImage(const ImageInfo *image_info,
             status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
             if (status == MagickFalse)
               {
-                DestroyImageList(image);
+                image=DestroyImageList(image);
                 return((Image *) NULL);
               }
           }
@@ -319,13 +333,15 @@ static Image *ReadYCBCRImage(const ImageInfo *image_info,
           q=GetImagePixels(image,0,y,image->columns,1);
           if (q == (PixelPacket *) NULL)
             break;
-          (void) PushImagePixels(image,GreenQuantum,scanline+x);
+          (void) ExportQuantumPixels(image,GreenQuantum,0,scanline+offset);
           if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(i,span) != MagickFalse)
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(i,span) != MagickFalse))
               {
-                status=MagickMonitor(LoadImageTag,i,span,&image->exception);
+                status=image->progress_monitor(LoadImageTag,i,span,
+                  image->client_data);
                 if (status == MagickFalse)
                   break;
               }
@@ -341,7 +357,7 @@ static Image *ReadYCBCRImage(const ImageInfo *image_info,
             status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
             if (status == MagickFalse)
               {
-                DestroyImageList(image);
+                image=DestroyImageList(image);
                 return((Image *) NULL);
               }
           }
@@ -353,13 +369,15 @@ static Image *ReadYCBCRImage(const ImageInfo *image_info,
           q=GetImagePixels(image,0,y,image->columns,1);
           if (q == (PixelPacket *) NULL)
             break;
-          (void) PushImagePixels(image,BlueQuantum,scanline+x);
+          (void) ExportQuantumPixels(image,BlueQuantum,0,scanline+offset);
           if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(i,span) != MagickFalse)
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(i,span) != MagickFalse))
               {
-                status=MagickMonitor(LoadImageTag,i,span,&image->exception);
+                status=image->progress_monitor(LoadImageTag,i,span,
+                  image->client_data);
                 if (status == MagickFalse)
                   break;
               }
@@ -380,7 +398,7 @@ static Image *ReadYCBCRImage(const ImageInfo *image_info,
                 status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
                 if (status == MagickFalse)
                   {
-                    DestroyImageList(image);
+                    image=DestroyImageList(image);
                     return((Image *) NULL);
                   }
               }
@@ -394,13 +412,15 @@ static Image *ReadYCBCRImage(const ImageInfo *image_info,
               q=GetImagePixels(image,0,y,image->columns,1);
               if (q == (PixelPacket *) NULL)
                 break;
-              (void) PushImagePixels(image,AlphaQuantum,scanline+x);
+              (void) ExportQuantumPixels(image,AlphaQuantum,0,scanline+offset);
               if (SyncImagePixels(image) == MagickFalse)
                 break;
               if (image->previous == (Image *) NULL)
-                if (QuantumTick(i,span) != MagickFalse)
+                if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                    (QuantumTick(i,span) != MagickFalse))
                   {
-                    status=MagickMonitor(LoadImageTag,i,span,&image->exception);
+                    status=image->progress_monitor(LoadImageTag,i,span,
+                      image->client_data);
                     if (status == MagickFalse)
                       break;
                   }
@@ -440,14 +460,17 @@ static Image *ReadYCBCRImage(const ImageInfo *image_info,
         AllocateNextImage(image_info,image);
         if (image->next == (Image *) NULL)
           {
-            DestroyImageList(image);
+            image=DestroyImageList(image);
             return((Image *) NULL);
           }
         image=SyncNextImageInList(image);
-        status=MagickMonitor(LoadImagesTag,TellBlob(image),GetBlobSize(image),
-          exception);
-        if (status == MagickFalse)
-          break;
+        if (image->progress_monitor != (MagickProgressMonitor) NULL)
+          {
+            status=image->progress_monitor(LoadImagesTag,TellBlob(image),
+              GetBlobSize(image),image->client_data);
+            if (status == MagickFalse)
+              break;
+          }
       }
   } while (count != 0);
   scanline=(unsigned char *) RelinquishMagickMemory(scanline);
@@ -572,6 +595,9 @@ static MagickBooleanType WriteYCBCRImage(const ImageInfo *image_info,
   unsigned char
     *pixels;
 
+  unsigned long
+    depth;
+
   /*
     Allocate memory for pixels.
   */
@@ -581,9 +607,10 @@ static MagickBooleanType WriteYCBCRImage(const ImageInfo *image_info,
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
-  packet_size=(size_t) (image->depth > 8 ? 6 : 3);
+  depth=GetImageQuantumDepth(image,MagickTrue);
+  packet_size=(size_t) (3*depth/8);
   if (LocaleCompare(image_info->magick,"YCbCrA") == 0)
-    packet_size=(size_t) (image->depth > 8 ? 8 : 6);
+    packet_size+=depth/8;
   pixels=(unsigned char *) AcquireMagickMemory(packet_size*image->columns);
   if (pixels == (unsigned char *) NULL)
     ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
@@ -621,19 +648,20 @@ static MagickBooleanType WriteYCBCRImage(const ImageInfo *image_info,
             break;
           if (LocaleCompare(image_info->magick,"YCbCrA") != 0)
             {
-              (void) PopImagePixels(image,RGBQuantum,pixels);
+              (void) ImportQuantumPixels(image,RGBQuantum,0,pixels);
               (void) WriteBlob(image,packet_size*image->columns,pixels);
             }
           else
             {
-              (void) PopImagePixels(image,RGBAQuantum,pixels);
+              (void) ImportQuantumPixels(image,RGBAQuantum,0,pixels);
               (void) WriteBlob(image,packet_size*image->columns,pixels);
             }
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows) != MagickFalse)
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(y,image->rows) != MagickFalse))
               {
-                status=MagickMonitor(SaveImageTag,y,image->rows,
-                  &image->exception);
+                status=image->progress_monitor(SaveImageTag,y,image->rows,
+                  image->client_data);
                 if (status == MagickFalse)
                   break;
               }
@@ -650,21 +678,22 @@ static MagickBooleanType WriteYCBCRImage(const ImageInfo *image_info,
           p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
-          (void) PopImagePixels(image,RedQuantum,pixels);
+          (void) ImportQuantumPixels(image,RedQuantum,0,pixels);
           (void) WriteBlob(image,(size_t) image->columns,pixels);
-          (void) PopImagePixels(image,GreenQuantum,pixels);
+          (void) ImportQuantumPixels(image,GreenQuantum,0,pixels);
           (void) WriteBlob(image,(size_t) image->columns,pixels);
-          (void) PopImagePixels(image,BlueQuantum,pixels);
+          (void) ImportQuantumPixels(image,BlueQuantum,0,pixels);
           (void) WriteBlob(image,(size_t) image->columns,pixels);
           if (LocaleCompare(image_info->magick,"YCbCrA") == 0)
             {
-              (void) PopImagePixels(image,AlphaQuantum,pixels);
+              (void) ImportQuantumPixels(image,AlphaQuantum,0,pixels);
               (void) WriteBlob(image,(size_t) image->columns,pixels);
             }
-          if (QuantumTick(y,image->rows) != MagickFalse)
+          if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+              (QuantumTick(y,image->rows) != MagickFalse))
             {
-              status=MagickMonitor(SaveImageTag,y,image->rows,
-                &image->exception);
+              status=image->progress_monitor(SaveImageTag,y,image->rows,
+                image->client_data);
               if (status == MagickFalse)
                 break;
             }
@@ -690,7 +719,7 @@ static MagickBooleanType WriteYCBCRImage(const ImageInfo *image_info,
           p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
-          (void) PopImagePixels(image,RedQuantum,pixels);
+          (void) ImportQuantumPixels(image,RedQuantum,0,pixels);
           (void) WriteBlob(image,(size_t) image->columns,pixels);
         }
         if (image_info->interlace == PartitionInterlace)
@@ -702,15 +731,19 @@ static MagickBooleanType WriteYCBCRImage(const ImageInfo *image_info,
             if (status == MagickFalse)
               return(status);
           }
-        status=MagickMonitor(SaveImageTag,100,400,&image->exception);
-        if (status == MagickFalse)
-          break;
+        if (image->progress_monitor != (MagickProgressMonitor) NULL)
+          {
+            status=image->progress_monitor(LoadImageTag,100,400,
+              image->client_data);
+            if (status == MagickFalse)
+              break;
+          }
         for (y=0; y < (long) image->rows; y++)
         {
           p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
-          (void) PopImagePixels(image,GreenQuantum,pixels);
+          (void) ImportQuantumPixels(image,GreenQuantum,0,pixels);
           (void) WriteBlob(image,(size_t) image->columns,pixels);
         }
         if (image_info->interlace == PartitionInterlace)
@@ -722,22 +755,30 @@ static MagickBooleanType WriteYCBCRImage(const ImageInfo *image_info,
             if (status == MagickFalse)
               return(status);
           }
-        status=MagickMonitor(SaveImageTag,200,400,&image->exception);
-        if (status == MagickFalse)
-          break;
+        if (image->progress_monitor != (MagickProgressMonitor) NULL)
+          {
+            status=image->progress_monitor(LoadImageTag,200,400,
+              image->client_data);
+            if (status == MagickFalse)
+              break;
+          }
         for (y=0; y < (long) image->rows; y++)
         {
           p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
-          (void) PopImagePixels(image,BlueQuantum,pixels);
+          (void) ImportQuantumPixels(image,BlueQuantum,0,pixels);
           (void) WriteBlob(image,(size_t) image->columns,pixels);
         }
         if (LocaleCompare(image_info->magick,"YCbCrA") == 0)
           {
-            status=MagickMonitor(SaveImageTag,300,400,&image->exception);
-            if (status == MagickFalse)
-              break;
+            if (image->progress_monitor != (MagickProgressMonitor) NULL)
+              {
+                status=image->progress_monitor(LoadImageTag,300,400,
+                  image->client_data);
+                if (status == MagickFalse)
+                  break;
+              }
             if (image_info->interlace == PartitionInterlace)
               {
                 CloseBlob(image);
@@ -753,26 +794,34 @@ static MagickBooleanType WriteYCBCRImage(const ImageInfo *image_info,
                 &image->exception);
               if (p == (const PixelPacket *) NULL)
                 break;
-              (void) PopImagePixels(image,AlphaQuantum,pixels);
+              (void) ImportQuantumPixels(image,AlphaQuantum,0,pixels);
               (void) WriteBlob(image,(size_t) image->columns,pixels);
             }
           }
         if (image_info->interlace == PartitionInterlace)
           (void) CopyMagickString(image->filename,image_info->filename,
             MaxTextExtent);
-        status=MagickMonitor(SaveImageTag,400,400,&image->exception);
-        if (status == MagickFalse)
-          break;
+        if (image->progress_monitor != (MagickProgressMonitor) NULL)
+          {
+            status=image->progress_monitor(LoadImageTag,400,400,
+              image->client_data);
+            if (status == MagickFalse)
+              break;
+          }
         break;
       }
     }
     if (image->next == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
-      &image->exception);
-    if (status == MagickFalse)
-      break;
+    if (image->progress_monitor != (MagickProgressMonitor) NULL)
+      {
+        status=image->progress_monitor(SaveImagesTag,scene,
+          GetImageListLength(image),image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
+    scene++;
   } while (image_info->adjoin != MagickFalse);
   pixels=(unsigned char *) RelinquishMagickMemory(pixels);
   CloseBlob(image);
