@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -41,11 +41,14 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
 #include "magick/colorspace.h"
 #include "magick/constitute.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/geometry.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
@@ -83,10 +86,6 @@ static MagickBooleanType
 %
 %  A description of each parameter follows:
 %
-%    o image:  Method ReadYUVImage returns a pointer to the image after
-%      reading.  A null image is returned if there is a memory shortage or
-%      if the image cannot be read.
-%
 %    o image_info: The image info.
 %
 %    o exception: return any errors or warnings in this structure.
@@ -181,7 +180,7 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
       status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
       if (status == MagickFalse)
         {
-          DestroyImageList(image);
+          image=DestroyImageList(image);
           return((Image *) NULL);
         }
       for (i=0; i < image->offset; i++)
@@ -215,7 +214,7 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
         status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
         if (status == MagickFalse)
           {
-            DestroyImageList(image);
+            image=DestroyImageList(image);
             return((Image *) NULL);
           }
       }
@@ -269,9 +268,14 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
       if (interlace == NoInterlace)
         if (SyncImagePixels(chroma_image) == MagickFalse)
           break;
-      if (image->previous == (Image *) NULL)
-        if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
-          break;
+      if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+          (QuantumTick(y,image->rows) != MagickFalse))
+        {
+          status=image->progress_monitor(LoadImageTag,y,image->rows,
+            image->client_data);
+          if (status == MagickFalse)
+            break;
+        }
     }
     if (interlace == PartitionInterlace)
       {
@@ -280,7 +284,7 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
         status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
         if (status == MagickFalse)
           {
-            DestroyImageList(image);
+            image=DestroyImageList(image);
             return((Image *) NULL);
           }
       }
@@ -310,7 +314,7 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
           status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
           if (status == MagickFalse)
             {
-              DestroyImageList(image);
+              image=DestroyImageList(image);
               return((Image *) NULL);
             }
         }
@@ -335,7 +339,7 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
     */
     resize_image=ResizeImage(chroma_image,image->columns,image->rows,
       TriangleFilter,1.0,exception);
-    DestroyImage(chroma_image);
+    chroma_image=DestroyImage(chroma_image);
     if (resize_image == (Image *) NULL)
       ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
     for (y=0; y < (long) image->rows; y++)
@@ -355,7 +359,7 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
       if (SyncImagePixels(image) == MagickFalse)
         break;
     }
-    DestroyImage(resize_image);
+    resize_image=DestroyImage(resize_image);
     image->colorspace=YCbCrColorspace;
     (void) SetImageColorspace(image,RGBColorspace);
     if (interlace == PartitionInterlace)
@@ -385,14 +389,17 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
         AllocateNextImage(image_info,image);
         if (image->next == (Image *) NULL)
           {
-            DestroyImageList(image);
+            image=DestroyImageList(image);
             return((Image *) NULL);
           }
         image=SyncNextImageInList(image);
-        status=MagickMonitor(LoadImagesTag,TellBlob(image),GetBlobSize(image),
-          exception);
-        if (status == MagickFalse)
-          break;
+        if (image->progress_monitor != (MagickProgressMonitor) NULL)
+          {
+            status=image->progress_monitor(LoadImagesTag,TellBlob(image),
+              GetBlobSize(image),image->client_data);
+            if (status == MagickFalse)
+              break;
+          }
       }
   } while (count != 0);
   scanline=(unsigned char *) RelinquishMagickMemory(scanline);
@@ -483,10 +490,6 @@ ModuleExport void UnregisterYUVImage(void)
 %
 %  A description of each parameter follows.
 %
-%    o status: Method WriteYUVImage returns True if the image is written.
-%      False is returned is there is a memory shortage or if the image file
-%      fails to write.
-%
 %    o image_info: The image info.
 %
 %    o image:  The image.
@@ -623,15 +626,16 @@ static MagickBooleanType WriteYUVImage(const ImageInfo *image_info,Image *image)
             x++;
           }
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows) != 0)
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(y,image->rows) != MagickFalse))
               {
-                status=MagickMonitor(SaveImageTag,y,image->rows,
-                  &image->exception);
+                status=image->progress_monitor(SaveImageTag,y,image->rows,
+                  image->client_data);
                 if (status == MagickFalse)
                   break;
               }
         }
-        DestroyImage(yuv_image);
+        yuv_image=DestroyImage(yuv_image);
       }
     else
       {
@@ -650,15 +654,16 @@ static MagickBooleanType WriteYUVImage(const ImageInfo *image_info,Image *image)
             p++;
           }
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows) != 0)
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(y,image->rows) != MagickFalse))
               {
-                status=MagickMonitor(SaveImageTag,y,image->rows,
-                  &image->exception);
+                status=image->progress_monitor(SaveImageTag,y,image->rows,
+                  image->client_data);
                 if (status == MagickFalse)
                   break;
               }
         }
-        DestroyImage(yuv_image);
+        yuv_image=DestroyImage(yuv_image);
         /*
           Initialize U channel.
         */
@@ -708,17 +713,21 @@ static MagickBooleanType WriteYUVImage(const ImageInfo *image_info,Image *image)
           }
         }
       }
-    DestroyImage(chroma_image);
+    chroma_image=DestroyImage(chroma_image);
     if (interlace == PartitionInterlace)
       (void) CopyMagickString(image->filename,image_info->filename,
         MaxTextExtent);
     if (image->next == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
-      &image->exception);
-    if (status == MagickFalse)
-      break;
+    if (image->progress_monitor != (MagickProgressMonitor) NULL)
+      {
+        status=image->progress_monitor(SaveImagesTag,scene,
+          GetImageListLength(image),image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
+    scene++;
   } while (image_info->adjoin != MagickFalse);
   CloseBlob(image);
   return(MagickTrue);
