@@ -12,7 +12,7 @@
 %                              December 1998                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -82,10 +82,10 @@ ModuleExport MagickBooleanType AnalyzeImage(Image **image,const int argc,
     hue,
     saturation;
 
-  int
+  long
     y;
 
-  register int
+  register long
     x;
 
   register PixelPacket
