@@ -1,5 +1,5 @@
 /* ltdl.c -- system independent dlopen wrapper
-   Copyright (C) 1998, 1999, 2000 Free Software Foundation, Inc.
+   Copyright (C) 1998, 1999, 2000, 2004 Free Software Foundation, Inc.
    Originally by Thomas Tanner <tanner@ffii.org>
    This file is part of GNU Libtool.
 
@@ -26,7 +26,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 */
 
 #if HAVE_CONFIG_H
-#  include "magick/magick_config.h"
+#  include <magick/magick_config.h>
 #endif
 
 #if HAVE_UNISTD_H
@@ -67,6 +67,55 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 #  include <errno.h>
 #endif
 
+
+#ifndef __WINDOWS__
+#  ifdef __WIN32__
+#    define __WINDOWS__
+#  endif
+#endif
+
+
+#undef LT_USE_POSIX_DIRENT
+#ifdef HAVE_CLOSEDIR
+#  ifdef HAVE_OPENDIR
+#    ifdef HAVE_READDIR
+#      ifdef HAVE_DIRENT_H
+#        define LT_USE_POSIX_DIRENT
+#      endif /* HAVE_DIRENT_H */
+#    endif /* HAVE_READDIR */
+#  endif /* HAVE_OPENDIR */
+#endif /* HAVE_CLOSEDIR */
+
+
+#undef LT_USE_WINDOWS_DIRENT_EMULATION
+#ifndef LT_USE_POSIX_DIRENT
+#  ifdef __WINDOWS__
+#    define LT_USE_WINDOWS_DIRENT_EMULATION
+#  endif /* __WINDOWS__ */
+#endif /* LT_USE_POSIX_DIRENT */
+
+
+#ifdef LT_USE_POSIX_DIRENT
+#  include <dirent.h>
+#  define LT_D_NAMLEN(dirent) (strlen((dirent)->d_name))
+#else
+#  ifdef LT_USE_WINDOWS_DIRENT_EMULATION
+#    define LT_D_NAMLEN(dirent) (strlen((dirent)->d_name))
+#  else
+#    define dirent direct
+#    define LT_D_NAMLEN(dirent) ((dirent)->d_namlen)
+#    if HAVE_SYS_NDIR_H
+#      include <sys/ndir.h>
+#    endif
+#    if HAVE_SYS_DIR_H
+#      include <sys/dir.h>
+#    endif
+#    if HAVE_NDIR_H
+#      include <ndir.h>
+#    endif
+#  endif
+#endif
+
 #if HAVE_ARGZ_H
 #  include <argz.h>
 #endif
@@ -79,31 +128,6 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 
 #include "ltdl.h"
 
-#if defined(HAVE_CLOSEDIR) && defined(HAVE_OPENDIR) && defined(HAVE_READDIR) && defined(HAVE_DIRENT_H)
-/* We have a fully operational dirent subsystem.  */
-# include <dirent.h>
-# define LT_D_NAMLEN(dirent) (strlen((dirent)->d_name))
-
-#elif !defined(__WINDOWS__)
-/* We are not on windows, so we can get the same functionality from the
-   `direct' API.  */
-# define dirent direct
-# define LT_D_NAMLEN(dirent) ((dirent)->d_namlen)
-# if HAVE_SYS_NDIR_H
-#   include <sys/ndir.h>
-# endif
-# if HAVE_SYS_DIR_H
-#   include <sys/dir.h>
-# endif
-# if HAVE_NDIR_H
-#   include <ndir.h>
-# endif
-
-#else  /* __WINDOWS__ */
-/* Use some wrapper code to emulate dirent on windows..  */
-# define LT_USE_WINDOWS_DIRENT_EMULATION
-#endif
-
 #if WITH_DMALLOC
 #  include <dmalloc.h>
 #endif
@@ -122,12 +146,34 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 
 /* fopen() mode flags for reading a text file */
 #undef	LT_READTEXT_MODE
-#if defined(__WINDOWS__) || defined(__CYGWIN__)
+#ifdef __WINDOWS__
 #  define LT_READTEXT_MODE "rt"
 #else
 #  define LT_READTEXT_MODE "r"
 #endif
 
+#ifdef LT_USE_WINDOWS_DIRENT_EMULATION
+
+#include <windows.h>
+
+#define dirent lt_dirent
+#define DIR lt_DIR
+
+struct dirent
+{
+  char d_name[2048];
+  int  d_namlen;
+};
+
+typedef struct _DIR
+{
+  HANDLE hSearch;
+  WIN32_FIND_DATA Win32FindData;
+  BOOL firsttime;
+  struct dirent file_info;
+} DIR;
+
+#endif /* LT_USE_WINDOWS_DIRENT_EMULATION */
 
 
 /* --- MANIFEST CONSTANTS --- */
@@ -339,11 +385,13 @@ memcpy (dest, src, size)
      const lt_ptr src;
      size_t size;
 {
-  size_t i = 0;
+  const char *	s = src;
+  char *	d = dest;
+  size_t	i = 0;
 
   for (i = 0; i < size; ++i)
     {
-      dest[i] = src[i];
+      d[i] = s[i];
     }
 
   return dest;
@@ -363,17 +411,21 @@ memmove (dest, src, size)
      const lt_ptr src;
      size_t size;
 {
-  size_t i;
+  const char *	s = src;
+  char *	d = dest;
+  size_t	i;
 
-  if (dest < src)
+  if (d < s)
     for (i = 0; i < size; ++i)
       {
-	dest[i] = src[i];
+	d[i] = s[i];
       }
-  else if (dest > src)
-    for (i = size -1; i >= 0; --i)
+  else if (d > s && size > 0)
+    for (i = size -1; ; --i)
       {
-	dest[i] = src[i];
+	d[i] = s[i];
+	if (i == 0)
+	  break;
       }
 
   return dest;
@@ -381,37 +433,17 @@ memmove (dest, src, size)
 
 #endif /* !HAVE_MEMMOVE */
 
-# if LT_USE_WINDOWS_DIRENT_EMULATION
-
-# include <windows.h>
-
-# define LT_D_NAMLEN(dirent) (strlen((dirent)->d_name))
-# define dirent lt_dirent
-# define DIR lt_DIR
-
-struct dirent
-{
-  char d_name[2048];
-  int  d_namlen;
-};
-
-typedef struct _DIR
-{
-  HANDLE hSearch;
-  WIN32_FIND_DATA Win32FindData;
-  BOOL firsttime;
-  struct dirent file_info;
-} DIR;
+#ifdef LT_USE_WINDOWS_DIRENT_EMULATION
 
 static void closedir LT_PARAMS((DIR *entry));
 
 static void
-closedir (entry)
-    DIR *entry;
+closedir(entry)
+  DIR *entry;
 {
-  assert (entry != (DIR *) NULL);
-  FindClose (entry->hSearch);
-  lt_dlfree ((lt_ptr) entry);
+  assert(entry != (DIR *) NULL);
+  FindClose(entry->hSearch);
+  lt_dlfree((lt_ptr)entry);
 }
 
 
@@ -419,65 +451,58 @@ static DIR * opendir LT_PARAMS((const char *path));
 
 static DIR*
 opendir (path)
-    const char *path;
+  const char *path;
 {
   char file_specification[LT_FILENAME_MAX];
   DIR *entry;
 
-  assert (path != (char *) NULL);
-  (void) strncpy (file_specification, path, LT_FILENAME_MAX-1);
-  (void) strcat (file_specification, "\\");
-  entry = LT_DLMALLOC (DIR, sizeof(DIR));
+  assert(path != (char *) NULL);
+  (void) strncpy(file_specification,path,LT_FILENAME_MAX-1);
+  (void) strcat(file_specification,"\\");
+  entry = LT_DLMALLOC (DIR,sizeof(DIR));
   if (entry != (DIR *) 0)
     {
       entry->firsttime = TRUE;
-      entry->hSearch = FindFirstFile (file_specification,
-				      &entry->Win32FindData);
+      entry->hSearch = FindFirstFile(file_specification,&entry->Win32FindData);
     }
-
   if (entry->hSearch == INVALID_HANDLE_VALUE)
     {
-      (void) strcat (file_specification, "\\*.*");
-      entry->hSearch = FindFirstFile (file_specification,
-				      &entry->Win32FindData);
+      (void) strcat(file_specification,"\\*.*");
+      entry->hSearch = FindFirstFile(file_specification,&entry->Win32FindData);
       if (entry->hSearch == INVALID_HANDLE_VALUE)
         {
           LT_DLFREE (entry);
           return (DIR *) 0;
         }
     }
-
-  return entry;
+  return(entry);
 }
 
 
 static struct dirent *readdir LT_PARAMS((DIR *entry));
 
-static struct dirent *
-readdir (entry)
-    DIR *entry;
+static struct dirent *readdir(entry)
+  DIR *entry;
 {
-  int status;
+  int
+    status;
 
   if (entry == (DIR *) 0)
-    return (struct dirent *) 0;
-
+    return((struct dirent *) 0);
   if (!entry->firsttime)
     {
-      status = FindNextFile (entry->hSearch, &entry->Win32FindData);
+      status = FindNextFile(entry->hSearch,&entry->Win32FindData);
       if (status == 0)
-        return (struct dirent *) 0;
+        return((struct dirent *) 0);
     }
-
   entry->firsttime = FALSE;
-  (void) strncpy (entry->file_info.d_name, entry->Win32FindData.cFileName,
-		  LT_FILENAME_MAX - 1);
-  entry->file_info.d_namlen = strlen (entry->file_info.d_name);
-
-  return &entry->file_info;
+  (void) strncpy(entry->file_info.d_name,entry->Win32FindData.cFileName,
+    LT_FILENAME_MAX-1);
+  entry->file_info.d_namlen = strlen(entry->file_info.d_name);
+  return(&entry->file_info);
 }
-#endif /* !LT_USE_WINDOWS_DIRENT_EMULATION */
 
+#endif /* LT_USE_WINDOWS_DIRENT_EMULATION */
 
 /* According to Alexandre Oliva <oliva@lsd.ic.unicamp.br>,
     ``realloc is not entirely portable''
@@ -1271,7 +1296,7 @@ static struct lt_user_dlloader sys_shl = {
 
 /* --- LOADLIBRARY() INTERFACE LOADER --- */
 
-#if defined(__WINDOWS__) || defined(__CYGWIN__)
+#ifdef __WINDOWS__
 
 /* dynamic linking for Win32 */
 
@@ -2183,31 +2208,26 @@ lt_dlinit ()
       handles = 0;
       user_search_path = 0; /* empty search path */
 
-      /* Append the available loaders to the internal list in the order
-	 they should be used -- if the first fails, then try again with
-	 the next loader in the chain.  */
-#     define LOADER_APPEND 0
-
-      errors += lt_dlloader_add (LOADER_APPEND, &presym,   "dlpreload");
-#if HAVE_DLD
-      errors += lt_dlloader_add (LOADER_APPEND, &sys_dld,  "dld");
-#endif
-#if HAVE_DYLD
-      errors += lt_dlloader_add (LOADER_APPEND, &sys_dyld, "dyld");
-      errors += sys_dyld_init();
-#endif
-#ifdef __BEOS__
-      errors += lt_dlloader_add (LOADER_APPEND, &sys_bedl, "dlopen");
+#if HAVE_LIBDL
+      errors += lt_dlloader_add (lt_dlloader_next (0), &sys_dl, "dlopen");
 #endif
 #if HAVE_SHL_LOAD
-      errors += lt_dlloader_add (LOADER_APPEND, &sys_shl,  "dlopen");
+      errors += lt_dlloader_add (lt_dlloader_next (0), &sys_shl, "dlopen");
 #endif
-#if HAVE_LIBDL
-      errors += lt_dlloader_add (LOADER_APPEND, &sys_dl,   "dlopen");
+#ifdef __WINDOWS__
+      errors += lt_dlloader_add (lt_dlloader_next (0), &sys_wll, "dlopen");
 #endif
-#if defined(__WINDOWS__) || defined(__CYGWIN__)
-      errors += lt_dlloader_add (LOADER_APPEND, &sys_wll,  "dlopen");
+#ifdef __BEOS__
+      errors += lt_dlloader_add (lt_dlloader_next (0), &sys_bedl, "dlopen");
 #endif
+#if HAVE_DLD
+      errors += lt_dlloader_add (lt_dlloader_next (0), &sys_dld, "dld");
+#endif
+#if HAVE_DYLD
+       errors += lt_dlloader_add (lt_dlloader_next (0), &sys_dyld, "dyld");
+       errors += sys_dyld_init();
+#endif
+      errors += lt_dlloader_add (lt_dlloader_next (0), &presym, "dlpreload");
 
       if (presym_init (presym.dlloader_data))
 	{
