@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -51,9 +51,11 @@
 #include "magick/constitute.h"
 #include "magick/draw.h"
 #include "magick/draw_private.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/gem.h"
 #include "magick/geometry.h"
+#include "magick/image_private.h"
 #include "magick/log.h"
 #include "magick/resource_.h"
 #include "magick/string_.h"
@@ -379,16 +381,22 @@ MagickExport MagickBooleanType AnnotateImage(Image *image,
     }
     if (draw_info->undercolor.opacity != TransparentOpacity)
       {
+        DrawInfo
+          *undercolor_info;
+
         /*
           Text box.
         */
-        annotate_info->fill=draw_info->undercolor;
-        annotate_info->affine.tx=offset.x-draw_info->affine.ry*metrics.ascent;
-        annotate_info->affine.ty=offset.y-draw_info->affine.sy*metrics.ascent;
+        undercolor_info=CloneDrawInfo((ImageInfo *) NULL,(DrawInfo *) NULL);
+        undercolor_info->fill=draw_info->undercolor;
+        undercolor_info->affine=draw_info->affine;
+        undercolor_info->affine.tx=offset.x-draw_info->affine.ry*metrics.ascent;
+        undercolor_info->affine.ty=offset.y-draw_info->affine.sy*metrics.ascent;
         (void) FormatMagickString(primitive,MaxTextExtent,
           "rectangle 0,0 %g,%ld",metrics.width,height);
-        (void) CloneString(&annotate_info->primitive,primitive);
-        (void) DrawImage(image,annotate_info);
+        (void) CloneString(&undercolor_info->primitive,primitive);
+        (void) DrawImage(image,undercolor_info);
+        DestroyDrawInfo(undercolor_info);
       }
     annotate_info->affine.tx=offset.x;
     annotate_info->affine.ty=offset.y;
@@ -427,8 +435,8 @@ MagickExport MagickBooleanType AnnotateImage(Image *image,
   /*
     Free resources.
   */
-  DestroyDrawInfo(annotate_info);
-  DestroyDrawInfo(annotate);
+  annotate_info=DestroyDrawInfo(annotate_info);
+  annotate=DestroyDrawInfo(annotate);
   text=(char *) RelinquishMagickMemory(text);
   for (i=0; textlist[i] != (char *) NULL; i++)
     textlist[i]=(char *) RelinquishMagickMemory(textlist[i]);
@@ -732,6 +740,128 @@ static unsigned short *EncodeUnicode(const Image *image,const char *text,
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   G e t M u l t i l i n e T y p e M e t r i c s                             %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  GetMultilineTypeMetrics() returns the following information for the
+%  specified font and text:
+%
+%    character width
+%    character height
+%    ascent
+%    descent
+%    text width
+%    text height
+%    maximum horizontal advance
+%    underline position
+%    underline thickness
+%
+%  This method is like GetTypeMetrics() but it returns the maximum text width
+%  and height for multiple lines of text.
+%
+%  The format of the GetMultilineTypeMetrics method is:
+%
+%      MagickBooleanType GetMultilineTypeMetrics(Image *image,
+%        const DrawInfo *draw_info,TypeMetric *metrics)
+%
+%  A description of each parameter follows:
+%
+%    o image: The image.
+%
+%    o draw_info: The draw info.
+%
+%    o metrics: Return the font metrics in this structure.
+%
+*/
+MagickExport MagickBooleanType GetMultilineTypeMetrics(Image *image,
+  const DrawInfo *draw_info,TypeMetric *metrics)
+{
+  char
+    *text,
+    **textlist;
+
+  double
+    max_width;
+
+  DrawInfo
+    *annotate_info;
+
+  MagickBooleanType
+    status;
+
+  PointInfo
+    offset;
+
+  register long
+    i;
+
+  unsigned long
+    number_lines;
+
+  assert(image != (Image *) NULL);
+  assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+  assert(draw_info != (DrawInfo *) NULL);
+  assert(draw_info->text != (char *) NULL);
+  assert(draw_info->signature == MagickSignature);
+  if (*draw_info->text == '\0')
+    return(MagickFalse);
+  annotate_info=CloneDrawInfo((ImageInfo *) NULL,draw_info);
+  annotate_info->text=(char *) RelinquishMagickMemory(annotate_info->text);
+  /*
+    Substitute values for formating characters (%x, etc).
+  */
+  text=TranslateText((ImageInfo *) NULL,image,draw_info->text);
+  if (text == (char *) NULL)
+    ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
+      image->filename);
+  /*
+    Convert newlines to multiple lines of text.
+  */
+  textlist=StringToList(text);
+  text=(char *) RelinquishMagickMemory(text);
+  if (textlist == (char **) NULL)
+    return(MagickFalse);
+  annotate_info->render=MagickFalse;
+  (void) ResetMagickMemory(metrics,0,sizeof(*metrics));
+  offset.x=0.0;
+  offset.y=0.0;
+  /*
+    Find the widest of the text lines.
+  */
+  annotate_info->text = textlist[0];
+  status=RenderType(image,annotate_info,&offset,metrics);
+  max_width = metrics->width;
+  for (i=1; textlist[i] != (char *) NULL; i++)
+  {
+    annotate_info->text = textlist[i];
+    status=RenderType(image,annotate_info,&offset,metrics);
+    if (metrics->width > max_width)
+      max_width=metrics->width;
+  }
+  number_lines= i;
+  metrics->width=max_width;
+  metrics->height=number_lines*(long) (metrics->ascent-metrics->descent+0.5)+1;
+  /*
+    Free resources.
+  */
+  annotate_info->text=(char *) NULL;
+  annotate_info=DestroyDrawInfo(annotate_info);
+  for (i=0; textlist[i] != (char *) NULL; i++)
+    textlist[i]=(char *) RelinquishMagickMemory(textlist[i]);
+  textlist=(char **) RelinquishMagickMemory(textlist);
+  return(status);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   G e t T y p e M e t r i c s                                               %
 %                                                                             %
 %                                                                             %
@@ -741,15 +871,15 @@ static unsigned short *EncodeUnicode(const Image *image,const char *text,
 %  GetTypeMetrics() returns the following information for the specified font
 %  and text:
 %
-%    o character width
-%    o character height
-%    o ascent
-%    o descent
-%    o text width
-%    o text height
-%    o maximum horizontal advance
-%    o underline position
-%    o underline thickness
+%    character width
+%    character height
+%    ascent
+%    descent
+%    text width
+%    text height
+%    maximum horizontal advance
+%    underline position
+%    underline thickness
 %
 %  The format of the GetTypeMetrics method is:
 %
@@ -791,7 +921,7 @@ MagickExport MagickBooleanType GetTypeMetrics(Image *image,
   offset.x=0.0;
   offset.y=0.0;
   status=RenderType(image,annotate_info,&offset,metrics);
-  DestroyDrawInfo(annotate_info);
+  annotate_info=DestroyDrawInfo(annotate_info);
   return(status);
 }
 
@@ -845,10 +975,12 @@ static MagickBooleanType RenderType(Image *image,const DrawInfo *draw_info,
         return(RenderFreetype(image,draw_info,(char *) NULL,offset,metrics));
       if (*draw_info->font == '-')
         return(RenderX11(image,draw_info,offset,metrics));
+      if (IsAccessible(draw_info->font) != MagickFalse)
+        return(RenderFreetype(image,draw_info,(char *) NULL,offset,metrics));
       type_info=GetTypeInfo(draw_info->font,&image->exception);
       if (type_info == (const TypeInfo *) NULL)
-        if (IsAccessible(draw_info->font) != MagickFalse)
-          return(RenderFreetype(image,draw_info,(char *) NULL,offset,metrics));
+        (void) ThrowMagickException(&image->exception,GetMagickModule(),
+          TypeWarning,"UnableToReadFont",draw_info->font);
     }
   if (type_info == (const TypeInfo *) NULL)
     type_info=GetTypeInfoByFamily(draw_info->family,draw_info->style,
@@ -868,7 +1000,7 @@ static MagickBooleanType RenderType(Image *image,const DrawInfo *draw_info,
   if (type_info->glyphs != (char *) NULL)
     (void) CloneString(&annotate_info->font,type_info->glyphs);
   status=RenderFreetype(image,annotate_info,type_info->encoding,offset,metrics);
-  DestroyDrawInfo(annotate_info);
+  annotate_info=DestroyDrawInfo(annotate_info);
   return(status);
 }
 
@@ -1127,8 +1259,8 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
   /*
     Set text size.
   */
-  resolution.x=72.0;
-  resolution.y=72.0;
+  resolution.x=DefaultResolution;
+  resolution.y=DefaultResolution;
   if (draw_info->density != (char *) NULL)
     {
       GeometryInfo
@@ -1368,7 +1500,7 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
     Free resources.
   */
   text=(unsigned short *) RelinquishMagickMemory(text);
-  DestroyDrawInfo(annotate_info);
+  annotate_info=DestroyDrawInfo(annotate_info);
   (void) FT_Done_Face(face);
   (void) FT_Done_FreeType(library);
   return(MagickTrue);
@@ -1528,8 +1660,8 @@ static MagickBooleanType RenderPostscript(Image *image,
     if (point.y > extent.y)
       extent.y=point.y;
   }
-  (void) fprintf(file,"%g %g moveto\n",identity  != MagickFalse ? 0.0 : extent.x/2.0,
-    extent.y/2.0);
+  (void) fprintf(file,"%g %g moveto\n",identity  != MagickFalse ? 0.0 :
+    extent.x/2.0,extent.y/2.0);
   (void) fprintf(file,"%g %g scale\n",draw_info->pointsize,
     draw_info->pointsize);
   if ((draw_info->font == (char *) NULL) || (*draw_info->font == '\0'))
@@ -1559,15 +1691,13 @@ static MagickBooleanType RenderPostscript(Image *image,
     (void) CloneString(&annotate_info->density,draw_info->density);
   annotate_info->antialias=draw_info->text_antialias;
   annotate_image=ReadImage(annotate_info,&image->exception);
-  DestroyImageInfo(annotate_info);
+  CatchException(&image->exception);
+  annotate_info=DestroyImageInfo(annotate_info);
   (void) RelinquishUniqueFileResource(filename);
-  if (image->exception.severity != UndefinedException)
-    MagickError(image->exception.severity,image->exception.reason,
-      image->exception.description);
   if (annotate_image == (Image *) NULL)
     return(MagickFalse);
-  resolution.x=72.0;
-  resolution.y=72.0;
+  resolution.x=DefaultResolution;
+  resolution.y=DefaultResolution;
   if (draw_info->density != (char *) NULL)
     {
       GeometryInfo
@@ -1583,21 +1713,21 @@ static MagickBooleanType RenderPostscript(Image *image,
         resolution.y=resolution.x;
     }
   if (identity == MagickFalse)
-    TransformImage(&annotate_image,"0x0",(char *) NULL);
+    (void) TransformImage(&annotate_image,"0x0",(char *) NULL);
   else
     {
       RectangleInfo
         crop_info;
 
       crop_info=GetImageBoundingBox(annotate_image,&annotate_image->exception);
-      crop_info.height=(unsigned long) ((resolution.y/72.0)*
+      crop_info.height=(unsigned long) ((resolution.y/DefaultResolution)*
         ExpandAffine(&draw_info->affine)*draw_info->pointsize+0.5);
-      crop_info.y=(long) ((resolution.y/72.0)*extent.y/8.0+0.5);
+      crop_info.y=(long) ((resolution.y/DefaultResolution)*extent.y/8.0+0.5);
       (void) FormatMagickString(geometry,MaxTextExtent,"%lux%lu%+ld%+ld",
         crop_info.width,crop_info.height,crop_info.x,crop_info.y);
-      TransformImage(&annotate_image,geometry,(char *) NULL);
+      (void) TransformImage(&annotate_image,geometry,(char *) NULL);
     }
-  metrics->pixels_per_em.x=(resolution.y/72.0)*
+  metrics->pixels_per_em.x=(resolution.y/DefaultResolution)*
     ExpandAffine(&draw_info->affine)*draw_info->pointsize;
   metrics->pixels_per_em.y=metrics->pixels_per_em.x;
   metrics->ascent=metrics->pixels_per_em.x;
@@ -1614,7 +1744,7 @@ static MagickBooleanType RenderPostscript(Image *image,
   metrics->underline_thickness=1.0;
   if (draw_info->render == MagickFalse)
     {
-      DestroyImage(annotate_image);
+      annotate_image=DestroyImage(annotate_image);
       return(MagickTrue);
     }
   if (draw_info->fill.opacity != TransparentOpacity)
@@ -1658,7 +1788,7 @@ static MagickBooleanType RenderPostscript(Image *image,
         (long) (offset->x+0.5),(long) (offset->y-(metrics->ascent+
         metrics->descent)+0.5));
     }
-  DestroyImage(annotate_image);
+  annotate_image=DestroyImage(annotate_image);
   return(MagickTrue);
 }
 
