@@ -1,4 +1,18 @@
 /*
+  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization
+  dedicated to making software imaging solutions freely available.
+  
+  You may not use this file except in compliance with the License.
+  obtain a copy of the License at
+  
+    http://www.imagemagick.org/www/Copyright.html
+  
+  Unless required by applicable law or agreed to in writing, software
+  distributed under the License is distributed on an "AS IS" BASIS,
+  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+  See the License for the specific language governing permissions and
+  limitations under the License.
+
   ImageMagick Application Programming Interface declarations.
 */
 
@@ -21,25 +35,30 @@ extern "C" {
 # endif
 #endif
 
-#if defined(HAVE_SYS_TYPES_H)
-#include <sys/types.h>
-#else
-#include <types.h>
-#endif
-#if defined(HAVE_STDINT_H)
-# include <stdint.h>
-#endif
-#if defined(HAVE_INTTYPES_H)
-# include <inttypes.h>
-#endif
-
 #if defined(__cplusplus) || defined(c_plusplus)
 # define storage_class  c_class
 #else
 # define storage_class  class
 #endif
+#undef index
 
-#if defined(WIN32) && !defined(__CYGWIN__)
+#if defined(__CYGWIN32__) && !defined(__CYGWIN__)
+# define __CYGWIN__ __CYGWIN32__
+#endif
+#if defined(__CYGWIN__)
+# if defined(__WINDOWS__)
+#   undef __WINDOWS__
+# endif
+#elif defined(_WIN32)
+# define __WINDOWS__ _WIN32
+#elif defined(WIN32)
+# define __WINDOWS__ WIN32
+#endif
+#if defined(__CYGWIN__) && defined(__WINDOWS__)
+# undef __WINDOWS__
+#endif
+
+#if defined(__WINDOWS__)
 # if defined(_MT) && defined(_DLL) && !defined(_MAGICKDLL_) && !defined(_LIB)
 #  define _MAGICKDLL_
 # endif
@@ -68,12 +87,12 @@ extern "C" {
 # if defined(_DLL) && !defined(_LIB)
 #  define ModuleExport  __declspec(dllexport)
 #  if defined(_VISUALC_)
-#   pragma message( "Magick module DLL export interface" ) 
+#   pragma message( "Magick module DLL export interface" )
 #  endif
 # else
 #  define ModuleExport
 #  if defined(_VISUALC_)
-#   pragma message( "Magick module static interface" ) 
+#   pragma message( "Magick module static interface" )
 #  endif
 
 # endif
@@ -91,41 +110,82 @@ extern "C" {
 # define MagickGlobal
 #endif
 
-#define MaxTextExtent  2053
+#define MaxTextExtent  ((size_t) 4096)
 #define MagickSignature  0xabacadabUL
 
-#include "magick/info.h"
-#include "magick/semaphore.h"
-#include "magick/error.h"
-#include "magick/timer.h"
-#include "magick/image.h"
-#include "magick/list.h"
-#include "magick/paint.h"
-#include "magick/render.h"
-#include "magick/gem.h"
-#include "magick/quantize.h"
-#include "magick/compress.h"
+#if !defined(magick_attribute)
+#  if !defined(__GNUC__)
+#    define magick_attribute(x)  /* nothing */
+#  else
+#    define magick_attribute  __attribute__
+#  endif
+#endif
+
+#if defined(MagickMethodPrefix)
+# include "magick/methods.h"
+#endif
+#include "magick/ImageMagick.h"
+#include "magick/animate.h"
+#include "magick/annotate.h"
 #include "magick/attribute.h"
-#include "magick/command.h"
-#include "magick/hashmap.h"
-#include "magick/utility.h"
-#include "magick/signature.h"
 #include "magick/blob.h"
 #include "magick/cache.h"
 #include "magick/cache_view.h"
-#include "magick/registry.h"
-#include "magick/magick.h"
-#include "magick/magic.h"
+#include "magick/coder.h"
+#include "magick/client.h"
+#include "magick/color.h"
+#include "magick/colorspace.h"
+#include "magick/compare.h"
+#include "magick/composite.h"
+#include "magick/compress.h"
+#include "magick/configure.h"
+#include "magick/constitute.h"
+#include "magick/convert.h"
+#include "magick/decorate.h"
 #include "magick/delegate.h"
-#include "magick/module.h"
-#include "magick/monitor.h"
-#include "magick/resource.h"
-#include "magick/log.h"
-#include "magick/version.h"
 #include "magick/deprecate.h"
-#include "magick/drawing_wand.h"
-#include "magick/pixel_wand.h"
-#include "magick/magick_wand.h"
+#include "magick/display.h"
+#include "magick/draw.h"
+#include "magick/effect.h"
+#include "magick/enhance.h"
+#include "magick/exception.h"
+#include "magick/fx.h"
+#include "magick/gem.h"
+#include "magick/geometry.h"
+#include "magick/hashmap.h"
+#include "magick/identify.h"
+#include "magick/image.h"
+#include "magick/import.h"
+#include "magick/list.h"
+#include "magick/locale_.h"
+#include "magick/log.h"
+#include "magick/magic.h"
+#include "magick/magick.h"
+#include "magick/memory_.h"
+#include "magick/module.h"
+#include "magick/mogrify.h"
+#include "magick/monitor.h"
+#include "magick/montage.h"
+#include "magick/option.h"
+#include "magick/paint.h"
+#include "magick/pixel.h"
+#include "magick/profile.h"
+#include "magick/quantize.h"
+#include "magick/registry.h"
+#include "magick/random.h"
+#include "magick/resize.h"
+#include "magick/resource_.h"
+#include "magick/segment.h"
+#include "magick/shear.h"
+#include "magick/signature.h"
+#include "magick/stream.h"
+#include "magick/string_.h"
+#include "magick/timer.h"
+#include "magick/token.h"
+#include "magick/transform.h"
+#include "magick/type.h"
+#include "magick/utility.h"
+#include "magick/version.h"
 
 #if defined(__cplusplus) || defined(c_plusplus)
 }
