@@ -17,7 +17,7 @@
 %                              February 2000                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -46,6 +46,7 @@
 #include "magick/attribute.h"
 #include "magick/blob.h"
 #include "magick/draw.h"
+#include "magick/exception_private.h"
 #include "magick/list.h"
 #include "magick/memory_.h"
 #include "magick/profile.h"
@@ -288,7 +289,7 @@ static MagickBooleanType Generate8BIMAttribute(Image *image,const char *key)
   /*
     There's no newlines in path names, so it's safe as terminator.
   */
-  profile=GetImageProfile(image,"8bim");
+  profile=GetImageProfile(image,"iptc");
   if (profile == (StringInfo *) NULL)
     return(MagickFalse);
   count=(ssize_t) sscanf(key,"8BIM:%ld,%ld:%[^\n]\n%[^\n]",&start,&stop,name,
@@ -955,11 +956,11 @@ static int GenerateEXIFAttribute(Image *image,const char *specification)
         }
         if ((t == TAG_EXIF_OFFSET) || (t == TAG_INTEROP_OFFSET))
           {
-            long
+            size_t
               offset;
 
-            offset=(long) ReadUint32(msb_order,pval);
-            if ((offset < (long) length) || (level < (DE_STACK_SIZE-2)))
+            offset=(size_t) ReadUint32(msb_order,pval);
+            if ((offset < length) && (level < (DE_STACK_SIZE-2)))
               {
                 /*
                   Push our current directory state onto the stack.
@@ -1225,7 +1226,7 @@ MagickExport const ImageAttribute *GetImageInfoAttribute(
           p=image;
           for (page=1; p->previous != (Image *) NULL; page++)
             p=p->previous;
-          (void) FormatMagickString(attribute,MaxTextExtent,"%u",page);
+          (void) FormatMagickString(attribute,MaxTextExtent,"%lu",page);
           break;
         }
       break;
@@ -1325,6 +1326,9 @@ MagickExport  MagickBooleanType SetImageAttribute(Image *image,const char *key,
   ImageAttribute
     *attribute;
 
+  register const char
+    *q;
+
   register ImageAttribute
     *p;
 
@@ -1367,13 +1371,15 @@ MagickExport  MagickBooleanType SetImageAttribute(Image *image,const char *key,
     }
   if (*value == '\0')
     return(MagickFalse);
-  attribute=(ImageAttribute *) AcquireMagickMemory(sizeof(ImageAttribute));
+  attribute=(ImageAttribute *) AcquireMagickMemory(sizeof(*attribute));
   if (attribute == (ImageAttribute *) NULL)
     return(MagickFalse);
   attribute->key=AcquireString(key);
-  if ((LocaleNCompare(key,"exif",4) == 0) ||
-      (LocaleNCompare(key,"iptc",4) == 0) ||
-      (LocaleNCompare(key,"8bim",4) == 0))
+  for (q=value; *q != '\0'; q++)
+    if (((int) ((unsigned char) *q) < 32) &&
+        (isspace((int) ((unsigned char) *q)) == 0))
+      break;
+  if (*q != '\0')
     attribute->value=AcquireString(value);
   else
     attribute->value=TranslateText((ImageInfo *) NULL,image,value);
@@ -1437,7 +1443,8 @@ MagickExport  MagickBooleanType SetImageAttribute(Image *image,const char *key,
 %
 */
 static char *TracePSClippath(unsigned char *blob,size_t length,
-  const unsigned long columns,const unsigned long rows)
+  const unsigned long magick_unused(columns),
+  const unsigned long magick_unused(rows))
 {
   char
     *path,
