@@ -23,7 +23,7 @@
 %                               February 2000                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -48,9 +48,11 @@
 */
 #include "magick/studio.h"
 #include "magick/cache.h"
+#include "magick/cache_private.h"
 #include "magick/cache_view.h"
 #include "magick/memory_.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/string_.h"
 
 /*
@@ -136,6 +138,8 @@ MagickExport void CloseCacheView(ViewInfo *view_info)
       view_info->image->filename);
   if (view_info->id != 0)
     DestroyCacheNexus(view_info->image->cache,view_info->id);
+  view_info->image=DestroyImage(view_info->image);
+  view_info->signature=(~MagickSignature);
   view_info=(ViewInfo *) RelinquishMagickMemory(view_info);
 }
 
@@ -289,8 +293,8 @@ MagickExport ViewInfo *OpenCacheView(Image *image)
     ThrowMagickFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
       image->filename);
   (void) ResetMagickMemory(view_info,0,sizeof(ViewInfo));
-  view_info->id=GetNexus(image->cache);
-  view_info->image=image;
+  view_info->image=ReferenceImage(image);
+  view_info->id=GetNexus(view_info->image->cache);
   view_info->signature=MagickSignature;
   return(view_info);
 }
