@@ -17,7 +17,7 @@
 %                                 May 2001                                    %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -45,7 +45,8 @@
 #include "magick/coder.h"
 #include "magick/configure.h"
 #include "magick/draw.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/hashmap.h"
 #include "magick/log.h"
 #include "magick/memory_.h"
@@ -59,7 +60,7 @@
 /*
   Define declarations.
 */
-#define MagickCoderFilename  "coder.mgk"
+#define MagickCoderFilename  "coder.xml"
 
 /*
   Declare coder map.
@@ -220,13 +221,12 @@ static void *DestroyCoderElement(void *coder_info)
 
 MagickExport void DestroyCoderList(void)
 {
-  AcquireSemaphoreInfo(&coder_semaphore);
+  coder_semaphore=AcquireSemaphoreInfo(coder_semaphore);
   if (coder_list != (LinkedListInfo *) NULL)
-    DestroyLinkedList(coder_list,DestroyCoderElement);
-  coder_list=(LinkedListInfo *) NULL;
+    coder_list=DestroyLinkedList(coder_list,DestroyCoderElement);
   instantiate_coder=MagickFalse;
-  RelinquishSemaphoreInfo(&coder_semaphore);
-  DestroySemaphoreInfo(&coder_semaphore);
+  RelinquishSemaphoreInfo(coder_semaphore);
+  coder_semaphore=DestroySemaphoreInfo(coder_semaphore);
 }
 
 /*
@@ -261,6 +261,7 @@ MagickExport const CoderInfo *GetCoderInfo(const char *name,
   register const CoderInfo
     *p;
 
+  assert(exception != (ExceptionInfo *) NULL);
   if ((coder_list == (LinkedListInfo *) NULL) ||
       (instantiate_coder == MagickFalse))
     if (InitializeCoderList(exception) == MagickFalse)
@@ -273,7 +274,7 @@ MagickExport const CoderInfo *GetCoderInfo(const char *name,
   /*
     Search for requested coder.
   */
-  AcquireSemaphoreInfo(&coder_semaphore);
+  coder_semaphore=AcquireSemaphoreInfo(coder_semaphore);
   ResetLinkedListIterator(coder_list);
   p=(const CoderInfo *) GetNextElementInLinkedList(coder_list);
   while (p != (const CoderInfo *) NULL)
@@ -282,7 +283,7 @@ MagickExport const CoderInfo *GetCoderInfo(const char *name,
       break;
     p=(const CoderInfo *) GetNextElementInLinkedList(coder_list);
   }
-  RelinquishSemaphoreInfo(&coder_semaphore);
+  RelinquishSemaphoreInfo(coder_semaphore);
   return(p);
 }
 
@@ -302,7 +303,7 @@ MagickExport const CoderInfo *GetCoderInfo(const char *name,
 %  The format of the GetCoderInfoList function is:
 %
 %      const CoderInfo **GetCoderInfoList(const char *pattern,
-%        unsigned long *number_coders)
+%        unsigned long *number_coders,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
@@ -310,6 +311,8 @@ MagickExport const CoderInfo *GetCoderInfo(const char *name,
 %
 %    o number_coders:  This integer returns the number of coders in the list.
 %
+%    o exception: Return any errors or warnings in this structure.
+%
 %
 */
 
@@ -327,14 +330,11 @@ static int CoderInfoCompare(const void *x,const void *y)
 }
 
 MagickExport const CoderInfo **GetCoderInfoList(const char *pattern,
-  unsigned long *number_coders)
+  unsigned long *number_coders,ExceptionInfo *exception)
 {
   const CoderInfo
     **coders;
 
-  ExceptionInfo
-    exception;
-
   register const CoderInfo
     *p;
 
@@ -348,9 +348,7 @@ MagickExport const CoderInfo **GetCoderInfoList(const char *pattern,
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),pattern);
   assert(number_coders != (unsigned long *) NULL);
   *number_coders=0;
-  GetExceptionInfo(&exception);
-  p=GetCoderInfo("*",&exception);
-  DestroyExceptionInfo(&exception);
+  p=GetCoderInfo("*",exception);
   if (p == (const CoderInfo *) NULL)
     return((const CoderInfo **) NULL);
   coders=(const CoderInfo **) AcquireMagickMemory((size_t)
@@ -360,7 +358,7 @@ MagickExport const CoderInfo **GetCoderInfoList(const char *pattern,
   /*
     Generate coder list.
   */
-  AcquireSemaphoreInfo(&coder_semaphore);
+  coder_semaphore=AcquireSemaphoreInfo(coder_semaphore);
   ResetLinkedListIterator(coder_list);
   p=(const CoderInfo *) GetNextElementInLinkedList(coder_list);
   for (i=0; p != (const CoderInfo *) NULL; )
@@ -370,7 +368,7 @@ MagickExport const CoderInfo **GetCoderInfoList(const char *pattern,
       coders[i++]=p;
     p=(const CoderInfo *) GetNextElementInLinkedList(coder_list);
   }
-  RelinquishSemaphoreInfo(&coder_semaphore);
+  RelinquishSemaphoreInfo(coder_semaphore);
   qsort((void *) coders,(size_t) i,sizeof(*coders),CoderInfoCompare);
   coders[i]=(CoderInfo *) NULL;
   *number_coders=(unsigned long) i;
@@ -392,7 +390,8 @@ MagickExport const CoderInfo **GetCoderInfoList(const char *pattern,
 %
 %  The format of the GetCoderList function is:
 %
-%      char **GetCoderList(const char *pattern,unsigned long *number_coders)
+%      char **GetCoderList(const char *pattern,unsigned long *number_coders,
+%        ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
@@ -400,6 +399,7 @@ MagickExport const CoderInfo **GetCoderInfoList(const char *pattern,
 %
 %    o number_coders:  This integer returns the number of coders in the list.
 %
+%    o exception: Return any errors or warnings in this structure.
 %
 */
 
@@ -415,14 +415,11 @@ static int CoderCompare(const void *x,const void *y)
 }
 
 MagickExport char **GetCoderList(const char *pattern,
-  unsigned long *number_coders)
+  unsigned long *number_coders,ExceptionInfo *exception)
 {
   char
     **coders;
 
-  ExceptionInfo
-    exception;
-
   register const CoderInfo
     *p;
 
@@ -436,9 +433,7 @@ MagickExport char **GetCoderList(const char *pattern,
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),pattern);
   assert(number_coders != (unsigned long *) NULL);
   *number_coders=0;
-  GetExceptionInfo(&exception);
-  p=GetCoderInfo("*",&exception);
-  DestroyExceptionInfo(&exception);
+  p=GetCoderInfo("*",exception);
   if (p == (const CoderInfo *) NULL)
     return((char **) NULL);
   coders=(char **) AcquireMagickMemory((size_t)
@@ -448,7 +443,7 @@ MagickExport char **GetCoderList(const char *pattern,
   /*
     Generate coder list.
   */
-  AcquireSemaphoreInfo(&coder_semaphore);
+  coder_semaphore=AcquireSemaphoreInfo(coder_semaphore);
   ResetLinkedListIterator(coder_list);
   p=(const CoderInfo *) GetNextElementInLinkedList(coder_list);
   for (i=0; p != (const CoderInfo *) NULL; )
@@ -458,7 +453,7 @@ MagickExport char **GetCoderList(const char *pattern,
       coders[i++]=AcquireString(p->name);
     p=(const CoderInfo *) GetNextElementInLinkedList(coder_list);
   }
-  RelinquishSemaphoreInfo(&coder_semaphore);
+  RelinquishSemaphoreInfo(coder_semaphore);
   qsort((void *) coders,(size_t) i,sizeof(*coders),CoderCompare);
   coders[i]=(char *) NULL;
   *number_coders=(unsigned long) i;
@@ -489,23 +484,17 @@ MagickExport char **GetCoderList(const char *pattern,
 */
 static MagickBooleanType InitializeCoderList(ExceptionInfo *exception)
 {
-  MagickBooleanType
-    active;
-
-  active=MagickFalse;
   if ((coder_list == (LinkedListInfo *) NULL) &&
       (instantiate_coder == MagickFalse))
     {
-      AcquireSemaphoreInfo(&coder_semaphore);
+      coder_semaphore=AcquireSemaphoreInfo(coder_semaphore);
       if ((coder_list == (LinkedListInfo *) NULL) &&
           (instantiate_coder == MagickFalse))
         {
+          (void) LoadCoderLists(MagickCoderFilename,exception);
           instantiate_coder=MagickTrue;
-          active=MagickTrue;
         }
-      RelinquishSemaphoreInfo(&coder_semaphore);
-      if (active == MagickTrue)
-        (void) LoadCoderLists(MagickCoderFilename,exception);
+      RelinquishSemaphoreInfo(coder_semaphore);
     }
   return(coder_list != (LinkedListInfo *) NULL ? MagickTrue : MagickFalse);
 }
@@ -555,7 +544,7 @@ MagickExport MagickBooleanType ListCoderInfo(FILE *file,
 
   if (file == (const FILE *) NULL)
     file=stdout;
-  coder_info=GetCoderInfoList("*",&number_coders);
+  coder_info=GetCoderInfoList("*",&number_coders,exception);
   if (coder_info == (const CoderInfo **) NULL)
     return(MagickFalse);
   path=(const char *) NULL;
@@ -806,6 +795,9 @@ static MagickBooleanType LoadCoderList(const char *xml,const char *filename,
 static MagickBooleanType LoadCoderLists(const char *filename,
   ExceptionInfo *exception)
 {
+  char
+    path[MaxTextExtent];
+
   const StringInfo
     *option;
 
@@ -816,17 +808,23 @@ static MagickBooleanType LoadCoderLists(const char *filename,
     status;
 
   status=MagickFalse;
+  *path='\0';
   options=GetConfigureOptions(filename,exception);
   option=(const StringInfo *) GetNextElementInLinkedList(options);
   while (option != (const StringInfo *) NULL)
   {
+    (void) CopyMagickString(path,option->path,MaxTextExtent);
     status|=LoadCoderList((const char *) option->datum,option->path,0,
       exception);
     option=(const StringInfo *) GetNextElementInLinkedList(options);
   }
-  DestroyConfigureOptions(options);
+  options=DestroyConfigureOptions(options);
   if ((coder_list == (LinkedListInfo *) NULL) || 
       (IsLinkedListEmpty(coder_list) != MagickFalse))
-    status|=LoadCoderList(CoderMap,"built-in",0,exception);
+    {
+      (void) ThrowMagickException(exception,GetMagickModule(),ConfigureError,
+        "UnableToOpenConfigureFile",path);
+      status|=LoadCoderList(CoderMap,"built-in",0,exception);
+    }
   return(status != 0 ? MagickTrue : MagickFalse);
 }
