@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -42,7 +42,8 @@
 #include "magick/studio.h"
 #include "magick/color.h"
 #include "magick/colorspace.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/image.h"
 #include "magick/gem.h"
 #include "magick/memory_.h"
@@ -632,9 +633,11 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
         }
         if (SyncImagePixels(image) == MagickFalse)
           break;
-        if (QuantumTick(y,image->rows) != MagickFalse)
+        if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+            (QuantumTick(y,image->rows) != MagickFalse))
           {
-            status=MagickMonitor(RGBTransformImageTag,y,image->rows,exception);
+            status=image->progress_monitor(RGBTransformImageTag,y,image->rows,
+              image->client_data);
             if (status == MagickFalse)
               break;
           }
@@ -920,9 +923,9 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
           break;
         for (x=(long) image->columns-1; x >= 0; x--)
         {
-          hue=QuantumScale*q->red;
-          saturation=QuantumScale*q->green;
-          luminosity=QuantumScale*q->blue;
+          hue=(double) (QuantumScale*q->red);
+          saturation=(double) (QuantumScale*q->green);
+          luminosity=(double) (QuantumScale*q->blue);
           HSBTransform(hue,saturation,luminosity,&q->red,&q->green,&q->blue);
           q++;
         }
@@ -954,9 +957,9 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
           break;
         for (x=(long) image->columns-1; x >= 0; x--)
         {
-          hue=QuantumScale*q->red;
-          saturation=QuantumScale*q->green;
-          luminosity=QuantumScale*q->blue;
+          hue=(double) (QuantumScale*q->red);
+          saturation=(double) (QuantumScale*q->green);
+          luminosity=(double) (QuantumScale*q->blue);
           HSLTransform(hue,saturation,luminosity,&q->red,&q->green,&q->blue);
           q++;
         }
@@ -988,9 +991,9 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
           break;
         for (x=(long) image->columns-1; x >= 0; x--)
         {
-          hue=QuantumScale*q->red;
-          whiteness=QuantumScale*q->green;
-          blackness=QuantumScale*q->blue;
+          hue=(double) (QuantumScale*q->red);
+          whiteness=(double) (QuantumScale*q->green);
+          blackness=(double) (QuantumScale*q->blue);
           HWBTransform(hue,whiteness,blackness,&q->red,&q->green,&q->blue);
           q++;
         }
@@ -1310,9 +1313,11 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
         }
         if (SyncImagePixels(image) == MagickFalse)
           break;
-        if (QuantumTick(y,image->rows) != MagickFalse)
+        if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+            (QuantumTick(y,image->rows) != MagickFalse))
           {
-            status=MagickMonitor(TransformRGBImageTag,y,image->rows,exception);
+            status=image->progress_monitor(TransformRGBImageTag,y,image->rows,
+              image->client_data);
             if (status == MagickFalse)
               break;
           }
