@@ -17,7 +17,7 @@
 %                               December 2001                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -68,7 +68,8 @@
 #include "magick/delegate.h"
 #include "magick/draw.h"
 #include "magick/effect.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/gem.h"
 #include "magick/geometry.h"
 #include "magick/image.h"
@@ -163,14 +164,16 @@ MagickExport MagickBooleanType ConjureImageCommand(ImageInfo *image_info,
 {
 #define DestroyConjure() \
 { \
-  DestroyImageList(image); \
+  image=DestroyImageList(image); \
   for (i=0; i < (long) argc; i++) \
     argv[i]=(char *) RelinquishMagickMemory(argv[i]); \
   argv=(char **) RelinquishMagickMemory(argv); \
 }
-#define ThrowConjureException(severity,tag,option) \
+#define ThrowConjureException(asperity,tag,option) \
 { \
-  (void) ThrowMagickException(exception,GetMagickModule(),severity,tag,option);\
+  if (exception->severity < (asperity)) \
+    (void) ThrowMagickException(exception,GetMagickModule(),asperity,tag, \
+      option); \
   DestroyConjure(); \
   return(MagickFalse); \
 }
@@ -203,7 +206,7 @@ MagickExport MagickBooleanType ConjureImageCommand(ImageInfo *image_info,
   assert(image_info != (ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(exception != (ExceptionInfo *) NULL);
   if (argc < 2)
     ConjureUsage();
@@ -286,18 +289,18 @@ MagickExport MagickBooleanType ConjureImageCommand(ImageInfo *image_info,
     (void) FormatMagickString(image_info->filename,MaxTextExtent,"msl:%s",
       argv[i]);
     image=ReadImage(image_info,exception);
-    if (exception->severity != UndefinedException)
-      CatchException(exception);
+    CatchException(exception);
     status&=image != (Image *) NULL;
     if (image != (Image *) NULL)
-      DestroyImageList(image);
+      image=DestroyImageList(image);
     number_images++;
   }
   if (i != argc)
     ThrowConjureException(OptionError,"MissingAnImageFilename",argv[i]);
   if (number_images == 0)
     ThrowConjureException(OptionError,"MissingAnImageFilename",argv[argc-1]);
-  DestroyImageList(image);
+  if (image != (Image *) NULL)
+    image=DestroyImageList(image);
   for (i=0; i < (long) argc; i++)
     argv[i]=(char *) RelinquishMagickMemory(argv[i]);
   argv=(char **) RelinquishMagickMemory(argv);
