@@ -17,7 +17,7 @@
 %                                   July 1992                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -41,8 +41,12 @@
   Include declarations.
 */
 #include "magick/studio.h"
+#include "magick/color_private.h"
+#include "magick/colorspace_private.h"
+#include "magick/composite.h"
 #include "magick/decorate.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/image.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
@@ -104,7 +108,7 @@ MagickExport Image *BorderImage(const Image *image,
     return((Image *) NULL);
   clone_image->matte_color=image->border_color;
   border_image=FrameImage(clone_image,&frame_info,exception);
-  DestroyImage(clone_image);
+  clone_image=DestroyImage(clone_image);
   if (border_image != (Image *) NULL)
     border_image->matte_color=image->matte_color;
   return(border_image);
@@ -153,8 +157,19 @@ MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
   long
     y;
 
-  register const PixelPacket
-    *p;
+  MagickBooleanType
+    status;
+
+  MagickPixelPacket
+    accentuate,
+    border,
+    highlight,
+    matte,
+    shadow,
+    trough;
+
+  register IndexPacket
+    *frame_indexes;
 
   register long
     x;
@@ -162,13 +177,6 @@ MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
   register PixelPacket
     *q;
 
-  PixelPacket
-    accentuate,
-    highlight,
-    matte,
-    shadow,
-    trough;
-
   unsigned long
     bevel_width,
     height,
@@ -197,91 +205,148 @@ MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
   frame_image->storage_class=DirectClass;
   if (frame_image->matte_color.opacity != OpaqueOpacity)
     frame_image->matte=MagickTrue;
+  frame_image->page=image->page;
+  if ((image->page.width != 0) && (image->page.height != 0))
+    {
+      frame_image->page.width+=frame_image->columns-image->columns;
+      frame_image->page.height+=frame_image->rows-image->rows;
+    }
   /*
     Initialize 3D effects color.
   */
-  matte=image->matte_color;
-  accentuate.red=(Quantum) (QuantumScale*(((MagickRealType) MaxRGB-
-    AccentuateModulate)*matte.red+((MagickRealType) MaxRGB*
-    AccentuateModulate))+0.5);
-  accentuate.green=(Quantum) (QuantumScale*(((MagickRealType) MaxRGB-
-    AccentuateModulate)*matte.green+((MagickRealType) MaxRGB*
-    AccentuateModulate))+0.5);
-  accentuate.blue=(Quantum) (QuantumScale*(((MagickRealType) MaxRGB-
-    AccentuateModulate)*matte.blue+((MagickRealType) MaxRGB*
-    AccentuateModulate))+0.5);
+  GetMagickPixelPacket(image,&matte);
+  matte.colorspace=RGBColorspace;
+  SetMagickPixelPacket(&image->matte_color,(IndexPacket *) NULL,&matte);
+  GetMagickPixelPacket(image,&border);
+  border.colorspace=RGBColorspace;
+  SetMagickPixelPacket(&image->border_color,(IndexPacket *) NULL,&border);
+  GetMagickPixelPacket(image,&accentuate);
+  accentuate.red=QuantumScale*((MaxRGB-AccentuateModulate)*matte.red+
+    (MaxRGB*AccentuateModulate));
+  accentuate.green=QuantumScale*((MaxRGB-AccentuateModulate)*matte.green+
+    (MaxRGB*AccentuateModulate));
+  accentuate.blue=QuantumScale*((MaxRGB-AccentuateModulate)*matte.blue+
+    (MaxRGB*AccentuateModulate));
   accentuate.opacity=matte.opacity;
-  highlight.red=(Quantum) (QuantumScale*(((MagickRealType) MaxRGB-
-    HighlightModulate)*matte.red+((MagickRealType) MaxRGB*
-    HighlightModulate))+0.5);
-  highlight.green=(Quantum) (QuantumScale*(((MagickRealType) MaxRGB-
-    HighlightModulate)*matte.green+((MagickRealType) MaxRGB*
-    HighlightModulate))+0.5);
-  highlight.blue=(Quantum) (QuantumScale*(((MagickRealType) MaxRGB-
-    HighlightModulate)*matte.blue+((MagickRealType) MaxRGB*
-    HighlightModulate))+0.5);
+  GetMagickPixelPacket(image,&highlight);
+  highlight.red=QuantumScale*((MaxRGB-HighlightModulate)*matte.red+
+    (MaxRGB*HighlightModulate));
+  highlight.green=QuantumScale*((MaxRGB-HighlightModulate)*matte.green+
+    (MaxRGB*HighlightModulate));
+  highlight.blue=QuantumScale*((MaxRGB-HighlightModulate)*matte.blue+
+    (MaxRGB*HighlightModulate));
   highlight.opacity=matte.opacity;
-  shadow.red=(Quantum) (QuantumScale*((MagickRealType) matte.red*
-    ShadowModulate)+0.5);
-  shadow.green=(Quantum) (QuantumScale*((MagickRealType) matte.green*
-    ShadowModulate)+0.5);
-  shadow.blue=(Quantum) (QuantumScale*((MagickRealType) matte.blue*
-    ShadowModulate)+0.5);
+  GetMagickPixelPacket(image,&shadow);
+  shadow.red=QuantumScale*matte.red*ShadowModulate;
+  shadow.green=QuantumScale*matte.green*ShadowModulate;
+  shadow.blue=QuantumScale*matte.blue*ShadowModulate;
   shadow.opacity=matte.opacity;
-  trough.red=(Quantum) (QuantumScale*((MagickRealType) matte.red*
-    TroughModulate)+0.5);
-  trough.green=(Quantum) (QuantumScale*((MagickRealType) matte.green*
-    TroughModulate)+0.5);
-  trough.blue=(Quantum) (QuantumScale*((MagickRealType) matte.blue*
-    TroughModulate)+0.5);
+  GetMagickPixelPacket(image,&trough);
+  trough.red=QuantumScale*matte.red*TroughModulate;
+  trough.green=QuantumScale*matte.green*TroughModulate;
+  trough.blue=QuantumScale*matte.blue*TroughModulate;
   trough.opacity=matte.opacity;
-  /*
-    Draw top of ornamental border.
-  */
+  if (image->colorspace == CMYKColorspace)
+    {
+      RGBtoCMYK(&matte);
+      RGBtoCMYK(&border);
+      RGBtoCMYK(&accentuate);
+      RGBtoCMYK(&highlight);
+      RGBtoCMYK(&shadow);
+      RGBtoCMYK(&trough);
+    }
   height=(unsigned long) (frame_info->outer_bevel+(frame_info->y-bevel_width)+
     frame_info->inner_bevel);
   q=SetImagePixels(frame_image,0,0,frame_image->columns,height);
+  frame_indexes=GetIndexes(frame_image);
   if (q != (PixelPacket *) NULL)
     {
+      /*
+        Draw top of ornamental border.
+      */
       for (y=0; y < (long) frame_info->outer_bevel; y++)
       {
         for (x=0; x < (long) (frame_image->columns-y); x++)
+        {
           if (x < y)
-            *q++=highlight;
+            SetPixelPacket(&highlight,q,frame_indexes);
           else
-            *q++=accentuate;
+            SetPixelPacket(&accentuate,q,frame_indexes);
+          q++;
+          frame_indexes++;
+        }
         for ( ; x < (long) frame_image->columns; x++)
-          *q++=shadow;
+        {
+          SetPixelPacket(&shadow,q,frame_indexes);
+          q++;
+          frame_indexes++;
+        }
       }
       for (y=0; y < (long) (frame_info->y-bevel_width); y++)
       {
         for (x=0; x < (long) frame_info->outer_bevel; x++)
-          *q++=highlight;
+        {
+          SetPixelPacket(&highlight,q,frame_indexes);
+          q++;
+          frame_indexes++;
+        }
         width=frame_image->columns-2*frame_info->outer_bevel;
         for (x=0; x < (long) width; x++)
-          *q++=matte;
+        {
+          SetPixelPacket(&matte,q,frame_indexes);
+          q++;
+          frame_indexes++;
+        }
         for (x=0; x < (long) frame_info->outer_bevel; x++)
-          *q++=shadow;
+        {
+          SetPixelPacket(&shadow,q,frame_indexes);
+          q++;
+          frame_indexes++;
+        }
       }
       for (y=0; y < (long) frame_info->inner_bevel; y++)
       {
         for (x=0; x < (long) frame_info->outer_bevel; x++)
-          *q++=highlight;
+        {
+          SetPixelPacket(&highlight,q,frame_indexes);
+          q++;
+          frame_indexes++;
+        }
         for (x=0; x < (long) (frame_info->x-bevel_width); x++)
-          *q++=matte;
+        {
+          SetPixelPacket(&matte,q,frame_indexes);
+          q++;
+          frame_indexes++;
+        }
         width=image->columns+((unsigned long) frame_info->inner_bevel << 1)-y;
         for (x=0; x < (long) width; x++)
+        {
           if (x < y)
-            *q++=shadow;
+            SetPixelPacket(&shadow,q,frame_indexes);
           else
-            *q++=trough;
+            SetPixelPacket(&trough,q,frame_indexes);
+          q++;
+          frame_indexes++;
+        }
         for ( ; x < (long) (image->columns+2*frame_info->inner_bevel); x++)
-          *q++=highlight;
+        {
+          SetPixelPacket(&highlight,q,frame_indexes);
+          q++;
+          frame_indexes++;
+        }
         width=frame_info->width-frame_info->x-image->columns-bevel_width;
         for (x=0; x < (long) width; x++)
-          *q++=matte;
+        {
+          SetPixelPacket(&matte,q,frame_indexes);
+          q++;
+          frame_indexes++;
+        }
         for (x=0; x < (long) frame_info->outer_bevel; x++)
-          *q++=shadow;
+        {
+          SetPixelPacket(&shadow,q,frame_indexes);
+          q++;
+          frame_indexes++;
+        }
       }
       (void) SyncImagePixels(frame_image);
     }
@@ -291,85 +356,170 @@ MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
   for (y=0; y < (long) image->rows; y++)
   {
     /*
-      Initialize scanline with border color.
+      Initialize scanline with matte color.
     */
-    p=AcquireImagePixels(image,0,y,image->columns,1,exception);
     q=SetImagePixels(frame_image,0,frame_info->y+y,frame_image->columns,1);
-    if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
+    if (q == (PixelPacket *) NULL)
       break;
+    frame_indexes=GetIndexes(frame_image);
     for (x=0; x < (long) frame_info->outer_bevel; x++)
-      *q++=highlight;
+    {
+      SetPixelPacket(&highlight,q,frame_indexes);
+      q++;
+      frame_indexes++;
+    }
     for (x=0; x < (long) (frame_info->x-bevel_width); x++)
-      *q++=matte;
+    {
+      SetPixelPacket(&matte,q,frame_indexes);
+      q++;
+      frame_indexes++;
+    }
     for (x=0; x < (long) frame_info->inner_bevel; x++)
-      *q++=shadow;
+    {
+      SetPixelPacket(&shadow,q,frame_indexes);
+      q++;
+      frame_indexes++;
+    }
     /*
-      Transfer scanline.
+      Set frame interior to border color.
     */
-    (void) CopyMagickMemory(q,p,(size_t) image->columns*sizeof(*q));
-    q+=image->columns;
+    for (x=0; x < (long) image->columns; x++)
+    {
+      SetPixelPacket(&border,q,frame_indexes);
+      q++;
+      frame_indexes++;
+    }
     for (x=0; x < (long) frame_info->inner_bevel; x++)
-      *q++=highlight;
+    {
+      SetPixelPacket(&highlight,q,frame_indexes);
+      q++;
+      frame_indexes++;
+    }
     width=frame_info->width-frame_info->x-image->columns-bevel_width;
     for (x=0; x < (long) width; x++)
-      *q++=matte;
+    {
+      SetPixelPacket(&matte,q,frame_indexes);
+      q++;
+      frame_indexes++;
+    }
     for (x=0; x < (long) frame_info->outer_bevel; x++)
-      *q++=shadow;
+    {
+      SetPixelPacket(&shadow,q,frame_indexes);
+      q++;
+      frame_indexes++;
+    }
     if (SyncImagePixels(frame_image) == MagickFalse)
       break;
-    if (QuantumTick(y,image->rows) != MagickFalse)
-      if (MagickMonitor(FrameImageTag,y,image->rows,exception) == MagickFalse)
-        break;
+    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+        (QuantumTick(y,image->rows) != MagickFalse))
+      {
+        status=image->progress_monitor(FrameImageTag,y,image->rows,
+          image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
   }
-  /*
-    Draw bottom of ornamental border.
-  */
   height=(unsigned long) (frame_info->inner_bevel+frame_info->height-
     frame_info->y-image->rows-bevel_width+frame_info->outer_bevel);
   q=SetImagePixels(frame_image,0,(long) (frame_image->rows-height),
     frame_image->columns,height);
-  if (q == (PixelPacket *) NULL)
-    return(frame_image);
-  for (y=frame_info->inner_bevel-1; y >= 0; y--)
-  {
-    for (x=0; x < (long) frame_info->outer_bevel; x++)
-      *q++=highlight;
-    for (x=0; x < (long) (frame_info->x-bevel_width); x++)
-      *q++=matte;
-    for (x=0; x < y; x++)
-      *q++=shadow;
-    for ( ; x < (long) (image->columns+2*frame_info->inner_bevel); x++)
-      if (x >= (long) (image->columns+2*frame_info->inner_bevel-y))
-        *q++=highlight;
-      else
-        *q++=accentuate;
-    width=frame_info->width-frame_info->x-image->columns-bevel_width;
-    for (x=0; x < (long) width; x++)
-      *q++=matte;
-    for (x=0; x < (long) frame_info->outer_bevel; x++)
-      *q++=shadow;
-  }
-  height=frame_info->height-frame_info->y-image->rows-bevel_width;
-  for (y=0; y < (long) height; y++)
-  {
-    for (x=0; x < (long) frame_info->outer_bevel; x++)
-      *q++=highlight;
-    for (x=0; x < (long) (frame_image->columns-2*frame_info->outer_bevel); x++)
-      *q++=matte;
-    for (x=0; x < (long) frame_info->outer_bevel; x++)
-      *q++=shadow;
-  }
-  for (y=frame_info->outer_bevel-1; y >= 0; y--)
-  {
-    for (x=0; x < y; x++)
-      *q++=highlight;
-    for ( ; x < (long) frame_image->columns; x++)
-      if (x >= (long) (frame_image->columns-y))
-        *q++=shadow;
-      else
-        *q++=trough;
-  }
-  (void) SyncImagePixels(frame_image);
+  if (q != (PixelPacket *) NULL)
+    {
+      /*
+        Draw bottom of ornamental border.
+      */
+      frame_indexes=GetIndexes(frame_image);
+      for (y=frame_info->inner_bevel-1; y >= 0; y--)
+      {
+        for (x=0; x < (long) frame_info->outer_bevel; x++)
+        {
+          SetPixelPacket(&highlight,q,frame_indexes);
+          q++;
+          frame_indexes++;
+        }
+        for (x=0; x < (long) (frame_info->x-bevel_width); x++)
+        {
+          SetPixelPacket(&matte,q,frame_indexes);
+          q++;
+          frame_indexes++;
+        }
+        for (x=0; x < y; x++)
+        {
+          SetPixelPacket(&shadow,q,frame_indexes);
+          q++;
+          frame_indexes++;
+        }
+        for ( ; x < (long) (image->columns+2*frame_info->inner_bevel); x++)
+        {
+          if (x >= (long) (image->columns+2*frame_info->inner_bevel-y))
+            SetPixelPacket(&highlight,q,frame_indexes);
+          else
+            SetPixelPacket(&accentuate,q,frame_indexes);
+          q++;
+          frame_indexes++;
+        }
+        width=frame_info->width-frame_info->x-image->columns-bevel_width;
+        for (x=0; x < (long) width; x++)
+        {
+          SetPixelPacket(&matte,q,frame_indexes);
+          q++;
+          frame_indexes++;
+        }
+        for (x=0; x < (long) frame_info->outer_bevel; x++)
+        {
+          SetPixelPacket(&shadow,q,frame_indexes);
+          q++;
+          frame_indexes++;
+        }
+      }
+      height=frame_info->height-frame_info->y-image->rows-bevel_width;
+      for (y=0; y < (long) height; y++)
+      {
+        for (x=0; x < (long) frame_info->outer_bevel; x++)
+        {
+          SetPixelPacket(&highlight,q,frame_indexes);
+          q++;
+          frame_indexes++;
+        }
+        width=frame_image->columns-2*frame_info->outer_bevel;
+        for (x=0; x < (long) width; x++)
+        {
+          SetPixelPacket(&matte,q,frame_indexes);
+          q++;
+          frame_indexes++;
+        }
+        for (x=0; x < (long) frame_info->outer_bevel; x++)
+        {
+          SetPixelPacket(&shadow,q,frame_indexes);
+          q++;
+          frame_indexes++;
+        }
+      }
+      for (y=frame_info->outer_bevel-1; y >= 0; y--)
+      {
+        for (x=0; x < y; x++)
+        {
+          SetPixelPacket(&highlight,q,frame_indexes);
+          q++;
+          frame_indexes++;
+        }
+        for ( ; x < (long) frame_image->columns; x++)
+        {
+          if (x >= (long) (frame_image->columns-y))
+            SetPixelPacket(&shadow,q,frame_indexes);
+          else
+            SetPixelPacket(&trough,q,frame_indexes);
+          q++;
+          frame_indexes++;
+        }
+      }
+      (void) SyncImagePixels(frame_image);
+    }
+  x=(long) (frame_info->outer_bevel+(frame_info->x-bevel_width)+
+    frame_info->inner_bevel);
+  y=(long) (frame_info->outer_bevel+(frame_info->y-bevel_width)+
+    frame_info->inner_bevel);
+  (void) CompositeImage(frame_image,image->compose,image,x,y);
   return(frame_image);
 }
 
@@ -489,9 +639,11 @@ MagickExport MagickBooleanType RaiseImage(Image *image,
     }
     if (SyncImagePixels(image) == MagickFalse)
       break;
-    if (QuantumTick(y,image->rows) != MagickFalse)
+    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+        (QuantumTick(y,image->rows) != MagickFalse))
       {
-        status=MagickMonitor(RaiseImageTag,y,image->rows,&image->exception);
+        status=image->progress_monitor(RaiseImageTag,y,image->rows,
+          image->client_data);
         if (status == MagickFalse)
           break;
       }
@@ -528,9 +680,11 @@ MagickExport MagickBooleanType RaiseImage(Image *image,
     }
     if (SyncImagePixels(image) == MagickFalse)
       break;
-    if (QuantumTick(y,image->rows) != MagickFalse)
+    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+        (QuantumTick(y,image->rows) != MagickFalse))
       {
-        status=MagickMonitor(RaiseImageTag,y,image->rows,&image->exception);
+        status=image->progress_monitor(RaiseImageTag,y,image->rows,
+          image->client_data);
         if (status == MagickFalse)
           break;
       }
@@ -575,9 +729,11 @@ MagickExport MagickBooleanType RaiseImage(Image *image,
     }
     if (SyncImagePixels(image) == MagickFalse)
       break;
-    if (QuantumTick(y,image->rows) != MagickFalse)
+    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+        (QuantumTick(y,image->rows) != MagickFalse))
       {
-        status=MagickMonitor(RaiseImageTag,y,image->rows,&image->exception);
+        status=image->progress_monitor(RaiseImageTag,y,image->rows,
+          image->client_data);
         if (status == MagickFalse)
           break;
       }
