@@ -17,7 +17,7 @@
 %                                October 2002                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -42,6 +42,7 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
 #include "magick/client.h"
 #include "magick/color.h"
 #include "magick/color_private.h"
@@ -49,7 +50,8 @@
 #include "magick/constitute.h"
 #include "magick/deprecate.h"
 #include "magick/effect.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/geometry.h"
 #include "magick/list.h"
 #include "magick/log.h"
@@ -57,6 +59,8 @@
 #include "magick/magick.h"
 #include "magick/monitor.h"
 #include "magick/nt_feature.h"
+#include "magick/paint.h"
+#include "magick/pixel.h"
 #include "magick/quantize.h"
 #include "magick/random.h"
 #include "magick/resource_.h"
@@ -65,6 +69,12 @@
 #include "magick/string_.h"
 #include "magick/utility.h"
 
+/*
+  Global declarations.
+*/
+static MonitorHandler
+  monitor_handler = (MonitorHandler) NULL;
+
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -340,40 +350,6 @@ MagickExport unsigned int DeleteImageList(Image *images,const long offset)
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%   D e s t r o y B l o b I n f o                                             %
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%
-%  DestroyBlobInfo() deallocates memory associated with an BlobInfo structure.
-%
-%  The format of the DestroyBlobInfo method is:
-%
-%      void DestroyBlobInfo(BlobInfo *blob)
-%
-%  A description of each parameter follows:
-%
-%    o blob: Specifies a pointer to a BlobInfo structure.
-%
-%
-*/
-MagickExport void DestroyBlobInfo(BlobInfo *blob)
-{
-  Image
-    *image;
-
-  image=AllocateImage((const ImageInfo *) NULL);
-  image->blob=blob;
-  DestroyBlob(image);
-  DestroyImage(image);
-}
-
-/*
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%                                                                             %
-%                                                                             %
-%                                                                             %
 %   D e s t r o y I m a g e s                                                 %
 %                                                                             %
 %                                                                             %
@@ -396,7 +372,7 @@ MagickExport void DestroyImages(Image *image)
 {
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(DeprecateEvent,GetMagickModule(),"last use: v5.4.3");
-  DestroyImageList(image);
+  image=DestroyImageList(image);
 }
 
 /*
@@ -709,7 +685,7 @@ MagickExport void *GetConfigureBlob(const char *filename,char *path,
 #endif
 #endif
   if (blob == (void *) NULL)
-    (void) ThrowMagickException(exception,GetMagickModule(),ConfigureError,
+    (void) ThrowMagickException(exception,GetMagickModule(),ConfigureWarning,
       "UnableToOpenConfigureFile",path);
   return(blob);
 }
@@ -1014,7 +990,7 @@ MagickExport Image *GetPreviousImage(const Image *images)
 */
 MagickExport void IdentityAffine(AffineMatrix *affine)
 {
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   (void) LogMagickEvent(DeprecateEvent,GetMagickModule(),"last use: v5.5.7");
   assert(affine != (AffineMatrix *) NULL);
   (void) ResetMagickMemory(affine,0,sizeof(AffineMatrix));
@@ -1119,10 +1095,10 @@ MagickExport void LiberateMemory(void **memory)
 %
 %
 */
-MagickExport void LiberateSemaphoreInfo(void **semaphore_info)
+MagickExport void LiberateSemaphoreInfo(SemaphoreInfo **semaphore_info)
 {
   (void) LogMagickEvent(DeprecateEvent,GetMagickModule(),"last use: v5.5.7");
-  RelinquishSemaphoreInfo((SemaphoreInfo **) semaphore_info);
+  RelinquishSemaphoreInfo(*semaphore_info);
 }
 
 /*
@@ -1160,6 +1136,188 @@ MagickExport void MagickIncarnate(const char *path)
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   M a g i c k M o n i t o r                                                 %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  MagickMonitor() calls the monitor handler method with a text string that
+%  describes the task and a measure of completion.  The method returns
+%  MagickTrue on success otherwise MagickFalse if an error is encountered, e.g.
+%  if there was a user interrupt.
+%
+%  The format of the MagickMonitor method is:
+%
+%      MagickBooleanType MagickMonitor(const char *text,
+%        const MagickOffsetType offset,const MagickSizeType span,
+%        void *client_data)
+%
+%  A description of each parameter follows:
+%
+%    o offset: The position relative to the span parameter which represents
+%      how much progress has been made toward completing a task.
+%
+%    o span: The span relative to completing a task.
+%
+%    o client_data: The client data.
+%
+*/
+MagickExport MagickBooleanType MagickMonitor(const char *text,
+  const MagickOffsetType offset,const MagickSizeType span,
+  void *client_data)
+{
+  ExceptionInfo
+    exception;
+
+  MagickBooleanType
+    status;
+
+  assert(text != (const char *) NULL);
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),text);
+  ProcessPendingEvents(text);
+  status=MagickTrue;
+  GetExceptionInfo(&exception);
+  if (monitor_handler != (MonitorHandler) NULL)
+    status=(*monitor_handler)(text,offset,span,&exception);
+  DestroyExceptionInfo(&exception);
+  return(status);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%     O p a q u e I m a g e                                                   %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  OpaqueImage() changes any pixel that matches color with the color
+%  defined by fill.
+%
+%  By default color must match a particular pixel color exactly.  However,
+%  in many cases two colors may differ by a small amount.  Fuzz defines
+%  how much tolerance is acceptable to consider two colors as the same.
+%  For example, set fuzz to 10 and the color red at intensities of 100 and
+%  102 respectively are now interpreted as the same color.
+%
+%  The format of the OpaqueImage method is:
+%
+%      MagickBooleanType OpaqueImage(Image *image,
+%        const PixelPacket *target,const PixelPacket fill)
+%
+%  A description of each parameter follows:
+%
+%    o image: The image.
+%
+%    o target: The RGB value of the target color.
+%
+%    o fill: The replacement color.
+%
+%
+*/
+MagickExport MagickBooleanType OpaqueImage(Image *image,
+  const PixelPacket target,const PixelPacket fill)
+{
+#define OpaqueImageTag  "Opaque/Image"
+
+  long
+    y;
+
+  MagickBooleanType
+    status;
+
+  register long
+    x;
+
+  register PixelPacket
+    *q;
+
+  register long
+    i;
+
+  /*
+    Make image color opaque.
+  */
+  assert(image != (Image *) NULL);
+  assert(image->signature == MagickSignature);
+  (void) LogMagickEvent(DeprecateEvent,GetMagickModule(),"last use: v6.1.0");
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+  switch (image->storage_class)
+  {
+    case DirectClass:
+    default:
+    {
+      /*
+        Make DirectClass image opaque.
+      */
+      for (y=0; y < (long) image->rows; y++)
+      {
+        q=GetImagePixels(image,0,y,image->columns,1);
+        if (q == (PixelPacket *) NULL)
+          break;
+        for (x=0; x < (long) image->columns; x++)
+        {
+          if (FuzzyColorCompare(image,q,&target) != MagickFalse)
+            *q=fill;
+          q++;
+        }
+        if (SyncImagePixels(image) == MagickFalse)
+          break;
+        if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+            (QuantumTick(y,image->rows) != MagickFalse))
+          {
+            status=image->progress_monitor(OpaqueImageTag,y,image->rows,
+              image->client_data);
+            if (status == MagickFalse)
+              break;
+          }
+      }
+      break;
+    }
+    case PseudoClass:
+    {
+      /*
+        Make PseudoClass image opaque.
+      */
+      for (i=0; i < (long) image->colors; i++)
+      {
+        if (FuzzyColorCompare(image,&image->colormap[i],&target) != MagickFalse)
+          image->colormap[i]=fill;
+      }
+      if (fill.opacity != OpaqueOpacity)
+        {
+          for (y=0; y < (long) image->rows; y++)
+          {
+            q=GetImagePixels(image,0,y,image->columns,1);
+            if (q == (PixelPacket *) NULL)
+              break;
+            for (x=0; x < (long) image->columns; x++)
+            {
+              if (FuzzyColorCompare(image,q,&target) != MagickFalse)
+                q->opacity=fill.opacity;
+              q++;
+            }
+            if (SyncImagePixels(image) == MagickFalse)
+              break;
+          }
+        }
+      (void) SyncImage(image);
+      break;
+    }
+  }
+  if (fill.opacity != OpaqueOpacity)
+    image->matte=MagickTrue;
+  return(MagickTrue);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 +   P a r s e I m a g e G e o m e t r y                                       %
 %                                                                             %
 %                                                                             %
@@ -1232,6 +1390,43 @@ MagickExport Image *PopImageList(Image **images)
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   P o p I m a g e P i x e l s                                               %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  PopImagePixels() transfers one or more pixel components from the image pixel
+%  cache to a user supplied buffer.  The pixels are returned in network byte
+%  order.  MagickTrue is returned if the pixels are successfully transferred,
+%  otherwise MagickFalse.
+%
+%  The format of the PopImagePixels method is:
+%
+%      MagickBooleanType PopImagePixels(Image *,const QuantumType quantum,
+%        unsigned char *destination)
+%
+%  A description of each parameter follows:
+%
+%    o image: The image.
+%
+%    o quantum: Declare which pixel components to transfer (RGB, RGBA, etc).
+%
+%    o destination:  The components are transferred to this buffer.
+%
+%
+*/
+MagickExport MagickBooleanType PopImagePixels(Image *image,
+  const QuantumType quantum,unsigned char *destination)
+{
+  return(ImportQuantumPixels(image,quantum,0,destination));
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %  P o s t s c r i p t G e o m e t r y                                        %
 %                                                                             %
 %                                                                             %
@@ -1300,6 +1495,44 @@ MagickExport unsigned int PushImageList(Image **images,const Image *image,
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   P u s h I m a g e P i x e l s                                             %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  PushImagePixels() transfers one or more pixel components from a user
+%  supplied buffer into the image pixel cache of an image.  The pixels are
+%  expected in network byte order.  It returns MagickTrue if the pixels are
+%  successfully transferred, otherwise MagickFalse.
+%
+%  The format of the PushImagePixels method is:
+%
+%      MagickBooleanType PushImagePixels(Image *image,const QuantumType quantum,
+%        const unsigned char *source)
+%
+%  A description of each parameter follows:
+%
+%    o image: The image.
+%
+%    o quantum: Declare which pixel components to transfer (red, green, blue,
+%      opacity, RGB, or RGBA).
+%
+%    o source:  The pixel components are transferred from this buffer.
+%
+%
+*/
+MagickExport MagickBooleanType PushImagePixels(Image *image,
+  const QuantumType quantum,const unsigned char *source)
+{
+  return(ExportQuantumPixels(image,quantum,0,source));
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %  Q u a n t i z a t i o n E r r o r                                          %
 %                                                                             %
 %                                                                             %
@@ -1682,6 +1915,46 @@ MagickExport unsigned int SetImageList(Image **images,const Image *image,
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   S e t M o n i t o r H a n d l e r                                         %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  SetMonitorHandler() sets the monitor handler to the specified method
+%  and returns the previous monitor handler.
+%
+%  The format of the SetMonitorHandler method is:
+%
+%      MonitorHandler SetMonitorHandler(MonitorHandler handler)
+%
+%  A description of each parameter follows:
+%
+%    o handler: Specifies a pointer to a method to handle monitors.
+%
+%
+*/
+
+MagickExport MonitorHandler GetMonitorHandler(void)
+{
+  return(monitor_handler);
+}
+
+MagickExport MonitorHandler SetMonitorHandler(MonitorHandler handler)
+{
+  MonitorHandler
+    previous_handler;
+
+  previous_handler=monitor_handler;
+  monitor_handler=handler;
+  return(previous_handler);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   S h i f t I m a g e L i s t                                               %
 %                                                                             %
 %                                                                             %
@@ -2127,6 +2400,92 @@ MagickExport unsigned int TransformColorspace(Image *image,
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
 %                                                                             %
+%     T r a n s p a r e n t I m a g e                                         %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  TransparentImage() changes the opacity value associated with any pixel
+%  that matches color to the value defined by opacity.
+%
+%  By default color must match a particular pixel color exactly.  However,
+%  in many cases two colors may differ by a small amount.  Fuzz defines
+%  how much tolerance is acceptable to consider two colors as the same.
+%  For example, set fuzz to 10 and the color red at intensities of 100 and
+%  102 respectively are now interpreted as the same color.
+%
+%  The format of the TransparentImage method is:
+%
+%      MagickBooleanType TransparentImage(Image *image,
+%        const PixelPacket target,const Quantum opacity)
+%
+%  A description of each parameter follows:
+%
+%    o image: The image.
+%
+%    o target: The RGB value of the target color.
+%
+%    o opacity: The replacement opacity value.
+%
+%
+*/
+MagickExport MagickBooleanType TransparentImage(Image *image,
+  const PixelPacket target,const Quantum opacity)
+{
+#define TransparentImageTag  "Transparent/Image"
+
+  long
+    y;
+
+  MagickBooleanType
+    status;
+
+  register long
+    x;
+
+  register PixelPacket
+    *q;
+
+  /*
+    Make image color transparent.
+  */
+  assert(image != (Image *) NULL);
+  assert(image->signature == MagickSignature);
+  (void) LogMagickEvent(DeprecateEvent,GetMagickModule(),"last use: v6.1.0");
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+  if (image->matte == MagickFalse)
+    SetImageOpacity(image,OpaqueOpacity);
+  for (y=0; y < (long) image->rows; y++)
+  {
+    q=GetImagePixels(image,0,y,image->columns,1);
+    if (q == (PixelPacket *) NULL)
+      break;
+    for (x=(long) image->columns-1; x >= 0; x--)
+    {
+      if (FuzzyColorCompare(image,q,&target) != MagickFalse)
+        q->opacity=opacity;
+      q++;
+    }
+    if (SyncImagePixels(image) == MagickFalse)
+      break;
+    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+        (QuantumTick(y,image->rows) != MagickFalse))
+      {
+        status=image->progress_monitor(TransparentImageTag,y,image->rows,
+          image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
+  }
+  return(MagickTrue);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
 %                                                                             %
 %   U n s h i f t I m a g e L i s t                                           %
 %                                                                             %
