@@ -17,7 +17,7 @@
 %                                 August 1996                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -78,7 +78,6 @@ MagickExport void Contrast(const int sign,Quantum *red,Quantum *green,
   Quantum *blue)
 {
   double
-    alpha,
     brightness,
     hue,
     saturation;
@@ -93,9 +92,7 @@ MagickExport void Contrast(const int sign,Quantum *red,Quantum *green,
   saturation=0.0;
   brightness=0.0;
   TransformHSB(*red,*green,*blue,&hue,&saturation,&brightness);
-  alpha=0.5+MagickEpsilon;
-  brightness+=
-    alpha*sign*(alpha*(sin(MagickPI*(brightness-alpha))+1.0)-brightness);
+  brightness+=0.5*sign*(0.5*(sin(MagickPI*(brightness-0.5))+1.0)-brightness);
   if (brightness > 1.0)
     brightness=1.0;
   else
@@ -184,10 +181,10 @@ MagickExport unsigned long GetOptimalKernelWidth1D(const double radius,
   register long
     u;
 
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(sigma != 0.0);
   if (radius > 0.0)
-    return(2*((unsigned long) radius)+1);
+    return(Max(2*((unsigned long) radius)+1,3));
   for (width=5; ;)
   {
     normalize=0.0;
@@ -218,9 +215,9 @@ MagickExport unsigned long GetOptimalKernelWidth2D(const double radius,
     v;
 
   assert(sigma != 0.0);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   if (radius > 0.0)
-    return(2*((unsigned long) radius)+1);
+    return(Max(2*((unsigned long) radius)+1,3));
   for (width=5; ;)
   {
     normalize=0.0;
@@ -298,13 +295,11 @@ MagickExport void HSBTransform(const double hue,const double saturation,
       *blue=(*red);
       return;
     }
-  h=6.0*hue;
-  if (h == 6.0)
-    h=0.0;
-  f=h-(int) h;
+  h=6.0*(hue-floor(hue));
+  f=h-floor((double) h);
   p=brightness*(1.0-saturation);
   q=brightness*(1.0-saturation*f);
-  t=brightness*(1.0-saturation*(1.0-f));
+  t=brightness*(1.0-(saturation*(1.0-f)));
   switch ((int) h)
   {
     case 0:
@@ -312,14 +307,14 @@ MagickExport void HSBTransform(const double hue,const double saturation,
     {
       *red=RoundToQuantum(MaxRGB*brightness);
       *green=RoundToQuantum(MaxRGB*t);
-      *blue= RoundToQuantum(MaxRGB*p);
+      *blue=RoundToQuantum(MaxRGB*p);
       break;
     }
     case 1:
     {
       *red=RoundToQuantum(MaxRGB*q);
       *green=RoundToQuantum(MaxRGB*brightness);
-      *blue= RoundToQuantum(MaxRGB*p);
+      *blue=RoundToQuantum(MaxRGB*p);
       break;
     }
     case 2:
@@ -681,8 +676,6 @@ MagickExport PixelPacket InterpolateColor(const Image *image,
 
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   p=AcquireImagePixels(image,(long) x_offset,(long) y_offset,2,2,exception);
   if (p == (const PixelPacket *) NULL)
     return(AcquireOnePixel(image,(long) x_offset,(long) y_offset,exception));
@@ -707,19 +700,19 @@ MagickExport PixelPacket InterpolateColor(const Image *image,
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%   M o d u l a t e                                                           %
+%   M o d u l a t e H S B                                                     %
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Modulate() modulates the hue, saturation, and brightness of an image.
+%  ModulateHSB() modulates the hue, saturation, and brightness of an image.
 %
-%  The format of the Modulate method is:
+%  The format of the ModulateHSB method is:
 %
-%      void Modulate(const double percent_hue,const double percent_saturation,
-%        const double percent_brightness,Quantum *red,Quantum *green,
-%        Quantum *blue)
+%      void ModulateHSB(const double percent_hue,
+%        const double percent_saturation,const double percent_brightness,
+%        Quantum *red,Quantum *green,Quantum *blue)
 %
 %  A description of each parameter follows:
 %
@@ -730,7 +723,7 @@ MagickExport PixelPacket InterpolateColor(const Image *image,
 %
 %
 */
-MagickExport void Modulate(const double percent_hue,
+MagickExport void ModulateHSB(const double percent_hue,
   const double percent_saturation,const double percent_brightness,
   Quantum *red,Quantum *green,Quantum *blue)
 {
@@ -746,13 +739,13 @@ MagickExport void Modulate(const double percent_hue,
   assert(green != (Quantum *) NULL);
   assert(blue != (Quantum *) NULL);
   TransformHSB(*red,*green,*blue,&hue,&saturation,&brightness);
-  brightness*=0.01*percent_brightness;
-  saturation*=0.01*percent_saturation;
-  hue*=0.01*percent_hue;
+  hue+=0.5*(0.01*percent_hue-1.0);
   while (hue < 0.0)
     hue+=1.0;
   while (hue > 1.0)
     hue-=1.0;
+  saturation*=0.01*percent_saturation;
+  brightness*=0.01*percent_brightness;
   HSBTransform(hue,saturation,brightness,red,green,blue);
 }
 
@@ -761,6 +754,114 @@ MagickExport void Modulate(const double percent_hue,
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   M o d u l a t e H S L                                                     %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  ModulateSB() modulates the hue, saturation, and brightness of an image.
+%
+%  The format of the ModulateHSL method is:
+%
+%      void ModulateHSL(const double percent_hue,
+%        const double percent_saturation,const double percent_luminosity,
+%        Quantum *red,Quantum *green,Quantum *blue)
+%
+%  A description of each parameter follows:
+%
+%    o percent_hue, percent_saturation, percent_luminosity: A double value
+%      representing the percent change in a component of the HSL color space.
+%
+%    o red, green, blue: A pointer to a pixel component of type Quantum.
+%
+%
+*/
+MagickExport void ModulateHSL(const double percent_hue,
+  const double percent_saturation,const double percent_luminosity,
+  Quantum *red,Quantum *green,Quantum *blue)
+{
+  double
+    hue,
+    luminosity,
+    saturation;
+
+  /*
+    Increase or decrease color luminosity, saturation, or hue.
+  */
+  assert(red != (Quantum *) NULL);
+  assert(green != (Quantum *) NULL);
+  assert(blue != (Quantum *) NULL);
+  TransformHSL(*red,*green,*blue,&hue,&saturation,&luminosity);
+  hue+=0.5*(0.01*percent_hue-1.0);
+  while (hue < 0.0)
+    hue+=1.0;
+  while (hue > 1.0)
+    hue-=1.0;
+  saturation*=0.01*percent_saturation;
+  luminosity*=0.01*percent_luminosity;
+  HSLTransform(hue,saturation,luminosity,red,green,blue);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   M o d u l a t e H W B                                                     %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  ModulateHWB() modulates the hue, whiteness, and blackness of an image.
+%
+%  The format of the ModulateHWB method is:
+%
+%      void ModulateHWB(const double percent_hue,
+%        const double percent_whiteness,const double percent_blackness,
+%        Quantum *red,Quantum *green,Quantum *blue)
+%
+%  A description of each parameter follows:
+%
+%    o percent_hue, percent_whiteness, percent_blackness: A double value
+%      representing the percent change in a component of the HWB color space.
+%
+%    o red, green, blue: A pointer to a pixel component of type Quantum.
+%
+%
+*/
+MagickExport void ModulateHWB(const double percent_hue,
+  const double percent_whiteness,const double percent_blackness,
+  Quantum *red,Quantum *green,Quantum *blue)
+{
+  double
+    blackness,
+    hue,
+    whiteness;
+
+  /*
+    Increase or decrease color blackness, whiteness, or hue.
+  */
+  assert(red != (Quantum *) NULL);
+  assert(green != (Quantum *) NULL);
+  assert(blue != (Quantum *) NULL);
+  TransformHWB(*red,*green,*blue,&hue,&whiteness,&blackness);
+  hue+=0.5*(0.01*percent_hue-1.0);
+  while (hue < 0.0)
+    hue+=1.0;
+  while (hue > 1.0)
+    hue-=1.0;
+  blackness*=0.01*percent_blackness;
+  whiteness*=0.01*percent_whiteness;
+  HWBTransform(hue,whiteness,blackness,red,green,blue);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   T r a n s f o r m H S B                                                   %
 %                                                                             %
 %                                                                             %
@@ -805,28 +906,25 @@ MagickExport void TransformHSB(const Quantum red,const Quantum green,
   min=(MagickRealType) (red < green ? red : green);
   if ((MagickRealType) blue < min)
     min=(MagickRealType) blue;
-  *hue=0.0;
-  *saturation=0.0;
-  if (max != 0.0)
-    *saturation=(max-min)/max;
-  *brightness=QuantumScale*max;
-  if (*saturation == 0.0)
-    return;
+  *brightness=(double) (QuantumScale*max);
+  if (max == 0.0)
+    {
+      *saturation=0.0;
+      *hue=0.0;
+      return;
+    }
+  *saturation=(double) (1.0-min/max);
   delta=max-min;
   if ((MagickRealType) red == max)
-    *hue=(green-(MagickRealType) blue)/delta;
+    *hue=(double) ((green-(MagickRealType) blue)/delta);
   else
     if ((MagickRealType) green == max)
-      *hue=2.0+(blue-(MagickRealType) red)/delta;
+      *hue=(double) (2.0+(blue-(MagickRealType) red)/delta);
     else
-      if ((MagickRealType) blue == max)
-        *hue=4.0+(red-(MagickRealType) green)/delta;
+      *hue=(double) (4.0+(red-(MagickRealType) green)/delta);
   *hue/=6.0;
   if (*hue < 0.0)
     *hue+=1.0;
-  else
-    if (*hue > 1.0)
-      *hue-=1.0;
 }
 
 /*
@@ -882,18 +980,19 @@ MagickExport void TransformHSL(const Quantum red,const Quantum green,
   min=Min(r,Min(g,b));
   *hue=0.0;
   *saturation=0.0;
-  *luminosity=(min+max)/2.0;
+  *luminosity=(double) ((min+max)/2.0);
   delta=max-min;
   if (delta == 0.0)
     return;
-  *saturation=delta/((*luminosity < 0.5) ? (min+max) : (2.0-max-min));
+  *saturation=(double) (delta/((*luminosity < 0.5) ? (min+max) :
+    (2.0-max-min)));
   if (r == max)
-    *hue=(g == min ? 5.0+(max-b)/delta : 1.0-(max-g)/delta);
+    *hue=(double) (g == min ? 5.0+(max-b)/delta : 1.0-(max-g)/delta);
   else
     if (g == max)
-      *hue=(b == min ? 1.0+(max-r)/delta : 3.0-(max-b)/delta);
+      *hue=(double) (b == min ? 1.0+(max-r)/delta : 3.0-(max-b)/delta);
     else
-      *hue=(r == min ? 3.0+(max-g)/delta : 5.0-(max-r)/delta);
+      *hue=(double) (r == min ? 3.0+(max-g)/delta : 5.0-(max-r)/delta);
   *hue/=6.0;
 }
 
@@ -957,8 +1056,8 @@ MagickExport void TransformHWB(const Quantum red,const Quantum green,
   f=(red == w) ? (MagickRealType) green-blue : ((green == w) ?
     (MagickRealType) blue-red : (MagickRealType) red-green);
   i=(red == w) ? 3 : ((green == w) ? 5 : 1);
-  *hue=(MagickRealType) (i-f/(v-w));
-  *whiteness=(MagickRealType) w/MaxRGB;
+  *hue=(double) (i-f/(v-w));
+  *whiteness=(double) w/MaxRGB;
 }
 
 /*
