@@ -1,5 +1,19 @@
 /*
-  Graphic Gems - Graphic Support Methods.
+  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization
+  dedicated to making software imaging solutions freely available.
+  
+  You may not use this file except in compliance with the License.
+  obtain a copy of the License at
+  
+    http://www.imagemagick.org/www/Copyright.html
+  
+  Unless required by applicable law or agreed to in writing, software
+  distributed under the License is distributed on an "AS IS" BASIS,
+  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+  See the License for the specific language governing permissions and
+  limitations under the License.
+
+  ImageMagick graphic gems methods.
 */
 #ifndef _MAGICK_GEM_H
 #define _MAGICK_GEM_H
@@ -8,33 +22,35 @@
 extern "C" {
 #endif
 
-/*
-  Graphic gems define declarations.
-*/
 extern MagickExport double
   ExpandAffine(const AffineMatrix *);
 
-extern MagickExport int
-  GetOptimalKernelWidth(const double,const double),
-  GetOptimalKernelWidth1D(const double,const double),
-  GetOptimalKernelWidth2D(const double,const double);
-
 extern MagickExport PixelPacket
   InterpolateColor(const Image *,const double,const double,ExceptionInfo *);
 
-extern MagickExport Quantum
-  GenerateNoise(const Quantum,const NoiseType);
+extern MagickExport unsigned long
+  GetOptimalKernelWidth(const double,const double),
+  GetOptimalKernelWidth1D(const double,const double),
+  GetOptimalKernelWidth2D(const double,const double);
 
 extern MagickExport void
   Contrast(const int,Quantum *,Quantum *,Quantum *),
+  HSBTransform(const double,const double,const double,Quantum *,Quantum *,
+    Quantum *),
   HSLTransform(const double,const double,const double,Quantum *,Quantum *,
     Quantum *),
   HWBTransform(const double,const double,const double,Quantum *,Quantum *,
     Quantum *),
   Hull(const long,const long,const unsigned long,const unsigned long,Quantum *,
     Quantum *,const int),
-  Modulate(const double,const double,const double,Quantum *,Quantum *,
+  ModulateHSB(const double,const double,const double,Quantum *,Quantum *,
     Quantum *),
+  ModulateHSL(const double,const double,const double,Quantum *,Quantum *,
+    Quantum *),
+  ModulateHWB(const double,const double,const double,Quantum *,Quantum *,
+    Quantum *),
+  TransformHSB(const Quantum,const Quantum,const Quantum,double *,double *,
+    double *),
   TransformHSL(const Quantum,const Quantum,const Quantum,double *,double *,
     double *),
   TransformHWB(const Quantum,const Quantum,const Quantum,double *,double *,
