@@ -17,7 +17,7 @@
 %                              January 2003                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -41,7 +41,8 @@
 */
 #include "magick/studio.h"
 #include "magick/constitute.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/geometry.h"
 #include "magick/memory_.h"
 #include "magick/string_.h"
@@ -376,8 +377,8 @@ MagickExport char *GetPageGeometry(const char *page_geometry)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  IsGeometry() returns MagickTrue if the geometry specification is valid.  Examples
-%  are 100, 100x200, x200, 100x200+10+20, +10+20, 200%, 200x200!, etc.
+%  IsGeometry() returns MagickTrue if the geometry specification is valid.
+%  Examples are 100, 100x200, x200, 100x200+10+20, +10+20, 200%, 200x200!, etc.
 %
 %  The format of the IsGeometry method is:
 %
@@ -730,6 +731,10 @@ MagickExport MagickStatusType ParseGravityGeometry(Image *image,
   MagickStatusType
     flags;
 
+  unsigned long
+    height,
+    width;
+
   SetGeometry(image,region_info);
   (void) FormatMagickString(region_geometry,MaxTextExtent,"%s!",geometry);
   flags=ParseAbsoluteGeometry(region_geometry,region_info);
@@ -739,58 +744,22 @@ MagickExport MagickStatusType ParseGravityGeometry(Image *image,
         OptionError,"InvalidGeometry",geometry);
       return(flags);
     }
-  switch (image->gravity)
-  {
-    case ForgetGravity:
-    case NorthWestGravity:
-      break;
-    case NorthGravity:
+  width=region_info->width;
+  height=region_info->height;
+  if (width == 0)
     {
-      region_info->x+=(long) (image->columns/2-region_info->width/2);
-      break;
+      if (image->page.width != 0)
+        width=image->page.width;
+      else
+        width=image->columns;
     }
-    case NorthEastGravity:
+  if (height == 0)
     {
-      region_info->x=(long) (image->columns-region_info->width-region_info->x);
-      break;
+      if (image->page.height != 0)
+        height=image->page.height;
+      else
+        height=image->rows;
     }
-    case WestGravity:
-    {
-      region_info->y+=(long) (image->rows/2-region_info->height/2);
-      break;
-    }
-    case StaticGravity:
-    case CenterGravity:
-    default:
-    {
-      region_info->x+=(long) (image->columns/2-region_info->width/2);
-      region_info->y+=(long) (image->rows/2-region_info->height/2);
-      break;
-    }
-    case EastGravity:
-    {
-      region_info->x=(long) (image->columns-region_info->width-region_info->x);
-      region_info->y+=(long) (image->rows/2-region_info->height/2);
-      break;
-    }
-    case SouthWestGravity:
-    {
-      region_info->y=(long) (image->rows-region_info->height-region_info->y);
-      break;
-    }
-    case SouthGravity:
-    {
-      region_info->x+=(long) (image->columns/2-region_info->width/2);
-      region_info->y=(long) (image->rows-region_info->height-region_info->y);
-      break;
-    }
-    case SouthEastGravity:
-    {
-      region_info->x=(long) (image->columns-region_info->width-region_info->x);
-      region_info->y=(long) (image->rows-region_info->height-region_info->y);
-      break;
-    }
-  }
   if ((flags & PercentValue) != 0)
     {
       GeometryInfo
@@ -816,6 +785,58 @@ MagickExport MagickStatusType ParseGravityGeometry(Image *image,
       region_info->width=(unsigned long) ((x_scale*image->columns/100.0)+0.5);
       region_info->height=(unsigned long) ((y_scale*image->rows/100.0)+0.5);
     }
+  switch (image->gravity)
+  {
+    case ForgetGravity:
+    case NorthWestGravity:
+      break;
+    case NorthGravity:
+    {
+      region_info->x+=(long) (image->columns/2-width/2);
+      break;
+    }
+    case NorthEastGravity:
+    {
+      region_info->x=(long) (image->columns-width-region_info->x);
+      break;
+    }
+    case WestGravity:
+    {
+      region_info->y+=(long) (image->rows/2-height/2);
+      break;
+    }
+    case StaticGravity:
+    case CenterGravity:
+    default:
+    {
+      region_info->x+=(long) (image->columns/2-width/2);
+      region_info->y+=(long) (image->rows/2-height/2);
+      break;
+    }
+    case EastGravity:
+    {
+      region_info->x=(long) (image->columns-width-region_info->x);
+      region_info->y+=(long) (image->rows/2-height/2);
+      break;
+    }
+    case SouthWestGravity:
+    {
+      region_info->y=(long) (image->rows-height-region_info->y);
+      break;
+    }
+    case SouthGravity:
+    {
+      region_info->x+=(long) (image->columns/2-width/2);
+      region_info->y=(long) (image->rows-height-region_info->y);
+      break;
+    }
+    case SouthEastGravity:
+    {
+      region_info->x=(long) (image->columns-width-region_info->x);
+      region_info->y=(long) (image->rows-height-region_info->y);
+      break;
+    }
+  }
   return(flags);
 }
 
@@ -951,7 +972,9 @@ MagickExport MagickStatusType ParseMetaGeometry(const char *geometry,long *x,
     {
       MagickRealType
         area,
-        distance;
+        distance,
+        x_scale,
+        y_scale;
 
       /*
         Geometry is a maximum area in pixels.
@@ -959,11 +982,14 @@ MagickExport MagickStatusType ParseMetaGeometry(const char *geometry,long *x,
       (void) ParseGeometry(geometry,&geometry_info);
       area=geometry_info.rho;
       distance=sqrt((double) former_width*former_height);
-      if (((former_width/(distance/sqrt(area))) < *width) ||
-          ((former_height/(distance/sqrt(area))) < *height))
+      x_scale=(MagickRealType) former_width/(distance/sqrt((double) area));
+      y_scale=(MagickRealType) former_height/(distance/sqrt((double) area));
+      if ((x_scale < (MagickRealType) *width) ||
+          (y_scale < (MagickRealType) *height))
         {
-          *width=(unsigned long) (former_width/(distance/sqrt(area)));
-          *height=(unsigned long) (former_height/(distance/sqrt(area)));
+          *width=(unsigned long) (former_width/(distance/sqrt((double) area)));
+          *height=(unsigned long) (former_height/(distance/
+            sqrt((double) area)));
         }
       former_width=(*width);
       former_height=(*height);
@@ -999,8 +1025,8 @@ MagickExport MagickStatusType ParseMetaGeometry(const char *geometry,long *x,
 %
 %
 */
-MagickExport MagickStatusType ParsePageGeometry(Image *image,const char *geometry,
-  RectangleInfo *region_info)
+MagickExport MagickStatusType ParsePageGeometry(Image *image,
+  const char *geometry,RectangleInfo *region_info)
 {
   char
     region_geometry[MaxTextExtent];
@@ -1092,7 +1118,7 @@ MagickExport void SetGeometry(const Image *image,RectangleInfo *geometry)
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   assert(geometry != (RectangleInfo *) NULL);
-  (void) ResetMagickMemory(geometry,0,sizeof(RectangleInfo));
+  (void) ResetMagickMemory(geometry,0,sizeof(*geometry));
   geometry->width=image->columns;
   geometry->height=image->rows;
 }
@@ -1123,6 +1149,6 @@ MagickExport void SetGeometry(const Image *image,RectangleInfo *geometry)
 MagickExport void SetGeometryInfo(GeometryInfo *geometry_info)
 {
   assert(geometry_info != (GeometryInfo *) NULL);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
-  (void) ResetMagickMemory(geometry_info,0,sizeof(GeometryInfo));
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
+  (void) ResetMagickMemory(geometry_info,0,sizeof(*geometry_info));
 }
