@@ -17,7 +17,7 @@
 %                               December 2002                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -43,7 +43,8 @@
   Include declarations.
 */
 #include "magick/studio.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/hashmap.h"
 #include "magick/memory_.h"
 #include "magick/semaphore.h"
@@ -157,7 +158,7 @@ MagickExport MagickBooleanType AppendElementToLinkedList(
   assert(list_info != (LinkedListInfo *) NULL);
   assert(list_info->signature == MagickSignature);
   if (list_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   if (list_info->elements == list_info->maximum_elements)
     return(MagickFalse);
   next=(ElementInfo *) AcquireMagickMemory(sizeof(*next));
@@ -165,7 +166,7 @@ MagickExport MagickBooleanType AppendElementToLinkedList(
     return(MagickFalse);
   next->datum=(void *) datum;
   next->next=(ElementInfo *) NULL;
-  AcquireSemaphoreInfo(&list_info->semaphore);
+  list_info->semaphore=AcquireSemaphoreInfo(list_info->semaphore);
   if (list_info->next == (ElementInfo *) NULL)
     list_info->next=next;
   if (list_info->elements == 0)
@@ -174,7 +175,7 @@ MagickExport MagickBooleanType AppendElementToLinkedList(
     list_info->tail->next=next;
   list_info->tail=next;
   list_info->elements++;
-  RelinquishSemaphoreInfo(&list_info->semaphore);
+  RelinquishSemaphoreInfo(list_info->semaphore);
   return(MagickTrue);
 }
 
@@ -216,8 +217,8 @@ MagickExport void ClearLinkedList(LinkedListInfo *list_info,
   assert(list_info != (LinkedListInfo *) NULL);
   assert(list_info->signature == MagickSignature);
   if (list_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
-  AcquireSemaphoreInfo(&list_info->semaphore);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
+  list_info->semaphore=AcquireSemaphoreInfo(list_info->semaphore);
   next=list_info->head;
   while (next != (ElementInfo *) NULL)
   {
@@ -231,7 +232,7 @@ MagickExport void ClearLinkedList(LinkedListInfo *list_info,
   list_info->tail=(ElementInfo *) NULL;
   list_info->next=(ElementInfo *) NULL;
   list_info->elements=0;
-  RelinquishSemaphoreInfo(&list_info->semaphore);
+  RelinquishSemaphoreInfo(list_info->semaphore);
 }
 
 /*
@@ -249,14 +250,14 @@ MagickExport void ClearLinkedList(LinkedListInfo *list_info,
 %
 %  The format of the DestroyHashmap method is:
 %
-%      DestroyHashmap(struct _HashmapInfo *hashmap_info)
+%      HashmapInfo *DestroyHashmap(struct _HashmapInfo *hashmap_info)
 %
 %  A description of each parameter follows:
 %
 %    o hashmap_info: The hashmap info.
 %
 */
-MagickExport void DestroyHashmap(HashmapInfo *hashmap_info)
+MagickExport HashmapInfo *DestroyHashmap(HashmapInfo *hashmap_info)
 {
   register EntryInfo
     *entry;
@@ -270,8 +271,8 @@ MagickExport void DestroyHashmap(HashmapInfo *hashmap_info)
   assert(hashmap_info != (HashmapInfo *) NULL);
   assert(hashmap_info->signature == MagickSignature);
   if (hashmap_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
-  AcquireSemaphoreInfo(&hashmap_info->semaphore);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
+  hashmap_info->semaphore=AcquireSemaphoreInfo(hashmap_info->semaphore);
   for (i=0; i < (long) hashmap_info->maximum_entries; i++)
   {
     list_info=hashmap_info->map[i];
@@ -289,13 +290,15 @@ MagickExport void DestroyHashmap(HashmapInfo *hashmap_info)
         }
       }
     if (list_info != (LinkedListInfo *) NULL)
-      DestroyLinkedList(list_info,RelinquishMagickMemory);
+      list_info=DestroyLinkedList(list_info,RelinquishMagickMemory);
   }
   hashmap_info->map=(struct _LinkedListInfo **)
     RelinquishMagickMemory(hashmap_info->map);
-  RelinquishSemaphoreInfo(&hashmap_info->semaphore);
-  DestroySemaphoreInfo(&hashmap_info->semaphore);
+  hashmap_info->signature=(~MagickSignature);
+  RelinquishSemaphoreInfo(hashmap_info->semaphore);
+  hashmap_info->semaphore=DestroySemaphoreInfo(hashmap_info->semaphore);
   hashmap_info=(HashmapInfo *) RelinquishMagickMemory(hashmap_info);
+  return(hashmap_info);
 }
 
 /*
@@ -313,7 +316,7 @@ MagickExport void DestroyHashmap(HashmapInfo *hashmap_info)
 %
 %  The format of the DestroyLinkedList method is:
 %
-%      DestroyLinkedList(LinkedListInfo *list_info,
+%      LinkedListInfo *DestroyLinkedList(LinkedListInfo *list_info,
 %        void *(*liberate_datum)(void *))
 %
 %  A description of each parameter follows:
@@ -324,7 +327,7 @@ MagickExport void DestroyHashmap(HashmapInfo *hashmap_info)
 %      RelinquishMagickMemory().
 %
 */
-MagickExport void DestroyLinkedList(LinkedListInfo *list_info,
+MagickExport LinkedListInfo *DestroyLinkedList(LinkedListInfo *list_info,
   void *(*liberate_datum)(void *))
 {
   ElementInfo
@@ -336,8 +339,8 @@ MagickExport void DestroyLinkedList(LinkedListInfo *list_info,
   assert(list_info != (LinkedListInfo *) NULL);
   assert(list_info->signature == MagickSignature);
   if (list_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
-  AcquireSemaphoreInfo(&list_info->semaphore);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
+  list_info->semaphore=AcquireSemaphoreInfo(list_info->semaphore);
   next=list_info->head;
   while (next != (ElementInfo *) NULL)
   {
@@ -347,9 +350,11 @@ MagickExport void DestroyLinkedList(LinkedListInfo *list_info,
     next=next->next;
     entry=(ElementInfo *) RelinquishMagickMemory(entry);
   }
-  RelinquishSemaphoreInfo(&list_info->semaphore);
-  DestroySemaphoreInfo(&list_info->semaphore);
+  list_info->signature=(~MagickSignature);
+  RelinquishSemaphoreInfo(list_info->semaphore);
+  list_info->semaphore=DestroySemaphoreInfo(list_info->semaphore);
   list_info=(LinkedListInfo *) RelinquishMagickMemory(list_info);
+  return(list_info);
 }
 
 /*
@@ -391,7 +396,7 @@ MagickExport void *GetEntryFromHashmap(const HashmapInfo *hashmap_info,
   assert(hashmap_info != (HashmapInfo *) NULL);
   assert(hashmap_info->signature == MagickSignature);
   if (hashmap_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   if (key == NULL)
     return((void *) NULL);
   hash=hashmap_info->hash(key);
@@ -448,7 +453,7 @@ MagickExport void *GetElementFromLinkedList(const LinkedListInfo *list_info,
   assert(list_info != (LinkedListInfo *) NULL);
   assert(list_info->signature == MagickSignature);
   if (list_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   if (index >= list_info->elements)
     return((void *) NULL);
   if (index == 0)
@@ -488,7 +493,7 @@ MagickExport void *GetLastElementInLinkedList(const LinkedListInfo *list_info)
   assert(list_info != (LinkedListInfo *) NULL);
   assert(list_info->signature == MagickSignature);
   if (list_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   if (list_info->elements == 0)
     return((void *) NULL);
   return(list_info->tail->datum);
@@ -529,7 +534,7 @@ MagickExport void *GetNextEntryInHashmap(HashmapInfo *hashmap_info)
   assert(hashmap_info != (HashmapInfo *) NULL);
   assert(hashmap_info->signature == MagickSignature);
   if (hashmap_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   while (hashmap_info->next < hashmap_info->maximum_entries)
   {
     list_info=hashmap_info->map[hashmap_info->next];
@@ -580,7 +585,7 @@ MagickExport void *GetNextElementInLinkedList(LinkedListInfo *list_info)
   assert(list_info != (LinkedListInfo *) NULL);
   assert(list_info->signature == MagickSignature);
   if (list_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   if (list_info->next == (ElementInfo *) NULL)
     return((void *) NULL);
   datum=list_info->next->datum;
@@ -616,7 +621,7 @@ MagickExport unsigned long GetNumberOfEntriesInHashmap(
   assert(hashmap_info != (HashmapInfo *) NULL);
   assert(hashmap_info->signature == MagickSignature);
   if (hashmap_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   return(hashmap_info->entries);
 }
 
@@ -650,7 +655,7 @@ MagickExport unsigned long GetNumberOfElementsInLinkedList(
   assert(list_info != (LinkedListInfo *) NULL);
   assert(list_info->signature == MagickSignature);
   if (list_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   return(list_info->elements);
 }
 
@@ -818,7 +823,7 @@ MagickExport MagickBooleanType InsertElementInLinkedList(
   assert(list_info != (LinkedListInfo *) NULL);
   assert(list_info->signature == MagickSignature);
   if (list_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   if (datum == NULL)
     return(MagickFalse);
   if ((index > list_info->elements) ||
@@ -829,7 +834,7 @@ MagickExport MagickBooleanType InsertElementInLinkedList(
     return(MagickFalse);
   next->datum=(void *) datum;
   next->next=(ElementInfo *) NULL;
-  AcquireSemaphoreInfo(&list_info->semaphore);
+  list_info->semaphore=AcquireSemaphoreInfo(list_info->semaphore);
   if (list_info->elements == 0)
     {
       if (list_info->next == (ElementInfo *) NULL)
@@ -873,7 +878,7 @@ MagickExport MagickBooleanType InsertElementInLinkedList(
           }
     }
   list_info->elements++;
-  RelinquishSemaphoreInfo(&list_info->semaphore);
+  RelinquishSemaphoreInfo(list_info->semaphore);
   return(MagickTrue);
 }
 
@@ -926,7 +931,7 @@ MagickExport MagickBooleanType InsertElementInSortedLinkedList(
   assert(list_info != (LinkedListInfo *) NULL);
   assert(list_info->signature == MagickSignature);
   if (list_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   if ((compare == NULL) || (datum == NULL))
     return(MagickFalse);
   if (list_info->elements == list_info->maximum_elements)
@@ -936,7 +941,7 @@ MagickExport MagickBooleanType InsertElementInSortedLinkedList(
     return(MagickFalse);
   next->datum=(void *) datum;
   element=(ElementInfo *) NULL;
-  AcquireSemaphoreInfo(&list_info->semaphore);
+  list_info->semaphore=AcquireSemaphoreInfo(list_info->semaphore);
   next->next=list_info->head;
   while (next->next != (ElementInfo *) NULL)
   {
@@ -970,7 +975,7 @@ MagickExport MagickBooleanType InsertElementInSortedLinkedList(
       list_info->tail=next;
     }
   list_info->elements++;
-  RelinquishSemaphoreInfo(&list_info->semaphore);
+  RelinquishSemaphoreInfo(list_info->semaphore);
   return(MagickTrue);
 }
 
@@ -1001,7 +1006,7 @@ MagickExport MagickBooleanType IsHashmapEmpty(const HashmapInfo *hashmap_info)
   assert(hashmap_info != (HashmapInfo *) NULL);
   assert(hashmap_info->signature == MagickSignature);
   if (hashmap_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   return((MagickBooleanType) (hashmap_info->entries == 0));
 }
 
@@ -1033,7 +1038,7 @@ MagickExport MagickBooleanType IsLinkedListEmpty(
   assert(list_info != (LinkedListInfo *) NULL);
   assert(list_info->signature == MagickSignature);
   if (list_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   return((MagickBooleanType) (list_info->elements == 0));
 }
 
@@ -1074,7 +1079,7 @@ MagickExport MagickBooleanType LinkedListToArray(LinkedListInfo *list_info,
   assert(list_info != (LinkedListInfo *) NULL);
   assert(list_info->signature == MagickSignature);
   if (list_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   if (array == (void **) NULL)
     return(MagickFalse);
   next=list_info->head;
@@ -1242,13 +1247,13 @@ MagickExport MagickBooleanType PutEntryInHashmap(HashmapInfo *hashmap_info,
   assert(hashmap_info != (HashmapInfo *) NULL);
   assert(hashmap_info->signature == MagickSignature);
   if (hashmap_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   if ((key == (void *) NULL) || (datum == (void *) NULL))
     return(MagickFalse);
   next=(EntryInfo *) AcquireMagickMemory(sizeof(*next));
   if (next == (EntryInfo *) NULL)
     return(MagickFalse);
-  AcquireSemaphoreInfo(&hashmap_info->semaphore);
+  hashmap_info->semaphore=AcquireSemaphoreInfo(hashmap_info->semaphore);
   next->hash=hashmap_info->hash(key);
   next->key=(void *) key;
   next->datum=(void *) datum;
@@ -1259,7 +1264,7 @@ MagickExport MagickBooleanType PutEntryInHashmap(HashmapInfo *hashmap_info,
       if (list_info == (LinkedListInfo *) NULL)
         {
           next=(EntryInfo *) RelinquishMagickMemory(next);
-          RelinquishSemaphoreInfo(&hashmap_info->semaphore);
+          RelinquishSemaphoreInfo(hashmap_info->semaphore);
           return(MagickFalse);
         }
       hashmap_info->map[next->hash % hashmap_info->maximum_entries]=list_info;
@@ -1286,11 +1291,11 @@ MagickExport MagickBooleanType PutEntryInHashmap(HashmapInfo *hashmap_info,
   if (InsertElementInLinkedList(list_info,0,next) == MagickFalse)
     {
       next=(EntryInfo *) RelinquishMagickMemory(next);
-      RelinquishSemaphoreInfo(&hashmap_info->semaphore);
+      RelinquishSemaphoreInfo(hashmap_info->semaphore);
       return(MagickFalse);
     }
   hashmap_info->entries++;
-  RelinquishSemaphoreInfo(&hashmap_info->semaphore);
+  RelinquishSemaphoreInfo(hashmap_info->semaphore);
   return(MagickTrue);
 }
 
@@ -1329,10 +1334,10 @@ MagickExport void *RemoveElementByValueFromLinkedList(LinkedListInfo *list_info,
   assert(list_info != (LinkedListInfo *) NULL);
   assert(list_info->signature == MagickSignature);
   if (list_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   if ((list_info->elements == 0) || (datum == NULL))
     return((void *) NULL);
-  AcquireSemaphoreInfo(&list_info->semaphore);
+  list_info->semaphore=AcquireSemaphoreInfo(list_info->semaphore);
   if (datum == list_info->head->datum)
     {
       if (list_info->next == list_info->head)
@@ -1352,7 +1357,7 @@ MagickExport void *RemoveElementByValueFromLinkedList(LinkedListInfo *list_info,
         next=next->next;
       if (next->next == (ElementInfo *) NULL)
         {
-          RelinquishSemaphoreInfo(&list_info->semaphore);
+          RelinquishSemaphoreInfo(list_info->semaphore);
           return((void *) NULL);
         }
       element=next->next;
@@ -1364,7 +1369,7 @@ MagickExport void *RemoveElementByValueFromLinkedList(LinkedListInfo *list_info,
       element=(ElementInfo *) RelinquishMagickMemory(element);
     }
   list_info->elements--;
-  RelinquishSemaphoreInfo(&list_info->semaphore);
+  RelinquishSemaphoreInfo(list_info->semaphore);
   return((void *) datum);
 }
 
@@ -1409,10 +1414,10 @@ MagickExport void *RemoveElementFromLinkedList(LinkedListInfo *list_info,
   assert(list_info != (LinkedListInfo *) NULL);
   assert(list_info->signature == MagickSignature);
   if (list_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   if (index >= list_info->elements)
     return((void *) NULL);
-  AcquireSemaphoreInfo(&list_info->semaphore);
+  list_info->semaphore=AcquireSemaphoreInfo(list_info->semaphore);
   if (index == 0)
     {
       if (list_info->next == list_info->head)
@@ -1440,7 +1445,7 @@ MagickExport void *RemoveElementFromLinkedList(LinkedListInfo *list_info,
       element=(ElementInfo *) RelinquishMagickMemory(element);
     }
   list_info->elements--;
-  RelinquishSemaphoreInfo(&list_info->semaphore);
+  RelinquishSemaphoreInfo(list_info->semaphore);
   return(entry);
 }
 
@@ -1489,10 +1494,10 @@ MagickExport void *RemoveEntryFromHashmap(HashmapInfo *hashmap_info,
   assert(hashmap_info != (HashmapInfo *) NULL);
   assert(hashmap_info->signature == MagickSignature);
   if (hashmap_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   if (key == NULL)
     return((void *) NULL);
-  AcquireSemaphoreInfo(&hashmap_info->semaphore);
+  hashmap_info->semaphore=AcquireSemaphoreInfo(hashmap_info->semaphore);
   hash=hashmap_info->hash(key);
   list_info=hashmap_info->map[hash % hashmap_info->maximum_entries];
   if (list_info != (LinkedListInfo *) NULL)
@@ -1506,7 +1511,7 @@ MagickExport void *RemoveEntryFromHashmap(HashmapInfo *hashmap_info,
             entry=(EntryInfo *) RemoveElementFromLinkedList(list_info,i);
             if (entry == (EntryInfo *) NULL)
               {
-                RelinquishSemaphoreInfo(&hashmap_info->semaphore);
+                RelinquishSemaphoreInfo(hashmap_info->semaphore);
                 return((void *) NULL);
               }
             if (hashmap_info->liberate_key != (void *(*)(void *)) NULL)
@@ -1514,13 +1519,13 @@ MagickExport void *RemoveEntryFromHashmap(HashmapInfo *hashmap_info,
             datum=entry->datum;
             entry=(EntryInfo *) RelinquishMagickMemory(entry);
             hashmap_info->entries--;
-            RelinquishSemaphoreInfo(&hashmap_info->semaphore);
+            RelinquishSemaphoreInfo(hashmap_info->semaphore);
             return(datum);
           }
         entry=(EntryInfo *) GetNextElementInLinkedList(list_info);
       }
     }
-  RelinquishSemaphoreInfo(&hashmap_info->semaphore);
+  RelinquishSemaphoreInfo(hashmap_info->semaphore);
   return((void *) NULL);
 }
 
@@ -1555,10 +1560,10 @@ MagickExport void *RemoveLastElementFromLinkedList(LinkedListInfo *list_info)
   assert(list_info != (LinkedListInfo *) NULL);
   assert(list_info->signature == MagickSignature);
   if (list_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   if (list_info->elements == 0)
     return((void *) NULL);
-  AcquireSemaphoreInfo(&list_info->semaphore);
+  list_info->semaphore=AcquireSemaphoreInfo(list_info->semaphore);
   if (list_info->next == list_info->tail)
     list_info->next=(ElementInfo *) NULL;
   if (list_info->elements == 1UL)
@@ -1581,7 +1586,7 @@ MagickExport void *RemoveLastElementFromLinkedList(LinkedListInfo *list_info)
       next->next=(ElementInfo *) NULL;
     }
   list_info->elements--;
-  RelinquishSemaphoreInfo(&list_info->semaphore);
+  RelinquishSemaphoreInfo(list_info->semaphore);
   return(entry);
 }
 
@@ -1613,7 +1618,7 @@ MagickExport void ResetHashmapIterator(HashmapInfo *hashmap_info)
   assert(hashmap_info != (HashmapInfo *) NULL);
   assert(hashmap_info->signature == MagickSignature);
   if (hashmap_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   hashmap_info->next=0;
   hashmap_info->head_of_list=MagickFalse;
 }
@@ -1647,6 +1652,6 @@ MagickExport void ResetLinkedListIterator(LinkedListInfo *list_info)
   assert(list_info != (LinkedListInfo *) NULL);
   assert(list_info->signature == MagickSignature);
   if (list_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   list_info->next=list_info->head;
 }
