@@ -1,4 +1,18 @@
 /*
+  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization
+  dedicated to making software imaging solutions freely available.
+  
+  You may not use this file except in compliance with the License.
+  obtain a copy of the License at
+  
+    http://www.imagemagick.org/www/Copyright.html
+  
+  Unless required by applicable law or agreed to in writing, software
+  distributed under the License is distributed on an "AS IS" BASIS,
+  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+  See the License for the specific language governing permissions and
+  limitations under the License.
+
   ImageMagick hash methods.
 */
 #ifndef _MAGICK_HASHMAP_H
@@ -19,10 +33,12 @@ typedef struct _LinkedListInfo
   LinkedListInfo;
 
 extern MagickExport HashmapInfo
+  *DestroyHashmap(HashmapInfo *),
   *NewHashmap(const unsigned long,size_t (*)(const void *),void *(*)(void *),
     void *(*)(void *));
 
 extern MagickExport LinkedListInfo
+  *DestroyLinkedList(LinkedListInfo *,void *(*)(void *)),
   *NewLinkedList(const unsigned long);
 
 extern MagickExport MagickBooleanType
@@ -46,8 +62,6 @@ extern MagickExport unsigned long
 
 extern MagickExport void
   ClearLinkedList(LinkedListInfo *,void *(*)(void *)),
-  DestroyHashmap(HashmapInfo *),
-  DestroyLinkedList(LinkedListInfo *,void *(*)(void *)),
   *GetElementFromLinkedList(const LinkedListInfo *,const unsigned long),
   *GetEntryFromHashmap(const HashmapInfo *,const void *),
   *GetLastElementInLinkedList(const LinkedListInfo *),
