@@ -17,7 +17,7 @@
 %                            September 1994                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -54,7 +54,8 @@
 #include "magick/delegate.h"
 #include "magick/draw.h"
 #include "magick/effect.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/gem.h"
 #include "magick/geometry.h"
 #include "magick/identify.h"
@@ -67,6 +68,7 @@
 #include "magick/memory_.h"
 #include "magick/module.h"
 #include "magick/mogrify.h"
+#include "magick/mogrify_private.h"
 #include "magick/monitor.h"
 #include "magick/montage.h"
 #include "magick/option.h"
@@ -135,6 +137,7 @@ static void IdentifyUsage(void)
       "-depth value         image depth",
       "-extract geometry    extract area from image",
       "-format \"string\"     output formatted image characteristics",
+      "-fuzz distance       colors within this distance are considered equal",
       "-help                print program options",
       "-interlace type      None, Line, Plane, or Partition",
       "-limit type value    Area, Disk, Map, or Memory resource limit",
@@ -145,6 +148,7 @@ static void IdentifyUsage(void)
       "                     horizontal and vertical sampling factor",
       "-size geometry       width and height of image",
       "-strip               strip image of all profiles and comments",
+      "-units type          PixelsPerInch, PixelsPerCentimeter, or Undefined",
       "-verbose             print detailed information about the image",
       "-version             print version information",
       "-virtual-pixel method",
@@ -167,16 +171,17 @@ MagickExport MagickBooleanType IdentifyImageCommand(ImageInfo *image_info,
 {
 #define DestroyIdentify() \
 { \
-  if (format != (char *) NULL) \
-    format=(char *) RelinquishMagickMemory(format); \
-  DestroyImageList(image); \
+  for ( ; k >= 0; k--) \
+    image_stack[k]=DestroyImageList(image_stack[k]); \
   for (i=0; i < (long) argc; i++) \
     argv[i]=(char *) RelinquishMagickMemory(argv[i]); \
   argv=(char **) RelinquishMagickMemory(argv); \
 }
-#define ThrowIdentifyException(severity,tag,option) \
+#define ThrowIdentifyException(asperity,tag,option) \
 { \
-  (void) ThrowMagickException(exception,GetMagickModule(),severity,tag,option);\
+  if (exception->severity < (asperity)) \
+    (void) ThrowMagickException(exception,GetMagickModule(),asperity,tag, \
+      option); \
   DestroyIdentify(); \
   return(MagickFalse); \
 }
@@ -190,25 +195,19 @@ MagickExport MagickBooleanType IdentifyImageCommand(ImageInfo *image_info,
 
   char
     *format,
-    *option,
-    *q;
+    *option;
 
   Image
-    *image;
-
-  ImageInfo
-    *identify_info;
+    *image_stack[MaxImageStackDepth+1];
 
   long
-    number_images;
+    j,
+    k;
 
-  MagickBooleanType
-    ping,
+  MagickStatusType
+    pend,
     status;
 
-  register Image
-    *p;
-
   register long
     i;
 
@@ -221,16 +220,18 @@ MagickExport MagickBooleanType IdentifyImageCommand(ImageInfo *image_info,
   assert(image_info != (ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(exception != (ExceptionInfo *) NULL);
   if (argc < 2)
     IdentifyUsage();
-  image=NewImageList();
   count=0;
   format=(char *) NULL;
-  number_images=0;
+  j=1;
+  k=0;
+  image_stack[k]=NewImageList();
   option=(char *) NULL;
-  ping=MagickTrue;
+  pend=MagickFalse;
+  status=MagickTrue;
   /*
     Identify an image.
   */
@@ -242,51 +243,64 @@ MagickExport MagickBooleanType IdentifyImageCommand(ImageInfo *image_info,
   for (i=1; i < (long) argc; i++)
   {
     option=argv[i];
-    if (LocaleCompare("-format",argv[i]) == 0)
+    if (LocaleCompare(option,"(") == 0)
       {
-        i++;
-        if (i == (long) argc)
-          ThrowIdentifyException(OptionError,"MissingArgument",option);
-        (void) CloneString(&format,argv[i]);
-        break;
+        if (k == MaxImageStackDepth)
+          ThrowIdentifyException(OptionError,"ParenthesisNestedTooDeeply",
+            option);
+        MogrifyImageStack(image_stack[k],MagickTrue,pend);
+        k++;
+        image_stack[k]=NewImageList();
+        continue;
+      }
+    if (LocaleCompare(option,")") == 0)
+      {
+        if (k == 0)
+          ThrowIdentifyException(OptionError,"UnableToParseExpression",option);
+        if (image_stack[k] != (Image *) NULL)
+          {
+            MogrifyImageStack(image_stack[k],MagickTrue,MagickTrue);
+            AppendImageToList(&image_stack[k-1],image_stack[k]);
+          }
+        k--;
+        continue;
       }
-  }
-  for (i=1; i < (long) argc; i++)
-  {
-    option=argv[i];
     if ((strlen(option) == 1) || ((*option != '-') && (*option != '+')))
       {
+        Image
+          *image;
+
+        ImageInfo
+          *identify_info;
+
         /*
-          Identify image.
+          Read input image.
         */
-        (void) CopyMagickString(image_info->filename,argv[i],MaxTextExtent);
-        if (format != (char *) NULL)
-          for (q=strchr(format,'%'); q != (char *) NULL; q=strchr(q+1,'%'))
-            if ((*(q+1) == 'g') || (*(q+1) == 'k') || (*(q+1) == 'z') ||
-                (*(q+1) == '#'))
-              {
-                ping=MagickFalse;
-                break;
-              }
+        MogrifyImageStack(image_stack[k],MagickFalse,pend);
         identify_info=CloneImageInfo(image_info);
         identify_info->verbose=MagickFalse;
-        if ((image_info->verbose != MagickFalse) || (ping == MagickFalse))
+        (void) CopyMagickString(identify_info->filename,argv[i],MaxTextExtent);
+        if ((image_info->verbose != MagickFalse) ||
+            (identify_info->ping == MagickFalse))
           image=ReadImage(identify_info,exception);
         else
           image=PingImage(identify_info,exception);
-        DestroyImageInfo(identify_info);
-        if (exception->severity != UndefinedException)
-          CatchException(exception);
-        status&=image != (Image *) NULL;
+        identify_info=DestroyImageInfo(identify_info);
+        status&=(image != (Image *) NULL) &&
+          (exception->severity < ErrorException);
         if (image == (Image *) NULL)
           continue;
-        for (p=image; p != (Image *) NULL; p=p->next)
+        GetImageException(image,exception);
+        AppendImageToList(&image_stack[k],image);
+        MogrifyImageStack(image_stack[k],MagickFalse,MagickTrue);
+        image=image_stack[k];
+        for ( ; image != (Image *) NULL; image=GetNextImageInList(image))
         {
-          if (p->scene == 0)
-            p->scene=count++;
+          if (image->scene == 0)
+            image->scene=count++;
           if (format == (char *) NULL)
             {
-              (void) DescribeImage(p,stdout,image_info->verbose);
+              (void) DescribeImage(image,stdout,image_info->verbose);
               continue;
             }
           if (metadata != (char **) NULL)
@@ -294,41 +308,33 @@ MagickExport MagickBooleanType IdentifyImageCommand(ImageInfo *image_info,
               char
                 *text;
 
-              text=TranslateText(image_info,p,format);
+              text=TranslateText(image_info,image,format);
               if (text == (char *) NULL)
                 ThrowIdentifyException(ResourceLimitError,
                   "MemoryAllocationFailed",strerror(errno));
               (void) ConcatenateString(&(*metadata),text);
               text=(char *) RelinquishMagickMemory(text);
+              if (LocaleCompare(format,"%n") == 0)
+                break;
             }
         }
-        DestroyImageList(image);
-        image=NewImageList();
-        number_images++;
+        image_stack[k]=DestroyImageList(image_stack[k]);
         continue;
       }
+    pend=image_stack[k] != (Image *) NULL ? MagickTrue : MagickFalse;
     switch (*(option+1))
     {
       case 'c':
       {
         if (LocaleCompare("cache",option+1) == 0)
           {
-            if (*option == '-')
-              {
-                unsigned long
-                  limit;
-
-                i++;
-                if (i == (long) argc)
-                  ThrowIdentifyException(OptionError,"MissingArgument",option);
-                if (IsGeometry(argv[i]) == MagickFalse)
-                  ThrowIdentifyInvalidArgumentException(option,argv[i]);
-                limit=(~0UL);
-                if (LocaleCompare("unlimited",argv[i]) != 0)
-                  limit=(unsigned long) atol(argv[i]);
-                (void) SetMagickResourceLimit(MemoryResource,limit);
-                (void) SetMagickResourceLimit(MapResource,2*limit);
-              }
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) argc)
+              ThrowIdentifyException(OptionError,"MissingArgument",option);
+            if (IsGeometry(argv[i]) == MagickFalse)
+              ThrowIdentifyInvalidArgumentException(option,argv[i]);
             break;
           }
         if (LocaleCompare("channel",option+1) == 0)
@@ -339,12 +345,13 @@ MagickExport MagickBooleanType IdentifyImageCommand(ImageInfo *image_info,
                   channel;
 
                 i++;
-                if (i == (long) argc)
+                if (i == (long) (argc-1))
                   ThrowIdentifyException(OptionError,"MissingArgument",option);
                 channel=ParseChannelOption(argv[i]);
                 if (channel < 0)
                   ThrowIdentifyException(OptionError,"UnrecognizedChannelType",
                     argv[i]);
+                break;
               }
             break;
           }
@@ -354,21 +361,18 @@ MagickExport MagickBooleanType IdentifyImageCommand(ImageInfo *image_info,
       {
         if (LocaleCompare("debug",option+1) == 0)
           {
-            (void) SetLogEventMask("None");
-            if (*option == '-')
-              {
-                LogEventType
-                  event_mask;
+            LogEventType
+              event_mask;
 
-                i++;
-                if (i == (long) argc)
-                  ThrowIdentifyException(OptionError,"MissingArgument",option);
-                event_mask=SetLogEventMask(argv[i]);
-                if (event_mask == UndefinedEvents)
-                  ThrowIdentifyException(OptionError,"UnrecognizedEventType",
-                    option);
-              }
-            image_info->debug=IsEventLogging();
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) argc)
+              ThrowIdentifyException(OptionError,"MissingArgument",option);
+            event_mask=SetLogEventMask(argv[i]);
+            if (event_mask == UndefinedEvents)
+              ThrowIdentifyException(OptionError,"UnrecognizedEventType",
+                option);
             break;
           }
         if (LocaleCompare("define",option+1) == 0)
@@ -378,46 +382,36 @@ MagickExport MagickBooleanType IdentifyImageCommand(ImageInfo *image_info,
               ThrowIdentifyException(OptionError,"MissingArgument",option);
             if (*option == '+')
               {
-                char
+                const char
                   *define;
 
-                define=RemoveImageOption(image_info,argv[i]);
-                if (define == (char *) NULL)
+                define=GetImageOption(image_info,argv[i]);
+                if (define == (const char *) NULL)
                   ThrowIdentifyException(OptionError,"NoSuchOption",argv[i]);
-                define=(char *) RelinquishMagickMemory(define);
                 break;
               }
-            status=DefineImageOption(image_info,argv[i]);
-            if (status == MagickFalse)
-              ThrowIdentifyException(OptionError,"UnrecognizedOption",argv[i]);
             break;
           }
         if (LocaleCompare("density",option+1) == 0)
           {
-            (void) CloneString(&image_info->density,(char *) NULL);
-            if (*option == '-')
-              {
-                i++;
-                if (i == (long) argc)
-                  ThrowIdentifyException(OptionError,"MissingArgument",option);
-                if (IsGeometry(argv[i]) == MagickFalse)
-                  ThrowIdentifyInvalidArgumentException(option,argv[i]);
-                (void) CloneString(&image_info->density,argv[i]);
-              }
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) argc)
+              ThrowIdentifyException(OptionError,"MissingArgument",option);
+            if (IsGeometry(argv[i]) == MagickFalse)
+              ThrowIdentifyInvalidArgumentException(option,argv[i]);
             break;
           }
         if (LocaleCompare("depth",option+1) == 0)
           {
-            image_info->depth=QuantumDepth;
-            if (*option == '-')
-              {
-                i++;
-                if (i == (long) argc)
-                  ThrowIdentifyException(OptionError,"MissingArgument",option);
-                if (IsGeometry(argv[i]) == MagickFalse)
-                  ThrowIdentifyInvalidArgumentException(option,argv[i]);
-                image_info->depth=(unsigned long) atol(argv[i]);
-              }
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) argc)
+              ThrowIdentifyException(OptionError,"MissingArgument",option);
+            if (IsGeometry(argv[i]) == MagickFalse)
+              ThrowIdentifyInvalidArgumentException(option,argv[i]);
             break;
           }
         ThrowIdentifyException(OptionError,"UnrecognizedOption",option)
@@ -426,12 +420,32 @@ MagickExport MagickBooleanType IdentifyImageCommand(ImageInfo *image_info,
       {
         if (LocaleCompare("format",option+1) == 0)
           {
-            if (*option == '-')
-              {
-                i++;
-                if (i == (long) argc)
-                  ThrowIdentifyException(OptionError,"MissingArgument",option);
-              }
+            register char
+              *q;
+
+            format=(char *) NULL;
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) argc)
+              ThrowIdentifyException(OptionError,"MissingArgument",option);
+            format=argv[i];
+            image_info->ping=MagickTrue;
+            for (q=strchr(format,'%'); q != (char *) NULL; q=strchr(q+1,'%'))
+              if ((*(q+1) == 'g') || (*(q+1) == 'k') || (*(q+1) == 'z') ||
+                  (*(q+1) == '@') || (*(q+1) == '#'))
+                image_info->ping=MagickFalse;
+            break;
+          }
+        if (LocaleCompare("fuzz",option+1) == 0)
+          {
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) (argc-1))
+              ThrowIdentifyException(OptionError,"MissingArgument",option);
+            if (IsGeometry(argv[i]) == MagickFalse)
+              ThrowIdentifyInvalidArgumentException(option,argv[i]);
             break;
           }
         ThrowIdentifyException(OptionError,"UnrecognizedOption",option)
@@ -446,22 +460,19 @@ MagickExport MagickBooleanType IdentifyImageCommand(ImageInfo *image_info,
       {
         if (LocaleCompare("interlace",option+1) == 0)
           {
-            image_info->interlace=UndefinedInterlace;
-            if (*option == '-')
-              {
-                long
-                  interlace;
+            long
+              interlace;
 
-                i++;
-                if (i == (long) argc)
-                  ThrowIdentifyException(OptionError,"MissingArgument",option);
-                interlace=ParseMagickOption(MagickInterlaceOptions,MagickFalse,
-                  argv[i]);
-                if (interlace < 0)
-                  ThrowIdentifyException(OptionError,
-                    "UnrecognizedInterlaceType",argv[i]);
-                image_info->interlace=(InterlaceType) interlace;
-              }
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) argc)
+              ThrowIdentifyException(OptionError,"MissingArgument",option);
+            interlace=ParseMagickOption(MagickInterlaceOptions,MagickFalse,
+              argv[i]);
+            if (interlace < 0)
+              ThrowIdentifyException(OptionError,
+                "UnrecognizedInterlaceType",argv[i]);
             break;
           }
         ThrowIdentifyException(OptionError,"UnrecognizedOption",option)
@@ -470,163 +481,52 @@ MagickExport MagickBooleanType IdentifyImageCommand(ImageInfo *image_info,
       {
         if (LocaleCompare("limit",option+1) == 0)
           {
-            if (*option == '-')
-              {
-                char
-                  *type;
+            long
+              resource;
 
-                long
-                  resource;
-
-                unsigned long
-                  limit;
-
-                i++;
-                if (i == (long) argc)
-                  ThrowIdentifyException(OptionError,"MissingArgument",option);
-                type=argv[i];
-                i++;
-                if (i == (long) argc)
-                  ThrowIdentifyException(OptionError,"MissingArgument",option);
-                if (IsGeometry(argv[i]) == MagickFalse)
-                  ThrowIdentifyInvalidArgumentException(option,argv[i]);
-                limit=(~0UL);
-                if (LocaleCompare("unlimited",argv[i]) != 0)
-                  limit=(unsigned long) atol(argv[i]);
-                resource=ParseMagickOption(MagickResourceOptions,MagickFalse,type);
-                if (resource < 0)
-                  ThrowIdentifyException(OptionError,
-                    "UnrecognizedResourceType",type);
-                (void) SetMagickResourceLimit((ResourceType) resource,limit);
-                break;
-              }
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) argc)
+              ThrowIdentifyException(OptionError,"MissingArgument",option);
+            resource=ParseMagickOption(MagickResourceOptions,MagickFalse,
+              argv[i]);
+            if (resource < 0)
+              ThrowIdentifyException(OptionError,"UnrecognizedResourceType",
+                argv[i]);
+            i++;
+            if (i == (long) argc)
+              ThrowIdentifyException(OptionError,"MissingArgument",option);
+            if ((LocaleCompare("unlimited",argv[i]) != 0) &&
+                (IsGeometry(argv[i]) == MagickFalse))
+              ThrowIdentifyInvalidArgumentException(option,argv[i]);
             break;
           }
         if (LocaleCompare("list",option+1) == 0)
           {
-            if (*option == '-')
-              {
-                i++;
-                if (i == (long) argc)
-                  ThrowIdentifyException(OptionError,"MissingArgument",option);
-                switch (*argv[i])
-                {
-                  case 'C':
-                  case 'c':
-                  {
-                    if (LocaleCompare("Coder",argv[i]) == 0)
-                      {
-                        (void) ListCoderInfo((FILE *) NULL,exception);
-                        break;
-                      }
-                    if (LocaleCompare("Color",argv[i]) == 0)
-                      {
-                        (void) ListColorInfo((FILE *) NULL,exception);
-                        break;
-                      }
-                    if (LocaleCompare("Configure",argv[i]) == 0)
-                      {
-                        (void) ListConfigureInfo((FILE *) NULL,exception);
-                        break;
-                      }
-                    ThrowIdentifyException(OptionError,"UnrecognizedListType",
-                      argv[i])
-                  }
-                  case 'D':
-                  case 'd':
-                  {
-                    if (LocaleCompare("Delegate",argv[i]) == 0)
-                      {
-                        (void) ListDelegateInfo((FILE *) NULL,exception);
-                        break;
-                      }
-                    ThrowIdentifyException(OptionError,"UnrecognizedListType",
-                      argv[i])
-                  }
-                  case 'F':
-                  case 'f':
-                  {
-                    if (LocaleCompare("Format",argv[i]) == 0)
-                      {
-                        (void) ListMagickInfo((FILE *) NULL,exception);
-                        break;
-                      }
-                    ThrowIdentifyException(OptionError,"UnrecognizedListType",
-                      argv[i])
-                  }
-                  case 'L':
-                  case 'l':
-                  {
-                    if (LocaleCompare("Locale",argv[i]) == 0)
-                      {
-                        (void) ListLocaleInfo((FILE *) NULL,exception);
-                        break;
-                      }
-                    if (LocaleCompare("Log",argv[i]) == 0)
-                      {
-                        (void) ListLogInfo((FILE *) NULL,exception);
-                        break;
-                      }
-                    ThrowIdentifyException(OptionError,"UnrecognizedListType",
-                      argv[i])
-                  }
-                  case 'M':
-                  case 'm':
-                  {
-                    if (LocaleCompare("Magic",argv[i]) == 0)
-                      {
-                        (void) ListMagicInfo((FILE *) NULL,exception);
-                        break;
-                      }
-#if defined(SupportMagickModules)
-                    if (LocaleCompare("Module",argv[i]) == 0)
-                      {
-                        (void) ListModuleInfo((FILE *) NULL,exception);
-                        break;
-                      }
-#endif
-                    ThrowIdentifyException(OptionError,"UnrecognizedListType",
-                      argv[i])
-                  }
-                  case 'R':
-                  case 'r':
-                  {
-                    if (LocaleCompare("Resource",argv[i]) == 0)
-                      {
-                        (void) ListMagickResourceInfo((FILE *) NULL,exception);
-                        break;
-                      }
-                    ThrowIdentifyException(OptionError,"UnrecognizedListType",
-                      argv[i])
-                  }
-                  case 'T':
-                  case 't':
-                  {
-                    if (LocaleCompare("Type",argv[i]) == 0)
-                      {
-                        (void) ListTypeInfo((FILE *) NULL,exception);
-                        break;
-                      }
-                    ThrowIdentifyException(OptionError,"UnrecognizedListType",
-                      argv[i])
-                  }
-                  default:
-                    ThrowIdentifyException(OptionError,"UnrecognizedListType",
-                      argv[i])
-                }
-                return(MagickTrue);
-              }
-            break;
+            long
+              list;
+
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) argc)
+              ThrowIdentifyException(OptionError,"MissingArgument",option);
+            list=ParseMagickOption(MagickListOptions,MagickFalse,argv[i]);
+            if (list < 0)
+              ThrowIdentifyException(OptionError,"UnrecognizedListType",
+                argv[i]);
+            (void) MogrifyImageInfo(image_info,(int) (i-j+1),argv+j,exception);
+            return(MagickTrue);
           }
         if (LocaleCompare("log",option+1) == 0)
           {
-            if (*option == '-')
-              {
-                i++;
-                if ((i == (long) argc) || (strchr(argv[i],'%') == (char *) NULL))
-                  ThrowIdentifyException(OptionError,"MissingArgument",option);
-                (void) SetLogFormat(argv[i]);
-              }
+            if (*option == '+')
+              break;
+            i++;
+            if ((i == (long) argc) ||
+                (strchr(argv[i],'%') == (char *) NULL))
+              ThrowIdentifyException(OptionError,"MissingArgument",option);
             break;
           }
         ThrowIdentifyException(OptionError,"UnrecognizedOption",option)
@@ -641,44 +541,58 @@ MagickExport MagickBooleanType IdentifyImageCommand(ImageInfo *image_info,
       {
         if (LocaleCompare("sampling-factor",option+1) == 0)
           {
-            (void) CloneString(&image_info->sampling_factor,(char *) NULL);
-            if (*option == '-')
-              {
-                i++;
-                if (i == (long) argc)
-                  ThrowIdentifyException(OptionError,"MissingArgument",option);
-                if (IsGeometry(argv[i]) == MagickFalse)
-                  ThrowIdentifyInvalidArgumentException(option,argv[i]);
-                (void) CloneString(&image_info->sampling_factor,argv[i]);
-              }
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) argc)
+              ThrowIdentifyException(OptionError,"MissingArgument",option);
+            if (IsGeometry(argv[i]) == MagickFalse)
+              ThrowIdentifyInvalidArgumentException(option,argv[i]);
             break;
           }
         if (LocaleCompare("size",option+1) == 0)
           {
-            (void) CloneString(&image_info->size,(char *) NULL);
-            if (*option == '-')
-              {
-                i++;
-                if (i == (long) argc)
-                  ThrowIdentifyException(OptionError,"MissingArgument",option);
-                if (IsGeometry(argv[i]) == MagickFalse)
-                  ThrowIdentifyInvalidArgumentException(option,argv[i]);
-                (void) CloneString(&image_info->size,argv[i]);
-              }
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) argc)
+              ThrowIdentifyException(OptionError,"MissingArgument",option);
+            if (IsGeometry(argv[i]) == MagickFalse)
+              ThrowIdentifyInvalidArgumentException(option,argv[i]);
             break;
           }
         if (LocaleCompare("strip",option+1) == 0)
           break;
         if (LocaleCompare("support",option+1) == 0)
           {
-            if (*option == '-')
-              {
-                i++;
-                if (i == (long) argc)
-                  ThrowIdentifyException(OptionError,"MissingArgument",option);
-                if (IsGeometry(argv[i]) == MagickFalse)
-                  ThrowIdentifyInvalidArgumentException(option,argv[i]);
-              }
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) argc)
+              ThrowIdentifyException(OptionError,"MissingArgument",option);
+            if (IsGeometry(argv[i]) == MagickFalse)
+              ThrowIdentifyInvalidArgumentException(option,argv[i]);
+            break;
+          }
+        ThrowIdentifyException(OptionError,"UnrecognizedOption",option)
+      }
+      case 'u':
+      {
+        if (LocaleCompare("units",option+1) == 0)
+          {
+            long
+              units;
+
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) (argc-1))
+              ThrowIdentifyException(OptionError,"MissingArgument",option);
+            units=ParseMagickOption(MagickResolutionOptions,MagickFalse,
+              argv[i]);
+            if (units < 0)
+              ThrowIdentifyException(OptionError,"UnrecognizedUnitsType",
+                argv[i]);
             break;
           }
         ThrowIdentifyException(OptionError,"UnrecognizedOption",option)
@@ -687,27 +601,25 @@ MagickExport MagickBooleanType IdentifyImageCommand(ImageInfo *image_info,
       {
         if (LocaleCompare("verbose",option+1) == 0)
           {
-            image_info->verbose=(MagickBooleanType) (*option == '-');
             break;
           }
         if (LocaleCompare("version",option+1) == 0)
           break;
         if (LocaleCompare("virtual-pixel",option+1) == 0)
           {
-            if (*option == '-')
-              {
-                long
-                  method;
+            long
+              method;
 
-                i++;
-                if (i == (long) (argc-1))
-                  ThrowIdentifyException(OptionError,"MissingArgument",option);
-                method=ParseMagickOption(MagickVirtualPixelOptions,MagickFalse,
-                  argv[i]);
-                if (method < 0)
-                  ThrowIdentifyException(OptionError,
-                    "UnrecognizedVirtualPixelMethod",argv[i]);
-              }
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) (argc-1))
+              ThrowIdentifyException(OptionError,"MissingArgument",option);
+            method=ParseMagickOption(MagickVirtualPixelOptions,MagickFalse,
+              argv[i]);
+            if (method < 0)
+              ThrowIdentifyException(OptionError,
+                "UnrecognizedVirtualPixelMethod",argv[i]);
             break;
           }
         ThrowIdentifyException(OptionError,"UnrecognizedOption",option)
@@ -717,13 +629,15 @@ MagickExport MagickBooleanType IdentifyImageCommand(ImageInfo *image_info,
       default:
         ThrowIdentifyException(OptionError,"UnrecognizedOption",option)
     }
+    status=(MagickStatusType)
+      ParseMagickOption(MagickMogrifyOptions,MagickFalse,option+1);
+    if (status == MagickTrue)
+      MogrifyImageStack(image_stack[k],MagickTrue,MagickTrue);
   }
+  if (k != 0)
+    ThrowIdentifyException(OptionError,"UnbalancedParenthesis",argv[i]);
   if (i != argc)
     ThrowIdentifyException(OptionError,"MissingAnImageFilename",argv[i]);
-  if (number_images == 0)
-    ThrowIdentifyException(OptionError,"MissingAnImageFilename",argv[argc-1]);
-  if (format != (char *) NULL)
-    format=(char *) RelinquishMagickMemory(format);
   DestroyIdentify();
-  return(status);
+  return(status != 0 ? MagickTrue : MagickFalse);
 }
