@@ -17,7 +17,7 @@
 %                                 July 2003                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -43,7 +43,8 @@
 #include "magick/blob.h"
 #include "magick/client.h"
 #include "magick/configure.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/hashmap.h"
 #include "magick/locale_.h"
 #include "magick/log.h"
@@ -57,7 +58,7 @@
 /*
   Define declarations.
 */
-#define LocaleFilename  "locale.mgk"
+#define LocaleFilename  "locale.xml"
 
 /*
   Static declarations.
@@ -66,7 +67,11 @@ static const char
   *LocaleMap = (char *)
     "<?xml version=\"1.0\"?>"
     "<localemap>"
-    "  <locale locale=\"C\" file=\"english.mgk\">"
+    "  <locale name=\"C\">"
+    "    <Exception>"
+    "     <Message name=\"\">"
+    "     </Message>"
+    "    </Exception>"
     "  </locale>"
     "</localemap>";
 
@@ -118,19 +123,18 @@ static void *DestroyLocaleElement(void *locale_info)
     p->tag=(char *) RelinquishMagickMemory(p->tag);
   if (p->message != (char *) NULL)
     p->message=(char *) RelinquishMagickMemory(p->message);
-  p=(LocaleInfo *) RelinquishMagickMemory(p);
-  return((void *) NULL);
+  return((LocaleInfo *) RelinquishMagickMemory(p));
 }
 
 MagickExport void DestroyLocaleList(void)
 {
-  AcquireSemaphoreInfo(&locale_semaphore);
+  locale_semaphore=AcquireSemaphoreInfo(locale_semaphore);
   if (locale_list != (LinkedListInfo *) NULL)
-    DestroyLinkedList(locale_list,DestroyLocaleElement);
+    locale_list=DestroyLinkedList(locale_list,DestroyLocaleElement);
   locale_list=(LinkedListInfo *) NULL;
   instantiate_locale=MagickFalse;
-  RelinquishSemaphoreInfo(&locale_semaphore);
-  DestroySemaphoreInfo(&locale_semaphore);
+  RelinquishSemaphoreInfo(locale_semaphore);
+  locale_semaphore=DestroySemaphoreInfo(locale_semaphore);
 }
 
 /*
@@ -149,7 +153,7 @@ MagickExport void DestroyLocaleList(void)
 %
 %  The format of the DestroyProfiles method is:
 %
-%      void DestroyLocaleOptions(Image *image)
+%      LinkedListInfo *DestroyLocaleOptions(Image *image)
 %
 %  A description of each parameter follows:
 %
@@ -160,15 +164,14 @@ MagickExport void DestroyLocaleList(void)
 
 static void *DestroyOptions(void *message)
 {
-  DestroyStringInfo((StringInfo *) message);
-  return((void *) NULL);
+  return(DestroyStringInfo((StringInfo *) message));
 }
 
-MagickExport void DestroyLocaleOptions(LinkedListInfo *messages)
+MagickExport LinkedListInfo *DestroyLocaleOptions(LinkedListInfo *messages)
 {
   assert(messages != (LinkedListInfo *) NULL);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
-  DestroyLinkedList(messages,DestroyOptions);
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
+  return(DestroyLinkedList(messages,DestroyOptions));
 }
 
 /*
@@ -204,9 +207,11 @@ MagickExport const LocaleInfo *GetLocaleInfo_(const char *tag,
   register const LocaleInfo
     *p;
 
+  assert(exception != (ExceptionInfo *) NULL);
   if ((locale_list == (LinkedListInfo *) NULL) ||
       (instantiate_locale == MagickFalse))
-    (void) InitializeLocaleList(exception);
+    if (InitializeLocaleList(exception) == MagickFalse)
+      return((const LocaleInfo *) NULL);
   if ((locale_list == (LinkedListInfo *) NULL) ||
       (IsLinkedListEmpty(locale_list) == MagickTrue))
     return((const LocaleInfo *) NULL);
@@ -215,7 +220,7 @@ MagickExport const LocaleInfo *GetLocaleInfo_(const char *tag,
   /*
     Search for named tag.
   */
-  AcquireSemaphoreInfo(&locale_semaphore);
+  locale_semaphore=AcquireSemaphoreInfo(locale_semaphore);
   ResetLinkedListIterator(locale_list);
   p=(const LocaleInfo *) GetNextElementInLinkedList(locale_list);
   while (p != (const LocaleInfo *) NULL)
@@ -224,7 +229,7 @@ MagickExport const LocaleInfo *GetLocaleInfo_(const char *tag,
       break;
     p=(const LocaleInfo *) GetNextElementInLinkedList(locale_list);
   }
-  RelinquishSemaphoreInfo(&locale_semaphore);
+  RelinquishSemaphoreInfo(locale_semaphore);
   return(p);
 }
 
@@ -245,7 +250,7 @@ MagickExport const LocaleInfo *GetLocaleInfo_(const char *tag,
 %  The format of the GetLocaleInfoList function is:
 %
 %      const LocaleInfo **GetLocaleInfoList(const char *pattern,
-%        unsigned long *number_messages)
+%        unsigned long *number_messages,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
@@ -254,6 +259,7 @@ MagickExport const LocaleInfo *GetLocaleInfo_(const char *tag,
 %    o number_messages:  This integer returns the number of locale messages in
 %    the list.
 %
+%    o exception: Return any errors or warnings in this structure.
 %
 */
 
@@ -279,14 +285,11 @@ static int LocaleInfoCompare(const void *x,const void *y)
 #endif
 
 MagickExport const LocaleInfo **GetLocaleInfoList(const char *pattern,
-  unsigned long *number_messages)
+  unsigned long *number_messages,ExceptionInfo *exception)
 {
   const LocaleInfo
     **messages;
 
-  ExceptionInfo
-    exception;
-
   register const LocaleInfo
     *p;
 
@@ -300,9 +303,7 @@ MagickExport const LocaleInfo **GetLocaleInfoList(const char *pattern,
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),pattern);
   assert(number_messages != (unsigned long *) NULL);
   *number_messages=0;
-  GetExceptionInfo(&exception);
-  p=GetLocaleInfo_("*",&exception);
-  DestroyExceptionInfo(&exception);
+  p=GetLocaleInfo_("*",exception);
   if (p == (const LocaleInfo *) NULL)
     return((const LocaleInfo **) NULL);
   messages=(const LocaleInfo **) AcquireMagickMemory((size_t)
@@ -312,7 +313,7 @@ MagickExport const LocaleInfo **GetLocaleInfoList(const char *pattern,
   /*
     Generate locale list.
   */
-  AcquireSemaphoreInfo(&locale_semaphore);
+  locale_semaphore=AcquireSemaphoreInfo(locale_semaphore);
   ResetLinkedListIterator(locale_list);
   p=(const LocaleInfo *) GetNextElementInLinkedList(locale_list);
   for (i=0; p != (const LocaleInfo *) NULL; )
@@ -322,7 +323,7 @@ MagickExport const LocaleInfo **GetLocaleInfoList(const char *pattern,
       messages[i++]=p;
     p=(const LocaleInfo *) GetNextElementInLinkedList(locale_list);
   }
-  RelinquishSemaphoreInfo(&locale_semaphore);
+  RelinquishSemaphoreInfo(locale_semaphore);
   qsort((void *) messages,(size_t) i,sizeof(*messages),LocaleInfoCompare);
   messages[i]=(LocaleInfo *) NULL;
   *number_messages=(unsigned long) i;
@@ -345,14 +346,17 @@ MagickExport const LocaleInfo **GetLocaleInfoList(const char *pattern,
 %
 %  The format of the GetLocaleList function is:
 %
-%      char **GetLocaleList(const char *pattern,unsigned long *number_messages)
+%      char **GetLocaleList(const char *pattern,unsigned long *number_messages,
+%        Exceptioninfo *exception)
 %
 %  A description of each parameter follows:
 %
 %    o pattern: Specifies a pointer to a text string containing a pattern.
 %
-%    o number_messages:  This integer returns the number of messages in the list.
+%    o number_messages:  This integer returns the number of messages in the
+%      list.
 %
+%    o exception: Return any errors or warnings in this structure.
 %
 */
 
@@ -376,14 +380,11 @@ static int LocaleTagCompare(const void *x,const void *y)
 #endif
 
 MagickExport char **GetLocaleList(const char *pattern,
-  unsigned long *number_messages)
+  unsigned long *number_messages,ExceptionInfo *exception)
 {
   char
     **messages;
 
-  ExceptionInfo
-    exception;
-
   register const LocaleInfo
     *p;
 
@@ -397,26 +398,25 @@ MagickExport char **GetLocaleList(const char *pattern,
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),pattern);
   assert(number_messages != (unsigned long *) NULL);
   *number_messages=0;
-  GetExceptionInfo(&exception);
-  p=GetLocaleInfo_("*",&exception);
-  DestroyExceptionInfo(&exception);
+  p=GetLocaleInfo_("*",exception);
   if (p == (const LocaleInfo *) NULL)
     return((char **) NULL);
-  AcquireSemaphoreInfo(&locale_semaphore);
-  RelinquishSemaphoreInfo(&locale_semaphore);
+  locale_semaphore=AcquireSemaphoreInfo(locale_semaphore);
+  RelinquishSemaphoreInfo(locale_semaphore);
   messages=(char **) AcquireMagickMemory((size_t)
     (GetNumberOfElementsInLinkedList(locale_list)+1)*sizeof(*messages));
   if (messages == (char **) NULL)
     return((char **) NULL);
-  AcquireSemaphoreInfo(&locale_semaphore);
+  locale_semaphore=AcquireSemaphoreInfo(locale_semaphore);
   p=(const LocaleInfo *) GetNextElementInLinkedList(locale_list);
   for (i=0; p != (const LocaleInfo *) NULL; )
   {
-    if ((p->stealth == MagickFalse) && (GlobExpression(p->tag,pattern) != MagickFalse))
+    if ((p->stealth == MagickFalse) &&
+        (GlobExpression(p->tag,pattern) != MagickFalse))
       messages[i++]=AcquireString(p->tag);
     p=(const LocaleInfo *) GetNextElementInLinkedList(locale_list);
   }
-  RelinquishSemaphoreInfo(&locale_semaphore);
+  RelinquishSemaphoreInfo(locale_semaphore);
   qsort((void *) messages,(size_t) i,sizeof(*messages),LocaleTagCompare);
   messages[i]=(char *) NULL;
   *number_messages=(unsigned long) i;
@@ -535,7 +535,7 @@ MagickExport LinkedListInfo *GetLocaleOptions(const char *filename,
           (void) AppendElementToLinkedList(messages,xml);
         element=(const char *) GetNextElementInLinkedList(paths);
       }
-      DestroyLinkedList(paths,RelinquishMagickMemory);
+      paths=DestroyLinkedList(paths,RelinquishMagickMemory);
     }
 #if defined(__WINDOWS__)
   {
@@ -551,9 +551,6 @@ MagickExport LinkedListInfo *GetLocaleOptions(const char *filename,
       }
   }
 #endif
-  if (GetNumberOfElementsInLinkedList(messages) == 0)
-    (void) ThrowMagickException(exception,GetMagickModule(),ConfigureError,
-      "UnableToOpenLocaleFile",filename);
   ResetLinkedListIterator(messages);
   return(messages);
 }
@@ -582,7 +579,7 @@ MagickExport LinkedListInfo *GetLocaleOptions(const char *filename,
 */
 MagickExport const char *GetLocaleValue(const LocaleInfo *locale_info)
 {
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(locale_info != (LocaleInfo *) NULL);
   assert(locale_info->signature == MagickSignature);
   return(locale_info->message);
@@ -612,22 +609,12 @@ MagickExport const char *GetLocaleValue(const LocaleInfo *locale_info)
 */
 static MagickBooleanType InitializeLocaleList(ExceptionInfo *exception)
 {
-  MagickBooleanType
-    active;
-
-  active=MagickFalse;
   if ((locale_list == (LinkedListInfo *) NULL) &&
       (instantiate_locale == MagickFalse))
     {
-      AcquireSemaphoreInfo(&locale_semaphore);
+      locale_semaphore=AcquireSemaphoreInfo(locale_semaphore);
       if ((locale_list == (LinkedListInfo *) NULL) &&
           (instantiate_locale == MagickFalse))
-        {
-          instantiate_locale=MagickTrue;
-          active=MagickTrue;
-        }
-      RelinquishSemaphoreInfo(&locale_semaphore);
-      if (active == MagickTrue)
         {
           char
             *locale;
@@ -644,9 +631,9 @@ static MagickBooleanType InitializeLocaleList(ExceptionInfo *exception)
           if ((locale == (char *) NULL) || (*locale == '\0'))
             locale="C";
           (void) LoadLocaleLists(LocaleFilename,locale,exception);
-          if (GetNumberOfElementsInLinkedList(locale_list) == 0)
-            (void) LoadLocaleLists("english.mgk",locale,exception);
+          instantiate_locale=MagickTrue;
         }
+      RelinquishSemaphoreInfo(locale_semaphore);
     }
   return((MagickBooleanType) (locale_list != (LinkedListInfo *) NULL));
 }
@@ -694,7 +681,7 @@ MagickExport MagickBooleanType ListLocaleInfo(FILE *file,
   if (file == (const FILE *) NULL)
     file=stdout;
   number_messages=0;
-  locale_info=GetLocaleInfoList("*",&number_messages);
+  locale_info=GetLocaleInfoList("*",&number_messages,exception);
   if (locale_info == (const LocaleInfo **) NULL)
     return(MagickFalse);
   path=(const char *) NULL;
@@ -774,6 +761,8 @@ static void ChopLocaleComponents(char *path,const unsigned long components)
         *p='\0';
         count++;
       }
+  if (count < (long) components)
+    *path='\0';
 }
 
 static MagickBooleanType LoadLocaleList(const char *xml,const char *filename,
@@ -960,7 +949,8 @@ static MagickBooleanType LoadLocaleList(const char *xml,const char *filename,
         if (*(keyword+1) == '/')
           {
             ChopLocaleComponents(tag,1);
-            (void) ConcatenateMagickString(tag,"/",MaxTextExtent);
+            if (*tag != '\0')
+              (void) ConcatenateMagickString(tag,"/",MaxTextExtent);
             continue;
           }
         token[strlen(token)-1]='\0';
@@ -1029,8 +1019,21 @@ static MagickBooleanType LoadLocaleLists(const char *filename,
       exception);
     option=(const StringInfo *) GetNextElementInLinkedList(options);
   }
-  DestroyLocaleOptions(options);
-  if ((locale_list == (LinkedListInfo *) NULL) || 
+  options=DestroyLocaleOptions(options);
+  if ((locale_list == (LinkedListInfo *) NULL) ||
+      (IsLinkedListEmpty(locale_list) != MagickFalse))
+    {
+      options=GetLocaleOptions("english.xml",exception);
+      option=(const StringInfo *) GetNextElementInLinkedList(options);
+      while (option != (const StringInfo *) NULL)
+      {
+        status|=LoadLocaleList((const char *) option->datum,option->path,locale,
+          0,exception);
+        option=(const StringInfo *) GetNextElementInLinkedList(options);
+      }
+      options=DestroyLocaleOptions(options);
+    }
+  if ((locale_list == (LinkedListInfo *) NULL) ||
       (IsLinkedListEmpty(locale_list) != MagickFalse))
     status|=LoadLocaleList(LocaleMap,"built-in",locale,0,exception);
   return(status != 0 ? MagickTrue : MagickFalse);
