@@ -18,7 +18,7 @@
 %                             November 1998                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -50,7 +50,8 @@
 #include "magick/constitute.h"
 #include "magick/delegate.h"
 #include "magick/draw.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/locale_.h"
 #include "magick/log.h"
 #include "magick/magic.h"
@@ -100,7 +101,7 @@ static SemaphoreInfo
   *magick_semaphore = (SemaphoreInfo *) NULL;
 
 static SignalHandler
-  *signal_handlers[SIGMAX];
+  *signal_handlers[SIGMAX] = { (SignalHandler *) NULL };
 
 static volatile MagickBooleanType
   instantiate_magick = MagickFalse;  /* double-checked locking pattern */
@@ -203,12 +204,11 @@ MagickExport void DestroyMagickList(void)
 #if !defined(BuildMagickModules)
   UnregisterStaticModules();
 #endif
-  AcquireSemaphoreInfo(&magick_semaphore);
+  magick_semaphore=AcquireSemaphoreInfo(magick_semaphore);
   if (magick_list != (LinkedListInfo *) NULL)
-    DestroyLinkedList(magick_list,DestroyMagickElement);
-  magick_list=(LinkedListInfo *) NULL;
-  RelinquishSemaphoreInfo(&magick_semaphore);
-  DestroySemaphoreInfo(&magick_semaphore);
+    magick_list=DestroyLinkedList(magick_list,DestroyMagickElement);
+  RelinquishSemaphoreInfo(magick_semaphore);
+  magick_semaphore=DestroySemaphoreInfo(magick_semaphore);
 }
 
 /*
@@ -247,14 +247,14 @@ MagickExport const char *GetImageMagick(const unsigned char *magick,
   register const MagickInfo
     *p;
 
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(magick != (const unsigned char *) NULL);
   GetExceptionInfo(&exception);
   p=GetMagickInfo("*",&exception);
   DestroyExceptionInfo(&exception);
   if (p == (const MagickInfo *) NULL)
     return((const char *) NULL);
-  AcquireSemaphoreInfo(&magick_semaphore);
+  magick_semaphore=AcquireSemaphoreInfo(magick_semaphore);
   ResetLinkedListIterator(magick_list);
   p=(const MagickInfo *) GetNextElementInLinkedList(magick_list);
   while (p != (const MagickInfo *) NULL)
@@ -264,7 +264,7 @@ MagickExport const char *GetImageMagick(const unsigned char *magick,
       break;
     p=(const MagickInfo *) GetNextElementInLinkedList(magick_list);
   }
-  RelinquishSemaphoreInfo(&magick_semaphore);
+  RelinquishSemaphoreInfo(magick_semaphore);
   if (p != (MagickInfo *) NULL)
     return(p->name);
   return((char *) NULL);
@@ -294,13 +294,12 @@ MagickExport const char *GetImageMagick(const unsigned char *magick,
 */
 MagickExport MagickBooleanType GetMagickAdjoin(const MagickInfo *magick_info)
 {
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(magick_info != (MagickInfo *) NULL);
   assert(magick_info->signature == MagickSignature);
   return(magick_info->adjoin);
 }
 
-
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -326,7 +325,7 @@ MagickExport MagickBooleanType GetMagickAdjoin(const MagickInfo *magick_info)
 MagickExport MagickBooleanType GetMagickBlobSupport(
   const MagickInfo *magick_info)
 {
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(magick_info != (MagickInfo *) NULL);
   assert(magick_info->signature == MagickSignature);
   return(magick_info->blob_support);
@@ -356,7 +355,7 @@ MagickExport MagickBooleanType GetMagickBlobSupport(
 */
 MagickExport DecoderHandler *GetMagickDecoder(const MagickInfo *magick_info)
 {
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(magick_info != (MagickInfo *) NULL);
   assert(magick_info->signature == MagickSignature);
   return((DecoderHandler *) magick_info->decoder);
@@ -386,7 +385,7 @@ MagickExport DecoderHandler *GetMagickDecoder(const MagickInfo *magick_info)
 */
 MagickExport const char *GetMagickDescription(const MagickInfo *magick_info)
 {
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(magick_info != (MagickInfo *) NULL);
   assert(magick_info->signature == MagickSignature);
   return(magick_info->description);
@@ -416,7 +415,7 @@ MagickExport const char *GetMagickDescription(const MagickInfo *magick_info)
 */
 MagickExport EncoderHandler *GetMagickEncoder(const MagickInfo *magick_info)
 {
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(magick_info != (MagickInfo *) NULL);
   assert(magick_info->signature == MagickSignature);
   return((EncoderHandler *) magick_info->encoder);
@@ -427,6 +426,39 @@ MagickExport EncoderHandler *GetMagickEncoder(const MagickInfo *magick_info)
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   G e t M a g i c k E n d i a n S u p p o r t                               %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  GetMagickEndianSupport() returns the MagickTrue if the coder respects
+%  endianess other than MSBEndian.
+%
+%  The format of the GetMagickEndianSupport method is:
+%
+%      EndianType GetMagickEndianSupport(const MagickInfo *magick_info)
+%
+%  A description of each parameter follows:
+%
+%    o magick_info:  The magick info.
+%
+*/
+MagickExport MagickBooleanType GetMagickEndianSupport(
+  const MagickInfo *magick_info)
+{
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
+  assert(magick_info != (MagickInfo *) NULL);
+  assert(magick_info->signature == MagickSignature);
+  return(magick_info->endian_support);
+}
+
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   G e t M a g i c k I n f o                                                 %
 %                                                                             %
 %                                                                             %
@@ -455,6 +487,7 @@ MagickExport const MagickInfo *GetMagickInfo(const char *name,
   register const MagickInfo
     *p;
 
+  assert(exception != (ExceptionInfo *) NULL);
   if ((magick_list == (LinkedListInfo *) NULL) ||
       (instantiate_magick == MagickFalse))
     if (InitializeMagickList(exception) == MagickFalse)
@@ -470,7 +503,7 @@ MagickExport const MagickInfo *GetMagickInfo(const char *name,
   /*
     Find name in list.
   */
-  AcquireSemaphoreInfo(&magick_semaphore);
+  magick_semaphore=AcquireSemaphoreInfo(magick_semaphore);
   ResetLinkedListIterator(magick_list);
   p=(const MagickInfo *) GetNextElementInLinkedList(magick_list);
   while (p != (const MagickInfo *) NULL)
@@ -480,14 +513,10 @@ MagickExport const MagickInfo *GetMagickInfo(const char *name,
     p=(const MagickInfo *) GetNextElementInLinkedList(magick_list);
   }
 #if defined(SupportMagickModules)
-  if (p == (MagickInfo *) NULL)
+  if (p == (const MagickInfo *) NULL)
     {
       if (*name != '\0')
-        {
-          RelinquishSemaphoreInfo(&magick_semaphore);
-          (void) OpenModule(name,exception);
-          AcquireSemaphoreInfo(&magick_semaphore);
-        }
+        (void) OpenModule(name,exception);
       ResetLinkedListIterator(magick_list);
       p=(const MagickInfo *) GetNextElementInLinkedList(magick_list);
       while (p != (const MagickInfo *) NULL)
@@ -498,7 +527,7 @@ MagickExport const MagickInfo *GetMagickInfo(const char *name,
       }
     }
 #endif
-  RelinquishSemaphoreInfo(&magick_semaphore);
+  RelinquishSemaphoreInfo(magick_semaphore);
   return(p);
 }
 
@@ -519,7 +548,7 @@ MagickExport const MagickInfo *GetMagickInfo(const char *name,
 %  The format of the GetMagickInfoList function is:
 %
 %      const MagickInfo **GetMagickInfoList(const char *pattern,
-%        unsigned long *number_formats)
+%        unsigned long *number_formats,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
@@ -527,6 +556,7 @@ MagickExport const MagickInfo *GetMagickInfo(const char *name,
 %
 %    o number_formats:  This integer returns the number of formats in the list.
 %
+%    o exception: Return any errors or warnings in this structure.
 %
 */
 
@@ -550,14 +580,11 @@ static int MagickInfoCompare(const void *x,const void *y)
 #endif
 
 MagickExport const MagickInfo **GetMagickInfoList(const char *pattern,
-  unsigned long *number_formats)
+  unsigned long *number_formats,ExceptionInfo *exception)
 {
   const MagickInfo
     **formats;
 
-  ExceptionInfo
-    exception;
-
   register const MagickInfo
     *p;
 
@@ -571,9 +598,7 @@ MagickExport const MagickInfo **GetMagickInfoList(const char *pattern,
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),pattern);
   assert(number_formats != (unsigned long *) NULL);
   *number_formats=0;
-  GetExceptionInfo(&exception);
-  p=GetMagickInfo("*",&exception);
-  DestroyExceptionInfo(&exception);
+  p=GetMagickInfo("*",exception);
   if (p == (const MagickInfo *) NULL)
     return((const MagickInfo **) NULL);
   formats=(const MagickInfo **) AcquireMagickMemory((size_t)
@@ -583,7 +608,7 @@ MagickExport const MagickInfo **GetMagickInfoList(const char *pattern,
   /*
     Generate magick list.
   */
-  AcquireSemaphoreInfo(&magick_semaphore);
+  magick_semaphore=AcquireSemaphoreInfo(magick_semaphore);
   ResetLinkedListIterator(magick_list);
   p=(const MagickInfo *) GetNextElementInLinkedList(magick_list);
   for (i=0; p != (const MagickInfo *) NULL; )
@@ -593,7 +618,7 @@ MagickExport const MagickInfo **GetMagickInfoList(const char *pattern,
       formats[i++]=p;
     p=(const MagickInfo *) GetNextElementInLinkedList(magick_list);
   }
-  RelinquishSemaphoreInfo(&magick_semaphore);
+  RelinquishSemaphoreInfo(magick_semaphore);
   qsort((void *) formats,(size_t) i,sizeof(*formats),MagickInfoCompare);
   formats[i]=(MagickInfo *) NULL;
   *number_formats=(unsigned long) i;
@@ -615,7 +640,8 @@ MagickExport const MagickInfo **GetMagickInfoList(const char *pattern,
 %
 %  The format of the GetMagickList function is:
 %
-%      char **GetMagickList(const char *pattern,unsigned long *number_formats)
+%      char **GetMagickList(const char *pattern,unsigned long *number_formats,
+%        ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
@@ -623,6 +649,7 @@ MagickExport const MagickInfo **GetMagickInfoList(const char *pattern,
 %
 %    o number_formats:  This integer returns the number of formats in the list.
 %
+%    o exception: Return any errors or warnings in this structure.
 %
 */
 
@@ -646,14 +673,11 @@ static int MagickCompare(const void *x,const void *y)
 #endif
 
 MagickExport char **GetMagickList(const char *pattern,
-  unsigned long *number_formats)
+  unsigned long *number_formats,ExceptionInfo *exception)
 {
   char
     **formats;
 
-  ExceptionInfo
-    exception;
-
   register const MagickInfo
     *p;
 
@@ -667,16 +691,14 @@ MagickExport char **GetMagickList(const char *pattern,
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),pattern);
   assert(number_formats != (unsigned long *) NULL);
   *number_formats=0;
-  GetExceptionInfo(&exception);
-  p=GetMagickInfo("*",&exception);
-  DestroyExceptionInfo(&exception);
+  p=GetMagickInfo("*",exception);
   if (p == (const MagickInfo *) NULL)
     return((char **) NULL);
   formats=(char **) AcquireMagickMemory((size_t)
     (GetNumberOfElementsInLinkedList(magick_list)+1)*sizeof(*formats));
   if (formats == (char **) NULL)
     return((char **) NULL);
-  AcquireSemaphoreInfo(&magick_semaphore);
+  magick_semaphore=AcquireSemaphoreInfo(magick_semaphore);
   ResetLinkedListIterator(magick_list);
   p=(const MagickInfo *) GetNextElementInLinkedList(magick_list);
   for (i=0; p != (const MagickInfo *) NULL; )
@@ -686,7 +708,7 @@ MagickExport char **GetMagickList(const char *pattern,
       formats[i++]=AcquireString(p->name);
     p=(const MagickInfo *) GetNextElementInLinkedList(magick_list);
   }
-  RelinquishSemaphoreInfo(&magick_semaphore);
+  RelinquishSemaphoreInfo(magick_semaphore);
   qsort((void *) formats,(size_t) i,sizeof(*formats),MagickCompare);
   formats[i]=(char *) NULL;
   *number_formats=(unsigned long) i;
@@ -719,7 +741,7 @@ MagickExport char **GetMagickList(const char *pattern,
 MagickExport MagickBooleanType GetMagickSeekableStream(
   const MagickInfo *magick_info)
 {
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(magick_info != (MagickInfo *) NULL);
   assert(magick_info->signature == MagickSignature);
   return(magick_info->seekable_stream);
@@ -750,7 +772,7 @@ MagickExport MagickBooleanType GetMagickSeekableStream(
 MagickExport MagickBooleanType GetMagickThreadSupport(
   const MagickInfo *magick_info)
 {
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(magick_info != (MagickInfo *) NULL);
   assert(magick_info->signature == MagickSignature);
   return(magick_info->thread_support);
@@ -851,7 +873,8 @@ static SignalHandler *RegisterMagickSignalHandler(int signal_number)
 MagickExport void InitializeMagick(const char *path)
 {
   char
-    execution_path[MaxTextExtent];
+    execution_path[MaxTextExtent],
+    filename[MaxTextExtent];
 
   const char
     *p;
@@ -891,68 +914,47 @@ MagickExport void InitializeMagick(const char *path)
   /*
     Set client name and execution path.
   */
-  *execution_path='\0';
-  if (IsAccessible(path) == MagickFalse)
-    (void) GetExecutionPath(execution_path);
-  else
+  (void) GetExecutionPath(execution_path);
+  if ((path != (const char *) NULL) && (*path != '\0'))
     (void) CopyMagickString(execution_path,path,MaxTextExtent);
-  if (IsAccessible(execution_path) == MagickFalse)
-    {
-      (void) getcwd(execution_path,MaxTextExtent);
-      if (path != (char *) NULL)
-        {
-          (void) ConcatenateMagickString(execution_path,
-            DirectorySeparator,MaxTextExtent);
-          if ((*path == '.') && (*(path+1) == *DirectorySeparator))
-            (void) ConcatenateMagickString(execution_path,path+2,
-              MaxTextExtent);
-          else
-            (void) ConcatenateMagickString(execution_path,path,
-              MaxTextExtent);
-        }
-    }
-  if (IsAccessible(execution_path) == MagickFalse)
-    {
-      if (path != (char *) NULL)
-        (void) SetClientName(path);
-    }
-  else
-    {
-      char
-        filename[MaxTextExtent];
-
-      GetPathComponent(execution_path,HeadPath,filename);
-      (void) SetClientPath(filename);
-      GetPathComponent(execution_path,TailPath,filename);
-      (void) SetClientName(filename);
-    }
+  GetPathComponent(execution_path,TailPath,filename);
+  (void) SetClientName(filename);
+  GetPathComponent(execution_path,HeadPath,execution_path);
+  (void) SetClientPath(execution_path);
   /*
     Set signal handlers.
   */
-  (void) ResetMagickMemory(signal_handlers,0,sizeof(signal_handlers));
 #if defined(SIGABRT)
-  signal_handlers[SIGABRT]=RegisterMagickSignalHandler(SIGABRT);
+  if (signal_handlers[SIGABRT] == (SignalHandler *) NULL)
+    signal_handlers[SIGABRT]=RegisterMagickSignalHandler(SIGABRT);
 #endif
 #if defined(SIGFPE)
-  signal_handlers[SIGFPE]=RegisterMagickSignalHandler(SIGFPE);
+  if (signal_handlers[SIGFPE] == (SignalHandler *) NULL)
+    signal_handlers[SIGFPE]=RegisterMagickSignalHandler(SIGFPE);
 #endif
 #if defined(SIGHUP)
-  signal_handlers[SIGHUP]=RegisterMagickSignalHandler(SIGHUP);
+  if (signal_handlers[SIGHUP] == (SignalHandler *) NULL)
+    signal_handlers[SIGHUP]=RegisterMagickSignalHandler(SIGHUP);
 #endif
 #if defined(SIGINT)
-  signal_handlers[SIGINT]=RegisterMagickSignalHandler(SIGINT);
+  if (signal_handlers[SIGINT] == (SignalHandler *) NULL)
+    signal_handlers[SIGINT]=RegisterMagickSignalHandler(SIGINT);
 #endif
 #if defined(SIGQUIT)
-  signal_handlers[SIGQUIT]=RegisterMagickSignalHandler(SIGQUIT);
+  if (signal_handlers[SIGQUIT] == (SignalHandler *) NULL)
+    signal_handlers[SIGQUIT]=RegisterMagickSignalHandler(SIGQUIT);
 #endif
 #if defined(SIGTERM)
-  signal_handlers[SIGTERM]=RegisterMagickSignalHandler(SIGTERM);
+  if (signal_handlers[SIGTERM] == (SignalHandler *) NULL)
+    signal_handlers[SIGTERM]=RegisterMagickSignalHandler(SIGTERM);
 #endif
 #if defined(SIGXCPU)
-  signal_handlers[SIGXCPU]=RegisterMagickSignalHandler(SIGXCPU);
+  if (signal_handlers[SIGXCPU] == (SignalHandler *) NULL)
+    signal_handlers[SIGXCPU]=RegisterMagickSignalHandler(SIGXCPU);
 #endif
 #if defined(SIGXFSZ)
-  signal_handlers[SIGXFSZ]=RegisterMagickSignalHandler(SIGXFSZ);
+  if (signal_handlers[SIGXFSZ] == (SignalHandler *) NULL)
+    signal_handlers[SIGXFSZ]=RegisterMagickSignalHandler(SIGXFSZ);
 #endif
   (void) atexit(MagickExitHandler);
   /*
@@ -986,36 +988,44 @@ MagickExport void InitializeMagick(const char *path)
 */
 static MagickBooleanType InitializeMagickList(ExceptionInfo *exception)
 {
-  MagickBooleanType
-    active;
-
-  active=MagickFalse;
   if ((magick_list == (LinkedListInfo *) NULL) &&
       (instantiate_magick == MagickFalse))
     {
-      AcquireSemaphoreInfo(&magick_semaphore);
+      magick_semaphore=AcquireSemaphoreInfo(magick_semaphore);
       if ((magick_list == (LinkedListInfo *) NULL) &&
           (instantiate_magick == MagickFalse))
         {
-          instantiate_magick=MagickTrue;
-          active=MagickTrue;
-        }
-      RelinquishSemaphoreInfo(&magick_semaphore);
-      if (active == MagickTrue)
-        {
+          MagickBooleanType
+            status;
+
           MagickInfo
             *magick_info;
 
+          magick_list=NewLinkedList(0);
+          if (magick_list == (LinkedListInfo *) NULL)
+            ThrowMagickFatalException(ResourceLimitFatalError,
+              "MemoryAllocationFailed",strerror(errno));
           magick_info=SetMagickInfo("tmp");
           magick_info->stealth=MagickTrue;
-          (void) RegisterMagickInfo(magick_info);
+          status=AppendElementToLinkedList(magick_list,magick_info);
+          if (status == MagickFalse)
+            ThrowMagickFatalException(ResourceLimitFatalError,
+              "UnableToAllocateMagickInfo",magick_info->name);
+          magick_info=SetMagickInfo("clipmask");
+          magick_info->stealth=MagickTrue;
+          status=AppendElementToLinkedList(magick_list,magick_info);
+          if (status == MagickFalse)
+            ThrowMagickFatalException(ResourceLimitFatalError,
+              "UnableToAllocateMagickInfo",magick_info->name);
 #if defined(SupportMagickModules)
           (void) GetModuleInfo((char *) NULL,exception);
 #endif
 #if !defined(BuildMagickModules)
           RegisterStaticModules();
 #endif
+          instantiate_magick=MagickTrue;
         }
+      RelinquishSemaphoreInfo(magick_semaphore);
     }
   return(magick_list != (LinkedListInfo *) NULL ? MagickTrue : MagickFalse);
 }
@@ -1122,7 +1132,7 @@ MagickExport MagickBooleanType ListMagickInfo(FILE *file,
 
   if (file == (FILE *) NULL)
     file=stdout;
-  magick_info=GetMagickInfoList("*",&number_formats);
+  magick_info=GetMagickInfoList("*",&number_formats,exception);
   if (magick_info == (const MagickInfo **) NULL)
     return(MagickFalse);
 #if !defined(SupportMagickModules)
@@ -1330,22 +1340,11 @@ MagickExport MagickInfo *RegisterMagickInfo(MagickInfo *magick_info)
   assert(magick_info->signature == MagickSignature);
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),magick_info->name);
   if (magick_list == (LinkedListInfo *) NULL)
-    {
-      magick_list=NewLinkedList(0);
-      if (magick_list == (LinkedListInfo *) NULL)
-        {
-          ThrowMagickFatalException(ResourceLimitFatalError,
-            "MemoryAllocationFailed",magick_info->name);
-          return((MagickInfo *) NULL);
-        }
-    }
-  (void) UnregisterMagickInfo(magick_info->name);
-  AcquireSemaphoreInfo(&magick_semaphore);
+    return((MagickInfo *) NULL);
   status=AppendElementToLinkedList(magick_list,magick_info);
   if (status == MagickFalse)
     ThrowMagickFatalException(ResourceLimitFatalError,
       "UnableToAllocateMagickInfo",magick_info->name);
-  RelinquishSemaphoreInfo(&magick_semaphore);
   return(magick_info);
 }
 
@@ -1435,7 +1434,7 @@ MagickExport MagickBooleanType UnregisterMagickInfo(const char *name)
   if (IsLinkedListEmpty(magick_list) == MagickTrue)
     return(MagickFalse);
   status=MagickFalse;
-  AcquireSemaphoreInfo(&magick_semaphore);
+  magick_semaphore=AcquireSemaphoreInfo(magick_semaphore);
   ResetLinkedListIterator(magick_list);
   p=(const MagickInfo *) GetNextElementInLinkedList(magick_list);
   while (p != (const MagickInfo *) NULL)
@@ -1444,6 +1443,7 @@ MagickExport MagickBooleanType UnregisterMagickInfo(const char *name)
       break;
     p=(const MagickInfo *) GetNextElementInLinkedList(magick_list);
   }
+  RelinquishSemaphoreInfo(magick_semaphore);
   if (p != (const MagickInfo *) NULL)
     {
       MagickInfo
@@ -1453,6 +1453,5 @@ MagickExport MagickBooleanType UnregisterMagickInfo(const char *name)
         RemoveElementByValueFromLinkedList(magick_list,p);
       (void) DestroyMagickElement(magick_info);
     }
-  RelinquishSemaphoreInfo(&magick_semaphore);
   return(status);
 }
