@@ -17,7 +17,7 @@
 %                                March 2000                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -45,7 +45,8 @@
 #include "magick/coder.h"
 #include "magick/client.h"
 #include "magick/configure.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/log.h"
 #include "magick/hashmap.h"
 #include "magick/magic.h"
@@ -87,11 +88,11 @@ typedef void *ModuleHandle;
 static SemaphoreInfo
   *module_semaphore = (SemaphoreInfo *) NULL;
 
-static MagickBooleanType
-  instantiate_module = MagickFalse;
-
 static LinkedListInfo
   *module_list = (LinkedListInfo *) NULL;
+
+static volatile MagickBooleanType
+  instantiate_module = MagickFalse;
 
 /*
   Forward declarations.
@@ -102,7 +103,7 @@ static const ModuleInfo
 static MagickBooleanType
   GetMagickModulePath(const char *,MagickModuleType,char *,ExceptionInfo *),
   InitializeModuleList(ExceptionInfo *),
-  UnloadModule(const ModuleInfo *,ExceptionInfo *);
+  UnregisterModule(const ModuleInfo *,ExceptionInfo *);
 
 static void
   TagToCoderModuleName(const char *,char *),
@@ -139,7 +140,7 @@ static void *DestroyModuleElement(void *module_info)
 
   GetExceptionInfo(&exception);
   p=(ModuleInfo *) module_info;
-  if (UnloadModule(p,&exception) == MagickFalse)
+  if (UnregisterModule(p,&exception) == MagickFalse)
     CatchException(&exception);
   if (p->tag != (char *) NULL)
     p->tag=(char *) RelinquishMagickMemory(p->tag);
@@ -155,17 +156,17 @@ MagickExport void DestroyModuleList(void)
   /*
     Deestroy magick modules.
   */
-  AcquireSemaphoreInfo(&module_semaphore);
+  module_semaphore=AcquireSemaphoreInfo(module_semaphore);
 #if defined(SupportMagickModules)
   if (module_list != (LinkedListInfo *) NULL)
-    DestroyLinkedList(module_list,DestroyModuleElement);
+    module_list=DestroyLinkedList(module_list,DestroyModuleElement);
   module_list=(LinkedListInfo *) NULL;
   if (instantiate_module != MagickFalse)
     (void) lt_dlexit();
 #endif
   instantiate_module=MagickFalse;
-  RelinquishSemaphoreInfo(&module_semaphore);
-  DestroySemaphoreInfo(&module_semaphore);
+  RelinquishSemaphoreInfo(module_semaphore);
+  module_semaphore=DestroySemaphoreInfo(module_semaphore);
 }
 
 /*
@@ -322,7 +323,7 @@ MagickExport const ModuleInfo *GetModuleInfo(const char *tag,
 #endif
       return((const ModuleInfo *) GetElementFromLinkedList(module_list,0));
     }
-  AcquireSemaphoreInfo(&module_semaphore);
+  module_semaphore=AcquireSemaphoreInfo(module_semaphore);
   ResetLinkedListIterator(module_list);
   p=(const ModuleInfo *) GetNextElementInLinkedList(module_list);
   while (p != (const ModuleInfo *) NULL)
@@ -334,7 +335,7 @@ MagickExport const ModuleInfo *GetModuleInfo(const char *tag,
   if (p == (ModuleInfo *) NULL)
     (void) ThrowMagickException(exception,GetMagickModule(),ModuleError,
       "UnrecognizedModule",tag);
-  RelinquishSemaphoreInfo(&module_semaphore);
+  RelinquishSemaphoreInfo(module_semaphore);
   return((const ModuleInfo *) p);
 }
 
@@ -354,7 +355,7 @@ MagickExport const ModuleInfo *GetModuleInfo(const char *tag,
 %  The format of the GetModuleInfoList function is:
 %
 %      const ModuleInfo **GetModuleInfoList(const char *pattern,
-%        unsigned long *number_modules)
+%        unsigned long *number_modules,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
@@ -362,6 +363,7 @@ MagickExport const ModuleInfo *GetModuleInfo(const char *tag,
 %
 %    o number_modules:  This integer returns the number of modules in the list.
 %
+%    o exception: Return any errors or warnings in this structure.
 %
 */
 
@@ -387,14 +389,11 @@ static int ModuleInfoCompare(const void *x,const void *y)
 #endif
 
 MagickExport const ModuleInfo **GetModuleInfoList(const char *pattern,
-  unsigned long *number_modules)
+  unsigned long *number_modules,ExceptionInfo *exception)
 {
   const ModuleInfo
     **modules;
 
-  ExceptionInfo
-    exception;
-
   register const ModuleInfo
     *p;
 
@@ -408,19 +407,17 @@ MagickExport const ModuleInfo **GetModuleInfoList(const char *pattern,
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),pattern);
   assert(number_modules != (unsigned long *) NULL);
   *number_modules=0;
-  GetExceptionInfo(&exception);
-  p=GetModuleInfo("*",&exception);
-  DestroyExceptionInfo(&exception);
+  p=GetModuleInfo("*",exception);
   if (p == (const ModuleInfo *) NULL)
     return((const ModuleInfo **) NULL);
   modules=(const ModuleInfo **) AcquireMagickMemory((size_t)
-    (GetNumberOfElementsInLinkedList(module_list)+1)*sizeof(ModuleInfo *));
+    (GetNumberOfElementsInLinkedList(module_list)+1)*sizeof(*modules));
   if (modules == (const ModuleInfo **) NULL)
     return((const ModuleInfo **) NULL);
   /*
     Generate module list.
   */
-  AcquireSemaphoreInfo(&module_semaphore);
+  module_semaphore=AcquireSemaphoreInfo(module_semaphore);
   ResetLinkedListIterator(module_list);
   p=(const ModuleInfo *) GetNextElementInLinkedList(module_list);
   for (i=0; p != (const ModuleInfo *) NULL; )
@@ -430,8 +427,8 @@ MagickExport const ModuleInfo **GetModuleInfoList(const char *pattern,
       modules[i++]=p;
     p=(const ModuleInfo *) GetNextElementInLinkedList(module_list);
   }
-  RelinquishSemaphoreInfo(&module_semaphore);
-  qsort((void *) modules,(size_t) i,sizeof(ModuleInfo **),ModuleInfoCompare);
+  RelinquishSemaphoreInfo(module_semaphore);
+  qsort((void *) modules,(size_t) i,sizeof(*modules),ModuleInfoCompare);
   modules[i]=(ModuleInfo *) NULL;
   *number_modules=(unsigned long) i;
   return(modules);
@@ -453,7 +450,8 @@ MagickExport const ModuleInfo **GetModuleInfoList(const char *pattern,
 %
 %  The format of the GetModuleList function is:
 %
-%      char **GetModuleList(const char *pattern,unsigned long *number_modules)
+%      char **GetModuleList(const char *pattern,unsigned long *number_modules,
+%        ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
@@ -462,6 +460,7 @@ MagickExport const ModuleInfo **GetModuleInfoList(const char *pattern,
 %    o number_modules:  This integer returns the number of modules in the
 %      list.
 %
+%    o exception: Return any errors or warnings in this structure.
 %
 */
 
@@ -485,7 +484,7 @@ static int ModuleCompare(const void *x,const void *y)
 #endif
 
 MagickExport char **GetModuleList(const char *pattern,
-  unsigned long *number_modules)
+  unsigned long *number_modules,ExceptionInfo *exception)
 {
   char
     **modules,
@@ -496,9 +495,6 @@ MagickExport char **GetModuleList(const char *pattern,
   DIR
     *directory;
 
-  ExceptionInfo
-    exception;
-
   MagickBooleanType
     status;
 
@@ -515,16 +511,14 @@ MagickExport char **GetModuleList(const char *pattern,
     Locate all modules in the coder path.
   */
   TagToCoderModuleName("magick",filename);
-  GetExceptionInfo(&exception);
   status=GetMagickModulePath(filename,MagickCoderModule,module_path,
-    &exception);
-  DestroyExceptionInfo(&exception);
+    exception);
   if (status == MagickFalse)
     return((char **) NULL);
   GetPathComponent(module_path,HeadPath,path);
   max_entries=255;
   modules=(char **)
-    AcquireMagickMemory((size_t) (max_entries+1)*sizeof(char *));
+    AcquireMagickMemory((size_t) (max_entries+1)*sizeof(*modules));
   if (modules == (char **) NULL)
     return((char **) NULL);
   *modules=(char *) NULL;
@@ -546,7 +540,7 @@ MagickExport char **GetModuleList(const char *pattern,
       {
         max_entries<<=1;
         modules=(char **)
-          ResizeMagickMemory(modules,(size_t) max_entries*sizeof(char *));
+          ResizeMagickMemory(modules,(size_t) max_entries*sizeof(*modules));
         if (modules == (char **) NULL)
           break;
       }
@@ -565,7 +559,7 @@ MagickExport char **GetModuleList(const char *pattern,
     entry=readdir(directory);
   }
   (void) closedir(directory);
-  qsort((void *) modules,(size_t) i,sizeof(char **),ModuleCompare);
+  qsort((void *) modules,(size_t) i,sizeof(*modules),ModuleCompare);
   modules[i]=(char *) NULL;
   *number_modules=(unsigned long) i;
   return(modules);
@@ -679,8 +673,8 @@ static MagickBooleanType GetMagickModulePath(const char *filename,
     (void) FormatMagickString(path,MaxTextExtent,"%s%s",directory,filename);
     if (IsAccessible(path) == MagickFalse)
       {
-        (void) ThrowMagickException(exception,GetMagickModule(),ConfigureError,
-          "UnableToOpenModuleFile",path);
+        (void) ThrowMagickException(exception,GetMagickModule(),
+          ConfigureWarning,"UnableToOpenModuleFile",path);
         return(MagickFalse);
       }
     return(MagickTrue);
@@ -720,11 +714,11 @@ static MagickBooleanType GetMagickModulePath(const char *filename,
       DirectorySeparator,filename);
     if (IsAccessible(path) == MagickFalse)
       {
-        ThrowMagickException(exception,GetMagickModule(),ConfigureError,
+        ThrowMagickException(exception,GetMagickModule(),ConfigureWarning,
           "UnableToOpenModuleFile",path);
         return(MagickFalse);
       }
-    return (MagickTrue);
+    return(MagickTrue);
   }
 #endif
 #endif
@@ -778,7 +772,7 @@ static MagickBooleanType GetMagickModulePath(const char *filename,
 #if !defined(POSIX)
       (void) FormatMagickString(path,MaxTextExtent,"%s%s%s",GetClientPath(),
         DirectorySeparator,filename);
-# else
+#else
       char
         prefix[MaxTextExtent];
 
@@ -808,6 +802,19 @@ static MagickBooleanType GetMagickModulePath(const char *filename,
       if (IsAccessible(path) != MagickFalse)
         return(MagickTrue);
     }
+#if defined(__WINDOWS__)
+  {
+    /*
+      Search module path.
+    */
+    if (NTGetModulePath("CORE_RL_magick_.dll",path) != MagickFalse)
+      if (IsAccessible(path) != MagickFalse)
+        return(MagickTrue);
+    if (NTGetModulePath("Magick.dll",path) != MagickFalse)
+      if (IsAccessible(path) != MagickFalse)
+        return(MagickTrue);
+  }
+#endif
   {
     const char
       *home;
@@ -830,7 +837,7 @@ static MagickBooleanType GetMagickModulePath(const char *filename,
   if (IsAccessible(path) != MagickFalse)
     return(MagickTrue);
   if (exception->severity < ConfigureError)
-    ThrowMagickException(exception,GetMagickModule(),ConfigureError,
+    ThrowMagickException(exception,GetMagickModule(),ConfigureWarning,
       "UnableToOpenModuleFile",path);
   return(MagickFalse);
 #endif
@@ -861,42 +868,44 @@ static MagickBooleanType GetMagickModulePath(const char *filename,
 */
 static MagickBooleanType InitializeModuleList(ExceptionInfo *exception)
 {
-  MagickBooleanType
-    active;
-
-  active=MagickFalse;
   if ((module_list == (LinkedListInfo *) NULL) &&
       (instantiate_module == MagickFalse))
     {
-      AcquireSemaphoreInfo(&module_semaphore);
+      module_semaphore=AcquireSemaphoreInfo(module_semaphore);
       if ((module_list == (LinkedListInfo *) NULL) &&
           (instantiate_module == MagickFalse))
         {
-          instantiate_module=MagickTrue;
-          active=MagickTrue;
-        }
-      RelinquishSemaphoreInfo(&module_semaphore);
-      if (active == MagickTrue)
-        {
+          MagickBooleanType
+            status;
+
           ModuleInfo
             *module_info;
 
-          module_info=(ModuleInfo *) AcquireMagickMemory(sizeof(ModuleInfo));
+          module_list=NewLinkedList(0);
+          if (module_list == (LinkedListInfo *) NULL)
+            ThrowMagickFatalException(ResourceLimitFatalError,
+              "MemoryAllocationFailed",strerror(errno));
+          module_info=(ModuleInfo *) AcquireMagickMemory(sizeof(*module_info));
           if (module_info == (ModuleInfo *) NULL)
             ThrowMagickFatalException(ResourceLimitFatalError,
-              "UnableToAllocateModuleInfo",strerror(errno));
-          (void) ResetMagickMemory(module_info,0,sizeof(ModuleInfo));
+              "MemoryAllocationFailed",strerror(errno));
+          (void) ResetMagickMemory(module_info,0,sizeof(*module_info));
           module_info->tag=AcquireString("[boot-strap]");
           module_info->stealth=MagickTrue;
-          module_info->signature=MagickSignature;
           (void) time(&module_info->load_time);
-          (void) RegisterModule(module_info,exception);
+          module_info->signature=MagickSignature;
+          status=AppendElementToLinkedList(module_list,module_info);
+          if (status == MagickFalse)
+            ThrowMagickFatalException(ResourceLimitFatalError,
+              "MemoryAllocationFailed",strerror(errno));
           if (lt_dlinit() != 0)
             ThrowMagickFatalException(ModuleFatalError,
               "UnableToInitializeModuleLoader",lt_dlerror());
+          instantiate_module=MagickTrue;
         }
+      RelinquishSemaphoreInfo(module_semaphore);
     }
-  return(module_list != (LinkedListInfo *) NULL);
+  return(module_list != (LinkedListInfo *) NULL ? MagickTrue : MagickFalse);
 }
 
 /*
@@ -938,9 +947,17 @@ MagickExport MagickBooleanType ListModuleInfo(FILE *file,
 
   if (file == (const FILE *) NULL)
     file=stdout;
-  module_info=GetModuleInfoList("*",&number_modules);
+  module_info=GetModuleInfoList("*",&number_modules,exception);
   if (module_info == (const ModuleInfo **) NULL)
     return(MagickFalse);
+  if (module_info[0]->path != (char *) NULL)
+    {
+      char
+        path[MaxTextExtent];
+
+      GetPathComponent(module_info[0]->path,HeadPath,path);
+      (void) fprintf(file,"\nPath: %s\n\n",path);
+    }
   (void) fprintf(file,"Module\n");
   (void) fprintf(file,"-------------------------------------------------"
     "------------------------------\n");
@@ -969,7 +986,7 @@ MagickExport MagickBooleanType ListModuleInfo(FILE *file,
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  OpenModule() loads a module, and invokes its registration module.  It
-%  returns MagickTrue on success, and False if there is an error.
+%  returns MagickTrue on success, and MagickFalse if there is an error.
 %
 %  The format of the OpenModule module is:
 %
@@ -1012,18 +1029,18 @@ MagickExport MagickBooleanType OpenModule(const char *module,
   if (module_info != (ModuleInfo *) NULL)
     return(MagickTrue);
   if (exception->severity == ModuleError)
-    exception->severity=UndefinedException;
-  (void) strncpy(module_name,module,MaxTextExtent-1);
+    (void) SetExceptionInfo(exception,UndefinedException);
+  (void) CopyMagickString(module_name,module,MaxTextExtent);
   p=GetCoderInfo(module,exception);
   if (p != (CoderInfo *) NULL)
-    (void) strncpy(module_name,p->name,MaxTextExtent-1);
+    (void) CopyMagickString(module_name,p->name,MaxTextExtent);
   /*
     Find module file.
   */
   handle=(ModuleHandle) NULL;
   TagToCoderModuleName(module_name,filename);
   (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
-    "Loading module \"%s\" from file \"%s\"", module_name,filename);
+    "Searching for module \"%s\" using filename \"%s\"",module_name,filename);
   *path='\0';
   status=GetMagickModulePath(filename,MagickCoderModule,path,exception);
   if (status == MagickFalse)
@@ -1038,20 +1055,22 @@ MagickExport MagickBooleanType OpenModule(const char *module,
         "UnableToLoadModule",path,lt_dlerror());
       return(MagickFalse);
     }
+  (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
+    "Opening module at path \"%s\"",path);
   /*
     Add module to module module list.
   */
-  module_info=(ModuleInfo *) AcquireMagickMemory(sizeof(ModuleInfo));
+  module_info=(ModuleInfo *) AcquireMagickMemory(sizeof(*module_info));
   if (module_info == (ModuleInfo *) NULL)
     ThrowMagickFatalException(ResourceLimitFatalError,
       "UnableToAllocateModuleInfo",module_name);
-  (void) ResetMagickMemory(module_info,0,sizeof(ModuleInfo));
+  (void) ResetMagickMemory(module_info,0,sizeof(*module_info));
   module_info->path=AcquireString(path);
   module_info->tag=AcquireString(module_name);
   module_info->signature=MagickSignature;
   module_info->handle=handle;
   (void) time(&module_info->load_time);
-  if (RegisterModule(module_info,exception) == MagickFalse)
+  if (RegisterModule(module_info,exception) == (ModuleInfo *) NULL)
     return(MagickFalse);
   /*
     Locate and record RegisterFORMATImage function
@@ -1126,7 +1145,7 @@ MagickExport MagickBooleanType OpenModules(ExceptionInfo *exception)
   */
   (void) GetMagickInfo((char *) NULL,exception);
   number_modules=0;
-  modules=GetModuleList("*",&number_modules);
+  modules=GetModuleList("*",&number_modules,exception);
   if (modules == (char **) NULL)
     return(MagickFalse);
   for (i=0; i < (long) number_modules; i++)
@@ -1179,19 +1198,11 @@ static const ModuleInfo *RegisterModule(const ModuleInfo *module_info,
   assert(module_info->signature == MagickSignature);
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),module_info->tag);
   if (module_list == (LinkedListInfo *) NULL)
-    {
-      module_list=NewLinkedList(0);
-      if (module_list == (LinkedListInfo *) NULL)
-        {
-          ThrowFileException(exception,ResourceLimitError,
-            "MemoryAllocationFailed",strerror(errno));
-          return(MagickFalse);
-        }
-    }
+    return((const ModuleInfo *) NULL);
   status=AppendElementToLinkedList(module_list,module_info);
   if (status == MagickFalse)
-    (void) ThrowMagickException(exception,GetMagickModule(),
-      ResourceLimitError,"MemoryAllocationFailed",module_info->tag);
+    (void) ThrowMagickException(exception,GetMagickModule(),ResourceLimitError,
+      "MemoryAllocationFailed",module_info->tag);
   return(module_info);
 }
 
@@ -1235,11 +1246,9 @@ static void TagToCoderModuleName(const char *tag,char *name)
   else
     {
 #if defined(_DEBUG)
-      (void) FormatMagickString(name,MaxTextExtent,"IM_MOD_DB_%s_.dll",
-        tag);
+      (void) FormatMagickString(name,MaxTextExtent,"IM_MOD_DB_%s_.dll",tag);
 #else
-      (void) FormatMagickString(name,MaxTextExtent,"IM_MOD_RL_%s_.dll",
-        tag);
+      (void) FormatMagickString(name,MaxTextExtent,"IM_MOD_RL_%s_.dll",tag);
 #endif
     }
 #endif
@@ -1342,18 +1351,18 @@ static void TagToModuleName(const char *tag,const char *format,char *module)
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%   U n l o a d M o d u l e                                                   %
+%   U n R e g i s t e r M o d u l e                                           %
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  UnloadModule() unloads a module, and invokes its de-registration module.
-%  Returns True on success, and False if there is an error.
+%  UnregisterModule() unloads a module, and invokes its de-registration module.
+%  Returns MagickTrue on success, and MagickFalse if there is an error.
 %
-%  The format of the UnloadModule module is:
+%  The format of the UnregisterModule module is:
 %
-%      MagickBooleanType UnloadModule(const ModuleInfo *module_info,
+%      MagickBooleanType UnregisterModule(const ModuleInfo *module_info,
 %        ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
@@ -1364,7 +1373,7 @@ static void TagToModuleName(const char *tag,const char *format,char *module)
 %
 %
 */
-static MagickBooleanType UnloadModule(const ModuleInfo *module_info,
+static MagickBooleanType UnregisterModule(const ModuleInfo *module_info,
   ExceptionInfo *exception)
 {
   char
@@ -1398,8 +1407,10 @@ MagickExport MagickBooleanType ExecuteModuleProcess(const char *tag,
 #if !defined(BuildMagickModules)
   {
     MagickBooleanType
-      (*method)(Image **,const int,char **) = NULL;
+      AnalyzeImage(Image **,const int,char **),
+      (*method)(Image **,const int,char **);
 
+    method=NULL;
     if (LocaleCompare("analyze",tag) == 0)
       method=AnalyzeImage;
     if (method != NULL)
