@@ -17,7 +17,7 @@
 %                               December 1995                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -44,58 +44,54 @@
 #include "magick/log.h"
 #include "magick/monitor.h"
 
-/*
-  Global declarations.
-*/
-static MonitorHandler
-  monitor_handler = (MonitorHandler) NULL;
-
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%   M a g i c k M o n i t o r                                                 %
+%   S e t I m a g e P r o g r e s s M o n i t o r                             %
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  MagickMonitor() calls the monitor handler method with a text string that
-%  describes the task and a measure of completion.  The method returns MagickTrue
-%  on success otherwise MagickFalse if an error is encountered, e.g. if there was a
-%  user interrupt.
+%  SetImageProgressMonitor() sets the image progress monitor to the specified
+%  method and returns the previous progress monitor if any.  The progress
+%  monitor method looks like this:
 %
-%  The format of the MagickMonitor method is:
+%    MagickBooleanType MagickProgressMonitor(const char *text,
+%      const MagickOffsetType offset,const MagickSizeType span,
+%      void *client_data)
 %
-%      MagickBooleanType MagickMonitor(const char *text,
-%        const MagickOffsetType offset,const MagickSizeType span,
-%        ExceptionInfo *exception)
+%  If the progress monitor returns MagickFalse, the current operation is
+%  interrupted.
+%
+%  The format of the SetImageProgressMonitor method is:
+%
+%      MagickProgressMonitor SetImageProgressMonitor(Image *image,
+%        const MagickProgressMonitor progress_monitor,void *client_data)
 %
 %  A description of each parameter follows:
 %
-%    o offset: The position relative to the span parameter which represents
-%      how much progress has been made toward completing a task.
+%    o image: The image.
 %
-%    o span: The span relative to completing a task.
+%    o progress_monitor: Specifies a pointer to a method to monitor progress of
+%      an image operation.
+%
+%    o client_data: Specifies a pointer to any client data.
 %
-%    o exception: Return any errors or warnings in this structure.
 %
 */
-MagickExport MagickBooleanType MagickMonitor(const char *text,
-  const MagickOffsetType offset,const MagickSizeType span,
-  ExceptionInfo *exception)
+MagickExport MagickProgressMonitor SetImageProgressMonitor(Image *image,
+  const MagickProgressMonitor progress_monitor,void *client_data)
 {
-  MagickBooleanType
-    status;
+  MagickProgressMonitor
+    previous_monitor;
 
-  assert(text != (const char *) NULL);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),text);
-  ProcessPendingEvents(text);
-  status=MagickTrue;
-  if (monitor_handler != (MonitorHandler) NULL)
-    status=(*monitor_handler)(text,offset,span,exception);
-  return(status);
+  previous_monitor=image->progress_monitor;
+  image->progress_monitor=progress_monitor;
+  image->client_data=client_data;
+  return(previous_monitor);
 }
 
 /*
@@ -103,31 +99,47 @@ MagickExport MagickBooleanType MagickMonitor(const char *text,
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%   S e t M o n i t o r H a n d l e r                                         %
+%   S e t I m a g e I n f o P r o g r e s s M o n i t o r                     %
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  SetMonitorHandler() sets the monitor handler to the specified method
-%  and returns the previous monitor handler.
+%  SetImageInfoProgressMonitor() sets the image_info progress monitor to the
+%  specified method and returns the previous progress monitor if any.  The
+%  progress monitor method looks like this:
 %
-%  The format of the SetMonitorHandler method is:
+%    MagickBooleanType MagickProgressMonitor(const char *text,
+%      const MagickOffsetType offset,const MagickSizeType span,
+%      void *client_data)
 %
-%      MonitorHandler SetMonitorHandler(MonitorHandler handler)
+%  If the progress monitor returns MagickFalse, the current operation is
+%  interrupted.
+%
+%  The format of the SetImageInfoProgressMonitor method is:
+%
+%      MagickProgressMonitor SetImageInfoProgressMonitor(ImageInfo *image_info,
+%        const MagickProgressMonitor progress_monitor,void *client_data)
 %
 %  A description of each parameter follows:
 %
-%    o handler: Specifies a pointer to a method to handle monitors.
+%    o image_info: The image info.
 %
+%    o progress_monitor: Specifies a pointer to a method to monitor progress of
+%      an image operation.
+%
+%    o client_data: Specifies a pointer to any client data.
 %
 */
-MagickExport MonitorHandler SetMonitorHandler(MonitorHandler handler)
+MagickExport MagickProgressMonitor SetImageInfoProgressMonitor(
+  ImageInfo *image_info,const MagickProgressMonitor progress_monitor,
+  void *client_data)
 {
-  MonitorHandler
-    previous_handler;
+  MagickProgressMonitor
+    previous_monitor;
 
-  previous_handler=monitor_handler;
-  monitor_handler=handler;
-  return(previous_handler);
+  previous_monitor=image_info->progress_monitor;
+  image_info->progress_monitor=progress_monitor;
+  image_info->client_data=client_data;
+  return(previous_monitor);
 }
