@@ -18,7 +18,7 @@
 %                                December 1996                                %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -48,7 +48,8 @@
 #include "magick/cache.h"
 #include "magick/colorspace.h"
 #include "magick/draw.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/hashmap.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
@@ -92,11 +93,14 @@
 MagickExport void *CropImageToHBITMAP(Image *image,
   const RectangleInfo *geometry,ExceptionInfo *exception)
 {
-#define CropImageText  "  Crop image...  "
+#define CropImageTag  "Crop/Image"
 
   long
     y;
 
+  MagickBooleanType
+    status;
+
   RectangleInfo
     page;
 
@@ -203,9 +207,14 @@ MagickExport void *CropImageToHBITMAP(Image *image,
           }
       }
 #endif
-    if (QuantumTick(y,page.height))
-      if (MagickMonitor(CropImageText,y,page.height-1,exception) == MagickFalse)
-        break;
+    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+        (QuantumTick(y,page.height) != MagickFalse))
+      {
+        status=image->progress_monitor(CropImageTag,y,page.height,
+          image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
   }
   if (y < (long) page.height)
     {
