@@ -16,7 +16,7 @@
 %                                 July 1998                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -40,11 +40,13 @@
 */
 #include "magick/studio.h"
 #include "magick/color.h"
+#include "magick/color_private.h"
 #include "magick/composite.h"
 #include "magick/composite_private.h"
 #include "magick/draw.h"
 #include "magick/draw_private.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/monitor.h"
 #include "magick/paint.h"
 #include "magick/string_.h"
@@ -53,16 +55,22 @@
   Define declarations.
 */
 #define MaxStacksize  (1 << 15)
-#define Push(up,left,right,delta) \
-  if ((s < (segment_stack+MaxStacksize)) && (((up)+(delta)) >= 0) && \
-      (((up)+(delta)) < (long) image->rows)) \
+#define PushSegmentStack(up,left,right,delta) \
+{ \
+  if (s >= (segment_stack+MaxStacksize)) \
+    ThrowBinaryException(DrawError,"SegmentStackOverflow",image->filename) \
+  else \
     { \
-      s->y1=(MagickRealType) (up); \
-      s->x1=(MagickRealType) (left); \
-      s->x2=(MagickRealType) (right); \
-      s->y2=(MagickRealType) (delta); \
-      s++; \
-    }
+      if ((((up)+(delta)) >= 0) && (((up)+(delta)) < (long) image->rows)) \
+        { \
+          s->x1=(double) (left); \
+          s->y1=(double) (up); \
+          s->x2=(double) (right); \
+          s->y2=(double) (delta); \
+          s++; \
+        } \
+    } \
+}
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -111,6 +119,9 @@ MagickExport MagickBooleanType ColorFloodfillImage(Image *image,
   const DrawInfo *draw_info,const PixelPacket target,const long x_offset,
   const long y_offset,const PaintMethod method)
 {
+  Image
+    *floodplane_image;
+
   long
     offset,
     start,
@@ -121,9 +132,6 @@ MagickExport MagickBooleanType ColorFloodfillImage(Image *image,
   MagickBooleanType
     skip;
 
-  MagickSizeType
-    number_pixels;
-
   PixelPacket
     fill_color;
 
@@ -142,9 +150,6 @@ MagickExport MagickBooleanType ColorFloodfillImage(Image *image,
   SegmentInfo
     *segment_stack;
 
-  unsigned char
-    *floodplane;
-
   /*
     Check boundary conditions.
   */
@@ -158,34 +163,34 @@ MagickExport MagickBooleanType ColorFloodfillImage(Image *image,
     return(MagickFalse);
   if ((y_offset < 0) || (y_offset >= (long) image->rows))
     return(MagickFalse);
-  /*
-    Set floodfill color.
-  */
-  number_pixels=(MagickSizeType) image->columns*image->rows;
-  if (number_pixels != (MagickSizeType) ((size_t) number_pixels))
-    ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
-      image->filename);
-  floodplane=(unsigned char *) AcquireMagickMemory((size_t) number_pixels);
-  segment_stack=(SegmentInfo *)
-    AcquireMagickMemory(MaxStacksize*sizeof(*segment_stack));
-  if ((floodplane== (unsigned char *) NULL) ||
-      (segment_stack == (SegmentInfo *) NULL))
-    ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
-      image->filename);
-  (void) ResetMagickMemory(floodplane,MagickFalse,(size_t) number_pixels);
-  floodplane[y_offset*image->columns+x_offset]=(unsigned char) MagickTrue;
-  /*
-    Push initial segment on stack.
-  */
   image->storage_class=DirectClass;
   if (image->matte == MagickFalse)
     SetImageOpacity(image,OpaqueOpacity);
+  floodplane_image=CloneImage(image,image->columns,image->rows,MagickTrue,
+    &image->exception);
+  if (floodplane_image == (Image *) NULL)
+    return(MagickFalse);
+  SetImageOpacity(floodplane_image,OpaqueOpacity);
+  /*
+    Set floodfill color.
+  */
+  segment_stack=(SegmentInfo *)
+    AcquireMagickMemory(MaxStacksize*sizeof(*segment_stack));
+  if (segment_stack == (SegmentInfo *) NULL)
+    {
+      floodplane_image=DestroyImage(floodplane_image);
+      ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
+        image->filename);
+    }
+  /*
+    Push initial segment on stack.
+  */
   x=x_offset;
   y=y_offset;
   start=0;
   s=segment_stack;
-  Push(y,x,x,1);
-  Push(y+1,x,x,-1);
+  PushSegmentStack(y,x,x,1);
+  PushSegmentStack(y+1,x,x,-1);
   while (s > segment_stack)
   {
     /*
@@ -200,30 +205,33 @@ MagickExport MagickBooleanType ColorFloodfillImage(Image *image,
       Recolor neighboring pixels.
     */
     p=AcquireImagePixels(image,0,y,(unsigned long) (x1+1),1,&image->exception);
-    if (p == (const PixelPacket *) NULL)
+    q=GetImagePixels(floodplane_image,0,y,(unsigned long) (x1+1),1);
+    if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
       break;
     p+=x1;
+    q+=x1;
     for (x=x1; x >= 0; x--)
     {
+      if (q->opacity == TransparentOpacity)
+        break;
       if (method == FloodfillMethod)
         {
-          if ((FuzzyColorCompare(image,p,&target) == MagickFalse) ||
-              ((int) floodplane[y*image->columns+x] != MagickFalse))
+          if (FuzzyColorCompare(image,p,&target) == MagickFalse)
             break;
         }
       else
-        if ((FuzzyColorCompare(image,p,&target) != MagickFalse) ||
-            ((int) floodplane[y*image->columns+x] != MagickFalse))
+        if (FuzzyColorCompare(image,p,&target) != MagickFalse)
           break;
-      floodplane[y*image->columns+x]=(unsigned char) MagickTrue;
+      q->opacity=TransparentOpacity;
       p--;
+      q--;
     }
     skip=(MagickBooleanType) (x >= x1);
     if (skip == MagickFalse)
       {
         start=x+1;
         if (start < x1)
-          Push(y,start,x1-1,-offset);
+          PushSegmentStack(y,start,x1-1,-offset);
         x=x1+1;
       }
     do
@@ -234,27 +242,30 @@ MagickExport MagickBooleanType ColorFloodfillImage(Image *image,
             {
               p=AcquireImagePixels(image,x,y,image->columns-x,1,
                 &image->exception);
-              if (p == (const PixelPacket *) NULL)
+              q=GetImagePixels(floodplane_image,x,y,image->columns-x,1);
+              if ((p == (const PixelPacket *) NULL) ||
+                  (q == (PixelPacket *) NULL))
                 break;
               for ( ; x < (long) image->columns; x++)
               {
+                if (q->opacity == TransparentOpacity)
+                  break;
                 if (method == FloodfillMethod)
                   {
-                    if ((FuzzyColorCompare(image,p,&target) == MagickFalse) ||
-                        ((int) floodplane[y*image->columns+x] != MagickFalse))
+                    if (FuzzyColorCompare(image,p,&target) == MagickFalse)
                       break;
                   }
                 else
-                  if ((FuzzyColorCompare(image,p,&target) != MagickFalse) ||
-                      ((int) floodplane[y*image->columns+x] != MagickFalse))
+                  if (FuzzyColorCompare(image,p,&target) != MagickFalse)
                     break;
-                floodplane[y*image->columns+x]=(unsigned char) MagickTrue;
+                q->opacity=TransparentOpacity;
                 p++;
+                q++;
               }
             }
-          Push(y,start,x-1,offset);
+          PushSegmentStack(y,start,x-1,offset);
           if (x > (x2+1))
-            Push(y,x2+1,x-1,-offset);
+            PushSegmentStack(y,x2+1,x-1,-offset);
         }
       skip=MagickFalse;
       x++;
@@ -262,23 +273,23 @@ MagickExport MagickBooleanType ColorFloodfillImage(Image *image,
         {
           p=AcquireImagePixels(image,x,y,(unsigned long) (x2-x+1),1,
             &image->exception);
-          if (p == (const PixelPacket *) NULL)
+          q=GetImagePixels(floodplane_image,x,y,(unsigned long) (x2-x+1),1);
+          if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
             break;
           for ( ; x <= x2; x++)
           {
+            if (q->opacity == TransparentOpacity)
+              break;
             if (method == FloodfillMethod)
               {
-                if ((FuzzyColorCompare(image,p,&target) != MagickFalse) &&
-                    ((int) floodplane[y*image->columns+x] == MagickFalse))
+                if (FuzzyColorCompare(image,p,&target) != MagickFalse)
                   break;
               }
             else
-              {
-                if ((FuzzyColorCompare(image,p,&target) == MagickFalse) &&
-                    ((int) floodplane[y*image->columns+x] == MagickFalse))
-                  break;
-              }
+              if (FuzzyColorCompare(image,p,&target) == MagickFalse)
+                break;
             p++;
+            q++;
           }
         }
       start=x;
@@ -289,24 +300,27 @@ MagickExport MagickBooleanType ColorFloodfillImage(Image *image,
     /*
       Tile fill color onto floodplane.
     */
+    p=AcquireImagePixels(floodplane_image,0,y,image->columns,1,
+      &image->exception);
     q=GetImagePixels(image,0,y,image->columns,1);
-    if (q == (PixelPacket *) NULL)
+    if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
       break;
     for (x=0; x < (long) image->columns; x++)
     {
-      if ((int) floodplane[y*image->columns+x] != MagickFalse)
+      if (p->opacity != OpaqueOpacity)
         {
           fill_color=GetFillColor(draw_info,x,y);
-          *q=MagickCompositeOver(&fill_color,(MagickRealType)
-            fill_color.opacity,q,(MagickRealType) q->opacity);
+          MagickCompositeOver(&fill_color,(MagickRealType) fill_color.opacity,q,
+            (MagickRealType) q->opacity,q);
         }
+      p++;
       q++;
     }
     if (SyncImagePixels(image) == MagickFalse)
       break;
   }
   segment_stack=(SegmentInfo *) RelinquishMagickMemory(segment_stack);
-  floodplane=(unsigned char *) RelinquishMagickMemory(floodplane);
+  floodplane_image=DestroyImage(floodplane_image);
   return((MagickBooleanType) (y == (long) image->rows));
 }
 
@@ -358,6 +372,9 @@ MagickExport MagickBooleanType MatteFloodfillImage(Image *image,
   const PixelPacket target,const Quantum opacity,const long x_offset,
   const long y_offset,const PaintMethod method)
 {
+  Image
+    *floodplane_image;
+
   long
     offset,
     start,
@@ -368,9 +385,6 @@ MagickExport MagickBooleanType MatteFloodfillImage(Image *image,
   MagickBooleanType
     skip;
 
-  MagickSizeType
-    number_pixels;
-
   register const PixelPacket
     *p;
 
@@ -386,9 +400,6 @@ MagickExport MagickBooleanType MatteFloodfillImage(Image *image,
   SegmentInfo
     *segment_stack;
 
-  unsigned char
-    *floodplane;
-
   /*
     Check boundary conditions.
   */
@@ -400,34 +411,34 @@ MagickExport MagickBooleanType MatteFloodfillImage(Image *image,
     return(MagickFalse);
   if ((y_offset < 0) || (y_offset >= (long) image->rows))
     return(MagickFalse);
-  /*
-    Set floodfill color.
-  */
-  number_pixels=(MagickSizeType) image->columns*image->rows;
-  if (number_pixels != (MagickSizeType) ((size_t) number_pixels))
-    ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
-      image->filename);
-  floodplane=(unsigned char *) AcquireMagickMemory((size_t) number_pixels);
-  segment_stack=(SegmentInfo *)
-    AcquireMagickMemory(MaxStacksize*sizeof(*segment_stack));
-  if ((floodplane== (unsigned char *) NULL) ||
-      (segment_stack == (SegmentInfo *) NULL))
-    ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
-      image->filename);
-  (void) ResetMagickMemory(floodplane,MagickFalse,(size_t) number_pixels);
-  floodplane[y_offset*image->columns+x_offset]=(unsigned char) MagickTrue;
-  /*
-    Push initial segment on stack.
-  */
   image->storage_class=DirectClass;
   if (image->matte == MagickFalse)
     SetImageOpacity(image,OpaqueOpacity);
+  floodplane_image=CloneImage(image,image->columns,image->rows,MagickTrue,
+    &image->exception);
+  if (floodplane_image == (Image *) NULL)
+    return(MagickFalse);
+  SetImageOpacity(floodplane_image,OpaqueOpacity);
+  /*
+    Set floodfill color.
+  */
+  segment_stack=(SegmentInfo *)
+    AcquireMagickMemory(MaxStacksize*sizeof(*segment_stack));
+  if (segment_stack == (SegmentInfo *) NULL)
+    {
+      floodplane_image=DestroyImage(floodplane_image);
+      ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
+        image->filename);
+    }
+  /*
+    Push initial segment on stack.
+  */
   x=x_offset;
   y=y_offset;
   start=0;
   s=segment_stack;
-  Push(y,x,x,1);
-  Push(y+1,x,x,-1);
+  PushSegmentStack(y,x,x,1);
+  PushSegmentStack(y+1,x,x,-1);
   while (s > segment_stack)
   {
     /*
@@ -442,22 +453,25 @@ MagickExport MagickBooleanType MatteFloodfillImage(Image *image,
       Recolor neighboring pixels.
     */
     p=AcquireImagePixels(image,0,y,(unsigned long) (x1+1),1,&image->exception);
-    if (p == (const PixelPacket *) NULL)
+    q=GetImagePixels(floodplane_image,0,y,(unsigned long) (x1+1),1);
+    if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
       break;
     p+=x1;
+    q+=x1;
     for (x=x1; x >= 0; x--)
     {
+      if (q->opacity == TransparentOpacity)
+        break;
       if (method == FloodfillMethod)
         {
-          if ((FuzzyColorCompare(image,p,&target) == MagickFalse) ||
-              ((int) floodplane[y*image->columns+x] != MagickFalse))
+          if (FuzzyColorCompare(image,p,&target) == MagickFalse)
             break;
         }
       else
-        if ((FuzzyColorCompare(image,p,&target) != MagickFalse) ||
-            ((int) floodplane[y*image->columns+x] != MagickFalse))
+        if (FuzzyColorCompare(image,p,&target) != MagickFalse)
           break;
-      floodplane[y*image->columns+x]=(unsigned char) MagickTrue;
+      q->opacity=TransparentOpacity;
+      q--;
       p--;
     }
     skip=(MagickBooleanType) (x >= x1);
@@ -465,7 +479,7 @@ MagickExport MagickBooleanType MatteFloodfillImage(Image *image,
       {
         start=x+1;
         if (start < x1)
-          Push(y,start,x1-1,-offset);
+          PushSegmentStack(y,start,x1-1,-offset);
         x=x1+1;
       }
     do
@@ -476,27 +490,30 @@ MagickExport MagickBooleanType MatteFloodfillImage(Image *image,
             {
               p=AcquireImagePixels(image,x,y,image->columns-x,1,
                 &image->exception);
-              if (p == (const PixelPacket *) NULL)
+              q=GetImagePixels(floodplane_image,x,y,image->columns-x,1);
+              if ((p == (const PixelPacket *) NULL) ||
+                  (q == (PixelPacket *) NULL))
                 break;
               for ( ; x < (long) image->columns; x++)
               {
+                if (q->opacity == TransparentOpacity)
+                  break;
                 if (method == FloodfillMethod)
                   {
-                    if ((FuzzyColorCompare(image,p,&target) == MagickFalse) ||
-                        ((int) floodplane[y*image->columns+x] != MagickFalse))
+                    if (FuzzyColorCompare(image,p,&target) == MagickFalse)
                       break;
                   }
                 else
-                  if ((FuzzyColorCompare(image,p,&target) != MagickFalse) ||
-                      ((int) floodplane[y*image->columns+x] != MagickFalse))
+                  if (FuzzyColorCompare(image,p,&target) != MagickFalse)
                     break;
-                floodplane[y*image->columns+x]=(unsigned char) MagickTrue;
+                q->opacity=TransparentOpacity;
+                q++;
                 p++;
               }
             }
-          Push(y,start,x-1,offset);
+          PushSegmentStack(y,start,x-1,offset);
           if (x > (x2+1))
-            Push(y,x2+1,x-1,-offset);
+            PushSegmentStack(y,x2+1,x-1,-offset);
         }
       skip=MagickFalse;
       x++;
@@ -504,23 +521,23 @@ MagickExport MagickBooleanType MatteFloodfillImage(Image *image,
         {
           p=AcquireImagePixels(image,x,y,(unsigned long) (x2-x+1),1,
             &image->exception);
-          if (p == (const PixelPacket *) NULL)
+          q=GetImagePixels(floodplane_image,x,y,(unsigned long) (x2-x+1),1);
+          if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
             break;
           for ( ; x <= x2; x++)
           {
+            if (q->opacity == TransparentOpacity)
+              break;
             if (method == FloodfillMethod)
               {
-                if ((FuzzyColorCompare(image,p,&target) != MagickFalse) &&
-                    ((int) floodplane[y*image->columns+x] == MagickFalse))
+                if (FuzzyColorCompare(image,p,&target) != MagickFalse)
                   break;
               }
             else
-              {
-                if ((FuzzyColorCompare(image,p,&target) == MagickFalse) &&
-                    ((int) floodplane[y*image->columns+x] == MagickFalse))
-                  break;
-              }
+              if (FuzzyColorCompare(image,p,&target) == MagickFalse)
+                break;
             p++;
+            q++;
           }
         }
       start=x;
@@ -531,20 +548,23 @@ MagickExport MagickBooleanType MatteFloodfillImage(Image *image,
     /*
       Tile fill color onto floodplane.
     */
+    p=AcquireImagePixels(floodplane_image,0,y,image->columns,1,
+      &image->exception);
     q=GetImagePixels(image,0,y,image->columns,1);
-    if (q == (PixelPacket *) NULL)
+    if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
       break;
     for (x=0; x < (long) image->columns; x++)
     {
-      if ((int) floodplane[y*image->columns+x] != MagickFalse)
+      if (p->opacity != OpaqueOpacity)
         q->opacity=opacity;
+      p++;
       q++;
     }
     if (SyncImagePixels(image) == MagickFalse)
       break;
   }
   segment_stack=(SegmentInfo *) RelinquishMagickMemory(segment_stack);
-  floodplane=(unsigned char *) RelinquishMagickMemory(floodplane);
+  floodplane_image=DestroyImage(floodplane_image);
   return((MagickBooleanType) (y == (long) image->rows));
 }
 
@@ -552,12 +572,12 @@ MagickExport MagickBooleanType MatteFloodfillImage(Image *image,
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
 %                                                                             %
-%     O p a q u e I m a g e                                                   %
+%     P a i n t O p a q u e I m a g e                                         %
 %                                                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  OpaqueImage() changes any pixel that matches color with the color
+%  PaintOpaqueImage() changes any pixel that matches color with the color
 %  defined by fill.
 %
 %  By default color must match a particular pixel color exactly.  However,
@@ -566,10 +586,10 @@ MagickExport MagickBooleanType MatteFloodfillImage(Image *image,
 %  For example, set fuzz to 10 and the color red at intensities of 100 and
 %  102 respectively are now interpreted as the same color.
 %
-%  The format of the OpaqueImage method is:
+%  The format of the PaintOpaqueImage method is:
 %
-%      MagickBooleanType OpaqueImage(Image *image,const PixelPacket target,
-%        const PixelPacket fill)
+%      MagickBooleanType PaintOpaqueImage(Image *image,
+%        const PixelPacket *target,const PixelPacket *fill)
 %
 %  A description of each parameter follows:
 %
@@ -581,10 +601,10 @@ MagickExport MagickBooleanType MatteFloodfillImage(Image *image,
 %
 %
 */
-MagickExport MagickBooleanType OpaqueImage(Image *image,
-  const PixelPacket target,const PixelPacket fill)
+MagickExport MagickBooleanType PaintOpaqueImage(Image *image,
+  const MagickPixelPacket *target,const MagickPixelPacket *fill)
 {
-#define OpaqueImageTag  "Opaque/Image"
+#define PaintOpaqueImageTag  "Opaque/Image"
 
   long
     y;
@@ -592,93 +612,53 @@ MagickExport MagickBooleanType OpaqueImage(Image *image,
   MagickBooleanType
     status;
 
+  MagickPixelPacket
+    pixel;
+
+  register IndexPacket
+    *indexes;
+
   register long
     x;
 
   register PixelPacket
     *q;
 
-  register long
-    i;
-
   /*
     Make image color opaque.
   */
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
+  assert(target != (MagickPixelPacket *) NULL);
+  assert(fill != (MagickPixelPacket *) NULL);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
-  switch (image->storage_class)
+  image->storage_class=DirectClass;
+  GetMagickPixelPacket(image,&pixel);
+  for (y=0; y < (long) image->rows; y++)
   {
-    case DirectClass:
-    default:
-    {
-      /*
-        Make DirectClass image opaque.
-      */
-      for (y=0; y < (long) image->rows; y++)
-      {
-        q=GetImagePixels(image,0,y,image->columns,1);
-        if (q == (PixelPacket *) NULL)
-          break;
-        for (x=0; x < (long) image->columns; x++)
-        {
-          if (FuzzyColorCompare(image,q,&target) != MagickFalse)
-            *q=fill;
-          q++;
-        }
-        if (SyncImagePixels(image) == MagickFalse)
-          break;
-        if (QuantumTick(y,image->rows) != 0)
-          {
-            status=MagickMonitor(OpaqueImageTag,y,image->rows,
-              &image->exception);
-            if (status == MagickFalse)
-              break;
-          }
-      }
+    q=GetImagePixels(image,0,y,image->columns,1);
+    if (q == (PixelPacket *) NULL)
       break;
-    }
-    case PseudoClass:
+    indexes=GetIndexes(image);
+    for (x=0; x < (long) image->columns; x++)
     {
-      /*
-        Make PseudoClass image opaque.
-      */
-      for (i=0; i < (long) image->colors; i++)
-      {
-        if (FuzzyColorCompare(image,&image->colormap[i],&target) != MagickFalse)
-          image->colormap[i]=fill;
-        if (QuantumTick(i,image->colors))
-          {
-            status=MagickMonitor(OpaqueImageTag,i,image->colors,
-              &image->exception);
-            if (status == MagickFalse)
-              break;
-          }
-      }
-      if (fill.opacity != OpaqueOpacity)
-        {
-          for (y=0; y < (long) image->rows; y++)
-          {
-            q=GetImagePixels(image,0,y,image->columns,1);
-            if (q == (PixelPacket *) NULL)
-              break;
-            for (x=0; x < (long) image->columns; x++)
-            {
-              if (FuzzyColorCompare(image,q,&target) != MagickFalse)
-                q->opacity=fill.opacity;
-              q++;
-            }
-            if (SyncImagePixels(image) == MagickFalse)
-              break;
-          }
-        }
-      (void) SyncImage(image);
-      break;
+      SetMagickPixelPacket(q,indexes+x,&pixel);
+      if (IsMagickColorSimilar(&pixel,target) != MagickFalse)
+        SetPixelPacket(fill,q,indexes+x);
+      q++;
     }
+    if (SyncImagePixels(image) == MagickFalse)
+      break;
+    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+        (QuantumTick(y,image->rows) != MagickFalse))
+      {
+        status=image->progress_monitor(PaintOpaqueImageTag,y,image->rows,
+          image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
   }
-  if (fill.opacity != OpaqueOpacity)
-    image->matte=MagickTrue;
   return(MagickTrue);
 }
 
@@ -686,13 +666,13 @@ MagickExport MagickBooleanType OpaqueImage(Image *image,
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
 %                                                                             %
-%     T r a n s p a r e n t I m a g e                                         %
+%     P a i n t T r a n s p a r e n t I m a g e                               %
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  TransparentImage() changes the opacity value associated with any pixel
+%  PaintTransparentImage() changes the opacity value associated with any pixel
 %  that matches color to the value defined by opacity.
 %
 %  By default color must match a particular pixel color exactly.  However,
@@ -701,10 +681,10 @@ MagickExport MagickBooleanType OpaqueImage(Image *image,
 %  For example, set fuzz to 10 and the color red at intensities of 100 and
 %  102 respectively are now interpreted as the same color.
 %
-%  The format of the TransparentImage method is:
+%  The format of the PaintTransparentImage method is:
 %
-%      MagickBooleanType TransparentImage(Image *image,const PixelPacket target,
-%        const Quantum opacity)
+%      MagickBooleanType PaintTransparentImage(Image *image,
+%        const MagickPixelPacket *target,const Quantum opacity)
 %
 %  A description of each parameter follows:
 %
@@ -716,10 +696,10 @@ MagickExport MagickBooleanType OpaqueImage(Image *image,
 %
 %
 */
-MagickExport MagickBooleanType TransparentImage(Image *image,
-  const PixelPacket target,const Quantum opacity)
+MagickExport MagickBooleanType PaintTransparentImage(Image *image,
+  const MagickPixelPacket *target,const Quantum opacity)
 {
-#define TransparentImageTag  "Transparent/Image"
+#define PaintTransparentImageTag  "Transparent/Image"
 
   long
     y;
@@ -727,6 +707,12 @@ MagickExport MagickBooleanType TransparentImage(Image *image,
   MagickBooleanType
     status;
 
+  MagickPixelPacket
+    pixel;
+
+  register IndexPacket
+    *indexes;
+
   register long
     x;
 
@@ -738,27 +724,32 @@ MagickExport MagickBooleanType TransparentImage(Image *image,
   */
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
+  assert(target != (MagickPixelPacket *) NULL);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   if (image->matte == MagickFalse)
     SetImageOpacity(image,OpaqueOpacity);
+  GetMagickPixelPacket(image,&pixel);
   for (y=0; y < (long) image->rows; y++)
   {
     q=GetImagePixels(image,0,y,image->columns,1);
     if (q == (PixelPacket *) NULL)
       break;
-    for (x=(long) image->columns-1; x >= 0; x--)
+    indexes=GetIndexes(image);
+    for (x=0; x < (long) image->columns; x++)
     {
-      if (FuzzyColorCompare(image,q,&target) != MagickFalse)
+      SetMagickPixelPacket(q,indexes+x,&pixel);
+      if (IsMagickColorSimilar(&pixel,target) != MagickFalse)
         q->opacity=opacity;
       q++;
     }
     if (SyncImagePixels(image) == MagickFalse)
       break;
-    if (QuantumTick(y,image->rows) != 0)
+    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+        (QuantumTick(y,image->rows) != MagickFalse))
       {
-        status=MagickMonitor(TransparentImageTag,y,image->rows,
-          &image->exception);
+        status=image->progress_monitor(PaintTransparentImageTag,y,image->rows,
+          image->client_data);
         if (status == MagickFalse)
           break;
       }
