@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -41,7 +41,8 @@
 */
 #include "magick/studio.h"
 #include "magick/color.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/hashmap.h"
 #include "magick/image.h"
 #include "magick/memory_.h"
@@ -137,8 +138,7 @@ MagickExport MagickBooleanType CloneImageProfiles(Image *image,
 MagickExport void DestroyImageProfiles(Image *image)
 {
   if (image->profiles != (HashmapInfo *) NULL)
-    DestroyHashmap(image->profiles);
-  image->profiles=(HashmapInfo *) NULL;
+    image->profiles=DestroyHashmap(image->profiles);
 }
 
 /*
@@ -294,7 +294,7 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
       profile=RemoveImageProfile(image,name);
       if (profile == (StringInfo *) NULL)
         ThrowBinaryException(OptionError,"NoProfileNameFound",name);
-      DestroyStringInfo(profile);
+      profile=DestroyStringInfo(profile);
       return(MagickTrue);
     }
   /*
@@ -311,7 +311,7 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
       if ((icc_profile != (StringInfo *) NULL) &&
           (CompareStringInfo(icc_profile,profile) != MagickFalse))
         {
-          DestroyStringInfo(profile);
+          profile=DestroyStringInfo(profile);
           return(MagickTrue);
         }
 #if !defined(HasLCMS)
@@ -547,8 +547,8 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
                 *p++=ScaleQuantumToShort(indexes[x]);
               q++;
             }
-            cmsDoTransform(transform,source_pixels,target_pixels,
-              (unsigned int) image->columns);
+            cmsDoTransform(transform,source_pixels,target_pixels,(unsigned int)
+              image->columns);
             p=target_pixels;
             q-=image->columns;
             for (x=0; x < (long) image->columns; x++)
@@ -582,7 +582,7 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
 #endif
     }
   status=SetImageProfile(image,name,profile);
-  DestroyStringInfo(profile);
+  profile=DestroyStringInfo(profile);
   return(status);
 }
 
@@ -709,8 +709,8 @@ MagickExport void ResetImageProfileIterator(const Image *image)
 
 static void *DestroyProfile(void *profile)
 {
-  DestroyStringInfo((StringInfo *) profile);
-  return((void *) NULL);
+  profile=(void *) DestroyStringInfo((StringInfo *) profile);
+  return(profile);
 }
 
 MagickExport MagickBooleanType SetImageProfile(Image *image,const char *name,
@@ -733,7 +733,8 @@ MagickExport MagickBooleanType SetImageProfile(Image *image,const char *name,
   LocaleLower(key);
   status=PutEntryInHashmap(image->profiles,AcquireString(key),
     CloneStringInfo(profile));
-  if ((status != MagickFalse) && (LocaleCompare(name,"icc") == 0))
+  if ((status != MagickFalse) &&
+      ((LocaleCompare(name,"icc") == 0) || (LocaleCompare(name,"icm") == 0)))
     {
       const StringInfo
         *icc_profile;
@@ -748,7 +749,8 @@ MagickExport MagickBooleanType SetImageProfile(Image *image,const char *name,
           image->color_profile.info=icc_profile->datum;
         }
     }
-  if ((status != MagickFalse) && (LocaleCompare(name,"iptc") == 0))
+  if ((status != MagickFalse) &&
+      ((LocaleCompare(name,"iptc") == 0) || (LocaleCompare(name,"8bim") == 0)))
     {
       const StringInfo
         *iptc_profile;
