@@ -17,7 +17,7 @@
 %                              July 1992                                      %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -179,8 +179,10 @@
 #include "magick/color.h"
 #include "magick/colorspace.h"
 #include "magick/enhance.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/image.h"
+#include "magick/list.h"
 #include "magick/memory_.h"
 #include "magick/quantize.h"
 #include "magick/monitor.h"
@@ -264,8 +266,8 @@ typedef struct _CubeInfo
   MagickRealType
     weights[ExceptionQueueLength];
 
-  const QuantizeInfo
-    *quantize_info;
+  QuantizeInfo
+    quantize_info;
 
   long
     x,
@@ -292,7 +294,8 @@ static void
   HilbertCurve(CubeInfo *,Image *,const unsigned long,const unsigned int),
   PruneLevel(CubeInfo *,const NodeInfo *),
   PruneToCubeDepth(CubeInfo *,const NodeInfo *),
-  ReduceImageColors(CubeInfo *,const unsigned long,ExceptionInfo *);
+  ReduceImageColors(CubeInfo *,const Image *,const unsigned long,
+    ExceptionInfo *);
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -389,7 +392,7 @@ static MagickBooleanType AssignImageColors(CubeInfo *cube_info,Image *image)
   /*
     Create a reduced color image.
   */
-  dither=cube_info->quantize_info->dither;
+  dither=cube_info->quantize_info.dither;
   if (dither != MagickFalse)
     dither=DitherImage(cube_info,image);
   if (dither == MagickFalse)
@@ -429,7 +432,7 @@ static MagickBooleanType AssignImageColors(CubeInfo *cube_info,Image *image)
         {
           if (image->storage_class == PseudoClass)
             indexes[x+i]=(IndexPacket) index;
-          if (cube_info->quantize_info->measure_error == MagickFalse)
+          if (cube_info->quantize_info.measure_error == MagickFalse)
             {
               q->red=image->colormap[index].red;
               q->green=image->colormap[index].green;
@@ -440,15 +443,17 @@ static MagickBooleanType AssignImageColors(CubeInfo *cube_info,Image *image)
       }
       if (SyncImagePixels(image) == MagickFalse)
         break;
-      if (QuantumTick(y,image->rows) != MagickFalse)
+      if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+          (QuantumTick(y,image->rows) != MagickFalse))
         {
-          status=MagickMonitor(AssignImageTag,y,image->rows,&image->exception);
+          status=image->progress_monitor(AssignImageTag,y,image->rows,
+            image->client_data);
           if (status == MagickFalse)
             break;
         }
     }
-  if ((cube_info->quantize_info->number_colors == 2) &&
-      (cube_info->quantize_info->colorspace == GRAYColorspace))
+  if ((cube_info->quantize_info.number_colors == 2) &&
+      (cube_info->quantize_info.colorspace == GRAYColorspace))
     {
       Quantum
         intensity;
@@ -467,7 +472,7 @@ static MagickBooleanType AssignImageColors(CubeInfo *cube_info,Image *image)
         q++;
       }
     }
-  if (cube_info->quantize_info->measure_error != MagickFalse)
+  if (cube_info->quantize_info.measure_error != MagickFalse)
     (void) GetImageQuantizeError(image);
   (void) SyncImage(image);
   image->storage_class=PseudoClass;
@@ -625,8 +630,8 @@ static MagickBooleanType ClassifyImageColors(CubeInfo *cube_info,
         pixel.red=QuantumScale*(p->red-mid.red);
         pixel.green=QuantumScale*(p->green-mid.green);
         pixel.blue=QuantumScale*(p->blue-mid.blue);
-        node_info->quantize_error+=count*sqrt(pixel.red*pixel.red+
-          pixel.green*pixel.green+pixel.blue*pixel.blue);
+        node_info->quantize_error+=count*sqrt((double) (pixel.red*pixel.red+
+          pixel.green*pixel.green+pixel.blue*pixel.blue));
         cube_info->root->quantize_error+=node_info->quantize_error;
         index--;
       }
@@ -639,9 +644,11 @@ static MagickBooleanType ClassifyImageColors(CubeInfo *cube_info,
       node_info->total_blue+=QuantumScale*((MagickRealType) count*p->blue);
       p+=count;
     }
-    if (QuantumTick(y,image->rows) != MagickFalse)
+    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+        (QuantumTick(y,image->rows) != MagickFalse))
       {
-        status=MagickMonitor(ClassifyImageTag,y,image->rows,exception);
+        status=image->progress_monitor(ClassifyImageTag,y,image->rows,
+          image->client_data);
         if (status == MagickFalse)
           break;
       }
@@ -705,8 +712,8 @@ static MagickBooleanType ClassifyImageColors(CubeInfo *cube_info,
         pixel.red=QuantumScale*(p->red-mid.red);
         pixel.green=QuantumScale*(p->green-mid.green);
         pixel.blue=QuantumScale*(p->blue-mid.blue);
-        node_info->quantize_error+=count*sqrt(pixel.red*pixel.red+
-          pixel.green*pixel.green+pixel.blue*pixel.blue);
+        node_info->quantize_error+=count*sqrt((double) (pixel.red*pixel.red+
+          pixel.green*pixel.green+pixel.blue*pixel.blue));
         cube_info->root->quantize_error+=node_info->quantize_error;
         index--;
       }
@@ -719,9 +726,11 @@ static MagickBooleanType ClassifyImageColors(CubeInfo *cube_info,
       node_info->total_blue+=QuantumScale*((MagickRealType) count*p->blue);
       p+=count;
     }
-    if (QuantumTick(y,image->rows) != MagickFalse)
+    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+        (QuantumTick(y,image->rows) != MagickFalse))
       {
-        status=MagickMonitor(ClassifyImageTag,y,image->rows,exception);
+        status=image->progress_monitor(ClassifyImageTag,y,image->rows,
+          image->client_data);
         if (status == MagickFalse)
           break;
       }
@@ -992,7 +1001,7 @@ static void DestroyCubeInfo(CubeInfo *cube_info)
       RelinquishMagickMemory(cube_info->node_queue);
     cube_info->node_queue=nodes;
   } while (cube_info->node_queue != (Nodes *) NULL);
-  if (cube_info->quantize_info->dither != MagickFalse)
+  if (cube_info->quantize_info.dither != MagickFalse)
     cube_info->cache=(long *) RelinquishMagickMemory(cube_info->cache);
   cube_info=(CubeInfo *) RelinquishMagickMemory(cube_info);
 }
@@ -1013,7 +1022,7 @@ static void DestroyCubeInfo(CubeInfo *cube_info)
 %
 %  The format of the DestroyQuantizeInfo method is:
 %
-%      DestroyQuantizeInfo(QuantizeInfo *quantize_info)
+%      QuantizeInfo *DestroyQuantizeInfo(QuantizeInfo *quantize_info)
 %
 %  A description of each parameter follows:
 %
@@ -1021,12 +1030,14 @@ static void DestroyCubeInfo(CubeInfo *cube_info)
 %
 %
 */
-MagickExport void DestroyQuantizeInfo(QuantizeInfo *quantize_info)
+MagickExport QuantizeInfo *DestroyQuantizeInfo(QuantizeInfo *quantize_info)
 {
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(quantize_info != (QuantizeInfo *) NULL);
   assert(quantize_info->signature == MagickSignature);
+  quantize_info->signature=(~MagickSignature);
   quantize_info=(QuantizeInfo *) RelinquishMagickMemory(quantize_info);
+  return(quantize_info);
 }
 
 /*
@@ -1145,7 +1156,7 @@ static MagickBooleanType Dither(CubeInfo *cube_info,Image *image,
       index=(IndexPacket) p->cache[i];
       if (image->storage_class == PseudoClass)
         *indexes=(IndexPacket) index;
-      if (cube_info->quantize_info->measure_error == MagickFalse)
+      if (cube_info->quantize_info.measure_error == MagickFalse)
         {
           q->red=image->colormap[index].red;
           q->green=image->colormap[index].green;
@@ -1301,8 +1312,8 @@ static CubeInfo *GetCubeInfo(const QuantizeInfo *quantize_info,
   if (cube_info->root == (NodeInfo *) NULL)
     return((CubeInfo *) NULL);
   cube_info->root->parent=cube_info->root;
-  cube_info->quantize_info=quantize_info;
-  if (cube_info->quantize_info->dither == MagickFalse)
+  cube_info->quantize_info=(*quantize_info);
+  if (cube_info->quantize_info.dither == MagickFalse)
     return(cube_info);
   /*
     Initialize dither resources.
@@ -1507,9 +1518,9 @@ MagickExport MagickBooleanType GetImageQuantizeError(Image *image)
       p++;
     }
   }
-  image->error.mean_error_per_pixel=mean_error_per_pixel/area;
-  image->error.normalized_mean_error=mean_error/area/MaxRGB/MaxRGB;
-  image->error.normalized_maximum_error=maximum_error/MaxRGB;
+  image->error.mean_error_per_pixel=(double) (mean_error_per_pixel/area);
+  image->error.normalized_mean_error=(double) (mean_error/area/MaxRGB/MaxRGB);
+  image->error.normalized_maximum_error=(double) (maximum_error/MaxRGB);
   return(MagickTrue);
 }
 
@@ -1538,7 +1549,7 @@ MagickExport MagickBooleanType GetImageQuantizeError(Image *image)
 */
 MagickExport void GetQuantizeInfo(QuantizeInfo *quantize_info)
 {
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(quantize_info != (QuantizeInfo *) NULL);
   (void) ResetMagickMemory(quantize_info,0,sizeof(*quantize_info));
   quantize_info->number_colors=256;
@@ -1731,6 +1742,8 @@ MagickExport MagickBooleanType MapImage(Image *image,const Image *map_image,
   if (cube_info == (CubeInfo *) NULL)
     ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
       image->filename);
+  if (image->colorspace == CMYKColorspace)
+    (void) SetImageColorspace(image,RGBColorspace);
   status=ClassifyImageColors(cube_info,map_image,&image->exception);
   if (status != MagickFalse)
     {
@@ -1815,6 +1828,8 @@ MagickExport MagickBooleanType MapImages(Image *images,const Image *map_image,
   if (cube_info == (CubeInfo *) NULL)
     ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
       image->filename);
+  if (image->colorspace == CMYKColorspace)
+    (void) SetImageColorspace(image,RGBColorspace);
   status=ClassifyImageColors(cube_info,map_image,&image->exception);
   if (status != MagickFalse)
     {
@@ -1824,8 +1839,8 @@ MagickExport MagickBooleanType MapImages(Image *images,const Image *map_image,
       quantize_info.number_colors=cube_info->colors;
       for (image=images; image != (Image *) NULL; image=image->next)
       {
-        quantize_info.colorspace=image->matte != MagickFalse ? TransparentColorspace :
-          RGBColorspace;
+        quantize_info.colorspace=image->matte != MagickFalse ?
+          TransparentColorspace : RGBColorspace;
         status=AssignImageColors(cube_info,image);
         if (status == MagickFalse)
           break;
@@ -1922,10 +1937,12 @@ MagickExport MagickBooleanType OrderedDitherImage(Image *image)
       q++;
     }
     if (SyncImagePixels(image) == MagickFalse)
-      break;
-    if (QuantumTick(y,image->rows) != MagickFalse)
+      break ;
+    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+        (QuantumTick(y,image->rows) != MagickFalse))
       {
-        status=MagickMonitor(DitherImageTag,y,image->rows,&image->exception);
+        status=image->progress_monitor(DitherImageTag,y,image->rows,
+          image->client_data);
         if (status == MagickFalse)
           break;
       }
@@ -2000,7 +2017,7 @@ MagickExport MagickBooleanType PosterizeImage(Image *image,
     return(MagickFalse);
   if (AllocateImageColormap(posterize_image,levels*levels*levels) == MagickFalse)
     {
-      DestroyImage(posterize_image);
+      posterize_image=DestroyImage(posterize_image);
       return(MagickFalse);
     }
   for (l=2; (l*l*l) <= (long) posterize_image->colors; l++);
@@ -2010,7 +2027,7 @@ MagickExport MagickBooleanType PosterizeImage(Image *image,
   q=SetImagePixels(posterize_image,0,0,posterize_image->columns,1);
   if (q == (PixelPacket *) NULL)
     {
-      DestroyImage(posterize_image);
+      posterize_image=DestroyImage(posterize_image);
       return(MagickFalse);
     }
   indexes=GetIndexes(posterize_image);
@@ -2028,11 +2045,11 @@ MagickExport MagickBooleanType PosterizeImage(Image *image,
       }
   if (SyncImagePixels(posterize_image) == MagickFalse)
     {
-      DestroyImage(posterize_image);
+      posterize_image=DestroyImage(posterize_image);
       return(MagickFalse);
     }
   status=MapImage(image,posterize_image,dither);
-  DestroyImage(posterize_image);
+  posterize_image=DestroyImage(posterize_image);
   return(status);
 }
 
@@ -2251,13 +2268,15 @@ MagickExport MagickBooleanType QuantizeImage(const QuantizeInfo *quantize_info,
   if (cube_info == (CubeInfo *) NULL)
     ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
       image->filename);
+  if (image->colorspace == CMYKColorspace)
+    (void) SetImageColorspace(image,RGBColorspace);
   status=ClassifyImageColors(cube_info,image,&image->exception);
   if (status != MagickFalse)
     {
       /*
         Reduce the number of colors in the image.
       */
-      ReduceImageColors(cube_info,number_colors,&image->exception);
+      ReduceImageColors(cube_info,image,number_colors,&image->exception);
       status=AssignImageColors(cube_info,image);
       if ((quantize_info->colorspace != GRAYColorspace) &&
           (quantize_info->colorspace != RGBColorspace))
@@ -2302,15 +2321,15 @@ MagickExport MagickBooleanType QuantizeImages(const QuantizeInfo *quantize_info,
   CubeInfo
     *cube_info;
 
-  MonitorHandler
-    handler;
-
   Image
     *image;
 
   MagickBooleanType
     status;
 
+  MagickProgressMonitor
+    progress_monitor;
+
   register long
     i;
 
@@ -2376,43 +2395,58 @@ MagickExport MagickBooleanType QuantizeImages(const QuantizeInfo *quantize_info,
   {
     if (quantize_info->colorspace != RGBColorspace)
       (void) SetImageColorspace(image,quantize_info->colorspace);
-    image=image->next;
+    image=GetNextImageInList(image);
   }
   number_images=(unsigned long) i;
   image=images;
   for (i=0; image != (Image *) NULL; i++)
   {
-    handler=SetMonitorHandler((MonitorHandler) NULL);
+    progress_monitor=SetImageProgressMonitor(image,(MagickProgressMonitor) NULL,
+      image->client_data);
+    if (image->colorspace == CMYKColorspace)
+      (void) SetImageColorspace(image,RGBColorspace);
     status=ClassifyImageColors(cube_info,image,&image->exception);
     if (status == MagickFalse)
       break;
-    image=image->next;
-    (void) SetMonitorHandler(handler);
-    status=MagickMonitor(ClassifyImageTag,i,number_images,&image->exception);
-    if (status == MagickFalse)
-      break;
+    (void) SetImageProgressMonitor(image,progress_monitor,image->client_data);
+    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+        (QuantumTick(i,number_images) != MagickFalse))
+      {
+        status=image->progress_monitor(AssignImageTag,i,number_images,
+          image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
+    image=GetNextImageInList(image);
   }
   if (status != MagickFalse)
     {
       /*
         Reduce the number of colors in an image sequence.
       */
-      ReduceImageColors(cube_info,number_colors,&image->exception);
       image=images;
+      ReduceImageColors(cube_info,image,number_colors,&image->exception);
       for (i=0; image != (Image *) NULL; i++)
       {
-        handler=SetMonitorHandler((MonitorHandler) NULL);
+        progress_monitor=SetImageProgressMonitor(image,
+          (MagickProgressMonitor) NULL,image->client_data);
         status=AssignImageColors(cube_info,image);
         if (status == MagickFalse)
           break;
         if ((quantize_info->colorspace != GRAYColorspace) &&
             (quantize_info->colorspace != RGBColorspace))
           (void) SetImageColorspace(image,quantize_info->colorspace);
-        image=image->next;
-        (void) SetMonitorHandler(handler);
-        status=MagickMonitor(AssignImageTag,i,number_images,&image->exception);
-        if (status == MagickFalse)
-          break;
+        (void) SetImageProgressMonitor(image,progress_monitor,
+          image->client_data);
+        if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+            (QuantumTick(i,number_images) != MagickFalse))
+          {
+            status=image->progress_monitor(AssignImageTag,i,number_images,
+              image->client_data);
+            if (status == MagickFalse)
+              break;
+          }
+        image=GetNextImageInList(image);
       }
     }
   DestroyCubeInfo(cube_info);
@@ -2515,11 +2549,13 @@ static void Reduce(CubeInfo *cube_info,const NodeInfo *node_info)
 %
 %  The format of the ReduceImageColors method is:
 %
-%      ReduceImageColors(CubeInfo *cube_info,const MagickBooleanType number_colors,
-%        ExceptionInfo *exception)
+%      ReduceImageColors(CubeInfo *cube_info,const Image *image,
+%        const MagickBooleanType number_colors,ExceptionInfo *exception)
 %
 %  A description of each parameter follows.
 %
+%    o image: The image.
+%
 %    o cube_info: A pointer to the Cube structure.
 %
 %    o number_colors: This integer value indicates the maximum number of
@@ -2530,7 +2566,7 @@ static void Reduce(CubeInfo *cube_info,const NodeInfo *node_info)
 %    o exception: Return any errors or warnings in this structure.
 %
 */
-static void ReduceImageColors(CubeInfo *cube_info,
+static void ReduceImageColors(CubeInfo *cube_info,const Image *image,
   const unsigned long number_colors,ExceptionInfo *exception)
 {
 #define ReduceImageTag  " Reduce/Image"
@@ -2549,9 +2585,13 @@ static void ReduceImageColors(CubeInfo *cube_info,
     cube_info->next_threshold=cube_info->root->quantize_error-1;
     cube_info->colors=0;
     Reduce(cube_info,cube_info->root);
-    status=MagickMonitor(ReduceImageTag,(long) span-cube_info->colors,span-
-      number_colors+1,exception);
-    if (status == MagickFalse)
-      break;
+    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+        (QuantumTick(span-cube_info->colors,span-number_colors+1) != MagickFalse))
+      {
+        status=image->progress_monitor(ReduceImageTag,(MagickOffsetType) span-
+          cube_info->colors,span-number_colors+1,image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
   }
 }
