@@ -1,5 +1,19 @@
 /*
-  ImageMagick Image Quantization Methods.
+  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization
+  dedicated to making software imaging solutions freely available.
+  
+  You may not use this file except in compliance with the License.
+  obtain a copy of the License at
+  
+    http://www.imagemagick.org/www/Copyright.html
+  
+  Unless required by applicable law or agreed to in writing, software
+  distributed under the License is distributed on an "AS IS" BASIS,
+  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+  See the License for the specific language governing permissions and
+  limitations under the License.
+
+  ImageMagick image quantization methods.
 */
 #ifndef _MAGICK_QUANTIZE_H
 #define _MAGICK_QUANTIZE_H
@@ -39,11 +53,11 @@ extern MagickExport MagickBooleanType
   QuantizeImages(const QuantizeInfo *,Image *);
 
 extern MagickExport QuantizeInfo
-  *CloneQuantizeInfo(const QuantizeInfo *);
+  *CloneQuantizeInfo(const QuantizeInfo *),
+  *DestroyQuantizeInfo(QuantizeInfo *);
 
 extern MagickExport void
   CompressImageColormap(Image *),
-  DestroyQuantizeInfo(QuantizeInfo *),
   GetQuantizeInfo(QuantizeInfo *);
 
 #if defined(__cplusplus) || defined(c_plusplus)
