@@ -16,7 +16,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -41,7 +41,8 @@
 #include "magick/studio.h"
 #include "magick/attribute.h"
 #include "magick/color.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/image.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
@@ -246,8 +247,9 @@ static MagickRealType BesselOrderOne(MagickRealType x)
     x=(-x);
   if (x < 8.0)
     return(p*J1(x));
-  q=sqrt(2.0/(MagickPI*x))*(P1(x)*(1.0/sqrt(2.0)*(sin(x)-cos(x)))-8.0/x*Q1(x)*
-    (-1.0/sqrt(2.0)*(sin(x)+cos(x))));
+  q=sqrt((double) (2.0/(MagickPI*x)))*(P1(x)*(1.0/sqrt(2.0)*(sin((double) x)-
+    cos((double) x)))-8.0/x*Q1(x)*(-1.0/sqrt(2.0)*(sin((double) x)+
+    cos((double) x))));
   if (p < 0.0)
     q=(-q);
   return(q);
@@ -387,24 +389,25 @@ MagickExport Image *MinifyImage(const Image *image,ExceptionInfo *exception)
 */
 
 static MagickRealType Bessel(const MagickRealType x,
-  const MagickRealType support)
+  const MagickRealType magick_unused(support))
 {
   if (x == 0.0)
     return((MagickRealType) (MagickPI/4.0));
   return(BesselOrderOne(MagickPI*x)/(2.0*x));
 }
 
-static MagickRealType Sinc(const MagickRealType x,const MagickRealType support)
+static MagickRealType Sinc(const MagickRealType x,
+  const MagickRealType magick_unused(support))
 {
   if (x == 0.0)
     return(1.0);
-  return(sin(MagickPI*x)/(MagickPI*x));
+  return(sin(MagickPI*(double) x)/(MagickPI*(double) x));
 }
 
 static MagickRealType Blackman(const MagickRealType x,
-  const MagickRealType support)
+  const MagickRealType magick_unused(support))
 {
-  return(0.42+0.5*cos(MagickPI*x)+0.08*cos(2*MagickPI*x));
+  return(0.42+0.5*cos(MagickPI*(double) x)+0.08*cos(2*MagickPI*(double) x));
 }
 
 static MagickRealType BlackmanBessel(const MagickRealType x,
@@ -419,7 +422,8 @@ static MagickRealType BlackmanSinc(const MagickRealType x,
   return(Blackman(x/support,support)*Sinc(x,support));
 }
 
-static MagickRealType Box(const MagickRealType x,const MagickRealType support)
+static MagickRealType Box(const MagickRealType x,
+  const MagickRealType magick_unused(support))
 {
   if (x < -0.5)
     return(0.0);
@@ -429,7 +433,7 @@ static MagickRealType Box(const MagickRealType x,const MagickRealType support)
 }
 
 static MagickRealType Catrom(const MagickRealType x,
-  const MagickRealType support)
+  const MagickRealType magick_unused(support))
 {
   if (x < -2.0)
     return(0.0);
@@ -444,7 +448,8 @@ static MagickRealType Catrom(const MagickRealType x,
   return(0.0);
 }
 
-static MagickRealType Cubic(const MagickRealType x,const MagickRealType support)
+static MagickRealType Cubic(const MagickRealType x,
+  const MagickRealType magick_unused(support))
 {
   if (x < -2.0)
     return(0.0);
@@ -460,25 +465,25 @@ static MagickRealType Cubic(const MagickRealType x,const MagickRealType support)
 }
 
 static MagickRealType Gaussian(const MagickRealType x,
-  const MagickRealType support)
+  const MagickRealType magick_unused(support))
 {
-  return(exp(-2.0*x*x)*sqrt(2.0/MagickPI));
+  return(exp((double) (-2.0*x*x))*sqrt(2.0/MagickPI));
 }
 
 static MagickRealType Hanning(const MagickRealType x,
-  const MagickRealType support)
+  const MagickRealType magick_unused(support))
 {
-  return(0.5+0.5*cos(MagickPI*x));
+  return(0.5+0.5*cos(MagickPI*(double) x));
 }
 
 static MagickRealType Hamming(const MagickRealType x,
-  const MagickRealType support)
+  const MagickRealType magick_unused(support))
 {
-  return(0.54+0.46*cos(MagickPI*x));
+  return(0.54+0.46*cos(MagickPI*(double) x));
 }
 
 static MagickRealType Hermite(const MagickRealType x,
-  const MagickRealType support)
+  const MagickRealType magick_unused(support))
 {
   if (x < -1.0)
     return(0.0);
@@ -502,7 +507,7 @@ static MagickRealType Lanczos(const MagickRealType x,
 }
 
 static MagickRealType Mitchell(const MagickRealType x,
-  const MagickRealType support)
+  const MagickRealType magick_unused(support))
 {
 #define B   (1.0/3.0)
 #define C   (1.0/3.0)
@@ -528,7 +533,7 @@ static MagickRealType Mitchell(const MagickRealType x,
 }
 
 static MagickRealType Quadratic(const MagickRealType x,
-  const MagickRealType support)
+  const MagickRealType magick_unused(support))
 {
   if (x < -1.5)
     return(0.0);
@@ -542,7 +547,7 @@ static MagickRealType Quadratic(const MagickRealType x,
 }
 
 static MagickRealType Triangle(const MagickRealType x,
-  const MagickRealType support)
+  const MagickRealType magick_unused(support))
 {
   if (x < -1.0)
     return(0.0);
@@ -689,10 +694,11 @@ static MagickBooleanType HorizontalFilter(const Image *source,
     }
     if (SyncImagePixels(destination) == MagickFalse)
       break;
-    if (QuantumTick(*quantum,span) != MagickFalse)
+    if ((source->progress_monitor != (MagickProgressMonitor) NULL) &&
+        (QuantumTick(*quantum,span) != MagickFalse))
       {
-        status=MagickMonitor(ResizeImageTag,(MagickOffsetType) *quantum,span,
-          exception);
+        status=source->progress_monitor(ResizeImageTag,(MagickOffsetType)
+          *quantum,span,source->client_data);
         if (status == MagickFalse)
           break;
       }
@@ -835,10 +841,11 @@ static MagickBooleanType VerticalFilter(const Image *source,Image *destination,
     }
     if (SyncImagePixels(destination) == MagickFalse)
       break;
-    if (QuantumTick(*quantum,span) != MagickFalse)
+    if ((source->progress_monitor != (MagickProgressMonitor) NULL) &&
+        (QuantumTick(*quantum,span) != MagickFalse))
       {
-        status=MagickMonitor(ResizeImageTag,(MagickOffsetType) *quantum,span,
-          exception);
+        status=source->progress_monitor(ResizeImageTag,(MagickOffsetType)
+          *quantum,span,source->client_data);
         if (status == MagickFalse)
           break;
       }
@@ -907,11 +914,9 @@ MagickExport Image *ResizeImage(const Image *image,const unsigned long columns,
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  assert((filter >= 0) && (filter <= SincFilter));
+  assert(((int) filter >= 0) && ((int) filter <= SincFilter));
   if ((columns == 0) || (rows == 0))
     ThrowImageException(ImageError,"NegativeOrZeroImageSize");
-  if ((columns == image->columns) && (rows == image->rows) && (blur == 1.0))
-    return(CloneImage(image,0,0,MagickTrue,exception));
   resize_image=CloneImage(image,columns,rows,MagickTrue,exception);
   if (resize_image == (Image *) NULL)
     return((Image *) NULL);
@@ -924,9 +929,12 @@ MagickExport Image *ResizeImage(const Image *image,const unsigned long columns,
   if (filter != UndefinedFilter)
     i=(long) filter;
   else
-    if ((image->storage_class == PseudoClass) ||
-        (image->matte != MagickFalse) || ((x_factor*y_factor) > 1.0))
-      i=(long) MitchellFilter;
+    if ((x_factor == 1.0) && (y_factor == 1.0))
+      i=(long) PointFilter;
+    else
+      if ((image->storage_class == PseudoClass) ||
+          (image->matte != MagickFalse) || ((x_factor*y_factor) > 1.0))
+        i=(long) MitchellFilter;
   x_support=blur*Max(1.0/x_factor,1.0)*filters[i].support;
   y_support=blur*Max(1.0/y_factor,1.0)*filters[i].support;
   support=Max(x_support,y_support);
@@ -936,7 +944,7 @@ MagickExport Image *ResizeImage(const Image *image,const unsigned long columns,
     (2.0*Max(support,0.5)+3)*sizeof(*contribution));
   if (contribution == (ContributionInfo *) NULL)
     {
-      DestroyImage(resize_image);
+      resize_image=DestroyImage(resize_image);
       ThrowImageException(ResourceLimitError,"MemoryAllocationFailed");
     }
   /*
@@ -951,7 +959,7 @@ MagickExport Image *ResizeImage(const Image *image,const unsigned long columns,
         {
           contribution=(ContributionInfo *)
             RelinquishMagickMemory(contribution);
-          DestroyImage(resize_image);
+          resize_image=DestroyImage(resize_image);
           return((Image *) NULL);
         }
       span=(MagickSizeType) (source_image->columns+resize_image->rows);
@@ -967,7 +975,7 @@ MagickExport Image *ResizeImage(const Image *image,const unsigned long columns,
         {
           contribution=(ContributionInfo *)
             RelinquishMagickMemory(contribution);
-          DestroyImage(resize_image);
+          resize_image=DestroyImage(resize_image);
           return((Image *) NULL);
         }
       span=(MagickSizeType) (resize_image->columns+source_image->rows);
@@ -980,10 +988,10 @@ MagickExport Image *ResizeImage(const Image *image,const unsigned long columns,
     Free allocated memory.
   */
   contribution=(ContributionInfo *) RelinquishMagickMemory(contribution);
-  DestroyImage(source_image);
+  source_image=DestroyImage(source_image);
   if (status == MagickFalse)
     {
-      DestroyImage(resize_image);
+      resize_image=DestroyImage(resize_image);
       ThrowImageException(ResourceLimitError,"MemoryAllocationFailed");
     }
   return(resize_image);
@@ -1065,8 +1073,6 @@ MagickExport Image *SampleImage(const Image *image,const unsigned long columns,
   assert(exception->signature == MagickSignature);
   if ((columns == 0) || (rows == 0))
     ThrowImageException(ImageError,"NegativeOrZeroImageSize");
-  if ((columns == image->columns) && (rows == image->rows))
-    return(CloneImage(image,0,0,MagickTrue,exception));
   sample_image=CloneImage(image,columns,rows,MagickTrue,exception);
   if (sample_image == (Image *) NULL)
     return((Image *) NULL);
@@ -1082,7 +1088,7 @@ MagickExport Image *SampleImage(const Image *image,const unsigned long columns,
   if ((pixels == (PixelPacket *) NULL) || (x_offset == (long *) NULL) ||
       (y_offset == (long *) NULL))
     {
-      DestroyImage(sample_image);
+      sample_image=DestroyImage(sample_image);
       ThrowImageException(ResourceLimitError,"MemoryAllocationFailed");
     }
   /*
@@ -1129,9 +1135,11 @@ MagickExport Image *SampleImage(const Image *image,const unsigned long columns,
         sample_indexes[x]=indexes[x_offset[x]];
     if (SyncImagePixels(sample_image) == MagickFalse)
       break;
-    if (QuantumTick(y,sample_image->rows) != MagickFalse)
+    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+        (QuantumTick(y,image->rows) != MagickFalse))
       {
-        status=MagickMonitor(SampleImageTag,y,sample_image->rows,exception);
+        status=image->progress_monitor(SampleImageTag,y,image->rows,
+          image->client_data);
         if (status == MagickFalse)
           break;
       }
@@ -1254,7 +1262,7 @@ MagickExport Image *ScaleImage(const Image *image,const unsigned long columns,
       (x_vector == (MagickPixelPacket *) NULL) ||
       (y_vector == (MagickPixelPacket *) NULL))
     {
-      DestroyImage(scale_image);
+      scale_image=DestroyImage(scale_image);
       ThrowImageException(ResourceLimitError,"MemoryAllocationFailed");
     }
   /*
@@ -1266,6 +1274,7 @@ MagickExport Image *ScaleImage(const Image *image,const unsigned long columns,
   y_scale=(MagickRealType) scale_image->rows/(MagickRealType) image->rows;
   (void) ResetMagickMemory(y_vector,0,(size_t) image->columns*
     sizeof(*y_vector));
+  GetMagickPixelPacket(image,&pixel);
   (void) ResetMagickMemory(&zero,0,sizeof(zero));
   i=0;
   for (y=0; y < (long) scale_image->rows; y++)
@@ -1509,9 +1518,11 @@ MagickExport Image *ScaleImage(const Image *image,const unsigned long columns,
     }
     if (SyncImagePixels(scale_image) == MagickFalse)
       break;
-    if (QuantumTick(y,scale_image->rows) != MagickFalse)
+    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+        (QuantumTick(y,image->rows) != MagickFalse))
       {
-        status=MagickMonitor(ScaleImageTag,y,scale_image->rows,exception);
+        status=image->progress_monitor(ScaleImageTag,y,image->rows,
+          image->client_data);
         if (status == MagickFalse)
           break;
       }
@@ -1588,7 +1599,7 @@ MagickExport Image *ThumbnailImage(const Image *image,
   if (sample_image == (Image *) NULL)
     return((Image *) NULL);
   thumbnail_image=ZoomImage(sample_image,columns,rows,exception);
-  DestroyImage(sample_image);
+  sample_image=DestroyImage(sample_image);
   (void) StripImage(thumbnail_image);
   return(thumbnail_image);
 }
