@@ -17,7 +17,7 @@
 %                               September 2002                                %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -40,7 +40,8 @@
   Include declarations.
 */
 #include "magick/studio.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/hashmap.h"
 #include "magick/log.h"
 #include "magick/image.h"
@@ -90,7 +91,7 @@ static SemaphoreInfo
 
 static ResourceInfo
   resource_info =
-    { 0, 0, 0, 0, 0, 128, 512, 1024, ResourceInfinity, 768 };
+    { 0, 0, 0, 0, 0, 256, 512, 1024, ResourceInfinity, 768 };
 
 static LinkedListInfo
   *temporary_resources = (LinkedListInfo *) NULL;
@@ -140,7 +141,7 @@ MagickExport MagickBooleanType AcquireMagickResource(const ResourceType type,
 
   status=MagickFalse;
   FormatSize(size,resource_request);
-  AcquireSemaphoreInfo(&resource_semaphore);
+  resource_semaphore=AcquireSemaphoreInfo(resource_semaphore);
   switch (type)
   {
     case AreaResource:
@@ -205,7 +206,7 @@ MagickExport MagickBooleanType AcquireMagickResource(const ResourceType type,
     default:
       break;
   }
-  RelinquishSemaphoreInfo(&resource_semaphore);
+  RelinquishSemaphoreInfo(resource_semaphore);
   (void) LogMagickEvent(ResourceEvent,GetMagickModule(),"%s: %s/%s/%s",
     resource_type,resource_request,resource_current,resource_limit);
   return(status);
@@ -290,7 +291,9 @@ static MagickBooleanType GetPathTemplate(char *path)
     file_info;
 
   (void) strcpy(path,"magick-XXXXXXXX");
-  directory=getenv("TMPDIR");
+  directory=getenv("MAGICK_TMPDIR");
+  if (directory == (char *) NULL)
+    directory=getenv("TMPDIR");
 #if defined(__WINDOWS__)
   if (directory == (char *) NULL)
     directory=getenv("TMP");
@@ -342,7 +345,7 @@ MagickExport int AcquireUniqueFileResource(char *path)
     key[8];
 
   assert(path != (char *) NULL);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   file=(-1);
   for (i=0; i < TMP_MAX; i++)
   {
@@ -371,7 +374,7 @@ MagickExport int AcquireUniqueFileResource(char *path)
     return(file);
   if (AcquireMagickResource(FileResource,1) == MagickFalse)
     {
-      (void) close(file);
+      file=close(file)-1;
       (void) remove(path);
       return(-1);
     }
@@ -402,7 +405,7 @@ MagickExport int AcquireUniqueFileResource(char *path)
 */
 MagickExport void DestroyMagickResources(void)
 {
-  AcquireSemaphoreInfo(&resource_semaphore);
+  resource_semaphore=AcquireSemaphoreInfo(resource_semaphore);
   if (temporary_resources != (LinkedListInfo *) NULL)
     {
       const char
@@ -418,11 +421,11 @@ MagickExport void DestroyMagickResources(void)
         (void) remove(path);
         path=(const char *) GetNextElementInLinkedList(temporary_resources);
       }
-      DestroyLinkedList(temporary_resources,RelinquishMagickMemory);
-      temporary_resources=(LinkedListInfo *) NULL;
+      temporary_resources=DestroyLinkedList(temporary_resources,
+        RelinquishMagickMemory);
     }
-  RelinquishSemaphoreInfo(&resource_semaphore);
-  DestroySemaphoreInfo(&resource_semaphore);
+  RelinquishSemaphoreInfo(resource_semaphore);
+  resource_semaphore=DestroySemaphoreInfo(resource_semaphore);
 }
 
 /*
@@ -454,7 +457,7 @@ MagickExport unsigned long GetMagickResource(const ResourceType type)
     resource;
 
   resource=0;
-  AcquireSemaphoreInfo(&resource_semaphore);
+  resource_semaphore=AcquireSemaphoreInfo(resource_semaphore);
   switch (type)
   {
     case AreaResource:
@@ -485,7 +488,7 @@ MagickExport unsigned long GetMagickResource(const ResourceType type)
     default:
       break;
   }
-  RelinquishSemaphoreInfo(&resource_semaphore);
+  RelinquishSemaphoreInfo(resource_semaphore);
   return(resource);
 }
 
@@ -542,7 +545,7 @@ MagickExport void InitializeMagickResources(void)
 #if defined(PixelCacheThreshold)
   memory=PixelCacheThreshold;
 #endif
-  (void) SetMagickResourceLimit(AreaResource,memory/sizeof(PixelPacket));
+  (void) SetMagickResourceLimit(AreaResource,memory/2);
   (void) SetMagickResourceLimit(MemoryResource,memory);
   (void) SetMagickResourceLimit(MapResource,2*memory);
   p=getenv("MAGICK_AREA_LIMIT");
@@ -609,7 +612,7 @@ MagickExport MagickBooleanType ListMagickResourceInfo(FILE *file,
 
   if (file == (const FILE *) NULL)
     file=stdout;
-  AcquireSemaphoreInfo(&resource_semaphore);
+  resource_semaphore=AcquireSemaphoreInfo(resource_semaphore);
   FormatSize(MegabytesToBytes(resource_info.area_limit),area_limit);
   FormatSize(GigabytesToBytes(resource_info.disk_limit),disk_limit);
   FormatSize((MagickSizeType) resource_info.file_limit,file_limit);
@@ -620,7 +623,7 @@ MagickExport MagickBooleanType ListMagickResourceInfo(FILE *file,
   (void) fprintf(file,"%4s %10s %10s %10s %10s\n",file_limit,area_limit,
     memory_limit,map_limit,disk_limit);
   (void) fflush(file);
-  RelinquishSemaphoreInfo(&resource_semaphore);
+  RelinquishSemaphoreInfo(resource_semaphore);
   return(MagickTrue);
 }
 
@@ -660,7 +663,7 @@ MagickExport void RelinquishMagickResource(const ResourceType type,
     resource_type[MaxTextExtent];
 
   FormatSize(size,resource_request);
-  AcquireSemaphoreInfo(&resource_semaphore);
+  resource_semaphore=AcquireSemaphoreInfo(resource_semaphore);
   switch (type)
   {
     case AreaResource:
@@ -706,7 +709,7 @@ MagickExport void RelinquishMagickResource(const ResourceType type,
     default:
       break;
   }
-  RelinquishSemaphoreInfo(&resource_semaphore);
+  RelinquishSemaphoreInfo(resource_semaphore);
   (void) LogMagickEvent(ResourceEvent,GetMagickModule(),"%s: %s/%s/%s",
     resource_type,resource_request,resource_current,resource_limit);
 }
@@ -801,7 +804,7 @@ MagickExport MagickBooleanType RelinquishUniqueFileResource(const char *path)
 MagickExport MagickBooleanType SetMagickResourceLimit(const ResourceType type,
   const unsigned long limit)
 {
-  AcquireSemaphoreInfo(&resource_semaphore);
+  resource_semaphore=AcquireSemaphoreInfo(resource_semaphore);
   switch (type)
   {
     case AreaResource:
@@ -832,6 +835,6 @@ MagickExport MagickBooleanType SetMagickResourceLimit(const ResourceType type,
     default:
       break;
   }
-  RelinquishSemaphoreInfo(&resource_semaphore);
+  RelinquishSemaphoreInfo(resource_semaphore);
   return(MagickTrue);
 }
