@@ -17,7 +17,7 @@
 %                                April 1993                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -85,7 +85,8 @@
 #include "magick/studio.h"
 #include "magick/color.h"
 #include "magick/colorspace.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/image.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
@@ -306,12 +307,13 @@ static MagickBooleanType Classify(Image *image,short **extrema,
         */
         if (head != (Cluster *) NULL)
           {
-            cluster->next=(Cluster *) AcquireMagickMemory(sizeof(Cluster));
+            cluster->next=(Cluster *)
+              AcquireMagickMemory(sizeof(*cluster->next));
             cluster=cluster->next;
           }
         else
           {
-            cluster=(Cluster *) AcquireMagickMemory(sizeof(Cluster));
+            cluster=(Cluster *) AcquireMagickMemory(sizeof(*cluster));
             head=cluster;
           }
         if (cluster == (Cluster *) NULL)
@@ -333,7 +335,7 @@ static MagickBooleanType Classify(Image *image,short **extrema,
       /*
         No classes were identified-- create one.
       */
-      cluster=(Cluster *) AcquireMagickMemory(sizeof(Cluster));
+      cluster=(Cluster *) AcquireMagickMemory(sizeof(*cluster));
       if (cluster == (Cluster *) NULL)
         ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
           image->filename);
@@ -387,9 +389,11 @@ static MagickBooleanType Classify(Image *image,short **extrema,
           }
       p++;
     }
-    if (QuantumTick(y,image->rows) != 0)
+    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+        (QuantumTick(y,2*image->rows) != MagickFalse))
       {
-        status=MagickMonitor(SegmentImageTag,y,2*image->rows,&image->exception);
+        status=image->progress_monitor(SegmentImageTag,y,2*image->rows,
+          image->client_data);
         if (status == MagickFalse)
           break;
       }
@@ -404,7 +408,7 @@ static MagickBooleanType Classify(Image *image,short **extrema,
   {
     next_cluster=cluster->next;
     if ((cluster->count > 0) &&
-        (cluster->count >= (count*cluster_threshold/100.0)))
+        ((MagickRealType) cluster->count >= (count*cluster_threshold/100.0)))
       {
         /*
           Initialize cluster.
@@ -476,7 +480,7 @@ static MagickBooleanType Classify(Image *image,short **extrema,
   /*
     Speed up distance calculations.
   */
-  squares=(MagickRealType *) AcquireMagickMemory(513*sizeof(MagickRealType));
+  squares=(MagickRealType *) AcquireMagickMemory(513*sizeof(*squares));
   if (squares == (MagickRealType *) NULL)
     ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
       image->filename);
@@ -578,10 +582,11 @@ static MagickBooleanType Classify(Image *image,short **extrema,
     }
     if (SyncImagePixels(image) == MagickFalse)
       break;
-    if (QuantumTick(y,image->rows) != 0)
+    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+        (QuantumTick(y,2*image->rows) != MagickFalse))
       {
-        status=MagickMonitor(SegmentImageTag,y+image->rows,2*image->rows,
-          &image->exception);
+        status=image->progress_monitor(SegmentImageTag,y,2*image->rows,
+          image->client_data);
         if (status == MagickFalse)
           break;
       }
@@ -881,6 +886,9 @@ MagickExport MagickPixelPacket GetImageDynamicThreshold(const Image *image,
     *histogram[MaxDimension],
     y;
 
+  MagickBooleanType
+    status;
+
   MagickPixelPacket
     pixel;
 
@@ -904,11 +912,11 @@ MagickExport MagickPixelPacket GetImageDynamicThreshold(const Image *image,
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
-  (void) ResetMagickMemory(&pixel,0,sizeof(PixelPacket));
+  (void) ResetMagickMemory(&pixel,0,sizeof(pixel));
   for (i=0; i < MaxDimension; i++)
   {
-    histogram[i]=(long *) AcquireMagickMemory(256*sizeof(long));
-    extrema[i]=(short *) AcquireMagickMemory(256*sizeof(short));
+    histogram[i]=(long *) AcquireMagickMemory(256*sizeof(**histogram));
+    extrema[i]=(short *) AcquireMagickMemory(256*sizeof(**histogram));
     if ((histogram[i] == (long *) NULL) || (extrema[i] == (short *) NULL))
       {
         for (i-- ; i >= 0; i--)
@@ -950,12 +958,13 @@ MagickExport MagickPixelPacket GetImageDynamicThreshold(const Image *image,
         */
         if (head != (Cluster *) NULL)
           {
-            cluster->next=(Cluster *) AcquireMagickMemory(sizeof(Cluster));
+            cluster->next=(Cluster *)
+              AcquireMagickMemory(sizeof(*cluster->next));
             cluster=cluster->next;
           }
         else
           {
-            cluster=(Cluster *) AcquireMagickMemory(sizeof(Cluster));
+            cluster=(Cluster *) AcquireMagickMemory(sizeof(*cluster));
             head=cluster;
           }
         if (cluster == (Cluster *) NULL)
@@ -980,7 +989,7 @@ MagickExport MagickPixelPacket GetImageDynamicThreshold(const Image *image,
       /*
         No classes were identified-- create one.
       */
-      cluster=(Cluster *) AcquireMagickMemory(sizeof(Cluster));
+      cluster=(Cluster *) AcquireMagickMemory(sizeof(*cluster));
       if (cluster == (Cluster *) NULL)
         {
           (void) ThrowMagickException(exception,GetMagickModule(),
@@ -1037,9 +1046,14 @@ MagickExport MagickPixelPacket GetImageDynamicThreshold(const Image *image,
           }
       p++;
     }
-    if (QuantumTick(y,image->rows) != 0)
-      if (MagickMonitor(SegmentImageTag,y,2*image->rows,exception) == MagickFalse)
-        break;
+    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+        (QuantumTick(y,2*image->rows) != MagickFalse))
+      {
+        status=image->progress_monitor(SegmentImageTag,y,2*image->rows,
+          image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
   }
   /*
     Remove clusters that do not meet minimum cluster threshold.
@@ -1281,13 +1295,13 @@ static IntervalTree *InitializeIntervalTree(const ZeroCrossing *zero_crossing,
   /*
     Allocate interval tree.
   */
-  list=(IntervalTree **) AcquireMagickMemory(TreeLength*sizeof(IntervalTree *));
+  list=(IntervalTree **) AcquireMagickMemory(TreeLength*sizeof(*list));
   if (list == (IntervalTree **) NULL)
     return((IntervalTree *) NULL);
   /*
     The root is the entire histogram.
   */
-  root=(IntervalTree *) AcquireMagickMemory(sizeof(IntervalTree));
+  root=(IntervalTree *) AcquireMagickMemory(sizeof(*root));
   root->child=(IntervalTree *) NULL;
   root->sibling=(IntervalTree *) NULL;
   root->tau=0.0;
@@ -1315,13 +1329,13 @@ static IntervalTree *InitializeIntervalTree(const ZeroCrossing *zero_crossing,
             if (node == head)
               {
                 node->child=(IntervalTree *)
-                  AcquireMagickMemory(sizeof(IntervalTree));
+                  AcquireMagickMemory(sizeof(*node->child));
                 node=node->child;
               }
             else
               {
                 node->sibling=(IntervalTree *)
-                  AcquireMagickMemory(sizeof(IntervalTree));
+                  AcquireMagickMemory(sizeof(*node->sibling));
                 node=node->sibling;
               }
             node->tau=zero_crossing[i+1].tau;
@@ -1334,7 +1348,8 @@ static IntervalTree *InitializeIntervalTree(const ZeroCrossing *zero_crossing,
         }
       if (left != head->left)
         {
-          node->sibling=(IntervalTree *) AcquireMagickMemory(sizeof(IntervalTree));
+          node->sibling=(IntervalTree *)
+            AcquireMagickMemory(sizeof(*node->sibling));
           node=node->sibling;
           node->tau=zero_crossing[i+1].tau;
           node->child=(IntervalTree *) NULL;
@@ -1449,7 +1464,7 @@ static MagickRealType OptimalTau(const long *histogram,const double max_tau,
   /*
     Allocate interval tree.
   */
-  list=(IntervalTree **) AcquireMagickMemory(TreeLength*sizeof(IntervalTree *));
+  list=(IntervalTree **) AcquireMagickMemory(TreeLength*sizeof(*list));
   if (list == (IntervalTree **) NULL)
     return(0.0);
   /*
@@ -1457,7 +1472,7 @@ static MagickRealType OptimalTau(const long *histogram,const double max_tau,
   */
   count=(unsigned long) ((max_tau-min_tau)/delta_tau)+2;
   zero_crossing=(ZeroCrossing *)
-    AcquireMagickMemory((size_t) count*sizeof(ZeroCrossing));
+    AcquireMagickMemory((size_t) count*sizeof(*zero_crossing));
   if (zero_crossing == (ZeroCrossing *) NULL)
     return(0.0);
   for (i=0; i < (long) count; i++)
@@ -1465,10 +1480,9 @@ static MagickRealType OptimalTau(const long *histogram,const double max_tau,
   /*
     Initialize zero crossing list.
   */
-  derivative=(MagickRealType *)
-    AcquireMagickMemory(256*sizeof(MagickRealType));
+  derivative=(MagickRealType *) AcquireMagickMemory(256*sizeof(*derivative));
   second_derivative=(MagickRealType *)
-    AcquireMagickMemory(256*sizeof(MagickRealType));
+    AcquireMagickMemory(256*sizeof(*second_derivative));
   if ((derivative == (MagickRealType *) NULL) ||
       (second_derivative == (MagickRealType *) NULL))
     ThrowMagickFatalException(ResourceLimitFatalError,
@@ -1630,7 +1644,7 @@ static void ScaleSpace(const long *histogram,const MagickRealType tau,
     u,
     x;
 
-  gamma=(MagickRealType *) AcquireMagickMemory(256*sizeof(MagickRealType));
+  gamma=(MagickRealType *) AcquireMagickMemory(256*sizeof(*gamma));
   if (gamma == (MagickRealType *) NULL)
     ThrowMagickFatalException(ResourceLimitFatalError,
       "UnableToAllocateGammaMap","ScaleSpace");
@@ -1719,8 +1733,8 @@ MagickExport MagickBooleanType SegmentImage(Image *image,
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   for (i=0; i < MaxDimension; i++)
   {
-    histogram[i]=(long *) AcquireMagickMemory(256*sizeof(long));
-    extrema[i]=(short *) AcquireMagickMemory(256*sizeof(short));
+    histogram[i]=(long *) AcquireMagickMemory(256*sizeof(**histogram));
+    extrema[i]=(short *) AcquireMagickMemory(256*sizeof(**extrema));
     if ((histogram[i] == (long *) NULL) || (extrema[i] == (short *) NULL))
       {
         for (i-- ; i >= 0; i--)
