@@ -2,6 +2,7 @@
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
 %                                                                             %
+%                                                                             %
 %        SSSSS  EEEEE  M   M   AAA   PPPP   H   H   OOO   RRRR   EEEEE        %
 %        SS     E      MM MM  A   A  P   P  H   H  O   O  R   R  E            %
 %         SSS   EEE    M M M  AAAAA  PPPP   HHHHH  O   O  RRRR   EEE          %
@@ -17,31 +18,19 @@
 %                                 June 2000                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -53,13 +42,15 @@
   Include declarations.
 */
 #include "magick/studio.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
+#include "magick/memory_.h"
 #include "magick/semaphore.h"
-#include "magick/utility.h"
-#if defined(HasPTHREADS)
+#include "magick/string_.h"
+#if defined(HAVE_PTHREAD)
 #include <pthread.h>
 #endif
-#if defined(WIN32)
+#if defined(__WINDOWS__)
 #include <windows.h>
 #endif
 
@@ -68,14 +59,14 @@
 */
 struct SemaphoreInfo
 {
-#if defined(HasPTHREADS)
+#if defined(HAVE_PTHREAD)
   pthread_mutex_t
     mutex;
 
   pthread_t
     id;
 #endif
-#if defined(WIN32)
+#if defined(__WINDOWS__)
   CRITICAL_SECTION
     mutex;
 
@@ -83,7 +74,7 @@ struct SemaphoreInfo
     id;
 #endif
 
-  unsigned int
+  MagickBooleanType
     lock;
 
   unsigned long
@@ -93,17 +84,14 @@ struct SemaphoreInfo
 /*
   Static declaractions.
 */
-#if defined(HasPTHREADS)
+#if defined(HAVE_PTHREAD)
 static pthread_mutex_t
   semaphore_mutex = PTHREAD_MUTEX_INITIALIZER;
 #endif
 
-#if defined(WIN32)
-static CRITICAL_SECTION
-  semaphore_mutex;
-
-static unsigned int
-  active_semaphore = False;
+#if defined(__WINDOWS__)
+static int
+  semaphore_mutex = 0;
 #endif
 
 /*
@@ -121,7 +109,7 @@ static unsigned int
 %
 %  The format of the AcquireSemaphoreInfo method is:
 %
-%      AcquireSemaphoreInfo(SemaphoreInfo **semaphore_info)
+%      SemaphoreInfo *AcquireSemaphoreInfo(SemaphoreInfo *semaphore_info)
 %
 %  A description of each parameter follows:
 %
@@ -129,27 +117,25 @@ static unsigned int
 %
 %
 */
-MagickExport void AcquireSemaphoreInfo(SemaphoreInfo **semaphore_info)
+MagickExport SemaphoreInfo *AcquireSemaphoreInfo(SemaphoreInfo *semaphore_info)
 {
-  assert(semaphore_info != (SemaphoreInfo **) NULL);
-#if defined(HasPTHREADS)
+#if defined(HAVE_PTHREAD)
   (void) pthread_mutex_lock(&semaphore_mutex);
 #endif
-#if defined(WIN32)
-  if (!active_semaphore)
-    InitializeCriticalSection(&semaphore_mutex);
-  active_semaphore=True;
-  EnterCriticalSection(&semaphore_mutex);
+#if defined(__WINDOWS__)
+  while (InterlockedCompareExchange(&semaphore_mutex,1L,0L) != 0)
+    Sleep(0);
 #endif
-  if (*semaphore_info == (SemaphoreInfo *) NULL)
-    *semaphore_info=AllocateSemaphoreInfo();
-#if defined(HasPTHREADS)
+  if (semaphore_info == (SemaphoreInfo *) NULL)
+    semaphore_info=AllocateSemaphoreInfo();
+#if defined(HAVE_PTHREAD)
   (void) pthread_mutex_unlock(&semaphore_mutex);
 #endif
-#if defined(WIN32)
-  LeaveCriticalSection(&semaphore_mutex);
+#if defined(__WINDOWS__)
+  InterlockedExchange(&semaphore_mutex,0L);
 #endif
-  (void) LockSemaphoreInfo(*semaphore_info);
+  (void) LockSemaphoreInfo(semaphore_info);
+  return(semaphore_info);
 }
 
 /*
@@ -186,13 +172,13 @@ MagickExport SemaphoreInfo *AllocateSemaphoreInfo(void)
   */
   semaphore_info=(SemaphoreInfo *) malloc(sizeof(SemaphoreInfo));
   if (semaphore_info == (SemaphoreInfo *) NULL)
-    MagickFatalError(ResourceLimitFatalError,"MemoryAllocationFailed",
-      "UnableToAllocateSemaphoreInfo");
-  memset(semaphore_info,0,sizeof(SemaphoreInfo));
+    ThrowMagickFatalException(ResourceLimitFatalError,
+      "UnableToAllocateSemaphoreInfo",strerror(errno));
+  (void) ResetMagickMemory(semaphore_info,0,sizeof(SemaphoreInfo));
   /*
     Initialize the semaphore.
   */
-#if defined(HasPTHREADS)
+#if defined(HAVE_PTHREAD)
   {
     int
       status;
@@ -206,10 +192,10 @@ MagickExport SemaphoreInfo *AllocateSemaphoreInfo(void)
       }
   }
 #endif
-#if defined(WIN32)
+#if defined(__WINDOWS__)
   InitializeCriticalSection(&semaphore_info->mutex);
 #endif
-  semaphore_info->lock=False;
+  semaphore_info->lock=MagickFalse;
   semaphore_info->signature=MagickSignature;
   return(semaphore_info);
 }
@@ -235,14 +221,9 @@ MagickExport SemaphoreInfo *AllocateSemaphoreInfo(void)
 */
 MagickExport void DestroySemaphore(void)
 {
-#if defined(HasPTHREADS)
+#if defined(HAVE_PTHREAD)
   (void) pthread_mutex_destroy(&semaphore_mutex);
 #endif
-#if defined(WIN32)
-  if (active_semaphore)
-    DeleteCriticalSection(&semaphore_mutex);
-  active_semaphore=False;
-#endif
 }
 
 /*
@@ -260,7 +241,7 @@ MagickExport void DestroySemaphore(void)
 %
 %  The format of the DestroySemaphoreInfo method is:
 %
-%      DestroySemaphoreInfo(SemaphoreInfo **semaphore_info)
+%      SemaphoreInfo *DestroySemaphoreInfo(SemaphoreInfo *semaphore_info)
 %
 %  A description of each parameter follows:
 %
@@ -268,36 +249,33 @@ MagickExport void DestroySemaphore(void)
 %
 %
 */
-MagickExport void DestroySemaphoreInfo(SemaphoreInfo **semaphore_info)
+MagickExport SemaphoreInfo *DestroySemaphoreInfo(SemaphoreInfo *semaphore_info)
 {
-  assert(semaphore_info != (SemaphoreInfo **) NULL);
-  if (*semaphore_info == (SemaphoreInfo *) NULL)
-    return;
-  assert((*semaphore_info)->signature == MagickSignature);
-  (void) UnlockSemaphoreInfo(*semaphore_info);
-#if defined(HasPTHREADS)
+  assert(semaphore_info != (SemaphoreInfo *) NULL);
+  assert(semaphore_info->signature == MagickSignature);
+#if defined(HAVE_PTHREAD)
   (void) pthread_mutex_lock(&semaphore_mutex);
 #endif
-#if defined(WIN32)
-  if (!active_semaphore)
-    InitializeCriticalSection(&semaphore_mutex);
-  active_semaphore=True;
-  EnterCriticalSection(&semaphore_mutex);
+#if defined(__WINDOWS__)
+  while (InterlockedCompareExchange(&semaphore_mutex,1L,0L) != 0)
+    Sleep(0);
 #endif
-#if defined(HasPTHREADS)
-  (void) pthread_mutex_destroy(&(*semaphore_info)->mutex);
+#if defined(HAVE_PTHREAD)
+  (void) pthread_mutex_destroy(&semaphore_info->mutex);
 #endif
-#if defined(WIN32)
-  DeleteCriticalSection(&(*semaphore_info)->mutex);
+#if defined(__WINDOWS__)
+  DeleteCriticalSection(&semaphore_info->mutex);
 #endif
-  free(*semaphore_info);
-  *semaphore_info=(SemaphoreInfo *) NULL;
-#if defined(HasPTHREADS)
+  semaphore_info->signature=(~MagickSignature);
+  free(semaphore_info);
+  semaphore_info=(SemaphoreInfo *) NULL;
+#if defined(HAVE_PTHREAD)
   (void) pthread_mutex_unlock(&semaphore_mutex);
 #endif
-#if defined(WIN32)
-  LeaveCriticalSection(&semaphore_mutex);
+#if defined(__WINDOWS__)
+  InterlockedExchange(&semaphore_mutex,0L);
 #endif
+  return(semaphore_info);
 }
 
 /*
@@ -321,47 +299,10 @@ MagickExport void DestroySemaphoreInfo(SemaphoreInfo **semaphore_info)
 */
 MagickExport void InitializeSemaphore(void)
 {
-#if defined(HasPTHREADS)
+#if defined(HAVE_PTHREAD)
   (void) pthread_mutex_init(&semaphore_mutex,
     (const pthread_mutexattr_t *) NULL);
 #endif
-#if defined(WIN32)
-  if (!active_semaphore)
-    InitializeCriticalSection(&semaphore_mutex);
-  active_semaphore=True;
-#endif
-}
-
-/*
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%   L i b e r a t e S e m a p h o r e I n f o                                 %
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%
-%  LiberateSemaphoreInfo() liberates a semaphore.
-%
-%  The format of the LiberateSemaphoreInfo method is:
-%
-%      LiberateSemaphoreInfo(SemaphoreInfo **semaphore_info)
-%
-%  A description of each parameter follows:
-%
-%    o semaphore_info: Specifies a pointer to an SemaphoreInfo structure.
-%
-%
-*/
-MagickExport void LiberateSemaphoreInfo(SemaphoreInfo **semaphore_info)
-{
-  assert(semaphore_info != (SemaphoreInfo **) NULL);
-  if (*semaphore_info == (SemaphoreInfo *) NULL)
-    return;
-  assert((*semaphore_info)->signature == MagickSignature);
-  (void) UnlockSemaphoreInfo(*semaphore_info);
 }
 
 /*
@@ -379,32 +320,59 @@ MagickExport void LiberateSemaphoreInfo(SemaphoreInfo **semaphore_info)
 %
 %  The format of the LockSemaphoreInfo method is:
 %
-%      unsigned int LockSemaphoreInfo(SemaphoreInfo *semaphore_info)
+%      MagickBooleanType LockSemaphoreInfo(SemaphoreInfo *semaphore_info)
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method LockSemaphoreInfo returns True on success otherwise
-%      False.
-%
 %    o semaphore_info: Specifies a pointer to an SemaphoreInfo structure.
 %
 %
 */
-MagickExport unsigned int LockSemaphoreInfo(SemaphoreInfo *semaphore_info)
+MagickExport MagickBooleanType LockSemaphoreInfo(SemaphoreInfo *semaphore_info)
 {
   assert(semaphore_info != (SemaphoreInfo *) NULL);
   assert(semaphore_info->signature == MagickSignature);
-#if defined(HasPTHREADS)
+#if defined(HAVE_PTHREAD)
   if (pthread_mutex_lock(&semaphore_info->mutex))
-    return(False);
+    return(MagickFalse);
   semaphore_info->id=pthread_self();
 #endif
-#if defined(WIN32)
+#if defined(__WINDOWS__)
   EnterCriticalSection(&semaphore_info->mutex);
   semaphore_info->id=GetCurrentThreadId();
 #endif
-  semaphore_info->lock=True;
-  return(True);
+  semaphore_info->lock=MagickTrue;
+  return(MagickTrue);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   R e l i n g u i s h S e m a p h o r e I n f o                             %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  RelinquishSemaphoreInfo() relinquishes a semaphore.
+%
+%  The format of the RelinquishSemaphoreInfo method is:
+%
+%      RelinquishSemaphoreInfo(SemaphoreInfo *semaphore_info)
+%
+%  A description of each parameter follows:
+%
+%    o semaphore_info: Specifies a pointer to an SemaphoreInfo structure.
+%
+%
+*/
+MagickExport void RelinquishSemaphoreInfo(SemaphoreInfo *semaphore_info)
+{
+  assert(semaphore_info != (SemaphoreInfo *) NULL);
+  assert(semaphore_info->signature == MagickSignature);
+  (void) UnlockSemaphoreInfo(semaphore_info);
 }
 
 /*
@@ -422,31 +390,30 @@ MagickExport unsigned int LockSemaphoreInfo(SemaphoreInfo *semaphore_info)
 %
 %  The format of the LockSemaphoreInfo method is:
 %
-%      unsigned int UnlockSemaphoreInfo(SemaphoreInfo *semaphore_info)
+%      MagickBooleanType UnlockSemaphoreInfo(SemaphoreInfo *semaphore_info)
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method UnlockSemaphoreInfo returns True on success otherwise
-%      False.
-%
 %    o semaphore_info: Specifies a pointer to an SemaphoreInfo structure.
 %
 %
 */
-MagickExport unsigned int UnlockSemaphoreInfo(SemaphoreInfo *semaphore_info)
+MagickExport MagickBooleanType UnlockSemaphoreInfo(
+  SemaphoreInfo *semaphore_info)
 {
   assert(semaphore_info != (SemaphoreInfo *) NULL);
   assert(semaphore_info->signature == MagickSignature);
-  if (semaphore_info->lock != True)
-    return(False);
-  semaphore_info->lock=False;
-#if defined(HasPTHREADS)
+  if (semaphore_info->lock == MagickFalse)
+    return(MagickFalse);
+  semaphore_info->lock=MagickFalse;
+#if defined(HAVE_PTHREAD)
+  assert(pthread_equal(semaphore_info->id,pthread_self()));
   if (pthread_mutex_unlock(&semaphore_info->mutex))
-    return(False);
+    return(MagickFalse);
 #endif
-#if defined(WIN32)
+#if defined(__WINDOWS__)
   assert(GetCurrentThreadId() == semaphore_info->id);
   LeaveCriticalSection(&semaphore_info->mutex);
 #endif
-  return(True);
+  return(MagickTrue);
 }
