@@ -17,7 +17,7 @@
 %                                  July 1992                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -50,7 +50,8 @@
 #include "magick/composite_private.h"
 #include "magick/decorate.h"
 #include "magick/draw.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/image.h"
 #include "magick/memory_.h"
 #include "magick/list.h"
@@ -96,17 +97,14 @@ MagickExport Image *AffineTransformImage(const Image *image,
   Image
     *affine_image;
 
-  long
-    y;
-
   PointInfo
     extent[4],
     min,
-    max;
+    max,
+    point;
 
   register long
-    i,
-    x;
+    i;
 
   /*
     Determine bounding box.
@@ -128,10 +126,9 @@ MagickExport Image *AffineTransformImage(const Image *image,
   extent[3].y=(double) image->rows;
   for (i=0; i < 4; i++)
   {
-    x=(long) (extent[i].x+0.5);
-    y=(long) (extent[i].y+0.5);
-    extent[i].x=(double) (x*affine->sx+y*affine->ry+affine->tx);
-    extent[i].y=(double) (x*affine->rx+y*affine->sy+affine->ty);
+    point=extent[i];
+    extent[i].x=(double) (point.x*affine->sx+point.y*affine->ry+affine->tx);
+    extent[i].y=(double) (point.x*affine->rx+point.y*affine->sy+affine->ty);
   }
   min=extent[0];
   max=extent[0];
@@ -158,8 +155,8 @@ MagickExport Image *AffineTransformImage(const Image *image,
   transform.rx=affine->rx;
   transform.ry=affine->ry;
   transform.sy=affine->sy;
-  transform.tx=(-min.x);
-  transform.ty=(-min.y);
+  transform.tx=min.x;
+  transform.ty=min.y;
   (void) DrawAffineImage(affine_image,image,&transform);
   return(affine_image);
 }
@@ -217,14 +214,14 @@ static inline void CropToFitImage(Image **image,const MagickRealType x_shear,
   /*
     Calculate the rotated image size.
   */
-  extent[0].x=(-width/2.0);
-  extent[0].y=(-height/2.0);
-  extent[1].x=width/2.0;
-  extent[1].y=(-height/2.0);
-  extent[2].x=(-width/2.0);
-  extent[2].y=height/2.0;
-  extent[3].x=width/2.0;
-  extent[3].y=height/2.0;
+  extent[0].x=(double) (-width/2.0);
+  extent[0].y=(double) (-height/2.0);
+  extent[1].x=(double) width/2.0;
+  extent[1].y=(double) (-height/2.0);
+  extent[2].x=(double) (-width/2.0);
+  extent[2].y=(double) height/2.0;
+  extent[3].x=(double) width/2.0;
+  extent[3].y=(double) height/2.0;
   for (i=0; i < 4; i++)
   {
     extent[i].x+=x_shear*extent[i].y;
@@ -255,7 +252,7 @@ static inline void CropToFitImage(Image **image,const MagickRealType x_shear,
   if (crop_image != (Image *) NULL)
     {
       crop_image->page=(*image)->page;
-      DestroyImage(*image);
+      *image=DestroyImage(*image);
       *image=crop_image;
     }
 }
@@ -357,9 +354,11 @@ static Image *IntegralRotateImage(const Image *image,unsigned long rotations,
             image->columns*sizeof(*rotate_indexes));
         if (SyncImagePixels(rotate_image) == MagickFalse)
           break;
-        if (QuantumTick(y,image->rows) != MagickFalse)
+        if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+            (QuantumTick(y,image->rows) != MagickFalse))
           {
-            status=MagickMonitor(RotateImageTag,y,image->rows,exception);
+            status=image->progress_monitor(RotateImageTag,y,image->rows,
+              image->client_data);
             if (status == MagickFalse)
               break;
           }
@@ -387,16 +386,19 @@ static Image *IntegralRotateImage(const Image *image,unsigned long rotations,
             image->columns*sizeof(*rotate_indexes));
         if (SyncImagePixels(rotate_image) == MagickFalse)
           break;
-        if (QuantumTick(y,image->rows) != MagickFalse)
+        if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+            (QuantumTick(y,image->rows) != MagickFalse))
           {
-            status=MagickMonitor(RotateImageTag,y,image->rows,exception);
+            status=image->progress_monitor(RotateImageTag,y,image->rows,
+              image->client_data);
             if (status == MagickFalse)
               break;
           }
       }
       Swap(page.width,page.height);
       Swap(page.x,page.y);
-      page.x=(long) (page.width-rotate_image->columns-page.x);
+      if (page.width != 0)
+        page.x=(long) (page.width-rotate_image->columns-page.x);
       break;
     }
     case 2:
@@ -422,15 +424,19 @@ static Image *IntegralRotateImage(const Image *image,unsigned long rotations,
           *--q=(*p++);
         if (SyncImagePixels(rotate_image) == MagickFalse)
           break;
-        if (QuantumTick(y,image->rows) != MagickFalse)
+        if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+            (QuantumTick(y,image->rows) != MagickFalse))
           {
-            status=MagickMonitor(RotateImageTag,y,image->rows,exception);
+            status=image->progress_monitor(RotateImageTag,y,image->rows,
+              image->client_data);
             if (status == MagickFalse)
               break;
           }
       }
-      page.x=(long) (page.width-rotate_image->columns-page.x);
-      page.y=(long) (page.height-rotate_image->rows-page.y);
+      if (page.width != 0)
+        page.x=(long) (page.width-rotate_image->columns-page.x);
+      if (page.height != 0)
+        page.y=(long) (page.height-rotate_image->rows-page.y);
       break;
     }
     case 3:
@@ -455,16 +461,19 @@ static Image *IntegralRotateImage(const Image *image,unsigned long rotations,
             rotate_indexes[image->columns-x-1]=indexes[x];
         if (SyncImagePixels(rotate_image) == MagickFalse)
           break;
-        if (QuantumTick(y,image->rows) != MagickFalse)
+        if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+            (QuantumTick(y,image->rows) != MagickFalse))
           {
-            status=MagickMonitor(RotateImageTag,y,image->rows,exception);
+            status=image->progress_monitor(RotateImageTag,y,image->rows,
+              image->client_data);
             if (status == MagickFalse)
               break;
           }
       }
       Swap(page.width,page.height);
       Swap(page.x,page.y);
-      page.y=(long) (page.height-rotate_image->rows-page.y);
+      if (page.height != 0)
+        page.y=(long) (page.height-rotate_image->rows-page.y);
       break;
     }
   }
@@ -554,7 +563,7 @@ static inline void XShearImage(Image *image,const MagickRealType degrees,
         displacement*=(-1.0);
         direction=LEFT;
       }
-    step=(long) floor(displacement);
+    step=(long) floor((double) displacement);
     opacity=MaxRGB*(MagickRealType) (displacement-step);
     step++;
     pixel=image->background_color;
@@ -621,9 +630,11 @@ static inline void XShearImage(Image *image,const MagickRealType degrees,
     }
     if (SyncImagePixels(image) == MagickFalse)
       break;
-    if (QuantumTick(y,height) != MagickFalse)
+    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+        (QuantumTick(y,height) != MagickFalse))
       {
-        status=MagickMonitor(XShearImageTag,y,height,&image->exception);
+        status=image->progress_monitor(XShearImageTag,y,height,
+          image->client_data);
         if (status == MagickFalse)
           break;
       }
@@ -710,7 +721,7 @@ static inline void YShearImage(Image *image,const MagickRealType degrees,
         displacement*=(-1.0);
         direction=UP;
       }
-    step=(long) floor(displacement);
+    step=(long) floor((double) displacement);
     opacity=MaxRGB*(MagickRealType) (displacement-step);
     step++;
     pixel=image->background_color;
@@ -777,9 +788,11 @@ static inline void YShearImage(Image *image,const MagickRealType degrees,
     }
     if (SyncImagePixels(image) == MagickFalse)
       break;
-    if (QuantumTick(y,width) != MagickFalse)
+    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+        (QuantumTick(y,width) != MagickFalse))
       {
-        status=MagickMonitor(YShearImageTag,y,width,&image->exception);
+        status=image->progress_monitor(XShearImageTag,y,width,
+          image->client_data);
         if (status == MagickFalse)
           break;
       }
@@ -873,8 +886,8 @@ MagickExport Image *RotateImage(const Image *image,const double degrees,
   integral_image=IntegralRotateImage(image,rotations,exception);
   if (integral_image == (Image *) NULL)
     ThrowImageException(ResourceLimitError,"MemoryAllocationFailed");
-  shear.x=(-tan(DegreesToRadians(angle)/2.0));
-  shear.y=sin(DegreesToRadians(angle));
+  shear.x=(-tan((double) DegreesToRadians(angle)/2.0));
+  shear.y=sin((double) DegreesToRadians(angle));
   if ((shear.x == 0.0) && (shear.y == 0.0))
     return(integral_image);
   integral_image->storage_class=DirectClass;
@@ -898,7 +911,7 @@ MagickExport Image *RotateImage(const Image *image,const double degrees,
   border_info.width=(unsigned long) x_offset;
   border_info.height=(unsigned long) y_offset;
   rotate_image=BorderImage(integral_image,&border_info,exception);
-  DestroyImage(integral_image);
+  integral_image=DestroyImage(integral_image);
   if (rotate_image == (Image *) NULL)
     ThrowImageException(ResourceLimitError,"MemoryAllocationFailed");
   /*
@@ -1014,7 +1027,7 @@ MagickExport Image *ShearImage(const Image *image,const double x_shear,
   shear_image=BorderImage(integral_image,&border_info,exception);
   if (shear_image == (Image *) NULL)
     ThrowImageException(ResourceLimitError,"MemoryAllocationFailed");
-  DestroyImage(integral_image);
+  integral_image=DestroyImage(integral_image);
   /*
     Shear the image.
   */
