@@ -16,7 +16,7 @@
 %                              December 1992                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -40,7 +40,8 @@
   Include declarations.
 */
 #include "magick/studio.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/attribute.h"
 #include "magick/image.h"
 #include "magick/memory_.h"
@@ -287,8 +288,8 @@ MagickExport MagickBooleanType SignatureImage(Image *image)
     signature_info.digest[1],signature_info.digest[2],signature_info.digest[3],
     signature_info.digest[4],signature_info.digest[5],signature_info.digest[6],
     signature_info.digest[7]);
-  (void) SetImageAttribute(image,"signature",(char *) NULL);
-  (void) SetImageAttribute(image,"signature",signature);
+  (void) SetImageAttribute(image,"Signature",(char *) NULL);
+  (void) SetImageAttribute(image,"Signature",signature);
   return(MagickTrue);
 }
 
