@@ -17,7 +17,7 @@
 %                                 March 2000                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -41,6 +41,7 @@
   Include declarations.
 */
 #include "magick/studio.h"
+#include "magick/exception_private.h"
 #include "magick/module.h"
 #include "magick/static.h"
 #include "magick/string_.h"
@@ -141,27 +142,41 @@ MagickExport void RegisterStaticModules(void)
   RegisterCINImage();
   RegisterCIPImage();
   RegisterCLIPImage();
+#if defined(HasWINGDI32)
   RegisterCLIPBOARDImage();
+#endif
   RegisterCMYKImage();
   RegisterCUTImage();
   RegisterDCMImage();
   RegisterDIBImage();
   RegisterDPSImage();
   RegisterDPXImage();
+#if defined(HasWINGDI32)
   RegisterEMFImage();
+#endif
+#if defined(HasTIFF)
   RegisterEPTImage();
+#endif
   RegisterFAXImage();
   RegisterFITSImage();
+#if defined(HasFPX)
   RegisterFPXImage();
+#endif
   RegisterGIFImage();
   RegisterGRAYImage();
   RegisterGRADIENTImage();
   RegisterHISTOGRAMImage();
   RegisterHTMLImage();
   RegisterICONImage();
+#if defined(HasJBIG)
   RegisterJBIGImage();
+#endif
+#if defined(HasJPEG)
   RegisterJPEGImage();
+#endif
+#if defined(HasJP2)
   RegisterJP2Image();
+#endif
   RegisterLABELImage();
   RegisterMAGICKImage();
   RegisterMAPImage();
@@ -188,7 +203,9 @@ MagickExport void RegisterStaticModules(void)
   RegisterPICTImage();
   RegisterPIXImage();
   RegisterPLASMAImage();
+#if defined(HasPNG)
   RegisterPNGImage();
+#endif
   RegisterPNMImage();
   RegisterPREVIEWImage();
   RegisterPSImage();
@@ -208,7 +225,9 @@ MagickExport void RegisterStaticModules(void)
   RegisterSUNImage();
   RegisterSVGImage();
   RegisterTGAImage();
+#if defined(HasTIFF)
   RegisterTIFFImage();
+#endif
   RegisterTILEImage();
   RegisterTIMImage();
   RegisterTTFImage();
@@ -222,7 +241,9 @@ MagickExport void RegisterStaticModules(void)
   RegisterWBMPImage();
   RegisterWMFImage();
   RegisterWPGImage();
+#if defined(HasX11)
   RegisterXImage();
+#endif
   RegisterXBMImage();
   RegisterXCImage();
   RegisterXCFImage();
@@ -230,7 +251,10 @@ MagickExport void RegisterStaticModules(void)
 #if defined(_VISUALC_)
   RegisterXTRNImage();
 #endif
+#if defined(HasX11)
   RegisterXWDImage();
+#endif
+  RegisterYCBCRImage();
   RegisterYUVImage();
 #endif
 }
@@ -266,27 +290,41 @@ MagickExport void UnregisterStaticModules(void)
   UnregisterCINImage();
   UnregisterCIPImage();
   UnregisterCLIPImage();
+#if defined(HasWINGDI32)
   UnregisterCLIPBOARDImage();
+#endif
   UnregisterCMYKImage();
   UnregisterCUTImage();
   UnregisterDCMImage();
   UnregisterDIBImage();
   UnregisterDPSImage();
   UnregisterDPXImage();
+#if defined(HasWINGDI32)
   UnregisterEMFImage();
+#endif
+#if defined(HasTIFF)
   UnregisterEPTImage();
+#endif
   UnregisterFAXImage();
   UnregisterFITSImage();
+#if defined(HasFPX)
   UnregisterFPXImage();
+#endif
   UnregisterGIFImage();
   UnregisterGRAYImage();
   UnregisterGRADIENTImage();
   UnregisterHISTOGRAMImage();
   UnregisterHTMLImage();
   UnregisterICONImage();
+#if defined(HasJBIG)
   UnregisterJBIGImage();
+#endif
+#if defined(HasJPEG)
   UnregisterJPEGImage();
+#endif
+#if defined(HasJP2)
   UnregisterJP2Image();
+#endif
   UnregisterLABELImage();
   UnregisterMAGICKImage();
   UnregisterMAPImage();
@@ -313,7 +351,9 @@ MagickExport void UnregisterStaticModules(void)
   UnregisterPICTImage();
   UnregisterPIXImage();
   UnregisterPLASMAImage();
+#if defined(HasPNG)
   UnregisterPNGImage();
+#endif
   UnregisterPNMImage();
   UnregisterPREVIEWImage();
   UnregisterPSImage();
@@ -333,7 +373,9 @@ MagickExport void UnregisterStaticModules(void)
   UnregisterSUNImage();
   UnregisterSVGImage();
   UnregisterTGAImage();
+#if defined(HasTIFF)
   UnregisterTIFFImage();
+#endif
   UnregisterTILEImage();
   UnregisterTIMImage();
   UnregisterTTFImage();
@@ -347,7 +389,9 @@ MagickExport void UnregisterStaticModules(void)
   UnregisterWBMPImage();
   UnregisterWMFImage();
   UnregisterWPGImage();
+#if defined(HasX11)
   UnregisterXImage();
+#endif
   UnregisterXBMImage();
   UnregisterXCImage();
   UnregisterXCFImage();
@@ -355,7 +399,10 @@ MagickExport void UnregisterStaticModules(void)
 #if defined(_VISUALC_)
   UnregisterXTRNImage();
 #endif
+#if defined(HasX11)
   UnregisterXWDImage();
+#endif
+  UnregisterYCBCRImage();
   UnregisterYUVImage();
 #endif
 }
