@@ -17,7 +17,7 @@
 %                                 March 2000                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -42,9 +42,12 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
 #include "magick/cache.h"
+#include "magick/cache_private.h"
 #include "magick/constitute.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/memory_.h"
 #include "magick/semaphore.h"
 #include "magick/string_.h"
@@ -52,7 +55,8 @@
 /*
   Typedef declaractions.
 */
-typedef CacheInfo StreamInfo;
+typedef CacheInfo
+  StreamInfo;
 
 /*
   Declare pixel cache interfaces.
@@ -158,9 +162,6 @@ static PixelPacket AcquireOnePixelFromStream(const Image *image,const long x,
 %
 %  A description of each parameter follows:
 %
-%    o status: Method GetPixelStream() returns a pointer to the pixels if they
-%      are transferred, otherwise a NULL is returned.
-%
 %    o image: The image.
 %
 %    o x,y,columns,rows:  These values define the perimeter of a region of
@@ -264,18 +265,18 @@ static void DestroyPixelStream(Image *image)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   stream_info=(StreamInfo *) image->cache;
   assert(stream_info->signature == MagickSignature);
-  AcquireSemaphoreInfo((SemaphoreInfo **) &stream_info->semaphore);
+  stream_info->semaphore=AcquireSemaphoreInfo(stream_info->semaphore);
   stream_info->reference_count--;
   if (stream_info->reference_count > 0)
     {
-      RelinquishSemaphoreInfo((SemaphoreInfo **) &stream_info->semaphore);
+      RelinquishSemaphoreInfo(stream_info->semaphore);
       return;
     }
   if (stream_info->pixels != (PixelPacket *) NULL)
     stream_info->pixels=(PixelPacket *)
       RelinquishMagickMemory(stream_info->pixels);
-  RelinquishSemaphoreInfo((SemaphoreInfo **) &stream_info->semaphore);
-  DestroySemaphoreInfo((SemaphoreInfo **) &stream_info->semaphore);
+  RelinquishSemaphoreInfo(stream_info->semaphore);
+  stream_info->semaphore=DestroySemaphoreInfo(stream_info->semaphore);
   stream_info=(StreamInfo *) RelinquishMagickMemory(stream_info);
 }
 
@@ -385,9 +386,6 @@ static PixelPacket GetOnePixelFromStream(Image *image,const long x,const long y)
 %
 %  A description of each parameter follows:
 %
-%    o status: Method GetPixelStream() returns a pointer to the pixels if they
-%      are transferred, otherwise a NULL is returned.
-%
 %    o image: The image.
 %
 %    o x,y,columns,rows:  These values define the perimeter of a region of
@@ -505,7 +503,7 @@ MagickExport Image *ReadStream(const ImageInfo *image_info,StreamHandler stream,
     AcquireOnePixelFromStream,GetOnePixelFromStream,DestroyPixelStream);
   read_info->stream=stream;
   image=ReadImage(read_info,exception);
-  DestroyImageInfo(read_info);
+  read_info=DestroyImageInfo(read_info);
   return(image);
 }
 
@@ -642,9 +640,6 @@ static PixelPacket *SetPixelStream(Image *image,const long x,const long y,
 %
 %  A description of each parameter follows:
 %
-%    o status: Method SyncPixelStream() returns MagickTrue if the image pixels are
-%      transferred to the in-memory or disk cache otherwise MagickFalse.
-%
 %    o image: The image.
 %
 %
@@ -724,6 +719,6 @@ MagickExport MagickBooleanType WriteStream(const ImageInfo *image_info,
   write_info=CloneImageInfo(image_info);
   write_info->stream=stream;
   status=WriteImage(write_info,image);
-  DestroyImageInfo(write_info);
+  write_info=DestroyImageInfo(write_info);
   return(status);
 }
