@@ -17,7 +17,7 @@
 %                               August 2003                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -42,7 +42,9 @@
 #include "magick/studio.h"
 #include "magick/attribute.h"
 #include "magick/blob.h"
-#include "magick/error.h"
+#include "magick/blob_private.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/list.h"
 #include "magick/log.h"
 #include "magick/memory_.h"
@@ -221,13 +223,12 @@ MagickExport char *CloneString(char **destination,const char *source)
   size_t
     length;
 
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(destination != (char **) NULL);
   if (source == (const char *) NULL)
     {
       if (*destination != (char *) NULL)
         *destination=(char *) RelinquishMagickMemory(*destination);
-      *destination=(char *) NULL;
       return(*destination);
     }
   if (*destination == (char *) NULL)
@@ -272,7 +273,7 @@ MagickExport StringInfo *CloneStringInfo(const StringInfo *string_info)
   StringInfo
     *clone_info;
 
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(string_info != (StringInfo *) NULL);
   assert(string_info->signature == MagickSignature);
   clone_info=AcquireStringInfo(string_info->length);
@@ -293,8 +294,8 @@ MagickExport StringInfo *CloneStringInfo(const StringInfo *string_info)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  CompareStringInfo() returns MagickTrue if the length and datum for two strings are
-%  equal.
+%  CompareStringInfo() returns MagickTrue if the length and datum for two
+%  strings are equal.
 %
 %  The format of the CompareStringInfo method is:
 %
@@ -311,7 +312,7 @@ MagickExport StringInfo *CloneStringInfo(const StringInfo *string_info)
 MagickExport MagickBooleanType CompareStringInfo(const StringInfo *target,
   const StringInfo *source)
 {
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(target != (StringInfo *) NULL);
   assert(target->signature == MagickSignature);
   assert(source != (StringInfo *) NULL);
@@ -478,7 +479,7 @@ MagickExport void ConcatenateStringInfo(StringInfo *string_info,
   size_t
     length;
 
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(string_info != (StringInfo *) NULL);
   assert(string_info->signature == MagickSignature);
   assert(source != (const StringInfo *) NULL);
@@ -568,17 +569,18 @@ MagickExport size_t CopyMagickString(char *destination,const char *source,
 %
 %  The format of the DestroyString method is:
 %
-%      void DestroyString(char *string)
+%      char *DestroyString(char *string)
 %
 %  A description of each parameter follows:
 %
 %    o string: The string.
 %
 */
-MagickExport void DestroyString(char *string)
+MagickExport char *DestroyString(char *string)
 {
   assert(string != (char *) NULL);
   string=(char *) RelinquishMagickMemory(string);
+  return(string);
 }
 
 /*
@@ -596,22 +598,24 @@ MagickExport void DestroyString(char *string)
 %
 %  The format of the DestroyStringInfo method is:
 %
-%      void DestroyStringInfo(StringInfo *string_info)
+%      StringInfo *DestroyStringInfo(StringInfo *string_info)
 %
 %  A description of each parameter follows:
 %
 %    o string_info: The string info.
 %
 */
-MagickExport void DestroyStringInfo(StringInfo *string_info)
+MagickExport StringInfo *DestroyStringInfo(StringInfo *string_info)
 {
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(string_info != (StringInfo *) NULL);
   assert(string_info->signature == MagickSignature);
   if (string_info->datum != (unsigned char *) NULL)
     string_info->datum=(unsigned char *)
       RelinquishMagickMemory(string_info->datum);
+  string_info->signature=(~MagickSignature);
   string_info=(StringInfo *) RelinquishMagickMemory(string_info);
+  return(string_info);
 }
 
 /*
@@ -629,22 +633,23 @@ MagickExport void DestroyStringInfo(StringInfo *string_info)
 %
 %  The format of the DestroyStringList method is:
 %
-%      void DestroyStringList(char **list)
+%      char **DestroyStringList(char **list)
 %
 %  A description of each parameter follows:
 %
 %    o list: The string list.
 %
 */
-MagickExport void DestroyStringList(char **list)
+MagickExport char **DestroyStringList(char **list)
 {
   register long
     i;
 
   assert(list != (char **) NULL);
   for (i=0; list[i] != (char *) NULL; i++)
-    DestroyString(list[i]);
+    list[i]=DestroyString(list[i]);
   list=(char **) RelinquishMagickMemory(list);
+  return(list);
 }
 
 /*
@@ -690,7 +695,7 @@ MagickExport char *EscapeString(const char *source,const char escape)
   size_t
     length;
 
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(source != (const char *) NULL);
   length=CheckOverflowException(strlen(source),1);
   for (p=source; *p != '\0'; p++)
@@ -762,14 +767,14 @@ MagickExport char *FileToString(const char *filename)
   offset=(MagickOffsetType) MagickSeek(file,0,SEEK_END);
   if ((offset < 0) || (offset != (MagickOffsetType) ((ssize_t) offset)))
     {
-      (void) close(file);
+      file=close(file)-1;
       return((char *) NULL);
     }
   length=(size_t) offset;
   string=(char *) AcquireMagickMemory((length+1)*sizeof(*string));
   if (string == (char *) NULL)
     {
-      (void) close(file);
+      file=close(file)-1;
       return((char *) NULL);
     }
   map=MapBlob(file,ReadMode,0,length);
@@ -789,7 +794,7 @@ MagickExport char *FileToString(const char *filename)
       (void) MagickSeek(file,0,SEEK_SET);
       for (i=0; i < length; i+=count)
       {
-        count=read(file,string+i,length-i);
+        count=read(file,string+i,Min(length-i,(size_t) MagickMaxBufferSize));
         if (count <= 0)
           {
             count=0;
@@ -799,13 +804,13 @@ MagickExport char *FileToString(const char *filename)
       }
       if (i < length)
         {
-          (void) close(file);
+          file=close(file)-1;
           string=(char *) RelinquishMagickMemory(string);
           return((char *) NULL);
         }
     }
   string[length]='\0';
-  (void) close(file);
+  file=close(file)-1;
   return(string);
 }
 
@@ -1178,7 +1183,7 @@ MagickExport void PrintStringInfo(const char *id,const StringInfo *string_info)
 */
 MagickExport void ResetStringInfo(StringInfo *string_info)
 {
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(string_info != (StringInfo *) NULL);
   assert(string_info->signature == MagickSignature);
   (void) ResetMagickMemory(string_info->datum,0,string_info->length);
@@ -1211,7 +1216,7 @@ MagickExport void ResetStringInfo(StringInfo *string_info)
 MagickExport void SetStringInfo(StringInfo *string_info,
   const StringInfo *source)
 {
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(string_info != (StringInfo *) NULL);
   assert(string_info->signature == MagickSignature);
   assert(source != (StringInfo *) NULL);
@@ -1250,7 +1255,7 @@ MagickExport void SetStringInfo(StringInfo *string_info,
 MagickExport void SetStringInfoDatum(StringInfo *string_info,
   const unsigned char *source)
 {
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(string_info != (StringInfo *) NULL);
   assert(string_info->signature == MagickSignature);
   assert(source != (const unsigned char *) NULL);
@@ -1284,7 +1289,7 @@ MagickExport void SetStringInfoDatum(StringInfo *string_info,
 MagickExport void SetStringInfoLength(StringInfo *string_info,
   const size_t length)
 {
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(string_info != (StringInfo *) NULL);
   assert(string_info->signature == MagickSignature);
   string_info->length=length;
@@ -1328,7 +1333,7 @@ MagickExport StringInfo *SplitStringInfo(StringInfo *string_info,
   StringInfo
     *split_info;
 
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(string_info != (StringInfo *) NULL);
   assert(string_info->signature == MagickSignature);
   if (offset > string_info->length)
@@ -1368,7 +1373,7 @@ MagickExport StringInfo *StringToStringInfo(const char *string)
   StringInfo
     *string_info;
 
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(string != (const char *) NULL);
   string_info=AcquireStringInfo(strlen(string));
   SetStringInfoDatum(string_info,(const unsigned char *) string);
@@ -1503,7 +1508,7 @@ MagickExport char **StringToArgv(const char *text,int *argc)
       else
         while ((isspace((int) ((unsigned char) *q)) == 0) && (*q != '\0'))
           q++;
-    argv[i]=(char *) AcquireMagickMemory(((q-p)+MaxTextExtent)*sizeof(*argv));
+    argv[i]=(char *) AcquireMagickMemory(((q-p)+MaxTextExtent)*sizeof(**argv));
     if (argv[i] == (char *) NULL)
       {
         for (i--; i >= 0; i--)
@@ -1775,7 +1780,7 @@ MagickExport void StripString(char *message)
 %
 %  The format of the SubstituteString method is:
 %
-%      MagickBooleanType SubstituteString(char **buffer,const char* search,
+%      MagickBooleanType SubstituteString(char **buffer,const char *search,
 %        const char *replace)
 %
 %  A description of each parameter follows:
@@ -1806,7 +1811,7 @@ MagickExport MagickBooleanType SubstituteString(char **buffer,
     result_length,
     search_length;
 
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(buffer != (char **) NULL);
   assert(*buffer != (char *) NULL);
   assert(search != (const char*) NULL);
@@ -1875,15 +1880,20 @@ MagickExport MagickBooleanType SubstituteString(char **buffer,
   result_length+=copy_length;
   if ((result_length+MaxTextExtent) >= allocated_length)
     {
+      long
+        delta;
+
+      delta=destination-result;
       allocated_length+=copy_length;
       result=(char *) ResizeMagickMemory(result,
         (allocated_length+MaxTextExtent)*sizeof(*result));
       if (result == (char *) NULL)
         ThrowMagickFatalException(ResourceLimitFatalError,
           "UnableToAcquireString",search);
+      destination=result+delta;
     }
   (void) strcat(destination,source);
-  *buffer=(char *) RelinquishMagickMemory(*buffer);
+  (void) RelinquishMagickMemory(*buffer);
   *buffer=result;
   return(MagickTrue);
 }
