@@ -1,5 +1,19 @@
 /*
-  ImageMagick String Methods.
+  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization
+  dedicated to making software imaging solutions freely available.
+  
+  You may not use this file except in compliance with the License.
+  obtain a copy of the License at
+  
+    http://www.imagemagick.org/www/Copyright.html
+  
+  Unless required by applicable law or agreed to in writing, software
+  distributed under the License is distributed on an "AS IS" BASIS,
+  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+  See the License for the specific language governing permissions and
+  limitations under the License.
+
+  ImageMagick string methods.
 */
 #ifndef _MAGICK_STRING_H_
 #define _MAGICK_STRING_H_
@@ -8,7 +22,7 @@
 extern "C" {
 #endif
 
-#include "magick/error.h"
+#include "magick/exception.h"
 
 typedef struct _StringInfo
 {
@@ -27,6 +41,9 @@ typedef struct _StringInfo
 
 extern MagickExport char
   *AcquireString(const char *),
+  *CloneString(char **,const char *),
+  *DestroyString(char *),
+  **DestroyStringList(char **),
   *EscapeString(const char *,const char),
   *FileToString(const char *),
   **StringToArgv(const char *,int *),
@@ -38,12 +55,11 @@ extern MagickExport double
 
 extern MagickExport int
   FormatMagickString(char *,const size_t,const char *,...)
-    __attribute__((format (printf,3,4))),
+    magick_attribute((format (printf,3,4))),
   LocaleCompare(const char *,const char *),
   LocaleNCompare(const char *,const char *,const size_t);
 
 extern MagickExport MagickBooleanType
-  CloneString(char **,const char *),
   CompareStringInfo(const StringInfo *,const StringInfo *),
   ConcatenateString(char **,const char *),
   SubstituteString(char **,const char *,const char *);
@@ -55,15 +71,13 @@ extern MagickExport size_t
 extern MagickExport StringInfo
   *AcquireStringInfo(const size_t),
   *CloneStringInfo(const StringInfo *),
+  *DestroyStringInfo(StringInfo *),
   *FileToStringInfo(const char *),
   *SplitStringInfo(StringInfo *,const size_t),
   *StringToStringInfo(const char *);
 
 extern MagickExport void
   ConcatenateStringInfo(StringInfo *,const StringInfo *),
-  DestroyString(char *),
-  DestroyStringInfo(StringInfo *),
-  DestroyStringList(char **),
   LocaleLower(char *),
   LocaleUpper(char *),
   PrintStringInfo(const char *,const StringInfo *),
