@@ -1,5 +1,19 @@
 /*
-  ImageMagick Application Programming Interface declarations.
+  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization
+  dedicated to making software imaging solutions freely available.
+  
+  You may not use this file except in compliance with the License.
+  obtain a copy of the License at
+  
+    http://www.imagemagick.org/www/Copyright.html
+  
+  Unless required by applicable law or agreed to in writing, software
+  distributed under the License is distributed on an "AS IS" BASIS,
+  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+  See the License for the specific language governing permissions and
+  limitations under the License.
+
+  ImageMagick private application programming interface declarations.
 */
 #ifndef _MAGICK_STUDIO_H
 #define _MAGICK_STUDIO_H
@@ -8,19 +22,28 @@
 extern "C" {
 #endif
 
-#if !defined(vms) && !defined(macintosh) && !defined(WIN32)
+#ifdef __CYGWIN32__
+#  ifndef __CYGWIN__
+#    define __CYGWIN__ __CYGWIN32__
+#  endif
+#endif
+#if defined(_WIN32) || defined(WIN32)
+#  ifndef __WINDOWS__
+#    ifdef _WIN32
+#      define __WINDOWS__ _WIN32
+#    else
+#      ifdef WIN32
+#        define __WINDOWS__ WIN32
+#      endif
+#    endif
+#  endif
+#endif
+
+#if !defined(vms) && !defined(macintosh) && !defined(__WINDOWS__)
 # define POSIX
 #endif
 
-/*
-  System include declarations.
-*/
-#define __EXTENSIONS__  1
-#define _GNU_SOURCE  1
-#define _ISOC99_SOURCE  1
-#define _POSIX_C_SOURCE  199506L
-#define _XOPEN_SOURCE  500
-#define _LARGEFILE64_SOURCE  1
+#define MAGICK_IMPLEMENTATION  1
 
 #if !defined(_MAGICK_CONFIG_H)
 # define _MAGICK_CONFIG_H
@@ -34,7 +57,23 @@ extern "C" {
 # endif
 #endif
 
-#if defined(WIN32) && !defined(__CYGWIN__)
+#if defined(MagickMethodPrefix)
+# include "magick/methods.h"
+#endif
+
+#if !defined(const)
+#  define STDC
+#endif
+
+#if defined(__BORLANDC__) && defined(_DLL)
+#  pragma message("BCBMagick lib DLL export interface")
+#  define _MAGICKDLL_
+#  define _MAGICKLIB_
+#  define SupportMagickModules
+#  undef BuildMagickModules
+#endif
+
+#if defined(__WINDOWS__)
 # if defined(_MT) && defined(_DLL) && !defined(_MAGICKDLL_) && !defined(_LIB)
 #  define _MAGICKDLL_
 # endif
@@ -63,12 +102,12 @@ extern "C" {
 # if defined(_DLL) && !defined(_LIB)
 #  define ModuleExport  __declspec(dllexport)
 #  if defined(_VISUALC_)
-#   pragma message( "Magick module DLL export interface" ) 
+#   pragma message( "Magick module DLL export interface" )
 #  endif
 # else
 #  define ModuleExport
 #  if defined(_VISUALC_)
-#   pragma message( "Magick module static interface" ) 
+#   pragma message( "Magick module static interface" )
 #  endif
 
 # endif
@@ -93,15 +132,15 @@ extern "C" {
 #endif
 
 #define MagickSignature  0xabacadabUL
-#define MaxTextExtent  2053
+#define MaxTextExtent  ((size_t) 4096)
 
 #include <stdarg.h>
 #include <stdio.h>
-#if defined(WIN32) && defined(_DEBUG)
+#if defined(__WINDOWS__) && defined(_DEBUG)
 #define _CRTDBG_MAP_ALLOC
 #endif
 #include <stdlib.h>
-#if !defined(WIN32)
+#if !defined(__WINDOWS__)
 # include <unistd.h>
 #else
 # include <direct.h>
@@ -110,15 +149,16 @@ extern "C" {
 # endif
 #endif
 
-#if defined(HAVE_INTTYPES_H)
-# include <inttypes.h>
+#if defined(_FILE_OFFSET_BITS)
+# if (_FILE_OFFSET_BITS == 64)
+#  define fseek  fseeko
+#  define ftell  ftello
+# endif
 #endif
 
-#if defined(_FILE_OFFSET_BITS) && (_FILE_OFFSET_BITS == 64)
-# define fseek  fseeko
-# define ftell  ftello
+#if defined(HAVE_STRINGS_H)
+# include <strings.h>
 #endif
-
 #include <string.h>
 #include <ctype.h>
 #include <locale.h>
@@ -130,7 +170,35 @@ extern "C" {
 #include <signal.h>
 #include <assert.h>
 
-#if defined(WIN32) || defined(POSIX)
+#if defined(HAVE_STRLCPY) && !defined(HAVE_STRLCPY_PROTOTYPE)
+extern size_t
+  strlcpy(char *,const char *,size_t);
+#endif
+
+#if defined(HAVE_VSNPRINTF) && !defined(HAVE_VSNPRINTF_PROTOTYPE)
+extern int
+  vsnprintf(char *,size_t,const char *,va_list);
+#endif
+
+#if !defined(magick_attribute)
+#  if !defined(__GNUC__)
+#    define magick_attribute(x)  /* nothing */
+#  else
+#    define magick_attribute  __attribute__
+#  endif
+#endif
+
+#if !defined(magick_unused)
+#  if defined(__GNUC__)
+#     define magick_unused(x)  magick_unused_ ## x __attribute__((unused))
+#  elif defined(__LCLINT__)
+#    define magick_unused(x) /*@unused@*/ x
+#  else
+#    define magick_unused(x) x
+#  endif
+#endif
+
+#if defined(__WINDOWS__) || defined(POSIX)
 # include <sys/types.h>
 # include <sys/stat.h>
 # if defined(HAVE_FTIME)
@@ -161,58 +229,46 @@ extern "C" {
 # if !defined(S_ISREG)
 #  define S_ISREG(mode) (((mode) & S_IFMT) == S_IFREG)
 # endif
-# include "magick/info.h"
-# include "magick/image.h"
-# if !defined(WIN32)
+# include "magick/ImageMagick.h"
+# if !defined(__WINDOWS__)
 #  include <sys/time.h>
-#if defined(HAVE_SYS_TIMES_H)
+# if defined(HAVE_SYS_TIMES_H)
 #  include <sys/times.h>
-#endif
-#if defined(HAVE_SYS_RESOURCE_H)
-#  include <sys/resource.h>
-#endif
 # endif
+# if defined(HAVE_SYS_RESOURCE_H)
+#  include <sys/resource.h>
+# endif
+#endif
 #else
 # include <types.h>
 # include <stat.h>
 # if defined(macintosh)
-#  include <SIOUX.h>
-#  include <console.h>
+#  if !defined(DISABLE_SIOUX)
+#   include <SIOUX.h>
+#   include <console.h>
+#  endif
 #  include <unix.h>
 # endif
-# include "magick/info.h"
-# include "magick/image.h"
+# include "magick/ImageMagick.h"
 #endif
 
 #if defined(S_IRUSR) && defined(S_IWUSR)
 # define S_MODE (S_IRUSR | S_IWUSR)
-#elif defined (WIN32)
+#elif defined (__WINDOWS__)
 # define S_MODE (_S_IREAD | _S_IWRITE)
 #else
 # define S_MODE  0600
 #endif
 
-#if defined(WIN32)
+#if defined(__WINDOWS__)
 # include "magick/nt_base.h"
 #endif
-#if defined(WIN32) || defined(__CYGWIN__)
-# include "magick/nt_feature.h"
-#endif
 #if defined(macintosh)
 # include "magick/mac.h"
 #endif
 #if defined(vms)
 # include "magick/vms.h"
 #endif
-#if defined(HAVE_MMAP) && !defined(WIN32)
-# include <sys/mman.h>
-#endif
-#if defined(HasPTHREADS)
-# include <pthread.h>
-#endif
-#if defined(HAVE_POLL)
-# include <sys/poll.h>
-#endif
 
 #undef index
 #undef pipe
@@ -231,7 +287,6 @@ extern "C" {
 # define ReadCommandlLine(argc,argv)
 # define SetNotifyHandlers
 #else
-
 # if defined(vms)
 #  define ApplicationDefaults  "decw$system_defaults:"
 #  define DirectorySeparator  ""
@@ -248,7 +303,6 @@ extern "C" {
 #  define ReadCommandlLine(argc,argv)
 #  define SetNotifyHandlers
 # endif
-
 # if defined(macintosh)
 #  define ApplicationDefaults  "/usr/lib/X11/app-defaults/"
 #  define DirectorySeparator  ":"
@@ -260,21 +314,28 @@ extern "C" {
 #  define MagickFilterModulesPath  ""
 #  define MagickSharePath  ""
 #  define PreferencesDefaults  "~/."
-#  define ReadCommandlLine(argc,argv)  argc=ccommand(argv); puts(MagickVersion);
-#  define SetNotifyHandlers \
-    SetErrorHandler(MACErrorHandler); \
-    SetWarningHandler(MACWarningHandler)
+#  if defined(DISABLE_SIOUX)
+#   define ReadCommandlLine(argc,argv)
+#   define SetNotifyHandlers \
+     SetFatalErrorHandler(MacFatalErrorHandler); \
+     SetErrorHandler(MACErrorHandler); \
+     SetWarningHandler(MACWarningHandler)
+#  else
+#   define ReadCommandlLine(argc,argv) argc=ccommand(argv); puts(MagickVersion);
+#   define SetNotifyHandlers \
+     SetErrorHandler(MACErrorHandler); \
+     SetWarningHandler(MACWarningHandler)
+#  endif
 # endif
-
-# if defined(WIN32)
+# if defined(__WINDOWS__)
 #  define DirectorySeparator  "\\"
 #  define DirectoryListSeparator  ';'
 #  define EditorOptions ""
 #  define IsBasenameSeparator(c)  (((c) == '/') || ((c) == '\\'))
 #  define ProcessPendingEvents(text)
-#if !defined(PreferencesDefaults)
-#  define PreferencesDefaults  "~\\."
-#endif /* PreferencesDefaults */
+#  if !defined(PreferencesDefaults)
+#    define PreferencesDefaults  "~\\."
+#  endif
 #  define ReadCommandlLine(argc,argv)
 #  define SetNotifyHandlers \
     SetErrorHandler(NTErrorHandler); \
@@ -287,37 +348,7 @@ extern "C" {
 # endif
 
 #endif
-
-/*
-  Define declarations.
-*/
-#define AbsoluteValue(x)  ((x) < 0 ? -(x) : (x))
-#define False  0
-#define DegreesToRadians(x) (MagickPI*(x)/180.0)
-#define IsGray(color)  \
-  (((color).red == (color).green) && ((color).green == (color).blue))
-#define MagickIncarnate(x)  InitializeMagick(x)
-#define MagickEpsilon  1.0e-12
-#define MagickPI  3.14159265358979323846264338327950288419716939937510
-#define MagickSQ2PI 2.50662827463100024161235523934010416269302368164062
-#define Max(x,y)  (((x) > (y)) ? (x) : (y))
-#define Min(x,y)  (((x) < (y)) ? (x) : (y))
-#define QuantumTick(i,span) ((((i) & 0xff) == 0) || (i == ((span)-1)))
-#define RadiansToDegrees(x) (180.0*(x)/MagickPI)
-#define ScaleColor5to8(x)  ((x) << 3)
-#define ScaleColor6to8(x)  ((x) << 2)
-#define Swap(x,y) ((x)^=(y), (y)^=(x), (x)^=(y))
-#define True  1
-
-/*
-  3D effects.
-*/
-#define AccentuateModulate  ScaleCharToQuantum(80)
-#define HighlightModulate  ScaleCharToQuantum(125)
-#define ShadowModulate  ScaleCharToQuantum(135)
-#define DepthModulate  ScaleCharToQuantum(185)
-#define TroughModulate  ScaleCharToQuantum(110)
-
+
 /*
   Define system symbols if not already defined.
 */
@@ -329,32 +360,61 @@ extern "C" {
 #define O_BINARY  0x00
 #endif
 
-#if !defined(MAP_FAILED)
-#define MAP_FAILED      ((void *) -1)
-#endif
-
-#if defined(HasLTDL) || ( defined(WIN32) && defined(_DLL) )
+#if defined(HasLTDL) || (defined(__WINDOWS__) && defined(_DLL))
 #  define SupportMagickModules
 #endif
 
 #if defined(_MAGICKMOD_)
-#  undef BuildMagickModules
-#  define BuildMagickModules
+# undef BuildMagickModules
+# define BuildMagickModules
 #endif
-
+
 /*
   I/O defines.
 */
-#if defined(WIN32) && !defined(Windows95) && !defined(__BORLANDC__)
+#if defined(__WINDOWS__) && !defined(Windows95) && !defined(__BORLANDC__)
 #define MagickSeek(file,offset,whence)  _lseeki64(file,offset,whence)
 #define MagickTell(file)  _telli64(file)
-#define MagickOffset ExtendedSignedIntegralType
 #else
 #define MagickSeek(file,offset,whence)  lseek(file,offset,whence)
 #define MagickTell(file) tell(file)
-#define MagickOffset off_t
 #endif
 
+/*
+  Magick defines.
+*/
+#define AbsoluteValue(x)  ((x) < 0 ? -(x) : (x))
+#define ClipToQuantum(value)  ((value) < 0.0 ? 0.0 : \
+  ((value) > (MagickRealType) MaxRGB) ? (MagickRealType) MaxRGB : (value))
+#define DegreesToRadians(x)  (MagickPI*(x)/180.0)
+#define IsGray(color)  ((MagickBooleanType) ( \
+  (((color).red == (color).green) && ((color).green == (color).blue))))
+#define MagickPI  3.14159265358979323846264338327950288419716939937510
+#define MagickSQ2PI  2.50662827463100024161235523934010416269302368164062
+#define Max(x,y)  (((x) > (y)) ? (x) : (y))
+#define Min(x,y)  (((x) < (y)) ? (x) : (y))
+#define QuantumScale  ((MagickRealType) 1.0/(MagickRealType) MaxRGB)
+#define QuantumTick(i,span) ((MagickBooleanType) ((((i) & ((i)-1)) == 0) || \
+   (((i) & 0xfff) == 0) || \
+   ((MagickOffsetType) (i) == ((MagickOffsetType) (span)-1))))
+#define RadiansToDegrees(x) (180.0*(x)/MagickPI)
+#define RoundToQuantum(value)  ((Quantum) ((value) < 0.0 ? 0.0 : \
+  ((value) > (MagickRealType) MaxRGB) ? (MagickRealType) MaxRGB : (value)+0.5))
+#define RoundToMap(value)  ((unsigned long) (value < 0.0 ? 0.0 : \
+  ((value) > (MagickRealType) MaxMap) ? (MagickRealType) MaxMap : (value)+0.5))
+#define ScaleXToQuantum(x,scale)  ((Quantum) \
+  (((MagickRealType) MaxRGB*(x))/(scale)+0.5))
+#define Swap(x,y) ((x)^=(y), (y)^=(x), (x)^=(y))
+
+/*
+  3D effects.
+*/
+#define AccentuateModulate  ScaleCharToQuantum(80)
+#define HighlightModulate  ScaleCharToQuantum(125)
+#define ShadowModulate  ScaleCharToQuantum(135)
+#define DepthModulate  ScaleCharToQuantum(185)
+#define TroughModulate  ScaleCharToQuantum(110)
+
 #if defined(__cplusplus) || defined(c_plusplus)
 }
 #endif
