@@ -12,37 +12,24 @@
 %                                                                             %
 %                        ImageMagick Timing Methods                           %
 %                                                                             %
-%                                                                             %
 %                             Software Design                                 %
 %                               John Cristy                                   %
 %                              January 1993                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -54,6 +41,8 @@
   Include declarations.
 */
 #include "magick/studio.h"
+#include "magick/log.h"
+#include "magick/memory_.h"
 #include "magick/timer.h"
 
 /*
@@ -86,24 +75,24 @@ static void
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ContinueTimer resumes a stopped stopwatch. The stopwatch continues
+%  ContinueTimer() resumes a stopped stopwatch. The stopwatch continues
 %  counting from the last StartTimer() onwards.
 %
 %  The format of the ContinueTimer method is:
 %
-%      unsigned int ContinueTimer(TimerInfo *time_info)
+%      MagickBooleanType ContinueTimer(TimerInfo *time_info)
 %
 %  A description of each parameter follows.
 %
 %    o  time_info: Time statistics structure.
 %
 */
-MagickExport unsigned int ContinueTimer(TimerInfo *time_info)
+MagickExport MagickBooleanType ContinueTimer(TimerInfo *time_info)
 {
   assert(time_info != (TimerInfo *) NULL);
   assert(time_info->signature == MagickSignature);
   if (time_info->state == UndefinedTimerState)
-    return(False);
+    return(MagickFalse);
   if (time_info->state == StoppedTimerState)
     {
       time_info->user.total-=time_info->user.stop-time_info->user.start;
@@ -111,7 +100,7 @@ MagickExport unsigned int ContinueTimer(TimerInfo *time_info)
         time_info->elapsed.stop-time_info->elapsed.start;
     }
   time_info->state=RunningTimerState;
-  return(True);
+  return(MagickTrue);
 }
 
 /*
@@ -141,7 +130,7 @@ static double ElapsedTime(void)
 
   return((double) times(&timer)/CLK_TCK);
 #else
-#if defined(WIN32)
+#if defined(__WINDOWS__)
   return(NTElapsedTime());
 #else
   return((double) clock()/CLK_TCK);
@@ -213,10 +202,10 @@ MagickExport void GetTimerInfo(TimerInfo *time_info)
     Create a stopwatch and start it.
   */
   assert(time_info != (TimerInfo *) NULL);
-  (void) memset(time_info,0,sizeof(TimerInfo));
+  (void) ResetMagickMemory(time_info,0,sizeof(*time_info));
   time_info->state=UndefinedTimerState;
   time_info->signature=MagickSignature;
-  StartTimer(time_info,True);
+  StartTimer(time_info,MagickTrue);
 }
 
 /*
@@ -300,22 +289,22 @@ MagickExport void ResetTimer(TimerInfo *time_info)
 %
 %  The format of the StartTimer method is:
 %
-%      void StartTimer(TimerInfo *time_info,const unsigned int reset)
+%      void StartTimer(TimerInfo *time_info,const MagickBooleanType reset)
 %
 %  A description of each parameter follows.
 %
 %    o  time_info: Timer statistics structure.
 %
-%    o  reset: If reset is True, then the stopwatch is reset prior to starting.
-%       If reset is False, then timing is continued without resetting the
-%       stopwatch.
+%    o  reset: If reset is MagickTrue, then the stopwatch is reset prior to
+%       starting.  If reset is MagickFalse, then timing is continued without
+%       resetting the stopwatch.
 %
 */
-MagickExport void StartTimer(TimerInfo *time_info,const unsigned int reset)
+MagickExport void StartTimer(TimerInfo *time_info,const MagickBooleanType reset)
 {
   assert(time_info != (TimerInfo *) NULL);
   assert(time_info->signature == MagickSignature);
-  if (reset)
+  if (reset != MagickFalse)
     {
       /*
         Reset the stopwatch before starting it.
@@ -397,7 +386,7 @@ static double UserTime(void)
   (void) times(&timer);
   return((double) (timer.tms_utime+timer.tms_stime)/CLK_TCK);
 #else
-#if defined(WIN32)
+#if defined(__WINDOWS__)
   return(NTUserTime());
 #else
   return((double) clock()/CLK_TCK);
