@@ -17,7 +17,7 @@
 %                              January 1993                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -41,7 +41,8 @@
   Include declarations.
 */
 #include "magick/studio.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/image.h"
 #include "magick/memory_.h"
 #include "magick/string_.h"
@@ -247,9 +248,12 @@ MagickExport MagickBooleanType GlobExpression(const char *expression,
       GetExceptionInfo(&exception);
       (void) SetImageInfo(image_info,MagickTrue,&exception);
       DestroyExceptionInfo(&exception);
-      DestroyImageInfo(image_info);
       if (LocaleCompare(image_info->filename,pattern) != 0)
-        return(MagickFalse);
+        {
+          image_info=DestroyImageInfo(image_info);
+          return(MagickFalse);
+        }
+      image_info=DestroyImageInfo(image_info);
     }
   /*
     Evaluate glob expression.
@@ -375,7 +379,8 @@ MagickExport MagickBooleanType GlobExpression(const char *expression,
           p=expression;
           match=MagickTrue;
           while ((*p != '\0') && (*pattern != '\0') &&
-                 (*pattern != ',') && (*pattern != '}') && (match != MagickFalse))
+                 (*pattern != ',') && (*pattern != '}') &&
+                 (match != MagickFalse))
           {
             if (*pattern == '\\')
               pattern++;
@@ -450,8 +455,8 @@ MagickExport MagickBooleanType GlobExpression(const char *expression,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  IsGlob() returns MagickTrue if the path specification contains a globbing patten.
-%  as determined by GetGlob.
+%  IsGlob() returns MagickTrue if the path specification contains a globbing
+%  patten.
 %
 %  The format of the IsGlob method is:
 %
