@@ -17,7 +17,7 @@
 %                              January 1993                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -43,7 +43,8 @@
 #include "magick/attribute.h"
 #include "magick/blob.h"
 #include "magick/color.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/geometry.h"
 #include "magick/list.h"
 #include "magick/log.h"
@@ -98,7 +99,7 @@ MagickExport MagickBooleanType AcquireUniqueFilename(char *path)
   file=AcquireUniqueFileResource(path);
   if (file == -1)
     return(MagickFalse);
-  (void) close(file);
+  file=close(file)-1;
   return(MagickTrue);
 }
 
@@ -199,7 +200,7 @@ MagickExport unsigned char *Base64Decode(const char *source,size_t *length)
   unsigned char
     *decode;
 
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(source != (char *) NULL);
   assert(length != (size_t *) NULL);
   *length=0;
@@ -355,7 +356,7 @@ MagickExport char *Base64Encode(const unsigned char *blob,
     max_length,
     remainder;
 
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(blob != (const unsigned char *) NULL);
   assert(blob_length != 0);
   assert(encode_length != (size_t *) NULL);
@@ -554,7 +555,8 @@ MagickExport MagickBooleanType ExpandFilenames(int *argc,char ***argv)
     **vector;
 
   long
-    count;
+    count,
+    parameters;
 
   register long
     i,
@@ -566,7 +568,7 @@ MagickExport MagickBooleanType ExpandFilenames(int *argc,char ***argv)
   /*
     Allocate argument vector.
   */
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(argc != (int *) NULL);
   assert(argv != (char ***) NULL);
   for (i=1; i < (long) *argc; i++)
@@ -585,15 +587,20 @@ MagickExport MagickBooleanType ExpandFilenames(int *argc,char ***argv)
   {
     option=(*argv)[i];
     vector[count++]=AcquireString(option);
-    if ((LocaleNCompare("+profile",option,8) == 0) ||
-        (LocaleNCompare("-format",option,7) == 0) ||
-        (LocaleNCompare("+define",option,7) == 0))
+    parameters=ParseMagickOption(MagickCommandOptions,MagickFalse,option);
+    if (parameters > 0)
       {
-        i++;
-        if (i == (long) *argc)
-          continue;
-        option=(*argv)[i];
-        vector[count++]=AcquireString(option);
+        /*
+          Do not expand command option parameters.
+        */
+        for (j=0; j < parameters; j++)
+        {
+          i++;
+          if (i == (long) *argc)
+            break;
+          option=(*argv)[i];
+          vector[count++]=AcquireString(option);
+        }
         continue;
       }
     if ((*option == '"') || (*option == '\''))
@@ -760,6 +767,7 @@ MagickExport void FormatSize(const MagickSizeType size,char *format)
 MagickExport MagickBooleanType GetExecutionPath(char *path)
 {
   *path='\0';
+  (void) getcwd(path,MaxTextExtent);
 #if defined(__WINDOWS__)
   return(NTGetExecutionPath(path));
 #endif
@@ -772,17 +780,12 @@ MagickExport MagickBooleanType GetExecutionPath(char *path)
     if (execution_path != (const char *) NULL)
       {
         if (*execution_path != *DirectorySeparator)
-          {
-            (void) getcwd(path,MaxTextExtent);
-            (void) ConcatenateMagickString(path,"/",MaxTextExtent);
-          }
+          (void) ConcatenateMagickString(path,"/",MaxTextExtent);
         (void) ConcatenateMagickString(path,execution_path,MaxTextExtent);
-        if (IsAccessible(path) != MagickFalse)
-          return(MagickTrue);
       }
   }
 #endif
-  return(MagickFalse);
+  return(MagickTrue);
 }
 
 /*
@@ -834,7 +837,7 @@ MagickExport void GetPathComponent(const char *path,PathType type,
     }
   (void) CopyMagickString(component,path,MaxTextExtent);
   *magick='\0';
-  if (IsAccessible(path) == MagickFalse)
+  if ((IsDirectory(path) < 0) && (IsAccessible(path) == MagickFalse))
     for (p=component; *p != '\0'; p++)
       if (*p == ':')
         {
@@ -1260,6 +1263,8 @@ MagickExport int SystemCommand(const MagickBooleanType verbose,
   int
     status;
 
+  if (verbose != MagickFalse)
+    (void) fputs(command,stdout);
 #if defined(POSIX)
   status=system(command);
 #elif defined(__WINDOWS__)
@@ -1271,9 +1276,17 @@ MagickExport int SystemCommand(const MagickBooleanType verbose,
 #else
 #  error No suitable system() method.
 #endif
-  if (verbose != MagickFalse)
-    MagickError(DelegateError,command,status < 0 ? strerror(errno) :
-      (char *) NULL);
+  if (status < 0)
+    {
+      ExceptionInfo
+        exception;
+
+      GetExceptionInfo(&exception);
+      (void) ThrowMagickException(&exception,GetMagickModule(),DelegateError,
+        command,strerror(errno));
+      CatchException(&exception);
+      DestroyExceptionInfo(&exception);
+    }
   return(status);
 }
 
@@ -1349,7 +1362,7 @@ MagickExport char *TranslateText(const ImageInfo *image_info,Image *image,
   */
   text_info=CloneImageInfo(image_info);
   translate_text=AcquireString(text);
-  length=MaxTextExtent;
+  length=strlen(text)+MaxTextExtent;
   p=text;
   for (q=translate_text; *p != '\0'; p++)
   {
@@ -1399,7 +1412,7 @@ MagickExport char *TranslateText(const ImageInfo *image_info,Image *image,
       }
       case 'c':
       {
-        attribute=GetImageAttribute(image,"comment");
+        attribute=GetImageAttribute(image,"Comment");
         if (attribute == (ImageAttribute *) NULL)
           break;
         (void) CopyMagickString(q,attribute->value,MaxTextExtent);
@@ -1460,7 +1473,7 @@ MagickExport char *TranslateText(const ImageInfo *image_info,Image *image,
       case 'h':
       {
         (void) FormatMagickString(q,MaxTextExtent,"%lu",
-          image->magick_rows != 0 ? image->magick_rows : 256);
+          image->magick_rows != 0 ? image->magick_rows : image->rows);
         while (*q != '\0')
           q++;
         break;
@@ -1526,7 +1539,7 @@ MagickExport char *TranslateText(const ImageInfo *image_info,Image *image,
       }
       case 'q':
       {
-        (void) FormatMagickString(q,MaxTextExtent,"%u",QuantumDepth);
+        (void) FormatMagickString(q,MaxTextExtent,"%lu",image->depth);
         while (*q != '\0')
           q++;
         break;
@@ -1560,7 +1573,7 @@ MagickExport char *TranslateText(const ImageInfo *image_info,Image *image,
       case 'w':
       {
         (void) FormatMagickString(q,MaxTextExtent,"%lu",
-          image->magick_columns != 0 ? image->magick_columns : 256);
+          image->magick_columns != 0 ? image->magick_columns : image->columns);
         while (*q != '\0')
           q++;
         break;
@@ -1652,7 +1665,7 @@ MagickExport char *TranslateText(const ImageInfo *image_info,Image *image,
       case '#':
       {
         (void) SignatureImage(image);
-        attribute=GetImageAttribute(image,"signature");
+        attribute=GetImageAttribute(image,"Signature");
         if (attribute == (ImageAttribute *) NULL)
           break;
         (void) CopyMagickString(q,attribute->value,MaxTextExtent);
@@ -1673,7 +1686,7 @@ MagickExport char *TranslateText(const ImageInfo *image_info,Image *image,
     }
   }
   *q='\0';
-  DestroyImageInfo(text_info);
+  text_info=DestroyImageInfo(text_info);
   if (text != (char *) embed_text)
     text=(char *) RelinquishMagickMemory(text);
   return(translate_text);
