@@ -10,14 +10,14 @@
 %                 V    EEEEE  R  R   SSSSS  IIIII   OOO   N   N               %
 %                                                                             %
 %                                                                             %
-%                         ImageMagick Version Methods                         %
+%                  ImageMagick Version and Copyright Methods                  %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                               September 2002                                %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -38,7 +38,8 @@
 
 #include "magick/studio.h"
 #include "magick/configure.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/hashmap.h"
 #include "magick/option.h"
 #include "magick/string_.h"
@@ -135,12 +136,14 @@ MagickExport char *GetMagickHomeURL(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  GetMagickName() returns the ImageMagick package name.
+%  GetMagickPackageName() returns the ImageMagick package name.
 %
 %  The format of the GetMagickName method is:
 %
 %      const char *GetMagickName(void)
 %
+%  No parameters are required.
+%
 %
 */
 MagickExport const char *GetMagickPackageName(void)
@@ -195,6 +198,8 @@ MagickExport const char *GetMagickQuantumDepth(unsigned long *depth)
 %
 %      const char *GetMagickReleaseDate(void)
 %
+%  No parameters are required.
+%
 %
 */
 MagickExport const char *GetMagickReleaseDate(void)
