@@ -1,5 +1,19 @@
 /*
-  ImageMagick X11 Widget Methods.
+  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization
+  dedicated to making software imaging solutions freely available.
+  
+  You may not use this file except in compliance with the License.
+  obtain a copy of the License at
+  
+    http://www.imagemagick.org/www/Copyright.html
+  
+  Unless required by applicable law or agreed to in writing, software
+  distributed under the License is distributed on an "AS IS" BASIS,
+  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+  See the License for the specific language governing permissions and
+  limitations under the License.
+
+  ImageMagick X11 widget methods.
 */
 #ifndef _MAGICK_WIDGET_H
 #define _MAGICK_WIDGET_H
@@ -10,6 +24,8 @@ extern "C" {
 
 #if defined(HasX11)
 
+#include "magick/xwindow.h"
+
 extern MagickExport int
   XCommandWidget(Display *,XWindows *,const char **,XEvent *),
   XConfirmWidget(Display *,XWindows *,const char *,const char *),
@@ -27,9 +43,9 @@ extern MagickExport void
   XInfoWidget(Display *,XWindows *,const char *),
   XListBrowserWidget(Display *,XWindows *,XWindowInfo *,const char **,
     const char *,const char *,char *),
-  XMonitorWidget(Display *,XWindows *,const char *,
-    const MagickOffsetType,const MagickSizeType),
   XNoticeWidget(Display *,XWindows *,const char *,const char *),
+  XProgressMonitorWidget(Display *,XWindows *,const char *,
+    const MagickOffsetType,const MagickSizeType),
   XTextViewWidget(Display *,const XResourceInfo *,XWindows *,
     const MagickBooleanType,const char *,const char **);
 
