@@ -12,7 +12,7 @@
  */
 
 #if !defined(_VISUALC_)
-#include <magick_config.h>
+#include <magick/magick_config.h>
 #endif
 #include <stdio.h>
 #include <stdlib.h>
@@ -31,6 +31,9 @@ int main ( int argc, char **argv )
     *final = (Image *) NULL,
     *original = (Image *) NULL;
 
+  const MagickInfo
+    *magick_info;
+
   size_t
     blob_length = 0;
 
@@ -38,7 +41,7 @@ int main ( int argc, char **argv )
     *blob = NULL,
     infile[MaxTextExtent],
     format[MaxTextExtent],
-    *size = NULL;
+    size[MaxTextExtent];
 
   int
     arg = 1,
@@ -51,7 +54,7 @@ int main ( int argc, char **argv )
     fuzz_factor = 0;
 
   ImageInfo
-    imageInfo;
+    *imageInfo;
 
   ExceptionInfo
     exception;
@@ -61,7 +64,7 @@ int main ( int argc, char **argv )
   else
     InitializeMagick(*argv);
 
-  GetImageInfo( &imageInfo );
+  imageInfo=CloneImageInfo(0);
   GetExceptionInfo( &exception );
 
   for (arg=1; arg < argc; arg++)
@@ -75,43 +78,43 @@ int main ( int argc, char **argv )
             {
               arg++;
               option=argv[arg];
-              imageInfo.compression=UndefinedCompression;
+              imageInfo->compression=UndefinedCompression;
               if (LocaleCompare("None",option) == 0)
-                imageInfo.compression=NoCompression;
+                imageInfo->compression=NoCompression;
               if (LocaleCompare("BZip",option) == 0)
-                imageInfo.compression=BZipCompression;
+                imageInfo->compression=BZipCompression;
               if (LocaleCompare("Fax",option) == 0)
-                imageInfo.compression=FaxCompression;
+                imageInfo->compression=FaxCompression;
               if (LocaleCompare("Group4",option) == 0)
-                imageInfo.compression=Group4Compression;
+                imageInfo->compression=Group4Compression;
               if (LocaleCompare("JPEG",option) == 0)
-                imageInfo.compression=JPEGCompression;
+                imageInfo->compression=JPEGCompression;
               if (LocaleCompare("Lossless",option) == 0)
-                imageInfo.compression=LosslessJPEGCompression;
+                imageInfo->compression=LosslessJPEGCompression;
               if (LocaleCompare("LZW",option) == 0)
-                imageInfo.compression=LZWCompression;
+                imageInfo->compression=LZWCompression;
               if (LocaleCompare("RLE",option) == 0)
-                imageInfo.compression=RunlengthEncodedCompression;
+                imageInfo->compression=RunlengthEncodedCompression;
               if (LocaleCompare("Zip",option) == 0)
-                imageInfo.compression=ZipCompression;
+                imageInfo->compression=ZipCompression;
             }
           else if (LocaleCompare("debug",option+1) == 0)
               (void) SetLogEventMask(argv[++arg]);
           else if (LocaleCompare("depth",option+1) == 0)
             {
-              imageInfo.depth=QuantumDepth;
+              imageInfo->depth=QuantumDepth;
               arg++;
-              if ((arg == argc) || !sscanf(argv[arg],"%ld",&imageInfo.depth))
+              if ((arg == argc) || !sscanf(argv[arg],"%ld",&imageInfo->depth))
                 {
-                  printf("-depth argument missing or not integer\n");
-                  fflush(stdout);
+                  (void) printf("-depth argument missing or not integer\n");
+                  (void) fflush(stdout);
                   exit_status = 1;
                   goto program_exit;
                 }
-              if(imageInfo.depth != 8 && imageInfo.depth != 16 && imageInfo.depth != 32)
+              if(imageInfo->depth != 8 && imageInfo->depth != 16 && imageInfo->depth != 32)
                 {
-                  printf("-depth (%ld) not 8, 16, or 32\n", imageInfo.depth);
-                  fflush(stdout);
+                  (void) printf("-depth (%ld) not 8, 16, or 32\n", imageInfo->depth);
+                  (void) fflush(stdout);
                   exit_status = 1;
                   goto program_exit;
                 }
@@ -125,12 +128,12 @@ int main ( int argc, char **argv )
               arg++;
               if ((arg == argc) || !IsGeometry(argv[arg]))
                 {
-                  printf("-size argument missing or not geometry\n");
-                  fflush(stdout);
+                  (void) printf("-size argument missing or not geometry\n");
+                  (void) fflush(stdout);
                   exit_status = 1;
                   goto program_exit;
                 }
-              (void) CloneString(&imageInfo.size,argv[arg]);
+              (void) CloneString(&imageInfo->size,argv[arg]);
             }
         }
       else
@@ -138,38 +141,43 @@ int main ( int argc, char **argv )
     }
   if (arg != argc-2)
     {
-      printf("arg=%d, argc=%d\n", arg, argc);
-      printf ( "Usage: %s [-compress algorithm -debug events -depth integer -log format -size geometry] infile format\n", argv[0] );
-      fflush(stdout);
+      (void) printf("arg=%d, argc=%d\n", arg, argc);
+      (void) printf ( "Usage: %s [-compress algorithm -debug events -depth integer -log format -size geometry] infile format\n", argv[0] );
+      (void) fflush(stdout);
       exit_status = 1;
       goto program_exit;
     }
 
-  strncpy(infile, argv[arg], MaxTextExtent-1 );
+  (void) strncpy(infile, argv[arg], MaxTextExtent-1 );
   arg++;
-  strncpy( format, argv[arg], MaxTextExtent-1 );
+  (void) strncpy( format, argv[arg], MaxTextExtent-1 );
 
   for (arg=0; arg < argc; arg++)
-    printf("%s ", argv[arg]);
-  printf("\n");
-  fflush(stdout);
+    (void) printf("%s ", argv[arg]);
+  (void) printf("\n");
+  (void) fflush(stdout);
 
   /*
    * Read original image
    */
-  GetImageInfo( &imageInfo );
+   imageInfo =DestroyImageInfo( imageInfo );
+  imageInfo=CloneImageInfo(0);
   GetExceptionInfo( &exception );
-  imageInfo.dither = 0;
-  strncpy( imageInfo.filename, infile, MaxTextExtent-1 );
+  imageInfo->dither = 0;
+  (void) strncpy( imageInfo->filename, infile, MaxTextExtent-1 );
   (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-    "Reading image %s", imageInfo.filename);
-  original = ReadImage ( &imageInfo, &exception );
-  if (exception.severity != UndefinedException)
-    CatchException(&exception);
+    "Reading image %s", imageInfo->filename);
+  original = ReadImage ( imageInfo, &exception );
+  if (exception.severity >= ErrorException)
+    {
+      CatchException(&exception);
+      exit_status = 1;
+      goto program_exit;
+    }
   if ( original == (Image *)NULL )
     {
-      printf ( "Failed to read original image %s\n", imageInfo.filename );
-      fflush(stdout);
+      (void) printf ( "Failed to read original image %s\n", imageInfo->filename );
+      (void) fflush(stdout);
       exit_status = 1;
       goto program_exit;
     }
@@ -179,66 +187,80 @@ int main ( int argc, char **argv )
    */
   rows    = original->rows;
   columns = original->columns;
-  size = (char *) AcquireMemory( 40 );
-  sprintf( size, "%dx%d", columns, rows );
+  size[0]='\0';
+  magick_info=GetMagickInfo(format,&exception);
+  if (magick_info && magick_info->raw)
+    FormatString( size, "%dx%d", columns, rows );
 
   /*
    * Save image to BLOB
    */
   blob_length = 8192;
-  strncpy( original->magick, format, MaxTextExtent-1 );
-  strcpy( imageInfo.filename, "" );
+  (void) strncpy( original->magick, format, MaxTextExtent-1 );
+  strcpy( imageInfo->filename, "" );
   original->delay = 10;
   (void) LogMagickEvent(CoderEvent,GetMagickModule(),
     "Writing image to BLOB");
-  blob =(char *) ImageToBlob ( &imageInfo, original, &blob_length, &exception );
-  if (exception.severity != UndefinedException)
-    CatchException(&exception);
-  if ( blob == NULL )
+  blob =(char *) ImageToBlob ( imageInfo, original, &blob_length, &exception );
+  if (exception.severity >= ErrorException)
     {
-      printf ( "Failed to write BLOB in format %s\n", imageInfo.magick );
-      fflush(stdout);
+      CatchException(&exception);
       exit_status = 1;
       goto program_exit;
     }
-  imageInfo.depth=original->depth;
-  DestroyImage( original );
+  if ( blob == NULL )
+    {
+      (void) printf ( "Failed to write BLOB in format %s\n", imageInfo->magick );
+      (void) fflush(stdout);
+      exit_status = 1;
+      goto program_exit;
+    }
+  imageInfo->depth=original->depth;
+   original =DestroyImage( original );
   original = (Image*)NULL;
 
   /*
    * Read image back from BLOB
    */
-  strncpy( imageInfo.magick, format, MaxTextExtent-1 );
-  strcpy( imageInfo.filename, "" );
-  if ( size != NULL )
-    CloneString( &imageInfo.size, size );
-  original = BlobToImage( &imageInfo, blob, blob_length, &exception );
-  if (exception.severity != UndefinedException)
-    CatchException(&exception);
-  if ( original == (Image *)NULL )
+  (void) strncpy( imageInfo->magick, format, MaxTextExtent-1 );
+  strcpy( imageInfo->filename, "" );
+  if ( size[0] != '\0' )
+    CloneString( &imageInfo->size, size );
+  original = BlobToImage( imageInfo, blob, blob_length, &exception );
+  if (exception.severity >= ErrorException)
     {
-      printf ( "Failed to read image from BLOB in format %s\n",imageInfo.magick );
-      fflush(stdout);
+      CatchException(&exception);
       exit_status = 1;
       goto program_exit;
     }
-  LiberateMemory( (void**)&blob );
+  if ( original == (Image *)NULL )
+    {
+      (void) printf ( "Failed to read image from BLOB in format %s\n",imageInfo->magick );
+      (void) fflush(stdout);
+      exit_status = 1;
+      goto program_exit;
+    }
+  blob=(void *) RelinquishMagickMemory(blob);
 
   /*
    * Save image to BLOB
    */
   blob_length = 8192;
-  strncpy( original->magick, format, MaxTextExtent-1 );
-  strcpy( imageInfo.filename, "" );
+  (void) strncpy( original->magick, format, MaxTextExtent-1 );
+  strcpy( imageInfo->filename, "" );
   original->delay = 10;
-  blob = (char *) ImageToBlob ( &imageInfo, original, &blob_length, &exception );
-  if (exception.severity != UndefinedException)
-    CatchException(&exception);
-  imageInfo.depth=original->depth;
+  blob = (char *) ImageToBlob ( imageInfo, original, &blob_length, &exception );
+  if (exception.severity >= ErrorException)
+    {
+      CatchException(&exception);
+      exit_status = 1;
+      goto program_exit;
+    }
+  imageInfo->depth=original->depth;
   if ( blob == NULL )
     {
-      printf ( "Failed to write BLOB in format %s\n", imageInfo.magick );
-      fflush(stdout);
+      (void) printf ( "Failed to write BLOB in format %s\n", imageInfo->magick );
+      (void) fflush(stdout);
       exit_status = 1;
       goto program_exit;
     }
@@ -246,23 +268,27 @@ int main ( int argc, char **argv )
   /*
    * Read image back from BLOB
    */
-  strncpy( imageInfo.magick, format, MaxTextExtent-1 );
-  strcpy( imageInfo.filename, "" );
-  if ( size != NULL )
-    CloneString( &imageInfo.size, size );
+  (void) strncpy( imageInfo->magick, format, MaxTextExtent-1 );
+  strcpy( imageInfo->filename, "" );
+  if ( size[0] != '\0' )
+    CloneString( &imageInfo->size, size );
   (void) LogMagickEvent(CoderEvent,GetMagickModule(),
     "Reading image from BLOB");
-  final = BlobToImage( &imageInfo, blob, blob_length, &exception );
-  if (exception.severity != UndefinedException)
-    CatchException(&exception);
-  if ( final == (Image *)NULL )
+  final = BlobToImage( imageInfo, blob, blob_length, &exception );
+  if (exception.severity >= ErrorException)
     {
-      printf ( "Failed to read image from BLOB in format %s\n",imageInfo.magick );
-      fflush(stdout);
+      CatchException(&exception);
       exit_status = 1;
       goto program_exit;
     }
-  LiberateMemory( (void**)&blob );
+  if ( final == (Image *)NULL )
+    {
+      (void) printf ( "Failed to read image from BLOB in format %s\n",imageInfo->magick );
+      (void) fflush(stdout);
+      exit_status = 1;
+      goto program_exit;
+    }
+  blob=(void *) RelinquishMagickMemory(blob);
 
   /*
    * Check final output
@@ -280,32 +306,33 @@ int main ( int argc, char **argv )
        !strcmp( "PCD", format ) ||
        !strcmp( "PCDS", format ) ||
        !strcmp( "UYVY", format ) ||
-       !strcmp( "YUV", format ) )
-    fuzz_factor = 0.008;
+       !strcmp( "YUV", format ) || (final->compression == JPEGCompression))
+    fuzz_factor = 0.06;
   
   if ( !IsImagesEqual(original, final ) &&
        (original->error.normalized_mean_error > fuzz_factor) )
     {
-      printf( "R/W file check for format \"%s\" failed: %u/%.6f/%.6fe\n",
+      (void) printf( "R/W file check for format \"%s\" failed: %u/%.6f/%.6fe\n",
               format,(unsigned int) original->error.mean_error_per_pixel,
               original->error.normalized_mean_error,
               original->error.normalized_maximum_error);
-      fflush(stdout);
+      (void) fflush(stdout);
       exit_status = 1;
       goto program_exit;
     }
 
  program_exit:
   if (original)
-    DestroyImage( original );
+     original =DestroyImage( original );
   original = (Image*)NULL;
   if (final)
-    DestroyImage( final );
+     final =DestroyImage( final );
   final = (Image*)NULL;
 
+   imageInfo =DestroyImageInfo( imageInfo );
   DestroyMagick();
 
   if (pause)
-    getc(stdin);
+    (void) getc(stdin);
   return exit_status;
 }
