@@ -17,7 +17,7 @@
 %                              July 1992                                      %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -45,7 +45,7 @@
 */
 #include "magick/studio.h"
 #include "magick/client.h"
-#include "magick/error.h"
+#include "magick/exception.h"
 #include "magick/image.h"
 #include "magick/log.h"
 #include "magick/magick.h"
@@ -104,9 +104,9 @@ int main(int argc,char **argv)
   }
   image_info=CloneImageInfo((ImageInfo *) NULL);
   status=MontageImageCommand(image_info,argc,argv,(char **) NULL,&exception);
-  if (exception.severity != UndefinedException)
+  if (exception.severity >= ErrorException)
     CatchException(&exception);
-  DestroyImageInfo(image_info);
+  image_info=DestroyImageInfo(image_info);
   DestroyExceptionInfo(&exception);
   DestroyMagick();
   Exit(!status);
