@@ -23,7 +23,7 @@
 %                                March 2003                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -47,13 +47,20 @@
   Include declarations.
 */
 #include "magick/studio.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/memory_.h"
 #include "magick/semaphore.h"
 #include "magick/string_.h"
 #include "wand/magick_wand.h"
+#include "wand/pixel_wand_private.h"
 #include "wand/wand.h"
 
+/*
+  Define declarations.
+*/
+#define PixelWandId  "PixelWand"
+
 /*
   Typedef declarations.
 */
@@ -72,7 +79,12 @@ struct _PixelWand
     pixel;
 
   unsigned long
-    count,
+    count;
+
+  MagickBooleanType
+    debug;
+
+  unsigned long
     signature;
 };
 
@@ -81,6 +93,41 @@ struct _PixelWand
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   C l e a r P i x e l W a n d                                               %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  ClearPixelWand() clears resources associated with the wand.
+%
+%  The format of the ClearPixelWand method is:
+%
+%      void ClearPixelWand(PixelWand *wand)
+%
+%  A description of each parameter follows:
+%
+%    o wand: The pixel wand.
+%
+%
+*/
+WandExport void ClearPixelWand(PixelWand *wand)
+{
+  assert(wand != (PixelWand *) NULL);
+  assert(wand->signature == MagickSignature);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),wand->name);
+  DestroyExceptionInfo(&wand->exception);
+  GetExceptionInfo(&wand->exception);
+  wand->pixel.colorspace=RGBColorspace;
+  wand->debug=IsEventLogging();
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   D e s t r o y P i x e l W a n d                                           %
 %                                                                             %
 %                                                                             %
@@ -91,7 +138,7 @@ struct _PixelWand
 %
 %  The format of the DestroyPixelWand method is:
 %
-%      void DestroyPixelWand(PixelWand *wand)
+%      PixelWand *DestroyPixelWand(PixelWand *wand)
 %
 %  A description of each parameter follows:
 %
@@ -99,14 +146,17 @@ struct _PixelWand
 %
 %
 */
-WandExport void DestroyPixelWand(PixelWand *wand)
+WandExport PixelWand *DestroyPixelWand(PixelWand *wand)
 {
   assert(wand != (PixelWand *) NULL);
   assert(wand->signature == MagickSignature);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),wand->name);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),wand->name);
   DestroyExceptionInfo(&wand->exception);
+  wand->signature=(~MagickSignature);
   RelinquishWandId(wand->id);
   wand=(PixelWand *) RelinquishMagickMemory(wand);
+  return(wand);
 }
 
 /*
@@ -125,7 +175,8 @@ WandExport void DestroyPixelWand(PixelWand *wand)
 %
 %  The format of the DestroyPixelWands method is:
 %
-%      void DestroyPixelWands(PixelWand **wand,const unsigned long number_wands)
+%      PixelWand **DestroyPixelWands(PixelWand **wand,
+%        const unsigned long number_wands)
 %
 %  A description of each parameter follows:
 %
@@ -135,7 +186,7 @@ WandExport void DestroyPixelWand(PixelWand *wand)
 %
 %
 */
-WandExport void DestroyPixelWands(PixelWand **wand,
+WandExport PixelWand **DestroyPixelWands(PixelWand **wand,
   const unsigned long number_wands)
 {
   register long
@@ -144,10 +195,45 @@ WandExport void DestroyPixelWands(PixelWand **wand,
   assert(wand != (PixelWand **) NULL);
   assert(*wand != (PixelWand *) NULL);
   assert((*wand)->signature == MagickSignature);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),(*wand)->name);
+  if ((*wand)->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),(*wand)->name);
   for (i=(long) number_wands-1; i >= 0; i--)
-    DestroyPixelWand(wand[i]);
+    wand[i]=DestroyPixelWand(wand[i]);
   wand=(PixelWand **) RelinquishMagickMemory(wand);
+  return(wand);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   I s P i x e l W a n d                                                     %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  IsPixelWand() returns MagickTrue if the wand is verified as a pixel wand.
+%
+%  The format of the IsPixelWand method is:
+%
+%      MagickBooleanType IsPixelWand(const PixelWand *wand)
+%
+%  A description of each parameter follows:
+%
+%    o wand: The magick wand.
+%
+*/
+WandExport MagickBooleanType IsPixelWand(const PixelWand *wand)
+{
+  if (wand == (const PixelWand *) NULL)
+    return(MagickFalse);
+  if (wand->signature != MagickSignature)
+    return(MagickFalse);
+  if (LocaleNCompare(wand->name,PixelWandId,strlen(PixelWandId)) != 0)
+    return(MagickFalse);
+  return(MagickTrue);
 }
 
 /*
@@ -171,19 +257,31 @@ WandExport void DestroyPixelWands(PixelWand **wand,
 */
 WandExport PixelWand *NewPixelWand(void)
 {
+  const char
+    *quantum;
+
   struct _PixelWand
     *wand;
 
-  wand=(struct _PixelWand *) AcquireMagickMemory(sizeof(struct _PixelWand));
+  unsigned long
+    depth;
+
+  quantum=GetMagickQuantumDepth(&depth);
+  if (depth != QuantumDepth)
+    ThrowMagickFatalException(WandError,"WandQuantumDepthMismatch",quantum);
+  wand=(struct _PixelWand *) AcquireMagickMemory(sizeof(*wand));
   if (wand == (PixelWand *) NULL)
     ThrowMagickFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
       strerror(errno));
-  (void) ResetMagickMemory(wand,0,sizeof(PixelWand));
+  (void) ResetMagickMemory(wand,0,sizeof(*wand));
   wand->id=AcquireWandId();
-  (void) FormatMagickString(wand->name,MaxTextExtent,"PixelWand-%lu",wand->id);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),wand->name);
+  (void) FormatMagickString(wand->name,MaxTextExtent,"%s-%lu",PixelWandId,
+    wand->id);
   GetExceptionInfo(&wand->exception);
   wand->pixel.colorspace=RGBColorspace;
+  wand->debug=IsEventLogging();
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),wand->name);
   wand->signature=MagickSignature;
   return(wand);
 }
@@ -219,7 +317,7 @@ WandExport PixelWand **NewPixelWands(const unsigned long number_wands)
     **wands;
 
   wands=(struct _PixelWand **)
-    AcquireMagickMemory((size_t) number_wands*sizeof(struct _PixelWand *));
+    AcquireMagickMemory((size_t) number_wands*sizeof(*wands));
   if (wands == (PixelWand **) NULL)
     ThrowMagickFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
       strerror(errno));
@@ -233,6 +331,37 @@ WandExport PixelWand **NewPixelWands(const unsigned long number_wands)
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   P i x e l C l e a r E x c e p t i o n                                     %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  PixelClearException() clear any exceptions associated with the iterator.
+%
+%  The format of the PixelClearException method is:
+%
+%      MagickBooleanType PixelClearException(PixelWand *wand)
+%
+%  A description of each parameter follows:
+%
+%    o wand: The pixel wand.
+%
+*/
+WandExport MagickBooleanType PixelClearException(PixelWand *wand)
+{
+  assert(wand != (PixelWand *) NULL);
+  assert(wand->signature == MagickSignature);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),wand->name);
+  return(SetExceptionInfo(&wand->exception,UndefinedException));
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   P i x e l G e t E x c e p t i o n                                         %
 %                                                                             %
 %                                                                             %
@@ -261,7 +390,8 @@ WandExport char *PixelGetException(const PixelWand *wand,
 
   assert(wand != (const PixelWand *) NULL);
   assert(wand->signature == MagickSignature);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),wand->name);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),wand->name);
   assert(severity != (ExceptionType *) NULL);
   *severity=wand->exception.severity;
   description=(char *) AcquireMagickMemory(2*MaxTextExtent);
@@ -309,8 +439,9 @@ WandExport double PixelGetAlpha(const PixelWand *wand)
 {
   assert(wand != (const PixelWand *) NULL);
   assert(wand->signature == MagickSignature);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),wand->name);
-  return(1.0-wand->pixel.opacity);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),wand->name);
+  return((double) (1.0-wand->pixel.opacity));
 }
 
 /*
@@ -340,7 +471,8 @@ WandExport Quantum PixelGetAlphaQuantum(const PixelWand *wand)
 {
   assert(wand != (const PixelWand *) NULL);
   assert(wand->signature == MagickSignature);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),wand->name);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),wand->name);
   return((Quantum) (MaxRGB-MaxRGB*wand->pixel.opacity+0.5));
 }
 
@@ -370,8 +502,9 @@ WandExport double PixelGetBlack(const PixelWand *wand)
 {
   assert(wand != (const PixelWand *) NULL);
   assert(wand->signature == MagickSignature);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),wand->name);
-  return(wand->pixel.index);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),wand->name);
+  return((double) wand->pixel.index);
 }
 
 /*
@@ -401,7 +534,8 @@ WandExport Quantum PixelGetBlackQuantum(const PixelWand *wand)
 {
   assert(wand != (const PixelWand *) NULL);
   assert(wand->signature == MagickSignature);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),wand->name);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),wand->name);
   return((Quantum) (MaxRGB*wand->pixel.index+0.5));
 }
 
@@ -431,8 +565,9 @@ WandExport double PixelGetBlue(const PixelWand *wand)
 {
   assert(wand != (const PixelWand *) NULL);
   assert(wand->signature == MagickSignature);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),wand->name);
-  return(wand->pixel.blue);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),wand->name);
+  return((double) wand->pixel.blue);
 }
 
 /*
@@ -462,7 +597,8 @@ WandExport Quantum PixelGetBlueQuantum(const PixelWand *wand)
 {
   assert(wand != (const PixelWand *) NULL);
   assert(wand->signature == MagickSignature);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),wand->name);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),wand->name);
   return((Quantum) (MaxRGB*wand->pixel.blue+0.5));
 }
 
@@ -495,7 +631,8 @@ WandExport char *PixelGetColorAsString(const PixelWand *wand)
 
   assert(wand != (const PixelWand *) NULL);
   assert(wand->signature == MagickSignature);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),wand->name);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),wand->name);
   (void) FormatMagickString(color,MaxTextExtent,"%u,%u,%u",
     (Quantum) (MaxRGB*wand->pixel.red+0.5),
     (Quantum) (MaxRGB*wand->pixel.green+0.5),
@@ -535,7 +672,8 @@ WandExport unsigned long PixelGetColorCount(const PixelWand *wand)
 {
   assert(wand != (const PixelWand *) NULL);
   assert(wand->signature == MagickSignature);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),wand->name);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),wand->name);
   return(wand->count);
 }
 
@@ -565,8 +703,9 @@ WandExport double PixelGetCyan(const PixelWand *wand)
 {
   assert(wand != (const PixelWand *) NULL);
   assert(wand->signature == MagickSignature);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),wand->name);
-  return(wand->pixel.red);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),wand->name);
+  return((double) wand->pixel.red);
 }
 
 /*
@@ -596,7 +735,8 @@ WandExport Quantum PixelGetCyanQuantum(const PixelWand *wand)
 {
   assert(wand != (const PixelWand *) NULL);
   assert(wand->signature == MagickSignature);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),wand->name);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),wand->name);
   return((Quantum) (MaxRGB*wand->pixel.red+0.5));
 }
 
@@ -626,8 +766,9 @@ WandExport double PixelGetGreen(const PixelWand *wand)
 {
   assert(wand != (const PixelWand *) NULL);
   assert(wand->signature == MagickSignature);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),wand->name);
-  return(wand->pixel.green);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),wand->name);
+  return((double) wand->pixel.green);
 }
 
 /*
@@ -657,7 +798,8 @@ WandExport Quantum PixelGetGreenQuantum(const PixelWand *wand)
 {
   assert(wand != (const PixelWand *) NULL);
   assert(wand->signature == MagickSignature);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),wand->name);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),wand->name);
   return((Quantum) (MaxRGB*wand->pixel.green+0.5));
 }
 
@@ -687,7 +829,8 @@ WandExport IndexPacket PixelGetIndex(const PixelWand *wand)
 {
   assert(wand != (const PixelWand *) NULL);
   assert(wand->signature == MagickSignature);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),wand->name);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),wand->name);
   return((IndexPacket) (wand->pixel.index+0.5));
 }
 
@@ -717,8 +860,9 @@ WandExport double PixelGetMagenta(const PixelWand *wand)
 {
   assert(wand != (const PixelWand *) NULL);
   assert(wand->signature == MagickSignature);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),wand->name);
-  return(wand->pixel.green);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),wand->name);
+  return((double) wand->pixel.green);
 }
 
 /*
@@ -748,7 +892,8 @@ WandExport Quantum PixelGetMagentaQuantum(const PixelWand *wand)
 {
   assert(wand != (const PixelWand *) NULL);
   assert(wand->signature == MagickSignature);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),wand->name);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),wand->name);
   return((Quantum) (MaxRGB*wand->pixel.green+0.5));
 }
 
@@ -757,6 +902,46 @@ WandExport Quantum PixelGetMagentaQuantum(const PixelWand *wand)
 %                                                                             %
 %                                                                             %
 %                                                                             %
++   P i x e l G e t M a g i c k C o l o r                                     %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  PixelGetMagickColor() gets the magick color of the pixel wand.
+%
+%  The format of the PixelGetMagickColor method is:
+%
+%      void PixelGetMagickColor(PixelWand *wand,MagickPixelPacket *color)
+%
+%  A description of each parameter follows:
+%
+%    o wand: The pixel wand.
+%
+%    o color:  The pixel wand color is returned here.
+%
+*/
+WandExport void PixelGetMagickColor(const PixelWand *wand,
+  MagickPixelPacket *color)
+{
+  assert(wand != (const PixelWand *) NULL);
+  assert(wand->signature == MagickSignature);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),wand->name);
+  assert(color != (MagickPixelPacket *) NULL);
+  *color=wand->pixel;
+  color->red=MaxRGB*wand->pixel.red;
+  color->green=MaxRGB*wand->pixel.green;
+  color->blue=MaxRGB*wand->pixel.blue;
+  color->opacity=MaxRGB*wand->pixel.opacity;
+  color->index=MaxRGB*wand->pixel.index;
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   P i x e l G e t O p a c i t y                                             %
 %                                                                             %
 %                                                                             %
@@ -779,8 +964,9 @@ WandExport double PixelGetOpacity(const PixelWand *wand)
 {
   assert(wand != (const PixelWand *) NULL);
   assert(wand->signature == MagickSignature);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),wand->name);
-  return(wand->pixel.opacity);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),wand->name);
+  return((double) wand->pixel.opacity);
 }
 
 /*
@@ -810,7 +996,8 @@ WandExport Quantum PixelGetOpacityQuantum(const PixelWand *wand)
 {
   assert(wand != (const PixelWand *) NULL);
   assert(wand->signature == MagickSignature);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),wand->name);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),wand->name);
   return((Quantum) (MaxRGB*wand->pixel.opacity+0.5));
 }
 
@@ -819,28 +1006,31 @@ WandExport Quantum PixelGetOpacityQuantum(const PixelWand *wand)
 %                                                                             %
 %                                                                             %
 %                                                                             %
-+   P i x e l G e t Q u a n t u m C o l o r                                   %
+%   P i x e l G e t Q u a n t u m C o l o r                                   %
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  PixelGetQuantumColor() gets the color of the pixel wand.
+%  PixelGetQuantumColor() gets the color of the pixel wand as a PixelPacket.
 %
 %  The format of the PixelGetQuantumColor method is:
 %
-%      void *PixelGetQuantumColor(PixelWand *wand)
+%      void PixelGetQuantumColor(PixelWand *wand,PixelPacket *color)
 %
 %  A description of each parameter follows:
 %
 %    o wand: The pixel wand.
 %
+%    o color:  The pixel wand color is returned here.
+%
 */
 WandExport void PixelGetQuantumColor(const PixelWand *wand,PixelPacket *color)
 {
   assert(wand != (const PixelWand *) NULL);
   assert(wand->signature == MagickSignature);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),wand->name);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),wand->name);
   assert(color != (PixelPacket *) NULL);
   color->red=(Quantum) (MaxRGB*wand->pixel.red+0.5);
   color->green=(Quantum) (MaxRGB*wand->pixel.green+0.5);
@@ -874,8 +1064,9 @@ WandExport double PixelGetRed(const PixelWand *wand)
 {
   assert(wand != (const PixelWand *) NULL);
   assert(wand->signature == MagickSignature);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),wand->name);
-  return(wand->pixel.red);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),wand->name);
+  return((double) wand->pixel.red);
 }
 
 /*
@@ -905,7 +1096,8 @@ WandExport Quantum PixelGetRedQuantum(const PixelWand *wand)
 {
   assert(wand != (const PixelWand *) NULL);
   assert(wand->signature == MagickSignature);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),wand->name);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),wand->name);
   return((Quantum) (MaxRGB*wand->pixel.red+0.5));
 }
 
@@ -935,8 +1127,9 @@ WandExport double PixelGetYellow(const PixelWand *wand)
 {
   assert(wand != (const PixelWand *) NULL);
   assert(wand->signature == MagickSignature);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),wand->name);
-  return(wand->pixel.blue);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),wand->name);
+  return((double) wand->pixel.blue);
 }
 
 /*
@@ -966,7 +1159,8 @@ WandExport Quantum PixelGetYellowQuantum(const PixelWand *wand)
 {
   assert(wand != (const PixelWand *) NULL);
   assert(wand->signature == MagickSignature);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),wand->name);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),wand->name);
   return((Quantum) (MaxRGB*wand->pixel.blue+0.5));
 }
 
@@ -998,7 +1192,8 @@ WandExport void PixelSetAlpha(PixelWand *wand,const double opacity)
 {
   assert(wand != (const PixelWand *) NULL);
   assert(wand->signature == MagickSignature);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),wand->name);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),wand->name);
   if (opacity > 1.0)
     wand->pixel.opacity=0.0;
   else
@@ -1038,7 +1233,8 @@ WandExport void PixelSetAlphaQuantum(PixelWand *wand,const Quantum opacity)
 {
   assert(wand != (const PixelWand *) NULL);
   assert(wand->signature == MagickSignature);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),wand->name);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),wand->name);
   wand->pixel.opacity=(MagickRealType) (MaxRGB-opacity)/MaxRGB;
 }
 
@@ -1070,7 +1266,8 @@ WandExport void PixelSetBlack(PixelWand *wand,const double black)
 {
   assert(wand != (const PixelWand *) NULL);
   assert(wand->signature == MagickSignature);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),wand->name);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),wand->name);
   if (black > 1.0)
     wand->pixel.index=1.0;
   else
@@ -1109,7 +1306,8 @@ WandExport void PixelSetBlackQuantum(PixelWand *wand,const Quantum black)
 {
   assert(wand != (const PixelWand *) NULL);
   assert(wand->signature == MagickSignature);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),wand->name);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),wand->name);
   wand->pixel.index=(MagickRealType) black/MaxRGB;
 }
 
@@ -1141,7 +1339,8 @@ WandExport void PixelSetBlue(PixelWand *wand,const double blue)
 {
   assert(wand != (const PixelWand *) NULL);
   assert(wand->signature == MagickSignature);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),wand->name);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),wand->name);
   if (blue > 1.0)
     wand->pixel.blue=1.0;
   else
@@ -1180,7 +1379,8 @@ WandExport void PixelSetBlueQuantum(PixelWand *wand,const Quantum blue)
 {
   assert(wand != (const PixelWand *) NULL);
   assert(wand->signature == MagickSignature);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),wand->name);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),wand->name);
   wand->pixel.blue=(MagickRealType) blue/MaxRGB;
 }
 
@@ -1220,7 +1420,8 @@ WandExport MagickBooleanType PixelSetColor(PixelWand *wand,const char *color)
 
   assert(wand != (const PixelWand *) NULL);
   assert(wand->signature == MagickSignature);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),wand->name);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),wand->name);
   status=QueryMagickColor(color,&pixel,&wand->exception);
   if (status == MagickFalse)
     return(status);
@@ -1262,7 +1463,8 @@ WandExport void PixelSetColorCount(PixelWand *wand,const unsigned long count)
 {
   assert(wand != (const PixelWand *) NULL);
   assert(wand->signature == MagickSignature);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),wand->name);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),wand->name);
   wand->count=count;
 }
 
@@ -1294,7 +1496,8 @@ WandExport void PixelSetCyan(PixelWand *wand,const double cyan)
 {
   assert(wand != (const PixelWand *) NULL);
   assert(wand->signature == MagickSignature);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),wand->name);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),wand->name);
   if (cyan > 1.0)
     wand->pixel.red=1.0;
   else
@@ -1333,7 +1536,8 @@ WandExport void PixelSetCyanQuantum(PixelWand *wand,const Quantum cyan)
 {
   assert(wand != (const PixelWand *) NULL);
   assert(wand->signature == MagickSignature);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),wand->name);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),wand->name);
   wand->pixel.red=(MagickRealType) cyan/MaxRGB;
 }
 
@@ -1365,7 +1569,8 @@ WandExport void PixelSetGreen(PixelWand *wand,const double green)
 {
   assert(wand != (const PixelWand *) NULL);
   assert(wand->signature == MagickSignature);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),wand->name);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),wand->name);
   if (green > 1.0)
     wand->pixel.green=1.0;
   else
@@ -1404,7 +1609,8 @@ WandExport void PixelSetGreenQuantum(PixelWand *wand,const Quantum green)
 {
   assert(wand != (const PixelWand *) NULL);
   assert(wand->signature == MagickSignature);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),wand->name);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),wand->name);
   wand->pixel.green=(MagickRealType) green/MaxRGB;
 }
 
@@ -1436,7 +1642,8 @@ WandExport void PixelSetIndex(PixelWand *wand,const IndexPacket index)
 {
   assert(wand != (const PixelWand *) NULL);
   assert(wand->signature == MagickSignature);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),wand->name);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),wand->name);
   wand->pixel.index=(MagickRealType) index;
 }
 
@@ -1468,7 +1675,8 @@ WandExport void PixelSetMagenta(PixelWand *wand,const double magenta)
 {
   assert(wand != (const PixelWand *) NULL);
   assert(wand->signature == MagickSignature);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),wand->name);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),wand->name);
   if (magenta > 1.0)
     wand->pixel.green=1.0;
   else
@@ -1508,7 +1716,8 @@ WandExport void PixelSetMagentaQuantum(PixelWand *wand,const Quantum magenta)
 {
   assert(wand != (const PixelWand *) NULL);
   assert(wand->signature == MagickSignature);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),wand->name);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),wand->name);
   wand->pixel.green=(MagickRealType) magenta/MaxRGB;
 }
 
@@ -1540,7 +1749,8 @@ WandExport void PixelSetOpacity(PixelWand *wand,const double opacity)
 {
   assert(wand != (const PixelWand *) NULL);
   assert(wand->signature == MagickSignature);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),wand->name);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),wand->name);
   if (opacity > 1.0)
     wand->pixel.opacity=1.0;
   else
@@ -1580,7 +1790,8 @@ WandExport void PixelSetOpacityQuantum(PixelWand *wand,const Quantum opacity)
 {
   assert(wand != (const PixelWand *) NULL);
   assert(wand->signature == MagickSignature);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),wand->name);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),wand->name);
   wand->pixel.opacity=(MagickRealType) opacity/MaxRGB;
 }
 
@@ -1612,7 +1823,8 @@ WandExport void PixelSetQuantumColor(PixelWand *wand,const PixelPacket *color)
 {
   assert(wand != (const PixelWand *) NULL);
   assert(wand->signature == MagickSignature);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),wand->name);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),wand->name);
   assert(color != (PixelPacket *) NULL);
   wand->pixel.red=(MagickRealType) color->red/MaxRGB;
   wand->pixel.green=(MagickRealType) color->green/MaxRGB;
@@ -1648,7 +1860,8 @@ WandExport void PixelSetRed(PixelWand *wand,const double red)
 {
   assert(wand != (const PixelWand *) NULL);
   assert(wand->signature == MagickSignature);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),wand->name);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),wand->name);
   if (red > 1.0)
     wand->pixel.red=1.0;
   else
@@ -1687,7 +1900,8 @@ WandExport void PixelSetRedQuantum(PixelWand *wand,const Quantum red)
 {
   assert(wand != (const PixelWand *) NULL);
   assert(wand->signature == MagickSignature);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),wand->name);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),wand->name);
   wand->pixel.red=(MagickRealType) red/MaxRGB;
 }
 
@@ -1719,7 +1933,8 @@ WandExport void PixelSetYellow(PixelWand *wand,const double yellow)
 {
   assert(wand != (const PixelWand *) NULL);
   assert(wand->signature == MagickSignature);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),wand->name);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),wand->name);
   if (yellow > 1.0)
     wand->pixel.blue=1.0;
   else
@@ -1758,6 +1973,7 @@ WandExport void PixelSetYellowQuantum(PixelWand *wand,const Quantum yellow)
 {
   assert(wand != (const PixelWand *) NULL);
   assert(wand->signature == MagickSignature);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),wand->name);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),wand->name);
   wand->pixel.green=(MagickRealType) yellow/MaxRGB;
 }
