@@ -2,6 +2,7 @@
   Display a smiley on your X server.
 */
 #include <stdio.h>
+#include <stdlib.h>
 #include <time.h>
 #include <sys/types.h>
 #include <magick/api.h>
@@ -136,14 +137,11 @@ int main(int argc,char **argv)
   InitializeMagick(*argv);
   image_info=CloneImageInfo((ImageInfo *) NULL);
   image=AllocateImage(image_info);
-  if (image == (Image *) NULL)
-    MagickError(ResourceLimitError,"Unable to display image",
-      "MemoryAllocationFailed");
-  /*
-    Initialize smiley image.
-  */
   image->columns=smile_width;
   image->rows=smile_height;
+  /*
+    Set smiley image.
+  */
   p=smile_bits;
   for (y=0; y < image->rows; y++)
   {
@@ -168,10 +166,8 @@ int main(int argc,char **argv)
   /*
     Free resources.
   */
-  DestroyImage(image);
-  DestroyImageInfo(image_info);
+  image=DestroyImage(image);
+  image_info=DestroyImageInfo(image_info);
   DestroyMagick();
+  exit(0);
 }
-<A HREF="http://promote.pair.com/direct.pl?imagemagick.org">
-  <IMG SRC="../images/promote.png" ALIGN=RIGHT WIDTH=120
-    HEIGHT=60 BORDER=0></A>
